// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#ifndef __OB_BalancerDatabase_h__
#define __OB_BalancerDatabase_h__

#include <OB/Basic.h>
#include <OB/Hashtable.h>

#include <BalancerLogger_fwd.h>
#include <BalancerDatabase_fwd.h>

#include <BalancerDB.h>
#include <Database.h>

namespace OBPortableServer
{

//
// Forward declaration
//
struct ObjectIdHasher;

} // End namespace OBPortableServer

namespace LoadBalancing
{

//
// Forward declarations
//
struct StrategyElement;
struct GroupElement;

//
// The balancer database.
//
class Database : public OB::RefCount, public JTCMonitor
{
public: // For some compilers this must be public

    typedef OB::Hashtable<PortableServer::ObjectId, StrategyElement,
    			  OBPortableServer::ObjectIdHasher> StrategyRecordMap;

    typedef OB::Hashtable<CORBA::String_var, GroupElement, 
                          OB::StringHasher> GroupRecordMap;
private:

    Database(const Database&);
    void operator=(const Database&);

    //
    // The Logger
    //
    Logger_var logger_;

    //
    // The database
    //
    OB::Database* db_;

    //
    // Strategy Records
    //
    StrategyRecordMap* strategies_;

    //
    // Group Records
    //
    GroupRecordMap* groups_;

    //
    // Are we persistent?
    //
    bool persistent_;

    //
    // Load the database.
    //
    void load();

public:

    Database(CORBA::ORB_ptr, OB::Properties_ptr, Logger_ptr);
    ~Database();

    static inline Database_ptr _duplicate(Database_ptr p)
    { if(p) p -> _OB_incRef(); return p; }
    static inline Database_ptr _nil()
    { return 0; }

    //
    // Are we persistent?
    //
    bool persistent() { return persistent_; }

    //
    // Strategy retrieval and modification methods.
    //
    StrategyRecordSeq* getStrategies();
    bool addStrategy(StrategyRecord&);
    bool removeStrategy(const PortableServer::ObjectId&);

    //
    // Group retrieval and modification methods.
    //
    GroupRecordSeq* getGroups();
    bool addGroup(GroupRecord&);
    bool updateGroup(GroupRecord&);
    bool removeGroup(const char*);
};

} // End namespace LoadBalancing

namespace CORBA
{

inline void
release(LoadBalancing::Database_ptr p)
{
    if(p)
        p -> _OB_decRef();
}

inline Boolean
is_nil(LoadBalancing::Database_ptr p)
{
    return p == 0;
}

} // End of namespace CORBA

#endif
