// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#ifndef __OB_BalancerLogger_h__
#define __OB_BalancerLogger_h__

#include <OB/Basic.h>
#include <OB/Properties_fwd.h>
#include <OB/Logger_fwd.h>

#include <BalancerLogger_fwd.h>

namespace LoadBalancing
{

//
// The balancer logger.
//
class Logger : public OB::RefCount
{
    //
    // The message logger
    //
    OB::Logger_var logger_;

    //
    // The trace levels
    //
    CORBA::ULong traceLifecycle_;
    CORBA::ULong traceLoadBalance_;
    CORBA::ULong traceDatabase_;

public:

    Logger(OB::Logger_ptr, OB::Properties_ptr);
    ~Logger();

    static inline Logger_ptr _duplicate(Logger_ptr p)
    { if(p) p -> _OB_incRef(); return p; }
    static inline Logger_ptr _nil()
    { return 0; }

    //
    // Methods to get trace levels.
    //
    CORBA::ULong traceLifecycle();
    CORBA::ULong traceLoadBalance();
    CORBA::ULong traceDatabase();

    //
    // Methods to perform logging.
    //
    void info(const char*);
    void error(const char*);
    void warning(const char*);
    void trace(const char*, const char*);
};

} // End of namespace LoadBalancing

namespace CORBA
{

inline void
release(LoadBalancing::Logger_ptr p)
{
    if(p)
        p -> _OB_decRef();
}

inline Boolean
is_nil(LoadBalancing::Logger_ptr p)
{
    return p == 0;
}

} // End of namespace CORBA

#endif
