// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>
#include <OB/PortableInterceptor.h>

#include <BalancerPolicies_impl.h>

using namespace CORBA;

// ----------------------------------------------------------------------
// MemberPolicy constructor/destructor
// ----------------------------------------------------------------------

LoadBalancing::MemberPolicy_impl::MemberPolicy_impl(
    const MemberPolicyValue& value)
    : value_(value)
{
}

LoadBalancing::MemberPolicy_impl::~MemberPolicy_impl()
{
}

// ----------------------------------------------------------------------
// MemberPolicy IDL to C++ Mapping
// ----------------------------------------------------------------------

LoadBalancing::MemberPolicyValue*
LoadBalancing::MemberPolicy_impl::value()
    throw(CORBA::SystemException)
{
    return new MemberPolicyValue(value_);
}

CORBA::PolicyType
LoadBalancing::MemberPolicy_impl::policy_type()
    throw(CORBA::SystemException)
{
    return MEMBER_POLICY_ID;
}

CORBA::Policy_ptr
LoadBalancing::MemberPolicy_impl::copy()
    throw(CORBA::SystemException)
{
    return CORBA::Policy::_duplicate(this);
}

void
LoadBalancing::MemberPolicy_impl::destroy()
    throw(CORBA::SystemException)
{
}

// ----------------------------------------------------------------------
// MemberPolicyFactory IDL to C++ Mapping
// ----------------------------------------------------------------------

CORBA::Policy_ptr
LoadBalancing::MemberPolicyFactory_impl::create_policy(CORBA::PolicyType type,
						       const CORBA::Any& any)
    throw(CORBA::PolicyError)
{
    if(type == MEMBER_POLICY_ID)
    {
	const MemberPolicyValue* value;
	if(any >>= value)
	    return new MemberPolicy_impl(*value);
	throw CORBA::PolicyError(CORBA::BAD_POLICY_TYPE);
    }
    throw CORBA::PolicyError(CORBA::BAD_POLICY);
    return CORBA::Policy::_nil(); // Some compilers need this
}

