// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#ifndef __OB_GroupFactory_impl_h__
#define __OB_GroupFactory_impl_h__

#include <OB/Balancer_skel.h>

#include <BalancerLogger_fwd.h>
#include <BalancerDatabase_fwd.h>
#include <StrategyFactory_fwd.h>

//
// IDL:ooc.com/LoadBalancing:1.0
//
namespace LoadBalancing
{

//
// Forward declaration
//
class Group_impl;

//
// IDL:ooc.com/LoadBalancing/GroupFactory:1.0
//
class GroupFactory_impl : virtual public POA_LoadBalancing::GroupFactory,
                          virtual public PortableServer::RefCountServantBase,
			  public JTCMutex
{
public: // For some compilers this must be public

    typedef OB::Hashtable<CORBA::String_var, Group_impl*, 
                          OB::StringHasher> GroupMap;

private:

    GroupFactory_impl(const GroupFactory_impl&);
    void operator=(const GroupFactory_impl&);

    //
    // The ORB
    //
    CORBA::ORB_var orb_;

    //
    // The POA
    //
    PortableServer::POA_var poa_;

    //
    // The group POA
    //
    PortableServer::POA_var groupPOA_;

    //
    // The strategy proxy POA
    //
    PortableServer::POA_var proxyPOA_;

    //
    // The logger
    //
    Logger_var logger_;

    //
    // The database
    //
    Database_var db_;

    //
    // The Strategy Factory
    //
    StrategyFactory_var strategyFactory_;

    //
    // The locator object reference template
    //
    ObjectReferenceTemplate_var locatorORT_;

    //
    // The groups
    //
    GroupMap* groups_;

    //
    // Are we running in administrative mode?
    //
    bool administrative_;

    //
    // Restore a group using a database record.
    //
    void create(const GroupRecord&)
        throw(LoadBalancing::GroupExists, CORBA::SystemException);

public:

    GroupFactory_impl(CORBA::ORB_ptr, PortableServer::POA_ptr,
		      PortableServer::POA_ptr, PortableServer::POA_ptr,
		      Logger_ptr, Database_ptr, StrategyFactory_ptr,
		      ObjectReferenceTemplate*, bool);
    ~GroupFactory_impl();

    virtual PortableServer::POA_ptr _default_POA();

    //
    // IDL:ooc.com/LoadBalancing/GroupFactory/create:1.0
    //
    virtual LoadBalancing::Group_ptr create(const char* group_id)
        throw(LoadBalancing::GroupExists,
	      CORBA::SystemException);

    //
    // IDL:ooc.com/LoadBalancing/GroupFactory/get:1.0
    //
    virtual LoadBalancing::Group_ptr get(const char* group_id)
        throw(LoadBalancing::GroupNotFound, CORBA::SystemException);
    //
    // IDL:ooc.com/LoadBalancing/GroupFactory/list:1.0
    //
    virtual LoadBalancing::GroupIdSeq* list()
        throw(CORBA::SystemException);

    //
    // IDL:ooc.com/LoadBalancing/GroupFactory/shutdown:1.0
    //
    virtual void shutdown()
        throw(CORBA::SystemException);
    
    //
    // Remove group from factory.
    //
    void remove(const char*);
};

} // End of namespace LoadBalancing

#endif
