// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>
#include <OB/Net.h> // For htonl()

#include <GroupORT_impl.h>

using namespace CORBA;

// ----------------------------------------------------------------------
// GroupORT_impl constructor and destructor
// ----------------------------------------------------------------------

LoadBalancing::GroupORT_impl::GroupORT_impl()
{
}

LoadBalancing::GroupORT_impl::GroupORT_impl(
    const char* group_id,
    PortableInterceptor::ObjectReferenceTemplate* real_ort)
{
    this -> group_id(group_id);
    this -> real_ort(real_ort);
}

LoadBalancing::GroupORT_impl::~GroupORT_impl()
{
}

// ----------------------------------------------------------------------
// GroupORT_impl public member implementation
// ----------------------------------------------------------------------

CORBA::ValueBase*
LoadBalancing::GroupORT_impl::_copy_value()
{
    GroupORT_impl* r = new GroupORT_impl(group_id(), real_ort());

    return r;
}

//
// IDL:omg.org/PortableInterceptor/ObjectReferenceTemplate/server_id:1.0
//
PortableInterceptor::ServerId
LoadBalancing::GroupORT_impl::server_id()
{
    return real_ort() -> server_id();
}

//
// IDL:omg.org/PortableInterceptor/ObjectReferenceTemplate/orb_id:1.0
//
PortableInterceptor::ORBId
LoadBalancing::GroupORT_impl::orb_id()
{
    return real_ort() -> orb_id();
}

//
// IDL:omg.org/PortableInterceptor/ObjectReferenceTemplate/adapter_name:1.0
//
PortableInterceptor::AdapterName*
LoadBalancing::GroupORT_impl::adapter_name()
{
    return real_ort() -> adapter_name();
}

//
// IDL:omg.org/PortableInterceptor/ObjectReferenceTemplate/make_object:1.0
//
CORBA::Object_ptr
LoadBalancing::GroupORT_impl::make_object(
    const char* repository_id,
    const PortableInterceptor::ObjectId& id)
{
    if(real_ort() == 0)
	return Object::_nil();

    //
    // Convert the group id to bytes.
    //
    PortableServer::ObjectId_var groupIdBytes =
	PortableServer::string_to_ObjectId(group_id());

    //
    // Convert the repository id to bytes.
    //
    PortableServer::ObjectId_var repositoryIdBytes =
	PortableServer::string_to_ObjectId(repository_id);

    //
    // Combine
    //  - the group id length bytes,
    //  - the group id bytes,
    //  - the repository id length bytes,
    //  - the repository id bytes, and
    //  - the given id
    // to create the new object id.
    //
    ULong len = 4 + groupIdBytes -> length() +
		4 + repositoryIdBytes -> length() +
		id.length();
    PortableInterceptor::ObjectId_var newOid =
	new PortableInterceptor::ObjectId(len);
    newOid -> length(len);
    Octet* ptr = newOid -> get_buffer();

    //
    // Copy the length of the group id bytes.
    //
    ULong groupIdBytesLen = groupIdBytes -> length(); 
    Long tmp = htonl((Long)groupIdBytesLen);
    memcpy(ptr, &tmp, 4);
    ptr += 4;

    //
    // Copy the group id bytes.
    //
    memcpy(ptr, groupIdBytes -> get_buffer(), groupIdBytesLen);
    ptr += groupIdBytesLen;

    //
    // Copy the length of the repository id bytes.
    //
    ULong repositoryIdBytesLen = repositoryIdBytes -> length(); 
    tmp = htonl((Long)repositoryIdBytesLen);
    memcpy(ptr, &tmp, 4);
    ptr += 4;

    //
    // Copy the repository id bytes.
    //
    memcpy(ptr, repositoryIdBytes -> get_buffer(), repositoryIdBytesLen);
    ptr += repositoryIdBytesLen;

    //
    // Copy the givin id.
    //
    memcpy(ptr, id.get_buffer(), id.length());

    //
    // Return the new object.
    //
    return real_ort() -> make_object(repository_id, newOid);
}

// ----------------------------------------------------------------------
// GroupORTFactory_impl constructor and destructor
// ----------------------------------------------------------------------

LoadBalancing::GroupORTFactory_impl::GroupORTFactory_impl()
{
}

LoadBalancing::GroupORTFactory_impl::~GroupORTFactory_impl()
{
}

// ----------------------------------------------------------------------
// GroupORTFactory_impl public member implementation
// ----------------------------------------------------------------------

CORBA::ValueBase*
LoadBalancing::GroupORTFactory_impl::create_for_unmarshal()
{
    return new LoadBalancing::GroupORT_impl();
}
