// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#ifndef __OB_Group_impl_h__
#define __OB_Group_impl_h__

#include <BalancerLogger_fwd.h>
#include <BalancerDatabase_fwd.h>
#include <StrategyFactory_fwd.h>

#include <BalancerInternal_skel.h>

//
// IDL:ooc.com/LoadBalancing:1.0
//
namespace LoadBalancing
{

//
// Forward declaration
//
class GroupFactory_impl;
class StrategyProxy_impl;
class MemberStrategy;
struct GroupRecord;

//
// IDL:ooc.com/LoadBalancing/Group:1.0
//
class Group_impl : virtual public POA_LoadBalancing::InternalGroup,
                   virtual public PortableServer::RefCountServantBase,
		   public JTCMutex
{
    Group_impl(const Group_impl&);
    void operator=(const Group_impl&);

    //
    // The POA
    //
    PortableServer::POA_var poa_;

    //
    // The strategy proxy POA
    //
    PortableServer::POA_var proxyPOA_;

    //
    // The logger
    //
    Logger_var logger_;

    //
    // The database
    //
    Database_var db_;

    //
    // The StrategyFactory
    //
    StrategyFactory_var strategyFactory_;

    //
    // The GroupFactory
    //
    GroupFactory_impl* factory_;

    //
    // The object reference template
    //
    ObjectReferenceTemplate_var groupORT_;

    //
    // The associated strategy
    //
    Strategy_var strategy_;

    //
    // The strategy proxy and its object id
    //
    StrategyProxy_impl* proxy_;
    PortableServer::ObjectId_var proxyOid_;

    //
    // The members
    //
    MemberStrategy* members_;

    //
    // The group id
    //
    GroupId_var id_;

    //
    // Are we running in administrative mode?
    //
    bool administrative_;

    //
    // Is the group destroyed?
    //
    bool destroyed_;

    //
    // Update the database.
    //
    void updateDatabase();

protected:

    //
    // Fill the given record.
    //
    void fillRecord(LoadBalancing::GroupRecord* rec);

    //
    // GroupFactory_impl requires access to fillRecord().
    //
    friend class GroupFactory_impl;

public:

    Group_impl(PortableServer::POA_ptr, PortableServer::POA_ptr,
	       Logger_ptr, Database_ptr, StrategyFactory_ptr,
	       GroupFactory_impl*,
	       PortableInterceptor::ObjectReferenceTemplate*,
	       const char*, Strategy_ptr, bool);
    Group_impl(PortableServer::POA_ptr, PortableServer::POA_ptr,
	       Logger_ptr, Database_ptr, StrategyFactory_ptr,
	       GroupFactory_impl*,
	       PortableInterceptor::ObjectReferenceTemplate*,
	       const GroupRecord&, bool);
    ~Group_impl();


    virtual PortableServer::POA_ptr _default_POA();

    //
    // IDL:ooc.com/LoadBalancing/Group/get_id:1.0
    //
    virtual LoadBalancing::GroupId get_id()
        throw(CORBA::SystemException);

    //
    // IDL:ooc.com/LoadBalancing/Group/get_ior:1.0
    //
    virtual CORBA::Object_ptr get_ior(const char* repository_id,
				      const LoadBalancing::ObjectId& oid)
        throw(CORBA::SystemException);

    //
    // IDL:ooc.com/LoadBalancing/Group/get_strategy_proxy:1.0
    //
    virtual LoadBalancing::StrategyProxy_ptr get_strategy_proxy()
        throw(CORBA::SystemException);

    //
    // IDL:ooc.com/LoadBalancing/Group/set_strategy:1.0
    //
    virtual void set_strategy(const char* name,
			      const LoadBalancing::PropertySeq& properties)
        throw(LoadBalancing::StrategyNotFound,
	      LoadBalancing::InvalidProperties,
	      CORBA::SystemException);

    //
    // IDL:ooc.com/LoadBalancing/Group/set_custom_strategy:1.0
    //
    virtual void set_custom_strategy(LoadBalancing::Strategy_ptr the_strategy)
        throw(CORBA::SystemException);

    //
    // IDL:ooc.com/LoadBalancing/Group/add_member:1.0
    //
    virtual void add_member(const char* member_id)
        throw(LoadBalancing::MemberExists,
              CORBA::SystemException);

    //
    // IDL:ooc.com/LoadBalancing/Group/remove_member:1.0
    //
    virtual void remove_member(const char* member_id)
        throw(LoadBalancing::MemberNotFound,
	      CORBA::SystemException);

    //
    // IDL:ooc.com/LoadBalancing/Group/set_load_alert:1.0
    //
    virtual void set_load_alert(const char* member_id,
				LoadBalancing::LoadAlert_ptr alert)
        throw(LoadBalancing::MemberNotFound,
	      CORBA::SystemException);

    //
    // IDL:ooc.com/LoadBalancing/Group/list_members:1.0
    //
    virtual LoadBalancing::MemberIdSeq* list_members()
        throw(CORBA::SystemException);

    //
    // IDL:ooc.com/LoadBalancing/Group/destroy:1.0
    //
    virtual void destroy()
        throw(CORBA::SystemException);

    //
    // IDL:ooc.com/LoadBalancing/InternalGroup/update_member:1.0
    //
    virtual LoadBalancing::ObjectReferenceTemplate* update_member(
	const char* member_id,
	LoadBalancing::ObjectReferenceTemplate* ort)
        throw(LoadBalancing::MemberNotFound,
              CORBA::SystemException);

    //
    // IDL:ooc.com/LoadBalancing/InternalGroup/get_next:1.0
    //
    virtual CORBA::Object_ptr get_next(const char* repository_id,
				       const LoadBalancing::ObjectId& oid)
        throw(LoadBalancing::MemberNotFound,
	      CORBA::SystemException);
};

} // End of namespace LoadBalancing

#endif
