// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>

#include <BalancerDatabase.h>
#include <StrategyBase.h>

using namespace CORBA;

// ----------------------------------------------------------------------
// StrategyBase constructor and destructor
// ----------------------------------------------------------------------

LoadBalancing::StrategyBase::StrategyBase(PortableServer::POA_ptr poa,
					  Database_ptr db)
    : poa_(PortableServer::POA::_duplicate(poa)),
      db_(Database::_duplicate(db))
{
}

LoadBalancing::StrategyBase::~StrategyBase()
{
}

// ----------------------------------------------------------------------
// StrategyBase public member implementation
// ----------------------------------------------------------------------

PortableServer::POA_ptr
LoadBalancing::StrategyBase::_default_POA()
{
    return PortableServer::POA::_duplicate(poa_);
}

//
// IDL:ooc.com/LoadBalancing/Strategy/get_properties:1.0
//
LoadBalancing::PropertySeq*
LoadBalancing::StrategyBase::get_properties()
    throw(CORBA::SystemException)
{
    return new PropertySeq(0);
}

//
// IDL:ooc.com/LoadBalancing/Strategy/push_load:1.0
//
void
LoadBalancing::StrategyBase::push_load(const char*,
				       CORBA::ULong)
    throw(LoadBalancing::MemberNotFound,
	  LoadBalancing::StrategyNotAdaptive,
	  CORBA::SystemException)
{
    throw StrategyNotAdaptive();
}

//
// IDL:ooc.com/LoadBalancing/Strategy/destroy:1.0
//
void
LoadBalancing::StrategyBase::destroy()
    throw(CORBA::SystemException)
{
    //
    // Deactivate the strategy.
    //
    PortableServer::ObjectId_var oid;
    try
    {
	oid = poa_ -> servant_to_id(this);
	poa_ -> deactivate_object(oid.in());
    }
    catch(...)
    {
	assert(false); // Should not occur
    }

    //
    // Remove the strategy from the database.
    //
    if(db_ -> persistent())
	db_ -> removeStrategy(oid.in());
}

