// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#ifndef __OB_StrategyFactory_h__
#define __OB_StrategyFactory_h__

#include <OB/Basic.h>
#include <OB/Balancer_fwd.h>

#include <BalancerLogger_fwd.h>
#include <BalancerDatabase_fwd.h>
#include <StrategyFactory_fwd.h>

namespace LoadBalancing
{

//
// The strategy factory.
//
class StrategyFactory : public OB::RefCount
{
    StrategyFactory(const StrategyFactory&);
    void operator=(const StrategyFactory&);

    //
    // The POA
    //
    PortableServer::POA_var poa_;

    //
    // The logger
    //
    Logger_var logger_;

    //
    // The database
    //
    Database_var db_;

    //
    // Create strategy with the given ObjectId.
    //
    void create(const char* name,
		const LoadBalancing::PropertySeq& properties,
		const PortableServer::ObjectId& oid)
        throw(LoadBalancing::StrategyNotFound,
	      LoadBalancing::InvalidProperties,
	      CORBA::SystemException);

public:

    StrategyFactory(PortableServer::POA_ptr, Logger_ptr, Database_ptr);
    ~StrategyFactory();

    static inline StrategyFactory_ptr _duplicate(StrategyFactory_ptr p)
    { if(p) p -> _OB_incRef(); return p; }
    static inline StrategyFactory_ptr _nil()
    { return 0; }

    //
    // Create strategy.
    //
    Strategy_ptr create(const char* name,
			const LoadBalancing::PropertySeq& properties)
        throw(LoadBalancing::StrategyNotFound,
	      LoadBalancing::InvalidProperties,
	      CORBA::SystemException);

    //
    // Create default strategy.
    //
    Strategy_ptr createDefault()
        throw(CORBA::SystemException);
};

} // End of namespace LoadBalancing

namespace CORBA
{

inline void
release(LoadBalancing::StrategyFactory_ptr p)
{
    if(p)
        p -> _OB_decRef();
}

inline Boolean
is_nil(LoadBalancing::StrategyFactory_ptr p)
{
    return p == 0;
}

} // End of namespace CORBA

#endif
