// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>

#include <BalancerDatabase.h>
#include <StrategyRR_impl.h>

using namespace CORBA;

// ----------------------------------------------------------------------
// StrategyRR_impl constructor and destructor
// ----------------------------------------------------------------------

LoadBalancing::StrategyRR_impl::StrategyRR_impl(PortableServer::POA_ptr poa,
						Database_ptr db)
    : StrategyBase(poa, db)
{
#ifdef LB_DEBUG
    OB_CERR << "StrategyRR_impl::StrategyRR_impl()" << OB_ENDL;
#endif
}

LoadBalancing::StrategyRR_impl::~StrategyRR_impl()
{
#ifdef LB_DEBUG
    OB_CERR << "StrategyRR_impl::~StrategyRR_impl()" << OB_ENDL;
#endif

    //
    // Delete the members.
    //
    MemberList::iterator it;
    for(it = members_.begin() ; it != members_.end() ; it++)
	delete *it;
}

// ----------------------------------------------------------------------
// StrategyRR_impl public member implementation
// ----------------------------------------------------------------------

//
// IDL:ooc.com/LoadBalancing/Strategy/get_name:1.0
//
char*
LoadBalancing::StrategyRR_impl::get_name()
    throw(CORBA::SystemException)
{
    return string_dup("round-robin");
}

//
// IDL:ooc.com/LoadBalancing/Strategy/adjust:1.0
//
void
LoadBalancing::StrategyRR_impl::adjust(
    const LoadBalancing::MemberDataSeq& members)
    throw(CORBA::SystemException)
{
    JTCSynchronized sync(*this);

    //
    // Create an array to help keep track of members that are updated.
    //
    short* updated = new short[members.length()];
    memset(updated, 0, members.length() * sizeof(short));

    //
    // Update the members that still exist and remove the members that
    // do not.  This will preserve the order and load of these members.
    //
    MemberList::iterator it;
    for(it = members_.begin() ; it != members_.end() ; it++)
    {
	bool found = false;
	for(ULong i = 0 ; i < members.length() ; i++)
	{
	    if(!updated[i] &&
	       strcmp((*it) -> member_id, members[i].member_id) == 0)
	    {
		//
		// Update the member.
		//
		*(*it) = members[i];

		//
		// Mark the member as updated.
		//
		updated[i] = 1;

		found = true;
		break;
	    }
	}

	if(!found)
	{
	    //
	    // The member no longer exists.
	    //
	    delete *it;
	    members_.erase(it);
	}
    }

    //
    // Add the new members.
    //
    for(ULong i = 0 ; i < members.length() ; i++)
    {
	if(!updated[i])
	{
	    MemberData* member = new MemberData(members[i]);
	    members_.push_back(member);
	}
    }

    delete [] updated;
}

//
// IDL:ooc.com/LoadBalancing/Strategy/get_next:1.0
//
LoadBalancing::MemberId
LoadBalancing::StrategyRR_impl::get_next()
    throw(LoadBalancing::MemberNotFound,
	  CORBA::SystemException)
{
    JTCSynchronized sync(*this);

    if(members_.empty())
	throw MemberNotFound();

    //
    // The element at the front of the list will be returned.
    //
    MemberData* member = members_.front();

    //
    // Move this element to the back of the list.
    //
    if(members_.size() > 1)
    {
	members_.pop_front();
	members_.push_back(member);
    }

    return string_dup(member -> member_id);
}

