// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#ifndef __OB_StrategyRand_impl_h__
#define __OB_StrategyRand_impl_h__

#include <BalancerDatabase_fwd.h>

#include <StrategyBase.h>

//
// IDL:ooc.com/LoadBalancing:1.0
//
namespace LoadBalancing
{

//
// IDL:ooc.com/LoadBalancing/Strategy:1.0
//
// This implements a simple random load balancing strategy.
//
class StrategyRand_impl : public StrategyBase,
			  public JTCMutex
{
    StrategyRand_impl(const StrategyRand_impl&);
    void operator=(const StrategyRand_impl&);

    //
    // The members
    //
    MemberDataSeq members_;

    //
    // The seed for generating random numbers
    //
    unsigned seed_;

    //
    // Get a random ULong between 0 and n - 1.
    //
    CORBA::ULong random(CORBA::ULong n);

public:

    StrategyRand_impl(PortableServer::POA_ptr, Database_ptr);
    ~StrategyRand_impl();

    //
    // IDL:ooc.com/LoadBalancing/Strategy/get_name:1.0
    //
    virtual char* get_name()
        throw(CORBA::SystemException);

    //
    // IDL:ooc.com/LoadBalancing/Strategy/adjust:1.0
    //
    virtual void adjust(const LoadBalancing::MemberDataSeq& members)
        throw(CORBA::SystemException);

    //
    // IDL:ooc.com/LoadBalancing/Strategy/get_next:1.0
    //
    virtual LoadBalancing::MemberId get_next()
        throw(LoadBalancing::MemberNotFound,
	      CORBA::SystemException);
};

} // End of namespace LoadBalancing

#endif
