// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>

#include <Tester.h>

using namespace CORBA;

void
usage(const char* prog)
{
    OB_CERR << "Usage: " << prog << " <test-case>\n"
	    << "<test-case> = general\n"
	    << "            | persist-stage1\n"
	    << "            | persist-stage2\n"
	    << "            | admin-stage1\n"
	    << "            | admin-stage2\n"
	    << "            | admin-stage3\n";
}

int
run(ORB_ptr orb, int argc, char* argv[])
{
    if(argc != 2)
    {
	usage(argv[0]);
	return EXIT_FAILURE;
    }

    //
    // Get tester object
    //
    CORBA::Object_var obj;
    try
    {
        obj = orb -> string_to_object("relfile:/Tester.ref");
    }
    catch(const CORBA::BAD_PARAM&)
    {
    }

    if(CORBA::is_nil(obj))
    {
	OB_CERR << argv[0] << ": cannot read IOR from Tester.ref" << OB_ENDL;
	return EXIT_FAILURE;
    }
    
    Tester_var tester = Tester::_narrow(obj);
    assert(!CORBA::is_nil(tester));

    try
    {
	if(strcmp(argv[1], "general") == 0)
	{
	    OB_COUT << "Testing GroupFactory... " << OB_FLUSH;
	    tester -> test_GroupFactory();
	    OB_COUT << " Done!" << OB_ENDL;

	    OB_COUT << "Testing Group... " << OB_FLUSH;
	    tester -> test_Group();
	    OB_COUT << " Done!" << OB_ENDL;

	    OB_COUT << "Testing random strategy... " << OB_FLUSH;
	    tester -> test_RandomStrategy();
	    OB_COUT << " Done!" << OB_ENDL;

	    OB_COUT << "Testing round-robin strategy... " << OB_FLUSH;
	    tester -> test_RoundRobinStrategy();
	    OB_COUT << " Done!" << OB_ENDL;

	    OB_COUT << "Testing least-load strategy... " << OB_FLUSH;
	    tester -> test_LeastLoadStrategy();
	    OB_COUT << " Done!" << OB_ENDL;

	    OB_COUT << "Testing min-dispersion strategy... " << OB_FLUSH;
	    tester -> test_MinDispStrategy();
	    OB_COUT << " Done!" << OB_ENDL;

	    OB_COUT << "Testing custom strategy... " << OB_FLUSH;
	    tester -> test_CustomStrategy();
	    OB_COUT << " Done!" << OB_ENDL;
	}
	else if(strcmp(argv[1], "persist-stage1") == 0)
	{
	    OB_COUT << "Testing persistence (stage 1)... " << OB_FLUSH;
	    tester -> test_persistence_stage1();
	    OB_COUT << " Done!" << OB_ENDL;
	}
	else if(strcmp(argv[1], "persist-stage2") == 0)
	{
	    OB_COUT << "Testing persistence (stage 2)... " << OB_FLUSH;
	    tester -> test_persistence_stage2();
	    OB_COUT << " Done!" << OB_ENDL;
	}
	else if(strcmp(argv[1], "admin-stage1") == 0)
	{
	    OB_COUT << "Testing administrative (stage 1)... " << OB_FLUSH;
	    tester -> test_administrative_stage1();
	    OB_COUT << " Done!" << OB_ENDL;
	}
	else if(strcmp(argv[1], "admin-stage2") == 0)
	{
	    OB_COUT << "Testing administrative (stage 2)... " << OB_FLUSH;
	    tester -> test_administrative_stage2();
	    OB_COUT << " Done!" << OB_ENDL;
	}
	else if(strcmp(argv[1], "admin-stage3") == 0)
	{
	    OB_COUT << "Testing administrative (stage 3)... " << OB_FLUSH;
	    tester -> test_administrative_stage3();
	    OB_COUT << " Done!" << OB_ENDL;
	}
	else if(strcmp(argv[1], "shutdown") == 0)
	{
	    OB_COUT << "Shutting down test server... " << OB_FLUSH;
	    tester -> shutdown();
	    OB_COUT << " Done!" << OB_ENDL;
	}
	else
	{
	    usage(argv[0]);
	    return EXIT_FAILURE;
	}
    }
    catch(const CORBA::COMM_FAILURE&)
    {
    }

    return EXIT_SUCCESS;
}

int
main(int argc, char* argv[], char*[])
{
    int status = EXIT_SUCCESS;
    ORB_var orb;

    try
    {
	orb = ORB_init(argc, argv);
	status = run(orb, argc, argv);
    }
    catch(const Exception& ex)
    {
	OB_CERR << ex << OB_ENDL;
	status = EXIT_FAILURE;
    }

    if(!is_nil(orb))
    {
	try
	{
	    orb -> destroy();
	}
	catch(const Exception& ex)
	{
	    OB_CERR << ex << OB_ENDL;
	    status = EXIT_FAILURE;
	}
    }
    
    return status;
}
