// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#ifndef __CustomStrategy_impl_h__
#define __CustomStrategy_impl_h__

#include <OB/list.h>
#include <OB/Balancer_skel.h>

//
// A custom load balancing strategy
//
class CustomStrategy_impl : virtual public POA_LoadBalancing::Strategy,
			    virtual public PortableServer::RefCountServantBase,
			    public JTCMutex
{
public: // For some compilers this must be public

    typedef OB::stl::list< LoadBalancing::MemberData* > MemberList;

private:

    CustomStrategy_impl(const CustomStrategy_impl&);
    void operator=(const CustomStrategy_impl&);

    //
    // The POA
    //
    PortableServer::POA_var poa_;

    //
    // The members
    //
    MemberList members_;

    //
    // Are we destroyed?
    //
    bool destroyed_;

public:

    CustomStrategy_impl(PortableServer::POA_ptr);
    ~CustomStrategy_impl();

    virtual PortableServer::POA_ptr _default_POA();

    //
    // Are we destroyed?
    //
    bool destroyed() { return destroyed_; }

    //
    // IDL:ooc.com/LoadBalancing/Strategy/get_name:1.0
    //
    virtual char* get_name()
        throw(CORBA::SystemException);

    //
    // IDL:ooc.com/LoadBalancing/Strategy/get_properties:1.0
    //
    virtual LoadBalancing::PropertySeq* get_properties()
        throw(CORBA::SystemException);

    //
    // IDL:ooc.com/LoadBalancing/Strategy/adjust:1.0
    //
    virtual void adjust(const LoadBalancing::MemberDataSeq& members)
        throw(CORBA::SystemException);

    //
    // IDL:ooc.com/LoadBalancing/Strategy/get_next:1.0
    //
    virtual LoadBalancing::MemberId get_next()
        throw(LoadBalancing::MemberNotFound,
	      CORBA::SystemException);

    //
    // IDL:ooc.com/LoadBalancing/Strategy/push_load:1.0
    //
    virtual void push_load(const char* member_id,
			   CORBA::ULong load)
        throw(LoadBalancing::MemberNotFound,
	      LoadBalancing::StrategyNotAdaptive,
	      CORBA::SystemException);

    //
    // IDL:ooc.com/LoadBalancing/Strategy/destroy:1.0
    //
    virtual void destroy()
        throw(CORBA::SystemException);
};

#endif
