// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>

#include <MemberObject_impl.h>
#include <Tester_impl.h>

using namespace CORBA;
using namespace LoadBalancing;

//
// IDL:Tester/test_GroupFactory:1.0
//
void
Tester_impl::test_GroupFactory()
    throw(CORBA::SystemException)
{
    try
    {
	GroupIdSeq ids;
	MemberIdSeq mids;

	//
	// Should have 0 groups
	//
	ids.length(0);
	TEST(checkGroupFactory(ids));

	//
	// Create group `g1'
	//
	Group_var g1 = factory_ -> create("g1");
	TEST(checkGroup(g1, "g1", mids));

	//
	// Should only have group `g1'
	//
	ids.length(1);
	ids[0] = string_dup("g1");
	TEST(checkGroupFactory(ids));

	//
	// Trying to re-create group `g1' should result in a GroupExists
	// exception
	//
	try
	{
	    g1 = factory_ -> create("g1");
	    TEST(false);
	}
	catch(const GroupExists&)
	{
	}

	//
	// Get and destroy group `g1'
	//
	g1 = factory_ -> get("g1");
	g1 -> destroy();

	//
	// Trying to get group `g1' should now result in a GroupNotFound
	// exception
	//
	try
	{
	    g1 = factory_ -> get("g1");
	    TEST(false);
	}
	catch(const GroupNotFound&)
	{
	}

	//
	// Should have 0 groups
	//
	ids.length(0);
	TEST(checkGroupFactory(ids));

	//
	// Create groups `g1', `g2' and `g3'
	//
	g1 = factory_ -> create("g1");
	TEST(checkGroup(g1, "g1", mids));

	Group_var g2 = factory_ -> create("g2");
	TEST(checkGroup(g2, "g2", mids));

	Group_var g3 = factory_ -> create("g3");
	TEST(checkGroup(g3, "g3", mids));

	//
	// Should have 3 groups: `g1', `g2' and `g3'
	//
	ids.length(3);
	ids[0] = string_dup("g1");
	ids[1] = string_dup("g2");
	ids[2] = string_dup("g3");
	TEST(checkGroupFactory(ids));

	//
	// Get and destroy group `g2'
	//
	g2 = factory_ -> get("g2");
	g2 -> destroy();

	//
	// Should have 2 groups: `g1' and `g3'
	//
	ids.length(2);
	ids[0] = string_dup("g1");
	ids[1] = string_dup("g3");
	TEST(checkGroupFactory(ids));

	//
	// Get and destroy `g1'
	//
	g1 = factory_ -> get("g1");
	g1 -> destroy();

	//
	// Should only have group `g3'
	//
	ids.length(1);
	ids[0] = string_dup("g3");
	TEST(checkGroupFactory(ids));

	//
	// Get and destroy group `g3'
	//
	g3 = factory_ -> get("g3");
	g3 -> destroy();

	//
	// Should have 0 groups
	//
	ids.length(0);
	TEST(checkGroupFactory(ids));
    }
    catch(const Exception& ex)
    {
	OB_CERR << ex << OB_ENDL;
	TEST(false);
    }
}
