// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#ifndef ___Tester_impl_h__
#define ___Tester_impl_h__

#include <OB/Balancer_skel.h>
#include <Tester_skel.h>
#include <MemberObject.h>

//
// Forward declaration
//
class MemberObject_impl;
class LoadAlert_impl;

//
// Macro for catching displaying errors
//
extern void TestMacro(const char* ex, const char* file, unsigned int line);
#define TEST(ex) ( (ex) ? ((void)0) : TestMacro(#ex , __FILE__, __LINE__) )

//
// test info
//
struct TestInfo
{
    LoadBalancing::Group_var group;
    LoadBalancing::StrategyProxy_var proxy;
    PortableServer::POA_var replicaPOA1;
    PortableServer::POA_var replicaPOA2;
    PortableServer::POA_var replicaPOA3;
    PortableServer::POA_var auxPOA;

    TestInfo();
    ~TestInfo();
};

//
// IDL:Tester:1.0
//
class Tester_impl : virtual public POA_Tester,
                    virtual public PortableServer::RefCountServantBase
{
    Tester_impl(const Tester_impl&);
    void operator=(const Tester_impl&);

    //
    // The ORB
    //
    CORBA::ORB_var orb_;

    //
    // The POA
    //
    PortableServer::POA_var poa_;

    //
    // The group factory
    //
    LoadBalancing::GroupFactory_var factory_;

    //
    // The stage 1 persistence test data
    //
    TestInfo** info_;

    //
    // The stage 1 administrative test data
    //
    int stage_;
    LoadAlert_impl* alert_;
    PortableServer::ServantBase_var dummy_;

    //
    // Methods for create test groups (used by the persistence tests)
    //
    TestInfo* createDefaultGroup();
    TestInfo* createAdaptiveGroup();
    TestInfo* createCustomGroup();

    //
    // Methods for checking test groups (used by the persistence tests)
    //
    bool checkDefaultGroup(TestInfo*);
    bool checkAdaptiveGroup(TestInfo*);
    bool checkCustomGroup(TestInfo*);

    //
    // Check the GroupFactory for the specified groups
    //
    bool checkGroupFactory(const LoadBalancing::GroupIdSeq& groupIds);

    //
    // Check the Group for the specified group id and members
    //
    bool checkGroup(LoadBalancing::Group_ptr group,
		    const char* groupId,
		    const LoadBalancing::MemberIdSeq& memberIds);

    //
    // Check the strategy for the specified name
    //
    bool checkStrategy(LoadBalancing::StrategyProxy_ptr proxy,
		       const char* strategyName);

    //
    // Check the least-load strategy
    //
    bool checkLeastLoadStrategy(LoadBalancing::StrategyProxy_ptr proxy,
				CORBA::ULong tolerance,
				CORBA::ULong loadPerClient,
				CORBA::ULong rejectLoad,
				CORBA::Float dampeningMultiplier,
				CORBA::ULong criticalLoad);

    //
    // Check the min-dispersion strategy
    //
    bool checkMinDispStrategy(LoadBalancing::StrategyProxy_ptr proxy,
			      CORBA::ULong tolerance,
			      CORBA::ULong loadPerClient,
			      CORBA::ULong rejectLoad,
			      CORBA::Float dampeningMultiplier);

    //
    // Check the group replicas -- must only return the replicas specified
    // in round-robin order
    //
    bool checkGroupReplicas(MemberObject_impl* impl,
			    const LoadBalancing::MemberIdSeq& memberIds);

    //
    // Check the group replicas -- must only return the replicas specified
    // in round-robin order
    //
    bool checkGroupReplicas(CORBA::Object_ptr obj,
			    const LoadBalancing::MemberIdSeq& memberIds);

    //
    // Check the next replica
    //
    bool checkNextReplica(CORBA::Object_ptr obj, const char* memberId);

    //
    // Create a replica POA
    //
    PortableServer::POA_ptr
    createReplicaPOA(const char* poaName,
		     const char* groupId,
		     const char* memberId);
    
public:

    Tester_impl(CORBA::ORB_ptr, PortableServer::POA_ptr,
		LoadBalancing::GroupFactory_ptr);
    ~Tester_impl();

    virtual PortableServer::POA_ptr _default_POA();

    //
    // IDL:Tester/test_GroupFactory:1.0
    //
    virtual void test_GroupFactory()
        throw(CORBA::SystemException);

    //
    // IDL:Tester/test_Group:1.0
    //
    virtual void test_Group()
        throw(CORBA::SystemException);

    //
    // IDL:Tester/test_RandomStrategy:1.0
    //
    virtual void test_RandomStrategy()
        throw(CORBA::SystemException);

    //
    // IDL:Tester/test_RoundRobinStrategy:1.0
    //
    virtual void test_RoundRobinStrategy()
        throw(CORBA::SystemException);

    //
    // IDL:Tester/test_LeastLoadStrategy:1.0
    //
    virtual void test_LeastLoadStrategy()
        throw(CORBA::SystemException);

    //
    // IDL:Tester/test_MinDispStrategy:1.0
    //
    virtual void test_MinDispStrategy()
        throw(CORBA::SystemException);

    //
    // IDL:Tester/test_CustomStrategy:1.0
    //
    virtual void test_CustomStrategy()
        throw(CORBA::SystemException);

    //
    // IDL:Tester/test_persistence_stage1:1.0
    //
    virtual void test_persistence_stage1()
        throw(CORBA::SystemException);

    //
    // IDL:Tester/test_persistence_stage2:1.0
    //
    virtual void test_persistence_stage2()
        throw(CORBA::SystemException);

    //
    // IDL:Tester/test_administrative_stage1:1.0
    //
    virtual void test_administrative_stage1()
        throw(CORBA::SystemException);

    //
    // IDL:Tester/test_administrative_stage2:1.0
    //
    virtual void test_administrative_stage2()
        throw(CORBA::SystemException);

    //
    // IDL:Tester/test_administrative_stage3:1.0
    //
    virtual void test_administrative_stage3()
        throw(CORBA::SystemException);

    //
    // IDL:ooc.com/shutdown:1.0
    //
    virtual void shutdown()
	throw(CORBA::SystemException);
};

#endif
