// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>

#if !defined(HAVE_SHARED) && !defined(OB_DLL)
#include <OB/OCI_init.h>
#include <OB/OCI_BiDir_init.h>
#endif

#include <Hello_skel.h>

#include <stdlib.h>

#ifdef HAVE_STD_IOSTREAM
using namespace std;
#endif

class HelloCallback_impl : public POA_HelloCallback,
                           public PortableServer::RefCountServantBase
{
    PortableServer::POA_var poa_;

public:

    HelloCallback_impl(PortableServer::POA_ptr poa)
        : poa_(PortableServer::POA::_duplicate(poa))
    {
    }

    virtual PortableServer::POA_ptr
    _default_POA()
    {
        return PortableServer::POA::_duplicate(poa_);
    }

    virtual void
    hello_callback()
        throw(CORBA::SystemException)
    {
        cout << "Callback says Hello World!" << endl;
    }
};

int
run(CORBA::ORB_ptr orb, int /* argc */, char* argv[])
{
    //
    // Resolve Root POA
    //
    CORBA::Object_var poaObj = orb -> resolve_initial_references("RootPOA");
    PortableServer::POA_var root = PortableServer::POA::_narrow(poaObj);
    PortableServer::POAManager_var manager = root -> the_POAManager();

    //
    // Create callback implementation object
    //
    HelloCallback_impl* cbImpl = new HelloCallback_impl(root);
    PortableServer::ServantBase_var cbVar = cbImpl;
    HelloCallback_var cb = cbImpl -> _this();

    //
    // Get "hello" object
    //
    CORBA::Object_var obj = orb -> string_to_object("relfile:/Hello.ref");
    if(CORBA::is_nil(obj))
    {
        cerr << argv[0] << ": cannot read IOR from Hello.ref" << endl;
        return EXIT_FAILURE;
    }

    Hello_var hello = Hello::_narrow(obj);
    assert(!CORBA::is_nil(hello));

    //
    // Activate the POAManager so that callbacks will be dispatched
    //
    manager -> activate();

    //
    // Main loop
    //
    cout << "Enter 'h' for hello, 's' to shutdown server or 'x' for exit:\n";
    char c;
    do
    {
        cout << "> ";
        cin >> c;
        if(c == 'h')
            hello -> say_hello(cb);
        else if(c == 's')
            hello -> deactivate();
    }
    while(cin.good() && c != 'x');

    return EXIT_SUCCESS;
}

int
main(int argc, char* argv[], char*[])
{
    int status = EXIT_SUCCESS;
    CORBA::ORB_var orb;

    try
    {
#if !defined(HAVE_SHARED) && !defined(OB_DLL)
        //
        // When linking statically, we need to explicitly register
        // the plug-in prior to ORB initialization
        //
        OCI::register_plugin("bidir", OCI_init_bidir);
#endif

        //
        // Initialize the ORB
        //
        orb = CORBA::ORB_init(argc, argv);

        status = run(orb, argc, argv);
    }
    catch(const CORBA::Exception& ex)
    {
        cerr << ex << endl;
        status = EXIT_FAILURE;
    }

    if(!CORBA::is_nil(orb))
    {
        try
        {
            orb -> destroy();
        }
        catch(const CORBA::Exception& ex)
        {
            cerr << ex << endl;
            status = EXIT_FAILURE;
        }
    }

    return status;
}
