// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#ifndef OCI_BIDIR_IMPL_H
#define OCI_BIDIR_IMPL_H

#include <OB/OCI_BiDir.h>
#include <OB/BiDirIOP.h>
#include <OB/URL.h>

#include <ActiveTransport.h>
#include <TransportManager_fwd.h>

namespace OCI
{

namespace BiDir
{

//
// Declare classes
//
class CorbalocProtocol_impl;
class Plugin_impl;
class AccFactoryInfo_impl;
class AccFactory_impl;
class AcceptorInfo_impl;
class Acceptor_impl;
class CallbackAcceptor_impl;
class CallbackTransportInfo_impl;
class CallbackTransport_impl;
class Connector_impl;
class ConnectorInfo_impl;
class ConFactory_impl;
class ConFactoryInfo_impl;
class SelfTransport_impl;
class SelfTransportInfo_impl;
class Transport_impl;
class TransportInfo_impl;

//
// The protocol object for parsing corbaloc URL addresses
//
class CorbalocProtocol_impl : public OB::CorbalocProtocol,
                              public OBCORBA::RefCountLocalObject
{
    CorbalocProtocol_impl(const CorbalocProtocol_impl&);
    void operator=(const CorbalocProtocol_impl&);

    CORBA::String_var id_;
    ProfileId tag_;
    OB::CorbalocProtocol_var delegate_;

public:

    CorbalocProtocol_impl(const char* id, ProfileId tag,
                          OB::CorbalocProtocol_ptr delegate);
    ~CorbalocProtocol_impl();

    virtual char* name()
        throw(CORBA::SystemException);

    virtual IOP::TaggedProfile*
    parse_address(const char* addr,
                  const OB::CorbalocProtocol::ObjectKey& key)
        throw(CORBA::SystemException);

    virtual void destroy()
        throw(CORBA::SystemException);
};

class Plugin_impl : public Plugin, public OBCORBA::RefCountLocalObject
{
    //
    // Hide copy-constructor and assignment operator
    //
    Plugin_impl(const Plugin_impl&);
    void operator=(const Plugin_impl&);

    CORBA::ORB_var orb_; // The ORB
    CORBA::String_var peer_; // The peer id

public:

    Plugin_impl(CORBA::ORB_ptr, const char*);
    virtual ~Plugin_impl();

    //
    // Standard IDL to C++ Mapping
    //
    virtual PluginId id()
        throw(CORBA::SystemException);

    virtual ProfileId tag()
        throw(CORBA::SystemException);

    virtual void init_client(const ParamSeq&)
        throw(CORBA::SystemException);

    virtual void init_server(const ParamSeq&)
        throw(CORBA::SystemException);
};

class AccFactoryInfo_impl : public AccFactoryInfo,
                            public OBCORBA::RefCountLocalObject
{
    //
    // Hide copy-constructor and assignment operator
    //
    AccFactoryInfo_impl(const AccFactoryInfo_impl&);
    void operator=(const AccFactoryInfo_impl&);

    //
    // The plugin id
    //
    PluginId_var id_;

    //
    // The profile tag
    //
    ProfileId tag_;

    //
    // AccFactory information delegate
    //
    OCI::AccFactoryInfo_var delegate_;

public:
    AccFactoryInfo_impl(const char*, ProfileId, OCI::AccFactoryInfo_ptr);
    virtual ~AccFactoryInfo_impl();

    //
    // Standard IDL to C++ Mapping
    //
    virtual PluginId id()
        throw(CORBA::SystemException);

    virtual ProfileId tag()
        throw(CORBA::SystemException);

    virtual char* describe()
        throw(CORBA::SystemException);
};

class AccFactory_impl : public AccFactory, public OBCORBA::RefCountLocalObject
{
    //
    // The acceptor factory delegate
    //
    AccFactory_var delegate_;

    //
    // The TransportManager
    //
    TransportManager_var transportManager_;

    //
    // AccFactory information
    //
    AccFactoryInfo_impl* info_;

public:
    AccFactory_impl(AccFactory_ptr, TransportManager_ptr);
    virtual ~AccFactory_impl();

    //
    // Standard IDL to C++ Mapping
    //
    virtual PluginId id()
        throw(CORBA::SystemException);

    virtual ProfileId tag()
        throw(CORBA::SystemException);

    virtual Acceptor_ptr create_acceptor(const ParamSeq&)
        throw(InvalidParam,
              CORBA::SystemException);

    virtual void change_key(IOP::IOR&, const ObjectKey&)
        throw(CORBA::SystemException);

    virtual OCI::AccFactoryInfo_ptr get_info()
        throw(CORBA::SystemException);

    //
    // ORBacus internal functions
    // Application programs must not use these functions directly
    //
    TransportManager_ptr _OB_transportManager();
};

class AcceptorInfo_impl : public AcceptorInfo,
                          public JTCRecursiveMutex,
                          public OBCORBA::RefCountLocalObject
{
    //
    // Hide copy-constructor and assignment operator
    //
    AcceptorInfo_impl(const AcceptorInfo_impl&);
    void operator=(const AcceptorInfo_impl&);

    //
    // The plugin id
    //
    PluginId_var id_;

    //
    // The profile tag
    //
    ProfileId tag_;

    //
    // The local peer id
    //
    CORBA::String_var localPeer_;

    //
    // Acceptor information delegate
    //
    OCI::AcceptorInfo_var delegate_;

    //
    // All accept callback objects
    //
    OB_STL::list< AcceptCB_var > acceptCBs_;

public:

    AcceptorInfo_impl(const char*, ProfileId, const char*,
                      OCI::AcceptorInfo_ptr);
    ~AcceptorInfo_impl();

    //
    // Standard IDL to C++ Mapping
    //
    virtual PluginId id()
        throw(CORBA::SystemException);

    virtual ProfileId tag()
        throw(CORBA::SystemException);

    virtual char* describe()
        throw(CORBA::SystemException);

    virtual void add_accept_cb(AcceptCB_ptr)
        throw(CORBA::SystemException);

    virtual void remove_accept_cb(AcceptCB_ptr)
        throw(CORBA::SystemException);

    //
    // ORBacus internal functions
    // Application programs must not use these functions directly
    //
    void _OB_callAcceptCB(OCI::TransportInfo_ptr);
};

class Acceptor_impl : public Acceptor, public OBCORBA::RefCountLocalObject
{
    //
    // Hide copy-constructor and assignment operator
    //
    Acceptor_impl(const Acceptor_impl&);
    void operator=(const Acceptor_impl&);

    //
    // The TransportManager
    //
    TransportManager_var transportManager_;

    //
    // The Acceptor delegate
    //
    Acceptor_var delegate_;

    //
    // Acceptor information
    //
    AcceptorInfo_impl* info_;

public:
    Acceptor_impl(TransportManager_ptr, Acceptor_ptr);
    ~Acceptor_impl();

    //
    // Standard IDL to C++ Mapping
    //
    virtual PluginId id()
        throw(CORBA::SystemException);

    virtual ProfileId tag()
        throw(CORBA::SystemException);

    virtual Handle handle()
        throw(CORBA::SystemException);

    virtual void close()
        throw(CORBA::SystemException);

    virtual void shutdown()
        throw(CORBA::SystemException);

    virtual void listen()
        throw(CORBA::SystemException);

    virtual Transport_ptr accept(CORBA::Boolean)
        throw(CORBA::SystemException);

    virtual Transport_ptr connect_self()
        throw(CORBA::SystemException);

    virtual void add_profiles(const ProfileInfo&, OCI::IOR&)
        throw(CORBA::SystemException);

    virtual ProfileInfoSeq* get_local_profiles(const OCI::IOR&)
        throw(CORBA::SystemException);

    virtual OCI::AcceptorInfo_ptr get_info()
        throw(CORBA::SystemException);
};

class CallbackAcceptor_impl : public Acceptor,
                              public OBCORBA::RefCountLocalObject
{
    //
    // Hide copy-constructor and assignment operator
    //
    CallbackAcceptor_impl(const CallbackAcceptor_impl&);
    void operator=(const CallbackAcceptor_impl&);

    //
    // The TransportManager
    //
    TransportManager_var transportManager_;

    //
    // Acceptor information
    //
    AcceptorInfo_impl* info_;

    //
    // Transports waiting to be accepted
    //
    OB_STL::list< Transport_var > transports_;
    JTCMonitor transportsMonitor_;

public:
    CallbackAcceptor_impl(TransportManager_ptr);
    ~CallbackAcceptor_impl();

    //
    // Standard IDL to C++ Mapping
    //
    virtual PluginId id()
        throw(CORBA::SystemException);

    virtual ProfileId tag()
        throw(CORBA::SystemException);

    virtual Handle handle()
        throw(CORBA::SystemException);

    virtual void close()
        throw(CORBA::SystemException);

    virtual void shutdown()
        throw(CORBA::SystemException);

    virtual void listen()
        throw(CORBA::SystemException);

    virtual Transport_ptr accept(CORBA::Boolean)
        throw(CORBA::SystemException);

    virtual Transport_ptr connect_self()
        throw(CORBA::SystemException);

    virtual void add_profiles(const ProfileInfo&, OCI::IOR&)
        throw(CORBA::SystemException);

    virtual ProfileInfoSeq* get_local_profiles(const OCI::IOR&)
        throw(CORBA::SystemException);

    virtual OCI::AcceptorInfo_ptr get_info()
        throw(CORBA::SystemException);

    //
    // ORBacus internal functions
    // Application programs must not use these functions directly
    //
    void _OB_connect(Transport_ptr);
};

class CallbackTransportInfo_impl : public TransportInfo,
                                   public JTCRecursiveMutex,
                                   public OBCORBA::RefCountLocalObject
{
    //
    // Hide copy-constructor and assignment operator
    //
    CallbackTransportInfo_impl(const CallbackTransportInfo_impl&);
    void operator=(const CallbackTransportInfo_impl&);

    //
    // The plugin id
    //
    PluginId_var id_;

    //
    // The profile tag
    //
    ProfileId tag_;

    //
    // The local peer id
    //
    CORBA::String_var localPeer_;

    //
    // The remote peer id
    //
    CORBA::String_var remotePeer_;

    //
    // Connector information
    //
    OCI::ConnectorInfo_var connectorInfo_;

    //
    // Acceptor information
    //
    OCI::AcceptorInfo_var acceptorInfo_;

    //
    // The Transport information delegate
    //
    OCI::TransportInfo_var delegate_;

    //
    // All close callback objects
    //
    OB_STL::list<CloseCB_var> closeCBs_;

public:

    CallbackTransportInfo_impl(const char*, ProfileId, const char*,
                               const char*, Connector_ptr,
                               OCI::TransportInfo_ptr);
    CallbackTransportInfo_impl(const char*, ProfileId, const char*,
                               const char*, Acceptor_ptr,
                               OCI::TransportInfo_ptr);
    ~CallbackTransportInfo_impl();

    //
    // Standard IDL to C++ Mapping
    //
    virtual PluginId id()
        throw(CORBA::SystemException);

    virtual ProfileId tag()
        throw(CORBA::SystemException);

    virtual OCI::ConnectorInfo_ptr connector_info()
        throw(CORBA::SystemException);

    virtual OCI::AcceptorInfo_ptr acceptor_info()
        throw(CORBA::SystemException);

    virtual char* describe()
        throw(CORBA::SystemException);

    virtual void add_close_cb(CloseCB_ptr)
        throw(CORBA::SystemException);

    virtual void remove_close_cb(CloseCB_ptr)
        throw(CORBA::SystemException);

    //
    // ORBacus internal functions
    // Application programs must not use these functions directly
    //
    void _OB_callCloseCB(OCI::TransportInfo_ptr);
};

class CallbackTransport_impl : public Transport,
                               public ShutdownStatus,
                               public OBCORBA::RefCountLocalObject
{
    //
    // Hide copy-constructor and assignment operator
    //
    CallbackTransport_impl(const CallbackTransport_impl&);
    void operator=(const CallbackTransport_impl&);

    //
    // The plugin id
    //
    PluginId_var id_;

    //
    // The profile tag
    //
    ProfileId tag_;

    //
    // The active transport
    //
    ActiveTransport_var transport_;

    //
    // Transport information
    //
    CallbackTransportInfo_impl* info_;

    //
    // Have we been shutdown?
    //
    bool shutdown_;

public:

    CallbackTransport_impl(const char*, ProfileId, ActiveTransport_ptr,
                           Connector_ptr);
    CallbackTransport_impl(const char*, ProfileId, ActiveTransport_ptr,
                           Acceptor_ptr);
    ~CallbackTransport_impl();

    //
    // Standard IDL to C++ Mapping
    //
    virtual PluginId id()
        throw(CORBA::SystemException);

    virtual ProfileId tag()
        throw(CORBA::SystemException);

    virtual SendReceiveMode mode()
        throw(CORBA::SystemException);

    virtual Handle handle()
        throw(CORBA::SystemException);

    virtual void close()
        throw(CORBA::SystemException);

    virtual void shutdown()
        throw(CORBA::SystemException);

    virtual void receive(Buffer_ptr, CORBA::Boolean)
        throw(CORBA::SystemException);

    virtual CORBA::Boolean receive_detect(Buffer_ptr, CORBA::Boolean)
        throw(CORBA::SystemException);

    virtual void receive_timeout(Buffer_ptr, CORBA::ULong)
        throw(CORBA::SystemException);

    virtual CORBA::Boolean receive_timeout_detect(Buffer_ptr,
                                                  CORBA::ULong)
        throw(CORBA::SystemException);

    virtual void send(Buffer_ptr, CORBA::Boolean)
        throw(CORBA::SystemException);

    virtual CORBA::Boolean send_detect(Buffer_ptr, CORBA::Boolean)
        throw(CORBA::SystemException);

    virtual void send_timeout(Buffer_ptr, CORBA::ULong)
        throw(CORBA::SystemException);

    virtual CORBA::Boolean send_timeout_detect(Buffer_ptr, CORBA::ULong)
        throw(CORBA::SystemException);

    virtual OCI::TransportInfo_ptr get_info()
        throw(CORBA::SystemException);

    //
    // ShutdownStatus
    //
    bool isShutdown() { return shutdown_; }
};

class ConFactoryInfo_impl : public ConFactoryInfo,
                            public JTCRecursiveMutex,
                            public OBCORBA::RefCountLocalObject
{
    //
    // Hide copy-constructor and assignment operator
    //
    ConFactoryInfo_impl(const ConFactoryInfo_impl&);
    void operator=(const ConFactoryInfo_impl&);

    //
    // The plugin id
    //
    PluginId_var id_;

    //
    // The profile tag
    //
    ProfileId tag_;

    //
    // The local peer id
    //
    CORBA::String_var localPeer_;

    //
    // Acceptor information delegate
    //
    OCI::ConFactoryInfo_var delegate_;

    //
    // All connect callback objects
    //
    OB_STL::list<ConnectCB_var> connectCBs_;

public:

    ConFactoryInfo_impl(const char*, ProfileId, const char*,
                        OCI::ConFactoryInfo_ptr);
    ~ConFactoryInfo_impl();

    //
    // Standard IDL to C++ Mapping
    //
    virtual PluginId id()
        throw(CORBA::SystemException);

    virtual ProfileId tag()
        throw(CORBA::SystemException);

    virtual char* describe()
        throw(CORBA::SystemException);

    virtual void add_connect_cb(ConnectCB_ptr)
        throw(CORBA::SystemException);

    virtual void remove_connect_cb(ConnectCB_ptr)
        throw(CORBA::SystemException);

    //
    // ORBacus internal functions
    // Application programs must not use these functions directly
    //
    ConnectCBSeq* _OB_getConnectCBSeq();
};

class ConFactory_impl : public ConFactory, public OBCORBA::RefCountLocalObject
{
    //
    // Hide copy-constructor and assignment operator
    //
    ConFactory_impl(const ConFactory_impl&);
    void operator=(const ConFactory_impl&);

    //
    // The ConFactory delegate
    //
    ConFactory_var delegate_;

    //
    // The TransportManager
    //
    TransportManager_var transportManager_;

    //
    // ConFactory information
    //
    ConFactoryInfo_impl* info_;

    bool compareBodies(const BiDirIOP::ProfileBody&,
                       const BiDirIOP::ProfileBody&);

public:

    ConFactory_impl(ConFactory_ptr, TransportManager_ptr);
    ~ConFactory_impl();

    //
    // Standard IDL to C++ Mapping
    //
    virtual PluginId id()
        throw(CORBA::SystemException);

    virtual ProfileId tag()
        throw(CORBA::SystemException);

    virtual char* describe_profile(const IOP::TaggedProfile&)
        throw(CORBA::SystemException);

    virtual ConnectorSeq* create_connectors(const OCI::IOR&,
                                                 const CORBA::PolicyList&)
        throw(CORBA::SystemException);

    virtual CORBA::Boolean equivalent(const OCI::IOR&, const OCI::IOR&)
        throw(CORBA::SystemException);

    virtual CORBA::ULong hash(const OCI::IOR&, CORBA::ULong)
        throw(CORBA::SystemException);

    virtual OCI::ConFactoryInfo_ptr get_info()
        throw(CORBA::SystemException);

    //
    // ORBacus internal functions
    // Application programs must not use these functions directly
    //
    TransportManager_ptr _OB_transportManager();
};

class ConnectorInfo_impl : public ConnectorInfo,
                           public JTCRecursiveMutex,
                           public OBCORBA::RefCountLocalObject
{
    //
    // Hide copy-constructor and assignment operator
    //
    ConnectorInfo_impl(const ConnectorInfo_impl&);
    void operator=(const ConnectorInfo_impl&);

    //
    // The plugin id
    //
    PluginId_var id_;

    //
    // The profile tag
    //
    ProfileId tag_;

    //
    // The local peer id
    //
    CORBA::String_var localPeer_;

    //
    // The remote peer id
    //
    CORBA::String_var remotePeer_;

    //
    // The ConnectorInfo delegate
    //
    OCI::ConnectorInfo_var delegate_;

    //
    // All connect callback objects
    //
    OB_STL::list< ConnectCB_var > connectCBs_;

public:

    ConnectorInfo_impl(const char*, ProfileId, const char*,
                       const char*, const ConnectCBSeq&,
                       OCI::ConnectorInfo_ptr);
    ~ConnectorInfo_impl();

    //
    // Standard IDL to C++ Mapping
    //
    virtual PluginId id()
        throw(CORBA::SystemException);

    virtual ProfileId tag()
        throw(CORBA::SystemException);

    virtual char* describe()
        throw(CORBA::SystemException);

    virtual void add_connect_cb(ConnectCB_ptr)
        throw(CORBA::SystemException);

    virtual void remove_connect_cb(ConnectCB_ptr)
        throw(CORBA::SystemException);

    //
    // ORBacus internal functions
    // Application programs must not use these functions directly
    //
    void _OB_callConnectCB(OCI::TransportInfo_ptr);
};

class Connector_impl : public Connector, public OBCORBA::RefCountLocalObject
{
    //
    // Hide copy-constructor and assignment operator
    //
    Connector_impl(const Connector_impl&);
    void operator=(const Connector_impl&);

    //
    // The TransportManager
    //
    TransportManager_var transportManager_;

    //
    // The profile body
    //
    BiDirIOP::ProfileBody profile_;

    //
    // The Connector delegate
    //
    Connector_var delegate_;

    //
    // Connector information
    //
    ConnectorInfo_impl* info_;

public:

    Connector_impl(TransportManager_ptr, const BiDirIOP::ProfileBody&,
                   const ConnectCBSeq&, Connector_ptr);
    ~Connector_impl();

    //
    // Standard IDL to C++ Mapping
    //
    virtual PluginId id()
        throw(CORBA::SystemException);

    virtual ProfileId tag()
        throw(CORBA::SystemException);

    virtual Transport_ptr connect()
        throw(CORBA::SystemException);

    virtual Transport_ptr connect_timeout(CORBA::ULong)
        throw(CORBA::SystemException);

    virtual ProfileInfoSeq* get_usable_profiles(const OCI::IOR&,
                                                     const CORBA::PolicyList&)
        throw(CORBA::SystemException);

    virtual CORBA::Boolean equal(Connector_ptr)
        throw(CORBA::SystemException);

    virtual OCI::ConnectorInfo_ptr get_info()
        throw(CORBA::SystemException);
};

class SelfTransportInfo_impl : public TransportInfo,
                               public OBCORBA::RefCountLocalObject
{
    //
    // Hide copy-constructor and assignment operator
    //
    SelfTransportInfo_impl(const SelfTransportInfo_impl&);
    void operator=(const SelfTransportInfo_impl&);

public:

    SelfTransportInfo_impl();
    ~SelfTransportInfo_impl();

    //
    // Standard IDL to C++ Mapping
    //
    virtual PluginId id()
        throw(CORBA::SystemException);

    virtual ProfileId tag()
        throw(CORBA::SystemException);

    virtual OCI::ConnectorInfo_ptr connector_info()
        throw(CORBA::SystemException);

    virtual OCI::AcceptorInfo_ptr acceptor_info()
        throw(CORBA::SystemException);

    virtual char* describe()
        throw(CORBA::SystemException);

    virtual void add_close_cb(CloseCB_ptr)
        throw(CORBA::SystemException);

    virtual void remove_close_cb(CloseCB_ptr)
        throw(CORBA::SystemException);
};

class SelfTransport_impl : public Transport,
                           public JTCMonitor,
                           public OBCORBA::RefCountLocalObject
{
    //
    // Hide copy-constructor and assignment operator
    //
    SelfTransport_impl(const SelfTransport_impl&);
    void operator=(const SelfTransport_impl&);

    //
    // The transport info
    //
    SelfTransportInfo_impl* info_;

    //
    // Have we been closed?
    //
    bool closed_;

public:

    SelfTransport_impl();
    ~SelfTransport_impl();

    //
    // Standard IDL to C++ Mapping
    //
    virtual PluginId id()
        throw(CORBA::SystemException);

    virtual ProfileId tag()
        throw(CORBA::SystemException);

    virtual SendReceiveMode mode()
        throw(CORBA::SystemException);

    virtual Handle handle()
        throw(CORBA::SystemException);

    virtual void close()
        throw(CORBA::SystemException);

    virtual void shutdown()
        throw(CORBA::SystemException);

    virtual void receive(Buffer_ptr, CORBA::Boolean)
        throw(CORBA::SystemException);

    virtual CORBA::Boolean receive_detect(Buffer_ptr, CORBA::Boolean)
        throw(CORBA::SystemException);

    virtual void receive_timeout(Buffer_ptr, CORBA::ULong)
        throw(CORBA::SystemException);

    virtual CORBA::Boolean receive_timeout_detect(Buffer_ptr, CORBA::ULong)
        throw(CORBA::SystemException);

    virtual void send(Buffer_ptr, CORBA::Boolean)
        throw(CORBA::SystemException);

    virtual CORBA::Boolean send_detect(Buffer_ptr, CORBA::Boolean)
        throw(CORBA::SystemException);

    virtual void send_timeout(Buffer_ptr, CORBA::ULong)
        throw(CORBA::SystemException);

    virtual CORBA::Boolean send_timeout_detect(Buffer_ptr, CORBA::ULong)
        throw(CORBA::SystemException);

    virtual OCI::TransportInfo_ptr get_info()
        throw(CORBA::SystemException);
};

class TransportInfo_impl : public TransportInfo,
                           public JTCRecursiveMutex,
                           public OBCORBA::RefCountLocalObject
{
    //
    // Hide copy-constructor and assignment operator
    //
    TransportInfo_impl(const TransportInfo_impl&);
    void operator=(const TransportInfo_impl&);

    //
    // The plugin id
    //
    PluginId_var id_;

    //
    // The profile tag
    //
    ProfileId tag_;

    //
    // The local peer id
    //
    CORBA::String_var localPeer_;

    //
    // The remote peer id
    //
    CORBA::String_var remotePeer_;

    //
    // Connector information
    //
    OCI::ConnectorInfo_var connectorInfo_;

    //
    // Acceptor information
    //
    OCI::AcceptorInfo_var acceptorInfo_;

    //
    // The Transport information delegate
    //
    OCI::TransportInfo_var delegate_;

    //
    // All close callback objects
    //
    OB_STL::list<CloseCB_var> closeCBs_;

public:

    TransportInfo_impl(const char*, ProfileId, const char*,
                       const char*, Connector_ptr,
                       OCI::TransportInfo_ptr);
    TransportInfo_impl(const char*, ProfileId, const char*,
                       const char*, Acceptor_ptr,
                       OCI::TransportInfo_ptr);
    ~TransportInfo_impl();

    //
    // Standard IDL to C++ Mapping
    //
    virtual PluginId id()
        throw(CORBA::SystemException);

    virtual ProfileId tag()
        throw(CORBA::SystemException);

    virtual OCI::ConnectorInfo_ptr connector_info()
        throw(CORBA::SystemException);

    virtual OCI::AcceptorInfo_ptr acceptor_info()
        throw(CORBA::SystemException);

    virtual char* describe()
        throw(CORBA::SystemException);

    virtual void add_close_cb(CloseCB_ptr)
        throw(CORBA::SystemException);

    virtual void remove_close_cb(CloseCB_ptr)
        throw(CORBA::SystemException);

    //
    // ORBacus internal functions
    // Application programs must not use these functions directly
    //
    void _OB_callCloseCB(OCI::TransportInfo_ptr);
};

class Transport_impl : public Transport, public OBCORBA::RefCountLocalObject
{
    //
    // Hide copy-constructor and assignment operator
    //
    Transport_impl(const Transport_impl&);
    void operator=(const Transport_impl&);

    //
    // The plugin id
    //
    PluginId_var id_;

    //
    // The profile tag
    //
    ProfileId tag_;

    //
    // The active transport
    //
    ActiveTransport_var transport_;

    //
    // Transport information
    //
    TransportInfo_impl* info_;

public:

    Transport_impl(const char*, ProfileId, ActiveTransport_ptr, Connector_ptr);
    Transport_impl(const char*, ProfileId, ActiveTransport_ptr, Acceptor_ptr);
    ~Transport_impl();

    //
    // Standard IDL to C++ Mapping
    //
    virtual PluginId id()
        throw(CORBA::SystemException);

    virtual ProfileId tag()
        throw(CORBA::SystemException);

    virtual SendReceiveMode mode()
        throw(CORBA::SystemException);

    virtual Handle handle()
        throw(CORBA::SystemException);

    virtual void close()
        throw(CORBA::SystemException);

    virtual void shutdown()
        throw(CORBA::SystemException);

    virtual void receive(Buffer_ptr, CORBA::Boolean)
        throw(CORBA::SystemException);

    virtual CORBA::Boolean receive_detect(Buffer_ptr, CORBA::Boolean)
        throw(CORBA::SystemException);

    virtual void receive_timeout(Buffer_ptr, CORBA::ULong)
        throw(CORBA::SystemException);

    virtual CORBA::Boolean receive_timeout_detect(Buffer_ptr, CORBA::ULong)
        throw(CORBA::SystemException);

    virtual void send(Buffer_ptr, CORBA::Boolean)
        throw(CORBA::SystemException);

    virtual CORBA::Boolean send_detect(Buffer_ptr, CORBA::Boolean)
        throw(CORBA::SystemException);

    virtual void send_timeout(Buffer_ptr, CORBA::ULong)
        throw(CORBA::SystemException);

    virtual CORBA::Boolean send_timeout_detect(Buffer_ptr, CORBA::ULong)
        throw(CORBA::SystemException);

    virtual OCI::TransportInfo_ptr get_info()
        throw(CORBA::SystemException);
};

} // End of namespace BiDir

} // End of namespace OCI

#endif
