// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>
#include <OB/OCI_BiDir_init.h>
#include <OB/Properties.h>
#include <OB/Timer.h>

#include <OCI_BiDir_impl.h>

using namespace CORBA;
using namespace OB;

static const char* BIDIRversion = "1.0";

// ----------------------------------------------------------------------
// Global functions
// ----------------------------------------------------------------------

extern "C"
void
OCI_version_bidir(CORBA::ORB_ptr /* orb */, const char* version)
{
    if(strcmp(BIDIRversion, version) != 0)
        throw INITIALIZE("bidir: incompatible OCI version");
}

extern "C"
OCI::Plugin_ptr
OCI_init_bidir(CORBA::ORB_ptr orb, int& ac, char** av)
{
    OBCORBA::ORB_var oborb = OBCORBA::ORB::_narrow(orb);
    Properties_var props = oborb -> properties();

    //
    // Parse arguments
    //
    OCI_parse_args_bidir(ac, av, props);

    const char* peer = props -> getProperty("ooc.bidir.peer");
    String_var str;
    if(peer == 0)
    {
        //
        // Compose a default peer ID from the current time
        //
        timeval now = TimeNow();
        str = string_dup("");
        str += now.tv_sec;
        str += '.';
        str += now.tv_usec;
        peer = str;
    }

    return new OCI::BiDir::Plugin_impl(orb, peer);
}

void
OCI_parse_args_bidir(int& ac, char** av, Properties_ptr props)
{
    //
    // Process the command-line arguments
    //
    int i = 0;
    while(i < ac)
    {
        if(strcmp(av[i], "-BIDIRpeer") == 0)
        {
            if(i + 1 >= ac)
                throw INITIALIZE("bidir: argument expected for -BIDIRpeer");

            props -> setProperty("ooc.bidir.peer", av[i + 1]);

            for(int j = i ; j + 2 < ac ; j++)
                av[j] = av[j + 2];

            ac -= 2;
        }
        else if(strncmp(av[i], "-BIDIR", 6) == 0)
        {
            String_var err = string_dup("bidir: unknown option `");
            err += av[i];
            err += "'";
            throw INITIALIZE(err);
        }
        else
            i++;
    }
}
