// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************
 
#ifndef ADMIN_BASE_H
#define ADMIN_BASE_H

//
// Forward declarations.
//
class ProxyBase;
typedef OB_STL::list< ProxyBase* > ProxyPtrList;

class PipeSegment;
typedef PipeSegment* PipeSegment_ptr;

class Reaper;
struct ProxyConstructorInfo;

class AdminBase : public JTCRecursiveMutex,
		  public PortableServer::RefCountServantBase
{
    //
    // No copy constructor/assignment operator.
    //
    AdminBase(const AdminBase&);
    void operator=(const AdminBase&);

protected:

    CORBA::ORB_var orb_; // The ORB
    
    PortableServer::POA_var poa_; // The POA

    OB::EventLogger_var logger_; // The logger

    PipeSegment_ptr myPipe_; // My pipe segment

    ProxyPtrList proxies_; // Set of ProxyConsumers

    CORBA::ULong nextId_; // ID of the next assigned proxy

    bool destroyed_; // Has this admin been destroyed?

    Reaper* reaper_; // The reaper

    virtual void deactivateObject() = 0;

    void addProxy(ProxyBase*);

    void reap();

    void fillProxyConstructorInfo(ProxyConstructorInfo&);

public:

    AdminBase(CORBA::ORB_ptr, PortableServer::POA_ptr, OB::EventLogger_ptr,
    	      Reaper*, PipeSegment_ptr);
    virtual ~AdminBase();

    //
    // Standard IDL to C++ Mapping.
    //
    
    //
    // Internal methods.
    //


    //
    // Called from the event channel upon destruction
    //
    void disconnect();

    //
    // Stop accepting new requests
    //
    void discardRequests();

    //
    // Get the default POA
    //
    virtual PortableServer::POA_ptr _default_POA();
};

#endif
