// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>

#include <OB/CosEventChannelAdmin_skel.h>

#include <EventLogger.h>
#include <PropagatorBaseThread.h> // SGI 7.0/7.1 needs this, and PusherBase...
#include <PusherBaseThread.h>
#include <PipeSegment.h>
#include <ProxyBase.h>
#include <EC_ProxyPushSupplier_impl.h>
#include <EC_ProxyPullSupplier_impl.h>
#include <AdminBase.h>

#include <EC_ConsumerAdmin_impl.h>

using namespace CORBA;

// ----------------------------------------------------------------------
// EC_ConsumerAdmin_impl private member implementation
// ----------------------------------------------------------------------

void
EC_ConsumerAdmin_impl::deactivateObject()
{
    PortableServer::POA_var poa = _default_POA();
    PortableServer::ObjectId_var oid = poa -> servant_to_id(this);
    poa -> deactivate_object(oid.in());
}

// ----------------------------------------------------------------------
// EC_ConsumerAdmin_impl constructor and destructor
// ----------------------------------------------------------------------

EC_ConsumerAdmin_impl::EC_ConsumerAdmin_impl(
    ORB_ptr orb,
    PortableServer::POA_ptr poa,
    OB::EventLogger_ptr logger,
    Reaper* reaper,
    PipeSegment_ptr pipe)
    : AdminBase(orb, poa, logger, reaper, pipe)
{
    if(logger_ -> traceLifecycle() >= 1)
        logger_ -> trace("event", "creating ConsumerAdmin");
}

EC_ConsumerAdmin_impl::~EC_ConsumerAdmin_impl()
{
    if(logger_ -> traceLifecycle() >= 1)
        logger_ -> trace("event", "destroying ConsumerAdmin");
    assert(proxies_.size() == 0);
}

// ----------------------------------------------------------------------
// Standard IDL to C++ mapping
// ----------------------------------------------------------------------

CosEventChannelAdmin::ProxyPushSupplier_ptr
EC_ConsumerAdmin_impl::obtain_push_supplier()
    throw(SystemException)
{
    JTCSynchronized sync(*this);

    if(destroyed_)
        throw OBJECT_NOT_EXIST();
    
    ProxyConstructorInfo info;
    fillProxyConstructorInfo(info);

    EC_ProxyPushSupplier_impl* p = new EC_ProxyPushSupplier_impl(info);
    addProxy(p);

    return p -> _this();
}

CosEventChannelAdmin::ProxyPullSupplier_ptr
EC_ConsumerAdmin_impl::obtain_pull_supplier()
    throw(SystemException)
{
    JTCSynchronized sync(*this);

    if(destroyed_)
        throw OBJECT_NOT_EXIST();
    
    ProxyConstructorInfo info;
    fillProxyConstructorInfo(info);

    EC_ProxyPullSupplier_impl* p = new EC_ProxyPullSupplier_impl(info);
    addProxy(p);

    return p -> _this();
}
