// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>

#include <OB/CosEventChannelAdmin_skel.h>

#include <EventLogger.h>
#include <Event_impl.h>
#include <EventUtil.h>
#include <PipeSegment.h>
#include <ProxyBase.h>

#include <EC_ProxyPushConsumer_impl.h>

using namespace CORBA;

// ----------------------------------------------------------------------
// EC_ProxyPushConsumer_impl constructor/destructor
// ----------------------------------------------------------------------

EC_ProxyPushConsumer_impl::EC_ProxyPushConsumer_impl(
    ProxyConstructorInfo& info)
{
    info.base = "ProxyPushConsumer";
    ProxyBase::init(info);
}

EC_ProxyPushConsumer_impl::~EC_ProxyPushConsumer_impl()
{
}

// ----------------------------------------------------------------------
// Standard IDL to C++ Mapping
// ----------------------------------------------------------------------

void
EC_ProxyPushConsumer_impl::connect_push_supplier(
    CosEventComm::PushSupplier_ptr supplier)
    throw(CosEventChannelAdmin::AlreadyConnected, SystemException)
{
    JTCSynchronized sync(*this);

    if(state_ == Destroyed)
        throw OBJECT_NOT_EXIST();

    if(state_ == Discarding)
        throw TRANSIENT();

    if(state_ == Connected)
	throw CosEventChannelAdmin::AlreadyConnected();

    //
    // The supplier is permitted to be nil
    //
    if(!is_nil(supplier))
    {
	QoSParameters params;
	resolveQoSParams(params);
	
	supplier_ = CosEventComm::PushSupplier::_narrow(
	    CORBA::Object_var(applyRequestTimeout(supplier, params))
	);
    }

    stampRequest();
    state_ = Connected;
}

void
EC_ProxyPushConsumer_impl::disconnect_push_consumer()
    throw(SystemException)
{
    JTCSynchronized sync(*this);

    if(state_ == Destroyed)
	throw OBJECT_NOT_EXIST();

    if(state_ == Discarding)
	throw TRANSIENT();

    disconnectImpl();
}

void
EC_ProxyPushConsumer_impl::push(const Any& any)
    throw(CosEventComm::Disconnected, SystemException)
{
    JTCSynchronized sync(*this);

    if(state_ == Destroyed)
	throw OBJECT_NOT_EXIST();

    if(state_ == Discarding)
	throw TRANSIENT();

    if(state_ != Connected)
	throw CosEventComm::Disconnected();

    stampRequest();

    Event_impl* event = new Event_impl(logger_, any, id_);

    parentPipe_ -> receive(event);
    event -> discard();
}

// ----------------------------------------------------------------------
// ProxyBase implementation
// ----------------------------------------------------------------------

void
EC_ProxyPushConsumer_impl::deactivateObject()
{
    PortableServer::POA_var poa = _default_POA();
    PortableServer::ObjectId_var oid = poa -> servant_to_id(this);
    poa -> deactivate_object(oid.in());
}

void
EC_ProxyPushConsumer_impl::disconnectPeer()
{
    //
    // If we're connected then tell the supplier.
    //
    if(!is_nil(supplier_))
    {
        try
        {
            supplier_ -> disconnect_push_supplier();
        }
        catch(SystemException&)
        {
        }
	supplier_ = CosEventComm::PushSupplier::_nil();
    }
}

bool
EC_ProxyPushConsumer_impl::canReap()
{
    return true;
}
