// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// *********************************************************************

#include <OB/CORBA.h>
#include <OB/TemplateI.h>

#include <EventLogger.h>
#include <Event_impl.h>

using namespace CORBA;

//
// The ID of the next event.
//
ULong Event_impl::nextId_ = 0;

//
// Mutex to protect the nextId_ member.
//
JTCMutex Event_impl::idMutex_;

// ----------------------------------------------------------------------
// Event_impl constructor/destructor
// ----------------------------------------------------------------------

void
Event_impl::init(ULong proxyId)
{
    {
        JTCSynchronized sync(idMutex_);
        id_ = nextId_++;
    }
    refCount_ = 1;

    if(logger_ -> traceEvents() >= 2)
    {
        String_var msg = string_dup("creating Event(");
        msg += id_;
        msg += ") from ";
        msg += proxyId;
        logger_ -> trace("event", msg);
    }
}

// ----------------------------------------------------------------------
// Event_impl constructor/destructor
// ----------------------------------------------------------------------

Event_impl::Event_impl(OB::EventLogger_ptr logger,
		       const Any& any,
		       ULong proxyId)
    : any_(any),
      logger_(OB::EventLogger::_duplicate(logger))
{
    init(proxyId);
}

Event_impl::Event_impl(OB::EventLogger_ptr logger,
		       NVList_ptr args,
		       const char* op,
                       ULong proxyId)
    : args_(NVList::_duplicate(args)),
      op_(string_dup(op)),
      logger_(OB::EventLogger::_duplicate(logger))
{
    init(proxyId);
}

Event_impl::~Event_impl()
{
    if(logger_ -> traceEvents() >= 2)
    {
        String_var msg = string_dup("destroying Event(");
        msg += id_;
        msg += ')';
        logger_ -> trace("event", msg);
    }
}

// ----------------------------------------------------------------------
// Event_impl public member functions
// ----------------------------------------------------------------------

void
Event_impl::discard()
{
    {
        JTCSynchronized sync(refMutex_);
        if(--refCount_ != 0)
            return;
    }
    delete this;
}

void
Event_impl::acquire()
{
    JTCSynchronized sync(refMutex_);
    ++refCount_;
}
