// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// *********************************************************************

#ifndef EVENT_IMPL_H
#define EVENT_IMPL_H

//
// This represents a reference counted event. An event can either be
// typed, or untyped.
//
class Event_impl
{
    //
    // No copy constructor/assignment operator.
    //
    Event_impl(const Event_impl&);
    void operator=(Event_impl&);

    //
    // For typed event.
    //
    CORBA::NVList_var args_; // The arguments.
    CORBA::String_var op_; // The operation name.

    //
    // For an untyped event.
    //
    CORBA::Any any_; // The event data.
    
    CORBA::ULong id_; // My ID.
    
    JTCMutex refMutex_;
    CORBA::UShort refCount_; // The reference count

    static JTCMutex idMutex_; 
    static CORBA::ULong nextId_; // The event identifier

    OB::EventLogger_var logger_; // The Logger

    //
    // Initialize the event
    //
    void init(CORBA::ULong);
    
public:
    
    Event_impl(OB::EventLogger_ptr, const CORBA::Any&, CORBA::ULong);
    Event_impl(OB::EventLogger_ptr, CORBA::NVList_ptr, const char*,
    	       CORBA::ULong);
    virtual ~Event_impl();

    //
    // For any events. This is doning a return by value since the 
    // marshalling of Anys is not thread safe. Thus a copy of the 
    // Any is required.
    //
    CORBA::Any any() const { return any_; }

    //
    // For typed events.
    //
    const char* op() const { return op_; }
    CORBA::NVList_ptr args() const { return args_; }
    
    void discard();
    void acquire();

    CORBA::ULong eventId() const { return id_; }
};

#endif
