// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#ifndef PIPE_SEGMENT_H
#define PIPE_SEGMENT_H

//
// The PipeSegment class
//
class PipeSegment;
typedef PipeSegment* PipeSegment_ptr;
typedef PipeSegment_ptr PipeSegmentRef;
void OBDuplicate(PipeSegment_ptr);
void OBRelease(PipeSegment_ptr);
typedef OB::ObjVar< PipeSegment > PipeSegment_var;
class OBUnique_PipeSegment { };
typedef OB::FixSeq< PipeSegment_ptr, OBUnique_PipeSegment > PipeSegmentList;

//
// Forward declarations
//
class Event_impl;

//
// A PipeSegment. An event channel is composed of a set of connected
// PipeSegments. Each segment can have 0 or 1 parents and 0, 1 or many
// children.
//
class PipeSegment : public OB::RefCount
{
    //
    // Mutex protection for the entire class. Don't use inheritance
    // since that will effect all child classes.
    //
    JTCMutex mutex_;

    //
    // The set of connected children
    //
    PipeSegmentList children_;

    //
    // The set of reaped children
    //
    PipeSegmentList reaped_;

    //
    // Is it safe to release the reaped children?
    //
    int canRelease_;

    //
    // Perform a reap of the supplied children
    //
    void reap(const PipeSegmentList&);
    
    //
    // Release all reaped children
    //
    void releaseReaped();

protected:

    //
    // Has the segment been destroyed?
    //
    bool destroy_;

public:

    PipeSegment();
    virtual ~PipeSegment();

    //
    // Standard IDL to C++ Mapping
    //
    static inline PipeSegment_ptr _duplicate(PipeSegment_ptr p)
    { if(p) p -> _OB_incRef(); return p; }
    static inline PipeSegment_ptr _nil()
    { return 0; }

    //
    // Receive an event
    //
    virtual bool receive(Event_impl*);

    //
    // Connect a PipeSegment as a child of this PipeSegment
    //
    void connect(PipeSegment_ptr);

    //
    // Destroy this PipeSegment
    //
    virtual void destroy();
};

namespace CORBA
{

inline void
release(PipeSegment_ptr p)
{
    if(p)
	p -> _OB_decRef();
}

inline Boolean
is_nil(PipeSegment_ptr p)
{
    return p == 0;
}

}

#endif
