// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// *********************************************************************

#ifndef PUSHER_BASE_THREAD_H
#define PUSHER_BASE_THREAD_H

//
// Forward declaration
//
class Event_impl;
typedef OB_STL::list<Event_impl*> EventList;

//
// This base class provides an implementation for a thread that pushes
// event data to a client.
//
class PusherBaseThread : public PropagatorBaseThread
{
    EventList events_; // The event queue.

    CORBA::ULong maxEvents_; // The maximum number of events to queue.
    
    //
    // Wait until we are terminated or for the next event to
    // arrive. Return the event if available, 0 if not.
    //
    Event_impl* waitEvent();

    //
    // Remove this event from the event queue.
    //
    void removeEvent(Event_impl*);

protected:

    //
    // This enumeration represents the possible return values from a
    // push operation.
    //
    enum PushResult
    {
        Success, // Push succeeded.
        Retry, // Run retry algorithm.
        Failed // Push failed.
    };

private:

    //
    // This method must be implemented by all classes deriving from
    // this class.
    //
    virtual PushResult pushEvent(Event_impl* event) = 0;

protected:
    
    CosEventComm::PushConsumer_var consumer_; // My push consumer.

    CosEventChannelAdmin::ProxyPushSupplier_var proxy_; // My proxy.

public:

    PusherBaseThread(CosEventComm::PushConsumer_ptr,
                     CosEventChannelAdmin::ProxyPushSupplier_ptr,
		     OB::EventLogger_ptr, CORBA::ULong,
                     CORBA::ULong, CORBA::ULong, CORBA::ULong, CORBA::Double);
    ~PusherBaseThread();

    //
    // Pass a new event to the Pusher.
    //
    void newEvent(Event_impl*);

    //
    // Run the pusher until stopped, or a push fails.
    //
    virtual void run();
};

//
// Declaration for a handle to a PusherBaseThread
//
typedef JTCHandleT< PusherBaseThread > PusherBaseThreadHandle;

#endif
