// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>
#include <OB/IFR.h>

#include <OB/CosTypedEventChannelAdmin_skel.h>

#include <EventLogger.h>
#include <Event_impl.h>
#include <PipeSegment.h>
#include <AdminBase.h>
#include <TEC_TypedConsumerAdmin_impl.h>
#include <TEC_TypedSupplierAdmin_impl.h>
#include <SetDefaultProperties.h>
#include <ChannelType.h>

#include <TEC_TypedEventChannel_impl.h>

using namespace CORBA;

// ----------------------------------------------------------------------
// TEC_TypedEventChannel_impl constructor and destructor
// ----------------------------------------------------------------------

TEC_TypedEventChannel_impl::TEC_TypedEventChannel_impl(
    ORB_ptr orb,
    PortableServer::POA_ptr poa,
    PortableServer::POA_ptr transientPOA,
    OB::EventLogger_ptr logger,
    Reaper* reaper,
    const char* myId)
    : poa_(PortableServer::POA::_duplicate(poa)),
      logger_(OB::EventLogger::_duplicate(logger)),
      myId_(myId),
      destroy_(false)
{
    myPipe_ = new PipeSegment();

    if(logger_ -> traceLifecycle() >= 1)
    {
	String_var msg = string_dup("creating TypedEventChannel(");
	msg += myId_;
	msg += ')';
	logger_ -> trace("event", msg);
    }

    type_ = new ChannelType(orb);
    consumerAdmin_ = new TEC_TypedConsumerAdmin_impl(orb, transientPOA, 
						     logger_, reaper, myPipe_, 
						     type_);
    supplierAdmin_ = new TEC_TypedSupplierAdmin_impl(orb, transientPOA, 
						     logger_, reaper, myPipe_, 
						     type_);

    SetDefaultProperties::setDefaults(orb, logger_);
}

TEC_TypedEventChannel_impl::~TEC_TypedEventChannel_impl()
{
    if(logger_ -> traceLifecycle() >= 1)
    {
	String_var msg = string_dup("destroying TypedEventChannel(");
	msg += myId_;
	msg += ')';
	logger_ -> trace("event", msg);
    }

    consumerAdmin_ -> _remove_ref();
    supplierAdmin_ -> _remove_ref();

    delete type_;
}

// ----------------------------------------------------------------------
// Standard IDL to C++ Mapping
// ----------------------------------------------------------------------

CosTypedEventChannelAdmin::TypedConsumerAdmin_ptr
TEC_TypedEventChannel_impl::for_consumers()
    throw(SystemException)
{
    JTCSynchronized sync(*this);

    if(destroy_)
	throw OBJECT_NOT_EXIST();

    return consumerAdmin_ -> _this();
}

CosTypedEventChannelAdmin::TypedSupplierAdmin_ptr
TEC_TypedEventChannel_impl::for_suppliers()
    throw(SystemException)
{
    JTCSynchronized sync(*this);

    if(destroy_)
	throw OBJECT_NOT_EXIST();

    return supplierAdmin_ -> _this();
}

void
TEC_TypedEventChannel_impl::destroy()
    throw(SystemException)
{
    JTCSynchronized sync(*this);

    if(destroy_)
	throw OBJECT_NOT_EXIST();

    destroy_ = true;

    PortableServer::ObjectId_var oid = poa_ -> servant_to_id(this);
    poa_ -> deactivate_object(oid.in());

    myPipe_ -> destroy();

    consumerAdmin_ -> disconnect();
    supplierAdmin_ -> disconnect();
}

PortableServer::POA_ptr
TEC_TypedEventChannel_impl::_default_POA()
{
    return PortableServer::POA::_duplicate(poa_);
}

// ----------------------------------------------------------------------
// Internal methods
// ----------------------------------------------------------------------

void
TEC_TypedEventChannel_impl::discardRequests()
{
    JTCSynchronized sync(*this);

    if(destroy_)
	return;

    myPipe_ -> destroy();

    consumerAdmin_ -> discardRequests();
    supplierAdmin_ -> discardRequests();
}

bool
TEC_TypedEventChannel_impl::destroyed()
{
    JTCSynchronized sync(*this);

    return destroy_;
}
