// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************
 
#ifndef COS_TYPED_EVENT_CHANNEL_IMPL_H
#define COS_TYPED_EVENT_CHANNEL_IMPL_H

//
// Forward declaration
//
class TEC_TypedConsumerAdmin_impl;
class TEC_TypedSupplierAdmin_impl;
class ChannelType;

class PipeSegment;
typedef PipeSegment* PipeSegment_ptr;
typedef OB::ObjVar< PipeSegment > PipeSegment_var;
void OBDuplicate(PipeSegment_ptr);
void OBRelease(PipeSegment_ptr);

class Reaper;

//
// The typed event channel implementation
//
class TEC_TypedEventChannel_impl :
    virtual public PortableServer::RefCountServantBase,
    public POA_CosTypedEventChannelAdmin::TypedEventChannel,
    public JTCRecursiveMutex
{
    //
    // No copy constructor/assignment operator
    //
    TEC_TypedEventChannel_impl(const TEC_TypedEventChannel_impl&);
    void operator=(const TEC_TypedEventChannel_impl&);
    
    PortableServer::POA_var poa_; // The POA

    OB::EventLogger_var logger_; // The Logger
    
    PipeSegment_var myPipe_; // My pipe segment

    ChannelType* type_; // Channel type information

    TEC_TypedConsumerAdmin_impl* consumerAdmin_; // The consumer admin
    
    TEC_TypedSupplierAdmin_impl* supplierAdmin_; // The supplier admin

    CORBA::String_var myId_; // The channel's Id

    bool destroy_; // Has the channel been destroyed?

public:

    TEC_TypedEventChannel_impl(CORBA::ORB_ptr, PortableServer::POA_ptr,
    			       PortableServer::POA_ptr, OB::EventLogger_ptr, 
			       Reaper*, const char*);
    ~TEC_TypedEventChannel_impl();

    //
    // Standard IDL to C++ Mapping
    //

    //
    // Return TypedConsumerAdmin object.
    //
    virtual CosTypedEventChannelAdmin::TypedConsumerAdmin_ptr for_consumers()
	throw(CORBA::SystemException);

    //
    // Return TypedSupplierAdmin object.
    //
    virtual CosTypedEventChannelAdmin::TypedSupplierAdmin_ptr for_suppliers()
	throw(CORBA::SystemException);

    //
    // Destroy this event channel.
    //
    virtual void destroy()
	throw(CORBA::SystemException);

    virtual PortableServer::POA_ptr
    _default_POA();
    
    //
    // Internal methods
    //

    //
    // Stop accepting new requests.
    //
    void discardRequests();

    //
    // Has the event channel been destroyed?
    //
    bool destroyed();
};

#endif
