// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>
#include <OB/IFR.h>

#include <OB/CosTypedEventChannelAdmin_skel.h>

#include <EventLogger.h>
#include <PropagatorBaseThread.h> // PusherBase needs this
#include <PusherBaseThread.h>
#include <PipeSegment.h>
#include <ProxyBase.h>
#include <TEC_TypedProxyPushConsumer_impl.h>
#include <AdminBase.h>
#include <ChannelType.h>

#include <TEC_TypedSupplierAdmin_impl.h>

using namespace CORBA;

// ----------------------------------------------------------------------
// TEC_TypedSupplierAdmin_impl private member implementation
// ----------------------------------------------------------------------

void
TEC_TypedSupplierAdmin_impl::deactivateObject()
{
    PortableServer::POA_var poa = _default_POA();
    PortableServer::ObjectId_var oid = poa -> servant_to_id(this);
    poa -> deactivate_object(oid.in());
}

// ----------------------------------------------------------------------
// TEC_TypedSupplierAdmin_impl constructor and destructor
// ----------------------------------------------------------------------

TEC_TypedSupplierAdmin_impl::TEC_TypedSupplierAdmin_impl(
    ORB_ptr orb,
    PortableServer::POA_ptr poa,
    OB::EventLogger_ptr logger,
    Reaper* reaper,
    PipeSegment_ptr pipe,
    ChannelType* type)
    : AdminBase(orb, poa, logger, reaper, pipe),
      type_(type)
{
    if(logger_ -> traceLifecycle() >= 1)
        logger_ -> trace("event", "creating TypedSupplierAdmin");
}

TEC_TypedSupplierAdmin_impl::~TEC_TypedSupplierAdmin_impl()
{
    if(logger_ -> traceLifecycle() >= 1)
        logger_ -> trace("event", "destroy TypedSupplierAdmin");
}

// ----------------------------------------------------------------------
// Standard IDL to C++ mapping
// ----------------------------------------------------------------------

CosEventChannelAdmin::ProxyPushConsumer_ptr
TEC_TypedSupplierAdmin_impl::obtain_push_consumer()
    throw(SystemException)
{
    throw NO_IMPLEMENT();
    return CosEventChannelAdmin::ProxyPushConsumer::_nil();
}

CosEventChannelAdmin::ProxyPullConsumer_ptr
TEC_TypedSupplierAdmin_impl::obtain_pull_consumer()
    throw(SystemException)
{
    throw NO_IMPLEMENT();
    return CosEventChannelAdmin::ProxyPullConsumer::_nil();
}

CosTypedEventChannelAdmin::TypedProxyPushConsumer_ptr
TEC_TypedSupplierAdmin_impl::obtain_typed_push_consumer(const char* typeId)
    throw(CosTypedEventChannelAdmin::InterfaceNotSupported,
	  SystemException)
{
    JTCSynchronized sync(*this);

    if(destroyed_)
        throw OBJECT_NOT_EXIST();

    //
    // Attempt to assign this type id to the channel.
    //
    if(!type_ -> assignTypeId(typeId))
    {
	throw CosTypedEventChannelAdmin::InterfaceNotSupported();
    }

    ProxyConstructorInfo info;
    fillProxyConstructorInfo(info);

    TEC_TypedProxyPushConsumer_impl* p =
        new TEC_TypedProxyPushConsumer_impl(info, type_);
    addProxy(p);

    return p -> _this();
}

CosEventChannelAdmin::ProxyPullConsumer_ptr
TEC_TypedSupplierAdmin_impl::obtain_typed_pull_consumer(const char*)
    throw(CosTypedEventChannelAdmin::NoSuchImplementation,
	  SystemException)
{
    throw NO_IMPLEMENT();
    return CosEventChannelAdmin::ProxyPullConsumer::_nil();
}
