// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************
 
#ifndef TYPED_EVENT_CHANNEL_FACTORY_IMPL_H
#define TYPED_EVENT_CHANNEL_FACTORY_IMPL_H

//
// Forward declarations
//

class TEC_TypedEventChannel_impl;
class Reaper;

//
// The event channel factory implementation
//
class TypedEventChannelFactory_impl
    : virtual public PortableServer::RefCountServantBase,
      public POA_OBTypedEventChannelFactory::TypedEventChannelFactory,
      public JTCMonitor
{
    //
    // No copy constructor/assignment operator
    //
    TypedEventChannelFactory_impl(const TypedEventChannelFactory_impl&);
    void operator=(const TypedEventChannelFactory_impl&);

public: // For some compilers this must be public

    typedef OB::SynchronizedHashtable<CORBA::String_var, 
	                              TEC_TypedEventChannel_impl*,
                                      OB::StringHasher> TypedEventChannelMap;

private:

    CORBA::ORB_var orb_; // The ORB
    PortableServer::POA_var poa_; // The POA

    PortableServer::POA_var transientPOA_; // Trasient POA for use by 
                                           // EventChannels

    OB::EventLogger_var logger_; // The logger
    Reaper* reaper_; // The reaper

    //
    // Mapping from CORBA::String_var to TEC_TypedEventChannel_impl*
    //
    TypedEventChannelMap* channels_;

    void reap();
    
public:

    TypedEventChannelFactory_impl(CORBA::ORB_ptr, PortableServer::POA_ptr,
				  PortableServer::POA_ptr, OB::EventLogger_ptr,
				  Reaper*);
    ~TypedEventChannelFactory_impl();

    //
    // Standard IDL to C++ Mapping
    //

    //
    // Create TypedEventChannel object
    //
    virtual CosTypedEventChannelAdmin::TypedEventChannel_ptr
    create_channel(const char*)
	throw(OBEventChannelFactory::ChannelAlreadyExists,
	      CORBA::SystemException);

    //
    // Get TypedEventChannel object by id
    //
    virtual CosTypedEventChannelAdmin::TypedEventChannel_ptr
    get_channel_by_id(const char*)
	throw(OBEventChannelFactory::ChannelNotAvailable,
	      CORBA::SystemException);

    //
    // Get TypedEventChannel object ids
    //
    virtual OBEventChannelFactory::ChannelIdSeq* get_channels()
	throw(CORBA::SystemException);

    //
    // Shutdown the event service.
    //
    virtual void shutdown()
	throw(CORBA::SystemException);

    virtual PortableServer::POA_ptr _default_POA();
};

#endif
