// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>
#include <OB/Properties.h>
#include <OB/Logger.h>
#include <OB/BootManager.h>

#include <OB/TypedEventService.h>
#include <OB/OBTypedEventChannelFactory_skel.h>

#include <EventLogger.h>
#include <TypedEventChannelFactory_impl.h>
#include <Reaper.h>

using namespace CORBA;
using namespace OB;

// ----------------------------------------------------------------------
// TypedEventService constructor/destructor
// ----------------------------------------------------------------------

TypedEventService::TypedEventService(ORB_ptr orb, PortableServer::POA_ptr poa)
    : orb_(ORB::_duplicate(orb)),
      poa_(PortableServer::POA::_duplicate(poa)),
      channelFactory_(0),
      initialized_(false)
{
}

TypedEventService::~TypedEventService()
{
    //
    // Stop the proxy reaper
    //
    if(reaper_)
    {
	reaper_ -> destroy();
	while(reaper_ -> isAlive())
	{
	    try
	    {
		reaper_ -> join();
	    }
	    catch(const JTCInterruptedException&)
	    {
	    }
	}
    }

    //
    // Shutdown the typed event service
    //
    if(channelFactory_ != 0)
    {
	channelFactory_ -> shutdown();
	channelFactory_ -> _OB_decRef();
    }
}

// ----------------------------------------------------------------------
// TypedEventService public member implementation
// ----------------------------------------------------------------------

//
// Initialize the typed event service
//
void
TypedEventService::initialize()
{
    if(initialized_)
	return;

    initialized_ = true;

    //
    // Create a transient POA
    //
    PortableServer::POA_var transientPOA;
    try
    {
	PolicyList policies;
	policies.length(1);
	policies[0] =
	    poa_ -> create_implicit_activation_policy(
		PortableServer::IMPLICIT_ACTIVATION);

	PortableServer::POAManager_var manager = poa_ -> the_POAManager();

        transientPOA = poa_ -> create_POA("EventServiceTransientPOA", 
					  manager, policies);
    }
    catch(const PortableServer::POA::AdapterAlreadyExists&)
    {
    }
    assert(!is_nil(transientPOA));

    //
    // Create the event logger
    //
    OBCORBA::ORB_var oborb = OBCORBA::ORB::_narrow(orb_);
    OB::Properties_var properties = oborb -> properties();
    OB::Logger_var logger = oborb -> logger();
    OB::EventLogger_var eventLogger = new OB::EventLogger(logger, properties);

    //
    // Create the proxy reaper
    //
    reaper_ = new Reaper(orb_, eventLogger);
    reaper_ -> start();
    
    //
    // Create and activate the typed event channel factory
    //
    channelFactory_ = new TypedEventChannelFactory_impl(orb_, poa_, 
							transientPOA, 
							eventLogger,
							reaper_.get());
    PortableServer::ObjectId_var oid =
	PortableServer::string_to_ObjectId("DefaultTypedEventChannelFactory");
    poa_ -> activate_object_with_id(oid.in(), channelFactory_);
}

//
// Get event channel factory
//
OBTypedEventChannelFactory::TypedEventChannelFactory_ptr
TypedEventService::typedEventChannelFactory()
{
    return channelFactory_ -> _this();
}
