#!/bin/sh
# **********************************************************************
#
# Copyright (c) 2002
# IONA Technologies, Inc.
# Waltham, MA, USA
#
# All Rights Reserved
#
# **********************************************************************

#
# Function to check for Java distribution and set the Java CLASSPATH.
#
check_for_JAVA()
{
    have_java=
    if test -n "$OB_JAVA_DIST" -a \
            -r "$OB_JAVA_DIST/naming/lib/com/ooc/CosNaming/Server.class"
    then
	have_java="yes"
    fi

    #
    # Set Java CLASSPATH
    #
    if test -n "$have_java"
    then
        CLASSPATH="$OB_JAVA_DIST/naming/test/classes"
        CLASSPATH="$OB_JAVA_DIST/naming/lib${SEPARATOR}$CLASSPATH"
        CLASSPATH="$OB_JAVA_DIST/ob/lib${SEPARATOR}$CLASSPATH"
        export CLASSPATH

        #
        # Set Java BOOTCLASSPATH
        #
        java_version=`$JAVA -version 2>&1`
        if `$echo $java_version | grep '1\.[234]' > /dev/null`
        then
            BOOTCLASSPATH="-Xbootclasspath/p:$CLASSPATH"
        fi
    fi
}

#
# Messages for the non-availability of Java and set the 
# clients and servers for auto testing.
#
set_auto_values_and_msgs()
{
    auto_servers="1 2 4 3 5 6"

    java_not_available=
    if test -z "$have_java"
    then
	java_not_available=" (not available)"
    else
	auto_servers="$auto_servers 7 8 9 10"
    fi

    #
    # Set name of database file
    #
    db="$top_srcdir/test/Naming.db"
}

#
# Function to set the server executable command
#
set_server()
{
    server_num=$1

    if test "$server_num" = "1"
    then
	server="$top_srcdir/bin/nameserv${exe} -ORBreactive -OAreactive"
    fi

    if test "$server_num" = "2"
    then
	server="$top_srcdir/bin/nameserv${exe} -ORBthreaded -OAthread_per_request"
    fi

    if test "$server_num" = "3"
    then
	server="$top_srcdir/bin/nameserv${exe} -ORBthreaded -OAthread_pool 10"
    fi

    if test "$server_num" = "4"
    then
	server="$top_srcdir/bin/nameserv${exe} --database $db --start -ORBreactive -OAreactive"
    fi

    if test "$server_num" = "5"
    then
	server="$top_srcdir/bin/nameserv${exe} --database $db --start -ORBthreaded -OAthread_per_request"
    fi

    if test "$server_num" = "6"
    then
	server="$top_srcdir/bin/nameserv${exe} --database $db --start -ORBthreaded -OAthread_pool 10"
    fi

    if test "$server_num" = "7" -a -n "$have_java"
    then
	server="$JAVA $BOOTCLASSPATH com.ooc.CosNaming.Server -ORBthreaded -OAthread_per_request"
    fi

    if test "$server_num" = "8" -a -n "$have_java"
    then
	server="$JAVA $BOOTCLASSPATH com.ooc.CosNaming.Server -ORBthreaded -OAthread_pool 10"
    fi

    if test "$server_num" = "9" -a -n "$have_java"
    then
	server="$JAVA $BOOTCLASSPATH com.ooc.CosNaming.Server --database $db --start -ORBthreaded -OAthread_per_request"
    fi

    if test "$server_num" = "10" -a -n "$have_java"
    then
	server="$JAVA $BOOTCLASSPATH com.ooc.CosNaming.Server --database $db --start -ORBthreaded -OAthread_pool 10"
    fi
}

#
# Ask for server
#
prompt_for_server()
{
    #
    # Print welcome message
    #
    $echo
    $echo "************************************"
    $echo "* ORBacus NamingService Test Suite *"
    $echo "************************************"

    while test -z "$server_num"
    do
	$echo
	$echo "Select test server to use:"
	$echo
	$echo "1: C++ server, transient - reactive"
	$echo "2: C++ server, transient - thread per request"
	$echo "3: C++ server, transient - thread pool"
	$echo
	$echo "4: C++ server, persistent - reactive"
	$echo "5: C++ server, persistent - thread per request"
	$echo "6: C++ server, persistent - thread pool"
	$echo
	if test -n "$have_java"
	then
	    $echo "7: Java server, transient - thread per request"
	    $echo "8: Java server, transient - thread pool"
	    $echo
	    $echo "9: Java server, persistent - thread per request"
	    $echo "10: Java server, persistent - thread pool"
	    $echo
	else
	    $echo "7: Java server, transient - thread per request (not available)"
	    $echo "8: Java server, transient - thread pool (not available)"
	    $echo
	    $echo "9: Java server, persistent - thread per request (not available)"
	    $echo "10: Java server, persistent - thread pool (not available)"
	    $echo
	fi
	$echo "---> \c"

	read server_num
    done
}

#
# Execute the naming service test executables.
#
execute_test()
{
    #
    # Display info
    #
    $echo
    if test -n "$have_java"
    then
	$echo Java CLASSPATH is \"CLASSPATH=$CLASSPATH\"
    fi

    #
    # Set name of reference file
    #
    ref="$top_srcdir/test/Naming.ref"

    client="$top_srcdir/test/client${exe}"
    $echo Client command is \"$client\"
    $echo Server command is \"$server\"

    #
    # Function to deactivate the server
    #
    srvid=0
    deactivate()
    {
	if test $srvid -ne 0
	then
	    kill $srvid
	fi
    }
    trap deactivate 1 2 3 4 5 6 7 8 10 12 13 14 15

    #
    # Start naming server
    #
    $echo
    $echo "Waiting for naming server to start up... \c"
    rm -f $ref $db
    $server --ior > $ref 2> server.log &
    srvid=$!
    count=0
    while test ! -s $ref -a $count -lt 10
    do
	sleep 1
	count=`expr $count + 1`
    done
    if test ! -s $ref
    then
	$echo "Failed!"
	$echo "(Server was not started)"
	exit
    else
	$echo "Done!"
        $echo
    fi

    #
    # Start client
    #
    $client -ORBInitRef NameService=`cat $ref` 2> client.log

    #
    # Deactivate server
    #
    deactivate
}
