// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>
#include <OB/CosNaming.h>

#include <NamingExample.h>

#ifdef HAVE_STD_IOSTREAM
using namespace std;
#endif

void
PrintBinding(const CosNaming::Binding& b)
{
    CORBA::ULong i;

    for(i = 0 ; i < b.binding_name.length() ; i++)
    {
	if(i > 0)
	    cout << " ; ";
	
	cout << b.binding_name[i].id;
    }
    
    switch(b.binding_type)
    {
    case CosNaming::nobject:
	cout << " (object)" << endl;
	break;
	
    case CosNaming::ncontext:
	cout << " (context)" << endl;
	break;
    }
}

int
run(CORBA::ORB_ptr orb, int /* argc */, char* argv[])
{
    //
    // Get naming service
    //
    CORBA::Object_var obj;

    try
    {
        obj = orb -> resolve_initial_references("NameService");
    }
    catch(const CORBA::ORB::InvalidName&)
    {
        cerr << argv[0] << ": can't resolve `NameService'" << endl;
        return EXIT_FAILURE;
    }
    
    if(CORBA::is_nil(obj.in()))
    {
        cerr << argv[0] << ": `NameService' is a nil object reference"
	     << endl;
        return EXIT_FAILURE;
    }

    CosNaming::NamingContext_var nc =
        CosNaming::NamingContext::_narrow(obj.in());

    if(CORBA::is_nil(nc.in()))
    {
        cerr << argv[0]
	     << ": `NameService' is not a NamingContext object reference"
	     << endl;
        return EXIT_FAILURE;
    }

    //
    // Get objects by name
    //
    try
    {
        CORBA::ULong i;

        //
        // Resolve names with the Naming Service
        //
        CosNaming::Name aName;
        aName.length(1);
        aName[0].id = CORBA::string_dup("a");
        aName[0].kind = CORBA::string_dup("");
        CORBA::Object_var aObj = nc -> resolve(aName);
        Named_var a = Named::_narrow(aObj.in());
        assert(!CORBA::is_nil(a.in()));
        cout << "Resolved `a'" << endl;

        CosNaming::Name nc1Name;
        nc1Name.length(1);
        nc1Name[0].id = CORBA::string_dup("nc1");
        nc1Name[0].kind = CORBA::string_dup("");
        CORBA::Object_var nc1Obj = nc -> resolve(nc1Name);
        CosNaming::NamingContext_var nc1 =
	    CosNaming::NamingContext::_narrow(nc1Obj.in());
        assert(!CORBA::is_nil(nc1.in()));
        cout << "Resolved `nc1'" << endl;

        CosNaming::Name bName;
        bName.length(2);
        bName[0].id = CORBA::string_dup("nc1");
        bName[0].kind = CORBA::string_dup("");
        bName[1].id = CORBA::string_dup("b");
        bName[1].kind = CORBA::string_dup("");
        CORBA::Object_var bObj = nc -> resolve(bName);
        Named_var b = Named::_narrow(bObj.in());
        assert(!CORBA::is_nil(b.in()));
        cout << "Resolved `b' in naming context `nc1'" << endl;

        CosNaming::Name nc2Name;
        nc2Name.length(2);
        nc2Name[0].id = CORBA::string_dup("nc1");
        nc2Name[0].kind = CORBA::string_dup("");
        nc2Name[1].id = CORBA::string_dup("nc2");
        nc2Name[1].kind = CORBA::string_dup("");
        CORBA::Object_var nc2Obj = nc -> resolve(nc2Name);
        CosNaming::NamingContext_var nc2 =
	    CosNaming::NamingContext::_narrow(nc2Obj.in());
        assert(!CORBA::is_nil(nc2.in()));
        cout << "Resolved `nc2' in naming context `nc1'" << endl;

        CosNaming::Name cName;
        cName.length(3);
        cName[0].id = CORBA::string_dup("nc1");
        cName[0].kind = CORBA::string_dup("");
        cName[1].id = CORBA::string_dup("nc2");
        cName[1].kind = CORBA::string_dup("");
        cName[2].id = CORBA::string_dup("c");
        cName[2].kind = CORBA::string_dup("");
        CORBA::Object_var cObj = nc -> resolve(cName);
        Named_var c = Named::_narrow(cObj.in());
        assert(!CORBA::is_nil(c.in()));
        cout << "Resolved `c' in naming context `nc2' in naming context `nc1'"
	     << endl;

        cout << "Listing all bindings:" << endl;
        CosNaming::BindingList_var bl;
        CosNaming::BindingIterator_var bi;
        nc -> list(999999, bl.out(), bi.out());
        for(i = 0 ; i < bl -> length() ; i++)
	    PrintBinding(bl[i]);
        assert(CORBA::is_nil(bi.in()));

        cout << "Now listing only two bindings:" << endl;
        CosNaming::BindingList_var bl2;
        CosNaming::BindingIterator_var bi2;
        nc -> list(2, bl2.out(), bi2.out());
        for(i = 0 ; i < bl2 -> length() ; i++)
	    PrintBinding(bl2[i]);
        assert(!CORBA::is_nil(bi2.in()));

        cout << "Listing the rest with the binding iterator:" << endl;
        CosNaming::BindingList_var bl3;
        bi2 -> next_n(999999, bl3);
        for(i = 0 ; i < bl3 -> length() ; i++)
	    PrintBinding(bl3[i]);
        bi2 -> destroy();
    }
    catch(const CosNaming::NamingContext::NotFound& ex)
    {
        cerr << argv[0] << ": Got a `NotFound' exception (";
        switch(ex.why)
        {
        case CosNaming::NamingContext::missing_node:
	    cerr << "missing node";
	    break;

        case CosNaming::NamingContext::not_context:
	    cerr << "not context";
	    break;

        case CosNaming::NamingContext::not_object:
	    cerr << "not object";
	    break;
        }
        cerr << ")" << endl;
        return EXIT_FAILURE;
    }
    catch(const CosNaming::NamingContext::CannotProceed&)
    {
        cerr << argv[0] << ": Got a `CannotProceed' exception" << endl;
        return EXIT_FAILURE;
    }
    catch(const CosNaming::NamingContext::InvalidName&)
    {
        cerr << argv[0] << ": Got an `InvalidName' exception" << endl;
        return EXIT_FAILURE;
    }
    catch(const CosNaming::NamingContext::AlreadyBound&)
    {
        cerr << argv[0] << ": Got an `AlreadyBound' exception" << endl;
        return EXIT_FAILURE;
    }
    catch(const CosNaming::NamingContext::NotEmpty&)
    {
        cerr << argv[0] << ": Got a `NotEmpty' exception" << endl;
        return EXIT_FAILURE;
    }

    return EXIT_SUCCESS;
}

int
main(int argc, char* argv[], char*[])
{
    int status = EXIT_SUCCESS;
    CORBA::ORB_var orb;

    try
    {
	orb = CORBA::ORB_init(argc, argv);
	status = run(orb, argc, argv);
    }
    catch(const CORBA::Exception& ex)
    {
	cerr << ex << endl;
	status = EXIT_FAILURE;
    }

    if(!CORBA::is_nil(orb))
    {
	try
	{
	    orb -> destroy();
	}
	catch(const CORBA::Exception& ex)
	{
	    cerr << ex << endl;
	    status = EXIT_FAILURE;
	}
    }
    
    return status;
}
