// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>
#include <OB/CosNaming.h>

#include <NamingExample_impl.h>

#ifdef HAVE_STD_IOSTREAM
using namespace std;
#endif

int
run(CORBA::ORB_ptr orb, int /* argc */, char* argv[])
{
    //
    // Resolve Root POA
    //
    CORBA::Object_var poaObj = orb -> resolve_initial_references("RootPOA");
    PortableServer::POA_var rootPOA =
	PortableServer::POA::_narrow(poaObj.in());
 
    //
    // Get a reference to the POA manager
    //
    PortableServer::POAManager_var manager = rootPOA -> the_POAManager();

    //
    // Create some implementations
    //
    Named_impl* ap = new Named_impl;
    Named_impl* a1p = new Named_impl;
    Named_impl* a2p = new Named_impl;
    Named_impl* a3p = new Named_impl;
    Named_impl* bp = new Named_impl;
    Named_impl* cp = new Named_impl;
    Named_var a = ap -> _this();
    Named_var a1 = a1p -> _this();
    Named_var a2 = a2p -> _this();
    Named_var a3 = a3p -> _this();
    Named_var b = bp -> _this();
    Named_var c = cp -> _this();
    ap -> _remove_ref();
    a1p -> _remove_ref();
    a2p -> _remove_ref();
    a3p -> _remove_ref();
    bp -> _remove_ref();
    cp -> _remove_ref();

    //
    // Get naming service
    //
    CORBA::Object_var obj;
    
    try
    {
        obj = orb -> resolve_initial_references("NameService");
    }
    catch(const CORBA::ORB::InvalidName&)
    {
        cerr << argv[0] << ": can't resolve `NameService'" << endl;
        return EXIT_FAILURE;
    }

    if(CORBA::is_nil(obj.in()))
    {
        cerr << argv[0] << ": `NameService' is a nil object reference" << endl;
        return EXIT_FAILURE;
    }

    CosNaming::NamingContext_var nc =
        CosNaming::NamingContext::_narrow(obj.in());

    if(CORBA::is_nil(nc.in()))
    {
        cerr << argv[0]
	     << ": `NameService' is not a NamingContext object reference"
	     << endl;
        return EXIT_FAILURE;
    }

    try
    {
        //
        // Create and bind some Naming Contexts
        //
        CosNaming::Name nc1Name;
        nc1Name.length(1);
        nc1Name[0].id = CORBA::string_dup("nc1");
        nc1Name[0].kind = CORBA::string_dup("");
        CosNaming::NamingContext_var nc1 = nc -> bind_new_context(nc1Name);

        CosNaming::Name nc2Name;
        nc2Name.length(2);
        nc2Name[0].id = CORBA::string_dup("nc1");
        nc2Name[0].kind = CORBA::string_dup("");
        nc2Name[1].id = CORBA::string_dup("nc2");
        nc2Name[1].kind = CORBA::string_dup("");
        CosNaming::NamingContext_var nc2 = nc -> bind_new_context(nc2Name);

        //
        // Bind names with the Naming Service
        //
        CosNaming::Name aName;
        aName.length(1);
        aName[0].id = CORBA::string_dup("a");
        aName[0].kind = CORBA::string_dup("");
        nc -> bind(aName, a.in());

        CosNaming::Name a1Name;
        a1Name.length(1);
        a1Name[0].id = CORBA::string_dup("a1");
        a1Name[0].kind = CORBA::string_dup("");
        nc -> bind(a1Name, a1.in());

        CosNaming::Name a2Name;
        a2Name.length(1);
        a2Name[0].id = CORBA::string_dup("a2");
        a2Name[0].kind = CORBA::string_dup("");
        nc -> bind(a2Name, a2.in());

        CosNaming::Name a3Name;
        a3Name.length(1);
        a3Name[0].id = CORBA::string_dup("a3");
        a3Name[0].kind = CORBA::string_dup("");
        nc -> bind(a3Name, a3.in());

        CosNaming::Name bName;
        bName.length(2);
        bName[0].id = CORBA::string_dup("nc1");
        bName[0].kind = CORBA::string_dup("");
        bName[1].id = CORBA::string_dup("b");
        bName[1].kind = CORBA::string_dup("");
        nc -> bind(bName, b.in());

        CosNaming::Name cName;
        cName.length(3);
        cName[0].id = CORBA::string_dup("nc1");
        cName[0].kind = CORBA::string_dup("");
        cName[1].id = CORBA::string_dup("nc2");
        cName[1].kind = CORBA::string_dup("");
        cName[2].id = CORBA::string_dup("c");
        cName[2].kind = CORBA::string_dup("");
        nc -> bind(cName, b.in());

        //
        // Run implementation
        //
        manager -> activate();
        orb -> run();

        //
        // Unregister names with the Naming Service
        //
        nc -> unbind(cName);
        nc -> unbind(bName);
        nc -> unbind(aName);
        nc -> unbind(a1Name);
        nc -> unbind(a2Name);
        nc -> unbind(a3Name);
	nc2 -> destroy();
        nc -> unbind(nc2Name);
	nc1 -> destroy();
        nc -> unbind(nc1Name);
    }
    catch(const CosNaming::NamingContext::NotFound& ex)
    {
        cerr << argv[0] << ": Got a `NotFound' exception (";
        switch(ex.why)
        {
        case CosNaming::NamingContext::missing_node:
	    cerr << "missing node";
	    break;

        case CosNaming::NamingContext::not_context:
	    cerr << "not context";
	    break;

        case CosNaming::NamingContext::not_object:
	    cerr << "not object";
	    break;
        }
        cerr << ")" << endl;
        return EXIT_FAILURE;
    }
    catch(const CosNaming::NamingContext::CannotProceed&)
    {
        cerr << argv[0] << ": Got a `CannotProceed' exception" << endl;
        return EXIT_FAILURE;
    }
    catch(const CosNaming::NamingContext::InvalidName&)
    {
        cerr << argv[0] << ": Got an `InvalidName' exception" << endl;
        return EXIT_FAILURE;
    }
    catch(const CosNaming::NamingContext::AlreadyBound&)
    {
        cerr << argv[0] << ": Got an `AlreadyBound' exception" << endl;
        return EXIT_FAILURE;
    }
    catch(const CosNaming::NamingContext::NotEmpty&)
    {
        cerr << argv[0] << ": Got a `NotEmpty' exception" << endl;
        return EXIT_FAILURE;
    }

    return EXIT_SUCCESS;
}

int
main(int argc, char* argv[], char*[])
{
    int status = EXIT_SUCCESS;
    CORBA::ORB_var orb;

    try
    {
	orb = CORBA::ORB_init(argc, argv);
	status = run(orb, argc, argv);
    }
    catch(const CORBA::Exception& ex)
    {
	cerr << ex << endl;
	status = EXIT_FAILURE;
    }

    if(!CORBA::is_nil(orb))
    {
	try
	{
	    orb -> destroy();
	}
	catch(const CORBA::Exception& ex)
	{
	    cerr << ex << endl;
	    status = EXIT_FAILURE;
	}
    }
    
    return status;
}
