// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>
#include <OB/NTService.h>

#include <NTNamingService.h>

using namespace CORBA;

static void
usage(const char* progName)
{
    OB_CERR << "Usage:\n";
    OB_CERR << progName << " [options]\n"
	"\n"
	"Options:\n"
	"-h, --help             Show this message.\n"
	"-i, --install          Install the service.\n"
	"-s, --start-install    Install the service (automatic startup).\n"
	"-u, --uninstall        Uninstall the service.\n"
        "-d, --debug            Run in console mode.\n"
	;
}

int
main(int argc, char* argv[])
{
    NTNamingService service;

    //
    // Next parse the command line.
    //
    Long i;
    for(i = 1 ; i < argc && *argv[i] == '-' ; i++)
    {
        if(strcmp(argv[i], "--help") == 0 ||
           strcmp(argv[i], "-h") == 0)
        {
            usage(argv[0]);
            return 0;
        }
        else if(strcmp(argv[i], "--install") == 0 ||
                strcmp(argv[i], "-i") == 0)
        {
            service.install();
            return 0;
        }
        else if(strcmp(argv[i], "--start-install") == 0 ||
                strcmp(argv[i], "-s") == 0)
        {
            service.install(true);
            return 0;
        }
        else if(strcmp(argv[i], "--uninstall") == 0 ||
                strcmp(argv[i], "-u") == 0)
        {
            service.uninstall();
            return 0;
        }
        else if(strcmp(argv[i], "--debug") == 0 ||
                strcmp(argv[i], "-d") == 0)
        {
	    service.setDebug();
        }
        else if(strncmp(argv[i], "-OA", 3) != 0 &&
		strncmp(argv[i], "-ORB", 4) != 0)
        {
            OB_CERR << argv[0] << ": unknown option `"
                 << argv[i] << "'\n" << OB_ENDL;
            usage(argv[0]);
            return 1;
        }
    }
    
    service.run(argc, argv);

    return 0;
}
