// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <assert.h>
#include <stdlib.h>
#include <string.h>

#include <OB/CORBA.h>
#include <OB/Reactor.h>

#include <OB/OBNaming.h>

#include "TestListener_impl.h"
#include "TestObject_impl.h"
#include "TestMacro.h"

//
// Dispatch events until the listener has received the callback
//
#define WAIT(reactor,listener) \
    while(!listener.isValid()) \
	reactor -> dispatch_one_event(250);

//
// Test extended callback features
//
void
TestCallbacks(CORBA::ORB_ptr orb, CosNaming::NamingContext_ptr root)
{
    CosNaming::OBNamingContext_var ext =
        CosNaming::OBNamingContext::_narrow(root);
    assert(!CORBA::is_nil(ext.in()));

    OBCORBA::ORB_var oborb = OBCORBA::ORB::_narrow(orb);
    OB::Reactor_var reactor = oborb -> reactor();

    assert(ext -> callbacks_active());

    TestListener_impl listener;
    CosNaming::BindingListener_var p = listener._this();
    ext -> add_listener(p.in());

    TestObject_impl impl;
    TestObject_var test = impl._this();
    CosNaming::Name name1, name2;
    CosNaming::NamingContext_var context;

    try
    {
        //
        // Test BindingListener::object_added
        //
        name1.length(1);
        name1[0].id = CORBA::string_dup("name1");
        name1[0].kind = CORBA::string_dup("kind1");
        listener.expect(TestListener_impl::OBJECT_ADDED, ext.in(), test.in(), 
			name1[0]);
        SUCCEED(ext -> bind(name1, test.in()));
        WAIT(reactor, listener);
        name2.length(1);
        name2[0].id = CORBA::string_dup("name2");
        name2[0].kind = CORBA::string_dup("kind2");
        listener.expect(TestListener_impl::OBJECT_ADDED, ext.in(), test.in(),
			name2[0]);
        SUCCEED(ext -> bind(name2, test.in()));
        WAIT(reactor, listener);

        //
        // Test BindingListener::object_removed
        //
        listener.expect(TestListener_impl::OBJECT_REMOVED, ext.in(),
			CORBA::Object::_nil(), name2[0]);
        SUCCEED(ext -> unbind(name2));
        WAIT(reactor, listener);
        listener.expect(TestListener_impl::OBJECT_REMOVED, ext.in(),
			CORBA::Object::_nil(), name1[0]);
        SUCCEED(ext -> unbind(name1));
        WAIT(reactor, listener);

        //
        // Test BindingListener::object_changed
        //
        name1.length(1);
        name1[0].id = CORBA::string_dup("name1");
        name1[0].kind = CORBA::string_dup("kind1");
        listener.expect(TestListener_impl::OBJECT_ADDED, ext.in(), test.in(),
			name1[0]);
        SUCCEED(ext -> bind(name1, test.in()));
        WAIT(reactor, listener);
        listener.expect(TestListener_impl::OBJECT_CHANGED, ext.in(),
			CORBA::Object::_nil(), name1[0]);
        SUCCEED(ext -> rebind(name1, CORBA::Object::_nil()));
        WAIT(reactor, listener);
        listener.expect(TestListener_impl::OBJECT_REMOVED, ext.in(),
			CORBA::Object::_nil(), name1[0]);
        SUCCEED(ext -> unbind(name1));
        WAIT(reactor, listener);

        //
        // Test BindingListener::context_added
        //
        context = ext -> new_context();
        name1.length(1);
        name1[0].id = CORBA::string_dup("context1");
        name1[0].kind = CORBA::string_dup("kind1");
        listener.expect(TestListener_impl::CONTEXT_ADDED, ext.in(),
			context.in(), name1[0]);
        SUCCEED(ext -> bind_context(name1, context.in()));
        WAIT(reactor, listener);
        name2.length(1);
        name2[0].id = CORBA::string_dup("context2");
        name2[0].kind = CORBA::string_dup("kind2");
        listener.expect(TestListener_impl::CONTEXT_ADDED, ext.in(),
			context.in(), name2[0]);
        SUCCEED(ext -> bind_context(name2, context.in()));
        WAIT(reactor, listener);

        //
        // Test BindingListener::context_removed
        //
        listener.expect(TestListener_impl::CONTEXT_REMOVED, ext.in(),
			CORBA::Object::_nil(), name2[0]);
        SUCCEED(ext -> unbind(name2));
        WAIT(reactor, listener);
        listener.expect(TestListener_impl::CONTEXT_REMOVED, ext.in(),
			CORBA::Object::_nil(), name1[0]);
        SUCCEED(ext -> unbind(name1));
        WAIT(reactor, listener);

        //
        // Test BindingListener::context_changed
        //
        name1.length(1);
        name1[0].id = CORBA::string_dup("context1");
        name1[0].kind = CORBA::string_dup("kind1");
        listener.expect(TestListener_impl::CONTEXT_ADDED, ext.in(),
			context.in(), name1[0]);
        SUCCEED(ext -> bind_context(name1, context.in()));
        WAIT(reactor, listener);
	CosNaming::NamingContext_var ctx = root -> new_context();
        listener.expect(TestListener_impl::CONTEXT_CHANGED, ext.in(), ctx.in(),
			name1[0]);
        SUCCEED(ext -> rebind_context(name1, ctx.in()));
        WAIT(reactor, listener);
        listener.expect(TestListener_impl::CONTEXT_REMOVED, ext.in(),
			CORBA::Object::_nil(), name1[0]);
        SUCCEED(ext -> unbind(name1));
        WAIT(reactor, listener);

	SUCCEED(ctx -> destroy());
        SUCCEED(context -> destroy());

        SUCCEED(ext -> remove_listener(p.in()));

        //
        // Since impl is a stack-based servant, we need to deactivate
        // it before it goes out of scope
        //
        PortableServer::POA_var poa = impl._default_POA();
        PortableServer::ObjectId_var id = poa -> servant_to_id(&impl);
        poa -> deactivate_object(id.in());

        poa = listener._default_POA();
        id = poa -> servant_to_id(&listener);
        poa -> deactivate_object(id.in());
    }
    catch(...)
    {
        //
        // Deactivate stack-based servants
        //
        PortableServer::POA_var poa = impl._default_POA();
        PortableServer::ObjectId_var id = poa -> servant_to_id(&impl);
        poa -> deactivate_object(id.in());

        poa = listener._default_POA();
        id = poa -> servant_to_id(&listener);
        poa -> deactivate_object(id.in());
        throw;
    }
}
