// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <assert.h>
#include <stdlib.h>
#include <string.h>

#include <OB/CORBA.h>

#include <OB/CosNaming.h>

#include "TestObject_impl.h"
#include "TestMacro.h"


//
// Test CosNaming::NamingContext::destroy()
//
void
TestDestroy(CORBA::ORB_ptr /* orb */, CosNaming::NamingContext_ptr root)
{
    TestObject_impl impl;
    TestObject_var test = impl._this();
    CosNaming::Name name1, name2;
    CosNaming::NamingContext_var context;

    try
    {
        //
        // Test exceptions
        //

        context = root -> new_context();
        name1.length(1);
        name1[0].id = CORBA::string_dup("ctx");
        name1[0].kind = CORBA::string_dup("");
        SUCCEED(root -> bind_context(name1, context.in()));

        name2.length(1);
        name2[0].id = CORBA::string_dup("object");
        name2[0].kind = CORBA::string_dup("");
        SUCCEED(context -> bind(name2, test.in()));

        try
        {
            //
            // Try to destroy a context that's not empty
            //
            context -> destroy();
            assert("destroy should not have succeeded" == 0);
        }
        catch(const CosNaming::NamingContext::NotEmpty&)
        {
            // Expected
        }

        SUCCEED(context -> unbind(name2));

        //
        // Destroy the context while there is still a binding in the root
        // context.
        //
        SUCCEED(context -> destroy());

        //
        // Ensure that the context has been destroyed.
        //
        try
        {
            context -> unbind(name1);
            assert("Call on context should not have succeeded." == 0);
        }
        catch(const CORBA::OBJECT_NOT_EXIST&)
        {
            // Expected
        }

        //
        // Resolve the context, and attempt to narrow it. Depending on the
        // server implementation, either the call to _narrow or the call
        // to unbind should result in OBJECT_NOT_EXIST.
        //
        try
        {
            CORBA::Object_var obj;
            SUCCEED(obj = root -> resolve(name1));
            CosNaming::NamingContext_var contextAgain =
                CosNaming::NamingContext::_narrow(obj.in());
            contextAgain -> unbind(name1);
            assert("Call to _narrow/unbind should not have succeeded." == 0);
        }
        catch(const CORBA::OBJECT_NOT_EXIST&)
        {
            // Expected
        }

        //
        // This test should succeed, but doesn't. This is because the
        // context is reconnected implicitely in the naming service.
        // Eventually this test should succeed.
        //
        //assert(context -> _is_equivalent(contextAgain));

        //
        // Now unbind
        //
        SUCCEED(root -> unbind(name1));

        //
        // Since impl is a stack-based servant, we need to deactivate
        // it before it goes out of scope
        //
        PortableServer::POA_var poa = impl._default_POA();
        PortableServer::ObjectId_var id = poa -> servant_to_id(&impl);
        poa -> deactivate_object(id.in());
    }
    catch(...)
    {
        //
        // Deactivate stack-based servant
        //
        PortableServer::POA_var poa = impl._default_POA();
        PortableServer::ObjectId_var id = poa -> servant_to_id(&impl);
        poa -> deactivate_object(id.in());
        throw;
    }
}
