// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <assert.h>
#include <stdlib.h>
#include <string.h>

#include <OB/CORBA.h>

#include <OB/CosNaming.h>

#include "TestObject_impl.h"
#include "TestMacro.h"


//
// Test CosNaming::NamingContext::list() and CosNaming::BindingIterator
//
void
TestList(CORBA::ORB_ptr /* orb */, CosNaming::NamingContext_ptr root)
{
    TestObject_impl impl1;
    TestObject_impl impl2;
    TestObject_impl impl3;
    TestObject_var test1 = impl1._this();
    TestObject_var test2 = impl2._this();
    TestObject_var test3 = impl3._this();
    CosNaming::Name name;
    CosNaming::BindingList_var list;
    CosNaming::BindingIterator_var iter;
    CosNaming::Binding_var binding;

    try
    {
        CosNaming::NamingContext_var context = root -> new_context();

        name.length(1);
        name[0].id = CORBA::string_dup("obj1");
        name[0].kind = CORBA::string_dup("kind1");
        SUCCEED(root -> bind(name, test1.in()));
        name[0].id = CORBA::string_dup("obj2");
        SUCCEED(root -> bind(name, test2.in()));
        name[0].id = CORBA::string_dup("obj3");
        SUCCEED(root -> bind(name, test3.in()));
        name[0].id = CORBA::string_dup("context");
        SUCCEED(root -> bind_context(name, context.in()));

        root -> list(4, list.out(), iter.out());
        assert(list -> length() == 4);
        assert(CORBA::is_nil(iter.in()));

        for(CORBA::ULong i = 0 ; i < list -> length() ; i++)
        {
            if(strcmp(list[i].binding_name[0].id.in(), "obj1") == 0 ||
               strcmp(list[i].binding_name[0].id.in(), "obj2") == 0 ||
               strcmp(list[i].binding_name[0].id.in(), "obj3") == 0)
                assert(list[i].binding_type == CosNaming::nobject);
            else if(strcmp(list[i].binding_name[0].id.in(), "context") == 0)
                assert(list[i].binding_type == CosNaming::ncontext);
            else
                assert("unknown binding in list" == 0);
        }

        root -> list(2, list.out(), iter.out());
        assert(list -> length() == 2);
        assert(!CORBA::is_nil(iter.in()));
        assert(iter -> next_one(binding));
        assert(iter -> next_one(binding));
        assert(!iter -> next_one(binding)); // no more
        iter -> destroy();

        root -> list(0, list.out(), iter.out());
        assert(list -> length() == 0);
        assert(!CORBA::is_nil(iter.in()));
        try
        {
            assert(iter -> next_n(0, list));
            assert("Call to next_n(0) should not have succeeded." == 0);
        }
        catch(const CORBA::BAD_PARAM&)
        {
            // Expected
        }
        assert(iter -> next_n(3, list));
        assert(list -> length() == 3);
        assert(iter -> next_n(1, list));
        assert(list -> length() == 1);
        assert(!iter -> next_n(1, list)); // no more
        iter -> destroy();

        //
        // Clean up
        //
        name.length(1);
        name[0].id = CORBA::string_dup("obj1");
        name[0].kind = CORBA::string_dup("kind1");
        SUCCEED(root -> unbind(name));
        name[0].id = CORBA::string_dup("obj2");
        SUCCEED(root -> unbind(name));
        name[0].id = CORBA::string_dup("obj3");
        SUCCEED(root -> unbind(name));
        name[0].id = CORBA::string_dup("context");
        SUCCEED(root -> unbind(name));
        SUCCEED(context -> destroy());

        //
        // Since impl is a stack-based servant, we need to deactivate
        // it before it goes out of scope
        //
        PortableServer::POA_var poa = impl1._default_POA();
        PortableServer::ObjectId_var id = poa -> servant_to_id(&impl1);
        poa -> deactivate_object(id.in());
        id = poa -> servant_to_id(&impl2);
        poa -> deactivate_object(id.in());
        id = poa -> servant_to_id(&impl3);
        poa -> deactivate_object(id.in());
    }
    catch(...)
    {
        //
        // Deactivate stack-based servants
        //
        PortableServer::POA_var poa = impl1._default_POA();
        PortableServer::ObjectId_var id = poa -> servant_to_id(&impl1);
        poa -> deactivate_object(id.in());
        id = poa -> servant_to_id(&impl2);
        poa -> deactivate_object(id.in());
        id = poa -> servant_to_id(&impl3);
        poa -> deactivate_object(id.in());
        throw;
    }
}
