// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <assert.h>
#include <stdlib.h>
#include <string.h>

#include <OB/CORBA.h>

#include <OB/OBNaming.h>

#include "TestObject_impl.h"
#include "TestMacro.h"


//
// Test CosNaming::OBNamingContext::list_extended()
//
void
TestListExtended(CORBA::ORB_ptr /* orb */, CosNaming::NamingContext_ptr root)
{
    CosNaming::OBNamingContext_var ext =
        CosNaming::OBNamingContext::_narrow(root);
    assert(! CORBA::is_nil(ext.in()));

    TestObject_impl impl;
    TestObject_var test = impl._this();
    CosNaming::Name name1;
    CosNaming::OBNamingContext::ExtendedBindingList_var list;

    try
    {
        CosNaming::NamingContext_var context = ext -> new_context();

        //
        // Add some bindings to test list_extended()
        //
        name1.length(1);
        name1[0].id = CORBA::string_dup("obj1");
        name1[0].kind = CORBA::string_dup("kind1");
        SUCCEED(ext -> bind(name1, test.in()));
        name1[0].id = CORBA::string_dup("obj2");
        SUCCEED(ext -> bind(name1, test.in()));
        name1[0].id = CORBA::string_dup("obj3");
        SUCCEED(ext -> bind(name1, test.in()));
        name1[0].id = CORBA::string_dup("context");
        SUCCEED(ext -> bind_context(name1, context.in()));

        list = ext -> list_extended();
        assert(list -> length() == 4);

        //
        // Timestamps
        //
        CORBA::Long tsObj1 = 0, tsObj2 = 0, tsObj3 = 0, tsContext = 0;

        //
        // Check BindingType and timestamp
        //
        for(CORBA::ULong i = 0 ; i < list -> length() ; i++)
        {
            assert(list[i].timestamp > 0);

            //
            // Remember timestamps
            //
            if(strcmp(list[i].binding_name[0].id.in(), "obj1") == 0)
               tsObj1 = list[i].timestamp;
            else if(strcmp(list[i].binding_name[0].id.in(), "obj2") == 0)
               tsObj2 = list[i].timestamp;
            else if(strcmp(list[i].binding_name[0].id.in(), "obj3") == 0)
               tsObj3 = list[i].timestamp;
            else if(strcmp(list[i].binding_name[0].id.in(), "context") == 0)
               tsContext = list[i].timestamp;

            if(strcmp(list[i].binding_name[0].id.in(), "obj1") == 0 ||
               strcmp(list[i].binding_name[0].id.in(), "obj2") == 0 ||
               strcmp(list[i].binding_name[0].id.in(), "obj3") == 0)
                assert(list[i].bound_obj._d() == CosNaming::nobject);
            else if(strcmp(list[i].binding_name[0].id.in(), "context") == 0)
                assert(list[i].bound_obj._d() == CosNaming::ncontext);
            else
                assert("unknown binding in list" == 0);
        }

        //
        // Verify timestamps - check order of creation
        //
        assert(tsContext >= tsObj3 && tsObj3 >= tsObj2 && tsObj2 >= tsObj1);

        //
        // Remove a binding and test again
        //
        name1[0].id = CORBA::string_dup("obj2");
        SUCCEED(ext -> unbind(name1));
        list = ext -> list_extended();
        assert(list -> length() == 3);

        //
        // Remove all bindings and test again
        //
        name1[0].id = CORBA::string_dup("obj1");
        SUCCEED(ext -> unbind(name1));
        name1[0].id = CORBA::string_dup("obj3");
        SUCCEED(ext -> unbind(name1));
        name1[0].id = CORBA::string_dup("context");
        SUCCEED(ext -> unbind(name1));
        list = ext -> list_extended();
        assert(list -> length() == 0);

        SUCCEED(context -> destroy());

        //
        // Since impl is a stack-based servant, we need to deactivate
        // it before it goes out of scope
        //
        PortableServer::POA_var poa = impl._default_POA();
        PortableServer::ObjectId_var id = poa -> servant_to_id(&impl);
        poa -> deactivate_object(id.in());
    }
    catch(...)
    {
        //
        // Deactivate stack-based servant
        //
        PortableServer::POA_var poa = impl._default_POA();
        PortableServer::ObjectId_var id = poa -> servant_to_id(&impl);
        poa -> deactivate_object(id.in());
        throw;
    }
}
