// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <assert.h>
#include <stdlib.h>
#include <string.h>

#include <OB/CORBA.h>

#include <OB/CosNaming.h>

#include "TestMacro.h"


//
// Test CosNaming::NamingContextExt::to_name()
//
void
TestToName(CORBA::ORB_ptr /* orb */, CosNaming::NamingContextExt_ptr root)
{
    CosNaming::Name_var name;
    CosNaming::NamingContextExt::StringName_var str;

    //
    // Test name components with both ID and Kind field
    //
    str = CORBA::string_dup("first.node/second.node");
    SUCCEED(name = root -> to_name(str.in()));
    assert(name -> length() == 2);
    assert(strcmp(name[0].id.in(), "first") == 0 &&
	   strcmp(name[0].kind.in(), "node") == 0);
    assert(strcmp(name[1].id.in(), "second") == 0 &&
	   strcmp(name[1].kind.in(), "node") == 0);

    //
    // Test name components with missing Kind field
    //
    str = CORBA::string_dup("first.node/second");
    SUCCEED(name = root -> to_name(str.in()));
    assert(name -> length() == 2);
    assert(strcmp(name[0].id.in(), "first") == 0 &&
	   strcmp(name[0].kind.in(), "node") == 0);
    assert(strcmp(name[1].id.in(), "second") == 0 &&
	   strcmp(name[1].kind.in(), "") == 0);

    //
    // Test name components with missing ID field
    //
    str = CORBA::string_dup("first.node/.node");
    SUCCEED(name = root -> to_name(str.in()));
    assert(name -> length() == 2);
    assert(strcmp(name[0].id.in(), "first") == 0 &&
	   strcmp(name[0].kind.in(), "node") == 0);
    assert(strcmp(name[1].id.in(), "") == 0 &&
	   strcmp(name[1].kind.in(), "node") == 0);

    //
    // Test node with empty ID and Kind field
    //
    str = CORBA::string_dup("first.node//third.node");
    SUCCEED(name = root -> to_name(str.in()));
    assert(name -> length() == 3);
    assert(strcmp(name[0].id.in(), "first") == 0 &&
	   strcmp(name[0].kind.in(), "node") == 0);
    assert(strcmp(name[1].id.in(), "") == 0 &&
	   strcmp(name[1].kind.in(), "") == 0);
    assert(strcmp(name[2].id.in(), "third") == 0 &&
	   strcmp(name[2].kind.in(), "node") == 0);

    //
    // Test nodes with characters that are escaped
    //
    str =  CORBA::string_dup(
	"first.node\\.bla/second\\/bla.node/third.\\\\node");
    SUCCEED(name = root -> to_name(str.in()));
    assert(name -> length() == 3);
    assert(strcmp(name[0].id.in(), "first") == 0 &&
	   strcmp(name[0].kind.in(), "node.bla") == 0);
    assert(strcmp(name[1].id.in(), "second/bla") == 0 &&
	   strcmp(name[1].kind.in(), "node") == 0);
    assert(strcmp(name[2].id.in(), "third") == 0 &&
	   strcmp(name[2].kind.in(), "\\node") == 0);

    //
    // Test empty name
    //
    try
    {
	str = CORBA::string_dup("");
	name = root -> to_name(str.in());
	assert("to_name should not have succeeded" == 0);
    }
    catch(const CosNaming::NamingContext::InvalidName&)
    {
	// Expected
    }
}
