// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <assert.h>
#include <stdlib.h>
#include <string.h>

#include <OB/CORBA.h>

#include <OB/CosNaming.h>

#include "TestMacro.h"


//
// Test CosNaming::NamingContextExt::to_string()
//
void
TestToString(CORBA::ORB_ptr /* orb */, CosNaming::NamingContextExt_ptr root)
{
    CosNaming::Name name;
    CosNaming::NamingContextExt::StringName_var str;

    //
    // Test name components with both ID and Kind field
    //
    name.length(2);
    name[0].id = CORBA::string_dup("first");
    name[0].kind = CORBA::string_dup("node");
    name[1].id = CORBA::string_dup("second");
    name[1].kind = CORBA::string_dup("node");
    SUCCEED(str = root -> to_string(name));
    assert(strcmp(str.in(), "first.node/second.node") == 0);

    //
    // Test name components with missing Kind field
    //
    name.length(2);
    name[0].id = CORBA::string_dup("first");
    name[0].kind = CORBA::string_dup("node");
    name[1].id = CORBA::string_dup("second");
    name[1].kind = CORBA::string_dup("");
    SUCCEED(str = root -> to_string(name));
    assert(strcmp(str.in(), "first.node/second") == 0);

    //
    // Test name components with missing ID field
    //
    name.length(2);
    name[0].id = CORBA::string_dup("first");
    name[0].kind = CORBA::string_dup("node");
    name[1].id = CORBA::string_dup("");
    name[1].kind = CORBA::string_dup("kind");
    SUCCEED(str = root -> to_string(name));
    assert(strcmp(str.in(), "first.node/.kind") == 0);

    //
    // Test node with empty ID and Kind field
    //
    name.length(3);
    name[0].id = CORBA::string_dup("first");
    name[0].kind = CORBA::string_dup("node");
    name[1].id = CORBA::string_dup("");
    name[1].kind = CORBA::string_dup("");
    name[2].id = CORBA::string_dup("third");
    name[2].kind = CORBA::string_dup("node");
    SUCCEED(str = root -> to_string(name));
    assert(strcmp(str.in(), "first.node//third.node") == 0);

    //
    // Test nodes with characters that have to be escaped
    //
    name.length(3);
    name[0].id = CORBA::string_dup("first");
    name[0].kind = CORBA::string_dup("node.bla");
    name[1].id = CORBA::string_dup("second/bla");
    name[1].kind = CORBA::string_dup("node");
    name[2].id = CORBA::string_dup("third");
    name[2].kind = CORBA::string_dup("\\node");
    SUCCEED(str = root -> to_string(name));
    assert(strcmp(str.in(),
		  "first.node\\.bla/second\\/bla.node/third.\\\\node") == 0);

    //
    // Test empty name
    //
    name.length(0);
    SUCCEED(str = root -> to_string(name));
    assert(strlen(str.in()) == 0);
}
