// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <assert.h>
#include <stdlib.h>
#include <string.h>

#include <OB/CORBA.h>

#include <OB/CosNaming.h>

#include "TestMacro.h"


//
// Test CosNaming::NamingContextExt::to_url()
//

static void 
strcmpURL(const char *expected, const char *result)
{
    int scmp = strcmp(expected, result);
    
    if(scmp != 0)
    {
        cerr << "to_url failure, received: " << result << endl;
        cerr << "                expected: " << expected << endl;
    }
    assert(scmp == 0);
}

    
void
TestToURL(CORBA::ORB_ptr /* orb */, CosNaming::NamingContextExt_ptr root)
{
    CosNaming::NamingContextExt::Address_var addr;
    CosNaming::NamingContextExt::StringName_var name;
    CosNaming::NamingContextExt::URLString_var str;

    //
    // Test stringified names without URL escapes
    //
    addr = CORBA::string_dup(":myhost.xyz.com");
    name = CORBA::string_dup("first.node");
    SUCCEED(str = root -> to_url(addr.in(), name.in()));
    strcmpURL(str.in(), "corbaname::myhost.xyz.com#first.node");

    addr = CORBA::string_dup(":myhost.555.xyz.com,:my_backup.555xyz.com:900");
    name = CORBA::string_dup("first.node");
    SUCCEED(str = root -> to_url(addr.in(), name.in()));
    strcmpURL(str.in(), "corbaname::myhost.555.xyz.com,"
              ":my_backup.555xyz.com:900#first.node");
    
        

    //
    // Test stringified names with URL escapes
    //
    addr = CORBA::string_dup(":xyz.com");
    name = CORBA::string_dup("<a>.b/c.d");
    SUCCEED(str = root -> to_url(addr.in(), name.in()));
    strcmpURL(str.in(), "corbaname::xyz.com#%3Ca%3E.b/c.d");

    name = CORBA::string_dup("a.b/  c.d");
    SUCCEED(str = root -> to_url(addr.in(), name.in()));
    strcmpURL(str.in(), "corbaname::xyz.com#a.b/%20%20c.d");

    name = CORBA::string_dup("a%b/c%d");
    SUCCEED(str = root -> to_url(addr.in(), name.in()));
    strcmpURL(str.in(), "corbaname::xyz.com#a%25b/c%25d");
}
