// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>
#include <OB/Properties.h>

#include <Example_skel.h>
#include <Account_impl.h>
#include <Currency_impl.h>

#include <stdlib.h>

#ifdef HAVE_STD_IOSTREAM
using namespace std;
#endif

int
run(CORBA::ORB_ptr orb, int /* argc */, char* argv[])
{
    //
    // Resolve Root POA
    //
    CORBA::Object_var obj = orb -> resolve_initial_references("RootPOA");
    assert(!CORBA::is_nil(obj));

    PortableServer::POA_var root = PortableServer::POA::_narrow(obj);
    assert(!CORBA::is_nil(root));

    PortableServer::POAManager_var manager = root -> the_POAManager();

    //
    // Activate the POAManager and ask the ORB to start accepting
    // method requests
    //
    manager -> activate();

    obj = orb -> string_to_object("relfile:/Example.ref");
    if(CORBA::is_nil(obj))
    {
        cerr << argv[0] << ": cannot read IOR from Example.ref" << endl;
        return EXIT_FAILURE;
    }

    Example_var example = Example::_narrow(obj);
    assert(!CORBA::is_nil(example));

    Account_impl* impl = new Account_impl;
    PortableServer::ServantBase_var servant = impl;
    Account_var acct = impl -> _this();
    example -> display(acct);

    Currency_var currency = new Currency_impl;
    example -> display(currency);

    return EXIT_SUCCESS;
}

int
main(int argc, char* argv[], char*[])
{
    int status = EXIT_SUCCESS;
    CORBA::ORB_var orb;

    try
    {
        OB::Properties_var properties = OB::Properties::getDefaultProperties();
	OB::ParseArgs(argc, argv, properties);

        //
        // Set the concurrency model
        //
	properties -> setProperty("ooc.orb.conc_model", "reactive");
	properties -> setProperty("ooc.orb.oa.conc_model", "reactive");

        //
        // Create ORB
        //
	orb = CORBA::ORB_init(argc, argv);
	status = run(orb, argc, argv);
    }
    catch(const CORBA::Exception& ex)
    {
	cerr << ex << endl;
	status = EXIT_FAILURE;
    }

    if(!CORBA::is_nil(orb))
    {
	try
	{
	    orb -> destroy();
	}
	catch(const CORBA::Exception& ex)
	{
	    cerr << ex << endl;
	    status = EXIT_FAILURE;
	}
    }
    
    return status;
}
