// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>

#include <CallBasicTester.h>
#include <Timer.h>

#ifdef HAVE_STD_IOSTREAM
using namespace std;
#endif

//----------------------------------------------------------------------
// Function:	ping
//
// Description:	Client-side function that illustrates how to invoke the
//		Benchmark::BasicTester::ping operation.
//----------------------------------------------------------------------

void
CallBasicTester::ping(
    Benchmark::BasicTester_ptr obj,
    long iterations)
{
    cout << iterations << " Pings: \t" << flush;
    Timer t;

    try 
    {
        //
    	// Warm up ping to establish connection
	//
    	obj -> ping();
		
    	t.start();

    	for(long i = 0; i < iterations; i++)
       	    obj -> ping();

        t.stop();
    }
    catch(CORBA::SystemException &ex)
    {
    	cout << "ping(): call failed"
    	     << endl
    	     << &ex
    	     << endl;
    	return;
    }
    t.stop();

    cout << "\t" << t.timeElapsed() << " seconds (" 
         << (iterations / t.timeElapsed()) << "/sec)" << endl;
}

//----------------------------------------------------------------------
// Function:	short_in_t
//
// Description:	Client-side function that illustrates how to invoke the
//		Benchmark::BasicTester::short_in_t operation.
//----------------------------------------------------------------------

void
CallBasicTester::short_in_t(
    Benchmark::BasicTester_ptr obj,
    long iterations)
{
    CORBA::Short s;

    s = 7;

    cout << iterations << " Shorts in:  " << flush;
    Timer t;

    try 
    {
    	t.start();

    	for(long i = 0; i < iterations; i++)
            obj -> short_in_t(s);

    	t.stop();
    }
    catch(CORBA::SystemException &ex)
    {
    	cout << "short_in_t(): call failed"
    	     << endl
    	     << &ex
    	     << endl;
    	return;
    }

    cout << "\t" << t.timeElapsed() << " seconds (" 
         << (iterations / t.timeElapsed()) << "/sec)" << endl;
}

//----------------------------------------------------------------------
// Function:	short_out_t
//
// Description:	Client-side function that illustrates how to invoke the
//		Benchmark::BasicTester::short_out_t operation.
//----------------------------------------------------------------------

void
CallBasicTester::short_out_t(
    Benchmark::BasicTester_ptr obj,
    long iterations)
{
    CORBA::Short s;

    cout << iterations << " Shorts out: " << flush;
    Timer t;

    try
    {
    	t.start();

    	for(long i = 0; i < iterations; i++)
    	    obj -> short_out_t(s);

    	t.stop();
    }
    catch(CORBA::SystemException &ex)
    {
    	cout << "short_out_t(): call failed"
    	     << endl
    	     << &ex
    	     << endl;
    	return;
    }

    cout << "\t" << t.timeElapsed() << " seconds ("
         << (iterations / t.timeElapsed()) << "/sec)" << endl;
}

//----------------------------------------------------------------------
// Function:	short_inout_t
//
// Description:	Client-side function that illustrates how to invoke the
//		Benchmark::BasicTester::short_inout_t operation.
//----------------------------------------------------------------------

void
CallBasicTester::short_inout_t(
    Benchmark::BasicTester_ptr obj,
    long iterations)
{
    CORBA::Short s;
    
    s = 18;
    
    cout << iterations << " Shorts i/o: " << flush;
    Timer t;
    
    try 
    {
    	t.start();
    
    	for(long i = 0; i < iterations; i++)
    	    obj -> short_inout_t(s);
    
    	t.stop();
    }
    catch(CORBA::SystemException &ex)
    {
   	cout << "short_inout_t(): call failed"
    	     << endl
    	     << &ex
    	     << endl;
    	return;
    }

    cout << "\t" << t.timeElapsed() << " seconds (" 
         << (iterations / t.timeElapsed()) << "/sec)" << endl;
}

//----------------------------------------------------------------------
// Function:	short_ret_t
//
// Description:	Client-side function that illustrates how to invoke the
//		Benchmark::BasicTester::short_ret_t operation.
//----------------------------------------------------------------------

void
CallBasicTester::short_ret_t(
    Benchmark::BasicTester_ptr obj,
    long iterations)
{
    CORBA::Short result;
    
    cout << iterations << " Shorts ret: " << flush;
    Timer t;
    
    try 
    {
    	t.start();
    
    	for(long i = 0; i < iterations; i++)
    	    result = obj -> short_ret_t();
    
    	t.stop();
    }
    catch(CORBA::SystemException &ex)
    {
    	cout << "short_ret_t(): call failed"
    	     << endl
    	     << &ex
    	     << endl;
    	return;
    }

    cout << "\t" << t.timeElapsed() << " seconds (" 
         << (iterations / t.timeElapsed()) << "/sec)" << endl;
}

//----------------------------------------------------------------------
// Function:	long_in_t
//
// Description:	Client-side function that illustrates how to invoke the
//		Benchmark::BasicTester::long_in_t operation.
//----------------------------------------------------------------------

void
CallBasicTester::long_in_t(
    Benchmark::BasicTester_ptr obj,
    long iterations)
{
    CORBA::Long l;
    
    l = 14l;
    
    cout << iterations << " Longs in:  " << flush;
    Timer t;
    
    try 
    {
        t.start();
	
        for(long i = 0; i < iterations; i++)
	    obj -> long_in_t(l);

        t.stop();
    }
    catch(CORBA::SystemException &ex) 
    {
   	cout << "long_in_t(): call failed"
    	     << endl
    	     << &ex
    	     << endl;
    	return;
    }

    cout << "\t" << t.timeElapsed() << " seconds (" 
         << (iterations / t.timeElapsed()) << "/sec)" << endl;
}

//----------------------------------------------------------------------
// Function:	long_out_t
//
// Description:	Client-side function that illustrates how to invoke the
//		Benchmark::BasicTester::long_out_t operation.
//----------------------------------------------------------------------

void
CallBasicTester::long_out_t(
    Benchmark::BasicTester_ptr obj,
    long iterations)
{
    CORBA::Long l;

    cout << iterations << " Longs out: " << flush;
    Timer t;

    try 
    {
        t.start();

        for(long i = 0; i < iterations; i++)
	    obj -> long_out_t(l);

        t.stop();
    }
    catch(CORBA::SystemException &ex) 
    {
    	cout << "long_out_t(): call failed"
    	     << endl
    	     << &ex
    	     << endl;
    	return;
    }

    cout << "\t" << t.timeElapsed() << " seconds (" 
         << (iterations / t.timeElapsed()) << "/sec)" << endl;
}

//----------------------------------------------------------------------
// Function:	long_inout_t
//
// Description:	Client-side function that illustrates how to invoke the
//		Benchmark::BasicTester::long_inout_t operation.
//----------------------------------------------------------------------

void
CallBasicTester::long_inout_t(
    Benchmark::BasicTester_ptr obj,
    long iterations)
{
    CORBA::Long l;

    l = 18400l;

    cout << iterations << " Longs i/o: " << flush;
    Timer t;

    try 
    {
        t.start();

        for(long i = 0; i < iterations; i++)
	    obj -> long_inout_t(l);

        t.stop();
    }
    catch(CORBA::SystemException &ex)
    {
    	cout << "long_inout_t(): call failed"
    	     << endl
    	     << &ex
    	     << endl;
    	return;
    }

    cout << "\t" << t.timeElapsed() << " seconds (" 
         << (iterations / t.timeElapsed()) << "/sec)" << endl;
}

//----------------------------------------------------------------------
// Function:	long_ret_t
//
// Description:	Client-side function that illustrates how to invoke the
//		Benchmark::BasicTester::long_ret_t operation.
//----------------------------------------------------------------------

void
CallBasicTester::long_ret_t(
    Benchmark::BasicTester_ptr obj,
    long iterations)
{
    CORBA::Long result;

    cout << iterations << " Longs ret: " << flush;

    Timer t;

    try 
    {
        t.start();
    
        for(long i = 0; i < iterations; i++)
    	    result = obj -> long_ret_t();

        t.stop();
    }
    catch(CORBA::SystemException &ex)
    {
    	cout << "long_ret_t(): call failed"
    	     << endl
    	     << &ex
    	     << endl;
    	return;
    }

    cout << "\t" << t.timeElapsed() << " seconds (" 
         << (iterations / t.timeElapsed()) << "/sec)" << endl;
}

//----------------------------------------------------------------------
// Function:	longlong_in_t
//
// Description:	Client-side function that illustrates how to invoke the
//		Benchmark::BasicTester::longlong_in_t operation.
//----------------------------------------------------------------------

void
CallBasicTester::longlong_in_t(
    Benchmark::BasicTester_ptr obj,
    long iterations)
{
    CORBA::LongLong ll;

#ifdef HAVE_VCPLUSPLUS_BUGS
    ll = 4183423456L;
#else
    ll = 4183423456LL;
#endif

    cout << iterations << " Long longs in:  " << flush;
    Timer t;
    
    try 
    {
        t.start();

        for(long i = 0; i < iterations; i++)
    	    obj -> longlong_in_t(ll);

        t.stop();
    }
    catch(CORBA::SystemException &ex)
    {
    	cout << "longlong_in_t(): call failed"
    	     << endl
    	     << &ex
    	     << endl;
    	return;
    }

    cout << "\t" << t.timeElapsed() << " seconds (" 
         << (iterations / t.timeElapsed()) << "/sec)" << endl;
}

//----------------------------------------------------------------------
// Function:	longlong_out_t
//
// Description:	Client-side function that illustrates how to invoke the
//		Benchmark::BasicTester::longlong_out_t operation.
//----------------------------------------------------------------------

void
CallBasicTester::longlong_out_t(
    Benchmark::BasicTester_ptr obj,
    long iterations)
{
    CORBA::LongLong ll;

    cout << iterations << " Long longs out: " << flush;
    Timer t;

    try 
    {
        t.start();

        for(long i = 0; i < iterations; i++)
    	    obj -> longlong_out_t(ll);

        t.stop();
    }
    catch(CORBA::SystemException &ex)
    {
   	cout << "longlong_out_t(): call failed"
    	     << endl
    	     << &ex
    	     << endl;
    	return;
    }

    cout << "\t" << t.timeElapsed() << " seconds ("
         << (iterations / t.timeElapsed()) << "/sec)" << endl;
}

//----------------------------------------------------------------------
// Function:	longlong_inout_t
//
// Description:	Client-side function that illustrates how to invoke the
//		Benchmark::BasicTester::longlong_inout_t operation.
//----------------------------------------------------------------------

void
CallBasicTester::longlong_inout_t(
    Benchmark::BasicTester_ptr obj,
    long iterations)
{
    CORBA::LongLong ll;

#ifdef HAVE_VCPLUSPLUS_BUGS
    ll = 1234548324L;
#else
    ll = 1234548324LL;
#endif

    cout << iterations << " Long longs i/o: " << flush;
    Timer t;

    try 
    {
        t.start();

	for(long i = 0; i < iterations; i++)
	    obj -> longlong_inout_t(ll);

        t.stop();
    }
    catch(CORBA::SystemException &ex)
    {
   	cout << "longlong_inout_t(): call failed"
    	     << endl
    	     << &ex
    	     << endl;
    	return;
    }

    cout << "\t" << t.timeElapsed() << " seconds (" 
         << (iterations / t.timeElapsed()) << "/sec)" << endl;
}

//----------------------------------------------------------------------
// Function:	longlong_ret_t
//
// Description:	Client-side function that illustrates how to invoke the
//		Benchmark::BasicTester::longlong_ret_t operation.
//----------------------------------------------------------------------

void
CallBasicTester::longlong_ret_t(
    Benchmark::BasicTester_ptr obj,
    long iterations)
{
    CORBA::LongLong result;

    cout << iterations << " Long longs ret: " << flush;
    Timer t;

    try 
    {
        t.start();

        for(long i = 0; i < iterations; i++)
    	    result = obj -> longlong_ret_t();

        t.stop();
    }
    catch(CORBA::SystemException &ex) 
    {
   	cout << "longlong_ret_t(): call failed"
    	     << endl
    	     << &ex
    	     << endl;
    	return;
    }

    cout << "\t" << t.timeElapsed() << " seconds (" 
         << (iterations / t.timeElapsed()) << "/sec)" << endl;
}

//----------------------------------------------------------------------
// Function:	ushort_in_t
//
// Description:	Client-side function that illustrates how to invoke the
//		Benchmark::BasicTester::ushort_in_t operation.
//----------------------------------------------------------------------

void
CallBasicTester::ushort_in_t(
    Benchmark::BasicTester_ptr obj,
    long iterations)
{
    CORBA::UShort us;

    us = 9942;

    cout << iterations << " Ushorts in:   " << flush;
    Timer t;

    try 
    {
        t.start();

        for(long i = 0; i < iterations; i++)
    	    obj -> ushort_in_t(us);

        t.stop();
    }
    catch(CORBA::SystemException &ex)
    {
   	cout << "ushort_in_t(): call failed"
    	     << endl
    	<< &ex
    	     << endl;
    	return;
    }

    cout << "\t" << t.timeElapsed() << " seconds (" 
         << (iterations / t.timeElapsed()) << "/sec)" << endl;
}

//----------------------------------------------------------------------
// Function:	ushort_out_t
//
// Description:	Client-side function that illustrates how to invoke the
//		Benchmark::BasicTester::ushort_out_t operation.
//----------------------------------------------------------------------

void
CallBasicTester::ushort_out_t(
    Benchmark::BasicTester_ptr obj,
    long iterations)
{
    CORBA::UShort us;
    
    cout << iterations << " Ushorts out: " << flush;
    Timer t;

    try 
    {
        t.start();

        for(long i = 0; i < iterations; i++)
    	    obj -> ushort_out_t(us);

        t.stop();
    }
    catch(CORBA::SystemException &ex) 
    {
    	cout << "ushort_out_t(): call failed"
    	     << endl
    	     << &ex
    	     << endl;
    	return;
    }

    cout << "\t" << t.timeElapsed() << " seconds (" 
         << (iterations / t.timeElapsed()) << "/sec)" << endl;
}

//----------------------------------------------------------------------
// Function:	ushort_inout_t
//
// Description:	Client-side function that illustrates how to invoke the
//		Benchmark::BasicTester::ushort_inout_t operation.
//----------------------------------------------------------------------

void
CallBasicTester::ushort_inout_t(
    Benchmark::BasicTester_ptr obj,
    long iterations)
{
    CORBA::UShort us;

    us = 9911;

    cout << iterations << " Ushorts i/o: " << flush;
    Timer t;

    try 
    {
        t.start();

        for(long i = 0; i < iterations; i++)
    	    obj -> ushort_inout_t(us);

        t.stop();
    }
    catch(CORBA::SystemException &ex)
    {
    	cout << "ushort_inout_t(): call failed"
    	     << endl
    	     << &ex
    	     << endl;
    	return;
    }

    cout << "\t" << t.timeElapsed() << " seconds (" 
         << (iterations / t.timeElapsed()) << "/sec)" << endl;
}

//----------------------------------------------------------------------
// Function:	ushort_ret_t
//
// Description:	Client-side function that illustrates how to invoke the
//		Benchmark::BasicTester::ushort_ret_t operation.
//----------------------------------------------------------------------

void
CallBasicTester::ushort_ret_t(
    Benchmark::BasicTester_ptr obj,
    long iterations)
{
    CORBA::UShort result;

    cout << iterations << " Ushorts ret: " << flush;
    Timer t;

    try 
    {
        t.start();

        for(long i = 0; i < iterations; i++)
   	    result = obj -> ushort_ret_t();

        t.stop();
    }
    catch(CORBA::SystemException &ex) 
    {
    	cout << "ushort_ret_t(): call failed"
    	     << endl
    	     << &ex
    	     << endl;
    	return;
    }

    cout << "\t" << t.timeElapsed() << " seconds (" 
         << (iterations / t.timeElapsed()) << "/sec)" << endl;
}

//----------------------------------------------------------------------
// Function:	ulong_in_t
//
// Description:	Client-side function that illustrates how to invoke the
//		Benchmark::BasicTester::ulong_in_t operation.
//----------------------------------------------------------------------

void
CallBasicTester::ulong_in_t(
    Benchmark::BasicTester_ptr obj,
    long iterations)
{
    CORBA::ULong ul;

    ul = 44200234;

    cout << iterations << " Ulongs in:   " << flush;
    Timer t;

    try
    {
        t.start();

        for(long i = 0; i < iterations; i++)
    	    obj -> ulong_in_t(ul);

        t.stop();
    }
    catch(CORBA::SystemException &ex)
    {
    	cout << "ulong_in_t(): call failed"
    	     << endl
    	     << &ex
    	     << endl;
    	return;
    }
    
    cout << "\t" << t.timeElapsed() << " seconds (" 
         << (iterations / t.timeElapsed()) << "/sec)" << endl;
}

//----------------------------------------------------------------------
// Function:	ulong_out_t
//
// Description:	Client-side function that illustrates how to invoke the
//		Benchmark::BasicTester::ulong_out_t operation.
//----------------------------------------------------------------------

void
CallBasicTester::ulong_out_t(
    Benchmark::BasicTester_ptr obj,
    long iterations)
{
    CORBA::ULong ul;
    
    cout << iterations << " Ulongs out: " << flush;
    Timer t;

    try
    {
        t.start();

        for(long i = 0; i < iterations; i++)
    	    obj -> ulong_out_t(ul);
    
        t.stop();
    }
    catch(CORBA::SystemException &ex)
    {
    	cout << "ulong_out_t(): call failed"
    	     << endl
    	     << &ex
    	     << endl;
    	return;
    }
    
    cout << "\t" << t.timeElapsed() << " seconds (" 
         << (iterations / t.timeElapsed()) << "/sec)" << endl;
}

//----------------------------------------------------------------------
// Function:	ulong_inout_t
//
// Description:	Client-side function that illustrates how to invoke the
//		Benchmark::BasicTester::ulong_inout_t operation.
//----------------------------------------------------------------------

void
CallBasicTester::ulong_inout_t(
    Benchmark::BasicTester_ptr obj,
    long iterations)
{
    CORBA::ULong ul;

    ul = 4420024;

    cout << iterations << " Ulongs i/o: " << flush;
    Timer t;
       
    try
    {
        t.start();

        for(long i = 0; i < iterations; i++)
    	    obj -> ulong_inout_t(ul);

        t.stop();
    }
    catch(CORBA::SystemException &ex)
    {
    	cout << "ulong_inout_t(): call failed"
    	     << endl
    	     << &ex
    	     << endl;
    	return;
    }
    
    cout << "\t" << t.timeElapsed() << " seconds (" 
         << (iterations / t.timeElapsed()) << "/sec)" << endl;
}

//----------------------------------------------------------------------
// Function:	ulong_ret_t
//
// Description:	Client-side function that illustrates how to invoke the
//		Benchmark::BasicTester::ulong_ret_t operation.
//----------------------------------------------------------------------

void
CallBasicTester::ulong_ret_t(
    Benchmark::BasicTester_ptr obj,
    long iterations)
{
    CORBA::ULong result;
    
    cout << iterations << " Ulongs ret: " << flush;
    Timer t;
    
    try
    {
        t.start();

        for(long i = 0; i < iterations; i++)
    	    result = obj -> ulong_ret_t();

        t.stop();
    }
    catch(CORBA::SystemException &ex)
    {
    	cout << "ulong_ret_t(): call failed"
    	     << endl
    	     << &ex
    	     << endl;
    	return;
    }

    cout << "\t" << t.timeElapsed() << " seconds (" 
         << (iterations / t.timeElapsed()) << "/sec)" << endl;
}

//----------------------------------------------------------------------
// Function:	ulonglong_in_t
//
// Description:	Client-side function that illustrates how to invoke the
//		Benchmark::BasicTester::ulonglong_in_t operation.
//----------------------------------------------------------------------

void
CallBasicTester::ulonglong_in_t(
    Benchmark::BasicTester_ptr obj,
    long iterations)
{
    CORBA::ULongLong ull;

#ifdef HAVE_VCPLUSPLUS_BUGS
    ull = 123456890L;
#else
    ull = 123456890LL;
#endif
    
    cout << iterations << " Ulonglongs in:   " << flush;
    Timer t;
    
    try
    {
        t.start();

        for(long i = 0; i < iterations; i++)
    	    obj -> ulonglong_in_t(ull);

        t.stop();
    }
    catch(CORBA::SystemException &ex)
    {
    	cout << "ulonglong_in_t(): call failed"
    	     << endl
    	     << &ex
    	     << endl;
    	return;
    }
    
    cout << "\t" << t.timeElapsed() << " seconds (" 
         << (iterations / t.timeElapsed()) << "/sec)" << endl;
}

//----------------------------------------------------------------------
// Function:	ulonglong_out_t
//
// Description:	Client-side function that illustrates how to invoke the
//		Benchmark::BasicTester::ulonglong_out_t operation.
//----------------------------------------------------------------------

void
CallBasicTester::ulonglong_out_t(
    Benchmark::BasicTester_ptr obj,
    long iterations)
{
    CORBA::ULongLong ull;
    
    cout << iterations << " Ulonglongs out: " << flush;
    Timer t;
    
    try
    {
        t.start();

        for(long i = 0; i < iterations; i++)
    	    obj -> ulonglong_out_t(ull);

        t.stop();
    }
    catch(CORBA::SystemException &ex)
    {
    	cout << "ulonglong_out_t(): call failed"
    	     << endl
    	     << &ex
    	     << endl;
       	return;
    }
    
    cout << "\t" << t.timeElapsed() << " seconds (" 
         << (iterations / t.timeElapsed()) << "/sec)" << endl;

}

//----------------------------------------------------------------------
// Function:	ulonglong_inout_t
//
// Description:	Client-side function that illustrates how to invoke the
//		Benchmark::BasicTester::ulonglong_inout_t operation.
//----------------------------------------------------------------------

void
CallBasicTester::ulonglong_inout_t(
    Benchmark::BasicTester_ptr obj,
    long iterations)
{
    CORBA::ULongLong ull;
    
#ifdef HAVE_VCPLUSPLUS_BUGS
    ull = 4L;
#else
    ull = 4LL;
#endif
    
    cout << iterations << " Ulonglongs i/o: " << flush;
    Timer t;

    try
    {
        t.start();

        for(long i = 0; i < iterations; i++)
    	    obj -> ulonglong_inout_t(ull);

        t.stop();
    }
    catch(CORBA::SystemException &ex)
    {
    	cout << "ulonglong_inout_t(): call failed"
    	     << endl
    	     << &ex
    	     << endl;
    	return;
    }

    cout << "\t" << t.timeElapsed() << " seconds (" 
         << (iterations / t.timeElapsed()) << "/sec)" << endl;
}

//----------------------------------------------------------------------
// Function:	ulonglong_ret_t
//
// Description:	Client-side function that illustrates how to invoke the
//		Benchmark::BasicTester::ulonglong_ret_t operation.
//----------------------------------------------------------------------

void
CallBasicTester::ulonglong_ret_t(
    Benchmark::BasicTester_ptr obj,
    long iterations)
{
    CORBA::ULongLong result;

    cout << iterations << " Ulonglongs ret: " << flush;
    Timer t;
    
    try
    {
        t.start();

        for(long i = 0; i < iterations; i++)
    	     result = obj -> ulonglong_ret_t();

        t.stop();
    }
    catch(CORBA::SystemException &ex)
    {
    	cout << "ulonglong_ret_t(): call failed"
    	     << endl
    	     << &ex
    	     << endl;
    	return;
    }

    cout << "\t" << t.timeElapsed() << " seconds (" 
         << (iterations / t.timeElapsed()) << "/sec)" << endl;
}

//----------------------------------------------------------------------
// Function:	float_in_t
//
// Description:	Client-side function that illustrates how to invoke the
//		Benchmark::BasicTester::float_in_t operation.
//----------------------------------------------------------------------

void
CallBasicTester::float_in_t(
    Benchmark::BasicTester_ptr obj,
    long iterations)
{
    CORBA::Float f;

    f = 145.88f;

    cout << iterations << " Floats in:  " << flush;
    Timer t;

    try
    {
        t.start();

        for(long i = 0; i < iterations; i++)
    	    obj -> float_in_t(f);

        t.stop();
    }
    catch(CORBA::SystemException &ex)
    {
    	cout << "float_in_t(): call failed"
    	     << endl
    	     << &ex
    	     << endl;
    	return;
    }

    cout << "\t" << t.timeElapsed() << " seconds (" 
         << (iterations / t.timeElapsed()) << "/sec)" << endl;
}

//----------------------------------------------------------------------
// Function:	float_out_t
//
// Description:	Client-side function that illustrates how to invoke the
//		Benchmark::BasicTester::float_out_t operation.
//----------------------------------------------------------------------

void
CallBasicTester::float_out_t(
    Benchmark::BasicTester_ptr obj,
    long iterations)
{
    CORBA::Float f;
    
    cout << iterations << " Floats out: " << flush;
    Timer t;
    
    try
    {
        t.start();

        for(long i = 0; i < iterations; i++)
    	    obj -> float_out_t(f);

        t.stop();
    }
    catch(CORBA::SystemException &ex)
    {
    	cout << "float_out_t(): call failed"
    	     << endl
    	     << &ex
    	     << endl;
    	return;
    }

    cout << "\t" << t.timeElapsed() << " seconds (" 
         << (iterations / t.timeElapsed()) << "/sec)" << endl;
}

//----------------------------------------------------------------------
// Function:	float_inout_t
//
// Description:	Client-side function that illustrates how to invoke the
//		Benchmark::BasicTester::float_inout_t operation.
//----------------------------------------------------------------------

void
CallBasicTester::float_inout_t(
    Benchmark::BasicTester_ptr obj,
    long iterations)
{
    CORBA::Float f;
    
    f = 888.888f;
    
    cout << iterations << " Floats i/o: " << flush;
    Timer t;

    try
    {
        t.start();

        for(long i = 0; i < iterations; i++)
    	    obj -> float_inout_t(f);

        t.stop();
    }
    catch(CORBA::SystemException &ex)
    {
    	cout << "float_inout_t(): call failed"
    	     << endl
    	     << &ex
    	     << endl;
    	return;
    }
    
    cout << "\t" << t.timeElapsed() << " seconds (" 
         << (iterations / t.timeElapsed()) << "/sec)" << endl;
}

//----------------------------------------------------------------------
// Function:	float_ret_t
//
// Description:	Client-side function that illustrates how to invoke the
//		Benchmark::BasicTester::float_ret_t operation.
//----------------------------------------------------------------------

void
CallBasicTester::float_ret_t(
    Benchmark::BasicTester_ptr obj,
    long iterations)
{
    CORBA::Float result;
    
    cout << iterations << " Floats ret: " << flush;
    Timer t;
    
    try
    {
        t.start();

        for(long i = 0; i < iterations; i++)
    	    result = obj -> float_ret_t();

        t.stop();
    }
    catch(CORBA::SystemException &ex)
    {
    	cout << "float_ret_t(): call failed"
    	     << endl
    	     << &ex
    	     << endl;
    	return;
    }
    
    cout << "\t" << t.timeElapsed() << " seconds (" 
         << (iterations / t.timeElapsed()) << "/sec)" << endl;
}

//----------------------------------------------------------------------
// Function:	double_in_t
//
// Description:	Client-side function that illustrates how to invoke the
//		Benchmark::BasicTester::double_in_t operation.
//----------------------------------------------------------------------

void
CallBasicTester::double_in_t(
    Benchmark::BasicTester_ptr obj,
    long iterations)
{
    CORBA::Double d;
    
    d = (CORBA::Double) 199.4;

    cout << iterations << " Doubles in:  " << flush;
    Timer t;

    try
    {
        t.start();

        for(long i = 0; i < iterations; i++)
    	    obj -> double_in_t(d);

        t.stop();
    }
    catch(CORBA::SystemException &ex)
    {
    	cout << "double_in_t(): call failed"
    	     << endl
    	     << &ex
    	     << endl;
    	return;
    }
    
    cout << "\t" << t.timeElapsed() << " seconds (" 
         << (iterations / t.timeElapsed()) << "/sec)" << endl;
}

//----------------------------------------------------------------------
// Function:	double_out_t
//
// Description:	Client-side function that illustrates how to invoke the
//		Benchmark::BasicTester::double_out_t operation.
//----------------------------------------------------------------------

void
CallBasicTester::double_out_t(
    Benchmark::BasicTester_ptr obj,
    long iterations)
{
    CORBA::Double d;
    
    cout << iterations << " Doubles out: " << flush;
    Timer t;

    try
    {
        t.start();

        for(long i = 0; i < iterations; i++)
    	    obj -> double_out_t(d);

        t.stop();
    }
    catch(CORBA::SystemException &ex)
    {
    	cout << "double_out_t(): call failed"
    	     << endl
    	     << &ex
    	     << endl;
    	return;
    }
    
    cout << "\t" << t.timeElapsed() << " seconds (" 
         << (iterations / t.timeElapsed()) << "/sec)" << endl;
}

//----------------------------------------------------------------------
// Function:	double_inout_t
//
// Description:	Client-side function that illustrates how to invoke the
//		Benchmark::BasicTester::double_inout_t operation.
//----------------------------------------------------------------------

void
CallBasicTester::double_inout_t(
    Benchmark::BasicTester_ptr obj,
    long iterations)
{
    CORBA::Double d;
    
    d = (CORBA::Double) 994.7;
    
    cout << iterations << " Doubles i/o: " << flush;
    Timer t;
    
    try
    {
        t.start();

        for(long i = 0; i < iterations; i++)
    	    obj -> double_inout_t(d);

        t.stop();
    }
    catch(CORBA::SystemException &ex)
    {
    	cout << "double_inout_t(): call failed"
    	     << endl
    	     << &ex
    	     << endl;
    	return;
    }
    
    cout << "\t" << t.timeElapsed() << " seconds (" 
         << (iterations / t.timeElapsed()) << "/sec)" << endl;
}

//----------------------------------------------------------------------
// Function:	double_ret_t
//
// Description:	Client-side function that illustrates how to invoke the
//		Benchmark::BasicTester::double_ret_t operation.
//----------------------------------------------------------------------

void
CallBasicTester::double_ret_t(
    Benchmark::BasicTester_ptr obj,
    long iterations)
{
    CORBA::Double result;
    
    cout << iterations << " Doubles ret: " << flush;
    Timer t;
    
    try
    {
        t.start();

        for(long i = 0; i < iterations; i++)
    	    result = obj -> double_ret_t();

        t.stop();
    }
    catch(CORBA::SystemException &ex)
    {
    	cout << "double_ret_t(): call failed"
    	     << endl
    	     << &ex
    	     << endl;
       	return;
    }

    cout << "\t" << t.timeElapsed() << " seconds (" 
         << (iterations / t.timeElapsed()) << "/sec)" << endl;
}

//----------------------------------------------------------------------
// Function:	char_in_t
//
// Description:	Client-side function that illustrates how to invoke the
//		Benchmark::BasicTester::char_in_t operation.
//----------------------------------------------------------------------

void
CallBasicTester::char_in_t(
    Benchmark::BasicTester_ptr obj,
    long iterations)
{
    CORBA::Char c;
    
    c = (CORBA::Char) '&';
    
    cout << iterations << " Chars in:  " << flush;
    Timer t;

    try
    {
        t.start();

        for(long i = 0; i < iterations; i++)
    	    obj -> char_in_t(c);

        t.stop();
    }
    catch(CORBA::SystemException &ex)
    {
    	cout << "char_in_t(): call failed"
    	     << endl
    	     << &ex
    	     << endl;
    	return;
    }
    
    cout << "\t" << t.timeElapsed() << " seconds (" 
         << (iterations / t.timeElapsed()) << "/sec)" << endl;
}

//----------------------------------------------------------------------
// Function:	char_out_t
//
// Description:	Client-side function that illustrates how to invoke the
//		Benchmark::BasicTester::char_out_t operation.
//----------------------------------------------------------------------

void
CallBasicTester::char_out_t(
    Benchmark::BasicTester_ptr obj,
    long iterations)
{
    CORBA::Char c;

    cout << iterations << " Chars out: " << flush;
    Timer t;

    try
    {
        t.start();

        for(long i = 0; i < iterations; i++)
    	    obj -> char_out_t(c);

        t.stop();
    }
    catch(CORBA::SystemException &ex)
    {
    	cout << "char_out_t(): call failed"
    	     << endl
    	     << &ex
    	     << endl;
    	return;
    }

    cout << "\t" << t.timeElapsed() << " seconds (" 
         << (iterations / t.timeElapsed()) << "/sec)" << endl;
}

//----------------------------------------------------------------------
// Function:	char_inout_t
//
// Description:	Client-side function that illustrates how to invoke the
//		Benchmark::BasicTester::char_inout_t operation.
//----------------------------------------------------------------------

void
CallBasicTester::char_inout_t(
    Benchmark::BasicTester_ptr obj,
    long iterations)
{
    CORBA::Char c;

    c = (CORBA::Char) 'Q';

    cout << iterations << " Chars i/o: " << flush;
    Timer t;
    
    try
    {
        t.start();

        for(long i = 0; i < iterations; i++)
    	    obj -> char_inout_t(c);

        t.stop();
    }
    catch(CORBA::SystemException &ex)
    {
    	cout << "char_inout_t(): call failed"
    	     << endl
    	     << &ex
    	     << endl;
    	return;
    }

    cout << "\t" << t.timeElapsed() << " seconds (" 
         << (iterations / t.timeElapsed()) << "/sec)" << endl;
}

//----------------------------------------------------------------------
// Function:	char_ret_t
//
// Description:	Client-side function that illustrates how to invoke the
//		Benchmark::BasicTester::char_ret_t operation.
//----------------------------------------------------------------------

void
CallBasicTester::char_ret_t(
    Benchmark::BasicTester_ptr obj,
    long iterations)
{
    CORBA::Char result;

    cout << iterations << " Chars ret: " << flush;
    Timer t;
    
    try
    {
        t.start();

        for(long i = 0; i < iterations; i++)
    	    result = obj -> char_ret_t();

        t.stop();
    }
    catch(CORBA::SystemException &ex)
    {
    	cout << "char_ret_t(): call failed"
    	     << endl
    	     << &ex
    	     << endl;
    	return;
    }

    cout << "\t" << t.timeElapsed() << " seconds (" 
         << (iterations / t.timeElapsed()) << "/sec)" << endl;
}

//----------------------------------------------------------------------
// Function:	wchar_in_t
//
// Description:	Client-side function that illustrates how to invoke the
//		Benchmark::BasicTester::wchar_in_t operation.
//----------------------------------------------------------------------

void
CallBasicTester::wchar_in_t(
    Benchmark::BasicTester_ptr obj,
    long iterations)
{
    CORBA::WChar wc;
    
    wc = (CORBA::WChar) L'X';
    
    cout << iterations << " Wchars in:  " << flush;
    Timer t;

    try
    {
        t.start();

        for(long i = 0; i < iterations; i++)
    	    obj -> wchar_in_t(wc);
    
        t.stop();
    }
    catch(CORBA::SystemException &ex)
    {
    	cout << "wchar_in_t(): call failed"
    	     << endl
    	     << &ex
    	     << endl;
    	return;
    }

    cout << "\t" << t.timeElapsed() << " seconds (" 
         << (iterations / t.timeElapsed()) << "/sec)" << endl;
}

//----------------------------------------------------------------------
// Function:	wchar_out_t
//
// Description:	Client-side function that illustrates how to invoke the
//		Benchmark::BasicTester::wchar_out_t operation.
//----------------------------------------------------------------------

void
CallBasicTester::wchar_out_t(
    Benchmark::BasicTester_ptr obj,
    long iterations)
{
    CORBA::WChar wc;
    
    cout << iterations << " Wchars out: " << flush;
    Timer t;
    
    try
    {
        t.start();

        for(long i = 0; i < iterations; i++)
    	    obj -> wchar_out_t(wc);

        t.stop();
    }
    catch(CORBA::SystemException &ex)
    {
    	cout << "wchar_out_t(): call failed"
    	     << endl
    	     << &ex
    	     << endl;
       	return;
    }

    cout << "\t" << t.timeElapsed() << " seconds (" 
         << (iterations / t.timeElapsed()) << "/sec)" << endl;
}

//----------------------------------------------------------------------
// Function:	wchar_inout_t
//
// Description:	Client-side function that illustrates how to invoke the
//		Benchmark::BasicTester::wchar_inout_t operation.
//----------------------------------------------------------------------

void
CallBasicTester::wchar_inout_t(
    Benchmark::BasicTester_ptr obj,
    long iterations)
{
    CORBA::WChar wc;
    
    wc = (CORBA::WChar) L'.';
    
    cout << iterations << " Wchars i/o: " << flush;
    Timer t;

    try
    {
        t.start();

        for(long i = 0; i < iterations; i++)
    	    obj -> wchar_inout_t(wc);

        t.stop();
    }
    catch(CORBA::SystemException &ex)
    {
    	cout << "wchar_inout_t(): call failed"
    	     << endl
    	     << &ex
    	     << endl;
    	return;
    }
    
    cout << "\t" << t.timeElapsed() << " seconds (" 
         << (iterations / t.timeElapsed()) << "/sec)" << endl;
}

//----------------------------------------------------------------------
// Function:	wchar_ret_t
//
// Description:	Client-side function that illustrates how to invoke the
//		Benchmark::BasicTester::wchar_ret_t operation.
//----------------------------------------------------------------------

void
CallBasicTester::wchar_ret_t(
    Benchmark::BasicTester_ptr obj,
    long iterations)
{
    CORBA::WChar result;
    
    cout << iterations << " Wchars ret: " << flush;
    Timer t;
    
    try
    {
        t.start();

        for(long i = 0; i < iterations; i++)
    	    result = obj -> wchar_ret_t();

        t.stop();
    }
    catch(CORBA::SystemException &ex)
    {
    	cout << "wchar_ret_t(): call failed"
    	     << endl
    	     << &ex
    	     << endl;
    	return;
    }
    
    cout << "\t" << t.timeElapsed() << " seconds (" 
         << (iterations / t.timeElapsed()) << "/sec)" << endl;
}

//----------------------------------------------------------------------
// Function:	string_in_t
//
// Description:	Client-side function that illustrates how to invoke the
//		Benchmark::BasicTester::string_in_t operation.
//----------------------------------------------------------------------

void
CallBasicTester::string_in_t(
    Benchmark::BasicTester_ptr obj,
    long iterations)
{
    CORBA::String_var s;
    
    s = CORBA::string_dup("Mountain bikes rule!");
    
    cout << iterations << " Strings in:  " << flush;
    Timer t;

    try
    {
        t.start();

        for(long i = 0; i < iterations; i++)
    	    obj -> string_in_t(s);

        t.stop();
    }
    catch(CORBA::SystemException &ex)
    {
    	cout << "string_in_t(): call failed"
    	     << endl
    	     << &ex
    	     << endl;
    	return;
    }
    
    cout << "\t" << t.timeElapsed() << " seconds (" 
         << (iterations / t.timeElapsed()) << "/sec)" << endl;
}

//----------------------------------------------------------------------
// Function:	string_out_t
//
// Description:	Client-side function that illustrates how to invoke the
//		Benchmark::BasicTester::string_out_t operation.
//----------------------------------------------------------------------

void
CallBasicTester::string_out_t(
    Benchmark::BasicTester_ptr obj,
    long iterations)
{
    CORBA::String_var s;
    
    cout << iterations << " Strings out: " << flush;
    Timer t;
    
    try
    {
        t.start();

        for(long i = 0; i < iterations; i++)
    	    obj -> string_out_t(s);
    
        t.stop();
    }
    catch(CORBA::SystemException &ex)
    {
    	cout << "string_out_t(): call failed"
    	     << endl
    	     << &ex
    	     << endl;
    	return;
    }
    
    cout << "\t" << t.timeElapsed() << " seconds (" 
         << (iterations / t.timeElapsed()) << "/sec)" << endl;
}

//----------------------------------------------------------------------
// Function:	string_inout_t
//
// Description:	Client-side function that illustrates how to invoke the
//		Benchmark::BasicTester::string_inout_t operation.
//----------------------------------------------------------------------

void
CallBasicTester::string_inout_t(
    Benchmark::BasicTester_ptr obj,
    long iterations)
{
    CORBA::String_var s;

    s = CORBA::string_dup("1400 Fashion Island Blvd, San Mateo, CA 94404");

    cout << iterations << " Strings i/o: " << flush;
    Timer t;

    try
    {
        t.start();

        for(long i = 0; i < iterations; i++)
    	    obj -> string_inout_t(s.inout());

        t.stop();
    }
    catch(CORBA::SystemException &ex)
    {
    	cout << "string_inout_t(): call failed"
    	     << endl
    	     << &ex
    	     << endl;
    	return;
    }
    
    cout << "\t" << t.timeElapsed() << " seconds (" 
         << (iterations / t.timeElapsed()) << "/sec)" << endl;
}

//----------------------------------------------------------------------
// Function:	string_ret_t
//
// Description:	Client-side function that illustrates how to invoke the
//		Benchmark::BasicTester::string_ret_t operation.
//----------------------------------------------------------------------

void
CallBasicTester::string_ret_t(
    Benchmark::BasicTester_ptr obj,
    long iterations)
{
    CORBA::String_var result;

    cout << iterations << " Strings ret: " << flush;
    Timer t;
    
    try
    {
        t.start();

        for(long i = 0; i < iterations; i++)
    	    result = obj -> string_ret_t();

        t.stop();
    }
    catch(CORBA::SystemException &ex)
    {
    	cout << "string_ret_t(): call failed"
    	     << endl
    	     << &ex
    	     << endl;
    	return;
    }
    
    cout << "\t" << t.timeElapsed() << " seconds (" 
         << (iterations / t.timeElapsed()) << "/sec)" << endl;
}

//----------------------------------------------------------------------
// Function:	wstring_in_t
//
// Description:	Client-side function that illustrates how to invoke the
//		Benchmark::BasicTester::string_in_t operation.
//----------------------------------------------------------------------

void
CallBasicTester::wstring_in_t(
    Benchmark::BasicTester_ptr obj,
    long iterations)
{
    CORBA::WString_var ws;

    ws = CORBA::wstring_dup(L"Mountain bikes rule!");
    
    cout << iterations << " WStrings in:  " << flush;
    Timer t;
    
    try
    {
        t.start();

        for(long i = 0; i < iterations; i++)
    	    obj -> wstring_in_t(ws);

        t.stop();
    }
    catch(CORBA::SystemException &ex)
    {
    	cout << "wstring_in_t(): call failed"
    	     << endl
    	     << &ex
    	     << endl;
    	return;
    }
    
    cout << "\t" << t.timeElapsed() << " seconds (" 
         << (iterations / t.timeElapsed()) << "/sec)" << endl;
}

//----------------------------------------------------------------------
// Function:	wstring_out_t
//
// Description:	Client-side function that illustrates how to invoke the
//		Benchmark::BasicTester::wstring_out_t operation.
//----------------------------------------------------------------------

void
CallBasicTester::wstring_out_t(
    Benchmark::BasicTester_ptr obj,
    long iterations)
{
    CORBA::WString_var ws;
    
    cout << iterations << " WStrings out: " << flush;
    Timer t;
    
    try
    {
        t.start();

        for(long i = 0; i < iterations; i++)
    	    obj -> wstring_out_t(ws);

        t.stop();
    }
    catch(CORBA::SystemException &ex)
    {
    	cout << "wstring_out_t(): call failed"
    	     << endl
    	     << &ex
    	     << endl;
    	return;
    }
    
    cout << "\t" << t.timeElapsed() << " seconds (" 
         << (iterations / t.timeElapsed()) << "/sec)" << endl;
}

//----------------------------------------------------------------------
// Function:	wstring_inout_t
//
// Description:	Client-side function that illustrates how to invoke the
//		Benchmark::BasicTester::wstring_inout_t operation.
//----------------------------------------------------------------------

void
CallBasicTester::wstring_inout_t(
    Benchmark::BasicTester_ptr obj,
    long iterations)
{
    CORBA::WString_var ws;

    ws = CORBA::wstring_dup(L"1400 Fashion Island Blvd, San Mateo, CA 94404");
    
    cout << iterations << " WStrings i/o: " << flush;
    Timer t;

    try
    {
        t.start();

        for(long i = 0; i < iterations; i++)
    	    obj -> wstring_inout_t(ws.inout());
    
        t.stop();
    }
    catch(CORBA::SystemException &ex)
    {
    	cout << "wstring_inout_t(): call failed"
    	     << endl
    	     << &ex
    	     << endl;
    	return;
    }
    
    cout << "\t" << t.timeElapsed() << " seconds (" 
         << (iterations / t.timeElapsed()) << "/sec)" << endl;
}

//----------------------------------------------------------------------
// Function:	wstring_ret_t
//
// Description:	Client-side function that illustrates how to invoke the
//		Benchmark::BasicTester::wstring_ret_t operation.
//----------------------------------------------------------------------

void
CallBasicTester::wstring_ret_t(
    Benchmark::BasicTester_ptr obj,
    long iterations)
{
    CORBA::WString_var result;
    
    cout << iterations << " WStrings ret: " << flush;
    Timer t;
    
    try
    {
        t.start();

        for(long i = 0; i < iterations; i++)
    	    result = obj -> wstring_ret_t();

        t.stop();
    }
    catch(CORBA::SystemException &ex)
    {
    	cout << "wstring_ret_t(): call failed"
    	     << endl
    	     << &ex
    	     << endl;
    	return;
    }
    
    cout << "\t" << t.timeElapsed() << " seconds (" 
         << (iterations / t.timeElapsed()) << "/sec)" << endl;
}

//----------------------------------------------------------------------
// Function:	boolean_in_t
//
// Description:	Client-side function that illustrates how to invoke the
//		Benchmark::BasicTester::boolean_in_t operation.
//----------------------------------------------------------------------

void
CallBasicTester::boolean_in_t(
    Benchmark::BasicTester_ptr obj,
    long iterations)
{
    CORBA::Boolean b;
    
    b = (CORBA::Boolean) true;
    
    cout << iterations << " Booleans in:  " << flush;
    Timer t;
    
    try
    {
        t.start();

        for(long i = 0; i < iterations; i++)
    	    obj -> boolean_in_t(b);

        t.stop();
    }
    catch(CORBA::SystemException &ex)
    {
   	cout << "boolean_in_t(): call failed"
    	     << endl
    	     << &ex
    	     << endl;
    	return;
    }

    cout << "\t" << t.timeElapsed() << " seconds (" 
         << (iterations / t.timeElapsed()) << "/sec)" << endl;
}

//----------------------------------------------------------------------
// Function:	boolean_out_t
//
// Description:	Client-side function that illustrates how to invoke the
//		Benchmark::BasicTester::boolean_out_t operation.
//----------------------------------------------------------------------

void
CallBasicTester::boolean_out_t(
    Benchmark::BasicTester_ptr obj,
    long iterations)
{
    CORBA::Boolean b;
    
    cout << iterations << " Booleans out: " << flush;
    Timer t;
    
    try
    {
        t.start();

        for(long i = 0; i < iterations; i++)
    	    obj -> boolean_out_t(b);

        t.stop();
    }
    catch(CORBA::SystemException &ex)
    {
    	cout << "boolean_out_t(): call failed"
    	     << endl
    	     << &ex
    	     << endl;
    	return;
    }
    
    cout << "\t" << t.timeElapsed() << " seconds (" 
         << (iterations / t.timeElapsed()) << "/sec)" << endl;
}

//----------------------------------------------------------------------
// Function:	boolean_inout_t
//
// Description:	Client-side function that illustrates how to invoke the
//		Benchmark::BasicTester::boolean_inout_t operation.
//----------------------------------------------------------------------

void
CallBasicTester::boolean_inout_t(
    Benchmark::BasicTester_ptr obj,
    long iterations)
{
    CORBA::Boolean b;
    
    b = (CORBA::Boolean) false;
    
    cout << iterations << " Booleans i/o: " << flush;
    Timer t;

    try
    {
        t.start();

        for(long i = 0; i < iterations; i++)
    	    obj -> boolean_inout_t(b);

        t.stop();
    }
    catch(CORBA::SystemException &ex)
    {
    	cout << "boolean_inout_t(): call failed"
    	     << endl
    	     << &ex
    	     << endl;
    	return;
    }
    
    cout << "\t" << t.timeElapsed() << " seconds (" 
         << (iterations / t.timeElapsed()) << "/sec)" << endl;
}

//----------------------------------------------------------------------
// Function:	boolean_ret_t
//
// Description:	Client-side function that illustrates how to invoke the
//		Benchmark::BasicTester::boolean_ret_t operation.
//----------------------------------------------------------------------

void
CallBasicTester::boolean_ret_t(
    Benchmark::BasicTester_ptr obj,
    long iterations)
{
    CORBA::Boolean result;
    
    cout << iterations << " Booleans ret: " << flush;
    Timer t;
          
    try
    {
        t.start();

        for(long i = 0; i < iterations; i++)
    	    result = obj -> boolean_ret_t();

        t.stop();
    }
    catch(CORBA::SystemException &ex)
    {
    	cout << "boolean_ret_t(): call failed"
    	     << endl
    	     << &ex
    	     << endl;
    	return;
    }
    
    cout << "\t" << t.timeElapsed() << " seconds (" 
         << (iterations / t.timeElapsed()) << "/sec)" << endl;
}

//----------------------------------------------------------------------
// Function:	octet_in_t
//
// Description:	Client-side function that illustrates how to invoke the
//		Benchmark::BasicTester::octet_in_t operation.
//----------------------------------------------------------------------

void
CallBasicTester::octet_in_t(
    Benchmark::BasicTester_ptr obj,
    long iterations)
{
    CORBA::Octet o;
    
    o = (CORBA::Octet) 7;
    
    cout << iterations << " Octets in:  " << flush;
    Timer t;

    try
    {
        t.start();

        for(long i = 0; i < iterations; i++)
    	    obj -> octet_in_t(o);

        t.stop();
    }
    catch(CORBA::SystemException &ex)
    {
    	cout << "octet_in_t(): call failed"
    	     << endl
    	     << &ex
    	     << endl;
    	return;
    }
    
    cout << "\t" << t.timeElapsed() << " seconds (" 
         << (iterations / t.timeElapsed()) << "/sec)" << endl;
}

//----------------------------------------------------------------------
// Function:	octet_out_t
//
// Description:	Client-side function that illustrates how to invoke the
//		Benchmark::BasicTester::octet_out_t operation.
//----------------------------------------------------------------------

void
CallBasicTester::octet_out_t(
    Benchmark::BasicTester_ptr obj,
    long iterations)
{
    CORBA::Octet o;
    
    cout << iterations << " Octets out: " << flush;
    Timer t;
    
    try
    {
        t.start();

        for(long i = 0; i < iterations; i++)
    	    obj -> octet_out_t(o);

        t.stop();
    }
    catch(CORBA::SystemException &ex)
    {
    	cout << "octet_out_t(): call failed"
    	     << endl
    	     << &ex
    	     << endl;
    	return;
    }
    
    cout << "\t" << t.timeElapsed() << " seconds (" 
         << (iterations / t.timeElapsed()) << "/sec)" << endl;
}

//----------------------------------------------------------------------
// Function:	octet_inout_t
//
// Description:	Client-side function that illustrates how to invoke the
//		Benchmark::BasicTester::octet_inout_t operation.
//----------------------------------------------------------------------

void
CallBasicTester::octet_inout_t(
    Benchmark::BasicTester_ptr obj,
    long iterations)
{
    CORBA::Octet o;
    
    o = (CORBA::Octet) 0;

    cout << iterations << " Octets i/o: " << flush;
    Timer t;

    try
    {
        t.start();

        for(long i = 0; i < iterations; i++)
    	    obj -> octet_inout_t(o);
    
        t.stop();
    }
    catch(CORBA::SystemException &ex)
    {
    	cout << "octet_inout_t(): call failed"
    	     << endl
    	     << &ex
    	     << endl;
    	return;
    }

    cout << "\t" << t.timeElapsed() << " seconds (" 
         << (iterations / t.timeElapsed()) << "/sec)" << endl;
}

//----------------------------------------------------------------------
// Function:	octet_ret_t
//
// Description:	Client-side function that illustrates how to invoke the
//		Benchmark::BasicTester::octet_ret_t operation.
//----------------------------------------------------------------------

void
CallBasicTester::octet_ret_t(
    Benchmark::BasicTester_ptr obj,
    long iterations)
{
    CORBA::Octet result;
    
    cout << iterations << " Octets ret: " << flush;
    Timer t;
    
    try
    {
        t.start();

        for(long i = 0; i < iterations; i++)
    	    result = obj -> octet_ret_t();

        t.stop();
    }
    catch(CORBA::SystemException &ex)
    {
    	cout << "octet_ret_t(): call failed"
    	     << endl
    	     << &ex
    	     << endl;
    	return;
    }
    
    cout << "\t" << t.timeElapsed() << " seconds (" 
         << (iterations / t.timeElapsed()) << "/sec)" << endl;
}

//----------------------------------------------------------------------
// Function:	any_in_t
//
// Description:	Client-side function that illustrates how to invoke the
//		Benchmark::BasicTester::any_in_t operation.
//----------------------------------------------------------------------

void
CallBasicTester::any_in_t(
    Benchmark::BasicTester_ptr obj,
    long iterations)
{
    CORBA::Any a;
    
    a <<= "What's up, Chuck?";

    cout << iterations << " Anys in: \t" << flush;
    Timer t;

    try
    {
        t.start();

        for(long i = 0; i < iterations; i++)
    	    obj -> any_in_t(a);

        t.stop();
    }
    catch(CORBA::SystemException &ex)
    {
    	cout << "any_in_t(): call failed"
    	     << endl
    	     << &ex
    	     << endl;
    	return;
    }
    
    cout << "\t" << t.timeElapsed() << " seconds (" 
         << (iterations / t.timeElapsed()) << "/sec)" << endl;
}

//----------------------------------------------------------------------
// Function:	any_out_t
//
// Description:	Client-side function that illustrates how to invoke the
//		Benchmark::BasicTester::any_out_t operation.
//----------------------------------------------------------------------

void
CallBasicTester::any_out_t(
    Benchmark::BasicTester_ptr obj,
    long iterations)
{
    CORBA::Any_var a;
    
    cout << iterations << " Anys out: \t" << flush;
    Timer t;
    
    try
    {
        t.start();

        for(long i = 0; i < iterations; i++)
    	    obj -> any_out_t(a);

        t.stop();
    }
    catch(CORBA::SystemException &ex)
    {
    	cout << "any_out_t(): call failed"
    	     << endl
    	     << &ex
    	     << endl;
    	return;
    }
    
    cout << "\t" << t.timeElapsed() << " seconds (" 
         << (iterations / t.timeElapsed()) << "/sec)" << endl;
}

//----------------------------------------------------------------------
// Function:	any_inout_t
//
// Description:	Client-side function that illustrates how to invoke the
//		Benchmark::BasicTester::any_inout_t operation.
//----------------------------------------------------------------------

void
CallBasicTester::any_inout_t(
    Benchmark::BasicTester_ptr obj,
    long iterations)
{
    CORBA::Any a;

    a <<= (CORBA::Float) 44.372;

    cout << iterations << " Anys i/o: \t" << flush;
    Timer t;
    
    try
    {
        t.start();

        for(long i = 0; i < iterations; i++)
    	    obj -> any_inout_t(a);
    
        t.stop();
    }
    catch(CORBA::SystemException &ex)
    {
    	cout << "any_inout_t(): call failed"
    	     << endl
    	     << &ex
    	     << endl;
    	return;
    }

    cout << "\t" << t.timeElapsed() << " seconds (" 
         << (iterations / t.timeElapsed()) << "/sec)" << endl;
}

//----------------------------------------------------------------------
// Function:	any_ret_t
//
// Description:	Client-side function that illustrates how to invoke the
//		Benchmark::BasicTester::any_ret_t operation.
//----------------------------------------------------------------------

void
CallBasicTester::any_ret_t(
    Benchmark::BasicTester_ptr obj,
    long iterations)
{
    CORBA::Any_var result;
    
    cout << iterations << " Anys ret: \t" << flush;
    Timer t;
    
    try
    {
        t.start();

        for(long i = 0; i < iterations; i++)
    	    result = obj -> any_ret_t();
    
        t.stop();
    }
    catch(CORBA::SystemException &ex)
    {
    	cout << "any_ret_t(): call failed"
    	     << endl
    	     << &ex
    	     << endl;
   	return;
    }

    cout << "\t" << t.timeElapsed() << " seconds (" 
         << (iterations / t.timeElapsed()) << "/sec)" << endl;
}
