// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>

#include <CallSequenceTester.h>
#include <Timer.h>
#include <Constants.h>

#ifdef HAVE_STD_IOSTREAM
using namespace std;
#endif

//----------------------------------------------------------------------
// Function:	ShortSeq_UB_in_t
//
// Description:	Client-side function that illustrates how to invoke the
//		Benchmark::SequenceTester::ShortSeq_UB_in_t operation.
//----------------------------------------------------------------------

void
CallSequenceTester::ShortSeq_UB_in_t(
    Benchmark::SequenceTester_ptr obj,
    long iterations)
{
    Benchmark::SequenceTester::ShortSeq_UB s_seq;
    
    s_seq.length(ub_sequence_length);

    for(CORBA::ULong i = 0; i < ub_sequence_length; i++)
        s_seq[i] = (CORBA::Short) i;

    //
    // Warm up "ping" to establish connection
    //
    obj -> ShortSeq_UB_in_t(
			    s_seq);
    cout << iterations << " ShortSeq_UB in:  " << flush;
    Timer t;

    try
    {
        t.start();

        for(long i = 0; i < iterations; i++)
    	    obj -> ShortSeq_UB_in_t(s_seq);

        t.stop();
    }
    catch(CORBA::SystemException &ex)
    {
        cout << "ShortSeq_UB_in_t(): call failed"
    	     << endl
    	     << &ex
    	     << endl;
        return;
    }

    cout << "\t" << t.timeElapsed() << " seconds (" 
         << (iterations / t.timeElapsed()) << "/sec)" << endl;
}

//----------------------------------------------------------------------
// Function:	ShortSeq_UB_out_t
//
// Description:	Client-side function that illustrates how to invoke the
//		Benchmark::SequenceTester::ShortSeq_UB_out_t operation.
//----------------------------------------------------------------------

void
CallSequenceTester::ShortSeq_UB_out_t(
    Benchmark::SequenceTester_ptr obj,
    long iterations)
{
    Benchmark::SequenceTester::ShortSeq_UB_var s_seq;

    cout << iterations << " ShortSeq_UB out: " << flush;
    Timer t;

    try
    {
        t.start();

        for(long i = 0; i < iterations; i++)
    	    obj -> ShortSeq_UB_out_t(s_seq);

        t.stop();
    }
    catch(CORBA::SystemException &ex)
    {
    	cout << "ShortSeq_UB_out_t(): call failed"
    	     << endl
    	     << &ex
    	     << endl;
    	return;
    }

    cout << "\t" << t.timeElapsed() << " seconds (" 
         << (iterations / t.timeElapsed()) << "/sec)" << endl;
}

//----------------------------------------------------------------------
// Function:	ShortSeq_UB_inout_t
//
// Description:	Client-side function that illustrates how to invoke the
//		Benchmark::SequenceTester::ShortSeq_UB_inout_t operation.
//----------------------------------------------------------------------

void
CallSequenceTester::ShortSeq_UB_inout_t(
    Benchmark::SequenceTester_ptr obj,
    long iterations)
{
    Benchmark::SequenceTester::ShortSeq_UB s_seq;

    s_seq.length(ub_sequence_length);
    
    for(CORBA::ULong i = 0; i < ub_sequence_length; i++)
        s_seq[i] = (CORBA::Short) i;
    
    cout << iterations << " ShortSeq_UB i/o: " << flush;
    Timer t;

    try
    {
        t.start();

        for(long i = 0; i < iterations; i++)
    	    obj -> ShortSeq_UB_inout_t(s_seq);

        t.stop();
    }
    catch(CORBA::SystemException &ex)
    {
        cout << "ShortSeq_UB_inout_t():"
	     << " call failed"
    	     << endl
    	     << &ex
    	     << endl;
    	return;
    }

    cout << "\t" << t.timeElapsed() << " seconds (" 
         << (iterations / t.timeElapsed()) << "/sec)" << endl;
}

//----------------------------------------------------------------------
// Function:	ShortSeq_UB_ret_t
//
// Description:	Client-side function that illustrates how to invoke the
//		Benchmark::SequenceTester::ShortSeq_UB_ret_t operation.
//----------------------------------------------------------------------

void
CallSequenceTester::ShortSeq_UB_ret_t(
    Benchmark::SequenceTester_ptr obj,
    long iterations)
{
    Benchmark::SequenceTester::ShortSeq_UB_var result;

    cout << iterations << " ShortSeq_UB ret: " << flush;
    Timer t;

    try
    {
        t.start();

        for(long i = 0; i < iterations; i++)
    	    result = obj -> ShortSeq_UB_ret_t();

        t.stop();
    }
    catch(CORBA::SystemException &ex)
    {
        cout << "ShortSeq_UB_ret_t():"
	     << " call failed"
    	     << endl
    	     << &ex
    	     << endl;
    	return;
    }

    cout << "\t" << t.timeElapsed() << " seconds (" 
         << (iterations / t.timeElapsed()) << "/sec)" << endl;
}

//----------------------------------------------------------------------
// Function:	FloatSeq_64K_in_t
//
// Description:	Client-side function that illustrates how to invoke the
//		Benchmark::SequenceTester::FloatSeq_64K_in_t operation.
//----------------------------------------------------------------------

void
CallSequenceTester::FloatSeq_64K_in_t(
    Benchmark::SequenceTester_ptr obj,
    long iterations)
{
    Benchmark::SequenceTester::FloatSeq_64K f_seq;

    f_seq.length(65536);

    for(long i = 0; i < 65536; i++)
        f_seq[i] = (CORBA::Float) i;

    cout << iterations << " FloatSeq_64K in:  " << flush;
    Timer t;

    try
    {
        t.start();

        for(long i = 0; i < iterations; i++)
    	    obj -> FloatSeq_64K_in_t(f_seq);

        t.stop();
    }
    catch(CORBA::SystemException &ex)
    {
        cout << "FloatSeq_64K_in_t():"
	     << " call failed"
    	     << endl
    	     << &ex
    	     << endl;
    	return;
    }

    cout << "\t" << t.timeElapsed() << " seconds (" 
         << (iterations / t.timeElapsed()) << "/sec)" << endl;
}

//----------------------------------------------------------------------
// Function:	FloatSeq_64K_out_t
//
// Description:	Client-side function that illustrates how to invoke the
//		Benchmark::SequenceTester::FloatSeq_64K_out_t operation.
//----------------------------------------------------------------------

void
CallSequenceTester::FloatSeq_64K_out_t(
    Benchmark::SequenceTester_ptr obj,
    long iterations)
{
    Benchmark::SequenceTester::FloatSeq_64K_var f_seq;

    cout << iterations << " FloatSeq_64K out: " << flush;
    Timer t;

    try
    {
        t.start();

        for(long i = 0; i < iterations; i++)
    	    obj -> FloatSeq_64K_out_t(f_seq);

        t.stop();
    }
    catch(CORBA::SystemException &ex)
    {
        cout << "FloatSeq_64K_out_t():"
	     << " call failed"
    	     << endl
    	     << &ex
    	     << endl;
    	return;
    }

    cout << "\t" << t.timeElapsed() << " seconds (" 
         << (iterations / t.timeElapsed()) << "/sec)" << endl;
}

//----------------------------------------------------------------------
// Function:	FloatSeq_64K_inout_t
//
// Description:	Client-side function that illustrates how to invoke the
//		Benchmark::SequenceTester::FloatSeq_64K_inout_t operation.
//----------------------------------------------------------------------

void
CallSequenceTester::FloatSeq_64K_inout_t(
    Benchmark::SequenceTester_ptr obj,
    long iterations)
{
    Benchmark::SequenceTester::FloatSeq_64K f_seq;

    f_seq.length(65536);

    for(long i = 0; i < 65536; i++)
        f_seq[i] = (CORBA::Float) i;

    cout << iterations << " FloatSeq_64K i/o: " << flush;
    Timer t;

    try
    {
        t.start();

        for(long i = 0; i < iterations; i++)
    	    obj -> FloatSeq_64K_inout_t(f_seq);

        t.stop();
    }
    catch(CORBA::SystemException &ex)
    {
        cout << "FloatSeq_64K_inout_t():"
	     << " call failed"
    	     << endl
    	     << &ex
    	     << endl;
    	return;
    }

    cout << "\t" << t.timeElapsed() << " seconds (" 
         << (iterations / t.timeElapsed()) << "/sec)" << endl;
}

//----------------------------------------------------------------------
// Function:	FloatSeq_64K_ret_t
//
// Description:	Client-side function that illustrates how to invoke the
//		Benchmark::SequenceTester::FloatSeq_64K_ret_t operation.
//----------------------------------------------------------------------

void
CallSequenceTester::FloatSeq_64K_ret_t(
    Benchmark::SequenceTester_ptr obj,
    long iterations)
{
    Benchmark::SequenceTester::FloatSeq_64K_var result;

    cout << iterations << " FloatSeq_64K ret: " << flush;
    Timer t;

    try
    {
        t.start();

        for(long i = 0; i < iterations; i++)
    	    result = obj -> FloatSeq_64K_ret_t();

        t.stop();
    }
    catch(CORBA::SystemException &ex)
    {
        cout << "FloatSeq_64K_ret_t():"
	     << " call failed"
    	     << endl
    	     << &ex
    	     << endl;
    	return;
    }

    cout << "\t" << t.timeElapsed() << " seconds (" 
         << (iterations / t.timeElapsed()) << "/sec)" << endl;
}

//----------------------------------------------------------------------
// Function:	StringSeq_UB_in_t
//
// Description:	Client-side function that illustrates how to invoke the
//		Benchmark::SequenceTester::StringSeq_UB_in_t operation.
//----------------------------------------------------------------------

void
CallSequenceTester::StringSeq_UB_in_t(
    Benchmark::SequenceTester_ptr obj,
    long iterations)
{
    Benchmark::SequenceTester::StringSeq_UB s_seq;

    s_seq.length(ub_sequence_length);

    for(CORBA::ULong i = 0; i < ub_sequence_length; i++)
        s_seq[i] = CORBA::string_dup ("Marathon Racing #1");

    cout << iterations << " StringSeq_UB in:  " << flush;
    Timer t;

    try
    {
        t.start();

        for(long i = 0; i < iterations; i++)
    	    obj -> StringSeq_UB_in_t(s_seq);

        t.stop();
    }
    catch(CORBA::SystemException &ex)
    {
        cout << "StringSeq_UB_in_t():"
	     << " call failed"
    	     << endl
    	     << &ex
    	     << endl;
    	return;
    }

    cout << "\t" << t.timeElapsed() << " seconds (" 
         << (iterations / t.timeElapsed()) << "/sec)" << endl;
}

//----------------------------------------------------------------------
// Function:	StringSeq_UB_out_t
//
// Description:	Client-side function that illustrates how to invoke the
//		Benchmark::SequenceTester::StringSeq_UB_out_t operation.
//----------------------------------------------------------------------

void
CallSequenceTester::StringSeq_UB_out_t(
    Benchmark::SequenceTester_ptr obj,
    long iterations)
{
    Benchmark::SequenceTester::StringSeq_UB_var s_seq;

    cout << iterations << " StringSeq_UB out: " << flush;
    Timer t;

    try
    {
        t.start();

        for(long i = 0; i < iterations; i++)
    	    obj -> StringSeq_UB_out_t(s_seq);

        t.stop();
    }
    catch(CORBA::SystemException &ex)
    {
        cout << "StringSeq_UB_out_t():"
	     << " call failed"
    	     << endl
    	     << &ex
    	     << endl;
    	return;
    }

    cout << "\t" << t.timeElapsed() << " seconds (" 
         << (iterations / t.timeElapsed()) << "/sec)" << endl;
}

//----------------------------------------------------------------------
// Function:	StringSeq_UB_inout_t
//
// Description:	Client-side function that illustrates how to invoke the
//		Benchmark::SequenceTester::StringSeq_UB_inout_t operation.
//----------------------------------------------------------------------

void
CallSequenceTester::StringSeq_UB_inout_t(
    Benchmark::SequenceTester_ptr obj,
    long iterations)
{
    Benchmark::SequenceTester::StringSeq_UB s_seq;

    s_seq.length(ub_sequence_length);

    for(CORBA::ULong i = 0; i < ub_sequence_length; i++)
        s_seq[i] = CORBA::string_dup ("Buy a Chia pet for Xmas!");

    cout << iterations << " StringSeq_UB i/o: " << flush;
    Timer t;

    try
    {
        t.start();

        for(long i = 0; i < iterations; i++)
    	    obj -> StringSeq_UB_inout_t(s_seq);

        t.stop();
    }
    catch(CORBA::SystemException &ex)
    {
        cout << "StringSeq_UB_inout_t():"
	     << " call failed"
    	     << endl
    	     << &ex
    	     << endl;
    	return;
    }

    cout << "\t" << t.timeElapsed() << " seconds (" 
         << (iterations / t.timeElapsed()) << "/sec)" << endl;
}

//----------------------------------------------------------------------
// Function:	StringSeq_UB_ret_t
//
// Description:	Client-side function that illustrates how to invoke the
//		Benchmark::SequenceTester::StringSeq_UB_ret_t operation.
//----------------------------------------------------------------------

void
CallSequenceTester::StringSeq_UB_ret_t(
    Benchmark::SequenceTester_ptr obj,
    long iterations)
{
    Benchmark::SequenceTester::StringSeq_UB_var result;

    cout << iterations << " StringSeq_UB ret: " << flush;
    Timer t;

    try
    {
        t.start();

        for(long i = 0; i < iterations; i++)
    	    result = obj -> StringSeq_UB_ret_t();

        t.stop();
    }
    catch(CORBA::SystemException &ex)
    {
        cout << "StringSeq_UB_ret_t():"
	     << " call failed"
    	     << endl
    	     << &ex
    	     << endl;
    	return;
    }

    cout << "\t" << t.timeElapsed() << " seconds (" 
         << (iterations / t.timeElapsed()) << "/sec)" << endl;
}

//----------------------------------------------------------------------
// Function:	OctetSeq_1M_in_t
//
// Description:	Client-side function that illustrates how to invoke the
//		Benchmark::SequenceTester::OctetSeq_1M_in_t operation.
//----------------------------------------------------------------------

void
CallSequenceTester::OctetSeq_1M_in_t(
    Benchmark::SequenceTester_ptr obj,
    long iterations)
{
    Benchmark::SequenceTester::OctetSeq_1M o_seq;

    o_seq.length (1024*1024);

    for(long i = 0; i < 1024*1024; i++)
        o_seq[i] = (CORBA::Octet) (i % 256);

    cout << iterations << " OctetSeq_1M in:  " << flush;
    Timer t;

    try
    {
        t.start();

        for(long i = 0; i < iterations; i++)
    	    obj -> OctetSeq_1M_in_t(o_seq);

        t.stop();
    }
    catch(CORBA::SystemException &ex)
    {
        cout << "OctetSeq_1M_in_t():"
	     << " call failed"
    	     << endl
    	     << &ex
    	     << endl;
    	return;
    }

    cout << "\t" << t.timeElapsed() << " seconds (" 
         << (iterations / t.timeElapsed()) << "/sec)" << endl;
}

//----------------------------------------------------------------------
// Function:	OctetSeq_1M_out_t
//
// Description:	Client-side function that illustrates how to invoke the
//		Benchmark::SequenceTester::OctetSeq_1M_out_t operation.
//----------------------------------------------------------------------

void
CallSequenceTester::OctetSeq_1M_out_t(
    Benchmark::SequenceTester_ptr obj,
    long iterations)
{
    Benchmark::SequenceTester::OctetSeq_1M_var o_seq;

    cout << iterations << " OctetSeq_1M out: " << flush;
    Timer t;

    try
    {
        t.start();

        for(long i = 0; i < iterations; i++)
    	    obj -> OctetSeq_1M_out_t(o_seq);

        t.stop();
    }
    catch(CORBA::SystemException &ex)
    {
        cout << "OctetSeq_1M_out_t():"
	     << " call failed"
    	     << endl
    	     << &ex
    	     << endl;
    	return;
    }

    cout << "\t" << t.timeElapsed() << " seconds (" 
         << (iterations / t.timeElapsed()) << "/sec)" << endl;
}

//----------------------------------------------------------------------
// Function:	OctetSeq_1M_inout_t
//
// Description:	Client-side function that illustrates how to invoke the
//		Benchmark::SequenceTester::OctetSeq_1M_inout_t operation.
//----------------------------------------------------------------------

void
CallSequenceTester::OctetSeq_1M_inout_t(
    Benchmark::SequenceTester_ptr obj,
    long iterations)
{
    Benchmark::SequenceTester::OctetSeq_1M o_seq;

    o_seq.length (1024*1024);

    for(long i = 0; i < 1024*1024; i++)
        o_seq[i] = (CORBA::Octet) (i % 256);

    cout << iterations << " OctetSeq_1M i/o: " << flush;
    Timer t;

    try
    {
        t.start();

        for(long i = 0; i < iterations; i++)
    	    obj -> OctetSeq_1M_inout_t(o_seq);

        t.stop();
    }
    catch(CORBA::SystemException &ex)
    {
        cout << "OctetSeq_1M_inout_t():"
	     << " call failed"
    	     << endl
    	     << &ex
    	     << endl;
    	return;
    }

    cout << "\t" << t.timeElapsed() << " seconds (" 
         << (iterations / t.timeElapsed()) << "/sec)" << endl;
}

//----------------------------------------------------------------------
// Function:	OctetSeq_1M_ret_t
//
// Description:	Client-side function that illustrates how to invoke the
//		Benchmark::SequenceTester::OctetSeq_1M_ret_t operation.
//----------------------------------------------------------------------

void
CallSequenceTester::OctetSeq_1M_ret_t(
    Benchmark::SequenceTester_ptr obj,
    long iterations)
{
    Benchmark::SequenceTester::OctetSeq_1M_var result;

    cout << iterations << " OctetSeq_1M ret: " << flush;
    Timer t;

    try
    {
        t.start();

        for(long i = 0; i < iterations; i++)
    	    result = obj -> OctetSeq_1M_ret_t();

        t.stop();
    }
    catch(CORBA::SystemException &ex)
    {
        cout << "OctetSeq_1M_ret_t():"
	     << " call failed"
    	     << endl
    	     << &ex
    	     << endl;
    	return;
    }

    cout << "\t" << t.timeElapsed() << " seconds (" 
         << (iterations / t.timeElapsed()) << "/sec)" << endl;
}

//----------------------------------------------------------------------
// Function:	AnySeq_UB_in_t
//
// Description:	Client-side function that illustrates how to invoke the
//		Benchmark::SequenceTester::AnySeq_UB_in_t operation.
//----------------------------------------------------------------------

void
CallSequenceTester::AnySeq_UB_in_t(
    Benchmark::SequenceTester_ptr obj,
    long iterations)
{
    Benchmark::SequenceTester::AnySeq_UB a_seq;

    a_seq.length (ub_sequence_length);

    for(CORBA::ULong i = 0; i < ub_sequence_length; i++)
        a_seq[i] <<= "What's up, Doc?";

    cout << iterations << " AnySeq_UB in:  " << flush;
    Timer t;

    try
    {
        t.start();

        for(long i = 0; i < iterations; i++)
    	    obj -> AnySeq_UB_in_t(a_seq);
    
        t.stop();
    }
    catch(CORBA::SystemException &ex)
    {
        cout << "AnySeq_UB_in_t():"
	     << " call failed"
    	     << endl
    	     << &ex
    	     << endl;
    	return;
    }

    cout << "\t" << t.timeElapsed() << " seconds (" 
         << (iterations / t.timeElapsed()) << "/sec)" << endl;
}

//----------------------------------------------------------------------
// Function:	AnySeq_UB_out_t
//
// Description:	Client-side function that illustrates how to invoke the
//		Benchmark::SequenceTester::AnySeq_UB_out_t operation.
//----------------------------------------------------------------------

void
CallSequenceTester::AnySeq_UB_out_t(
    Benchmark::SequenceTester_ptr obj,
    long iterations)
{
    Benchmark::SequenceTester::AnySeq_UB_var a_seq;

    cout << iterations << " AnySeq_UB out: " << flush;
    Timer t;

    try
    {
        t.start();

        for(long i = 0; i < iterations; i++)
    	    obj -> AnySeq_UB_out_t(a_seq);

        t.stop();
    }
    catch(CORBA::SystemException &ex)
    {
        cout << "AnySeq_UB_out_t():"
	     << " call failed"
    	     << endl
    	     << &ex
    	     << endl;
    	return;
    }

    cout << "\t" << t.timeElapsed() << " seconds (" 
         << (iterations / t.timeElapsed()) << "/sec)" << endl;
}

//----------------------------------------------------------------------
// Function:	AnySeq_UB_inout_t
//
// Description:	Client-side function that illustrates how to invoke the
//		Benchmark::SequenceTester::AnySeq_UB_inout_t operation.
//----------------------------------------------------------------------

void
CallSequenceTester::AnySeq_UB_inout_t(
    Benchmark::SequenceTester_ptr obj,
    long iterations)
{
    Benchmark::SequenceTester::AnySeq_UB a_seq;

    a_seq.length (ub_sequence_length);

    for(CORBA::ULong i = 0; i < ub_sequence_length; i++)
        a_seq[i] <<= (CORBA::Short) 5;

    cout << iterations << " AnySeq_UB i/o:  " << flush;
    Timer t;

    try
    {
        t.start();

        for(long i = 0; i < iterations; i++)
    	    obj -> AnySeq_UB_inout_t(a_seq);

        t.stop();
    }
    catch(CORBA::SystemException &ex)
    {
        cout << "AnySeq_UB_inout_t():"
	     << " call failed"
    	     << endl
    	     << &ex
    	     << endl;
    	return;
    }

    cout << "\t" << t.timeElapsed() << " seconds (" 
         << (iterations / t.timeElapsed()) << "/sec)" << endl;
}

//----------------------------------------------------------------------
// Function:	AnySeq_UB_ret_t
//
// Description:	Client-side function that illustrates how to invoke the
//		Benchmark::SequenceTester::AnySeq_UB_ret_t operation.
//----------------------------------------------------------------------

void
CallSequenceTester::AnySeq_UB_ret_t(
    Benchmark::SequenceTester_ptr obj,
    long iterations)
{
    Benchmark::SequenceTester::AnySeq_UB_var result;

    cout << iterations << " AnySeq_UB ret: " << flush;
    Timer t;

    try
    {
        t.start();

        for(long i = 0; i < iterations; i++)
    	    obj -> AnySeq_UB_ret_t();
    
        t.stop();
    }
    catch(CORBA::SystemException &ex)
    {
        cout << "AnySeq_UB_ret_t():"
	     << " call failed"
    	     << endl
    	     << &ex
    	     << endl;
    	return;
    }

    cout << "\t" << t.timeElapsed() << " seconds (" 
         << (iterations / t.timeElapsed()) << "/sec)" << endl;
}

//----------------------------------------------------------------------
// Function:	U2Seq_UB_in_t
//
// Description:	Client-side function that illustrates how to invoke the
//		Benchmark::SequenceTester::U2Seq_UB_in_t operation.
//----------------------------------------------------------------------

void
CallSequenceTester::U2Seq_UB_in_t(
    Benchmark::SequenceTester_ptr obj,
    long iterations)
{
    Benchmark::SequenceTester::U2Seq_UB u2_seq;

    u2_seq.length (ub_sequence_length);

    UserTypes::u2   u;

    u.d((CORBA::Double) 6.8);

    for(CORBA::ULong i = 0; i < ub_sequence_length; i++)
    {
        u2_seq[i] = u;
    }

    cout << iterations << " U2Seq_UB in:  " << flush;
    Timer t;

    try
    {
        t.start();

        for(long i = 0; i < iterations; i++)
    	    obj -> U2Seq_UB_in_t(u2_seq);

        t.stop();
    }
    catch(CORBA::SystemException &ex)
    {
        cout << "U2Seq_UB_in_t():"
	     << " call failed"
    	     << endl
    	     << &ex
    	     << endl;
    	return;
    }

    cout << "\t" << t.timeElapsed() << " seconds (" 
         << (iterations / t.timeElapsed()) << "/sec)" << endl;
}

//----------------------------------------------------------------------
// Function:	U2Seq_UB_out_t
//
// Description:	Client-side function that illustrates how to invoke the
//		Benchmark::SequenceTester::U2Seq_UB_out_t operation.
//----------------------------------------------------------------------

void
CallSequenceTester::U2Seq_UB_out_t(
    Benchmark::SequenceTester_ptr obj,
    long iterations)
{
    Benchmark::SequenceTester::U2Seq_UB_var u2_seq;

    cout << iterations << " U2Seq_UB out: " << flush;
    Timer t;

    try
    {
        t.start();

        for(long i = 0; i < iterations; i++)
    	    obj -> U2Seq_UB_out_t(u2_seq);

        t.stop();
    }
    catch(CORBA::SystemException &ex)
    {
        cout << "U2Seq_UB_out_t():"
	     << " call failed"
    	     << endl
    	     << &ex
    	     << endl;
    	return;
    }

    cout << "\t" << t.timeElapsed() << " seconds (" 
         << (iterations / t.timeElapsed()) << "/sec)" << endl;
}

//----------------------------------------------------------------------
// Function:	U2Seq_UB_inout_t
//
// Description:	Client-side function that illustrates how to invoke the
//		Benchmark::SequenceTester::U2Seq_UB_inout_t operation.
//----------------------------------------------------------------------

void
CallSequenceTester::U2Seq_UB_inout_t(
    Benchmark::SequenceTester_ptr obj,
    long iterations)
{
    Benchmark::SequenceTester::U2Seq_UB u2_seq;

    u2_seq.length(ub_sequence_length);

    UserTypes::u2 u;

    u.d((CORBA::Double) 7.7);

    for(CORBA::ULong i = 0; i < ub_sequence_length; i++)
        u2_seq[i] = u;

    cout << iterations << " U2Seq_UB i/o: " << flush;
    Timer t;

    try
    {
        t.start();

        for(long i = 0; i < iterations; i++)
    	    obj -> U2Seq_UB_inout_t(u2_seq);

        t.stop();
    }
    catch(CORBA::SystemException &ex)
    {
        cout << "U2Seq_UB_inout_t():"
	     << " call failed"
    	     << endl
    	     << &ex
    	     << endl;
    	return;
    }

    cout << "\t" << t.timeElapsed() << " seconds (" 
         << (iterations / t.timeElapsed()) << "/sec)" << endl;
}

//----------------------------------------------------------------------
// Function:	U2Seq_UB_ret_t
//
// Description:	Client-side function that illustrates how to invoke the
//		Benchmark::SequenceTester::U2Seq_UB_ret_t operation.
//----------------------------------------------------------------------

void
CallSequenceTester::U2Seq_UB_ret_t(
    Benchmark::SequenceTester_ptr obj,
    long iterations)
{
    Benchmark::SequenceTester::U2Seq_UB_var result;

    cout << iterations << " U2Seq_UB ret: " << flush;
    Timer t;

    try
    {
        t.start();

        for(long i = 0; i < iterations; i++)
    	    result = obj -> U2Seq_UB_ret_t();
    
        t.stop();
    }
    catch(CORBA::SystemException &ex)
    {
        cout << "U2Seq_UB_ret_t():"
	     << " call failed"
    	     << endl
    	     << &ex
    	     << endl;
    	return;
    }

    cout << "\t" << t.timeElapsed() << " seconds (" 
         << (iterations / t.timeElapsed()) << "/sec)" << endl;
}

//----------------------------------------------------------------------
// Function:	S2Seq_UB_in_t
//
// Description:	Client-side function that illustrates how to invoke the
//		Benchmark::SequenceTester::S2Seq_UB_in_t operation.
//----------------------------------------------------------------------

void
CallSequenceTester::S2Seq_UB_in_t(
    Benchmark::SequenceTester_ptr obj,
    long iterations)
{
    Benchmark::SequenceTester::S2Seq_UB s2_seq;

    s2_seq.length (ub_sequence_length);

    UserTypes::s2                   s;

    s.s = (CORBA::Short) 234;
    s.l = (CORBA::Long) 723456235;
#ifdef HAVE_VCPLUSPLUS_BUGS
    s.ll = (CORBA::LongLong) 2303454020023040L;
#else
    s.ll = (CORBA::LongLong) 2303454020023040LL;
#endif
    s.us = (CORBA::UShort) 43343;
    s.ul = (CORBA::ULong) 338;
    s.f = (CORBA::Float) 45434.857;
    s.d = (CORBA::Double) 1857.37;
    s.c = (CORBA::Char) 'q';
    s.wc = (CORBA::WChar) 'b';
    s.st = CORBA::string_dup("Lemond");
    s.b = (CORBA::Boolean) false;
    s.o = (CORBA::Octet) 7;
    s.a <<= "Erin go bragh!";

    for(CORBA::ULong i = 0; i < ub_sequence_length; i++)
        s2_seq[i] = s;

    cout << iterations << " S2Seq_UB in:  " << flush;
    Timer t;

    try
    {
        t.start();

        for(long i = 0; i < iterations; i++)
    	    obj -> S2Seq_UB_in_t(s2_seq);

        t.stop();
    }
    catch(CORBA::SystemException &ex)
    {
        cout << "S2Seq_UB_in_t():"
	     << " call failed"
    	     << endl
    	     << &ex
    	     << endl;
    	return;
    }

    cout << "\t" << t.timeElapsed() << " seconds (" 
         << (iterations / t.timeElapsed()) << "/sec)" << endl;
}

//----------------------------------------------------------------------
// Function:	S2Seq_UB_out_t
//
// Description:	Client-side function that illustrates how to invoke the
//		Benchmark::SequenceTester::S2Seq_UB_out_t operation.
//----------------------------------------------------------------------

void
CallSequenceTester::S2Seq_UB_out_t(
    Benchmark::SequenceTester_ptr obj,
    long iterations)
{
    Benchmark::SequenceTester::S2Seq_UB_var s2_seq;

    cout << iterations << " S2Seq_UB out: " << flush;
    Timer t;

    try
    {
        t.start();

        for(long i = 0; i < iterations; i++)
    	    obj -> S2Seq_UB_out_t(s2_seq);

        t.stop();
    }
    catch(CORBA::SystemException &ex)
    {
        cout << "S2Seq_UB_out_t():"
	     << " call failed"
    	     << endl
    	     << &ex
    	     << endl;
    	return;
    }

    cout << "\t" << t.timeElapsed() << " seconds (" 
         << (iterations / t.timeElapsed()) << "/sec)" << endl;
}

//----------------------------------------------------------------------
// Function:	S2Seq_UB_inout_t
//
// Description:	Client-side function that illustrates how to invoke the
//		Benchmark::SequenceTester::S2Seq_UB_inout_t operation.
//----------------------------------------------------------------------

void
CallSequenceTester::S2Seq_UB_inout_t(
    Benchmark::SequenceTester_ptr obj,
    long iterations)
{
    Benchmark::SequenceTester::S2Seq_UB s2_seq;

    s2_seq.length (ub_sequence_length);

    UserTypes::s2                   s;

    s.s = (CORBA::Short) 234;
    s.l = (CORBA::Long) 723456235;
#ifdef HAVE_VCPLUSPLUS_BUGS
    s.ll = (CORBA::LongLong) 2303454020023040L;
#else
    s.ll = (CORBA::LongLong) 2303454020023040LL;
#endif
    s.us = (CORBA::UShort) 43343;
    s.ul = (CORBA::ULong) 338;
    s.f = (CORBA::Float) 45434.857;
    s.d = (CORBA::Double) 1857.37;
    s.c = (CORBA::Char) 'q';
    s.wc = (CORBA::WChar) 'b';
    s.st = CORBA::string_dup("Lemond");
    s.b = (CORBA::Boolean) false;
    s.o = (CORBA::Octet) 7;
    s.a <<= "Good thing this isn't a wstring or Visi would gag!";

    for(CORBA::ULong i = 0; i < ub_sequence_length; i++)
        s2_seq[i] = s;

    cout << iterations << " S2Seq_UB i/o: " << flush;
    Timer t;

    try
    {
        t.start();

        for(long i = 0; i < iterations; i++)
    	    obj -> S2Seq_UB_inout_t(s2_seq);

        t.stop();
    }
    catch(CORBA::SystemException &ex)
    {
        cout << "S2Seq_UB_inout_t():"
	     << " call failed"
    	     << endl
    	     << &ex
    	     << endl;
    	return;
    }

    cout << "\t" << t.timeElapsed() << " seconds (" 
         << (iterations / t.timeElapsed()) << "/sec)" << endl;
}

//----------------------------------------------------------------------
// Function:	S2Seq_UB_ret_t
//
// Description:	Client-side function that illustrates how to invoke the
//		Benchmark::SequenceTester::S2Seq_UB_ret_ operation.
//----------------------------------------------------------------------

void
CallSequenceTester::S2Seq_UB_ret_t(
    Benchmark::SequenceTester_ptr obj,
    long iterations)
{
    Benchmark::SequenceTester::S2Seq_UB_var result;

    cout << iterations << " S2Seq_UB ret: " << flush;
    Timer t;

    try
    {
        t.start();

        for(long i = 0; i < iterations; i++)
    	    result = obj -> S2Seq_UB_ret_t();

        t.stop();
    }
    catch(CORBA::SystemException &ex)
    {
        cout << "S2Seq_UB_ret_t():"
	     << " call failed"
    	     << endl
    	     << &ex
    	     << endl;
    	return;
    }

    cout << "\t" << t.timeElapsed() << " seconds (" 
         << (iterations / t.timeElapsed()) << "/sec)" << endl;
}

//----------------------------------------------------------------------
// Function:	InterfaceSeq_in_t
//
// Description:	Client-side function that illustrates how to invoke the
//		Benchmark::SequenceTester::InterfaceSeq_in_t operation.
//----------------------------------------------------------------------

void
CallSequenceTester::InterfaceSeq_in_t(
    Benchmark::SequenceTester_ptr obj,
    long iterations)
{
    Benchmark::SequenceTester::InterfaceSeq i_seq;

    i_seq.length(ub_sequence_length);

    UserTypes::I_ptr myI;
    myI = UserTypes::I::_nil();

    for(CORBA::ULong i = 0; i < ub_sequence_length; i++)
        i_seq[i] = myI;

    cout << iterations << " InterfaceSeq in:  " << flush;
    Timer t;

    try
    {
        t.start();

        for(long i = 0; i < iterations; i++)
    	    obj -> InterfaceSeq_in_t(i_seq);

        t.stop();
    }
    catch(CORBA::SystemException &ex)
    {
        cout << "InterfaceSeq_in_t():"
	     << " call failed"
    	     << endl
    	     << &ex
    	     << endl;
    	return;
    }

    cout << "\t" << t.timeElapsed() << " seconds (" 
         << (iterations / t.timeElapsed()) << "/sec)" << endl;
}

//----------------------------------------------------------------------
// Function:	InterfaceSeq_out_t
//
// Description:	Client-side function that illustrates how to invoke the
//		Benchmark::SequenceTester::InterfaceSeq_out_t operation.
//----------------------------------------------------------------------

void
CallSequenceTester::InterfaceSeq_out_t(
    Benchmark::SequenceTester_ptr obj,
    long iterations)
{
    Benchmark::SequenceTester::InterfaceSeq_var i_seq;

    cout << iterations << " InterfaceSeq out: " << flush;
    Timer t;

    try
    {
        t.start();

        for(long i = 0; i < iterations; i++)
    	    obj -> InterfaceSeq_out_t(i_seq);

        t.stop();
    }
    catch(CORBA::SystemException &ex)
    {
        cout << "InterfaceSeq_out_t():"
	     << " call failed"
    	     << endl
    	     << &ex
    	     << endl;
    	return;
    }

    cout << "\t" << t.timeElapsed() << " seconds (" 
         << (iterations / t.timeElapsed()) << "/sec)" << endl;
}

//----------------------------------------------------------------------
// Function:	InterfaceSeq_inout_t
//
// Description:	Client-side function that illustrates how to invoke the
//		Benchmark::SequenceTester::InterfaceSeq_inout_t operation.
//----------------------------------------------------------------------

void
CallSequenceTester::InterfaceSeq_inout_t(
    Benchmark::SequenceTester_ptr obj,
    long iterations)
{
    Benchmark::SequenceTester::InterfaceSeq i_seq;

    i_seq.length (ub_sequence_length);

    UserTypes::I_ptr myI;
    myI = UserTypes::I::_nil();

    for(CORBA::ULong i = 0; i < ub_sequence_length; i++)
        i_seq[i] = myI;

    cout << iterations << " InterfaceSeq i/o: " << flush;
    Timer t;

    try
    {
        t.start();

        for(long i = 0; i < iterations; i++)
    	    obj -> InterfaceSeq_inout_t(i_seq);

        t.stop();
    }
    catch(CORBA::SystemException &ex)
    {
        cout << "InterfaceSeq_inout_t():"
	     << " call failed"
    	     << endl
    	     << &ex
    	     << endl;
    	return;
    }

    cout << "\t" << t.timeElapsed() << " seconds (" 
         << (iterations / t.timeElapsed()) << "/sec)" << endl;
}

//----------------------------------------------------------------------
// Function:	InterfaceSeq_ret_t
//
// Description:	Client-side function that illustrates how to invoke the
//		Benchmark::SequenceTester::InterfaceSeq_ret_t operation.
//----------------------------------------------------------------------

void
CallSequenceTester::InterfaceSeq_ret_t(
    Benchmark::SequenceTester_ptr obj,
    long iterations)
{
    Benchmark::SequenceTester::InterfaceSeq_var result;

    cout << iterations << " InterfaceSeq ret: " << flush;
    Timer t;

    try
    {
        t.start();

        for(long i = 0; i < iterations; i++)
    	    result = obj -> InterfaceSeq_ret_t();

        t.stop();
    }
    catch(CORBA::SystemException &ex)
    {
        cout << "InterfaceSeq_ret_t():"
	     << " call failed"
    	     << endl
    	     << &ex
    	     << endl;
    	return;
    }

    cout << "\t" << t.timeElapsed() << " seconds (" 
         << (iterations / t.timeElapsed()) << "/sec)" << endl;
}
