// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>

#include <CallUserTypeTester.h>
#include <Timer.h>

#ifdef HAVE_STD_IOSTREAM
using namespace std;
#endif

//----------------------------------------------------------------------
// Function:	u1_in_t
//
// Description:	Client-side function that illustrates how to invoke the
//		Benchmark::UserTypeTester::u1_in_t operation.
//----------------------------------------------------------------------

void
CallUserTypeTester::u1_in_t(
    Benchmark::UserTypeTester_ptr obj,
    long iterations)
{
    UserTypes::u1 u;

    u.s(7);

    //
    // Warm up "ping" to establish connection
    //
    obj -> u1_in_t(u);

    cout << iterations << " Union small in:  " << flush;
    Timer t;

    try
    {
        t.start();

        for(long i = 0; i < iterations; i++)
    	    obj -> u1_in_t(u);

        t.stop();
    }
    catch(CORBA::SystemException &ex)
    {
	cout << "u1_in_t(): call failed"
	     << endl
	     << &ex
	     << endl;
	return;
    }

    cout << "\t" << t.timeElapsed() << " seconds (" 
         << (iterations / t.timeElapsed()) << "/sec)" << endl;
}

//----------------------------------------------------------------------
// Function:	u1_out_t
//
// Description:	Client-side function that illustrates how to invoke the
//		Benchmark::UserTypeTester::u1_out_t operation.
//----------------------------------------------------------------------

void
CallUserTypeTester::u1_out_t(
    Benchmark::UserTypeTester_ptr obj,
    long iterations)
{
    UserTypes::u1 u;

    cout << iterations << " Union small out: " << flush;
    Timer t;

    try
    {
        t.start();

        for(long i = 0; i < iterations; i++)
    	    obj -> u1_out_t(u);

        t.stop();
    }
    catch(CORBA::SystemException &ex)
    {
	cout << "u1_out_t(): call failed"
	     << endl
	     << &ex
	     << endl;
	return;
    }

    cout << "\t" << t.timeElapsed() << " seconds (" 
         << (iterations / t.timeElapsed()) << "/sec)" << endl;
}

//----------------------------------------------------------------------
// Function:	u1_inout_t
//
// Description:	Client-side function that illustrates how to invoke the
//		Benchmark::UserTypeTester::u1_inout_t operation.
//----------------------------------------------------------------------

void
CallUserTypeTester::u1_inout_t(
    Benchmark::UserTypeTester_ptr obj,
    long iterations)
{
    UserTypes::u1 u;

    u.c((CORBA::Char) 'g');
    
    cout << iterations << " Union small i/o: " << flush;
    Timer t;

    try
    {
        t.start();

        for(long i = 0; i < iterations; i++)
    	    obj -> u1_inout_t(u);

        t.stop();
    }
    catch(CORBA::SystemException &ex)
    {
	cout << "u1_inout_t(): call failed"
	     << endl
	     << &ex
	     << endl;
	return;
    }

    cout << "\t" << t.timeElapsed() << " seconds (" 
         << (iterations / t.timeElapsed()) << "/sec)" << endl;
}

//----------------------------------------------------------------------
// Function:	u1_ret_t
//
// Description:	Client-side function that illustrates how to invoke the
//		Benchmark::UserTypeTester::u1_ret_t operation.
//----------------------------------------------------------------------

void
CallUserTypeTester::u1_ret_t(
    Benchmark::UserTypeTester_ptr obj,
    long iterations)
{
    UserTypes::u1 result;

    cout << iterations << " Union small ret: " << flush;
    Timer t;

    try
    {
        t.start();

        for(long i = 0; i < iterations; i++)
    	    result = obj -> u1_ret_t();

        t.stop();
    }
    catch(CORBA::SystemException &ex)
    {
	cout << "u1_ret_t(): call failed"
	     << endl
	     << &ex
	     << endl;
	return;
    }
    
    cout << "\t" << t.timeElapsed() << " seconds (" 
         << (iterations / t.timeElapsed()) << "/sec)" << endl;
}

//----------------------------------------------------------------------
// Function:	u2_in_t
//
// Description:	Client-side function that illustrates how to invoke the
//		Benchmark::UserTypeTester::u2_in_t operation.
//----------------------------------------------------------------------

void
CallUserTypeTester::u2_in_t(
    Benchmark::UserTypeTester_ptr obj,
    long iterations)
{
    UserTypes::u2 u;

    u.ll(443);

    cout << iterations << " Union large in:  " << flush;
    Timer t;

    try
    {
        t.start();

        for(long i = 0; i < iterations; i++)
    	    obj -> u2_in_t(u);

        t.stop();
    }
    catch(CORBA::SystemException &ex)
    {
	cout << "u2_in_t(): call failed"
	     << endl
	     << &ex
	     << endl;
	return;
    }

    cout << "\t" << t.timeElapsed() << " seconds (" 
         << (iterations / t.timeElapsed()) << "/sec)" << endl;
}

//----------------------------------------------------------------------
// Function:	u2_out_t
//
// Description:	Client-side function that illustrates how to invoke the
//		Benchmark::UserTypeTester::u2_out_t operation.
//----------------------------------------------------------------------

void
CallUserTypeTester::u2_out_t(
    Benchmark::UserTypeTester_ptr obj,
    long iterations)
{
    UserTypes::u2_var u;

    cout << iterations << " Union large out: " << flush;
    Timer t;

    try
    {
        t.start();

        for(long i = 0; i < iterations; i++)
    	    obj -> u2_out_t(u);

        t.stop();
    }
    catch(CORBA::SystemException &ex)
    {
	cout << "u2_out_t(): call failed"
	     << endl
	     << &ex
	     << endl;
	return;
    }

    cout << "\t" << t.timeElapsed() << " seconds (" 
         << (iterations / t.timeElapsed()) << "/sec)" << endl;
}

//----------------------------------------------------------------------
// Function:	u2_inout_t
//
// Description:	Client-side function that illustrates how to invoke the
//		Benchmark::UserTypeTester::u2_inout_t operation.
//----------------------------------------------------------------------

void
CallUserTypeTester::u2_inout_t(
    Benchmark::UserTypeTester_ptr obj,
    long iterations)
{
    UserTypes::u2 u;

    u.d((CORBA::Double) 234343.772);

    cout << iterations << " Union large i/o: " << flush;
    Timer t;
    
    try
    {
        t.start();

        for(long i = 0; i < iterations; i++)
    	    obj -> u2_inout_t(u);

        t.stop();
    }
    catch(CORBA::SystemException &ex)
    {
	cout << "u2_inout_t(): call failed"
	     << endl
	     << &ex
	     << endl;
	return;
    }

    cout << "\t" << t.timeElapsed() << " seconds (" 
         << (iterations / t.timeElapsed()) << "/sec)" << endl;
}

//----------------------------------------------------------------------
// Function:	u2_ret_t
//
// Description:	Client-side function that illustrates how to invoke the
//		Benchmark::UserTypeTester::u2_ret_t operation.
//----------------------------------------------------------------------

void
CallUserTypeTester::u2_ret_t(
    Benchmark::UserTypeTester_ptr obj,
    long iterations)
{
    UserTypes::u2_var result;
    
    cout << iterations << " Union large ret: " << flush;
    Timer t;
    
    try
    {
        t.start();

        for(long i = 0; i < iterations; i++)
    	    result = obj -> u2_ret_t();

        t.stop();
    }
    catch(CORBA::SystemException &ex)
    {
	cout << "u2_ret_t(): call failed"
	     << endl
	     << &ex
	     << endl;
	return;
    }

    cout << "\t" << t.timeElapsed() << " seconds (" 
         << (iterations / t.timeElapsed()) << "/sec)" << endl;
}

//----------------------------------------------------------------------
// Function:	s1_in_t
//
// Description:	Client-side function that illustrates how to invoke the
//		Benchmark::UserTypeTester::s1_in_t operation.
//----------------------------------------------------------------------

void
CallUserTypeTester::s1_in_t(
    Benchmark::UserTypeTester_ptr obj,
    long iterations)
{
    UserTypes::s1 s;
    
    s.s = (CORBA::Short) 44;
    
    cout << iterations << " Struct small in:  " << flush;
    Timer t;

    try
    {
        t.start();

        for(long i = 0; i < iterations; i++)
    	    obj -> s1_in_t(s);

        t.stop();
    }
    catch(CORBA::SystemException &ex)
    {
	cout << "s1_in_t(): call failed"
	     << endl
	     << &ex
	     << endl;
	return;
    }

    cout << "\t" << t.timeElapsed() << " seconds (" 
         << (iterations / t.timeElapsed()) << "/sec)" << endl;
}

//----------------------------------------------------------------------
// Function:	s1_out_t
//
// Description:	Client-side function that illustrates how to invoke the
//		Benchmark::UserTypeTester::s1_out_t operation.
//----------------------------------------------------------------------

void
CallUserTypeTester::s1_out_t(
    Benchmark::UserTypeTester_ptr obj,
    long iterations)
{
    UserTypes::s1 s;

    cout << iterations << " Struct small out: " << flush;
    Timer t;

    try
    {
        t.start();

        for(long i = 0; i < iterations; i++)
    	    obj -> s1_out_t(s);

        t.stop();
    }
    catch(CORBA::SystemException &ex)
    {
	cout << "s1_out_t(): call failed"
	     << endl
	     << &ex
	     << endl;
	return;
    }

    cout << "\t" << t.timeElapsed() << " seconds (" 
         << (iterations / t.timeElapsed()) << "/sec)" << endl;
}

//----------------------------------------------------------------------
// Function:	s1_inout_t
//
// Description:	Client-side function that illustrates how to invoke the
//		Benchmark::UserTypeTester::s1_inout_t operation.
//----------------------------------------------------------------------

void
CallUserTypeTester::s1_inout_t(
    Benchmark::UserTypeTester_ptr obj,
    long iterations)
{
    UserTypes::s1 s;

    s.s = (CORBA::Short) 34;

    cout << iterations << " Struct small i/o: " << flush;
    Timer t;

    try
    {
        t.start();

        for(long i = 0; i < iterations; i++)
    	    obj -> s1_inout_t(s);

        t.stop();
    }
    catch(CORBA::SystemException &ex)
    {
	cout << "s1_inout_t(): call failed"
	     << endl
	     << &ex
	     << endl;
	return;
    }

    cout << "\t" << t.timeElapsed() << " seconds (" 
         << (iterations / t.timeElapsed()) << "/sec)" << endl;
}

//----------------------------------------------------------------------
// Function:	s1_ret_t
//
// Description:	Client-side function that illustrates how to invoke the
//		Benchmark::UserTypeTester::s1_ret_t operation.
//----------------------------------------------------------------------

void
CallUserTypeTester::s1_ret_t(
    Benchmark::UserTypeTester_ptr obj,
    long iterations)
{
    UserTypes::s1 result;

    cout << iterations << " Struct small ret: " << flush;
    Timer t;

    try
    {
        t.start();

        for(long i = 0; i < iterations; i++)
    	    result = obj -> s1_ret_t();

        t.stop();
    }
    catch(CORBA::SystemException &ex)
    {
	cout << "s1_ret_t(): call failed"
	     << endl
	     << &ex
	     << endl;
	return;
    }

    cout << "\t" << t.timeElapsed() << " seconds (" 
         << (iterations / t.timeElapsed()) << "/sec)" << endl;
}

//----------------------------------------------------------------------
// Function:	s2_in_t
//
// Description:	Client-side function that illustrates how to invoke the
//		Benchmark::UserTypeTester::s2_in_t operation.
//----------------------------------------------------------------------

void
CallUserTypeTester::s2_in_t(
    Benchmark::UserTypeTester_ptr obj,
    long iterations)
{
    UserTypes::s2 s;

    s.s = (CORBA::Short) 4;
    s.l = (CORBA::Long) 234235;
#ifdef HAVE_VCPLUSPLUS_BUGS
    s.ll = (CORBA::LongLong) 2304020023040L;
#else
    s.ll = (CORBA::LongLong) 2304020023040LL;
#endif
    s.us = (CORBA::UShort) 443;
    s.ul = (CORBA::ULong) 8;
    s.f = (CORBA::Float) 4434.87;
    s.d = (CORBA::Double) 187.7;
    s.c = (CORBA::Char) 'h';
    s.wc = (CORBA::WChar) '(';
    s.st = CORBA::string_dup("Hi, Mom!");
    s.b = (CORBA::Boolean) false;
    s.o = (CORBA::Octet) 7;
    s.a <<= "I'm going in an any...";

    cout << iterations << " Struct large in:  " << flush;
    Timer t;

    try
    {
        t.start();

        for(long i = 0; i < iterations; i++)
    	    obj -> s2_in_t(s);

        t.stop();
    }
    catch(CORBA::SystemException &ex)
    {
	cout << "s2_in_t(): call failed"
	     << endl
	     << &ex
	     << endl;
	return;
    }
    
    cout << "\t" << t.timeElapsed() << " seconds (" 
         << (iterations / t.timeElapsed()) << "/sec)" << endl;
}

//----------------------------------------------------------------------
// Function:	s2_out_t
//
// Description:	Client-side function that illustrates how to invoke the
//		Benchmark::UserTypeTester::s2_out_t operation.
//----------------------------------------------------------------------

void
CallUserTypeTester::s2_out_t(
    Benchmark::UserTypeTester_ptr obj,
    long iterations)
{
    UserTypes::s2_var s;

    cout << iterations << " Struct large out: " << flush;
    Timer t;

    try
    {
        t.start();

        for(long i = 0; i < iterations; i++)
    	    obj -> s2_out_t(s);

        t.stop();
    }
    catch(CORBA::SystemException &ex)
    {
	cout << "s2_out_t(): call failed"
	     << endl
	     << &ex
	     << endl;
	return;
    }

    cout << "\t" << t.timeElapsed() << " seconds (" 
         << (iterations / t.timeElapsed()) << "/sec)" << endl;
}

//----------------------------------------------------------------------
// Function:	s2_inout_t
//
// Description:	Client-side function that illustrates how to invoke the
//		Benchmark::UserTypeTester::s2_inout_t operation.
//----------------------------------------------------------------------

void
CallUserTypeTester::s2_inout_t(
    Benchmark::UserTypeTester_ptr obj,
    long iterations)
{
    UserTypes::s2 s;

    s.s = (CORBA::Short) 234;
    s.l = (CORBA::Long) 723456235;
#ifdef HAVE_VCPLUSPLUS_BUGS
    s.ll = (CORBA::LongLong) 2303454020023040L;
#else
    s.ll = (CORBA::LongLong) 2303454020023040LL;
#endif
    s.us = (CORBA::UShort) 43343;
    s.ul = (CORBA::ULong) 338;
    s.f = (CORBA::Float) 45434.857;
    s.d = (CORBA::Double) 1857.37;
    s.c = (CORBA::Char) 'q';
    s.wc = (CORBA::WChar) 'b';
    s.st = CORBA::string_dup("Lemond");
    s.b = (CORBA::Boolean) false;
    s.o = (CORBA::Octet) 7;
    s.a <<= "Erin go bragh!";

    cout << iterations << " Struct large i/o: " << flush;
    Timer t;

    try
    {
        t.start();

        for(long i = 0; i < iterations; i++)
    	    obj -> s2_inout_t(s);

        t.stop();
    }
    catch(CORBA::SystemException &ex)
    {
	cout << "s2_inout_t(): call failed"
	     << endl
	     << &ex
	     << endl;
	return;
    } 

    cout << "\t" << t.timeElapsed() << " seconds (" 
         << (iterations / t.timeElapsed()) << "/sec)" << endl;
}

//----------------------------------------------------------------------
// Function:	s2_ret_t
//
// Description:	Client-side function that illustrates how to invoke the
//		Benchmark::UserTypeTester::s2_ret_t operation.
//----------------------------------------------------------------------

void
CallUserTypeTester::s2_ret_t(
    Benchmark::UserTypeTester_ptr obj,
    long iterations)
{
    UserTypes::s2_var result;

    cout << iterations << " Struct large ret: " << flush;
    Timer t;

    try
    {
        t.start();

        for(long i = 0; i < iterations; i++)
    	    result = obj -> s2_ret_t();

        t.stop();
    }
    catch(CORBA::SystemException &ex)
    {
	cout << "s2_ret_t(): call failed"
	     << endl
	     << &ex
	     << endl;
	return;
    }
    
    cout << "\t" << t.timeElapsed() << " seconds (" 
         << (iterations / t.timeElapsed()) << "/sec)" << endl;
}

//----------------------------------------------------------------------
// Function:	interface_in_t
//
// Description:	Client-side function that illustrates how to invoke the
//		Benchmark::UserTypeTester::interface_in_t operation.
//----------------------------------------------------------------------

void
CallUserTypeTester::interface_in_t(
    Benchmark::UserTypeTester_ptr obj,
    long iterations)
{
    UserTypes::I_var interf;
    
    interf = UserTypes::I::_nil();

    cout << iterations << " Interfaces in:  " << flush;
    Timer t;

    try
    {
        t.start();

        for(long i = 0; i < iterations; i++)
    	    obj -> interface_in_t(interf);

        t.stop();
    }
    catch(CORBA::SystemException &ex)
    {
	cout << "interface_in_t(): call failed"
	     << endl
	     << &ex
	     << endl;
	return;
    }

    cout << "\t" << t.timeElapsed() << " seconds (" 
         << (iterations / t.timeElapsed()) << "/sec)" << endl;
}

//----------------------------------------------------------------------
// Function:	interface_out_t
//
// Description:	Client-side function that illustrates how to invoke the
//		Benchmark::UserTypeTester::interface_out_t operation.
//----------------------------------------------------------------------

void
CallUserTypeTester::interface_out_t(
    Benchmark::UserTypeTester_ptr obj,
    long iterations)
{
    UserTypes::I_var interf;

    cout << iterations << " Interfaces out: " << flush;
    Timer t;

    try
    {
        t.start();

        for(long i = 0; i < iterations; i++)
    	    obj -> interface_out_t(interf);

        t.stop();
    }
    catch(CORBA::SystemException &ex)
    {
	cout << "interface_out_t(): call failed"
	     << endl
	     << &ex
	     << endl;
	return;
    }

    cout << "\t" << t.timeElapsed() << " seconds (" 
         << (iterations / t.timeElapsed()) << "/sec)" << endl;
}

//----------------------------------------------------------------------
// Function:	interface_inout_t
//
// Description:	Client-side function that illustrates how to invoke the
//		Benchmark::UserTypeTester::interface_inout_t operation.
//----------------------------------------------------------------------

void
CallUserTypeTester::interface_inout_t(
    Benchmark::UserTypeTester_ptr obj,
    long iterations)
{
    UserTypes::I_var interf;

    interf = UserTypes::I::_nil();

    cout << iterations << " Interfaces i/o: " << flush;
    Timer t;

    try
    {
        t.start();

        for(long i = 0; i < iterations; i++)
    	    obj -> interface_inout_t(interf.inout());

        t.stop();
    }
    catch(CORBA::SystemException &ex)
    {
	cout << "interface_inout_t(): call failed"
	     << endl
	     << &ex
	     << endl;
	return;
    }

    cout << "\t" << t.timeElapsed() << " seconds (" 
         << (iterations / t.timeElapsed()) << "/sec)" << endl;
}

//----------------------------------------------------------------------
// Function:	interface_ret_t
//
// Description:	Client-side function that illustrates how to invoke the
//		Benchmark::UserTypeTester::interface_ret_t operation.
//----------------------------------------------------------------------

void
CallUserTypeTester::interface_ret_t(
    Benchmark::UserTypeTester_ptr obj,
    long iterations)
{
    UserTypes::I_var result;

    cout << iterations << " Interfaces ret: " << flush;
    Timer t;

    try
    {
        t.start();

        for(long i = 0; i < iterations; i++)
    	    result = obj -> interface_ret_t();

        t.stop();
    }
    catch(CORBA::SystemException &ex)
    {
	cout << "interface_ret_t(): call failed"
	     << endl
	     << &ex
	     << endl;
	return;
    }

    cout << "\t" << t.timeElapsed() << " seconds (" 
         << (iterations / t.timeElapsed()) << "/sec)" << endl;
}
