// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>

#include <SequenceTester_impl.h>
#include <Constants.h>

//
// Servant which implements the Benchmark::SequenceTester interface.
//

//
// SequenceTester_impl constructor
//
//
SequenceTester_impl::SequenceTester_impl(
    PortableServer::POA_ptr poa
) :
    m_poa(PortableServer::POA::_duplicate(poa))
{
    CORBA::ULong i;

    //
    // initialize user-defined types
    //
    UserTypes::u2 u;

    u.d((CORBA::Double) 234343.772);

    UserTypes::s2 s;

    s.s = (CORBA::Short) 234;
    s.l = (CORBA::Long) 723456235;
#ifdef HAVE_VCPLUSPLUS_BUGS
    s.ll = (CORBA::LongLong) 2303454020023040L;
#else
    s.ll = (CORBA::LongLong) 2303454020023040LL;
#endif
    s.us = (CORBA::UShort) 43343;
    s.ul = (CORBA::ULong) 338;
    s.f = (CORBA::Float) 45434.857;
    s.d = (CORBA::Double) 1857.37;
    s.c = (CORBA::Char) 'q';
    s.wc = (CORBA::WChar) 'b';
    s.st = CORBA::string_dup("Lemond");
    s.ws = CORBA::wstring_dup(L"Steve Rex Cycles");
    s.b = (CORBA::Boolean) false;
    s.o = (CORBA::Octet) 7;
    s.a <<= "No wchars here, buddy...";

    //
    // allocate space for the buffers
    //
    m_shortseq_buf = new CORBA::Short[ub_sequence_length];
    m_floatseq_buf = new CORBA::Float[64*1024];
    m_stringseq_buf =
        Benchmark::SequenceTester::StringSeq_UB::allocbuf(ub_sequence_length);
    m_octetseq_buf = new CORBA::Octet[1024*1024];
    m_anyseq_buf = new CORBA::Any[ub_sequence_length];
    m_u2seq_buf = new UserTypes::u2[ub_sequence_length];
    m_s2seq_buf = new UserTypes::s2[ub_sequence_length];
    m_Interfaceseq_buf = new UserTypes::I_ptr[ub_sequence_length];

    //
    // initialize bufs of length ub_sequence_length;
    //
    for(i = 0; i < ub_sequence_length; i++) 
    {
        m_shortseq_buf[i] = (CORBA::Short) i;
        m_stringseq_buf[i] = CORBA::string_dup("Hello, world!");
        m_anyseq_buf[i] <<= (CORBA::Long) i;
        m_u2seq_buf[i] = u;
        m_s2seq_buf[i] = s;
        m_Interfaceseq_buf[i] = UserTypes::I::_nil();
    }

    //
    // initialize floatseq_buf
    //
    for(i = 0; i < 65536; i++)
        m_floatseq_buf[i] = (CORBA::Float) i;

    //
    // initialize m_OctetSeq_1M
    //
    for(i = 0; i < 1048576; i++)
        m_octetseq_buf[i] = (CORBA::Octet) i;
}


//
// ~SequenceTester_impl destructor.
//
SequenceTester_impl::~SequenceTester_impl()
{
    //
    // clean up buffers
    //
    delete[] m_shortseq_buf;
    delete[] m_floatseq_buf;
    delete[] m_stringseq_buf;
    delete[] m_octetseq_buf;
    delete[] m_anyseq_buf;
    delete[] m_u2seq_buf;
    delete[] m_s2seq_buf;
    delete[] m_Interfaceseq_buf;
}

PortableServer::POA_ptr
SequenceTester_impl::_default_POA()
{
    return PortableServer::POA::_duplicate(m_poa);
}

void
SequenceTester_impl::ShortSeq_UB_in_t(
    const Benchmark::SequenceTester::ShortSeq_UB&)
    throw(CORBA::SystemException)
{
}

void
SequenceTester_impl::ShortSeq_UB_out_t(
    Benchmark::SequenceTester::ShortSeq_UB_out s_seq)
    throw(CORBA::SystemException)
{
    s_seq = new Benchmark::SequenceTester::ShortSeq_UB (ub_sequence_length,
                                                        ub_sequence_length,
                                                        m_shortseq_buf,
                                                        0
                                                        );
}

void
SequenceTester_impl::ShortSeq_UB_inout_t(
    Benchmark::SequenceTester::ShortSeq_UB& s_seq)
    throw(CORBA::SystemException)
{
    s_seq.replace(ub_sequence_length,
                  ub_sequence_length,
                  m_shortseq_buf,
                  0
                  );
}

Benchmark::SequenceTester::ShortSeq_UB*
SequenceTester_impl::ShortSeq_UB_ret_t()
    throw(CORBA::SystemException)
{
    Benchmark::SequenceTester::ShortSeq_UB* result;

    result = new Benchmark::SequenceTester::ShortSeq_UB (ub_sequence_length,
                                                          ub_sequence_length,
                                                          m_shortseq_buf,
                                                          0);
    return result;
}

void
SequenceTester_impl::FloatSeq_64K_in_t(
    const Benchmark::SequenceTester::FloatSeq_64K&)
    throw(CORBA::SystemException)
{
}

void
SequenceTester_impl::FloatSeq_64K_out_t(
    Benchmark::SequenceTester::FloatSeq_64K_out f_seq)
    throw(CORBA::SystemException)
{
    f_seq = new Benchmark::SequenceTester::FloatSeq_64K (65536,
                                                         m_floatseq_buf,
                                                         0
                                                         );
}

void
SequenceTester_impl::FloatSeq_64K_inout_t(
    Benchmark::SequenceTester::FloatSeq_64K& f_seq)
    throw(CORBA::SystemException)
{
    f_seq.replace (65536,
                   m_floatseq_buf,
                   0
                   );
}

Benchmark::SequenceTester::FloatSeq_64K*
SequenceTester_impl::FloatSeq_64K_ret_t()
    throw(CORBA::SystemException)
{
    Benchmark::SequenceTester::FloatSeq_64K* result;

    result = new Benchmark::SequenceTester::FloatSeq_64K (65536,
                                                           m_floatseq_buf,
                                                           0
                                                           );
    return result;
}

void
SequenceTester_impl::StringSeq_UB_in_t(
    const Benchmark::SequenceTester::StringSeq_UB&)
    throw(CORBA::SystemException)
{
}

void
SequenceTester_impl::StringSeq_UB_out_t(
	Benchmark::SequenceTester::StringSeq_UB_out s_seq)
    throw(CORBA::SystemException)
{
    s_seq = new Benchmark::SequenceTester::StringSeq_UB (ub_sequence_length,
                                                         ub_sequence_length,
                                                         m_stringseq_buf,
                                                         0
                                                         );
}

void
SequenceTester_impl::StringSeq_UB_inout_t(
    Benchmark::SequenceTester::StringSeq_UB& s_seq)
    throw(CORBA::SystemException)
{
    s_seq.replace (ub_sequence_length,
                   ub_sequence_length,
                   m_stringseq_buf,
                   0
                   );
}

Benchmark::SequenceTester::StringSeq_UB*
SequenceTester_impl::StringSeq_UB_ret_t()
    throw(CORBA::SystemException)
{
    Benchmark::SequenceTester::StringSeq_UB* result;

    result = new Benchmark::SequenceTester::StringSeq_UB (ub_sequence_length,
                                                           ub_sequence_length,
                                                           m_stringseq_buf,
                                                           0
                                                           );
    return result;
}

void
SequenceTester_impl::OctetSeq_1M_in_t(
    const Benchmark::SequenceTester::OctetSeq_1M&)
    throw(CORBA::SystemException)
{
}

void
SequenceTester_impl::OctetSeq_1M_out_t(
    Benchmark::SequenceTester::OctetSeq_1M_out o_seq)
    throw(CORBA::SystemException)
{
    o_seq = new Benchmark::SequenceTester::OctetSeq_1M (1024*1024,
                                                        m_octetseq_buf,
                                                        0
                                                        );
}

void
SequenceTester_impl::OctetSeq_1M_inout_t(
    Benchmark::SequenceTester::OctetSeq_1M& o_seq)
    throw(CORBA::SystemException)
{
    o_seq.replace (1024*1024,
                   m_octetseq_buf,
                   0
                   );
}

Benchmark::SequenceTester::OctetSeq_1M*
SequenceTester_impl::OctetSeq_1M_ret_t()
    throw(CORBA::SystemException)
{
    Benchmark::SequenceTester::OctetSeq_1M * result;

    result = new Benchmark::SequenceTester::OctetSeq_1M (1024*1024,
                                                          m_octetseq_buf,
                                                          0
                                                          );
    return result;
}

void
SequenceTester_impl::AnySeq_UB_in_t(
    const Benchmark::SequenceTester::AnySeq_UB&)
    throw(CORBA::SystemException)
{
}

void
SequenceTester_impl::AnySeq_UB_out_t(
    Benchmark::SequenceTester::AnySeq_UB_out a_seq)
    throw(CORBA::SystemException)
{
    a_seq = new Benchmark::SequenceTester::AnySeq_UB (ub_sequence_length,
                                                      ub_sequence_length,
                                                      m_anyseq_buf,
                                                      0
                                                      );
}

void
SequenceTester_impl::AnySeq_UB_inout_t(
    Benchmark::SequenceTester::AnySeq_UB& a_seq)
    throw(CORBA::SystemException)
{
    a_seq.replace (ub_sequence_length,
                   ub_sequence_length,
                   m_anyseq_buf,
                   0
                   );
}

Benchmark::SequenceTester::AnySeq_UB*
SequenceTester_impl::AnySeq_UB_ret_t()
    throw(CORBA::SystemException)
{
    Benchmark::SequenceTester::AnySeq_UB* result;

    result = new Benchmark::SequenceTester::AnySeq_UB(ub_sequence_length,
                                                       ub_sequence_length,
                                                       m_anyseq_buf,
                                                       0
                                                       );
    return result;
}

void
SequenceTester_impl::U2Seq_UB_in_t(
    const Benchmark::SequenceTester::U2Seq_UB&)
    throw(CORBA::SystemException)
{
}

void
SequenceTester_impl::U2Seq_UB_out_t(
    Benchmark::SequenceTester::U2Seq_UB_out u2_seq)
    throw(CORBA::SystemException)
{
    u2_seq = new Benchmark::SequenceTester::U2Seq_UB(ub_sequence_length,
                                                     ub_sequence_length,
                                                     m_u2seq_buf,
                                                     0
                                                     );
}

void
SequenceTester_impl::U2Seq_UB_inout_t(
    Benchmark::SequenceTester::U2Seq_UB& u2_seq)
    throw(CORBA::SystemException)
{
    u2_seq.replace(ub_sequence_length,
                   ub_sequence_length,
                   m_u2seq_buf,
                   0
                   );
}

Benchmark::SequenceTester::U2Seq_UB*
SequenceTester_impl::U2Seq_UB_ret_t()
    throw(CORBA::SystemException)
{
    Benchmark::SequenceTester::U2Seq_UB* result;

    result = new Benchmark::SequenceTester::U2Seq_UB(ub_sequence_length,
                                                      ub_sequence_length,
                                                      m_u2seq_buf,
                                                      0
                                                      );
    return result;
}

void
SequenceTester_impl::S2Seq_UB_in_t(
    const Benchmark::SequenceTester::S2Seq_UB&)
    throw(CORBA::SystemException)
{
}

void
SequenceTester_impl::S2Seq_UB_out_t(
    Benchmark::SequenceTester::S2Seq_UB_out s2_seq)
    throw(CORBA::SystemException)
{
    s2_seq = new Benchmark::SequenceTester::S2Seq_UB(ub_sequence_length,
                                                     ub_sequence_length,
                                                     m_s2seq_buf,
                                                     0
                                                     );
}

void
SequenceTester_impl::S2Seq_UB_inout_t(
    Benchmark::SequenceTester::S2Seq_UB& s2_seq)
    throw(CORBA::SystemException)
{
    s2_seq.replace(ub_sequence_length,
               ub_sequence_length,
               m_s2seq_buf,
               0
               );
}

Benchmark::SequenceTester::S2Seq_UB*
SequenceTester_impl::S2Seq_UB_ret_t()
    throw(CORBA::SystemException)
{
    Benchmark::SequenceTester::S2Seq_UB* result;

    result = new Benchmark::SequenceTester::S2Seq_UB(ub_sequence_length,
                                                      ub_sequence_length,
                                                      m_s2seq_buf,
                                                      0
                                                      );
    return result;
}

void
SequenceTester_impl::InterfaceSeq_in_t(
    const Benchmark::SequenceTester::InterfaceSeq&)
    throw(CORBA::SystemException)
{
}

void
SequenceTester_impl::InterfaceSeq_out_t(
    Benchmark::SequenceTester::InterfaceSeq_out i_seq)
    throw(CORBA::SystemException)
{
    i_seq = new Benchmark::SequenceTester::InterfaceSeq(ub_sequence_length,
                                                        ub_sequence_length,
                                                        m_Interfaceseq_buf,
                                                        0
                                                        );
}

void
SequenceTester_impl::InterfaceSeq_inout_t(
    Benchmark::SequenceTester::InterfaceSeq& i_seq)
    throw(CORBA::SystemException)
{
    i_seq.replace(ub_sequence_length,
                  ub_sequence_length,
                  m_Interfaceseq_buf,
                  0
                  );
}

Benchmark::SequenceTester::InterfaceSeq*
SequenceTester_impl::InterfaceSeq_ret_t()
    throw(CORBA::SystemException)
{
    Benchmark::SequenceTester::InterfaceSeq* result;

    result = new Benchmark::SequenceTester::InterfaceSeq(ub_sequence_length,
                                                          ub_sequence_length,
                                                          m_Interfaceseq_buf,
                                                          0
                                                          );
    return result;
}
