// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>

#include <BasicTester_impl.h>
#include <UserTypeTester_impl.h>
#include <SequenceTester_impl.h>

#include <stdlib.h>

#ifdef HAVE_FSTREAM
#   include <fstream>
#else
#   include <fstream.h>
#endif

#ifdef HAVE_STD_IOSTREAM
using namespace std;
#endif

//
// write_reference() -- export object reference to file.
//
void
write_reference(const char* refFile, const char* ref)
{
    ofstream os(refFile);
    os << ref;
    if(!os.good())
    {
        cerr << "Failed to write to " << refFile << endl;
    }
}


int
run(CORBA::ORB_ptr orb, int /* argc */, char*[])
{
    //
    // Resolve Root POA
    //
    CORBA::Object_var poaObj = orb -> resolve_initial_references("RootPOA");
    PortableServer::POA_var rootPOA = PortableServer::POA::_narrow(poaObj);

    //
    // Get a reference to the POA manager
    //
    PortableServer::POAManager_var manager = rootPOA -> the_POAManager();

    //
    // Create BasicTester implementation object
    //
    BasicTester_impl* basicTesterImpl = new BasicTester_impl(rootPOA);
    PortableServer::ServantBase_var servant1 = basicTesterImpl;
    Benchmark::BasicTester_var basicTester = basicTesterImpl -> _this();
    CORBA::String_var ref = orb -> object_to_string(basicTester);
    write_reference("Benchmark_BasicTester.ref", ref);

    //
    // Create UserTypeTester implementation object
    //
    UserTypeTester_impl* userTypeTesterImpl = new UserTypeTester_impl(rootPOA);
    PortableServer::ServantBase_var servant2 = userTypeTesterImpl;
    Benchmark::UserTypeTester_var userTypeTester =
        userTypeTesterImpl -> _this();
    ref = orb -> object_to_string(userTypeTester);
    write_reference("Benchmark_UserTypeTester.ref", ref);

    //
    // Create SequenceTester implementation object
    //
    SequenceTester_impl* sequenceTesterImpl = new SequenceTester_impl(rootPOA);
    PortableServer::ServantBase_var servant3 = sequenceTesterImpl;
    Benchmark::SequenceTester_var sequenceTester =
        sequenceTesterImpl -> _this();
    ref = orb -> object_to_string(sequenceTester);
    write_reference("Benchmark_SequenceTester.ref", ref);

    //
    // Activate the POA Manager and let the ORB process requests.
    //
    manager -> activate();
    cout << "Waiting for requests..." << endl;
    orb -> run();

    return EXIT_SUCCESS;
}

int
main(int argc, char* argv[], char*[])
{
    int status = EXIT_SUCCESS;
    CORBA::ORB_var orb;

    try
    {
        orb = CORBA::ORB_init(argc, argv, "Benchmark-Server");
        status = run(orb, argc, argv);
    }
    catch(const CORBA::Exception& ex)
    {
        cerr << ex << endl;
        status = EXIT_FAILURE;
    }

    if(!CORBA::is_nil(orb))
    {
        try
        {
            orb -> destroy();
        }
        catch(const CORBA::Exception& ex)
        {
            cerr << ex << endl;
            status = EXIT_FAILURE;
        }
    }

    return status;
}
