#include <OB/CORBA.h>
#include <Types.hh>

#include <string.h>

#include <Hello.hh>

void show_TC(CORBA::TypeCode_ptr);

using namespace CORBA;

int
main(int, char**)
{
    Any any;
    ULong i;
    TypeCode_ptr tc;

    {
	// interface

	show_TC(_tc_Hello);
    }

    {
	// short

	Short v = -32768;

	any <<= v;
	tc = any.type();
	show_TC(tc);
    }

    {
	// unsigned short

	UShort v = 65535U;

	any <<= v;
	tc = any.type();
	show_TC(tc);
    }
    
    {
	// long

	Long v = -2147483647 - 1;

	any <<= v;
	tc = any.type();
	show_TC(tc);
    }
    
    {
	// unsigned long

	ULong v = 2147483647;

	any <<= v;
	tc = any.type();
	show_TC(tc);
    }

    {
	// long long

#ifndef HAVE_VCPLUSPLUS_BUGS
	LongLong v = -9223372036854775807LL - 1;
#else
	LongLong v = -9223372036854775807L - 1;
#endif

	any <<= v;
	tc = any.type();
	show_TC(tc);
    }
    
    {
	// unsigned long long

#ifndef HAVE_VCPLUSPLUS_BUGS
	ULongLong v = 9223372036854775807LL;
#else
	ULongLong v = 9223372036854775807L;
#endif

	any <<= v;
	tc = any.type();
	show_TC(tc);
    }

    {
	// float

	Float v = (Float)1.23456789;

	any <<= v;
	tc = any.type();
	show_TC(tc);
    }
    
    {
	// double

	Double v = 1E200;

	any <<= v;
	tc = any.type();
	show_TC(tc);
    }

#if SIZEOF_LONG_DOUBLE >= 12
    {
	// long double

	LongDouble v = 1E200;

	any <<= v;
	tc = any.type();
	show_TC(tc);
    }
#endif

    {
	// octet

	Octet v = 0xff;

	any <<= Any::from_octet(v);
	tc = any.type();
	show_TC(tc);
    }

    {
	// char

	Char v = 'x';

	any <<= Any::from_char(v);
	tc = any.type();
	show_TC(tc);
    }

    {
	// wide char

	WChar v = L'x';

	any <<= Any::from_wchar(v);
	tc = any.type();
	show_TC(tc);
    }

    {
	// boolean

	Boolean v = true;

	any <<= Any::from_boolean(v);
	tc = any.type();
	show_TC(tc);
    }

    {
	// string

	const char* v = "Hello!";

	Any any;
	any <<= v;
	tc = any.type();
	show_TC(tc);
    }

    {
	// wide string

	const wchar_t* v = L"Hello!";

	Any any;
	any <<= v;
	tc = any.type();
	show_TC(tc);
    }

    {
	// fixed

	TestFixed v1 = "-123456789";
	TestFixed v1Save = v1;

	any <<= Any::from_fixed(v1, 24, 0);
	tc = any.type();
	show_TC(tc);
    }

    {
	// structure

	TestStruct1 v;
	v.s = -32768;
	v.l = -2147483647 - 1;
	v.d = 1E200;
	v.b = true;
	v.c = 'x';
	v.o = 0xff;
	v.str = (const char*)"abc";

	any <<= v;
	tc = any.type();
	show_TC(tc);

	Any any2(any);

	// nested structure

	TestStruct2* v2p = new TestStruct2;
	v2p -> s = v;
	v2p -> a = any2;
	v2p -> da[0][0][0] = 1.23;
	v2p -> da[2][3][4] = -1.11;
	v2p -> sa[50] = string_dup("Hi!");

	any <<= v2p;
	tc = any.type();
	show_TC(tc);
    }

    {
	// enum

	TestEnum v = B;
	any <<= v;
	tc = any.type();
	show_TC(tc);
    }


    {
	// typedef

	String40Seq v;
	v.length(40);

	for(i = 0 ; i < 40 ; i++)
	    v[i] = string_dup("abc");

	any <<= v;
	tc = any.type();
	show_TC(tc);
    }

    {
	// typecode 

	TypeCode_ptr tc;

	any <<= _tc_TestStruct1;
	tc = any.type();
	show_TC(tc);
    }

    {
	// union 

        TestUnion u;
        u.l(123);
    
        u.tc(_tc_TestUnion);
        tc = u.tc();
        show_TC(tc);
    }
}
