#include <OB/CORBA.h>

#if defined(HAVE_STD_IOSTREAM)
using namespace std;
#endif

#ifdef HAVE_IOMANIP
#    include <iomanip>
#else
#    include <iomanip.h>
#endif

#include "time.hh"

//-----------------------------------------------------------------------------

int
run(CORBA::ORB_ptr orb, int argc, char* argv[])
{
    // Check arguments
    if (argc != 2) {
        cerr << "Usage: client IOR_string" << endl;
        throw 0;
    }

    // Destringify argv[1]
    CORBA::Object_var obj = orb->string_to_object(argv[1]);
    if (CORBA::is_nil(obj)) {
        cerr << "Nil Time reference" << endl;
        throw 0;
    }

    // Narrow
    Time_var tm = Time::_narrow(obj);
    if (CORBA::is_nil(tm)) {
        cerr << "Argument is not a Time reference" << endl;
        throw 0;
    }

    // Get time
    TimeOfDay tod = tm->get_gmt();
    cout << "Time in Greenwich is "
         << setw(2) << setfill('0') << tod.hour << ":"
         << setw(2) << setfill('0') << tod.minute << ":"
         << setw(2) << setfill('0') << tod.second << endl;

    return 0;
}

int
main(int argc, char* argv[], char*[])
{
    int status = 0;
    CORBA::ORB_var orb;

    try
    {
	orb = CORBA::ORB_init(argc, argv);
	status = run(orb, argc, argv);
    }
    catch(const CORBA::Exception& ex)
    {
	cerr << ex << endl;
	status = 1;
    }
    catch(...)
    {
        status = 1;
    }

    if(!CORBA::is_nil(orb))
    {
	try
	{
	    orb -> destroy();
	}
	catch(const CORBA::Exception& ex)
	{
	    cerr << ex << endl;
	    status = 1;
	}
    }
    
    return status;
}
