// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>

#include <Chain.h>

#include <stdlib.h>

#ifdef HAVE_STD_IOSTREAM
using namespace std;
#endif

int
run(CORBA::ORB_ptr orb, int argc, char* argv[])
{
    if(argc < 4)
    {
        cerr << argv[0] << ": master msgSize repetitions" << endl;
        return EXIT_FAILURE;
    }

    //
    // Get "Link" object
    //
    CORBA::String_var ior = CORBA::string_dup("relfile:/");
    ior += argv[1];
    CORBA::Object_var obj = orb -> string_to_object(ior);
    if(CORBA::is_nil(obj))
    {
        cerr << argv[0] << ": cannot read IOR from " << argv[1] << endl;
        return EXIT_FAILURE;
    }
	
    Link_var link = Link::_narrow(obj);
    assert(!CORBA::is_nil(link));

    long msgSize = atol(argv[2]);
    cout << "msgSize is " << msgSize << endl;

    long reps = atol(argv[3]);
    cout << "repetitions are " << reps << endl;
	
    //
    // Close the chain.
    //
    link -> close_loop();

    //
    // Send msgSize messages reps times.
    //
    link -> send(reps, msgSize);
    
    return EXIT_SUCCESS;
}

int
main(int argc, char* argv[], char*[])
{
    int status = EXIT_SUCCESS;
    CORBA::ORB_var orb;

    try
    {
	orb = CORBA::ORB_init(argc, argv, "Chain-Client");
	status = run(orb, argc, argv);
    }
    catch(const CORBA::Exception& ex)
    {
	cerr << ex << endl;
	status = EXIT_FAILURE;
    }

    if(!CORBA::is_nil(orb))
    {
	try
	{
	    orb -> destroy();
	}
	catch(const CORBA::Exception& ex)
	{
	    cerr << ex << endl;
	    status = EXIT_FAILURE;
	}
    }
    
    return status;
}
