// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#ifndef RECEIVER_BASE_IMPL_H
#define RECEIVER_BASE_IMPL_H

#include <Broadcaster.h>
#include <Receiver_skel.h>


namespace chat
{

class ReceiverBase_impl : public POA_chat::Receiver
{
protected:

    //
    // The ORB and POA
    //
    CORBA::ORB_var orb_;
    PortableServer::POA_var poa_;

    //
    // My ID, host and nickname
    //
    CORBA::String_var id_;
    CORBA::String_var host_;
    CORBA::String_var nick_;

    //
    // The server object references
    //
    BroadcasterAdmin_var admin_;
    Broadcaster_var      broadcaster_;

    //
    // Handle common errors
    //
    void subscribeAgain(const CORBA::SystemException&);
    void unknownNick(const char*);
    void nickExists(const char*);

    //
    // Subscribe with the server
    //
    virtual void subscribe();

public:

    ReceiverBase_impl(CORBA::ORB_ptr orb, 
		      PortableServer::POA_ptr poa,
		      BroadcasterAdmin_ptr admin );

    PortableServer::POA_ptr _default_POA();

    //
    // Parse input
    //
    virtual void parse(const char*);

    //
    // Execute command
    //
    virtual void execute(const char*);

    //
    // Execute specific receiver commands
    //
    virtual void cmdAbout();
    virtual void cmdHelp();
    virtual void cmdQuit();
    virtual void cmdMsg(const char*, const char*);
    virtual void cmdNick(const char*);
    virtual void cmdWho();
    virtual void cmdWhois(const char*);

    //
    // Print `about' and `help' text
    //
    virtual void about();
    virtual void help();

    //
    // Quit program
    //
    virtual void quit();

    //
    // Operations from the .idl file
    //
    virtual void print(const char*)
        throw(CORBA::SystemException);

}; // class ReceiverBase_impl 


}; // namespace chat

#endif
