// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#ifndef RECEIVER_IMPL_H
#define RECEIVER_IMPL_H

#include <OB/EventHandlerBase.h>
#include <OB/Reactor.h>

#include <Broadcaster.h>
#include <ReceiverBase_impl.h>

#if defined(__FreeBSD__) || defined(__APPLE__)
# include <termios.h>
#else
# include <termio.h>
#endif

namespace chat 
{

//
// The Receiver_impl class
//
class Receiver_impl : public ReceiverBase_impl,
		      public PortableServer::RefCountServantBase
{
    //
    // Our event handler
    //
    OB::EventHandlerBase_var handler_;

    //
    // Input string
    //
    CORBA::String_var input_;

public:

    Receiver_impl(CORBA::ORB_ptr orb, 
                  PortableServer::POA_ptr poa,
		  BroadcasterAdmin_ptr);

    ~Receiver_impl();

    //
    // Event handling
    //
    virtual OB::Mask handleEvent(OB::Mask);

    //
    // Operations from the .idl file
    //
    virtual void print(const char*)
        throw(CORBA::SystemException);

}; // class Receiver_impl


//
// A separate EventHandler class is needed, because EventHandlerBase
// is reference counted
//
class EventHandler : public OB::EventHandlerBase
{
    //
    // The associated Receiver_impl
    //
    Receiver_impl* receiver_;

public:

    EventHandler(Receiver_impl* receiver) : receiver_(receiver) { }

    //
    // Destroy this event handler
    //
    void destroy() { receiver_ = 0; };

    //
    // Event handling
    //
    virtual OB::Mask handleEvent(OB::Mask mask)
    {
	if(receiver_)
	{
	    return receiver_ -> handleEvent(mask);
	}
	else
	{
	    return 0;
	}
    }

}; // class EventHandler

#endif

} // namespace chat
