// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>
#include <OB/Properties.h>
#include <OB/Logger.h>
#include <OB/Reactor.h>
#include <OB/X11.h>

#include <Xm/Protocols.h>

#include <Receiver_impl.h>

#include <stdlib.h>

#ifdef HAVE_STD_IOSTREAM
using namespace std;
#endif

using namespace chat;

int
run(OBCORBA::ORB_ptr orb, const char* host, const char* port,
    Widget topLevel, int /* argc */, char*[])
{
    //
    // Resolve Root POA
    //
    CORBA::Object_var poaObj = orb -> resolve_initial_references("RootPOA");
    PortableServer::POA_var rootPoa = PortableServer::POA::_narrow(poaObj);
    
    PortableServer::POAManager_var poa_manager = rootPoa -> the_POAManager();
    
    //
    // Get the broadcaster object
    //
    CORBA::String_var str = CORBA::string_dup("corbaloc::");
    str += host;
    str += ":";
    str += port;
    str += "/broadcasterAdmin";
    CORBA::Object_var obj = orb -> string_to_object(str);
    assert(!CORBA::is_nil(obj));
    chat::BroadcasterAdmin_var server = chat::BroadcasterAdmin::_narrow(obj);
    assert(!CORBA::is_nil(server));
    
    //
    // Activate and initialize the POA
    //
    poa_manager -> activate();
    
    //
    // Create and activate the receiver object
    //
    Receiver_impl* receiverImpl = 
	new Receiver_impl(topLevel, orb, rootPoa, server);
    PortableServer::ServantBase_var servant = receiverImpl;
    Receiver_var client = receiverImpl -> _this();
    
    //
    // Run implementation
    //
    orb -> run();
    
    return EXIT_SUCCESS;
}

int
main(int argc, char* argv[], char*[])
{
    //
    // Get host and port
    //
    char host[1024];
    char port[256];
    cout << "Host name? " << flush;
    cin >> host;
    cout << "Port number? " << flush;
    cin >> port;
    
    char c;
    cin.read(&c, 1);
    
    //
    // Initialize application
    //
    static String fallbackResources[] =
    {
	(String)"*sgiMode: true",   // This is only for SGI, but it doesn't
	(String)"*useSchemes: all", // hurt to have it also on non-SGI computer
	(String)"*fileMenuButton.labelString: File",
        (String)"*quitButton.labelString: Quit",
	(String)"*helpMenuButton.labelString: Help",
	(String)"*aboutButton.labelString: About X11-chat...",
	(String)"*commandButton.labelString: Commands...",
        (String)"*textOutput.height: 400",
        (String)"*textOutput.width: 600",
	0
    };
    
    XtAppContext appContext;
    Widget topLevel = XtAppInitialize(&appContext,
				      "X11-Chat",
				      0,
				      0,
				      &argc,
				      argv,
				      fallbackResources,
				      0,
				      0);
    
    XtVaSetValues(topLevel,
		  XmNdeleteResponse, XmDO_NOTHING,
		  0);

    int status = EXIT_SUCCESS;
    OBCORBA::ORB_var orb;

    try
    {
        OB::Properties_var properties = OB::Properties::getDefaultProperties();
	properties -> setProperty("ooc.orb.conc_model", "reactive");
	properties -> setProperty("ooc.orb.oa.conc_model", "reactive");
	properties -> setProperty("ooc.orb.id", "Chat-X11-Client");
	
	OB::Reactor_var reactor = OB::GetX11Reactor(appContext);
	
	orb = OBCORBA::ORB_init(argc, argv, properties,
				OB::Logger::_nil(), reactor);

	status = run(orb, host, port, topLevel, argc, argv);
    }
    catch(const CORBA::Exception& ex)
    {
	cerr << ex << endl;
	status = EXIT_FAILURE;
    }

    //
    // Shutdown
    //
    if(!CORBA::is_nil(orb))
    {
	try
	{
	    orb -> destroy();
	}
	catch(const CORBA::Exception& ex)
	{
	    cerr << ex << endl;
	    status = EXIT_FAILURE;
	}
    }

    return status;
}
