// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>

#include <Pizzeria.h>
#include <Customer_impl.h>

// ----------------------------------------------------------------------
// Customer_impl constructor and destructor
// ----------------------------------------------------------------------

//
// IDL:Customer:1.0
//
Customer_impl::Customer_impl()
{
}

Customer_impl::~Customer_impl()
{
}

// ----------------------------------------------------------------------
// Customer_impl public member implementation
// ----------------------------------------------------------------------

CORBA::OctetSeq*
Customer_impl::encrypt(const char* str)
{
    CORBA::ULong len = strlen(str);
    CORBA::OctetSeq* seq = new CORBA::OctetSeq;
    seq -> length(len);

    for(CORBA::ULong i = 0 ; i < len ; i++)
    {
        //
        // Surely you can do better than this!
        //
        (*seq)[i] = (CORBA::Octet)(str[i] ^ 42);
    }

    return seq;
}

char*
Customer_impl::decrypt(const CORBA::OctetSeq& seq)
{
    CORBA::ULong len = seq.length();
    char* str = CORBA::string_alloc(len);

    for(CORBA::ULong i = 0 ; i < len ; i++)
    {
        //
        // Surely you can do better than this!
        //
        str[i] = (char)(seq[i] ^ 42);
    }

    str[len] = '\0';
    return str;
}

CORBA::ValueBase*
Customer_impl::_copy_value()
{
    Customer_impl* result = new Customer_impl;
    result -> name(name());
    result -> credit_card(credit_card());
    result -> expires(expires());
    return result;
}

void
Customer_impl::marshal(CORBA::DataOutputStream* os)
{
    CORBA::OctetSeq_var seq;

    seq = encrypt(name());
    os -> write_ulong(seq -> length());
    os -> write_octet_array(seq, 0, seq -> length());

    seq = encrypt(credit_card());
    os -> write_ulong(seq -> length());
    os -> write_octet_array(seq, 0, seq -> length());

    seq = encrypt(expires());
    os -> write_ulong(seq -> length());
    os -> write_octet_array(seq, 0, seq -> length());
}

void
Customer_impl::unmarshal(CORBA::DataInputStream* is)
{
    CORBA::String_var str;
    CORBA::OctetSeq seq;
    CORBA::ULong len;

    len = is -> read_ulong();
    seq.length(len);
    is -> read_octet_array(seq, 0, len);
    str = decrypt(seq);
    name(str);

    len = is -> read_ulong();
    seq.length(len);
    is -> read_octet_array(seq, 0, len);
    str = decrypt(seq);
    credit_card(str);

    len = is -> read_ulong();
    seq.length(len);
    is -> read_octet_array(seq, 0, len);
    str = decrypt(seq);
    expires(str);
}

// ----------------------------------------------------------------------
// CustomerFactory_impl private member implementation
// ----------------------------------------------------------------------

CORBA::ValueBase*
CustomerFactory_impl::create_for_unmarshal()
{
    return new Customer_impl;
}

// ----------------------------------------------------------------------
// CustomerFactory_impl constructor and destructor
// ----------------------------------------------------------------------

CustomerFactory_impl::CustomerFactory_impl()
{
}

CustomerFactory_impl::~CustomerFactory_impl()
{
}
