// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>
#include <OB/BootManager.h>

#include <Echo_impl.h>

#include <stdlib.h>
#include <stdio.h>
#include <errno.h>

int
run(CORBA::ORB_ptr orb, int /* argc */, char* argv[])
{
    //
    // Resolve Root POA
    //
    PortableServer::POA_var rootPOA = PortableServer::POA::_narrow(
	CORBA::Object_var(orb -> resolve_initial_references("RootPOA")));

    //
    // Get a reference to the POA manager
    //
    PortableServer::POAManager_var manager = rootPOA -> the_POAManager();

    //
    // Create implementation object
    //
    Echo_impl* echoImpl = new Echo_impl(orb, rootPOA);
    PortableServer::ServantBase_var servant = echoImpl;
    Echo_var echo = echoImpl -> _this();

    //
    // Save reference
    //
    CORBA::String_var s = orb -> object_to_string(echo);
    
    const char* refFile = "Echo.ref";
    FILE* f = fopen(refFile, "w");
    if(f == 0)
    {
	fprintf(stderr, "%s: can't open `%s': %s\n",
		argv[0], refFile, strerror(errno));
	return EXIT_FAILURE;
    }

    fputs(s.in(), f);
    fclose(f);
    
    //
    // Resolve the Boot Manager - necessary for connection via corbaloc URLs
    //
    CORBA::Object_var bmgrObj =
        orb -> resolve_initial_references("BootManager");
    OB::BootManager_var bootManager = OB::BootManager::_narrow(bmgrObj);
 
    // 
    // Create binding for administrative object in the BootManager
    // - necessary for client connections via corbaloc URLs
    //
    PortableServer::ObjectId_var id =
        PortableServer::string_to_ObjectId("Echo");
    bootManager -> add_binding(id, echo);

    //
    // Run implementation
    //
    manager -> activate();
    orb -> run();

    return EXIT_SUCCESS;
}

int
main(int argc, char* argv[], char*[])
{
    int status = EXIT_SUCCESS;
    CORBA::ORB_var orb;

    try
    {
	orb = CORBA::ORB_init(argc, argv);
	status = run(orb, argc, argv);
    }
    catch(const CORBA::Exception& ex)
    {
	fprintf(stderr, "%s\n", ex._rep_id());
	status = EXIT_FAILURE;
    }

    if(!CORBA::is_nil(orb))
    {
	try
	{
	    orb -> destroy();
	}
	catch(const CORBA::Exception& ex)
	{
	    fprintf(stderr, "%s\n", ex._rep_id());
	    status = EXIT_FAILURE;
	}
    }
    
    return status;
}
