// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>
#include <OB/OBPolicies.h>

#include <Hello.h>

#include <stdlib.h>

#ifdef HAVE_STD_IOSTREAM
using namespace std;
#endif

int
run(CORBA::ORB_ptr orb, int argc, char* argv[])
{
    //
    // We need to set the retry policy to RETRY_ALWAYS
    //
    CORBA::Any any;
    any <<= OB::RETRY_ALWAYS;
    CORBA::PolicyList pl;
    pl.length(1);
    pl[0] = orb -> create_policy(OB::RETRY_POLICY_ID, any);
    CORBA::Object_var pmObj =
        orb -> resolve_initial_references("ORBPolicyManager");
    CORBA::PolicyManager_var pm = CORBA::PolicyManager::_narrow(pmObj);
    pm -> add_policy_overrides(pl);
	
    //
    // Get "hello" object
    //
    CORBA::String_var file = CORBA::string_dup("relfile:///");
    if(argc > 1)
    {
        file += argv[1];
    }
    else
    {
#ifdef WIN32
        const char* env = getenv("TEMP");
        if(env == 0)
            env = "C:";
        file += env;
        file += "\\";
#else
	file += "/tmp/";
#endif
  	file += "Hello.ref";
    }
        
    CORBA::Object_var obj = orb -> string_to_object(file);
    if(CORBA::is_nil(obj))
    {
        cerr << argv[0] << ": cannot read IOR from "
    	     << file << endl;
        return EXIT_FAILURE;
    }
	
    Hello_var hello = Hello::_narrow(obj);
    assert(!CORBA::is_nil(hello));

    //
    // Main loop
    //
    cout << "Enter 'h' for hello, 's' to terminate server, or "
		"'x' for exit:\n";
    char c;
    do
    {
        cout << "> ";
        cin >> c;
        if(c == 'h')
    	    hello -> say_hello();
        else if(c == 's')
    	    hello -> shutdown();
    }
    while(cin.good() && c != 'x');

    return EXIT_SUCCESS;
}

int
main(int argc, char* argv[], char*[])
{
    int status = EXIT_SUCCESS;
    CORBA::ORB_var orb;

    try
    {
	orb = CORBA::ORB_init(argc, argv);
	status = run(orb, argc, argv);
    }
    catch(const CORBA::Exception& ex)
    {
	cerr << ex << endl;
	status = EXIT_FAILURE;
    }

    if(!CORBA::is_nil(orb))
    {
	try
	{
	    orb -> destroy();
	}
	catch(const CORBA::Exception& ex)
	{
	    cerr << ex << endl;
	    status = EXIT_FAILURE;
	}
    }
    
    return status;
}
