// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>

#include <OB/Properties.h>
#include <OB/Logger.h>

#include <Hello_impl.h>

#include <FileLogger.h>

#ifdef HAVE_FSTREAM
#   include <fstream>
#else
#   include <fstream.h>
#endif

#include <errno.h>

#ifdef HAVE_STD_IOSTREAM
using namespace std;
#endif

int
run(CORBA::ORB_ptr orb, int /* argc */, char* argv[])
{
    OBCORBA::ORB_var oborb = OBCORBA::ORB::_narrow(orb);
    OB::Logger_var logger = oborb -> logger();

    //
    // Resolve Root POA
    //
    CORBA::Object_var poaObj =
        orb -> resolve_initial_references("RootPOA");
    PortableServer::POA_var rootPOA = PortableServer::POA::_narrow(poaObj);

    //
    // Get a reference to the POA manager
    //
    PortableServer::POAManager_var manager = rootPOA -> the_POAManager();

    CORBA::PolicyList pl(2);
    pl.length(2);
    pl[0] = rootPOA -> create_lifespan_policy(PortableServer::PERSISTENT);
    pl[1] = rootPOA -> create_id_assignment_policy(
        PortableServer::USER_ID);

    PortableServer::POA_var helloPOA =
        rootPOA -> create_POA("hello", manager, pl);

    //
    // Create implementation object
    //
    Hello_impl* helloImpl = new Hello_impl(orb, helloPOA);
    PortableServer::ServantBase_var servant = helloImpl;

    PortableServer::ObjectId_var oid =
            PortableServer::string_to_ObjectId("hello");
    helloPOA -> activate_object_with_id(oid, servant);
    CORBA::Object_var hello = helloImpl -> _this();

    //
    // Save reference
    //
    CORBA::String_var s = orb -> object_to_string(hello);

    CORBA::String_var file;
#ifdef WIN32
    const char* env = getenv("TEMP");
    if(env == 0)
	env = "C:";
    file = env;
#else
    file = CORBA::string_dup("/tmp");
#endif
    file += "/Hello.ref";

    ofstream out(file);
    if(out.fail())
    {
        const char* err = strerror(errno);
        CORBA::String_var msg = CORBA::string_dup(argv[0]);
        msg += ": can't open `";
        msg += file;
        msg += "': ";
        msg += err;
        logger -> error(msg);
        return EXIT_FAILURE;
    }

    out << s << endl;
    out.close();

    //
    // Run implementation
    //
    manager -> activate();
    orb -> run();

    return EXIT_SUCCESS;
}

int
main(int argc, char* argv[], char*[])
{
    int status = EXIT_SUCCESS;
    CORBA::ORB_var orb;

    try
    {
        CORBA::String_var logfile;
#ifdef WIN32
        const char* env = getenv("TEMP");
        if(env == 0)
            env = "C:";
        logfile = env;
        logfile += "\\hello.log";
#else
        logfile = CORBA::string_dup("/tmp/hello.log");
#endif

	//
	// Set server name if it has not already been set
	//
	OB::Properties_var properties = OB::Properties::getDefaultProperties();
	OB::ParseArgs(argc, argv, properties);
	const char* server = properties -> getProperty("ooc.orb.server_name");
	if(server == 0)
	    properties -> setProperty("ooc.orb.server_name", "hello");

        //
        // Create log.
        //
  	OB::Logger_var logger = new FileLogger(logfile);
	
        //
        // Print command-line arguments.
        //
        logger -> info("arguments:");
        for(int c = 0; c < argc; c++)
            logger -> info(argv[c]);

	orb = OBCORBA::ORB_init(argc, argv, OB::Properties::_nil(), logger);
	status = run(orb, argc, argv);
    }
    catch(const CORBA::Exception& ex)
    {
	cerr << ex << endl;
	status = EXIT_FAILURE;
    }

    if(!CORBA::is_nil(orb))
    {
	try
	{
	    orb -> destroy();
	}
	catch(const CORBA::Exception& ex)
	{
	    cerr << ex << endl;
	    status = EXIT_FAILURE;
	}
    }
    
    return status;
}
