// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>
#include <OB/Properties.h>

#include <Peer_impl.h>

#include <stdlib.h>
#include <errno.h>

#ifdef HAVE_FSTREAM
#   include <fstream>
#else
#   include <fstream.h>
#endif

#ifdef HAVE_STD_IOSTREAM
using namespace std;
#endif


int
run(CORBA::ORB_ptr orb, int /* argc */, char* argv[])
{
    //
    // Resolve the Root POA
    //
    PortableServer::POA_var rootPOA = PortableServer::POA::_narrow(
        CORBA::Object_var(orb -> resolve_initial_references("RootPOA")));
    PortableServer::POAManager_var manager = rootPOA -> the_POAManager();

    //
    // Create implementation object
    //
    Peer_impl* peerImpl = new Peer_impl(rootPOA);
    PortableServer::ServantBase_var servant = peerImpl;
    Peer_var p1 = peerImpl -> _this();
    
    //
    // Save reference
    //
    CORBA::String_var s = orb -> object_to_string(p1);

    const char* refFile = "Peer.ref";
    ofstream out(refFile);
    if(out.fail())
    {
        cerr << argv[0] << ": can't open `" << refFile << "': "
    	     << strerror(errno) << endl;
        return EXIT_FAILURE;
    }

    out << s << endl;
    out.close();

    //
    // Run implementation
    //
    manager -> activate();
    orb -> run();

    return EXIT_SUCCESS;
}

int
main(int argc, char* argv[], char*[])
{
    int status = EXIT_SUCCESS;
    CORBA::ORB_var orb;

    try
    {
	OB::Properties_var properties = OB::Properties::getDefaultProperties();
	OB::ParseArgs(argc, argv, properties);

        //
        // Check to see if the concurrency model is usable for nested
        // method invocations. Either reactive/reactive or threaded/
        // thread_per_request/thread_pool is needed.
        //
        const char* oaModel =
	    properties -> getProperty("ooc.orb.oa.conc_model");
        const char* orbModel =
	    properties -> getProperty("ooc.orb.conc_model");
        bool valid;
        if(oaModel != 0 && orbModel != 0)
        {
            valid = ((strcmp(orbModel, "reactive") == 0 &&
                     strcmp(oaModel, "reactive") == 0) ||
                     (strcmp(orbModel, "threaded") == 0 &&
                     (strcmp(oaModel, "thread_per_request") == 0 ||
                      strcmp(oaModel, "thread_pool") == 0)));
        }
        else
            valid = false;

	//
	// Set the concurrency model to reactive/reactive if a valid
        // concurrency model wasn't selected
	//
        if(!valid)
        {
	    properties -> setProperty("ooc.orb.conc_model", "reactive");
	    properties -> setProperty("ooc.orb.oa.conc_model", "reactive");
        }
	
	properties -> setProperty("ooc.orb.id", "Nested-Peer1");

	orb = CORBA::ORB_init(argc, argv);
	status = run(orb, argc, argv);
    }
    catch(const CORBA::Exception& ex)
    {
	cerr << ex << endl;
	status = EXIT_FAILURE;
    }

    if(!CORBA::is_nil(orb))
    {
	try
	{
	    orb -> destroy();
	}
	catch(const CORBA::Exception& ex)
	{
	    cerr << ex << endl;
	    status = EXIT_FAILURE;
	}
    }
    
    return status;
}
