// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>

#include <Nested_impl.h>

#include <stdlib.h>
#include <errno.h>

#ifdef HAVE_FSTREAM
#   include <fstream>
#else
#   include <fstream.h>
#endif

#ifdef HAVE_STD_IOSTREAM
using namespace std;
#endif

using namespace CORBA;

int
run(CORBA::ORB_ptr orb, int /* argc */, char* argv[])
{
    //
    // Resolve the Root POA
    //
    PortableServer::POA_var root = PortableServer::POA::_narrow(
	CORBA::Object_var(orb -> resolve_initial_references("RootPOA")));

    //
    // Create implementation object
    //
    I_impl* impl = new I_impl();
    PortableServer::ServantBase_var servant = impl;
    I_var p = impl -> _this();

    //
    // Save reference
    //
    String_var s = orb -> object_to_string(p);

    const char* refFile = "I.ref";
    ofstream out(refFile);
    if(out.fail())
    {
        cerr << argv[0] << ": can't open `" << refFile << "': "
    	     << strerror(errno) << endl;
        return EXIT_FAILURE;
    }

    out << s << endl;
    out.close();

    //
    // Run implementation
    //
    PortableServer::POAManager_var manager = root -> the_POAManager();
    manager -> activate();
    orb -> run();

    return EXIT_SUCCESS;
}

int
main(int argc, char* argv[], char*[])
{
    int status = EXIT_SUCCESS;
    CORBA::ORB_var orb;

    try
    {
	orb = CORBA::ORB_init(argc, argv);
	status = run(orb, argc, argv);
    }
    catch(const CORBA::Exception& ex)
    {
	cerr << ex << endl;
	status = EXIT_FAILURE;
    }

    if(!CORBA::is_nil(orb))
    {
	try
	{
	    orb -> destroy();
	}
	catch(const CORBA::Exception& ex)
	{
	    cerr << ex << endl;
	    status = EXIT_FAILURE;
	}
    }
    
    return status;
}
