// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>

#include <Simple.h>

#include <stdlib.h>

using namespace Simple;

#ifdef HAVE_STD_IOSTREAM
using namespace std;
#endif

int
run(CORBA::ORB_ptr orb, int /* argc */, char* argv[])
{
    //
    // Get "simple" objects
    //
    CORBA::Object_var obj1;
    CORBA::Object_var obj2;

    try
    {
        obj1 = orb -> string_to_object("relfile:/simple1.ior");
    }
    catch(const CORBA::BAD_PARAM&)
    {
    }

    if(CORBA::is_nil(obj1))
    {
	cerr << argv[0] << ": cannot read IOR from simple1.ior" << endl;
	return EXIT_FAILURE;
    }

    try
    {
        obj2 = orb -> string_to_object("relfile:/simple2.ior");
    }
    catch(const CORBA::BAD_PARAM&)
    {
    }

    if(CORBA::is_nil(obj2))
    {
	cerr << argv[0] << ": cannot read IOR from simple2.ior" << endl;
	return EXIT_FAILURE;
    }
    
    SimpleObject_var simple1 = SimpleObject::_narrow(obj1);
    assert(!CORBA::is_nil(simple1));

    SimpleObject_var simple2 = SimpleObject::_narrow(obj2);
    assert(!CORBA::is_nil(simple2));
    
    //
    // Call implementations
    //
    simple1 -> call_me();
    simple2 -> call_me();

    return EXIT_SUCCESS;
}

int
main(int argc, char* argv[], char*[])
{
    int status = EXIT_SUCCESS;
    CORBA::ORB_var orb;

    try
    {
	orb = CORBA::ORB_init(argc, argv, "Simple-Client");
	status = run(orb, argc, argv);
    }
    catch(const CORBA::Exception& ex)
    {
	cerr << ex << endl;
	status = EXIT_FAILURE;
    }

    if(!CORBA::is_nil(orb))
    {
	try
	{
	    orb -> destroy();
	}
	catch(const CORBA::Exception& ex)
	{
	    cerr << ex << endl;
	    status = EXIT_FAILURE;
	}
    }
    
    return status;
}
