// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>

#include <Finance.h>
#include <EmployeeFactory_impl.h>

#include <stdlib.h>

#ifdef HAVE_STD_IOSTREAM
using namespace std;
#endif

int
run(CORBA::ORB_ptr orb, int /* argc */, char* argv[])
{
    //
    // Instantiate value factory
    //
    // NOTE: There's no need to register this factory with the ORB
    //       unless we will be receiving a Employee value.
    //
    EmployeeFactory_impl* factory = new EmployeeFactory_impl();
    CORBA::ValueFactoryBase_var factoryVar = factory;

    //
    // Obtain the server's object reference from a file
    //
    CORBA::Object_var obj =
        orb -> string_to_object("relfile:/Payroll.ref");
    if(CORBA::is_nil(obj))
    {      
        cerr << argv[0] << ": cannot read IOR from Payroll.ref" << endl;
        return EXIT_FAILURE;
    }        

    Payroll_var payroll = Payroll::_narrow(obj);
    assert(!CORBA::is_nil(payroll));

    //
    // Test with null value
    //
    payroll -> pay_employee((Employee*)0);

    //
    // Test with real value
    //
    Employee_var emp1 = factory -> create((CORBA::Float)1000.0);
    emp1 -> emp_id(999);
    emp1 -> name((const char*)"John Smith");
    Employee::Address addr;
    addr.street = CORBA::string_dup("1234 Maiden Lane");
    addr.city = CORBA::string_dup("Somewhere");
    addr.state = CORBA::string_dup("CA");
    addr.zip = CORBA::string_dup("95000");
    emp1 -> addr(addr);
    emp1 -> ssn((const char*)"123-45-6789");
    cout << "Paying " << emp1 -> name() << endl;
    payroll -> pay_employee(emp1);

    return EXIT_SUCCESS;
}

int
main(int argc, char* argv[], char*[])
{
    int status = EXIT_SUCCESS;
    CORBA::ORB_var orb;

    try
    {
	orb = CORBA::ORB_init(argc, argv);
	status = run(orb, argc, argv);
    }
    catch(const CORBA::Exception& ex)
    {
	cerr << ex << endl;
	status = EXIT_FAILURE;
    }

    if(!CORBA::is_nil(orb))
    {
	try
	{
	    orb -> destroy();
	}
	catch(const CORBA::Exception& ex)
	{
	    cerr << ex << endl;
	    status = EXIT_FAILURE;
	}
    }
    
    return status;
}
