// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>
#include <OB/Codec.h>

#include <IMRUtil.h>

const char*
IMRUtil::serverStatusToStr(IMR::ServerStatus status)
{
    const char* str = "UNKNOWN";
    switch(status)
    {
    case IMR::FORKED:
        str = "forked";
        break;
    case IMR::STARTING:
        str = "starting";
        break;
    case IMR::RUNNING:
        str = "running";
        break;
    case IMR::STOPPING:
        str = "stopping";
        break;
    case IMR::STOPPED:
        str = "stopped";
        break;
    }
    return str;
}

const char*
IMRUtil::poaStatusToStr(IMR::POAStatus status)
{
    const char* str = "UNKNOWN";
    switch(status)
    {
    case IMR::INACTIVE:
        str = "inactive";
        break;
    case IMR::ACTIVE:
        str = "active";
        break;
    case IMR::HOLDING:
        str = "holding";
        break;
    case IMR::DISCARDING:
        str = "discarding";
        break;
    case IMR::NON_EXISTENT:
        str = "nonexistent";
        break;
    }
    return str;
}

bool
IMRUtil::poacmp(const IMR::POAName& p1,
		const IMR::POAName& p2,
		bool skipFirst)
{
    //
    // Whether we should skip the first entry in the second sequence
    //
    CORBA::ULong offset = skipFirst ? 1 : 0;

    //
    // Different length, doesn't match.
    //
    if(p1.length() + offset != p2.length())
        return false;

    //
    // Same length, verify match.
    //
    for(CORBA::ULong i = 0; i < p1.length(); i++)
        if(strcmp(p1[i], p2[i + offset]) != 0)
            return false;

    return true;
}

IOP::Codec_ptr
IMRUtil::codec(CORBA::ORB_ptr orb)
{
    //
    // Get codec factory
    //
    IOP::CodecFactory_var factory = IOP::CodecFactory::_narrow(
        CORBA::Object_var(orb -> resolve_initial_references("CodecFactory")));
    assert(!CORBA::is_nil(factory));

    struct IOP::Encoding how;
    how.format = IOP::ENCODING_CDR_ENCAPS;
    how.major_version = 1;
    how.minor_version = 2;

    return factory -> create_codec(how);
}
