// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#ifndef DOMAIN_ADMIN_IMPL_H
#define DOMAIN_ADMIN_IMPL_H

#include <IMRInternal_skel.h>
#include <IMRLogger_fwd.h>

namespace IMR
{

//
// Forward declarations
//
class ActiveStateFactory;
class ServerFactory_impl;
class OADDatabase;
class ServerDatabase;
class OADEndpointManager_impl;
class ServerStarter;

class InternalDomain_impl : public POA_IMR::InternalDomain
{
    OADDatabase* oaddb_;
    PortableServer::POA_var poa_;
    ServerFactory_impl* serverFactory_;
    ActiveStateFactory* activeStateFactory_;
    OADEndpointManager_impl* endpoint_;
    bool administrative_;

public:

    InternalDomain_impl(CORBA::ORB_ptr, PortableServer::POA_ptr,
                        PortableServer::POA_ptr, OADDatabase*, ServerDatabase*,
			OADEndpointManager_impl*, IMROAD::Logger_ptr, bool,
			ServerStarter*);
    ~InternalDomain_impl();

    ActiveStateFactory* get_active_state_factory();

    //
    // Standard IDL to C++ Mapping
    //

    //
    // Mapping for IMR::Domain
    //
    virtual void registerServer(const char*, const char*, const char*)
        throw(ServerAlreadyRegistered, CORBA::SystemException);

    virtual ActiveState_ptr 
    startup(const char*, const char*,
    	    PortableInterceptor::ObjectReferenceTemplate*,
	    OAD::ProcessEndpointManager_out)
        throw(NoSuchServer, NoSuchOAD, OADNotRunning, CORBA::SystemException);

    //
    // Mapping for IMR::ServerDomain
    //
    virtual ServerFactory_ptr get_server_factory()
        throw(CORBA::SystemException);

    virtual void create_oad_record(const char* host)
	throw(IMR::OADAlreadyExists, CORBA::SystemException);

    virtual void remove_oad_record(const char* host)
	throw(IMR::NoSuchOAD, IMR::OADRunning, CORBA::SystemException);

    OADInfo* get_oad_record(const char* host)
	throw(IMR::NoSuchOAD, CORBA::SystemException);

    virtual OADInfoSeq* list_oads()
	throw(CORBA::SystemException);

    //
    // Mapping for IMR::InternalDomain
    //
    virtual OADEndpointManager_ptr get_endpoint_manager()
        throw(CORBA::SystemException);


    virtual PortableServer::POA_ptr _default_POA();
};

}; // End namespace IMR

#endif
