// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#ifndef OAD_REGISTRAR_IMPL_H
#define OAD_REGISTRAR_IMPL_H

#include <IMRLogger_fwd.h>
#include <IMROAD_skel.h>
#include <IMRDB.h>

namespace IMR
{

//
// Forward declarations
//
class OADDatabase;
class ServerDatabase;
class ServerStarter;

//
// Endpoint manager implementation
//
class OADEndpointManager_impl : public POA_IMR::OADEndpointManager
{
    //
    // The ORB
    //
    OBCORBA::ORB_var orb_;

    //
    // The POA
    //
    PortableServer::POA_var poa_;

    //
    // The message logger
    //
    IMROAD::Logger_var logger_;

    //
    // The OAD Database
    //
    OADDatabase* oaddb_;

    //
    // The server database
    //
    ServerDatabase* db_;

    //
    // The server observer
    //
    IMR::ServerObserver_var observer_;

    //
    // The retry policy
    //
    CORBA::Policy_var retryPolicy_;

    //
    // The OCI::Current
    //
    OCI::Current_var current_;

    //
    // The server starter
    //
    IMR::ServerStarter* serverStarter_;

    //
    // Called when an OAD terminates to mark each associated server as
    // STOPPED and reset state
    //
    void updateServersForOAD(IMRDB_4_0_2::OADRecord&,
			     const OAD::ProcessList&);

public:

    OADEndpointManager_impl(CORBA::ORB_ptr, PortableServer::POA_ptr,
    			    IMROAD::Logger_ptr, OADDatabase*, ServerDatabase*, 
			    ServerStarter*);
    ~OADEndpointManager_impl();

    //
    // Standard IDL to C++ Mapping
    //

    //
    // IDL:ooc.com/IMR/OADEndpointManager/register_oad:1.0
    //
    virtual void
    register_oad(const char*,
		 const OAD::ProcessList&,
		 OAD::ProcessController_ptr,
                 OAD::ProcessEndpointManager_ptr,
                 ServerObserver_out)
        throw(IMR::NoSuchOAD, CORBA::SystemException);
    
    //
    // Internal methods
    //
    void _OB_setNotifier(IMR::ServerObserver_ptr);
    void _OB_connectOADs(bool);
    void _OB_disconnect(const char*);

    virtual PortableServer::POA_ptr _default_POA();
};

}; // End namespace IMR

#endif
