// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#ifndef PROCESS_ENDPOINT_MANAGER_IMPL_H
#define PROCESS_ENDPOINT_MANAGER_IMPL_H

#include <OB/OADProcessEndpoint_skel.h>

#include <IMRLogger_fwd.h>

namespace OAD
{

//
// Forward declarations
//
class ProcessDatabase;
class ProcObserver;

//
// This class acts as a registrar for ProcController objects. Each
// ProcController connects to an instance of this class to receive
// requests to reconnect.
//
class ProcessEndpointManager_impl :
	public POA_OAD::ProcessEndpointManager
{
    //
    // The POA
    //
    PortableServer::POA_var poa_;
    
    //
    // The OCI::Current object
    //
    OCI::Current_var current_;

    //
    // The process database
    //
    ProcessDatabase* db_;

    //
    // The process controller
    //
    ProcObserver* observer_;

    //
    // Trace Levels
    //
    IMROAD::Logger_var logger_;

    //
    // Shuuting down?
    //
    bool terminating_;

public:

    ProcessEndpointManager_impl(CORBA::ORB_ptr,
				PortableServer::POA_ptr,
				ProcessDatabase*,
				ProcObserver*,
				IMROAD::Logger_ptr);
    ~ProcessEndpointManager_impl();

    void shutdown();

    //
    // Standard IDL to C++ Mapping
    //
    virtual void
    establish_link(const char*, const char*, IMR::ProcessID,
    		   ProcessEndpoint_ptr)
	throw(AlreadyLinked, CORBA::SystemException);

    virtual PortableServer::POA_ptr
    _default_POA();

    //
    // Internal methods
    //

    //
    // Called on a connection close callback
    //
    void
    disconnect(const char*, const char*);
};

}; // End namespace OAD

#endif
