// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>

#include <IMRInternal.h>
#include <ServerStarter.h>
#include <ActiveStateFactory.h>

using namespace IMR;

// ----------------------------------------------------------------------
// InternalServer_impl constructor and destructor
// ----------------------------------------------------------------------

ServerStarter::ServerStarter()
    : stopped_(false)
{
}

ServerStarter::~ServerStarter()
{
}

// ----------------------------------------------------------------------
// InternalServer_impl public member implementation
// ----------------------------------------------------------------------

void
ServerStarter::run()
{
    for(;;)
    {
        JTCSynchronized synchronized(*this);

	//
	// Done?
	//
        if(stopped_)
	    return;

	while(serverList_.size() != 0)
	{
	    //
	    // Attempt to fork the servers. Only one attempt is made.
	    //
	    try
	    {
	        ActiveState_var s =
	            factory_ -> get_active_state(serverList_.front());
	        InternalServer_var server = IMR::InternalServer::_narrow(s);

	        if(!CORBA::is_nil(server))
	            server -> fork();

	    }
	    catch(...)
	    {
	    }
	    serverList_.pop_front();
	}

	wait();
    }
}

void
ServerStarter::terminate()
{
    JTCSynchronized synchronized(*this);
    stopped_ = true;
    notify();
}

void
ServerStarter::add(const char* server)
{
    JTCSynchronized synchronized(*this);
    serverList_.push_back(server);
    notify();
}

void
ServerStarter::setActiveStateFactory(ActiveStateFactory* factory)
{
    factory_ = factory;
}
