// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#ifndef SERVER_IMPL_H
#define SERVER_IMPL_H

#include <OB/IMR_skel.h>

#include <IMRLogger_fwd.h>
#include <ServerRecord_fwd.h>
#include <IMRDB.h>

namespace IMR
{

//
// Forward declarations
//
class OADDatabase;
class ServerDatabase;
class ServerStarter;

class Server_impl : public POA_IMR::Server
{
    ServerDatabase* db_;
    OADDatabase* oaddb_;

    PortableServer::Current_var current_;

    IMROAD::Logger_var logger_;

    bool administrative_;

    ServerStarter* serverStarter_;

    ServerRecord_ptr getRecord();

public:

    Server_impl(ServerDatabase*, OADDatabase*, CORBA::ORB_ptr,
		IMROAD::Logger_ptr, bool, ServerStarter*);
    ~Server_impl();

    //
    // Standard IDL to C++ Mapping
    //
    
    //
    // IDL:ooc.com/IMR/Server/id:1.0
    //
    virtual IMR::ServerID id()
        throw(CORBA::SystemException);

    //
    // IDL:ooc.com/IMR/Server/status:1.0
    //
    virtual IMR::ServerStatus status()
        throw(CORBA::SystemException);

    //
    // IDL:ooc.com/IMR/Server/manual:1.0
    //
    virtual CORBA::Boolean manual()
        throw(CORBA::SystemException);

    //
    // IDL:ooc.com/IMR/Server/updateTime:1.0
    //
    virtual CORBA::Long updateTime()
        throw(CORBA::SystemException);

    //
    // IDL:ooc.com/IMR/Server/timesForked:1.0
    //
    virtual CORBA::Short timesForked()
        throw(CORBA::SystemException);

    //
    // IDL:ooc.com/IMR/Server/name:1.0
    //
    virtual char* name()
        throw(CORBA::SystemException);
    virtual void name(const char*)
        throw(CORBA::SystemException);

    //
    // IDL:ooc.com/IMR/Server/host:1.0
    //
    virtual char* host()
        throw(CORBA::SystemException);
    virtual void host(const char*)
        throw(CORBA::SystemException);

    //
    // IDL:ooc.com/IMR/Server/exec:1.0
    //
    virtual char* exec()
        throw(CORBA::SystemException);
    virtual void exec(const char*)
        throw(CORBA::SystemException);

    //
    // IDL:ooc.com/IMR/Server/directory:1.0
    //
    virtual char* directory()
        throw(CORBA::SystemException);
    virtual void directory(const char*)
        throw(CORBA::SystemException);

    //
    // IDL:ooc.com/IMR/Server/args:1.0
    //
    virtual IMR::ArgList* args()
        throw(CORBA::SystemException);
    virtual void args(const IMR::ArgList&)
        throw(CORBA::SystemException);

    //
    // IDL:ooc.com/IMR/Server/mode:1.0
    //
    virtual IMR::ServerActivationMode mode()
        throw(CORBA::SystemException);
    virtual void mode(IMR::ServerActivationMode)
        throw(CORBA::SystemException);

    //
    // IDL:ooc.com/IMR/Server/updateTimeout:1.0
    //
    virtual CORBA::Long updateTimeout()
        throw(CORBA::SystemException);
    virtual void updateTimeout(CORBA::Long)
        throw(CORBA::SystemException);

    //
    // IDL:ooc.com/IMR/Server/failureTimeout:1.0
    //
    virtual CORBA::Long failureTimeout()
        throw(CORBA::SystemException);
    virtual void failureTimeout(CORBA::Long)
        throw(CORBA::SystemException);

    //
    // IDL:ooc.com/IMR/Server/maxForks:1.0
    //
    virtual CORBA::Short maxForks()
        throw(CORBA::SystemException);
    virtual void maxForks(CORBA::Short)
        throw(CORBA::SystemException);

    //
    // IDL:ooc.com/IMR/Server/activatePOAs:1.0
    //
    virtual CORBA::Boolean activatePOAs()
        throw(CORBA::SystemException);
    virtual void activatePOAs(CORBA::Boolean)
        throw(CORBA::SystemException);

    //
    // IDL:ooc.com/IMR/Server/create_poa_record:1.0
    //
    virtual void create_poa_record(const IMR::POAName& poa)
        throw(IMR::POAAlreadyRegistered,
              CORBA::SystemException);

    //
    // IDL:ooc.com/IMR/Server/remove_poa_record:1.0
    //
    virtual void remove_poa_record(const IMR::POAName& poa)
        throw(IMR::NoSuchPOA, CORBA::SystemException);

    //
    // IDL:ooc.com/IMR/Server/get_poa_info:1.0
    //
    virtual IMR::POAInfo* get_poa_info(const IMR::POAName& poa)
        throw(IMR::NoSuchPOA, CORBA::SystemException);

    //
    // IDL:ooc.com/IMR/Server/list_poas:1.0
    //
    virtual IMR::POAInfoSeq* list_poas()
        throw(CORBA::SystemException);

    //
    // IDL:ooc.com/IMR/Server/clear_error_state:1.0
    //
    virtual void clear_error_state()
        throw(CORBA::SystemException);

    //
    // IDL:ooc.com/IMR/Server/start:1.0
    //
    virtual void start()
	throw(IMR::ServerRunning, CORBA::SystemException);

    //
    // IDL:ooc.com/IMR/Server/stop:1.0
    //
    virtual void stop()
	throw(IMR::OADNotRunning, IMR::ServerNotRunning,
	      CORBA::SystemException);

    //
    // IDL:ooc.com/IMR/Server/destroy:1.0
    //
    virtual void destroy()
        throw(IMR::ServerRunning, CORBA::SystemException);
};

}; // End namespace IMR

#endif
