// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#ifndef OB_CODE_SET_DATABASE_H
#define OB_CODE_SET_DATABASE_H

#include <OB/TypeCode.h>

namespace CONV_FRAME
{

//
// Forward declaration
//
struct CodeSetComponent;

}; // End of namespace CONV_FRAME

namespace OB
{

//
// Forward declaration
//
class CodeConverterBase;

class CodeSetDatabase : public JTCMutex
{
public:

    //
    // The code set registry IDs for the supported code sets
    //
    OB_IMPORT static const CORBA::ULong ISOLATIN1;
    OB_IMPORT static const CORBA::ULong ISOLATIN2;
    OB_IMPORT static const CORBA::ULong ISOLATIN3;
    OB_IMPORT static const CORBA::ULong ISOLATIN4;
    OB_IMPORT static const CORBA::ULong ISOLATIN5;
    OB_IMPORT static const CORBA::ULong ISOLATIN7;
    OB_IMPORT static const CORBA::ULong ISOLATIN9;
    OB_IMPORT static const CORBA::ULong PCS;
    OB_IMPORT static const CORBA::ULong UTF8;
    OB_IMPORT static const CORBA::ULong UTF16;
    OB_IMPORT static const CORBA::ULong UCS2;

    //
    // The data required for the conversion
    //
    struct CodeSetInfo
    {
        const char* description;
        CORBA::ULong rgy_value;
        CORBA::ULong char_values_size;
        const CORBA::UShort* char_values;
        CORBA::UShort max_bytes;
    };

    //
    // The character mapping data
    //
    struct CharMapInfo
    {
	const char* name;
	CORBA::ULong upper_bound;
//	CORBA::ULong map_values_size;
	const CORBA::UShort* map_values;
    };

private:

    typedef FixSeq< CodeConverterBase*, int> CodeConverterSeq;
    CodeConverterSeq codeConverters_;

    //
    // The CodeSetDatabase singleton
    //
    static JTCMutex instanceMutex_;
    static OB_IMPORT CodeSetDatabase* instance_;
    friend class CodeSetDatabaseDestroyer;

    //
    // The array with code set infos and the size of that array
    //
    static const CodeSetInfo codeSetInfoArray_[];
    static const CORBA::ULong codeSetInfoArraySize_;

    //
    // The array with character mapping infos and the size of that array
    //
    static const CharMapInfo charMapInfoArray_[];
    static const CORBA::ULong charMapInfoArraySize_;

    CodeSetDatabase();
    ~CodeSetDatabase();

    //
    // Check code set compatibility
    //
    bool isCompatible(CORBA::ULong, CORBA::ULong) const;

    //
    // Check for particular code set id in CodeSetComponent
    //
    bool checkCodeSetId(const CONV_FRAME::CodeSetComponent&,
			CORBA::ULong) const;

public:

    //
    // Get singleton instance
    //
    static CodeSetDatabase* instance();

    //
    // Get CodeSetInfo for a particular registry ID
    //
    const CodeSetInfo* getCodeSetInfo(CORBA::ULong) const;

    //
    // Determine transmission codeset (TCS) from client/server code sets
    //
    CORBA::ULong determineTCS(const CONV_FRAME::CodeSetComponent&,
			      const CONV_FRAME::CodeSetComponent&,
			      CORBA::ULong) const;

    //
    // Convert code set name to OSF code set ID
    //
    CORBA::ULong nameToId(const char*) const;

    //
    // Get converter
    //
    CodeConverterBase* getConverter(CORBA::ULong, CORBA::ULong);

    //
    // Get Unicode character map for a particular code set
    //
    const CharMapInfo* getCharMapInfo(CORBA::ULong) const;
};

} // End of namespace OB

#endif
