// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/Client.h>
#include <OB/Except.h>
#include <OB/OCI.h>
#include <OB/Stream.h>

#include <CollocatedClient.h>
#include <CollocatedServer.h>

using namespace CORBA;
using namespace OB;

// ----------------------------------------------------------------------
// External, non-inline duplicate/release for templates
// ----------------------------------------------------------------------

void
OBDuplicate(OB::CollocatedClient_ptr p)
{
    if(p)
	p -> _OB_incRef();
}

void
OBRelease(OB::CollocatedClient_ptr p)
{
    if(p)
	p -> _OB_decRef();
}

// ----------------------------------------------------------------------
// CollocatedClient constructor and destructor
// ----------------------------------------------------------------------

CollocatedClient::CollocatedClient(CollocatedServer_ptr server,
				   Client::ConcModel concModel,
				   const CodeConverters& conv)
    : Client(concModel, conv),
      server_(CollocatedServer::_duplicate(server))
{
}

CollocatedClient::~CollocatedClient()
{
}

// ----------------------------------------------------------------------
// CollocatedClient public member implementation
// ----------------------------------------------------------------------

void
CollocatedClient::destroy()
{
    // Nothing to do here
}

ULong
CollocatedClient::requestId()
{
    //
    // This operation *must* delegate to CollocatedServer, because
    // request IDs must be unique per CollocatedServer, not per
    // CollocatedClient.
    //
    return server_ -> requestId();
}

OCI::ProfileInfoSeq*
CollocatedClient::getUsableProfiles(const OCI::IOR& ior,
				    const PolicyList& policies)
{
    return server_ -> getUsableProfiles(ior, policies);
}

OCI::ConnectorInfo_ptr
CollocatedClient::connectorInfo()
{
    return OCI::ConnectorInfo::_nil(); // There is no connector
}

OCI::TransportInfo_ptr
CollocatedClient::transportInfo()
{
    return OCI::TransportInfo::_nil(); // There is no transport
}
    
DowncallEmitter_ptr
CollocatedClient::startDowncall(Downcall_ptr, OutputStream_out out)
    throw()
{
    OCI::Buffer_var buf = new OCI::Buffer;
    out = new OutputStream(buf);
    return DowncallEmitter::_duplicate(this);
}

bool
CollocatedClient::equal(Client_ptr cl)
{
    CollocatedClient_ptr client = dynamic_cast<CollocatedClient_ptr>(cl);
    if(is_nil(client))
	return false;
    
    if(server_.in() != client -> server_.in())
	return false;
    
    return true;
}

void
CollocatedClient::bind(CORBA::Long /*connectTimeout*/)
{
    // nothing to do
}

bool
CollocatedClient::twoway()
{
    return true;
}

bool
CollocatedClient::send(Downcall_ptr down, bool block)
    throw()
{
    return server_ -> send(down, block);
}

bool
CollocatedClient::receive(Downcall_ptr down, bool block)
    throw()
{
    return server_ -> receive(down, block);
}

bool
CollocatedClient::sendReceive(Downcall_ptr down)
    throw()
{
    return server_ -> sendReceive(down);
}
