// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#ifndef OB_COLLOCATED_CLIENT_H
#define OB_COLLOCATED_CLIENT_H

#include <OB/Client.h>
#include <OB/Downcall.h>

#include <CollocatedClient_fwd.h>
#include <CollocatedServer_fwd.h>

namespace OB
{

//
// The CollocatedClient class
//
class CollocatedClient : public Client, public DowncallEmitter
{
    //
    // Hide copy-constructor and assignment operator
    //
    CollocatedClient(const CollocatedClient&);
    void operator=(const CollocatedClient&);

    //
    // My server
    //
    CollocatedServer_var server_;

public:

    CollocatedClient(CollocatedServer_ptr, Client::ConcModel,
		     const CodeConverters&);
    virtual ~CollocatedClient();

    static inline CollocatedClient_ptr _duplicate(CollocatedClient_ptr p)
    { if(p) p -> _OB_incRef(); return p; }
    static inline CollocatedClient_ptr _nil()
    { return 0; }

    //
    // Destroy the client
    //
    virtual void destroy();

    //
    // Get a new request ID
    //
    virtual CORBA::ULong requestId();

    //
    // Get all profiles that are usable with this client
    //
    virtual OCI::ProfileInfoSeq* getUsableProfiles(const OCI::IOR&,
						   const CORBA::PolicyList&);

    //
    // Get the OCI Connector info
    //
    virtual OCI::ConnectorInfo_ptr connectorInfo();

    //
    // Get the OCI Transport info
    //
    virtual OCI::TransportInfo_ptr transportInfo();
    
    //
    // Start a downcall, returning a downcall emitter and an
    // OutputStream for marshalling a request
    //
    virtual DowncallEmitter_ptr startDowncall(Downcall_ptr, OutputStream_out)
	throw();

    //
    // Checks whether this client is equal to another client
    //
    virtual bool equal(Client_ptr);

    //
    // Force connection establishment
    //
    virtual void bind(CORBA::Long);

    //
    // Determines whether this client supports twoway invocations
    //
    virtual bool twoway();

    //
    // Called to emit downcalls
    //
    virtual bool send(Downcall_ptr, bool)
        throw();
    virtual bool receive(Downcall_ptr, bool)
        throw();
    virtual bool sendReceive(Downcall_ptr)
        throw();
};

} // End of namespace OB

namespace CORBA
{

inline void
release(OB::CollocatedClient_ptr p)
{
    if(p)
	p -> _OB_decRef();
}

inline Boolean
is_nil(OB::CollocatedClient_ptr p)
{
    return p == 0;
}

} // End of namespace CORBA

#endif
