// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#ifndef OB_COLLOCATED_SERVER_H
#define OB_COLLOCATED_SERVER_H

#include <OB/OAInterface_fwd.h>
#include <OB/Downcall_fwd.h>
#include <CollocatedServer_fwd.h>

#include <OB/Upcall.h>
#include <OB/Hashtable.h>

#include <Server.h>

namespace OB
{

//
// The CollocatedServer class
//
class CollocatedServer : public Server, public UpcallReturn, public JTCMonitor
{
    //
    // Hide copy-constructor and assignment operator
    //
    CollocatedServer(const CollocatedServer&);
    void operator=(const CollocatedServer&);

    //
    // The next request ID and the corresponding mutex
    //
    CORBA::ULong nextRequestId_;
    JTCMutex nextRequestIdMutex_;

    //
    // The call map
    //
    typedef Hashtable<CORBA::ULong, Downcall_var, ULongHasher> CallMap;
    CallMap callMap_;

    //
    // True if destroy() was called
    //
    bool destroy_;

    //
    // True if holding
    //
    bool hold_;

    //
    // The object adapter interface
    //
    OAInterface_var oaInterface_;

public:
    
    CollocatedServer(OAInterface_ptr, Server::ConcModel);
    virtual ~CollocatedServer();

    static inline CollocatedServer_ptr _duplicate(CollocatedServer_ptr p)
    { if(p) p -> _OB_incRef(); return p; }
    static inline CollocatedServer_ptr _nil()
    { return 0; }

    //
    // Destroy the server
    //
    virtual void destroy();

    //
    // Hold any new requests that arrive for the Server
    //
    virtual void hold();

    //
    // Dispatch any requests that arrive for the Server
    //
    virtual void activate();

    //
    // Called to emit downcalls
    //
    virtual bool send(Downcall_ptr, bool)
        throw();
    virtual bool receive(Downcall_ptr, bool)
        throw();
    virtual bool sendReceive(Downcall_ptr)
        throw();

    //
    // Get a new request ID
    //
    CORBA::ULong requestId();

    //
    // Get the usable profiles
    //
    OCI::ProfileInfoSeq* getUsableProfiles(const OCI::IOR&,
					   const CORBA::PolicyList&);

    //
    // Called upon return from an upcall
    //
    virtual void upcallBeginReply(Upcall_ptr,
				  const IOP::ServiceContextList&);
    virtual void upcallEndReply(Upcall_ptr);
    virtual void upcallBeginUserException(Upcall_ptr,
					  const IOP::ServiceContextList&);
    virtual void upcallEndUserException(Upcall_ptr);
    virtual void upcallUserException(Upcall_ptr,
                                     const CORBA::UserException&,
                                     const IOP::ServiceContextList&);
    virtual void upcallSystemException(Upcall_ptr,
				       const CORBA::SystemException&,
				       const IOP::ServiceContextList&);
    virtual void upcallForward(Upcall_ptr, RefCountIOR_ptr, bool,
			       const IOP::ServiceContextList&);
};

} // End of namespace OB

namespace CORBA
{

inline void
release(OB::CollocatedServer_ptr p)
{
    if(p)
	p -> _OB_decRef();
}

inline Boolean
is_nil(OB::CollocatedServer_ptr p)
{
    return p == 0;
}

} // End of namespace CORBA

#endif
