// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/TypeCode.h>
#include <OB/Environment.h>
#include <OB/Context.h>
#include <OB/NamedValue.h>
#include <OB/DII.h>
#include <OB/ORBInstance.h>
#include <OB/MultiRequestSender.h>
#include <OB/Bounds.h>
#include <OB/Stream.h>
#include <OB/IFR2.h>
#include <OB/RefCounted.h>
#include <OB/Downcall.h> // TODO: Remove!!!

#include <ORB_impl.h>

using namespace CORBA;
using namespace OB;

// ----------------------------------------------------------------------
// External, non-inline duplicate/release for templates
// ----------------------------------------------------------------------

void
OBDuplicate(CORBA::ExceptionList_ptr p)
{
    if(p)
	p -> _OB_incRef();
}

void
OBRelease(CORBA::ExceptionList_ptr p)
{
    if(p)
	p -> _OB_decRef();
}

void
OBDuplicate(CORBA::ContextList_ptr p)
{
    if(p)
	p -> _OB_incRef();
}

void
OBRelease(CORBA::ContextList_ptr p)
{
    if(p)
	p -> _OB_decRef();
}

void
OBDuplicate(CORBA::Request_ptr p)
{
    if(p)
	p -> _OB_incRef();
}

void
OBRelease(CORBA::Request_ptr p)
{
    if(p)
	p -> _OB_decRef();
}

// ----------------------------------------------------------------------
// ExceptionList public member implementation
// ----------------------------------------------------------------------

ULong
ExceptionList::count() const
{
    return typeCodes_.size();
}

void
ExceptionList::add(TypeCode_ptr tc)
{
    typeCodes_.push_back(TypeCode::_duplicate(tc));
}

void
ExceptionList::add_consume(TypeCode_ptr tc)
{
    typeCodes_.push_back(tc);
}

TypeCode_ptr
ExceptionList::item(ULong n)
{
    if(n >= typeCodes_.size())
        throw Bounds();
    
    OB_STL::list<TypeCode_var>::iterator iter;
    for(iter = typeCodes_.begin();
        iter != typeCodes_.end() && n > 0;
	iter++, n--)
	 ; // Do nothing

    assert(n == 0);
    return *iter;
}

void
ExceptionList::remove(ULong n)
{
    if(n >= typeCodes_.size())
        throw Bounds();

    OB_STL::list<TypeCode_var>::iterator iter;
    for (iter = typeCodes_.begin();
         iter != typeCodes_.end() && n > 0;
	 iter++, n--)
	 ; // Do nothing

    assert(n == 0);
    typeCodes_.erase(iter);
}

// ----------------------------------------------------------------------
// ContextList public member implementation
// ----------------------------------------------------------------------

ULong
ContextList::count() const
{
    return strings_.size();
}

void
ContextList::add(const char* ctxt)
{
    assert_nca(ctxt != 0, NCANullString);
    strings_.push_back(ctxt);
}

void
ContextList::add_consume(char* ctxt)
{
    assert_nca(ctxt != 0, NCANullString);
    strings_.push_back(ctxt);
}

const char*
ContextList::item(ULong n)
{
    if(n >= strings_.size())
        throw Bounds();

    OB_STL::list<String_var>::iterator iter;
    for(iter = strings_.begin();
        iter != strings_.end() && n > 0;
	iter++, n--)
	 ; // Do nothing

    assert(n == 0);
    return *iter;
}

void
ContextList::remove(ULong n)
{
    if(n >= strings_.size())
        throw Bounds();

    OB_STL::list<String_var>::iterator iter;
    for(iter = strings_.begin();
	iter != strings_.end() && n > 0;
	iter++, n--)
	 ; // Do nothing

    assert(n == 0);
    strings_.erase(iter);
}

// ----------------------------------------------------------------------
// Request private member functions
// ----------------------------------------------------------------------

void
Request::marshal()
{
    assert(!is_nil(marshalStub_));
    assert(!is_nil(downcall_));

    OutputStream_ptr out = marshalStub_ -> _OB_preMarshal(downcall_);

    try
    {
	ULong i;
	
	StrSeq< int > ctxSeq;
	if(!is_nil(ctx_))
	{
	    for(i = 0 ; i < contexts_ -> count() ; i++)
	    {
		const char* item = contexts_ -> item(i);
		ctx_ -> _OB_getValues("", 0, item, ctxSeq);
	    }
	}
	
	for(i = 0 ; i < arguments_ -> count() ; i++)
	{
	    NamedValue_ptr nv = arguments_ -> item(i);
	    // Note: Don't use != ARG_OUT here, because flags can also
	    // have values other than ARG_IN, ARG_OUT, or ARG_INOUT
	    if(nv -> flags() == ARG_IN || nv -> flags() == ARG_INOUT)
	    {
		Any* any = nv -> value();
		out -> write_any_NoTypeCode(*any);
	    }
	}
	
	if(!is_nil(ctx_))
	{
	    out -> write_ulong(ctxSeq.length());
	    for(i = 0 ; i < ctxSeq.length() ; i++)
		out -> write_string(ctxSeq[i]);
	}
    }
    catch(const SystemException& ex)
    {
	marshalStub_ -> _OB_marshalEx(downcall_, ex);
    }

    marshalStub_ -> _OB_postMarshal(downcall_);
}

void
Request::unmarshal()
{
    assert(!is_nil(marshalStub_));
    assert(!is_nil(downcall_));

    bool uex;
    InputStream_ptr in = marshalStub_ -> _OB_preUnmarshal(downcall_, uex);

    if(is_nil(in))
    {
	assert(!uex);
	marshalStub_ -> _OB_postUnmarshal(downcall_);
	return;
    }

    if(uex)
    {
	try
	{
	    String_var id = marshalStub_ ->
		_OB_unmarshalExceptionId(downcall_);
	    
	    for(ULong i = 0 ; i < exceptions_ -> count() ; i++)
	    {
		TypeCode_ptr tc = exceptions_ -> item(i);
		if(strcmp(tc -> id(), id) == 0)
		{
		    UnknownUserException* ex = new UnknownUserException;
		    Any& any = ex -> exception();
		    in -> read_any_NoTypeCode(any, tc);
		    marshalStub_ -> _OB_setUserException(downcall_, ex, id);
		    break;
		}
	    }
	}
	catch(const SystemException& ex)
	{
	    marshalStub_ -> _OB_unmarshalEx(downcall_, ex);
	}
	marshalStub_ -> _OB_postUnmarshal(downcall_);
    }
    else
    {
	try
	{
	    Any* any = result_ -> value();
	    TypeCode_var tc = any -> type();
	    in -> read_any_NoTypeCode(*any, tc);

	    for(ULong i = 0 ; i < arguments_ -> count() ; i++)
	    {
		NamedValue_ptr nv = arguments_ -> item(i);
		// Note: Don't use != ARG_IN here, because flags can
		// also have values other than ARG_IN, ARG_OUT, or
		// ARG_INOUT
		if(nv -> flags() == ARG_OUT || nv -> flags() == ARG_INOUT)
		{
		    any = nv -> value();
		    tc = any -> type();
		    in -> read_any_NoTypeCode(*any, tc);
		}
	    }
	}
	catch(const SystemException& ex)
	{
	    marshalStub_ -> _OB_unmarshalEx(downcall_, ex);
	}
	marshalStub_ -> _OB_postUnmarshal(downcall_);
    }
}

// ----------------------------------------------------------------------
// Request constructor and destructor
// ----------------------------------------------------------------------

Request::Request(OBCORBA::Object_ptr target,
		 const char* operation,
		 NVList_ptr arguments,
		 NamedValue_ptr result,
		 ExceptionList_ptr exceptions,
		 ContextList_ptr contexts)
    : target_(OBCORBA::Object::_duplicate(target)),
      operation_(operation),
      arguments_(NVList::_duplicate(arguments)),
      result_(NamedValue::_duplicate(result)),
      environment_(new Environment),
      exceptions_(ExceptionList::_duplicate(exceptions)),
      contexts_(ContextList::_duplicate(contexts)),
      state_(RequestStateUnsent),
      pollable_(false),
      polling_(false)
{
}

Request::Request(OBCORBA::Object_ptr target,
		 const char* operation,
		 NVList_ptr arguments,
		 NamedValue_ptr result)
    : target_(OBCORBA::Object::_duplicate(target)),
      operation_(operation),
      arguments_(NVList::_duplicate(arguments)),
      result_(NamedValue::_duplicate(result)),
      environment_(new Environment),
      exceptions_(new ExceptionList),
      contexts_(new ContextList),
      state_(RequestStateUnsent),
      pollable_(false),
      polling_(false)
{
}

Request::Request(OBCORBA::Object_ptr target, const char* operation)
    : target_(OBCORBA::Object::_duplicate(target)),
      operation_(operation),
      arguments_(new NVList),
      result_(new NamedValue(string_dup(""), new Any, ARG_OUT)),
      environment_(new Environment),
      exceptions_(new ExceptionList),
      contexts_(new ContextList),
      state_(RequestStateUnsent),
      pollable_(false),
      polling_(false)
{
}

Request::~Request()
{
    if(state_ == RequestStateSent)
    {
	//
	// TODO: This is a memory leak, as a request has been
	// sent, but the response has never been picked
	// up. The correct thing would be to tell the Downcall
	// object to cancel the request. But we don't have
	// this ability yet.
	//
    }

    //
    // Find out whether this was a deferred request for which
    // get_response() hasn't been called yet.
    //
    ORBInstance_var orbInstance = target_ -> _OB_ORBInstance();
    MultiRequestSender_var multi = orbInstance -> getMultiRequestSender();
    if(!is_nil(multi)) // It might be possible that the
	               // MultiRequestSender is already destroyed
    {
	//
	// Remove this request from the list of the outstanding
	// deferred requests
	//
	multi -> removeDeferredRequest(this);
    }
}

// ----------------------------------------------------------------------
// Request public member implementation
// ----------------------------------------------------------------------

Object_ptr
Request::target() const
{
    return target_;
}

const char*
Request::operation() const
{
    return operation_;
}

NVList_ptr
Request::arguments()
{
    return arguments_;
}

NamedValue_ptr
Request::result()
{
    return result_;
}

Environment_ptr
Request::env()
{
    return environment_;
}

ExceptionList_ptr
Request::exceptions()
{
    return exceptions_;
}

ContextList_ptr
Request::contexts()
{
    return contexts_;
}

void
Request::ctx(Context_ptr ctx)
{
    ctx_ = Context::_duplicate(ctx);
}

Context_ptr
Request::ctx() const
{
    return ctx_;
}

Any&
Request::add_in_arg()
{
    NamedValue_ptr p = arguments_ -> add(ARG_IN);
    return *(p -> value());
}

Any&
Request::add_in_arg(const char* name)
{
    assert_nca(name != 0, NCANullString);
    NamedValue_ptr p = arguments_ -> add_item(name, ARG_IN);
    return *(p -> value());
}

Any&
Request::add_inout_arg()
{
    NamedValue_ptr p = arguments_ -> add(ARG_INOUT);
    return *(p -> value());
}

Any&
Request::add_inout_arg(const char* name)
{
    assert_nca(name != 0, NCANullString);
    NamedValue_ptr p = arguments_ -> add_item(name, ARG_INOUT);
    return *(p -> value());
}

Any&
Request::add_out_arg()
{
    NamedValue_ptr p = arguments_ -> add(ARG_OUT);
    return *(p -> value());
}

Any&
Request::add_out_arg(const char* name)
{
    assert_nca(name != 0, NCANullString);
    NamedValue_ptr p = arguments_ -> add_item(name, ARG_OUT);
    return *(p -> value());
}

void
Request::set_return_type(TypeCode_ptr tc)
{
    result_ -> value() -> replace(tc, 0);
}

Any&
Request::return_value()
{
    return *(result_ -> value());
}

void
Request::invoke()
{
    {
        JTCSynchronized synchronized(stateMutex_);

        if(state_ == RequestStateDone)
            throw BAD_INV_ORDER(MinorRequestAlreadySent, COMPLETED_NO);
        else if(state_ != RequestStateUnsent)
            throw BAD_INV_ORDER(MinorDuplicateSend, COMPLETED_NO);

        state_ = RequestStatePending;
    }
  
    assert(is_nil(marshalStub_));
    assert(is_nil(downcall_));
  
    try
    {
	ULong retry = 0, hop = 0;
	while(true)
	{
	    try
	    {
	        marshalStub_ = target_ -> _OB_getMarshalStubImpl();

		while(true)
		{
		    downcall_ = marshalStub_ ->
			_OB_createPIDIIDowncall(operation_,
						true,
						arguments_,
						result_,
						exceptions_);
		    try
		    {
			marshal();
			marshalStub_ -> _OB_request(downcall_);
			unmarshal();
                        {
                            JTCSynchronized synchronized(stateMutex_);
                            state_ = RequestStateDone;
                        }
			return;
		    }
		    catch(const OB::FailureException& ex)
		    {
			marshalStub_ -> _OB_handleFailureException(downcall_,
								   ex);
		    }
		} // while(true)
	    }
	    catch(const ExceptionBase& ex)
	    {
		target_ -> _OB_handleException(ex, retry, hop);
	    }
	} // while(true)
    }
    catch(const CORBA::Exception& ex)
    {
	environment_ -> exception(ex._OB_clone());
        {
            JTCSynchronized synchronized(stateMutex_);
            state_ = RequestStateDone;
        }
	return;
    }
}

void
Request::send_oneway()
{
    {
        JTCSynchronized synchronized(stateMutex_);

        if(state_ == RequestStateDone)
            throw BAD_INV_ORDER(MinorRequestAlreadySent, COMPLETED_NO);
        else if(state_ != RequestStateUnsent)
            throw BAD_INV_ORDER(MinorDuplicateSend, COMPLETED_NO);

        state_ = RequestStatePending;
    }

    assert(is_nil(marshalStub_));
    assert(is_nil(downcall_));
    
    try
    {
	ULong retry = 0, hop = 0;
	while(true)
	{
	    try
	    {
                marshalStub_ = target_ -> _OB_getMarshalStubImpl();
	    
		while(true)
		{
		    downcall_ = marshalStub_ ->
			_OB_createPIDIIDowncall(operation_,
						false,
						arguments_,
						result_,
						exceptions_);

		    try
		    {
			marshal();
			marshalStub_ -> _OB_oneway(downcall_);
			unmarshal();
                        {
                            JTCSynchronized synchronized(stateMutex_);
                            state_ = RequestStateDone;
                        }
			return;
		    }
		    catch(const OB::FailureException& ex)
		    {
			marshalStub_ -> _OB_handleFailureException(downcall_,
								   ex);
		    }
		} // while(true)
	    }
	    catch(const ExceptionBase& ex)
	    {
		target_ -> _OB_handleException(ex, retry, hop);
	    }
	} // while(true)
    }
    catch(const CORBA::Exception& ex)
    {
	environment_ -> exception(ex._OB_clone());
        {
            JTCSynchronized synchronized(stateMutex_);
            state_ = RequestStateDone;
        }
	return;
    }
}

void
Request::send_deferred()
{
    {
        JTCSynchronized synchronized(stateMutex_);

        if(state_ == RequestStateDone)
            throw BAD_INV_ORDER(MinorRequestAlreadySent, COMPLETED_NO);
        else if(state_ != RequestStateUnsent)
            throw BAD_INV_ORDER(MinorDuplicateSend, COMPLETED_NO);

        state_ = RequestStatePending;
        pollable_ = true;
    }

    assert(is_nil(marshalStub_));
    assert(is_nil(downcall_));

    ORBInstance_var orbInstance = target_ -> _OB_ORBInstance();
    MultiRequestSender_var multi = orbInstance -> getMultiRequestSender();

    try
    {
	ULong retry = 0, hop = 0;
	while(true)
	{
	    try
	    {
                marshalStub_ = target_ -> _OB_getMarshalStubImpl();
	    
		while(true)
		{
		    downcall_ = marshalStub_ ->
			_OB_createPIDIIDowncall(operation_,
						true,
						arguments_,
						result_,
						exceptions_);
		    try
		    {
			marshal();
			marshalStub_ -> _OB_deferred(downcall_);
                        {
                            JTCSynchronized synchronized(stateMutex_);
                            multi -> addDeferredRequest(this);
                            state_ = RequestStateSent;
                        }
			return;
		    }
		    catch(const OB::FailureException& ex)
		    {
			marshalStub_ -> _OB_handleFailureException(downcall_,
								   ex);
		    }
		} // while(true)
	    }
	    catch(const ExceptionBase& ex)
	    {
		target_ -> _OB_handleException(ex, retry, hop);
	    }
	} // while(true)
    }
    catch(const CORBA::Exception& ex)
    {
	environment_ -> exception(ex._OB_clone());
        {
            JTCSynchronized synchronized(stateMutex_);
            multi -> addDeferredRequest(this);
            state_ = RequestStateReceived;
        }
	return;
    }
}

void
Request::get_response()
{
    ORBInstance_var orbInstance = target_ -> _OB_ORBInstance();
    MultiRequestSender_var multi = orbInstance -> getMultiRequestSender();

    {
        JTCSynchronized synchronized(stateMutex_);

        if(!pollable_)
            throw BAD_INV_ORDER(MinorSynchronousRequest, COMPLETED_NO);

        switch(state_)
        {
        case RequestStateUnsent:
        case RequestStatePending:
            throw BAD_INV_ORDER(MinorRequestNotSent, COMPLETED_NO);
        case RequestStateSent:
            break;
        case RequestStateReceiving:
        case RequestStateDone:
            throw BAD_INV_ORDER(MinorResponseAlreadyReceived, COMPLETED_NO);
        case RequestStateReceived:
            multi -> removeDeferredRequest(this);
            state_ = RequestStateDone;
            return;
        }

        //
        // Allow thread calling poll_response to complete
        //
        while(polling_)
        {
            try
            {
                stateMutex_.wait();
            }
            catch(const JTCInterruptedException&)
            {
            }
        }

        if(state_ == RequestStateReceived)
        {
            multi -> removeDeferredRequest(this);
            state_ = RequestStateDone;
            return;
        }
        else
        {
            assert(state_ == RequestStateSent);
            multi -> removeDeferredRequest(this);
            state_ = RequestStateReceiving;
        }
    }

    assert(!is_nil(marshalStub_)); 
    assert(!is_nil(downcall_));

    bool send = false;
    try
    {
	ULong retry = 0, hop = 0;
	while(true)
	{
	    try
	    {
                if(send)
                    marshalStub_ = target_ -> _OB_getMarshalStubImpl();
	    
		while(true)
		{
		    if(send)
		    {
			downcall_ = marshalStub_ ->
			    _OB_createPIDIIDowncall(operation_,
						    true,
						    arguments_,
						    result_,
						    exceptions_);
		    }

		    try
		    {
			if(send)
			{
			    marshal();
			    marshalStub_ -> _OB_deferred(downcall_);
			    send = false;
			}

			marshalStub_ -> _OB_response(downcall_);
			unmarshal();
                        {
                            JTCSynchronized synchronized(stateMutex_);
                            state_ = RequestStateDone;
                        }
			return;
		    }
		    catch(const OB::FailureException& ex)
		    {
			marshalStub_ -> _OB_handleFailureException(downcall_,
								   ex);
			send = true;
		    }
		} // while(true)
	    }
	    catch(const ExceptionBase& ex)
	    {
		target_ -> _OB_handleException(ex, retry, hop);
		send = true;
	    }
	} // while(true)
    }
    catch(const CORBA::Exception& ex)
    {
	environment_ -> exception(ex._OB_clone());
        {
            JTCSynchronized synchronized(stateMutex_);
            state_ = RequestStateDone;
        }
	return;
    }
}

Boolean
Request::poll_response()
{
    {
        JTCSynchronized synchronized(stateMutex_);

        if(!pollable_)
            throw BAD_INV_ORDER(MinorSynchronousRequest, COMPLETED_NO);
        if(polling_)
            return false; // poll_response in progress

        switch(state_)
        {
        case RequestStateUnsent:
        case RequestStatePending:
            throw BAD_INV_ORDER(MinorRequestNotSent, COMPLETED_NO);
        case RequestStateSent:
            break;
        case RequestStateReceiving: // get_response in progress
            return false;
        case RequestStateReceived:
            return true;
        case RequestStateDone:
            throw BAD_INV_ORDER(MinorResponseAlreadyReceived, COMPLETED_NO);
        }

        polling_ = true;
    }

    assert(!is_nil(marshalStub_));
    assert(!is_nil(downcall_));

    bool send = false;
    try
    {
	ULong retry = 0, hop = 0;
	while(true)
	{
	    try
	    {
                if(send)
                    marshalStub_ = target_ -> _OB_getMarshalStubImpl();

		while(true)
		{
		    if(send)
		    {
			downcall_ = marshalStub_ ->
			    _OB_createPIDIIDowncall(operation_,
						    true,
						    arguments_,
						    result_,
						    exceptions_);
		    }

		    try
		    {
			if(send)
			{
			    marshal();
			    marshalStub_ -> _OB_deferred(downcall_);
			    send = false;
			}

			if(marshalStub_ -> _OB_poll(downcall_))
			{
			    unmarshal();
                            {
                                JTCSynchronized synchronized(stateMutex_);
                                state_ = RequestStateReceived;
                                polling_ = false;
                                stateMutex_.notify();
                            }
			    return true;
			}
			else
			{
                            {
                                JTCSynchronized synchronized(stateMutex_);
                                polling_ = false;
                                stateMutex_.notify();
                            }
			    return false;
			}
		    }
		    catch(const OB::FailureException& ex)
		    {
			marshalStub_ -> _OB_handleFailureException(downcall_,
								   ex);
			send = true;
		    }
		} // while(true)
	    }
	    catch(const ExceptionBase& ex)
	    {
		target_ -> _OB_handleException(ex, retry, hop);
		send = true;
	    }
	} // while(true)
    }
    catch(const CORBA::Exception& ex)
    {
	environment_ -> exception(ex._OB_clone());
        {
            JTCSynchronized synchronized(stateMutex_);
            state_ = RequestStateReceived;
            polling_ = false;
            stateMutex_.notify();
        }
	return true;
    }

    return false; // Some compilers need this
}

bool
Request::_OB_completed() const
{
    JTCSynchronized synchronized(stateMutex_);

    return state_ == RequestStateReceived;
}

// ----------------------------------------------------------------------
// Object public member implementations that are related to the DII
// ----------------------------------------------------------------------

void
OBCORBA::Object::_create_request(Context_ptr ctx,
                                 const char* operation,
                                 NVList_ptr arg_list,
                                 NamedValue_ptr result,
                                 Request_out request,
                                 Flags /*req_flags*/)
{
    assert_nca(operation != 0, NCANullString);
    request = new Request(this, operation, arg_list, result);
    request -> ctx(ctx);
}

void
OBCORBA::Object::_create_request(Context_ptr ctx,
				 const char* operation,
				 NVList_ptr arg_list,
				 NamedValue_ptr result,
				 ExceptionList_ptr exceptions,
				 ContextList_ptr contexts,
				 Request_out request,
				 Flags /*req_flags*/)
{
    assert_nca(operation != 0, NCANullString);
    request = new Request(this, operation, arg_list, result, exceptions,
                          contexts);
    request -> ctx(ctx);
}

Request_ptr
OBCORBA::Object::_request(const char* operation)
{
    assert_nca(operation != 0, NCANullString);
    return new Request(this, operation);
}

// ----------------------------------------------------------------------
// ORB_impl public member implementations that are related to the DII
// ----------------------------------------------------------------------

void
OBCORBA::ORB_impl::create_list(Long n, NVList_out list)
{
    if(n < 0)
	n = 0;

    list = new NVList((ULong)n);
}

void
OBCORBA::ORB_impl::create_operation_list(OperationDef_ptr opDef,
					 NVList_out list)
{
    //
    // Get operation description
    //
    Contained::Description_var d = opDef -> describe();
    const OperationDescription* desc;
    Boolean b = d -> value >>= desc;
    assert(b);

    //
    // Create list
    //
    list = new NVList;
    for(ULong i = 0 ; i < desc -> parameters.length() ; i++)
    {
	const ParameterDescription& par = desc -> parameters[i];

	char* name = string_dup(par.name);

	Any* any = new Any;
	any -> replace(par.type, 0);

	Flags flags;
	switch(par.mode)
	{
	case PARAM_IN:
	    flags = ARG_IN;
	    break;
	    
	case PARAM_OUT:
	    flags = ARG_OUT;
	    break;
	    
	case PARAM_INOUT:
	    flags = ARG_INOUT;
	    break;
	}

	list -> add_value_consume(name, any, flags);
    }
}

void
OBCORBA::ORB_impl::create_named_value(NamedValue_out nmval)
{
    nmval = new NamedValue;
}

void
OBCORBA::ORB_impl::create_exception_list(ExceptionList_out exclist)
{
    exclist = new ExceptionList;
}

void
OBCORBA::ORB_impl::create_context_list(ContextList_out ctxtlist)
{
    ctxtlist = new ContextList;
}
void
OBCORBA::ORB_impl::send_multiple_requests_oneway(
    const CORBA::ORB::RequestSeq& requests)
{
    MultiRequestSender_var multi = orbInstance_ -> getMultiRequestSender();
    multi -> sendMultipleRequestsOneway(requests);
}

void
OBCORBA::ORB_impl::send_multiple_requests_deferred(
    const CORBA::ORB::RequestSeq& requests)
{
    MultiRequestSender_var multi = orbInstance_ -> getMultiRequestSender();
    multi -> sendMultipleRequestsDeferred(requests);
}

Boolean
OBCORBA::ORB_impl::poll_next_response()
{
    MultiRequestSender_var multi = orbInstance_ -> getMultiRequestSender();
    return multi -> pollNextResponse();
}

void
OBCORBA::ORB_impl::get_next_response(Request_out request)
{
    MultiRequestSender_var multi = orbInstance_ -> getMultiRequestSender();
    multi -> getNextResponse(request);
}
