// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#ifndef OB_DOWNCALL_UPCALL_PAIR_H
#define OB_DOWNCALL_UPCALL_PAIR_H

#include <OB/Basic.h>
#include <OB/Except.h>
#include <OB/IOP.h>

#include <OB/RefCounted_fwd.h>
#include <OB/Upcall_fwd.h>
#include <OB/Downcall_fwd.h>

#include <DowncallUpcallPair_fwd.h>

namespace OB
{

//
// The DowncallUpcallPair class
//
class DowncallUpcallPair : public RefCount, public JTCMonitor
{
    //
    // The Upcall
    //
    Upcall_var upcall_;

    //
    // The Downcall
    //
    Downcall_var downcall_;

public:

    DowncallUpcallPair(Downcall_ptr);

    static inline DowncallUpcallPair_ptr _duplicate(DowncallUpcallPair_ptr p)
    { if(p) p -> _OB_incRef(); return p; }
    static inline DowncallUpcallPair_ptr _nil()
    { return 0; }

    //
    // Set the Upcall
    //
    void setUpcall(Upcall_ptr);

    //
    // Wait for a reply. If a reply is available return true, false
    // otherwise.
    //
    bool waitReply(bool, CORBA::Long)
	throw(CORBA::SystemException);

    //
    // Set the reply service context list
    //
    void setReplySCL(const IOP::ServiceContextList&);

    //
    // End the reply
    //
    void endReply();

    //
    // End the user exception
    //
    void endUserException();

    //
    // Set the system exception
    //
    void systemException(const CORBA::SystemException&,
			 const IOP::ServiceContextList&);

    //
    // Set a location forward and the service context list
    //
    void forward(RefCountIOR_ptr, bool, const IOP::ServiceContextList&);
};

} // End namespace OB
namespace CORBA
{

inline void
release(OB::DowncallUpcallPair_ptr p)
{
    if(p)
	p -> _OB_decRef();
}

inline Boolean
is_nil(OB::DowncallUpcallPair_ptr p)
{
    return p == 0;
}

} // End of namespace CORBA

#endif
