// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/DII.h>
#include <OB/MultiRequestSender.h>

using namespace CORBA;
using namespace OB;

// ----------------------------------------------------------------------
// External, non-inline duplicate/release for templates
// ----------------------------------------------------------------------

void
OBDuplicate(OB::MultiRequestSender_ptr p)
{
    if(p)
	p -> _OB_incRef();
}

void
OBRelease(OB::MultiRequestSender_ptr p)
{
    if(p)
	p -> _OB_decRef();
}

// ----------------------------------------------------------------------
// MultiRequestSender public member implementation
// ----------------------------------------------------------------------

void
MultiRequestSender::sendMultipleRequestsOneway(
    const CORBA::ORB::RequestSeq& requests)
{
    //
    // Send all requests oneway
    //
    for(ULong i = 0 ; i < requests.length() ; i++)
	requests[i] -> send_oneway();
}

void
MultiRequestSender::sendMultipleRequestsDeferred(
    const CORBA::ORB::RequestSeq& requests)
{
    //
    // Send all requests deferred
    //
    for(ULong i = 0 ; i < requests.length() ; i++)
	requests[i] -> send_deferred();
}

Boolean
MultiRequestSender::pollNextResponse()
{
    JTCSynchronized synchronized(*this);

    if(deferredRequests_.size() == 0)
        throw BAD_INV_ORDER(MinorRequestNotSent, COMPLETED_NO);

    //
    // Poll all deferred requests
    //
    for(OB_STL::list<CORBA::Request_var>::iterator iter =
            deferredRequests_.begin() ;
        iter != deferredRequests_.end() ;
	iter++)
    {
	if((*iter) -> poll_response())
	    return true;
    }

    return false;
}

void
MultiRequestSender::getNextResponse(Request_out request)
{
    JTCSynchronized synchronized(*this);

    //
    // Try to find a deferred request that has completed already
    //
    for(OB_STL::list<CORBA::Request_var>::iterator iter =
            deferredRequests_.begin() ;
        iter != deferredRequests_.end() ;
	iter++)
    {
	if((*iter) -> _OB_completed())
	{
	    request = Request::_duplicate(*iter);
	    deferredRequests_.erase(iter);
	    return;
	}
    }

    //
    // No completed deferred request. Let's simply get the response of
    // the first request.
    //
    if(deferredRequests_.size() > 0)
    {
        request = Request::_duplicate(deferredRequests_.front());
        request -> get_response();
        return;
    }

    throw BAD_INV_ORDER(MinorRequestNotSent, COMPLETED_NO);
}

bool
MultiRequestSender::findDeferredRequest(Request_ptr request)
{
    JTCSynchronized synchronized(*this);

    for(OB_STL::list<CORBA::Request_var>::iterator iter =
            deferredRequests_.begin() ;
        iter != deferredRequests_.end() ;
	iter++)
    {
	if((*iter).in() == request)
	    return true;
    }

    return false;
}

void
MultiRequestSender::addDeferredRequest(Request_ptr request)
{
    JTCSynchronized synchronized(*this);
    deferredRequests_.push_back(Request::_duplicate(request));
}

void
MultiRequestSender::removeDeferredRequest(Request_ptr request)
{
    JTCSynchronized synchronized(*this);

    for(OB_STL::list<CORBA::Request_var>::iterator iter =
            deferredRequests_.begin() ;
        iter != deferredRequests_.end() ;
	iter++)
    {
	if((*iter).in() == request)
	{
	    deferredRequests_.erase(iter);
	    return;
	}
    }
}
