// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/Any.h>
#include <OB/Except.h>
#include <OB/NamedValue.h>
#include <OB/Bounds.h>

using namespace CORBA;
using namespace OB;

// ----------------------------------------------------------------------
// External, non-inline duplicate/release for templates
// ----------------------------------------------------------------------

void
OBDuplicate(CORBA::NamedValue_ptr p)
{
    if(p)
	p -> _OB_incRef();
}

void
OBRelease(CORBA::NamedValue_ptr p)
{
    if(p)
	p -> _OB_decRef();
}

void
OBDuplicate(CORBA::NVList_ptr p)
{
    if(p)
	p -> _OB_incRef();
}

void
OBRelease(CORBA::NVList_ptr p)
{
    if(p)
	p -> _OB_decRef();
}

// ----------------------------------------------------------------------
// NamedValue constructor and destructor
// ----------------------------------------------------------------------

NamedValue::NamedValue()
    :  name_(string_dup("")), value_(new Any), flags_(0)
{
}

NamedValue::NamedValue(char* n, Any* a, Flags f)
    :  name_(n), value_(a), flags_(f)
{
}

// ----------------------------------------------------------------------
// NamedValue public member implementation
// ----------------------------------------------------------------------

const char*
NamedValue::name() const
{
    return name_;
}

Any*
NamedValue::value() const
{
#ifdef HAVE_NO_MUTABLE
    return ((NamedValue*)this) -> value_.ptr();
#else
    return value_.ptr();
#endif
}

Flags
NamedValue::flags() const
{
    return flags_;
}

// ----------------------------------------------------------------------
// NVList constructor
// ----------------------------------------------------------------------

NVList::NVList(ULong n)
    : namedValues_(n)
{
    //
    // The parameter n is only a "hint" for how many elements the user
    // intends to create with operations such as add_item(). Thus the
    // following code is disabled:
    //
//      namedValues_.length(n);
//      for(ULong i = 0 ; i < n ; i++)
//  	namedValues_[i] = new NamedValue;
}

// ----------------------------------------------------------------------
// NVList public member implementation
// ----------------------------------------------------------------------

ULong
NVList::count() const
{
    return namedValues_.length();
}

NamedValue_ptr
NVList::add(Flags f)
{
    NamedValue_ptr p = new NamedValue(string_dup(""), new Any, f);

    CORBA::ULong namedValuesLength = namedValues_.length();
    namedValues_.length(namedValuesLength + 1);
    namedValues_[namedValuesLength] = p;

    return p;
}

NamedValue_ptr
NVList::add_item(const char* n, Flags f)
{
    assert_nca(n != 0, NCANullString);
    NamedValue_ptr p = new NamedValue(string_dup(n), new Any, f);

    CORBA::ULong namedValuesLength = namedValues_.length();
    namedValues_.length(namedValuesLength + 1);
    namedValues_[namedValuesLength] = p;

    return p;
}

NamedValue_ptr
NVList::add_value(const char* n, const Any& a, Flags f)
{
    assert_nca(n != 0, NCANullString);
    NamedValue_ptr p = new NamedValue(string_dup(n), new Any(a), f);

    CORBA::ULong namedValuesLength = namedValues_.length();
    namedValues_.length(namedValuesLength + 1);
    namedValues_[namedValuesLength] = p;

    return p;
}

NamedValue_ptr
NVList::add_item_consume(char* n, Flags f)
{
    assert_nca(n != 0, NCANullString);
    NamedValue_ptr p = new NamedValue(n, new Any, f);

    CORBA::ULong namedValuesLength = namedValues_.length();
    namedValues_.length(namedValuesLength + 1);
    namedValues_[namedValuesLength] = p;

    return p;
}

NamedValue_ptr
NVList::add_value_consume(char* n, Any* a, Flags f)
{
    assert_nca(n != 0, NCANullString);
    NamedValue_ptr p = new NamedValue(n, a, f);

    CORBA::ULong namedValuesLength = namedValues_.length();
    namedValues_.length(namedValuesLength + 1);
    namedValues_[namedValuesLength] = p;

    return p;
}

NamedValue_ptr
NVList::item(ULong n)
{
    if(n >= namedValues_.length())
        throw Bounds();
    
    return namedValues_[n];
}

void
NVList::remove(ULong n)
{
    if(n >= namedValues_.length())
        throw Bounds();

    OB::ObjSeq< NamedValue, int > newSeq;
    newSeq.length(namedValues_.length());
    
    ULong i, j;
    for(i = 0, j = 0; i < namedValues_.length(); i++, j++) {
        if (i == n) {
	  j--;
	  continue;
	}

	newSeq[j] = namedValues_[i];
    }

    newSeq.length(j);

    namedValues_ = newSeq;
}
