// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OADProcessEndpoint_impl.h>
#include <OBPolicies_impl.h>

#include <sys/types.h>
#ifdef HAVE_UNISTD_H
#   include <unistd.h>
#endif

using namespace CORBA;
using namespace OB;

// ----------------------------------------------------------------------
// ProcessController_impl constructor and destructor
// ----------------------------------------------------------------------

OAD::ProcessEndpoint_impl::ProcessEndpoint_impl(const char* name,
						const char* id,
    						PortableServer::POA_ptr poa,
						OB::ORBControl_ptr orbControl)
    : name_(name),
      id_(id),
      poa_(PortableServer::POA::_duplicate(poa)),
      orbControl_(OB::ORBControl::_duplicate(orbControl))
{
    //
    // Create a PolicyList for RETRY_ALWAYS
    //
    pl_.length(1);
    pl_[0] = new RetryPolicy_impl(RETRY_ALWAYS, 0, 1, false);
}

OAD::ProcessEndpoint_impl::~ProcessEndpoint_impl()
{
}

// ----------------------------------------------------------------------
// ProcessEndpoint_impl public member implementation
// ----------------------------------------------------------------------

//
// Standard IDL to C++ Mapping
//
void
OAD::ProcessEndpoint_impl::reestablish_link(OAD::ProcessEndpointManager_ptr d)
    throw(CORBA::SystemException)
{
    //
    // Set the retry policy on this object
    //
    OAD::ProcessEndpointManager_var manager =
	OAD::ProcessEndpointManager::_narrow(
	    Object_var(d -> _set_policy_overrides(pl_, SET_OVERRIDE)));
    assert(!is_nil(manager));
    
    ProcessEndpoint_var cb = _this();

    IMR::ProcessID pid;
#ifdef WIN32
    pid = GetCurrentProcessId();
#else
    pid = getpid();
#endif

    //
    // Establish a new link with the ProcessEndpointManager
    //
    try
    {
        manager -> establish_link(name_, id_, pid, cb);
    }
    catch(const OAD::AlreadyLinked&)
    {
    }
    catch(const CORBA::SystemException& e)
    {
        String_var msg = string_dup("connect_server failed: ");
        String_var s = e._to_string();
        msg += s;
        //logger -> error(msg);
    }
}

void
OAD::ProcessEndpoint_impl::stop()
    throw(CORBA::SystemException)
{
    orbControl_ -> shutdownServer(false);
}

PortableServer::POA_ptr
OAD::ProcessEndpoint_impl::_default_POA()
{
   return PortableServer::POA::_duplicate(poa_);
}
