// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/Basic.h>
#include <OB/ORBInstance.h>
#include <OB/PIManager.h>
#include <OB/ValueBase.h>
#include <OB/PIIOR.h>
#include <OB/InitialServiceManager.h>
#include <OB/Client.h>
#include <OB/PolicyFactoryManager.h>
#include <OB/ValueFactoryManager.h>
#include <OB/ORBControl.h>
#include <OB/PluginManager.h>
#include <OB/MultiRequestSender.h>
#include <OB/Stream.h>
#include <OB/Context.h>
#include <OB/Environment.h>
#include <OB/RefCounted.h>
#include <OB/DirectStubImpl.h>
#include <OB/CoreTraceLevels.h>
#include <OB/Logger.h>  
#include <OB/Properties.h>
#include <OB/OCI_init.h>
#include <OB/OCI_IIOP_init.h>
#include <OB/seq_converter.h>
#include <OB/IMRORT_impl.h>

#include <ORB_impl.h>
#include <CodeSetUtil.h>
#include <TypeCodeFactory.h>
#include <OCI_impl.h>
#include <POAManagerFactory_impl.h>
#include <DynamicAny_impl.h>
#include <CodecFactory_impl.h>
#include <Dispatch_impl.h>
#include <BootManager_impl.h>
#include <OBORBInitInfo_impl.h>
#include <OBPolicies_impl.h>
#include <Messaging_impl.h>
#include <ParseParams.h>
#include <PIORT_impl.h>
#include <ModuleManager.h>
#include <UnknownException_impl.h>
#include <URL_impl.h>

#if !defined(WIN32) 
# include <sys/time.h>
#endif

using namespace CORBA;
using namespace OB;

//
// All registered ORBInitializers
//
typedef OB_STL::list<PortableInterceptor::ORBInitializer_var> 
    ORBInitializerList;
static ORBInitializerList orbInitializers;

namespace PortableInterceptor
{

void
register_orb_initializer(PortableInterceptor::ORBInitializer_ptr init)
{
    //
    // Append the initializer to the initializer list
    //
    orbInitializers.push_back(
	PortableInterceptor::ORBInitializer::_duplicate(init));
}

} // End PortableInterceptor

//
// The one and only module manager
//
namespace OB
{

//
// TODO: shouldn't this and the pluginManager be owned by ORBInstance?
//
class GlobalModuleManager
{
private:
    static ModuleManager* modManager_;
    static JTCMutex mutex_;
    static int referenceCount_;

public:
    static void
    createModuleManager(Properties_ptr props, 
			Logger_ptr logger, 
			const char* modules)
    {
	mutex_.lock();

	if(referenceCount_++ == 0)
	{
	    //
	    // Create the Module manager
	    //
	    modManager_ = new ModuleManager;
	    if(modules != 0)
		modManager_ -> load(props, logger, modules);
	}

	mutex_.unlock();
    }

    static void
    destroyModuleManager()
    {
	mutex_.lock();

    	assert(referenceCount_ > 0);
    	if(--referenceCount_ == 0)
	{
	    //
	    // Clear the ORB initializers
	    //
	    orbInitializers.clear();

	    delete modManager_;
	    modManager_ = 0;
	}

	mutex_.unlock();
    }

    static bool
    exists()
    {
    	return (modManager_ != 0);
    }
};

JTCMutex GlobalModuleManager::mutex_;
ModuleManager* GlobalModuleManager::modManager_ = 0;
int GlobalModuleManager::referenceCount_ = 0;

//
// On DEC UNIX we must unload the module before main returns. See the
// ModuleManagerDestroyer in ORB_impl::destroy()
//
// static ModuleManagerDestroyer destroyer;

}

//
// The ORB implementation map
//
OBCORBA::ORB_impl::ORBImplMap OBCORBA::ORB_impl::orbImplMap_(7);


// ----------------------------------------------------------------------
// ORB_impl constructor and destructor
// ----------------------------------------------------------------------

OBCORBA::ORB_impl::ORB_impl(int& ac, char**& av, const char* orbId,
			    const char* serverId,
			    const char* serverInstance,
			    Client::ConcModel concModel,
			    CoreTraceLevels_ptr coreTraceLevels,
			    Properties_ptr properties,
			    Logger_ptr logger,
			    Reactor_ptr reactor,
			    ULong nativeCs, ULong nativeWcs, ULong defaultWcs)

    : destroy_(false)
{
    try
    {
	//
	// It's not necessary to lock the orbImplMap here since ORB_init
	// locks the map during ORB creation
	//
	//JTCSynchronized orbImplMapSync(orbImplMap_);
	//

	//
	// No mutex protection is necessary since this is within the orb
	// creation mutex.
	//
	const char* modules = properties -> getProperty("ooc.orb.modules");
	GlobalModuleManager::createModuleManager(properties, logger, modules);

	//
	// Create the ORBControl
	//
	orbControl_ = new ORBControl;

        //
        // Create the OCI Plugin Manager
        //
        pluginManager_ = new PluginManager(this);

	//
	// Create the ORBInstance object
	//
	//
	InitialServiceManager_var initServiceManager =
	    new InitialServiceManager;
	ClientManager_var clientManager =
	new ClientManager((Client::ConcModel)concModel);
	ObjectFactory_var objectFactory = new ObjectFactory;
	DirectStubImplFactory_var directStubImplFactory =
	new DirectStubImplFactory;
	PolicyFactoryManager_var pfManager = new PolicyFactoryManager;
	PIManager_var piManager = new PIManager;
	ValueFactoryManager_var valueFactoryManager = new ValueFactoryManager;
	IOP::CodecFactory_impl* codecFactoryImpl = new IOP::CodecFactory_impl;
	IOP::CodecFactory_var codecFactory = codecFactoryImpl;
	OBPortableServer::POAManagerFactory_impl* pmFactoryImpl =
	    new OBPortableServer::POAManagerFactory_impl;
	OBPortableServer::POAManagerFactory_var pmFactory = pmFactoryImpl;
	MultiRequestSender_var multiRequestSender = new MultiRequestSender;
	DispatchStrategyFactory_impl* dsfImpl =
	    new DispatchStrategyFactory_impl;
	DispatchStrategyFactory_var dsf = dsfImpl;
	BootManager_impl* bootManagerImpl = new BootManager_impl;
	BootManager_var bootManager = bootManagerImpl;
	OCI::ConFactoryRegistry_var conFactoryRegistry =
            new OCI::ConFactoryRegistry_impl();
	OCI::AccFactoryRegistry_var accFactoryRegistry =
            new OCI::AccFactoryRegistry_impl();
        UnknownExceptionStrategy_var unknownExceptionStrategy =
            new UnknownExceptionStrategy_impl(this);
        URLRegistry_impl* urlRegistryImpl = new URLRegistry_impl;
        URLRegistry_var urlRegistry = urlRegistryImpl;
	
	orbInstance_ = new ORBInstance(orbId,
				       serverId,
				       serverInstance,
				       objectFactory,
				       directStubImplFactory,
				       clientManager,
				       pfManager,
				       piManager,
				       initServiceManager,
				       valueFactoryManager,
				       codecFactory,
				       pmFactory,
				       multiRequestSender,
				       properties,
				       dsf,
				       bootManager,
				       reactor,
				       logger,
				       coreTraceLevels,
				       conFactoryRegistry,
				       accFactoryRegistry,
                                       unknownExceptionStrategy,
                                       urlRegistry,
				       nativeCs,
				       nativeWcs,
				       defaultWcs);

	objectFactory -> setORBInstance(orbInstance_);
	initServiceManager -> setORBInstance(orbInstance_);
	pmFactoryImpl -> _OB_setORBInstance(orbInstance_);
	piManager -> setORBInstance(orbInstance_);
	codecFactoryImpl -> _OB_setORBInstance(orbInstance_);
	clientManager -> setORBInstance(orbInstance_);
	directStubImplFactory -> setORBInstance(orbInstance_);
	dsfImpl -> _OB_setORBInstance(orbInstance_);

        URLScheme_var scheme;
        scheme = new IORURLScheme_impl(orbInstance_);
        urlRegistry -> add_scheme(scheme);
        scheme = new FileURLScheme_impl(false, urlRegistry);
        urlRegistry -> add_scheme(scheme);
        scheme = new FileURLScheme_impl(true, urlRegistry);
        urlRegistry -> add_scheme(scheme);
        scheme = new CorbalocURLScheme_impl(orbInstance_);
        urlRegistry -> add_scheme(scheme);
        scheme = new CorbanameURLScheme_impl(this, urlRegistry);
        urlRegistry -> add_scheme(scheme);

	//
	// Set the ORBInstance on the ORBControl
	//
	orbControl_ -> setORBInstance(orbInstance_);

	//
	// Initialize default policies
	//
	initializeDefaultPolicies();

	//
	// Create the DynamicAny::DynAnyFactory object
	//
	DynamicAny::DynAnyFactory_var dynAnyFactory =
	    new DynamicAny::DynAnyFactory_impl(orbInstance_);

	//
	// Add initial references
	//
	initServiceManager -> addInitialReference("POAManagerFactory",
						  pmFactory);
	initServiceManager -> addInitialReference("DynAnyFactory",
						  dynAnyFactory);
	initServiceManager -> addInitialReference("CodecFactory",
						  codecFactory);
	initServiceManager -> addInitialReference("DispatchStrategyFactory",
						  dsf);
	initServiceManager -> addInitialReference("BootManager",
						  bootManager);
	initServiceManager -> addInitialReference("RootPOA",
						  Object::_nil()); // Dummy
	initServiceManager -> addInitialReference("OCIConFactoryRegistry",
						  conFactoryRegistry);
	initServiceManager -> addInitialReference("OCIAccFactoryRegistry",
						  accFactoryRegistry);
        initServiceManager -> addInitialReference("URLRegistry", urlRegistry);

        //
        // Register initialization function for OCI IIOP plug-in.
        // (It may have already been registered by another ORB.)
        //
        if(OCI::lookup_plugin("iiop") == 0)
            OCI::register_plugin("iiop", (OCI::INIT)OCI_init_iiop);

        //
        // Initialize the OCI client plug-ins
        //
        {
            String_var prop = properties -> getProperty("ooc.oci.client");
            if(prop == 0)
                prop = string_dup("iiop");
            const char* next = prop;
            while(next != 0)
            {
                OB_STL::list<String_var> paramList;
                next = parse_params(next, paramList);
                String_var name = paramList.front();
                paramList.pop_front();
                OCI::ParamSeq params;
                params.length(paramList.size());
                copy_to_seq(paramList.begin(), paramList.end(), params);

                OCI::Plugin_var plugin =
                    pluginManager_ -> initPlugin(name, ac, av);
                if(is_nil(plugin))
                {
                    String_var err = string_dup("OCI client initialization "
                                                "failed for `");
                    err += name;
                    err += "'";
                    throw INITIALIZE(err);
                }
                else
                    plugin -> init_client(params);
            }
        }

        //
        // Initialize the OCI server plug-ins
        //
        {
            String_var prop = properties -> getProperty("ooc.oci.server");
            if(prop == 0)
                prop = string_dup("iiop");

            const char* next = prop;
            while(next != 0)
            {
                OB_STL::list<String_var> paramList;
                next = parse_params(next, paramList);
                String_var name = paramList.front();
                paramList.pop_front();
                OCI::ParamSeq params;
                params.length(paramList.size());
                copy_to_seq(paramList.begin(), paramList.end(), params);

                OCI::Plugin_var plugin =
                    pluginManager_ -> initPlugin(name, ac, av);
                if(is_nil(plugin))
                {
                    String_var err = string_dup("OCI server initialization "
                                                "failed for `");
                    err += name;
                    err += "'";
                    throw INITIALIZE(err);
                }
                else
                    plugin -> init_server(params);
            }
        }

	//
	// Initialize Portable Interceptors - this must be done after
	// installing the OCI plug-ins to allow an ORBInitializer or
	// interceptor to make a remote invocation
	//

        //
        // Install IOR interceptor for code sets
        //
	PortableInterceptor::IORInterceptor_var interceptor =
	    new PortableInterceptor::CodeSetIORInterceptor_impl(
		nativeCs, nativeWcs);

	try
	{
	    piManager -> addIORInterceptor(interceptor);
	}
	catch(const PortableInterceptor::ORBInitInfo::DuplicateName&)
	{
	    assert(false);
	}

	//
	// Register the valuetype factory for the persistent POA Object
	// Reference Template and the IMR Object Reference Template.
	//
	ValueFactoryBase_var ortFactory =
	    new OBPortableInterceptor::TransientORTFactory_impl(orbInstance_);
	valueFactoryManager -> registerValueFactory(
            "IDL:ooc.com/OBPortableInterceptor/TransientORT:1.0",
	    ortFactory);

	ortFactory =
	    new OBPortableInterceptor::PersistentORTFactory_impl(orbInstance_);
	valueFactoryManager -> registerValueFactory(
            "IDL:ooc.com/OBPortableInterceptor/PersistentORT:1.0",
	    ortFactory);

	ortFactory =
	    new OBPortableInterceptor::IMRORTFactory_impl();
	valueFactoryManager -> registerValueFactory(
	    "IDL:ooc.com/OBPortableInterceptor/IMRORT:1.0",
	    ortFactory);

	//
	// Call each of the ORB initializers. If there are no ORB
	// initializers it's not necessary to setup the PIManager
	// since no interceptors will be called.
	//
	if(orbInitializers.size() > 0)
	{
	    OBPortableInterceptor::ORBInitInfo_impl* infoImpl =
                new OBPortableInterceptor::ORBInitInfo_impl(
                    this,
                    ac, av, orbId, 
                    piManager,
                    initServiceManager,
                    codecFactory);

	    OBPortableInterceptor::ORBInitInfo_var info = infoImpl;

	    ORBInitializerList::iterator it;
	    for(it = orbInitializers.begin(); 
	        it != orbInitializers.end();
		++it)
		(*it) -> pre_init(info);

	    //TODO: change state
	    for(it = orbInitializers.begin(); 
	        it != orbInitializers.end();
		++it)
		(*it) -> post_init(info);

	    infoImpl -> _OB_destroy();
	}
	piManager -> setupComplete();

	//
	// Add this ORB to the implementation map -- this must be done
	// last.
	//
	orbImplMap_.put(orbId, ORB::_duplicate(this));
    }
    catch(...)
    {
	//
	// Here the same thing as ORB::destroy must be done since
	// although the ORB itself isn't fully initialized all of the
	// ORB components may be.
	//
	if(GlobalModuleManager::exists())
	{
	    GlobalModuleManager::destroyModuleManager();
	}
	if(!is_nil(orbControl_))
	{
	    orbControl_ -> shutdownServerClient();
	    orbControl_ -> destroy();
	}
        if(!is_nil(pluginManager_))
        {
            pluginManager_ -> destroy();
        }
	if(!is_nil(orbInstance_))
	{
	    orbInstance_ -> destroy();
	    orbInstance_ = ORBInstance::_nil();
	}
	_OB_setRef(0);
	throw;
    }
}

OBCORBA::ORB_impl::~ORB_impl()
{
    //
    // Check whether the ORB has been properly destroyed
    //
    if(!destroy_)
    {
	Logger_var logger = orbInstance_ -> getLogger();
	logger -> warning("ORB::destroy() was not called. "
			  "This may result in memory leaks.");
    }
}

// ----------------------------------------------------------------------
// ORB_impl public member implementation
// ----------------------------------------------------------------------

char*
OBCORBA::ORB_impl::object_to_string(CORBA::Object_ptr p)
{
    {
	JTCSynchronized synchronized(*this);
	if(destroy_)
	    throw OBJECT_NOT_EXIST();
    }

    RefCountIOR_var ior;
    
    if(is_nil(p))
    {
	IOP::IOR empty;
	ior = new RefCountIOR(empty);
    }
    else
    {
	ior = p -> _OB_origIOR(); 
    }

    OCI::BufferImpl buf;
    OutputStreamImpl out(&buf);
    out._OB_writeEndian();
    ior -> value._OB_marshal(&out);
    String_var s = OctetsToAscii(buf.data(), buf.length());
    
    static const char* prefix = "IOR:";
    char* result = string_alloc(strlen(prefix) + strlen(s));
    strcpy(result, prefix);
    strcat(result, s);

    return result;
}

Object_ptr
OBCORBA::ORB_impl::string_to_object(const char* ior)
{
    JTCSynchronized synchronized(*this);
    if(destroy_)
	throw OBJECT_NOT_EXIST();

    ObjectFactory_var objectFactory = orbInstance_ -> getObjectFactory();
    return objectFactory -> stringToObject(ior);
}

/* DII related operations are in DII.cpp */

void
OBCORBA::ORB_impl::get_default_context(Context_out ctx)
{
    JTCSynchronized synchronized(*this);
    if(destroy_)
	throw OBJECT_NOT_EXIST();

    ctx = new Context("");
}

void
OBCORBA::ORB_impl::create_environment(Environment_out new_env)
{
    JTCSynchronized synchronized(*this);
    if(destroy_)
	throw OBJECT_NOT_EXIST();

    new_env = new Environment;
}

CORBA::ORB::ObjectIdList*
OBCORBA::ORB_impl::list_initial_services()
{
    JTCSynchronized synchronized(*this);
    if(destroy_)
	throw OBJECT_NOT_EXIST();

    InitialServiceManager_var initServiceManager =
	orbInstance_ -> getInitialServiceManager();
    return initServiceManager -> listInitialServices();
}

Boolean
OBCORBA::ORB_impl::get_service_information(ServiceType,
					   ServiceInformation_out info)
{
    info = new ServiceInformation;
    return false;
}

Object_ptr
OBCORBA::ORB_impl::resolve_initial_references(const char* identifier)
{
    JTCSynchronized synchronized(*this);
    if(destroy_)
	throw OBJECT_NOT_EXIST();

    InitialServiceManager_var initServiceManager =
        orbInstance_ -> getInitialServiceManager();

    CORBA::Object_var obj;

    try
    {
        obj = initServiceManager -> resolveInitialReferences(identifier);
    }
    catch(const InvalidName&)
    {
        //
        // If the service is the RootPOA and it hasn't yet been
        // initialized, create it. We could put in some automatic
        // method here for late binding of objects at some later
        // point.
        //
        if(strcmp(identifier, "RootPOA") == 0)
        {
            orbControl_ -> initializeRootPOA(this);
            return resolve_initial_references(identifier);
        }
        else
            throw;
    }

    return obj._retn();
}

void
OBCORBA::ORB_impl::register_initial_reference(const char* name,
					      CORBA::Object_ptr obj)
{
    JTCSynchronized synchronized(*this);
    if(destroy_)
	throw OBJECT_NOT_EXIST();

    InitialServiceManager_var initServiceManager =
        orbInstance_ -> getInitialServiceManager();
    initServiceManager -> addInitialReference(name, obj);
}

TypeCode_ptr
OBCORBA::ORB_impl::create_struct_tc(const char* id, const char* name,
			     const StructMemberSeq& members)
{
    JTCSynchronized synchronized(*this);
    if(destroy_)
	throw OBJECT_NOT_EXIST();

    return TypeCodeFactory::createStructTC(id, name, members);
}

TypeCode_ptr
OBCORBA::ORB_impl::create_union_tc(const char* id, const char* name,
			  TypeCode_ptr discriminator_type,
			  const UnionMemberSeq& members)
{
    JTCSynchronized synchronized(*this);
    if(destroy_)
	throw OBJECT_NOT_EXIST();

    return TypeCodeFactory::createUnionTC(id, name, discriminator_type,
					  members);
}

TypeCode_ptr
OBCORBA::ORB_impl::create_enum_tc(const char* id, const char* name,
			 const EnumMemberSeq& members)
{
    JTCSynchronized synchronized(*this);
    if(destroy_)
	throw OBJECT_NOT_EXIST();

    return TypeCodeFactory::createEnumTC(id, name, members);
}

TypeCode_ptr
OBCORBA::ORB_impl::create_alias_tc(const char* id, const char* name,
			  TypeCode_ptr original_type)
{
    JTCSynchronized synchronized(*this);
    if(destroy_)
	throw OBJECT_NOT_EXIST();

    return TypeCodeFactory::createAliasTC(id, name, original_type);
}

TypeCode_ptr
OBCORBA::ORB_impl::create_exception_tc(const char* id, const char* name,
			      const StructMemberSeq& members)
{
    JTCSynchronized synchronized(*this);
    if(destroy_)
	throw OBJECT_NOT_EXIST();

    return TypeCodeFactory::createExceptionTC(id, name, members);
}

TypeCode_ptr
OBCORBA::ORB_impl::create_interface_tc(const char* id, const char* name)
{
    JTCSynchronized synchronized(*this);
    if(destroy_)
	throw OBJECT_NOT_EXIST();

    return TypeCodeFactory::createInterfaceTC(id, name);
}

TypeCode_ptr
OBCORBA::ORB_impl::create_string_tc(ULong bound)
{
    JTCSynchronized synchronized(*this);
    if(destroy_)
	throw OBJECT_NOT_EXIST();

    return TypeCodeFactory::createStringTC(bound);
}

TypeCode_ptr
OBCORBA::ORB_impl::create_wstring_tc(ULong bound)
{
    JTCSynchronized synchronized(*this);
    if(destroy_)
	throw OBJECT_NOT_EXIST();

    return TypeCodeFactory::createWstringTC(bound);
}

TypeCode_ptr
OBCORBA::ORB_impl::create_fixed_tc(UShort digits, Short scale)
{
    JTCSynchronized synchronized(*this);
    if(destroy_)
	throw OBJECT_NOT_EXIST();

    return TypeCodeFactory::createFixedTC(digits, scale);
}

TypeCode_ptr
OBCORBA::ORB_impl::create_sequence_tc(ULong bound, TypeCode_ptr element_type)
{
    JTCSynchronized synchronized(*this);
    if(destroy_)
	throw OBJECT_NOT_EXIST();

    return TypeCodeFactory::createSequenceTC(bound, element_type);
}

TypeCode_ptr
OBCORBA::ORB_impl::create_recursive_sequence_tc(ULong bound, ULong offset)
{
    JTCSynchronized synchronized(*this);
    if(destroy_)
	throw OBJECT_NOT_EXIST();

    return TypeCodeFactory::createRecursiveSequenceTC(bound, offset);
}

TypeCode_ptr
OBCORBA::ORB_impl::create_array_tc(ULong length, TypeCode_ptr element_type)
{
    JTCSynchronized synchronized(*this);
    if(destroy_)
	throw OBJECT_NOT_EXIST();

    return TypeCodeFactory::createArrayTC(length, element_type);
}

TypeCode_ptr
OBCORBA::ORB_impl::create_value_tc(const char* id, const char* name,
			    ValueModifier type_modifier,
			    TypeCode_ptr concrete_base,
			    const ValueMemberSeq& members)
{
    JTCSynchronized synchronized(*this);
    if(destroy_)
	throw OBJECT_NOT_EXIST();

    return TypeCodeFactory::createValueTC(id, name, type_modifier,
					  concrete_base, members);
}

TypeCode_ptr
OBCORBA::ORB_impl::create_value_box_tc(const char* id, const char* name,
			       TypeCode_ptr boxed_type)
{
    JTCSynchronized synchronized(*this);
    if(destroy_)
	throw OBJECT_NOT_EXIST();

    return TypeCodeFactory::createValueBoxTC(id, name, boxed_type);
}

TypeCode_ptr
OBCORBA::ORB_impl::create_native_tc(const char* id, const char* name)
{
    JTCSynchronized synchronized(*this);
    if(destroy_)
	throw OBJECT_NOT_EXIST();

    return TypeCodeFactory::createNativeTC(id, name);
}

TypeCode_ptr
OBCORBA::ORB_impl::create_recursive_tc(const char* id)
{
    JTCSynchronized synchronized(*this);
    if(destroy_)
	throw OBJECT_NOT_EXIST();

    return TypeCodeFactory::createRecursiveTC(id);
}

TypeCode_ptr
OBCORBA::ORB_impl::create_abstract_interface_tc(const char* id,
						const char* name)
{
    JTCSynchronized synchronized(*this);
    if(destroy_)
	throw OBJECT_NOT_EXIST();

    return TypeCodeFactory::createAbstractInterfaceTC(id, name);
}

TypeCode_ptr
OBCORBA::ORB_impl::create_local_interface_tc(const char* id,
					     const char* name)
{
    JTCSynchronized synchronized(*this);
    if(destroy_)
	throw OBJECT_NOT_EXIST();

    return TypeCodeFactory::createLocalInterfaceTC(id, name);
}

Boolean
OBCORBA::ORB_impl::work_pending()
{
    //
    // Ensure that the ORB mutex is not locked during the call to
    // ORBControl methods
    //
    if(destroy_)
	throw OBJECT_NOT_EXIST();

    //
    // Ensure that other threads get a chance to execute if
    // work_pending() is being called in a tight loop.
    //
    JTCThread::sleep(1000);

    return orbControl_ -> workPending();
}

void
OBCORBA::ORB_impl::perform_work()
{
    //
    // Ensure that the ORB mutex is not locked during the call to
    // ORBControl methods
    //
    if(destroy_)
	throw OBJECT_NOT_EXIST();
    orbControl_ -> performWork();
}

void
OBCORBA::ORB_impl::run()
{
    //
    // Ensure that the ORB mutex is not locked during the call to
    // ORBControl methods
    //
    if(destroy_)
	throw OBJECT_NOT_EXIST();
    orbControl_ -> run();
}

//
// ASYNC SAFE if called with wait_for_completion = false
//
void
OBCORBA::ORB_impl::shutdown(Boolean wait_for_completion)
{
    //
    // Ensure that the ORB mutex is not locked during the call to
    // ORBControl methods
    //
    if(destroy_)
	throw OBJECT_NOT_EXIST();
    orbControl_ -> shutdownServer(wait_for_completion);
}

void
OBCORBA::ORB_impl::destroy()
{
    //
    // From the specification:
    //
    // This operation destroys the ORB so that its resources can be
    // reclaimed by the application.  Any operation invoked on a
    // destroyed ORB reference will raise the OBJECT_NOT_EXIST
    // exception.  Once an ORB has been destroyed, another call to
    // ORB_init with the same ORBid will return a reference to a newly
    // constructed ORB.
    //
    // If destroy is called on an ORB that has not been shut down, it
    // will start the shut down process and block until the ORB has
    // shut down before it destroys the ORB.  If an application calls
    // destroy in a thread that is currently servicing an invocation,
    // the BAD_INV_ORDER system exception will be raised with the OMG
    // minor code 3, since blocking would result in a deadlock.
    //
    // For maximum portability and to avoid resource leaks, an
    // application should always call shutdown and destroy on all ORB
    // instances before exiting.
    //
    JTCSynchronized synchronized(*this);

    //
    // Has the ORB been destroyed yet?
    //
    if(destroy_)
	throw OBJECT_NOT_EXIST();

    //
    // Shutdown both the server & client side of the ORB
    //
    orbControl_ -> shutdownServerClient();

    //
    // Destroy the ORBControl. Don't set to _nil.
    //
    orbControl_ -> destroy();
    //orbControl_ = ORBControl::_nil();

    CORBA::String_var orbId = orbInstance_ -> getOrbId();

    //
    // Destroy the ORBInstance object
    //
    orbInstance_ -> destroy();
    orbInstance_ = ORBInstance::_nil();

    //
    // Destroy the OCI Plugin Manager. This must be done after all
    // the OCI objects have been destroyed.
    //
    pluginManager_ -> destroy();
    pluginManager_ = PluginManager::_nil();

    //
    // Unload any modules
    //
    GlobalModuleManager::destroyModuleManager();

    //
    // Mark the ORB as destroyed
    //
    destroy_ = true;

    //
    // Remove the ORB from the implementation map
    //
    orbImplMap_.remove(orbId);
}

Policy_ptr
OBCORBA::ORB_impl::create_policy(PolicyType type, const Any& any)
{
    JTCSynchronized synchronized(*this);
    if(destroy_)
	throw OBJECT_NOT_EXIST();

    PolicyFactoryManager_var manager =
	orbInstance_ -> getPolicyFactoryManager();
    return manager -> createPolicy(type, any);
}

ValueFactory
OBCORBA::ORB_impl::register_value_factory(const char* id, ValueFactory factory)
{
    JTCSynchronized synchronized(*this);
    if(destroy_)
	throw OBJECT_NOT_EXIST();

    ValueFactoryManager_var manager = orbInstance_ -> getValueFactoryManager();
    return manager -> registerValueFactory(id, factory);
}

void
OBCORBA::ORB_impl::unregister_value_factory(const char* id)
{
    JTCSynchronized synchronized(*this);
    if(destroy_)
	throw OBJECT_NOT_EXIST();

    ValueFactoryManager_var manager = orbInstance_ -> getValueFactoryManager();
    manager -> unregisterValueFactory(id);
}

ValueFactory
OBCORBA::ORB_impl::lookup_value_factory(const char* id)
{
    JTCSynchronized synchronized(*this);
    if(destroy_)
	throw OBJECT_NOT_EXIST();

    ValueFactoryManager_var manager = orbInstance_ -> getValueFactoryManager();
    return manager -> lookupValueFactory(id);
}

OB::PropertySet
OBCORBA::ORB_impl::properties()
{
    JTCSynchronized synchronized(*this);
    if(destroy_)
	throw OBJECT_NOT_EXIST();

    return orbInstance_ -> getProperties();
}

OB::Logger_ptr
OBCORBA::ORB_impl::logger()
{
    JTCSynchronized synchronized(*this);
    if(destroy_)
	throw OBJECT_NOT_EXIST();

    return orbInstance_ -> getLogger();
}

OB::Reactor_ptr
OBCORBA::ORB_impl::reactor()
{
    JTCSynchronized synchronized(*this);
    if(destroy_)
	throw OBJECT_NOT_EXIST();

    return orbInstance_ -> getReactor();
}

OB::UnknownExceptionStrategy_ptr
OBCORBA::ORB_impl::set_unknown_exception_strategy(
    OB::UnknownExceptionStrategy_ptr strategy)
{
    JTCSynchronized synchronized(*this);
    if(destroy_)
        throw OBJECT_NOT_EXIST();

    return orbInstance_ -> setUnknownExceptionStrategy(strategy);
}

OB::ConnectionReusePolicy_ptr
OBCORBA::ORB_impl::create_connection_reuse_policy(CORBA::Boolean value)
{
    return new OB::ConnectionReusePolicy_impl(value);
}

OB::ProtocolPolicy_ptr
OBCORBA::ORB_impl::create_protocol_policy(const OCI::PluginIdSeq& value)
{
    return new OB::ProtocolPolicy_impl(value);
}

OB::RetryPolicy_ptr
OBCORBA::ORB_impl::create_retry_policy(CORBA::Short mode,
				       CORBA::ULong interval,
				       CORBA::ULong max,
				       CORBA::Boolean remote)
{
    return new OB::RetryPolicy_impl(mode, interval, max, remote);
}

OB::TimeoutPolicy_ptr
OBCORBA::ORB_impl::create_timeout_policy(CORBA::ULong value)
{
    return new OB::TimeoutPolicy_impl(value);
}

OB::LocationTransparencyPolicy_ptr
OBCORBA::ORB_impl::create_location_transparency_policy(CORBA::Short value)
{
    return new OB::LocationTransparencyPolicy_impl(value);
}

Messaging::RebindPolicy_ptr
OBCORBA::ORB_impl::create_rebind_policy(Messaging::RebindMode value)
{
    return new Messaging::RebindPolicy_impl(value);
}

Messaging::SyncScopePolicy_ptr
OBCORBA::ORB_impl::create_sync_scope_policy(Messaging::SyncScope value)
{
    return new Messaging::SyncScopePolicy_impl(value);
}

Messaging::RequestStartTimePolicy_ptr
OBCORBA::ORB_impl::create_request_start_time_policy(TimeBase::UtcT value)
{
    return new Messaging::RequestStartTimePolicy_impl(value);
}

Messaging::RequestEndTimePolicy_ptr
OBCORBA::ORB_impl::create_request_end_time_policy(TimeBase::UtcT value)
{
    return new Messaging::RequestEndTimePolicy_impl(value);
}

Messaging::ReplyStartTimePolicy_ptr
OBCORBA::ORB_impl::create_reply_start_time_policy(TimeBase::UtcT value)
{
    return new Messaging::ReplyStartTimePolicy_impl(value);
}

Messaging::ReplyEndTimePolicy_ptr
OBCORBA::ORB_impl::create_reply_end_time_policy(TimeBase::UtcT value)
{
    return new Messaging::ReplyEndTimePolicy_impl(value);
}

Messaging::RelativeRequestTimeoutPolicy_ptr
OBCORBA::ORB_impl::create_relative_request_timeout_policy(
    TimeBase::TimeT value)
{
    return new Messaging::RelativeRequestTimeoutPolicy_impl(value);
}

Messaging::RelativeRoundTripTimeoutPolicy_ptr
OBCORBA::ORB_impl::create_relative_round_trip_timeout_policy(
    TimeBase::TimeT value)
{
    return new Messaging::RelativeRoundTripTimeoutPolicy_impl(value);
}

OB::InterceptorPolicy_ptr
OBCORBA::ORB_impl::create_interceptor_policy(CORBA::Boolean value)
{
    return new OB::InterceptorPolicy_impl(value);
}

OB::ConnectTimeoutPolicy_ptr
OBCORBA::ORB_impl::create_connection_timeout_policy(CORBA::ULong value)
{
    return new OB::ConnectTimeoutPolicy_impl(value);
}

OB::RequestTimeoutPolicy_ptr
OBCORBA::ORB_impl::create_request_timeout_policy(CORBA::ULong value)
{
    return new OB::RequestTimeoutPolicy_impl(value);
}

// ----------------------------------------------------------------------
// Get an ORBInstance, given an ORB_ptr
// ----------------------------------------------------------------------

ORBInstance_ptr
OB::GetORBInstance(CORBA::ORB_ptr p)
{
    if(is_nil(p))
	return ORBInstance::_nil();
    
    OBCORBA::ORB_impl* impl = dynamic_cast<OBCORBA::ORB_impl*>(p);
    if(!impl)
	throw INITIALIZE();
    
    return ORBInstance::_duplicate(impl -> orbInstance_);
}
