// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#ifndef PI_CLIENT_IMPL_H
#define PI_CLIENT_IMPL_H

#include <OB/PIClient.h>

#include <PIRequest_impl.h>

namespace PortableInterceptor
{

//
// Typedef for sequence of ClientRequestInterceptor
//
class OBUnique_ClientRequestInterceptorSeq;
typedef OB::ObjSeq< ClientRequestInterceptor,
    OBUnique_ClientRequestInterceptorSeq > ClientRequestInterceptorSeq;

//
// Implementation of ClientRequestInfo
//
class ClientRequestInfo_impl :
    public virtual ClientRequestInfo,
    public RequestInfo_impl
{
    //
    // Hide copy constructor and assignment operator
    //
    ClientRequestInfo_impl(const ClientRequestInfo_impl&);
    void operator=(const ClientRequestInfo_impl&);

    //
    // Sequence of ClientRequestInterceptors to call on reply
    //
    ClientRequestInterceptorSeq interceptors_;

    //
    // The effective IOR
    //
    OB::RefCountIOR_var IOR_;

    //
    // The original IOR
    //
    OB::RefCountIOR_var origIOR_;

    //
    // The ProfileInfo
    //
    OB::RefCountProfileInfo_var profileInfo_;

    //
    // Slot data for the request PICurrent
    //
    CORBA::AnySeq_var currentSlots_;

public:

    //
    // No arguments
    //
    ClientRequestInfo_impl(CORBA::ULong, const char*, bool,
			   OB::RefCountIOR_ptr,
			   OB::RefCountIOR_ptr,
			   OB::RefCountProfileInfo_ptr,
			   OB::RefCountPolicyList_ptr,
                           IOP::ServiceContextList&,
                           IOP::ServiceContextList&,
                           OB::ORBInstance_ptr,
                           Current_impl*);

    //
    // DII style
    //
    ClientRequestInfo_impl(CORBA::ULong, const char*, bool,
			   OB::RefCountIOR_ptr,
			   OB::RefCountIOR_ptr,
			   OB::RefCountProfileInfo_ptr,
			   OB::RefCountPolicyList_ptr,
                           IOP::ServiceContextList&,
                           IOP::ServiceContextList&,
                           OB::ORBInstance_ptr,
			   Current_impl*,
                           CORBA::NVList_ptr,
                           CORBA::NamedValue_ptr,
                           CORBA::ExceptionList_ptr);

    //
    // SII style
    //
    ClientRequestInfo_impl(CORBA::ULong, const char*, bool,
			   OB::RefCountIOR_ptr,
			   OB::RefCountIOR_ptr,
			   OB::RefCountProfileInfo_ptr,
			   OB::RefCountPolicyList_ptr,
                           IOP::ServiceContextList&,
                           IOP::ServiceContextList&,
                           OB::ORBInstance_ptr,
			   Current_impl*,
                           OB::ParameterDesc*, CORBA::ULong,
                           OB::ParameterDesc*,
                           CORBA::TypeCode_ptr*, CORBA::ULong);

    ~ClientRequestInfo_impl();

    //
    // IDL to C++ Mapping
    //
    virtual CORBA::Object_ptr target()
	throw(CORBA::SystemException);

    virtual CORBA::Object_ptr effective_target()
	throw(CORBA::SystemException);

    virtual IOP::TaggedProfile* effective_profile()
	throw(CORBA::SystemException);
    
    virtual CORBA::Any* received_exception()
	throw(CORBA::SystemException);

    virtual char* received_exception_id()
	throw(CORBA::SystemException);

    virtual IOP::TaggedComponent* get_effective_component(IOP::ComponentId)
	throw(CORBA::SystemException);

    virtual IOP::TaggedComponentSeq*
    get_effective_components(IOP::ComponentId)
	throw(CORBA::SystemException);

    virtual CORBA::Policy_ptr get_request_policy (CORBA::PolicyType)
	throw(CORBA::SystemException);

    virtual void add_request_service_context (const IOP::ServiceContext&,
                                              CORBA::Boolean)
	throw(CORBA::SystemException);

    //
    // ORBacus internal functions
    // Application programs must not use these functions directly
    //
    void _OB_request(const ClientRequestInterceptorSeq&)
	throw(CORBA::SystemException, OB::LocationForward);

    void _OB_reply()
	throw(CORBA::SystemException, OB::LocationForward);
};

} // End of namespace PortableInterceptor

#endif
