// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/Basic.h>
#include <OB/Object.h>
#include <OB/Any.h> // Needed by gcc
#include <OB/OCI.h>
#include <OB/ORBInstance.h>
#include <OB/RefCounted.h>
#include <OB/ObjectKey.h>
#include <OB/OBObject.h> // For ObjectFactory

#include <PIORT_impl.h>

using namespace OB;
using namespace CORBA;

// ----------------------------------------------------------------------
// OBPortableInterceptor::TransientORT_impl
// constructor and destructor
// ----------------------------------------------------------------------

OBPortableInterceptor::TransientORT_impl::TransientORT_impl(
    ORBInstance_ptr orbInstance)
    : orbInstance_(ORBInstance::_duplicate(orbInstance))
{
}


OBPortableInterceptor::TransientORT_impl::TransientORT_impl(
    ORBInstance_ptr orbInstance,
    const char* serverId,
    const char* orbId,
    const PortableInterceptor::AdapterName& adapterName,
    time_t createTime,
    const IOP::IOR& iorTemplate)
    : orbInstance_(ORBInstance::_duplicate(orbInstance))
{
    the_server_id(serverId);
    the_orb_id(orbId);
    the_adapter_name(adapterName);
    the_create_time(createTime);
    the_ior_template(iorTemplate);
}

OBPortableInterceptor::TransientORT_impl::~TransientORT_impl()
{
}
// ----------------------------------------------------------------------
// OBPortableInterceptor::TransientORT_impl private member implementation
// ----------------------------------------------------------------------

Object_ptr
OBPortableInterceptor::TransientORT_impl::makeObject(
    const char* repoid,
    const PortableInterceptor::ObjectId& id,
    const PortableInterceptor::AdapterName& name)
    throw(SystemException)
{
    //
    // Create the actual ORBacus ObjectKey
    //
    // TODO:
    // CreatePersistentObjectKey/CreateTransientObjectKey instead of
    // populating this ObjectKey data to avoid the copy?
    //
    OB::ObjectKeyData obkey;
    obkey.serverId = the_server_id();
    if(strlen(obkey.serverId) == 0)
        obkey.serverId = string_dup("_RootPOA");
    obkey.poaId = name;
    obkey.oid.length(id.length());
    memcpy(obkey.oid.get_buffer(), id.get_buffer(), id.length());
    obkey.persistent = false;
    obkey.createTime = the_create_time();

    OCI::ObjectKey_var key = CreateObjectKey(obkey);

    IOP::IOR_var ior = new IOP::IOR(the_ior_template());
    ior -> type_id = repoid;

    OCI::AccFactoryRegistry_var registry =
        orbInstance_ -> getAccFactoryRegistry();
    OCI::AccFactorySeq_var factories = registry -> get_factories();
    for(ULong i = 0 ; i < factories -> length() ; ++i)
	factories[i] -> change_key(ior.inout(), key);

    RefCountIOR_var refior = new RefCountIOR(ior);
    
    //
    // Create and return reference
    //
    ObjectFactory_var objectFactory = orbInstance_ -> getObjectFactory();
    return objectFactory -> createObject(refior);
}

// ----------------------------------------------------------------------
// OBPortableInterceptor::TransientORT_impl public member implementation
// ----------------------------------------------------------------------

ValueBase*
OBPortableInterceptor::TransientORT_impl::_copy_value()
    throw(SystemException)
{
    TransientORT_impl* r =
	new TransientORT_impl(orbInstance_, the_server_id(), the_orb_id(),
			      the_adapter_name(), the_create_time(),
			      the_ior_template());
    return r;
}

PortableInterceptor::ServerId
OBPortableInterceptor::TransientORT_impl::server_id()
    throw(SystemException)
{
    return string_dup(the_server_id());
}

PortableInterceptor::ORBId
OBPortableInterceptor::TransientORT_impl::orb_id()
    throw(SystemException)
{
    return string_dup(the_orb_id());
}

PortableInterceptor::AdapterName*
OBPortableInterceptor::TransientORT_impl::adapter_name()
    throw(SystemException)
{
    return new PortableInterceptor::AdapterName(the_adapter_name());
}

Object_ptr
OBPortableInterceptor::TransientORT_impl::make_object(
    const char* repoid,
    const PortableInterceptor::ObjectId& id)
    throw(SystemException)
{
    return makeObject(repoid, id, the_adapter_name());
}

Object_ptr
OBPortableInterceptor::TransientORT_impl::make_object_for(
    const char* repoid,
    const PortableInterceptor::ObjectId& id,
    const PortableInterceptor::AdapterName& name)
    throw(SystemException)
{
    return makeObject(repoid, id, name);
}

// ----------------------------------------------------------------------
// OBPortableInterceptor::PersistentORT_impl
// constructor and destructor
// ----------------------------------------------------------------------

OBPortableInterceptor::PersistentORT_impl::PersistentORT_impl(
    ORBInstance_ptr orbInstance)
    : orbInstance_(ORBInstance::_duplicate(orbInstance))
{
}


OBPortableInterceptor::PersistentORT_impl::PersistentORT_impl(
    ORBInstance_ptr orbInstance,
    const char* serverId,
    const char* orbId,
    const PortableInterceptor::AdapterName& adapterName,
    const IOP::IOR& iorTemplate)
    : orbInstance_(ORBInstance::_duplicate(orbInstance))
{
    the_server_id(serverId);
    the_orb_id(orbId);
    the_adapter_name(adapterName);
    the_ior_template(iorTemplate);
}


OBPortableInterceptor::PersistentORT_impl::~PersistentORT_impl()
{
}

// ----------------------------------------------------------------------
// PersistentORT_impl private member implementation
// ----------------------------------------------------------------------

Object_ptr
OBPortableInterceptor::PersistentORT_impl::makeObject(
    const char* repoid,
    const PortableInterceptor::ObjectId& id,
    const PortableInterceptor::AdapterName& name)
    throw(SystemException)
{
    //
    // Create the actual ORBacus ObjectKey
    //
    // TODO:
    // CreatePersistentObjectKey/CreateTransientObjectKey instead of
    // populating this ObjectKey data to avoid the copy?
    //
    OB::ObjectKeyData obkey;
    obkey.serverId = the_server_id();
    if(strlen(obkey.serverId) == 0)
        obkey.serverId = string_dup("_RootPOA");
    obkey.poaId = name;
    obkey.oid.length(id.length());
    memcpy(obkey.oid.get_buffer(), id.get_buffer(), id.length());
    obkey.persistent = true;
    obkey.createTime = 0;

    OCI::ObjectKey_var key = CreateObjectKey(obkey);

    IOP::IOR_var ior = new IOP::IOR(the_ior_template());
    ior -> type_id = repoid;

    OCI::AccFactoryRegistry_var registry =
        orbInstance_ -> getAccFactoryRegistry();
    OCI::AccFactorySeq_var factories = registry -> get_factories();
    for(ULong i = 0 ; i < factories -> length() ; ++i)
	factories[i] -> change_key(ior.inout(), key);

    RefCountIOR_var refior = new RefCountIOR(ior);
    
    //
    // Create and return reference
    //
    ObjectFactory_var objectFactory = orbInstance_ -> getObjectFactory();
    return objectFactory -> createObject(refior);
}

// ----------------------------------------------------------------------
// PersistentORT_impl public member implementation
// ----------------------------------------------------------------------

ValueBase*
OBPortableInterceptor::PersistentORT_impl::_copy_value()
    throw(SystemException)
{
    PersistentORT_impl* r =
	new PersistentORT_impl(orbInstance_, the_server_id(), the_orb_id(),
			       the_adapter_name(), the_ior_template());
    return r;
}

PortableInterceptor::ServerId
OBPortableInterceptor::PersistentORT_impl::server_id()
    throw(SystemException)
{
    return string_dup(the_server_id());
}

PortableInterceptor::ORBId
OBPortableInterceptor::PersistentORT_impl::orb_id()
    throw(SystemException)
{
    return string_dup(the_orb_id());
}

PortableInterceptor::AdapterName*
OBPortableInterceptor::PersistentORT_impl::adapter_name()
    throw(SystemException)
{
    return new PortableInterceptor::AdapterName(the_adapter_name());
}

Object_ptr
OBPortableInterceptor::PersistentORT_impl::make_object(
    const char* repoid,
    const PortableInterceptor::ObjectId& id)
    throw(SystemException)
{
    return makeObject(repoid, id, the_adapter_name());
}

Object_ptr
OBPortableInterceptor::PersistentORT_impl::make_object_for(
    const char*,
    const PortableInterceptor::ObjectId&,
    const PortableInterceptor::AdapterName&)
    throw(SystemException)
{
    throw NO_IMPLEMENT();

    return 0; // Some compilers need this
}

// ----------------------------------------------------------------------
// TransientORTFactory_impl constructor and destructor
// ----------------------------------------------------------------------

OBPortableInterceptor::
TransientORTFactory_impl::TransientORTFactory_impl(
    ORBInstance_ptr orbInstance)
    : orbInstance_(ORBInstance::_duplicate(orbInstance))
{
}

OBPortableInterceptor::
TransientORTFactory_impl::~TransientORTFactory_impl()
{
}

// ----------------------------------------------------------------------
// TransientORTFactory_impl public member implementation
// ----------------------------------------------------------------------

ValueBase*
OBPortableInterceptor::
TransientORTFactory_impl::create_for_unmarshal()
    throw(SystemException)
{
    return new TransientORT_impl(orbInstance_);
}

// ----------------------------------------------------------------------
// PersistentORTFactory_impl constructor and destructor
// ----------------------------------------------------------------------

OBPortableInterceptor::
PersistentORTFactory_impl::PersistentORTFactory_impl(
    ORBInstance_ptr orbInstance)
    : orbInstance_(ORBInstance::_duplicate(orbInstance))
{
}

OBPortableInterceptor::
PersistentORTFactory_impl::~PersistentORTFactory_impl()
{
}

// ----------------------------------------------------------------------
// PersistentORTFactory_impl public member implementation
// ----------------------------------------------------------------------

ValueBase*
OBPortableInterceptor::
PersistentORTFactory_impl::create_for_unmarshal()
    throw(SystemException)
{
    return new PersistentORT_impl(orbInstance_);
}
