// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#ifndef PI_ORT_IMPL_H
#define PI_ORT_IMPL_H

#include <OB/ValueBase.h>
#include <OB/ValueFactoryBase.h>
#include <OB/OBPIORT.h>
#include <OB/ORBInstance_fwd.h>

namespace OBPortableInterceptor
{

//
// Used for the creation of transient object references
//
class TransientORT_impl : public OBV_OBPortableInterceptor::TransientORT,
			  public CORBA::DefaultValueRefCountBase
{
    //
    // Hide copy-constructor and assignment operator
    //
    TransientORT_impl(const TransientORT_impl&);
    void operator=(const TransientORT_impl&);

    //
    // The ORBInstance object
    //
    OB::ORBInstance_var orbInstance_;

    CORBA::Object_ptr
    makeObject(const char*,
    	       const PortableInterceptor::ObjectId&,
	       const PortableInterceptor::AdapterName&)
	throw(CORBA::SystemException);

public:

    TransientORT_impl(OB::ORBInstance_ptr);
    TransientORT_impl(OB::ORBInstance_ptr,
		      const char*,
		      const char*,
		      const PortableInterceptor::AdapterName&,
		      time_t,
		      const IOP::IOR&);

    virtual ~TransientORT_impl();

    //
    // Standard IDL to C++ Mapping
    //

    //
    // ObjectReferenceTemplate_impl cannot be copied or marshalled
    //
    virtual CORBA::ValueBase* _copy_value()
	throw(CORBA::SystemException);

    virtual PortableInterceptor::ServerId server_id()
        throw(CORBA::SystemException);

    virtual PortableInterceptor::ORBId orb_id()
	throw(CORBA::SystemException);

    virtual PortableInterceptor::AdapterName* adapter_name()
	throw(CORBA::SystemException);

    virtual CORBA::Object_ptr make_object(const char*,
                                          const PortableInterceptor::ObjectId&)
	throw(CORBA::SystemException);

    virtual CORBA::Object_ptr make_object_for(const char*,
				const PortableInterceptor::ObjectId&,
				const PortableInterceptor::AdapterName&)
	throw(CORBA::SystemException);
};

//
// Used for the creation of persistent object-references
//
class PersistentORT_impl : public OBV_OBPortableInterceptor::PersistentORT,
			   public CORBA::DefaultValueRefCountBase
{
    //
    // Hide copy-constructor and assignment operator
    //
    PersistentORT_impl(const PersistentORT_impl&);
    void operator=(const PersistentORT_impl&);

    //
    // The ORBInstance object
    //
    OB::ORBInstance_var orbInstance_;

    CORBA::Object_ptr
    makeObject(const char*,
	       const PortableInterceptor::ObjectId&,
	       const PortableInterceptor::AdapterName&)
	throw(CORBA::SystemException);

public:

    PersistentORT_impl(OB::ORBInstance_ptr);
    PersistentORT_impl(OB::ORBInstance_ptr,
		       const char*,
		       const char*,
		       const PortableInterceptor::AdapterName&,
		       const IOP::IOR&);

    ~PersistentORT_impl();

    //
    // Standard IDL to C++ Mapping
    //

    virtual CORBA::ValueBase* _copy_value()
	throw(CORBA::SystemException);

    virtual PortableInterceptor::ServerId server_id()
	throw(CORBA::SystemException);

    virtual PortableInterceptor::ORBId orb_id()
	throw(CORBA::SystemException);

    virtual PortableInterceptor::AdapterName* adapter_name()
	throw(CORBA::SystemException);

    virtual CORBA::Object_ptr make_object(const char*,
                                          const PortableInterceptor::ObjectId&)
	throw(CORBA::SystemException);

    virtual CORBA::Object_ptr make_object_for(const char*,
				const PortableInterceptor::ObjectId&,
				const PortableInterceptor::AdapterName&)
	throw(CORBA::SystemException);
};

//
// The ValueType Factories
//
class TransientORTFactory_impl : public CORBA::ValueFactoryBase
{
    //
    // The ORBInstance object
    //
    OB::ORBInstance_var orbInstance_;

    virtual CORBA::ValueBase* create_for_unmarshal()
	throw(CORBA::SystemException);

public:

    TransientORTFactory_impl(OB::ORBInstance_ptr);
    virtual ~TransientORTFactory_impl();
};

class PersistentORTFactory_impl : public CORBA::ValueFactoryBase
{
    //
    // The ORBInstance object
    //
    OB::ORBInstance_var orbInstance_;

    virtual CORBA::ValueBase* create_for_unmarshal()
	throw(CORBA::SystemException);

public:

    PersistentORTFactory_impl(OB::ORBInstance_ptr);
    virtual ~PersistentORTFactory_impl();
};

} // End namespace OBPortableInterceptor

#endif
