// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#ifndef PI_SERVER_IMPL_H
#define PI_SERVER_IMPL_H

#include <OB/POAInterface_fwd.h>
#include <OB/POAServantBase_fwd.h>
#include <OB/PIORT_fwd.h>
#include <OB/PIServer.h>

#include <PIRequest_impl.h>

namespace PortableInterceptor
{

//
// Typedef for sequence of ServerRequestInterceptor
//
class OBUnique_ServerRequestInterceptorSeq;
typedef OB::ObjSeq< ServerRequestInterceptor,
    OBUnique_ServerRequestInterceptorSeq > ServerRequestInterceptorSeq;

//
// Implementation of ServerRequestInfo
//
class ServerRequestInfo_impl :
    public virtual ServerRequestInfo,
    public RequestInfo_impl
{
    ServerRequestInfo_impl(const ServerRequestInfo_impl&);
    void operator=(const ServerRequestInfo_impl&);

    //
    // Sequence of ServerRequestInterceptors to call on reply
    //
    ServerRequestInterceptorSeq interceptors_;

    //
    // The adapter ID
    //
    const CORBA::OctetSeq& adapterId_;

    //
    // The object-id
    //
    CORBA::OctetSeq objectId_;

    //
    // The servant
    //
    PortableServer::ServantBase* servant_;

    //
    // The POA
    //
    PortableServer::POA_ptr poa_;

    //
    // The adapter template
    //
    ObjectReferenceTemplate* adapterTemplate_;

    //
    // The adapter name
    //
    AdapterName adapterName_;

public:

    ServerRequestInfo_impl(CORBA::ULong, 
			   const char*, bool,
			   OB::RefCountPolicyList_ptr,
                           const CORBA::OctetSeq&,
                           const CORBA::OctetSeq&,
			   ObjectReferenceTemplate*,
                           IOP::ServiceContextList&,
                           IOP::ServiceContextList&,
                           OB::ORBInstance_ptr,
                           Current_impl*);
    ~ServerRequestInfo_impl();

    //
    // IDL to C++ Mapping
    //
    virtual CORBA::Any* sending_exception()
	throw(CORBA::SystemException);

    virtual CORBA::OctetSeq* object_id()
	throw(CORBA::SystemException);

    virtual CORBA::OctetSeq* adapter_id()
	throw(CORBA::SystemException);

    virtual char* target_most_derived_interface()
	throw(CORBA::SystemException);

    virtual PortableInterceptor::ServerId server_id()
        throw(CORBA::SystemException);

    virtual PortableInterceptor::ORBId orb_id()
        throw(CORBA::SystemException);

    virtual PortableInterceptor::AdapterName* adapter_name()
        throw(CORBA::SystemException);

    virtual CORBA::Policy_ptr get_server_policy (CORBA::PolicyType)
	throw(CORBA::SystemException);
    
    virtual void set_slot(SlotId, const CORBA::Any&)
	throw(CORBA::SystemException, PortableInterceptor::InvalidSlot);

    virtual CORBA::Boolean target_is_a(const char*)
	throw(CORBA::SystemException);

    virtual void add_reply_service_context (const IOP::ServiceContext&,
                                            CORBA::Boolean)
	throw(CORBA::SystemException);

    //
    // ORBacus internal functions
    // Application programs must not use these functions directly
    //
    void _OB_requestServiceContext(const ServerRequestInterceptorSeq&)
	throw(CORBA::SystemException, OB::LocationForward);

    void _OB_request()
	throw(CORBA::SystemException, OB::LocationForward);
    
    void _OB_sendReply()
	throw(CORBA::SystemException);

    void _OB_sendException()
	throw(CORBA::SystemException, OB::LocationForward);
    
    void _OB_sendOther()
	throw(CORBA::SystemException, OB::LocationForward);

    void _OB_servant(PortableServer::ServantBase*,
                     PortableServer::POA_ptr)
	throw();

    void _OB_contextSwitch()
	throw();

    void _OB_parameterDesc(
        OB::ParameterDesc*, /*args*/
        int,
        OB::ParameterDesc*, /*result*/
        CORBA::TypeCode_ptr*, /*exceptions*/
        int)
	throw();

    void _OB_arguments(CORBA::NVList_ptr)
	throw();

    void _OB_result(const CORBA::Any&)
	throw();
};

} // End of namespace PortableInterceptor

#endif
