// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#ifndef POA_LOCATOR_H
#define POA_LOCATOR_H

#include <OB/POAInterface_fwd.h>

#include <OB/Object.h>
#include <OB/StubImpl.h>
#include <OB/Hashtable.h>
 
namespace OB
{

//
// Other declarations
// 
struct ObjectKeyData;

} // End namespace OB

namespace OBPortableServer
{

//
// Set of associated POAs
//
struct POANameHasher
{
    static CORBA::ULong hash(const CORBA::StringSeq&);
    static bool comp(const CORBA::StringSeq&, const CORBA::StringSeq&);
};
typedef OB::Hashtable< CORBA::StringSeq , PortableServer::POA_var,
		       POANameHasher> POAHashTable;

//
// This class keeps track of all POAs in the ORB -- this is distinct
// from the POAManager since it only tracks the POAs that it manages.
//
class POALocator : public JTCRecursiveMutex
{
    //
    // Hide copy constructor/assignment operator
    //
    POALocator(const POALocator&);
    void operator=(const POALocator&);

    //
    // The set of POAs
    //
    POAHashTable poas_;

public:

    POALocator();
    ~POALocator();

    //
    // Locate a POA
    //
    PortableServer::POA_ptr locate(const OB::ObjectKeyData&)
        throw(OB::LocationForward, CORBA::SystemException);

    //
    // Add a POA
    //
    void add(PortableServer::POA_ptr, const CORBA::StringSeq&);

    //
    // Remove a POA
    //
    void remove(const CORBA::StringSeq&);
};

} // End namespace OBPortableServer

#endif
