// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#ifndef OB_POA_OA_INTERFACE_IMPL_H
#define OB_POA_OA_INTERFACE_IMPL_H

#include <OB/OAInterface.h>

#include <OB/ORBInstance_fwd.h>
#include <OB/BootManager_fwd.h>

namespace OBPortableServer
{

//
// Forward declaration
//
class POAManager_impl;

} // End of namespace OBPortableServer

namespace OB
{

//
// Forward declaration
//
class BootManager_impl;
    
//
// We don't need any sort of concurrency protection on this class
// since its lifecycle is tightly coupled with the POAManager. Once
// the POAManager destroys this class it should not be possible for
// any further requests to arrive (or any requests in the process of
// being dispatched).
//
class POAOAInterface_impl : public OAInterface
{
    //
    // The ORBInstance
    //
    ORBInstance_var orbInstance_;

    //
    // The POAManager implementation
    //
    OBPortableServer::POAManager_impl* poaManager_;

    //
    // The boot manager and implementation
    //
    BootManager_var bootManager_;
    BootManager_impl* bootManagerImpl_;

    //
    // Is the POAOAInterface discarding requests?
    //
    bool discard_;

public:

    POAOAInterface_impl(OBPortableServer::POAManager_impl*,
			ORBInstance_ptr);
    ~POAOAInterface_impl();

    //
    // Note: For efficiency reasons, this operation assumes ownership
    // for all parameter marked with "/**/"
    //
    virtual Upcall_ptr
    createUpcall(UpcallReturn_ptr,
		 /**/ OCI::ProfileInfo*,
		 /**/ OCI::TransportInfo_ptr,
		 CORBA::ULong,
		 /**/ char*,
		 /**/ InputStream_ptr,
		 /**/ IOP::ServiceContextList*)
	throw();
    
    virtual LocateStatus
    findByKey(const OCI::ObjectKey&, RefCountIOR_out)
	throw();

    virtual OCI::ProfileInfoSeq*
    getUsableProfiles(const OCI::IOR&, const CORBA::PolicyList&)
	throw();

    //
    // Discard all incoming requests with a TRANSIENT exception
    //
    // ASYNC SAFE
    //
    virtual void discard()
	throw();

    //
    // Allow associated POAs to receive requests
    //
    virtual void activate()
	throw();
};

} // End namespace OB

#endif
