// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBAServer.h>
#include <OB/POAServantLocator_skel.h>
#include <OB/OCI.h>

using namespace CORBA;
using namespace OB;

namespace POA_PortableServer
{

//
// The ServantLocatorDelegate. This class is used as a weak reference
// to the actual servant locator implementation. This is necessary so
// that in the case that reference counting is not used on the servant
// manager implementation that user code doesn't get a direct pointer
// to the servant itself.
//
class ServantLocatorDelegate : public PortableServer::ServantLocator,
			       public OBCORBA::RefCountLocalObject
{
    //
    // This cannot be a _var type otherwise in the reference counted
    // case there will be a circular reference
    //
    PortableServer::ServantLocator_ptr delegate_;

public:

    ServantLocatorDelegate(PortableServer::ServantLocator_ptr delegate)
	: delegate_(delegate)
    {
    }

    ~ServantLocatorDelegate()
    {
    }

    static inline ServantLocatorDelegate_ptr
    _duplicate(ServantLocatorDelegate_ptr p)
    { if(p) p -> _add_ref(); return p; }

    static inline ServantLocatorDelegate_ptr _nil()
    { return 0; }

    virtual PortableServer::Servant
    preinvoke(const PortableServer::ObjectId& oid,
	      PortableServer::POA_ptr adapter,
	      const char* operation,
	      Cookie& cookie)
    {
	assert(delegate_);
	return delegate_ -> preinvoke(oid, adapter, operation, cookie);
    }

    virtual void
    postinvoke(const PortableServer::ObjectId& oid,
	       PortableServer::POA_ptr adapter,
	       const char* operation,
	       Cookie cookie,
	       PortableServer::Servant servant)
    {
	assert(delegate_);
	delegate_ -> postinvoke(oid, adapter, operation, cookie, servant);
    }

    void
    clear()
    {
	delegate_ = PortableServer::ServantLocator::_nil();
    }
};

} // End namespace POA_PortableServer

// ----------------------------------------------------------------------
// External, non-inline duplicate/release for templates
// ----------------------------------------------------------------------

void
OBRelease(POA_PortableServer::ServantLocatorDelegate_ptr o)
{
    if(o)
	o -> _remove_ref();
}

void
OBDuplicate(POA_PortableServer::ServantLocatorDelegate_ptr o)
{
    if(o)
	o -> _add_ref();
}

// ----------------------------------------------------------------------
// ServantLocator constructor/destructor
// ----------------------------------------------------------------------

POA_PortableServer::ServantLocator::ServantLocator()
    : delegate_(new ServantLocatorDelegate(this))
{
}

POA_PortableServer::ServantLocator::~ServantLocator()
{
    delegate_ -> clear();
}

// ----------------------------------------------------------------------
// ServantLocator public member implementation
// ----------------------------------------------------------------------

PortableServer::ServantLocator_ptr
POA_PortableServer::ServantLocator::_this()
{
    return ServantLocatorDelegate::_duplicate(delegate_);
}

CORBA::RepositoryId
POA_PortableServer::ServantLocator::_primary_interface(
    const PortableServer::ObjectId&, PortableServer::POA_ptr)
{
    throw CORBA::NO_IMPLEMENT();
    return 0; // Some compilers need this
}

void
POA_PortableServer::ServantLocator::_OB_dispatch(OB::Upcall_ptr)
{
    throw CORBA::NO_IMPLEMENT();
}
