// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/Except.h>

#include <ParseParams.h>

using namespace CORBA;
using namespace OB;

const char*
OB::parse_params(const char* str, OB_STL::list<String_var>& params)
{
    const char* cp = str;
    bool error = false;
    const char* result = 0;
    while(*cp != '\0' && result == 0 && !error)
    {
        //
        // Skip leading whitespace
        //
        while(*cp == ' ' || *cp == '\t' || *cp == '\n' || *cp == '\r')
            cp++;

        const char* start = cp;
        const char* end = 0;
        char quote = 0;
        bool done = false;
        while(!done && !error)
        {
            switch(*cp)
            {
            case '\0':
                end = cp;
                done = true;
                break;

            case ',':
                if(quote == 0)
                {
                    end = cp;
                    result = cp + 1;
                    done = true;
                }
                else
                    cp++;
                break;

            case ' ':
            case '\t':
            case '\n':
            case '\r':
                if(quote == 0)
                {
                    end = cp;
                    done = true;
                }
                else
                    cp++;
                break;

            case '\\':
                cp++;
                if(*cp == '\0')
                    error = true;
                else
                    cp++;
                break;

            case '\'':
            case '"':
                if(quote == 0)
                {
                    if(cp > start)
                    {
                        done = true;
                        end = cp;
                    }
                    else
                    {
                        quote = *cp;
                        cp++;
                    }
                }
                else if(quote == *cp)
                {
                    quote = 0;
                    done = true;
                    end = cp;
                    cp++;
                }
                else
                    error = true; // mismatched quotes
                break;

            default:
                cp++;
                break;
            }
        }

        if(quote != 0)
            error = true;

        if(!error && cp > start)
        {
            if(*start == '\'' || *start == '"')
                start++;

            ULong len = end - start;
            char* p = string_alloc(len);
            const char* s = start;
            ULong i = 0;
            while(s < end)
            {
                if(*s == '\\')
                    s++;
                else
                    p[i++] = *s++;
            }
            p[i] = '\0';
            params.push_back(p);
        }
    }

    return result;
}
