// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#ifndef OB_PARSE_PARAMS_H
#define OB_PARSE_PARAMS_H

#include <OB/Basic.h>
#include <OB/list.h>

#include <ParseParams.h>

namespace OB
{

//
// Parse a parameter list. Parameters are separated by whitespace.
// Quoted parameters are supported, as are escaped characters.
// Parsing stops when the end of the string is reached, or when
// an unescaped, unquoted comma character is encountered.
//
// The return value is 0 on end of string, or a pointer into the
// input string marking the location where parsing should resume.
//
// For example, consider this input string:
//
// abc 123 'ABC 123' "abc,123", def ghi
//
// The first execution will return a list containing the elements
// {"abc", "123", "ABC 123", "abc,123"}. The return value will point
// to the string " def ghi".
//
// A BAD_PARAM exception will be raised if a formatting error
// occurs.
//
const char* parse_params(const char*, OB_STL::list<CORBA::String_var>&);

} // End of namespace OB

#endif
