// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/PolicyFactoryManager.h>
#include <OB/PIPolicy.h>

using namespace CORBA;
using namespace OB;

// ----------------------------------------------------------------------
// External, non-inline duplicate/release for templates
// ----------------------------------------------------------------------

void
OBDuplicate(PolicyFactoryManager_ptr p)
{
    if(p)
	p -> _OB_incRef();
}

void
OBRelease(PolicyFactoryManager_ptr p)
{
    if(p)
	p -> _OB_decRef();
}

// ----------------------------------------------------------------------
// PolicyFactoryManager constructor and destructor
// ----------------------------------------------------------------------

PolicyFactoryManager::PolicyFactoryManager()
    : destroy_(false)
{
}

PolicyFactoryManager::~PolicyFactoryManager()
{
}

// ----------------------------------------------------------------------
// PolicyFactoryManager public member implementation
// ----------------------------------------------------------------------

//
// Destroy the PolicyFactoryManager
//
void
PolicyFactoryManager::destroy()
{
    assert(!destroy_);
    destroy_ = true;

    policyFactoryTableInternal_.clear();
    policyFactoryTableExternal_.clear();
}

void
PolicyFactoryManager::registerPolicyFactory(
    PolicyType type,
    PortableInterceptor::PolicyFactory_ptr factory,
    bool internal)
{
    //
    // TODO: some sensible error
    //
    PolicyFactoryTable* table =
        (internal)
        ? &policyFactoryTableInternal_
        : &policyFactoryTableExternal_;

    if(table -> containsKey(type))
        throw BAD_PARAM();
    table -> put(type, PortableInterceptor::PolicyFactory::
		 _duplicate(factory));
}

Policy_ptr
PolicyFactoryManager::createPolicy(PolicyType type, const Any& any)
{
    PortableInterceptor::PolicyFactory_var factory;
    if(policyFactoryTableInternal_.get(type, factory))
        return factory -> create_policy(type, any);
    if(policyFactoryTableExternal_.get(type, factory))
        return factory -> create_policy(type, any);

    throw PolicyError(BAD_POLICY);
}
