// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/RefCounted.h>
#include <OB/Object.h>
#include <OB/Except.h>
#include <OB/OBPolicies.h>
#include <OB/Messaging.h>
#include <OB/TimeHelper.h>

using namespace CORBA;
using namespace OB;

// ----------------------------------------------------------------------
// External, non-inline duplicate/release for templates
// ----------------------------------------------------------------------

void
OBDuplicate(RefCountIOR_ptr p)
{
    if(p)
	p -> _OB_incRef();
}

void
OBRelease(RefCountIOR_ptr p)
{
    if(p)
	p -> _OB_decRef();
}

void
OBDuplicate(RefCountPolicyList_ptr p)
{
    if(p)
	p -> _OB_incRef();
}

void
OBRelease(RefCountPolicyList_ptr p)
{
    if(p)
	p -> _OB_decRef();
}

void
OBDuplicate(RefCountProfileInfo_ptr p)
{
    if(p)
	p -> _OB_incRef();
}

void
OBRelease(RefCountProfileInfo_ptr p)
{
    if(p)
	p -> _OB_decRef();
}

// ----------------------------------------------------------------------
// RefCountPolicyList constructor
// ----------------------------------------------------------------------

RetryAttributes
RefCountPolicyList::getRetry(const PolicyList& policies)
{
    RetryAttributes attributes;
    for(ULong i = 0 ; i < policies.length() ; i++)
    {
	if(policies[i] -> policy_type() == RETRY_POLICY_ID)
	{
	    RetryPolicy_var policy = RetryPolicy::_narrow(policies[i]);
	    assert(!is_nil(policy));

    	    attributes.mode = policy -> retry_mode();
    	    attributes.interval = policy -> retry_interval();
    	    attributes.max = policy -> retry_max();
    	    attributes.remote = policy -> retry_remote();

	    return attributes;
	}
    }
    attributes.mode = RETRY_STRICT;
    attributes.interval = 0;
    attributes.max = 1;
    attributes.remote = false;

    return attributes;
}

Long
RefCountPolicyList::getConnectTimeout(const PolicyList& policies)
{
    ULong i;
    for(i = 0 ; i < policies.length() ; i++)
    {
        if(policies[i] -> policy_type() == CONNECT_TIMEOUT_POLICY_ID)
        {
            ConnectTimeoutPolicy_var policy =
                ConnectTimeoutPolicy::_narrow(policies[i]);
            assert(!is_nil(policy));
            return (Long)policy -> value();
        }
    }

    //
    // Fall back to TimeoutPolicy
    //
    for(i = 0 ; i < policies.length() ; i++)
    {
        if(policies[i] -> policy_type() == TIMEOUT_POLICY_ID)
        {
            TimeoutPolicy_var policy = TimeoutPolicy::_narrow(policies[i]);
            assert(!is_nil(policy));
            return (Long)policy -> value();
        }
    }

    return -1;
}

//
// TODO: This needs to be replaced with the new messaging timeout
// policies below.
//
Long
RefCountPolicyList::getRequestTimeout(const PolicyList& policies)
{
    ULong i;
    for(i = 0 ; i < policies.length() ; i++)
    {
        if(policies[i] -> policy_type() == REQUEST_TIMEOUT_POLICY_ID)
        {
            RequestTimeoutPolicy_var policy =
                RequestTimeoutPolicy::_narrow(policies[i]);
            assert(!is_nil(policy));
            return (Long)policy -> value();
        }
    }

    //
    // Fall back to TimeoutPolicy
    //
    for(i = 0 ; i < policies.length() ; i++)
    {
        if(policies[i] -> policy_type() == TIMEOUT_POLICY_ID)
        {
            TimeoutPolicy_var policy = TimeoutPolicy::_narrow(policies[i]);
            assert(!is_nil(policy));
            return (Long)policy -> value();
        }
    }

    return -1;
}

TimeBase::UtcT
RefCountPolicyList::getRequestStartTime(const PolicyList& policies)
{
    for(ULong i = 0 ; i < policies.length() ; i++)
    {
	if(policies[i] -> policy_type() ==
	   Messaging::REQUEST_START_TIME_POLICY_TYPE)
	{
	    Messaging::RequestStartTimePolicy_var policy =
		Messaging::RequestStartTimePolicy::_narrow(policies[i]);
	    assert(!is_nil(policy));
	    return policy -> start_time();
	}
    }

    return TimeHelper::utcMin(); 
}

TimeBase::UtcT
RefCountPolicyList::getRequestEndTime(const PolicyList& policies)
{
    for(ULong i = 0 ; i < policies.length() ; i++)
    {
	if(policies[i] -> policy_type() ==
	   Messaging::REQUEST_END_TIME_POLICY_TYPE)
	{
	    Messaging::RequestEndTimePolicy_var policy =
		Messaging::RequestEndTimePolicy::_narrow(policies[i]);
	    assert(!is_nil(policy));
	    return policy -> end_time();
	}
    }

    return TimeHelper::utcMin();
}

TimeBase::UtcT
RefCountPolicyList::getReplyStartTime(const PolicyList& policies)
{
    for(ULong i = 0 ; i < policies.length() ; i++)
    {
	if(policies[i] -> policy_type() ==
	   Messaging::REPLY_START_TIME_POLICY_TYPE)
	{
	    Messaging::ReplyStartTimePolicy_var policy =
		Messaging::ReplyStartTimePolicy::_narrow(policies[i]);
	    assert(!is_nil(policy));
	    return policy -> start_time();
	}
    }

    return TimeHelper::utcMin();
}

TimeBase::UtcT
RefCountPolicyList::getReplyEndTime(const PolicyList& policies)
{
    for(ULong i = 0 ; i < policies.length() ; i++)
    {
	if(policies[i] -> policy_type() ==
	   Messaging::REPLY_END_TIME_POLICY_TYPE)
	{
	    Messaging::ReplyEndTimePolicy_var policy =
		Messaging::ReplyEndTimePolicy::_narrow(policies[i]);
	    assert(!is_nil(policy));
	    return policy -> end_time();
	}
    }

    return TimeHelper::utcMin();
}

TimeBase::TimeT
RefCountPolicyList::getRelativeRequestTimeout(const PolicyList& policies)
{
    for(ULong i = 0 ; i < policies.length() ; i++)
    {
	if(policies[i] -> policy_type() ==
	   Messaging::RELATIVE_REQ_TIMEOUT_POLICY_TYPE)
	{
	    Messaging::RelativeRequestTimeoutPolicy_var policy =
		Messaging::RelativeRequestTimeoutPolicy::_narrow(policies[i]);
	    assert(!is_nil(policy));
	    return policy -> relative_expiry();
	}
    }

    return 0;
}

TimeBase::TimeT
RefCountPolicyList::getRelativeRoundTripTimeout(const PolicyList& policies)
{
    for(ULong i = 0 ; i < policies.length() ; i++)
    {
	if(policies[i] -> policy_type() ==
	   Messaging::RELATIVE_RT_TIMEOUT_POLICY_TYPE)
	{
	    Messaging::RelativeRoundTripTimeoutPolicy_var policy =
		Messaging::RelativeRoundTripTimeoutPolicy::
		    _narrow(policies[i]);
	    assert(!is_nil(policy));
	    return policy -> relative_expiry();
	}
    }

    return 0;
}

Short
RefCountPolicyList::getRebindMode(const PolicyList& policies)
{
    for(ULong i = 0 ; i < policies.length() ; i++)
    {
	if(policies[i] -> policy_type() == Messaging::REBIND_POLICY_TYPE)
	{
	    Messaging::RebindPolicy_var policy =
		Messaging::RebindPolicy::_narrow(policies[i]);
	    assert(!is_nil(policy));
	    return policy -> rebind_mode();
	}
    }

    return Messaging::TRANSPARENT;
}

Short
RefCountPolicyList::getSyncScope(const PolicyList& policies)
{
    for(ULong i = 0 ; i < policies.length() ; i++)
    {
	if(policies[i] -> policy_type() == Messaging::SYNC_SCOPE_POLICY_TYPE)
	{
	    Messaging::SyncScopePolicy_var policy =
		Messaging::SyncScopePolicy::_narrow(policies[i]);
	    assert(!is_nil(policy));
	    return policy -> synchronization();
	}
    }

    return Messaging::SYNC_WITH_TRANSPORT;
}

Short
RefCountPolicyList::getLocationTransparency(const PolicyList& policies)
{
    for(ULong i = 0 ; i < policies.length() ; i++)
    {
	if(policies[i] -> policy_type() == LOCATION_TRANSPARENCY_POLICY_ID)
	{
	    LocationTransparencyPolicy_var policy =
		LocationTransparencyPolicy::_narrow(policies[i]);
	    assert(!is_nil(policy));
	    return policy -> value();
	}
    }

    return LOCATION_TRANSPARENCY_RELAXED;
}

bool
RefCountPolicyList::getInterceptor(const PolicyList& policies)
{
    for(ULong i = 0 ; i < policies.length() ; i++)
    {
	if(policies[i] -> policy_type() == INTERCEPTOR_POLICY_ID)
	{
	    InterceptorPolicy_var policy =
		InterceptorPolicy::_narrow(policies[i]);
	    assert(!is_nil(policy));
	    return policy -> value();
	}
    }

    return true;
}

bool
RefCountPolicyList::getLocateRequest(const PolicyList& policies)
{
    for(ULong i = 0 ; i < policies.length() ; i++)
    {
        if(policies[i] -> policy_type() == LOCATE_REQUEST_POLICY_ID)
        {
            LocateRequestPolicy_var policy =
                LocateRequestPolicy::_narrow(policies[i]);
            assert(!is_nil(policy));
            return policy -> value();
        }
    }

    return false;
}

// ----------------------------------------------------------------------
// RefCountPolicyList constructor
// ----------------------------------------------------------------------

RefCountPolicyList::RefCountPolicyList(const PolicyList& v)
    : value(v),
      retry(getRetry(v)),
      connectTimeout(getConnectTimeout(v)),
      requestTimeout(getRequestTimeout(v)),
      requestStartTime(getRequestStartTime(v)),
      requestEndTime(getRequestEndTime(v)),
      replyStartTime(getReplyStartTime(v)),
      replyEndTime(getReplyEndTime(v)),
      relativeRequestTimeout(getRelativeRequestTimeout(v)),
      relativeRoundTripTimeout(getRelativeRoundTripTimeout(v)),
      rebindMode(getRebindMode(v)),
      syncScope(getSyncScope(v)),
      locationTransparency(getLocationTransparency(v)),
      interceptor(getInterceptor(v)),
      locateRequest(getLocateRequest(v))
{
}
