// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/TypeCode.h>
#include <OB/Principal.h>
#include <OB/MarshalStubImpl.h>
#include <OB/ValueBase.h>
#include <OB/AbstractBase.h>
#include <OB/Stream.h>
#include <OB/ORBInstance.h>
#include <OB/ValueFactoryManager.h>
#include <OB/IFR1.h>
#include <OB/RefCounted.h>
#include <OB/ValueReaderWriter.h>
#include <OB/TypeCodeCache.h>

#include <CodeSetDatabase.h>
#include <CodeConverter.h>
#include <Custom_impl.h>

#include <stdio.h>
#include <stdlib.h>

using namespace CORBA;
using namespace OB;

// ----------------------------------------------------------------------
// External, non-inline duplicate/release for templates
// ----------------------------------------------------------------------

void
OBDuplicate(OB::InputStream_ptr p)
{
    if(p)
        p -> _OB_incRef();
}

void
OBRelease(OB::InputStream_ptr p)
{
    if(p)
        p -> _OB_decRef();
}

void
OBDuplicate(OB::OutputStream_ptr p)
{
    if(p)
        p -> _OB_incRef();
}

void
OBRelease(OB::OutputStream_ptr p)
{
    if(p)
        p -> _OB_decRef();
}

// ----------------------------------------------------------------------
// OutputStreamImpl private and protected member implementations
// ----------------------------------------------------------------------

ULong
OutputStreamImpl::writeGap()
{
    addCapacity(4, 4);
    ULong result = buf_ -> cur_ - buf_ -> data_;
    buf_ -> cur_ += 4;
    return result;
}

void
OutputStreamImpl::writeLength(ULong start)
{
    ULong length = buf_ -> cur_ - buf_ -> data_ - (start + 4);
    Octet* p = buf_ -> data_ + start;
    *((ULong*)p) = length;
}

void
OutputStreamImpl::writeTypeCodeImpl(TypeCode_ptr tc, WriteTCHistory& history)
{
    if(tc -> recId_.in())
    {
        if(is_nil(tc -> recType_))
            throw BAD_TYPECODE(MinorIncompleteTypeCode, COMPLETED_NO);
        writeTypeCodeImpl(tc -> recType_, history);
        return;
    }

    //
    // For performance reasons, handle the primitive TypeCodes first
    //
    switch(tc -> kind())
    {
    case tk_null:
    case tk_void:
    case tk_short:
    case tk_long:
    case tk_longlong:
    case tk_ushort:
    case tk_ulong:
    case tk_ulonglong:
    case tk_float:
    case tk_double:
    case tk_longdouble:
    case tk_boolean:
    case tk_char:
    case tk_wchar:
    case tk_octet:
    case tk_any:
    case tk_TypeCode:
    case tk_Principal:
        write_ulong((ULong)tc -> kind());
	return;

    default:
	break;
    }	

    Long indirectionPos;
    if(history.get(tc, indirectionPos))
    {
	//
	// Write an indirection since the TypeCode has already been
	// marshaled in the enclosing TypeCode.
	//
        write_ulong(0xffffffff);
        Long offs = indirectionPos - (Long)(buf_ -> cur_ - buf_ -> data_);
        write_long(offs);
    }
    else
    {
        write_ulong((ULong)tc -> kind());
        Long oldPos = (Long)(buf_ -> cur_ - buf_ -> data_ - 4);

        switch(tc -> kind())
        {
        case tk_fixed:
        {
            history.put(tc, oldPos);

            write_ushort(tc -> fixedDigits_);
            write_short(tc -> fixedScale_);

            break;
        }

	case tk_local_interface:
        case tk_objref:
        case tk_abstract_interface:
        case tk_native:
        {
            history.put(tc, oldPos);

            ULong start = writeGap();
            _OB_writeEndian();
            write_string(tc -> id_);
            write_string(tc -> name_);
            writeLength(start);

            break;
        }

        case tk_struct:
        case tk_except:
        {
            history.put(tc, oldPos);

            ULong i;

            ULong start = writeGap();
            _OB_writeEndian();
            write_string(tc -> id_);
            write_string(tc -> name_);
            write_ulong(tc -> memberNames_.length());
            for(i = 0 ; i < tc -> memberNames_.length() ; i++)
            {
                write_string(tc -> memberNames_[i]);
                writeTypeCodeImpl(tc -> memberTypes_[i], history);
            }
            writeLength(start);

            break;
        }

        case tk_union:
        {
            history.put(tc, oldPos);

            ULong start = writeGap();
            _OB_writeEndian();
            write_string(tc -> id_);
            write_string(tc -> name_);
            writeTypeCodeImpl(tc -> discriminatorType_, history);
            Long defaultIndex = tc -> default_index();
            write_long(defaultIndex);
            write_ulong(tc -> memberNames_.length());
            for(ULong i = 0 ; i < tc -> memberNames_.length() ; i++)
            {
		TypeCode_ptr origDiscType =
		    GetOrigType_nodup(tc -> discriminatorType_);

                //
                // Check for default label value
                //
                if(i == (ULong)defaultIndex)
                {
                    //
                    // Marshal a dummy value of the appropriate size
                    // for the discriminator type
                    //
                    switch(origDiscType -> kind())
                    {
                    case tk_short:
                        write_short(0);
                        break;
                    case tk_ushort:
                        write_ushort(0);
                        break;
                    case tk_long:
                        write_long(0);
                        break;
                    case tk_ulong:
                        write_ulong(0);
                        break;
                    case tk_longlong:
                        write_longlong(0);
                        break;
                    case tk_ulonglong:
                        write_ulonglong(0);
                        break;
                    case tk_boolean:
                        write_boolean(false);
                        break;
                    case tk_char:
                        write_char(0);
                        break;
                    case tk_enum:
                        write_ulong(0);
                        break;
                    default:
                        assert(false);
                        break;
                    }
                }
                else
                {
		    const void* value = tc -> labels_[i].value();
		    assert(value != 0);

                    switch(origDiscType -> kind())
                    {
                    case tk_short:
                        write_short(*(const Short*)value);
                        break;
                    case tk_ushort:
                        write_ushort(*(const UShort*)value);
                        break;
                    case tk_long:
                        write_long(*(const Long*)value);
                        break;
                    case tk_enum:
                    case tk_ulong:
                        write_ulong(*(const ULong*)value);
                        break;
                    case tk_longlong:
                        write_longlong(*(const LongLong*)value);
                        break;
                    case tk_ulonglong:
                        write_ulonglong(*(const ULongLong*)value);
                        break;
                    case tk_boolean:
                        write_boolean(*(const Boolean*)value);
                        break;
                    case tk_char:
                        write_char(*(const Char*)value);
                        break;
                    default:
                        assert(false);
                        break;
                    }
                }

                write_string(tc -> memberNames_[i]);
                writeTypeCodeImpl(tc -> memberTypes_[i], history);
            }
            writeLength(start);

            break;
        }

        case tk_enum:
        {
            history.put(tc, oldPos);

            ULong start = writeGap();
            _OB_writeEndian();
            write_string(tc -> id_);
            write_string(tc -> name_);
            write_ulong(tc -> memberNames_.length());
            for(ULong i = 0 ; i < tc -> memberNames_.length() ; i++)
                write_string(tc -> memberNames_[i]);
            writeLength(start);

            break;
        }

        case tk_string:
        case tk_wstring:
            write_ulong(tc -> length_);
            break;

        case tk_sequence:
        case tk_array:
        {
            history.put(tc, oldPos);

            ULong start = writeGap();
            _OB_writeEndian();
            writeTypeCodeImpl(tc -> contentType_, history);
            write_ulong(tc -> length_);
            writeLength(start);

            break;
        }

        case tk_alias:
        {
            history.put(tc, oldPos);

            ULong start = writeGap();
            _OB_writeEndian();
            write_string(tc -> id_);
            write_string(tc -> name_);
            writeTypeCodeImpl(tc -> contentType_, history);
            writeLength(start);

            break;
        }

        case tk_value:
        {
            history.put(tc, oldPos);

            TypeCode_ptr concreteBase;
            if(!is_nil(tc -> concreteBaseType_))
                concreteBase = tc -> concreteBaseType_.in();
            else
                concreteBase = _tc_null;

            ULong start = writeGap();
            _OB_writeEndian();
            write_string(tc -> id_);
            write_string(tc -> name_);
            write_short(tc -> typeModifier_);
            writeTypeCodeImpl(concreteBase, history);
            write_ulong(tc -> memberNames_.length());
            for(ULong i = 0 ; i < tc -> memberNames_.length() ; i++)
            {
                write_string(tc -> memberNames_[i]);
                writeTypeCodeImpl(tc -> memberTypes_[i], history);
                write_short(tc -> memberVisibility_[i]);
            }
            writeLength(start);

            break;
        }

        case tk_value_box:
        {
            history.put(tc, oldPos);

            ULong start = writeGap();
            _OB_writeEndian();
            write_string(tc -> id_);
            write_string(tc -> name_);
            writeTypeCodeImpl(tc -> contentType_, history);
            writeLength(start);

            break;
        }

	default:
	    assert(false);
	    break;
	}
    }
}

//
// Must be called prior to any writes
//
void
OutputStreamImpl::checkBeginChunk()
{
    assert(!is_nil(valueWriter_));
    valueWriter_ -> checkBeginChunk();
}

ValueWriter_ptr
OutputStreamImpl::valueWriter()
{
    if(is_nil(valueWriter_))
        valueWriter_ = new ValueWriter(this);

    return ValueWriter::_duplicate(valueWriter_);
}

// ----------------------------------------------------------------------
// InputStreamImpl private and protected member implementations
// ----------------------------------------------------------------------

TypeCode_ptr
InputStreamImpl::checkCache(const char* id, Octet* startPos, ULong length)
{
    assert(id && cache_);

    TypeCode_var tc;

    if(*id != '\0' && cache_ -> get(id, tc))
    {
	_OB_skip(length + 8 + startPos - buf_ -> cur_);
    }

    return tc._retn();
}

TypeCode_ptr
InputStreamImpl::readTypeCodeImpl(ReadTCHistory& history, bool isTopLevel)
{
    TCKind kind = (TCKind)read_ulong();
    Octet* oldPos = buf_ -> cur_ - 4;

    TypeCode_var tc;
    if(kind == (TCKind)0xffffffff)
    {
	Long offs = read_long();
	Octet* indirectionPos = buf_ -> cur_ - 4 + offs;
	// Adjust for alignment
	indirectionPos += ULong((long)indirectionPos) % 4;
	TypeCode_ptr p;
	if(!history.get(indirectionPos, p))
	    throw MARSHAL(MinorReadInvTypeCodeIndirection, COMPLETED_NO);
	history.put(oldPos, p);
	tc = TypeCode::_duplicate(p);
    }
    else
    {
        switch(kind)
        {
        case tk_null:
	{
	    tc = TypeCode::_duplicate(_tc_null);
            history.put(oldPos, tc);
	    break;
	}

        case tk_void:
	{
	    tc = TypeCode::_duplicate(_tc_void);
            history.put(oldPos, tc);
	    break;
	}

        case tk_short:
	{
	    tc = TypeCode::_duplicate(_tc_short);
            history.put(oldPos, tc);
	    break;
	}

        case tk_long:
	{
	    tc = TypeCode::_duplicate(_tc_long);
            history.put(oldPos, tc);
	    break;
	}

        case tk_longlong:
	{
	    tc = TypeCode::_duplicate(_tc_longlong);
            history.put(oldPos, tc);
	    break;
	}

        case tk_ushort:
	{
	    tc = TypeCode::_duplicate(_tc_ushort);
            history.put(oldPos, tc);
	    break;
	}

        case tk_ulong:
	{
	    tc = TypeCode::_duplicate(_tc_ulong);
            history.put(oldPos, tc);
	    break;
	}

        case tk_ulonglong:
	{
	    tc = TypeCode::_duplicate(_tc_ulonglong);
            history.put(oldPos, tc);
	    break;
	}

        case tk_float:
	{
	    tc = TypeCode::_duplicate(_tc_float);
            history.put(oldPos, tc);
	    break;
	}

        case tk_double:
	{
	    tc = TypeCode::_duplicate(_tc_double);
            history.put(oldPos, tc);
	    break;
	}

        case tk_longdouble:
	{
	    tc = TypeCode::_duplicate(_tc_longdouble);
            history.put(oldPos, tc);
	    break;
	}

        case tk_boolean:
	{
	    tc = TypeCode::_duplicate(_tc_boolean);
            history.put(oldPos, tc);
	    break;
	}

        case tk_char:
	{
	    tc = TypeCode::_duplicate(_tc_char);
            history.put(oldPos, tc);
	    break;
	}

        case tk_wchar:
	{
	    tc = TypeCode::_duplicate(_tc_wchar);
            history.put(oldPos, tc);
	    break;
	}

        case tk_octet:
	{
	    tc = TypeCode::_duplicate(_tc_octet);
            history.put(oldPos, tc);
	    break;
	}

        case tk_any:
	{
	    tc = TypeCode::_duplicate(_tc_any);
            history.put(oldPos, tc);
	    break;
	}

        case tk_TypeCode:
	{
	    tc = TypeCode::_duplicate(_tc_TypeCode);
            history.put(oldPos, tc);
	    break;
	}

        case tk_Principal:
        {
	    tc = TypeCode::_duplicate(_tc_Principal);
            history.put(oldPos, tc);
	    break;
        }

        case tk_fixed:
        {
            UShort digits = read_ushort();
            Short scale = read_short();
            tc = new TypeCode;
	    tc -> _OB_disableGC();
            history.put(oldPos, tc);
            tc -> kind_ = kind;
            tc -> fixedDigits_ = digits;
            tc -> fixedScale_ = scale;
            break;
        }

        case tk_local_interface:
        case tk_abstract_interface:
        case tk_native:
        case tk_objref:
        {
            ULong length = read_ulong(); // encapsulation length
            bool swap = swap_;
            _OB_readEndian();

	    String_var id = read_string();

	    if(isTopLevel && cache_)
		tc = checkCache(id, oldPos, length);
	    if(is_nil(tc))
	    {
		tc = new TypeCode;
		tc -> _OB_disableGC();
		tc -> kind_ = kind;
		tc -> id_ = id;
		tc -> name_ = read_string();

		if(cache_ && *id.in() != '\0')
		    cache_ -> put(id, tc);
	    }

	    history.put(oldPos, tc);
            swap_ = swap;
            break;
        }

        case tk_struct:
        case tk_except:
        {
            ULong length = read_ulong(); // encapsulation length
            bool swap = swap_;
            _OB_readEndian();

	    String_var id = read_string();

	    if(isTopLevel && cache_)
		tc = checkCache(id, oldPos, length);
	    if(is_nil(tc))
	    {
		tc = new TypeCode;
		history.put(oldPos, tc);
		tc -> kind_ = kind;
		tc -> id_ = id;
		tc -> name_ = read_string();
		ULong num = read_ulong();
		tc -> memberNames_.length(num);
		tc -> memberTypes_.length(num);
		for(ULong i = 0 ; i < num ; i++)
		{
		    tc -> memberNames_[i] = read_string();
		    tc -> memberTypes_[i] = readTypeCodeImpl(history);
		}

		if(cache_ && *id.in() != '\0') 
		    cache_ -> put(id, tc);
	    }

            swap_ = swap;
            break;
        }

        case tk_union:
        {
            ULong length = read_ulong(); // encapsulation length
            bool swap = swap_;
            _OB_readEndian();

	    String_var id = read_string();

	    if(isTopLevel && cache_)
		tc = checkCache(id, oldPos, length);
	    if(is_nil(tc))
	    {
		tc = new TypeCode;
		history.put(oldPos, tc);
		tc -> kind_ = kind;
		tc -> id_ = id;
		tc -> name_ = read_string();
		tc -> discriminatorType_ = readTypeCodeImpl(history);
		Long defaultIndex = read_long();
		ULong num = read_ulong();
		tc -> labels_.length(num);
		tc -> memberNames_.length(num);
		tc -> memberTypes_.length(num);
		TypeCode_ptr origDisc = GetOrigType_nodup(
		    tc -> discriminatorType_
  	        );
		for(ULong i = 0 ; i < num; i++)
		{
		    if((Long)i == defaultIndex)
		    {
			tc -> labels_[i].setDefault();

			//
			// Unmarshal a dummy value of the appropriate size
			// for the discriminator type
			//
			switch(origDisc -> kind())
			{
			case tk_short:
			    read_short();
			    break;
			case tk_ushort:
			    read_ushort();
			    break;
			case tk_long:
			    read_long();
			    break;
			case tk_ulong:
			case tk_enum:
			    read_ulong();
			    break;
			case tk_longlong:
			    read_longlong();
			    break;
			case tk_ulonglong:
			    read_ulonglong();
			    break;
			case tk_boolean:
			    read_boolean();
			    break;
			case tk_char:
			    read_char();
			    break;
			default:
			    throw BAD_TYPECODE(MinorInvalidUnionDiscriminator,
					       COMPLETED_NO);
			    break;
			}
		    }
		    else
		    {
			void* val = 0;
		    
			switch(origDisc -> kind())
			{
			case tk_short:
			{
			    Short v = read_short();
			    val = new Short(v);
			    break;
			}

			case tk_ushort:
			{
			    UShort v = read_ushort();
			    val = new UShort(v);
			    break;
			}

			case tk_long:
			{
			    Long v = read_long();
			    val = new Long(v);
			    break;
			}
		
			case tk_ulong:
			case tk_enum:
			{
			    ULong v = read_ulong();
			    val = new ULong(v);
			    break;
			}
		
			case tk_longlong:
			{
			    LongLong v = read_longlong();
			    val = new LongLong(v);
			    break;
			}
		
			case tk_ulonglong:
			{
			    ULongLong v = read_ulonglong();
			    val = new ULongLong(v);
			    break;
			}
		
			case tk_boolean:
			{
			    Boolean v = read_boolean();
			    val = new Boolean(v);
			    break;
			}

			case tk_char:
			{
			    Char v = read_char();
			    val = new Char(v);
			    break;
			}

			default:
			    throw BAD_TYPECODE(MinorInvalidUnionDiscriminator,
					       COMPLETED_NO);
			    break;
			}

			tc -> labels_[i].value(origDisc -> kind(), val);
		    }
		    
		    tc -> memberNames_[i] = read_string();
		    tc -> memberTypes_[i] = readTypeCodeImpl(history);
		}

		if(cache_ && *id.in() != '\0') 
		    cache_ -> put(id, tc);
	    }

            swap_ = swap;
            break;
        }

        case tk_enum:
        {
            ULong length = read_ulong(); // encapsulation length
            bool swap = swap_;
            _OB_readEndian();

	    String_var id = read_string();

	    if(isTopLevel && cache_)
		tc = checkCache(id, oldPos, length);
	    if(is_nil(tc))
	    {
		tc = new TypeCode;
		tc -> _OB_disableGC();
		history.put(oldPos, tc);
		tc -> kind_ = kind;
		tc -> id_ = id;
		tc -> name_ = read_string();
		ULong num = read_ulong();
		tc -> memberNames_.length(num);
		for(ULong i = 0 ; i < num ; i++)
		    tc -> memberNames_[i] = read_string();

		if(cache_ && *id.in() != '\0') 
		    cache_ -> put(id, tc);
	    }

            swap_ = swap;
            break;
        }

        case tk_string:
        case tk_wstring:
        {
            ULong bound = read_ulong();
            tc = new TypeCode;
	    tc -> _OB_disableGC();
            history.put(oldPos, tc);
            tc -> kind_ = kind;
            tc -> length_ = bound;
            break;
        }

        case tk_sequence:
        case tk_array:
        {
            read_ulong(); // encapsulation length
            bool swap = swap_;
            _OB_readEndian();

            tc = new TypeCode;
            history.put(oldPos, tc);
            tc -> kind_ = kind;
            tc -> contentType_ = readTypeCodeImpl(history);
            tc -> length_ = read_ulong();

            swap_ = swap;
            break;
        }

        case tk_alias:
        {
            ULong length = read_ulong(); // encapsulation length
            bool swap = swap_;
            _OB_readEndian();

	    String_var id = read_string();

	    if(isTopLevel && cache_)
		tc = checkCache(id, oldPos, length);
	    if(is_nil(tc))
	    {
		tc = new TypeCode;
		history.put(oldPos, tc);
		tc -> kind_ = kind;
		tc -> id_ = id;
		tc -> name_ = read_string();
		tc -> contentType_ = readTypeCodeImpl(history);

		if(cache_ && *id.in() != '\0') 
		    cache_ -> put(id, tc);
	    }

            swap_ = swap;
            break;
        }

        case tk_value:
        {
            ULong length = read_ulong(); // encapsulation length
            bool swap = swap_;
            _OB_readEndian();

	    String_var id = read_string();

	    if(isTopLevel && cache_)
		tc = checkCache(id, oldPos, length);
	    if(is_nil(tc))
	    {
		tc = new TypeCode;
		history.put(oldPos, tc);
		tc -> kind_ = kind;
		tc -> id_ = id;
		tc -> name_ = read_string();
		tc -> typeModifier_ = read_short();
		tc -> concreteBaseType_ = readTypeCodeImpl(history);
		if(tc -> concreteBaseType_ -> kind() == tk_null)
		    tc -> concreteBaseType_ = TypeCode::_nil();
		ULong num = read_ulong();
		tc -> memberNames_.length(num);
		tc -> memberTypes_.length(num);
		tc -> memberVisibility_.length(num);
		for(ULong i = 0 ; i < num ; i++)
		{
		    tc -> memberNames_[i] = read_string();
		    tc -> memberTypes_[i] = readTypeCodeImpl(history);
		    tc -> memberVisibility_[i] = read_short();
		}

		if(cache_ && *id.in() != '\0') 
		    cache_ -> put(id, tc);
	    }

            swap_ = swap;
            break;
        }

        case tk_value_box:
        {
            ULong length = read_ulong(); // encapsulation length
            bool swap = swap_;
            _OB_readEndian();

	    String_var id = read_string();

	    if(isTopLevel && cache_)
		tc = checkCache(id, oldPos, length);
	    if(is_nil(tc))
	    {
		tc = new TypeCode;
		history.put(oldPos, tc);
		tc -> kind_ = kind;
		tc -> id_ = id;
		tc -> name_ = read_string();
		tc -> contentType_ = readTypeCodeImpl(history);

		if(cache_ && *id.in() != '\0') 
		    cache_ -> put(id, tc);
	    }

            swap_ = swap;
            break;
        }

        default:
            throw BAD_TYPECODE(); // unknown kind
        }
    }

    assert(!is_nil(tc));
    return tc._retn();
}

void
InputStreamImpl::checkChunk()
{
    assert(!is_nil(valueReader_));
    valueReader_ -> checkChunk();
}

ValueReader_ptr
InputStreamImpl::valueReader()
{
    if(is_nil(valueReader_))
        valueReader_ = new ValueReader(this);

    return ValueReader::_duplicate(valueReader_);
}

// ----------------------------------------------------------------------
// InputStreamImpl constructors and destructor
// ----------------------------------------------------------------------

InputStreamImpl::InputStreamImpl(OCI::BufferImpl* buf, ULong offs, bool swap,
				 const CodeConverters* codeConverters,
				 UShort GIOPVersion)
    : buf_(buf), origSwap_(swap), swap_(swap), GIOPVersion_(0x0100), cache_(0)
{
    buf_ -> cur_ = buf_ -> data_ + offs;
    origCur_ = buf_ -> cur_;

    _OB_codeConverters(codeConverters, GIOPVersion);
}

InputStreamImpl::~InputStreamImpl()
{
}

// ----------------------------------------------------------------------
// InputStreamImpl public member implementations
// ----------------------------------------------------------------------

void
InputStreamImpl::_OB_codeConverters(const CodeConverters* converters,
				    UShort GIOPVersion)
{
    if(GIOPVersion)
	GIOPVersion_ = GIOPVersion;

    charConversionRequired_ = false;
    charReaderOrConversionRequired_ = false;
    wCharConversionRequired_ = false;

    if(converters)
    {
	codeConverters_ = *converters;

	if(codeConverters_.inputCharConverter)
	{
	    charConversionRequired_ = true;
	    charReaderRequired_ =
		codeConverters_.inputCharConverter -> readerRequired();
	    charReaderOrConversionRequired_ = true;
	}

	if(codeConverters_.inputWcharConverter)
	{
	    wCharConversionRequired_ =
		codeConverters_.inputWcharConverter -> conversionRequired();
	}
    }
    else
    {
	codeConverters_.inputCharConverter = 0;
	codeConverters_.outputCharConverter = 0;
	codeConverters_.inputWcharConverter = 0;
	codeConverters_.outputWcharConverter = 0;
    }
}

void
InputStreamImpl::_OB_reset()
{
    buf_ -> cur_ = origCur_;
    swap_ = origSwap_;
}

ORBInstance_ptr
InputStreamImpl::_OB_ORBInstance()
{
    return ORBInstance::_duplicate(orbInstance_);
}

void
InputStreamImpl::_OB_ORBInstance(ORBInstance_ptr orbInstance)
{
    orbInstance_ = ORBInstance::_duplicate(orbInstance);
    
    if(orbInstance_ && orbInstance_ -> useTypeCodeCache())
    {
	//
	// Get the TypeCodeCache of this ORBInstance
	//
	cache_ = TypeCodeCache::instance();
    }
}

void
InputStreamImpl::_OB_readEndian()
{
    swap_ = (read_boolean() != Endian);
}

Long
InputStreamImpl::_OB_readLongUnchecked()
{
    //
    // The chunking code needs to read a long value without entering
    // an infinite loop
    //

    Long val;

    register Octet* newCur = buf_ -> cur_ + 3;
    newCur -= ULong((long)newCur) % 4;

    register Octet* nextCur = newCur + 4;
    if(nextCur > buf_ -> last_)
        throw MARSHAL(MinorReadLongOverflow, COMPLETED_NO);

    val = *(const Long*)newCur;
    buf_ -> cur_ = nextCur;

    if(swap_)
    {
        ULong& v = (ULong&)val;
        v = ((v << 24) | ((v & 0xff00) << 8)
             | ((v >> 8) & 0xff00) | (v >> 24));
    }

    return val;
}

void
InputStreamImpl::_OB_beginValue()
{
    ValueReader_var r = valueReader();
    r -> beginValue();
}

void
InputStreamImpl::_OB_endValue()
{
    ValueReader_var r = valueReader();
    r -> endValue();
}

void
InputStreamImpl::_OB_remarshalValue(TypeCode_ptr tc, OutputStreamImpl* out)
{
    ValueReader_var r = valueReader();
    r -> remarshalValue(tc, out);
}

DataInputStream*
InputStreamImpl::_OB_createDataInputStream()
{
    return new DataInputStream_impl(this);
}

Char
InputStreamImpl::read_char()
{
    if(valueReader_.in())
        checkChunk();

    if(buf_ -> cur_ > buf_ -> last_)
        throw MARSHAL(MinorReadCharOverflow, COMPLETED_NO);

    if(charReaderOrConversionRequired_)
    {
        CodeConverterBase* converter = codeConverters_.inputCharConverter;

	if(charReaderRequired_)
	    return converter -> convert(converter -> read_char(buf_));
	else
	    return converter -> convert((Char)(*(buf_ -> cur_++)));
    }
    else
    {
	return *(buf_ -> cur_++);
    }
}

void
InputStreamImpl::read_char_array(Char* value, ULong length)
{
    if(length)
    {
	assert_nca(value != 0, NCANullValue);

	if(valueReader_.in())
	    checkChunk();

	if(buf_ -> cur_ + length < buf_ -> cur_ ||
	   buf_ -> cur_ + length > buf_ -> last_
        )
	    throw MARSHAL(MinorReadCharArrayOverflow, COMPLETED_NO);

        if(!charConversionRequired_)
        {
            memcpy(value, buf_ -> cur_, length);
            buf_ -> cur_ += length;
        }
        else
	{
            CodeConverterBase* converter = codeConverters_.inputCharConverter;

	    for(ULong i = 0 ; i < length ; i++)
	    {
		value[i] = converter -> convert((Char)(*(buf_ -> cur_++)));
	    }
        }
    }
}

WChar
InputStreamImpl::read_wchar()
{
    if(valueReader_.in())
        checkChunk();

    if(wCharConversionRequired_)
    {
	CodeConverterBase* converter = codeConverters_.inputWcharConverter;

	WChar value;

	//
	// For GIOP 1.1 non byte-oriented wide characters are written
	// as ushort or ulong, depending on their maximum length
	// listed in the code set registry.
	//
	switch(GIOPVersion_)
	{

	case 0x0101:
	{
	    if(converter -> getFrom() -> max_bytes <= 2)
		value = (WChar)read_ushort();
	    else
		value = (WChar)read_ulong();

	    break;
	}

	default:
	{
	    ULong wcharLen = *(buf_ -> cur_++);
	    if(buf_ -> cur_ + wcharLen > buf_ -> last_)
		throw MARSHAL(MinorReadWCharOverflow, COMPLETED_NO);

	    value = converter -> read_wchar(buf_, wcharLen);

	    break;
	}

	}

	return converter -> convert(value);
    }
    //
    // UTF-16
    //
    else
    {
	switch(GIOPVersion_)
	{

	//
        // Orbix2000/ORBacus/E compatible GIOP 1.0 marshaling
        //
	case 0x0100:
	{
	    register Octet* newCur = buf_ -> cur_;
	    newCur += ULong((long)newCur) % 2;
	    buf_ -> cur_ = newCur;

	    if(newCur + 2 > buf_ -> last_)
		throw MARSHAL(MinorReadWCharOverflow, COMPLETED_NO);

	    //
	    // Assume big endian
	    //
	    WChar v;
	    v = *buf_ -> cur_++ << 8;
	    v |= *buf_ -> cur_++;

	    return v;
	}

	case 0x0101:
	{
	    return (WChar)read_ushort();
	}

	default:
	{
	    ULong wcharLen = *(buf_ -> cur_++);
	    if(buf_ -> cur_ + wcharLen > buf_ -> last_)
		throw MARSHAL(MinorReadWCharOverflow, COMPLETED_NO);

	    WChar v;
	    v = *(buf_ -> cur_++) << 8;
	    v |= *(buf_ -> cur_++);

	    return v;
	}

	}
    }
}

Short
InputStreamImpl::read_short()
{
    Short val;

    if(valueReader_.in())
        checkChunk();

    register Octet* newCur = buf_ -> cur_;
    newCur += ULong((long)newCur) % 2;

    register Octet* nextCur = newCur + 2;
    if(nextCur > buf_ -> last_)
        throw MARSHAL(MinorReadShortOverflow, COMPLETED_NO);

    val = *(const Short*)newCur;
    buf_ -> cur_ = nextCur;

    if(swap_)
    {
        UShort& v = (UShort&)val;
        v = (v << 8) | (v >> 8);
    }

    return val;
}

UShort
InputStreamImpl::read_ushort()
{
    UShort val;

    if(valueReader_.in())
        checkChunk();

    register Octet* newCur = buf_ -> cur_;
    newCur += ULong((long)newCur) % 2;

    register Octet* nextCur = newCur + 2;
    if(nextCur > buf_ -> last_)
        throw MARSHAL(MinorReadUShortOverflow, COMPLETED_NO);

    val = *(const UShort*)newCur;
    buf_ -> cur_ = nextCur;

    if(swap_)
        val = (val << 8) | (val >> 8);

    return val;
}

Long
InputStreamImpl::read_long()
{
    if(valueReader_.in())
        checkChunk();

    return _OB_readLongUnchecked();
}

ULong
InputStreamImpl::read_ulong()
{
    ULong val;

    if(valueReader_.in())
        checkChunk();

    register Octet* newCur = buf_ -> cur_ + 3;
    newCur -= ULong((long)newCur) % 4;

    register Octet* nextCur = newCur + 4;
    if(nextCur > buf_ -> last_)
        throw MARSHAL(MinorReadULongOverflow, COMPLETED_NO);

    val = *(const ULong*)newCur;
    buf_ -> cur_ = nextCur;

    if(swap_)
        val = ((val << 24) | ((val & 0xff00) << 8)
               | ((val >> 8) & 0xff00) | (val >> 24));

    return val;
}

LongLong
InputStreamImpl::read_longlong()
{
    LongLong val;

    if(valueReader_.in())
        checkChunk();

    register Octet* newCur = buf_ -> cur_ + 7;
    newCur -= ULong((long)newCur) % 8;

    register Octet* nextCur = newCur + 8;
    if(nextCur > buf_ -> last_)
        throw MARSHAL(MinorReadLongLongOverflow, COMPLETED_NO);

    val = *(const LongLong*)newCur;
    buf_ -> cur_ = nextCur;

    if(swap_)
    {
	ULongLong& v = (ULongLong&)val;
	v = ((v << 56) | ((v & 0xff00) << 40)
	     | ((v & 0xff0000) << 24) | ((v & 0xff000000) << 8)
	     | ((v >> 8) & 0xff000000) | ((v >> 24) & 0xff0000)
	     | ((v >> 40) & 0xff00) | (v >> 56));
    }

    return val;
}

ULongLong
InputStreamImpl::read_ulonglong()
{
    ULongLong val;

    if(valueReader_.in())
        checkChunk();

    register Octet* newCur = buf_ -> cur_ + 7;
    newCur -= ULong((long)newCur) % 8;

    register Octet* nextCur = newCur + 8;
    if(nextCur > buf_ -> last_)
        throw MARSHAL(MinorReadULongLongOverflow, COMPLETED_NO);

    val = *(const ULongLong*)newCur;
    buf_ -> cur_ = nextCur;

    if(swap_)
        val = ((val << 56) | ((val & 0xff00) << 40)
               | ((val & 0xff0000) << 24) | ((val & 0xff000000) << 8)
               | ((val >> 8) & 0xff000000) | ((val >> 24) & 0xff0000)
               | ((val >> 40) & 0xff00) | (val >> 56));

    return val;
}

Float
InputStreamImpl::read_float()
{
    Float val;

    if(valueReader_.in())
        checkChunk();

    register Octet* newCur = buf_ -> cur_ + 3;
    newCur -= ULong((long)newCur) % 4;

    register Octet* nextCur = newCur + 4;
    if(nextCur > buf_ -> last_)
        throw MARSHAL(MinorReadFloatOverflow, COMPLETED_NO);

    val = *(const Float*)newCur;
    buf_ -> cur_ = nextCur;

    if(swap_)
    {
        ULong* v = (ULong*)&val;
        *v = ((*v << 24) | ((*v & 0xff00) << 8)
              | ((*v >> 8) & 0xff00) | (*v >> 24));
    }

    return val;
}

Double
InputStreamImpl::read_double()
{
    Double val;

    if(valueReader_.in())
        checkChunk();

    register Octet* newCur = buf_ -> cur_ + 7;
    newCur -= ULong((long)newCur) % 8;

    register Octet* nextCur = newCur + 8;
    if(nextCur > buf_ -> last_)
        throw MARSHAL(MinorReadDoubleOverflow, COMPLETED_NO);

    val = *(const Double*)newCur;
    buf_ -> cur_ = nextCur;

    if(swap_)
    {
        ULong v0 = ((ULong*)&val)[0];
        ULong v1 = ((ULong*)&val)[1];
        v0 = ((v0 << 24) | ((v0 & 0xff00) << 8)
              | ((v0 >> 8) & 0xff00) | (v0 >> 24));
        v1 = ((v1 << 24) | ((v1 & 0xff00) << 8)
              | ((v1 >> 8) & 0xff00) | (v1 >> 24));
        ((ULong*)&val)[0] = v1;
        ((ULong*)&val)[1] = v0;
    }

    return val;
}

#if SIZEOF_LONG_DOUBLE < 12

LongDouble
InputStreamImpl::read_longdouble()
{
    throw DATA_CONVERSION(MinorLongDoubleNotSupported, COMPLETED_NO);
    return LongDouble(); // Some compilers need this
}

#else

LongDouble
InputStreamImpl::read_longdouble()
{
    LongDouble val;

    if(valueReader_.in())
        checkChunk();

    register Octet* newCur = buf_ -> cur_ + 7;
    newCur -= ULong((long)newCur) % 8;

    register Octet* nextCur = newCur + 16;
    if(nextCur > buf_ -> last_)
        throw MARSHAL(MinorReadLongDoubleOverflow, COMPLETED_NO);

#if SIZEOF_LONG_DOUBLE == 16

    val = *(const LongDouble*)newCur;
    buf_ -> cur_ = nextCur;

    if(swap_)
    {
        ULongLong v0 = ((ULongLong*)&val)[0];
        ULongLong v1 = ((ULongLong*)&val)[1];
        v0 = ((v0 << 56) | ((v0 & 0xff00) << 40)
              | ((v0 & 0xff0000) << 24) | ((v0 & 0xff000000) << 8)
              | ((v0 >> 8) & 0xff000000) | ((v0 >> 24) & 0xff0000)
              | ((v0 >> 40) & 0xff00) | (v0 >> 56));
        v1 = ((v1 << 56) | ((v1 & 0xff00) << 40)
              | ((v1 & 0xff0000) << 24) | ((v1 & 0xff000000) << 8)
              | ((v1 >> 8) & 0xff000000) | ((v1 >> 24) & 0xff0000)
              | ((v1 >> 40) & 0xff00) | (v1 >> 56));
        ((ULongLong*)&val)[0] = v1;
        ((ULongLong*)&val)[1] = v0;
    }

#endif

#if SIZEOF_LONG_DOUBLE == 12

#   ifdef WORDS_BIGENDIAN

#       error 12-bytes long doubles only supported for little endian!

#   else

    Octet tmp[18];

    if(swap_)
    {
	*(UShort*)&tmp = 0;
	memcpy(tmp + 2, newCur, 14);

        ULongLong v0 = ((ULongLong*)&tmp)[0];
        ULongLong v1 = ((ULongLong*)&tmp)[1];
        v0 = ((v0 << 56) | ((v0 & 0xff00) << 40)
              | ((v0 & 0xff0000) << 24) | ((v0 & 0xff000000) << 8)
              | ((v0 >> 8) & 0xff000000) | ((v0 >> 24) & 0xff0000)
              | ((v0 >> 40) & 0xff00) | (v0 >> 56));
        v1 = ((v1 << 56) | ((v1 & 0xff00) << 40)
              | ((v1 & 0xff0000) << 24) | ((v1 & 0xff000000) << 8)
              | ((v1 >> 8) & 0xff000000) | ((v1 >> 24) & 0xff0000)
              | ((v1 >> 40) & 0xff00) | (v1 >> 56));
        ((ULongLong*)&tmp)[0] = v1;
        ((ULongLong*)&tmp)[1] = v0;

	*(ULongLong*)(tmp + 4) >>= 1; // Adjust mantissa
	tmp[11] |= 0x80;              // Set leading significant bit

	val = *(const LongDouble*)(tmp + 4);
    }
    else
    {
	*(UShort*)&tmp[16] = 0;
	memcpy(tmp + 2, newCur + 2, 14);

	*(ULongLong*)(tmp + 6) >>= 1; // Adjust mantissa
	tmp[13] |= 0x80;              // Set leading significant bit

	val = *(const LongDouble*)(tmp + 6);
    }

    buf_ -> cur_ = nextCur;

#   endif

#endif

    return val;
}

#endif

char*
InputStreamImpl::read_string()
{
    char* s;

    if(valueReader_.in())
        checkChunk();

    ULong len = read_ulong();
    if(len == 0)
        throw MARSHAL(MinorReadStringZeroLength, COMPLETED_NO);

    if(buf_ -> cur_ + len < buf_ -> cur_ || buf_ -> cur_ + len > buf_ -> last_)
        throw MARSHAL(MinorReadStringOverflow, COMPLETED_NO);

    s = string_alloc(len - 1);
    if(!charConversionRequired_)
    {
        memcpy(s, buf_ -> cur_, len - 1);
        buf_ -> cur_ += len - 1;
    }
    else
    {
	CodeConverterBase* converter = codeConverters_.inputCharConverter;

	for(ULong i = 0 ; i < len - 1 ; i++)
	{
	    if(charReaderRequired_)
		s[i] = converter -> convert(converter -> read_char(buf_));
	    else
		s[i] = converter -> convert((Char)(*(buf_ -> cur_++)));
	}
    }

    //
    // Check for terminating null char
    //
    if(*(buf_ -> cur_++) != '\0')
    {
        string_free(s);
	throw MARSHAL(MinorReadStringNoTerminator, COMPLETED_NO);
    }

    s[len - 1] = '\0';

    if(strlen(s) + 1 != len)
    {
        string_free(s);
        throw MARSHAL(MinorReadStringNullChar, COMPLETED_NO);
    }

    return s;
}

wchar_t*
InputStreamImpl::read_wstring()
{
    wchar_t* s;

    if(valueReader_.in())
        checkChunk();

    ULong len = read_ulong();

    //
    // 15.3.2.7: For GIOP version 1.1, a wide string is encoded as an
    // unsigned long indicating the length of the string in octets or
    // unsigned integers (determined by the transfer syntax for wchar)
    // followed by the individual wide characters. Both the string length
    // and contents include a terminating null. The terminating null
    // character for a wstring is also a wide character.
    //
    // For GIOP 1.1 the length must not be 0.
    //
    switch(GIOPVersion_)
    {

    case 0x0100:
    case 0x0101:
    {
	if(!len)
	    throw MARSHAL(MinorReadWStringZeroLength, COMPLETED_NO);

	s = wstring_alloc(len);
	read_wchar_array(s, len);

	//
	// Check for terminating null wchar
	//
	if(s[len - 1])
	{
	    wstring_free(s);
	    throw MARSHAL(MinorReadWStringNoTerminator, COMPLETED_NO);
	}

	break;
    }

    default:
    {
	WString_var ws;

	if(wCharConversionRequired_)
	{
	    CodeConverterBase* converter =
		codeConverters_.inputWcharConverter;

            // Work out the number of WChars in the string
            ULong wStringLen = 0;
	    ULong currentChar = 0;
	    while(currentChar < len)
	    {
		wStringLen += converter ->
		                  count_wchar(*(buf_ -> cur_ + currentChar));
		currentChar += wStringLen;
	    }

	    ws = wstring_alloc(wStringLen);

	    ULong currentWChar = 0;

	    while(len > 0)
	    {
		ULong wcharLen = converter -> count_wchar(*(buf_ -> cur_));
		len -= wcharLen;
		if(buf_ -> cur_ + wcharLen > buf_ -> last_)
		    throw MARSHAL(MinorReadWStringOverflow, COMPLETED_NO);

		WChar c = converter -> read_wchar(buf_, wcharLen);

		c = converter -> convert(c);

		//
		// String must not contain null characters
		//
		if(!c)
		    throw MARSHAL(MinorReadWStringNullWChar, COMPLETED_NO);
		ws[currentWChar++] = c;
	    }

	    ws[currentWChar] = L'\0';
	}
	//
	// UTF-16
	//
	else
	{
	    const ULong wcharLen = 2;

	    ws = wstring_alloc(len / wcharLen);
	    wchar_t* p = ws.inout();

   	    while(len > 0)
	    {
		len -= wcharLen;
		if(buf_ -> cur_ + wcharLen > buf_ -> last_)
		    throw MARSHAL(MinorReadWStringOverflow, COMPLETED_NO);

		*p = *(buf_ -> cur_++) << 8;
		*p |= *(buf_ -> cur_++);

		//
		// String must not contain null characters
		//
		if(!(*p))
		    throw MARSHAL(MinorReadWStringNullWChar, COMPLETED_NO);

		p++;
	    }
	    *p = L'\0';
	}

	s = ws._retn();

	break;
    }

    }

    return s;
}

Object_ptr
InputStreamImpl::read_Object()
{
    if(valueReader_.in())
        checkChunk();

    IOP::IOR ior;
    IOP::IOR::_OB_unmarshal(ior, this);

    if(strlen(ior.type_id) == 0 && ior.profiles.length() == 0)
        return Object::_nil();
    else
    {
        assert(!is_nil(orbInstance_));
        ObjectFactory_var objectFactory = orbInstance_ -> getObjectFactory();
        RefCountIOR_var refIOR = new RefCountIOR(ior);
        return objectFactory -> createObject(refIOR);
    }
}

TypeCode_ptr
InputStreamImpl::read_TypeCode()
{
    //
    // NOTE:
    //
    // No data with natural alignment of greater than four octets is
    // needed for TypeCode. Therefore it is not necessary to do
    // encapsulation in a seperate buffer.
    //

    if(valueReader_.in())
        checkChunk();

    ReadTCHistory history;
    return readTypeCodeImpl(history, true);
}

Principal_ptr
InputStreamImpl::read_Principal()
{
    if(valueReader_.in())
        checkChunk();

    //
    // Read a sequence of octets
    //
    ULong len = read_ulong();
    _OB_skip(len);

    return Principal::_nil();
}

Any*
InputStreamImpl::read_any()
{
    if(valueReader_.in())
        checkChunk();

    TypeCode_var tc = read_TypeCode();
    Any_var any = new Any;
    read_any_NoTypeCode(any.inout(), tc);
    return any._retn();
}

// 
// This version of read_any is needed to avoid excessive copying
// of inout parameters
//
void
InputStreamImpl::read_any(Any& any)
{
    if(valueReader_.in())
        checkChunk();

    TypeCode_var tc = read_TypeCode();
    read_any_NoTypeCode(any, tc);
}

Fixed
InputStreamImpl::read_fixed()
{
    if(valueReader_.in())
        checkChunk();

    String_var v = string_dup("");

    String_var s = string_dup("");

    register Octet* newCur = buf_ -> cur_;
    bool first = true;
    while(true)
    {
        if(newCur > buf_ -> last_)
            throw MARSHAL(MinorReadFixedOverflow, COMPLETED_NO);

        Octet b = *newCur++;

        Octet hi = (b >> 4) & 0x0f;
        if(hi > 9)
            throw MARSHAL(MinorReadFixedInvalid, COMPLETED_NO);

	//
	// 0 as high nibble is only valid if it's not the first nibble
	//
	if(!first || hi)
	    v += (char)(hi + '0');

        Octet lo = b & 0x0f;
        if(lo < 10)
            v += (char)(lo + '0');
        else if(lo == 0x0c || lo == 0x0d)
        {
            if(lo == 0x0d)
                s = string_dup("-");
            break;
        }
        else
            throw MARSHAL(MinorReadFixedInvalid, COMPLETED_NO);

	first = false;
    }

    buf_ -> cur_ = newCur;

    s += v;
    if(strlen(s) == 0)
	s = string_dup("0");

    return Fixed((const char*)s);
}

ValueBase*
InputStreamImpl::read_value(const char* id)
{
    ValueReader_var r = valueReader();
    return r -> readValue(id);
}

void
InputStreamImpl::read_value(Any& any, TypeCode_ptr tc)
{
    ValueReader_var r = valueReader();
    r -> readValueAny(any, tc);
}

AbstractBase_ptr
InputStreamImpl::read_abstract_interface()
{
    ValueReader_var r = valueReader();
    return r -> readAbstractInterface();
}

void
InputStreamImpl::read_wchar_array(WChar* value, ULong length)
{
    if(length)
    {
        assert_nca(value != 0, NCANullValue);

        if(valueReader_.in())
            checkChunk();

        for(ULong i = 0 ; i < length ; i++)
            value[i] = read_wchar();
    }
}

void
InputStreamImpl::read_short_array(Short* value, ULong length)
{
    if(length)
    {
        assert_nca(value != 0, NCANullValue);

        if(valueReader_.in())
            checkChunk();

        register Octet* newCur = buf_ -> cur_;
        newCur += ULong((long)newCur) % 2;
        ULong arrsize = length * 2;

        register Octet* nextCur = newCur + arrsize;
        if(nextCur < buf_ -> cur_ || nextCur > buf_ -> last_)
            throw MARSHAL(MinorReadShortArrayOverflow, COMPLETED_NO);

        memcpy(value, newCur, arrsize);
        buf_ -> cur_ = nextCur;

        if(swap_)
        {
            UShort* v = (UShort*)value;
            while(length-- > 0)
            {
                *v = (*v << 8) | (*v >> 8);
                v++;
            }
        }
    }
}

void
InputStreamImpl::read_ushort_array(UShort* value, ULong length)
{
    if(length)
    {
        assert_nca(value != 0, NCANullValue);

        if(valueReader_.in())
            checkChunk();

        register Octet* newCur = buf_ -> cur_;
        newCur += ULong((long)newCur) % 2;
        ULong arrsize = length * 2;

        register Octet* nextCur = newCur + arrsize;
        if(nextCur < buf_ -> cur_ || nextCur > buf_ -> last_)
            throw MARSHAL(MinorReadUShortArrayOverflow, COMPLETED_NO);

        memcpy(value, newCur, arrsize);
        buf_ -> cur_ = nextCur;

        if(swap_)
        {
            while(length-- > 0)
            {
                *value = (*value << 8) | (*value >> 8);
                value++;
            }
        }
    }
}

void
InputStreamImpl::read_long_array(Long* value, ULong length)
{
    if(length)
    {
        assert_nca(value != 0, NCANullValue);

        if(valueReader_.in())
            checkChunk();

        register Octet* newCur = buf_ -> cur_ + 3;
        newCur -= ULong((long)newCur) % 4;
        ULong arrsize = length * 4;

        register Octet* nextCur = newCur + arrsize;
        if(nextCur < buf_ -> cur_ || nextCur > buf_ -> last_)
            throw MARSHAL(MinorReadLongArrayOverflow, COMPLETED_NO);

        memcpy(value, newCur, arrsize);
        buf_ -> cur_ = nextCur;

        if(swap_)
        {
            ULong* v = (ULong*)value;
            while(length-- > 0)
            {
                *v = ((*v << 24) | ((*v & 0xff00) << 8)
                      | ((*v >> 8) & 0xff00) | (*v >> 24));
                v++;
            }
        }
    }
}

void
InputStreamImpl::read_ulong_array(ULong* value, ULong length)
{
    if(length)
    {
        assert_nca(value != 0, NCANullValue);

        if(valueReader_.in())
            checkChunk();

        register Octet* newCur = buf_ -> cur_ + 3;
        newCur -= ULong((long)newCur) % 4;
        ULong arrsize = length * 4;

        register Octet* nextCur = newCur + arrsize;
        if(nextCur < buf_ -> cur_ || nextCur > buf_ -> last_)
            throw MARSHAL(MinorReadULongArrayOverflow, COMPLETED_NO);

        memcpy(value, newCur, arrsize);
        buf_ -> cur_ = nextCur;

        if(swap_)
        {
            while(length-- > 0)
            {
                *value = ((*value << 24) | ((*value & 0xff00) << 8)
                      | ((*value >> 8) & 0xff00) | (*value >> 24));
                value++;
            }
        }
    }
}

void
InputStreamImpl::read_longlong_array(LongLong* value, ULong length)
{
    if(length)
    {
        assert_nca(value != 0, NCANullValue);

        if(valueReader_.in())
            checkChunk();

        register Octet* newCur = buf_ -> cur_ + 7;
        newCur -= ULong((long)newCur) % 8;
        ULong arrsize = length * 8;

        register Octet* nextCur = newCur + arrsize;
        if(nextCur < buf_ -> cur_ || nextCur > buf_ -> last_)
            throw MARSHAL(MinorReadLongLongArrayOverflow, COMPLETED_NO);

        memcpy(value, newCur, arrsize);
        buf_ -> cur_ = nextCur;

        if(swap_)
        {
            ULongLong* v = (ULongLong*)value;
            while(length-- > 0)
            {
                *v = ((*v << 56) | ((*v & 0xff00) << 40)
                      | ((*v & 0xff0000) << 24) | ((*v & 0xff000000) << 8)
                      | ((*v >> 8) & 0xff000000) | ((*v >> 24) & 0xff0000)
                      | ((*v >> 40) & 0xff00) | (*v >> 56));
                v++;
            }
        }
    }
}

void
InputStreamImpl::read_ulonglong_array(ULongLong* value, ULong length)
{
    if(length)
    {
        assert_nca(value != 0, NCANullValue);

        if(valueReader_.in())
            checkChunk();

        register Octet* newCur = buf_ -> cur_ + 7;
        newCur -= ULong((long)newCur) % 8;
        ULong arrsize = length * 8;

        register Octet* nextCur = newCur + arrsize;
        if(nextCur < buf_ -> cur_ || nextCur > buf_ -> last_)
            throw MARSHAL(MinorReadULongLongArrayOverflow, COMPLETED_NO);

        memcpy(value, newCur, arrsize);
        buf_ -> cur_ = nextCur;

        if(swap_)
        {
            ULongLong* v = value;
            while(length-- > 0)
            {
                *v = ((*v << 56) | ((*v & 0xff00) << 40)
                      | ((*v & 0xff0000) << 24) | ((*v & 0xff000000) << 8)
                      | ((*v >> 8) & 0xff000000) | ((*v >> 24) & 0xff0000)
                      | ((*v >> 40) & 0xff00) | (*v >> 56));
                v++;
            }
        }
    }
}

void
InputStreamImpl::read_float_array(Float* value, ULong length)
{
    if(length)
    {
        assert_nca(value != 0, NCANullValue);

        if(valueReader_.in())
            checkChunk();

        register Octet* newCur = buf_ -> cur_ + 3;
        newCur -= ULong((long)newCur) % 4;
        ULong arrsize = length * 4;

        register Octet* nextCur = newCur + arrsize;
        if(nextCur < buf_ -> cur_ || nextCur > buf_ -> last_)
            throw MARSHAL(MinorReadFloatArrayOverflow, COMPLETED_NO);

        memcpy(value, newCur, arrsize);
        buf_ -> cur_ = nextCur;

        if(swap_)
        {
            ULong* v = (ULong*)value;
            while(length-- > 0)
            {
                *v = ((*v << 24) | ((*v & 0xff00) << 8)
                      | ((*v >> 8) & 0xff00) | (*v >> 24));
                v++;
            }
        }
    }
}

void
InputStreamImpl::read_double_array(Double* value, ULong length)
{
    if(length)
    {
        assert_nca(value != 0, NCANullValue);

        if(valueReader_.in())
            checkChunk();

        register Octet* newCur = buf_ -> cur_ + 7;
        newCur -= ULong((long)newCur) % 8;
        ULong arrsize = length * 8;

        register Octet* nextCur = newCur + arrsize;
        if(nextCur < buf_ -> cur_ || nextCur > buf_ -> last_)
            throw MARSHAL(MinorReadDoubleArrayOverflow, COMPLETED_NO);

        memcpy(value, newCur, arrsize);
        buf_ -> cur_ = nextCur;

        if(swap_)
        {
            ULong* v = (ULong*)value;
            while(length-- > 0)
            {
                ULong v0 = v[0];
                ULong v1 = v[1];
                v0 = ((v0 << 24) | ((v0 & 0xff00) << 8)
                      | ((v0 >> 8) & 0xff00) | (v0 >> 24));
                v1 = ((v1 << 24) | ((v1 & 0xff00) << 8)
                      | ((v1 >> 8) & 0xff00) | (v1 >> 24));
                v[0] = v1;
                v[1] = v0;
                v += 2;
            }
        }
    }
}

#if SIZEOF_LONG_DOUBLE < 12

void
InputStreamImpl::read_longdouble_array(LongDouble* value, ULong length)
{
    throw DATA_CONVERSION(MinorLongDoubleNotSupported, COMPLETED_NO);
}

#else

void
InputStreamImpl::read_longdouble_array(LongDouble* value, ULong length)
{
    if(length)
    {
        assert_nca(value != 0, NCANullValue);

        if(valueReader_.in())
            checkChunk();

#if SIZEOF_LONG_DOUBLE == 16

        register Octet* newCur = buf_ -> cur_ + 7;
        newCur -= ULong((long)newCur) % 8;
        ULong arrsize = length * 16;

        register Octet* nextCur = newCur + arrsize;
        if(nextCur < buf_ -> cur_ || nextCur > buf_ -> last_)
            throw MARSHAL(MinorReadLongDoubleArrayOverflow, COMPLETED_NO);

        memcpy(value, newCur, arrsize);
        buf_ -> cur_ = nextCur;

        if(swap_)
        {
            ULongLong* v = (ULongLong*)value;
            while(length-- > 0)
            {
                ULongLong v0 = v[0];
                ULongLong v1 = v[1];
                v0 = ((v0 << 56) | ((v0 & 0xff00) << 40)
                      | ((v0 & 0xff0000) << 24) | ((v0 & 0xff000000) << 8)
                      | ((v0 >> 8) & 0xff000000) | ((v0 >> 24) & 0xff0000)
                      | ((v0 >> 40) & 0xff00) | (v0 >> 56));
                v1 = ((v1 << 56) | ((v1 & 0xff00) << 40)
                      | ((v1 & 0xff0000) << 24) | ((v1 & 0xff000000) << 8)
                      | ((v1 >> 8) & 0xff000000) | ((v1 >> 24) & 0xff0000)
                      | ((v1 >> 40) & 0xff00) | (v1 >> 56));
                v[0] = v1;
                v[1] = v0;
                v += 2;
            }
        }

#endif

#if SIZEOF_LONG_DOUBLE == 12

        ULong i;
        for(i = 0 ; i < length ; i++)
            value[i] = read_longdouble();

#endif
    }
}

#endif

void
InputStreamImpl::read_any_NoTypeCode(Any& any, TypeCode_ptr tc)
{
    any._OB_ORBInstance(orbInstance_);

    void* val = 0;
    
    TypeCode_ptr p = GetOrigType_nodup(tc);
    
    switch(p -> kind())
    {
    case tk_null:
    case tk_void:
	break;
	
    case tk_short:
    {
	Short v = read_short();
	val = new Short(v);
	break;
    }

    case tk_long:
    {
	Long v = read_long();
	val = new Long(v);
	break;
    }

    case tk_longlong:
    {
	LongLong v = read_longlong();
	val = new LongLong(v);
	break;
     }

    case tk_ushort:
    {
	UShort v = read_ushort();
	val = new UShort(v);
	break;
    }

    case tk_ulong:
    case tk_enum:
    {
	ULong v = read_ulong();
	val = new ULong(v);
	break;
    }

    case tk_ulonglong:
    {
	ULongLong v = read_ulonglong();
	val = new ULongLong(v);
	break;
    }

    case tk_float:
    {
	Float v = read_float();
	val = new Float(v);
	break;
    }

    case tk_double:
    {
	Double v = read_double();
	val = new Double(v);
	break;
    }

    case tk_longdouble:
    {
	LongDouble v = read_longdouble();
	val = new LongDouble(v);
	break;
    }

    case tk_fixed:
    {
	Fixed v = read_fixed();
	val = new Fixed(v, p -> fixed_scale());
	break;
    }

    case tk_boolean:
    {
	Boolean v = read_boolean();
	val = new Boolean(v);
	break;
    }

    case tk_char:
    {
	Char v = read_char();
	val = new Char(v);
	break;
    }

    case tk_wchar:
    {
	WChar v = read_wchar();
	val = new WChar(v);
	break;
    }

    case tk_octet:
    {
	Octet v = read_octet();
	val = new Octet(v);
	break;
    }

    case tk_any:
    {
	val = read_any();
	break;
    }

    case tk_TypeCode:
    {
	val = read_TypeCode();
	break;
    }

    case tk_Principal:
    {
	val = read_Principal();
	break;
    }
	
    case tk_objref:
    {
	val = read_Object();
	break;
    }

    case tk_struct:
    case tk_sequence:
    case tk_union:
    case tk_array:
    case tk_except:

	if(p -> _OB_isSystemException())
	{
	    val = SystemException::_OB_unmarshal(this);
	}
	else
	{
	    OCI::BufferImpl buf;
	    OutputStreamImpl out(&buf, &codeConverters_);
            out.write_InputStreamImpl(this, p);
            InputStream_ptr in = out.create_input_stream();
            in -> _OB_ORBInstance(orbInstance_);
            val = in;
	}
	break;

    case tk_string:
    {
	val = read_string();
	break;
    }
	
    case tk_wstring:
    {
	wchar_t* s = read_wstring();
	val = s;
	break;
    }

    case tk_value:
    case tk_value_box:
    case tk_abstract_interface:
    {
        read_value(any, tc);
        return;
    }

    case tk_local_interface:
    case tk_native:
	throw MARSHAL(MinorLocalObject, COMPLETED_NO);
	break;

    case tk_alias: // Alias not possible here
        assert(false);
        break;     // this is to keep the compiler happy
    }

    any.replace(tc, val, true);
}

// ----------------------------------------------------------------------
// OutputStreamImpl constructors and destructor
// ----------------------------------------------------------------------

OutputStreamImpl::OutputStreamImpl(OCI::BufferImpl* buf,
				   const CodeConverters* converters,
				   UShort GIOPVersion)
    : buf_(buf), GIOPVersion_(0x0100), alignNext_(0)
{
    if(GIOPVersion)
	GIOPVersion_ = GIOPVersion;

    charConversionRequired_ = false;
    charWriterOrConversionRequired_ = false;
    wCharConversionRequired_ = false;

    if(converters)
    {
	codeConverters_ = *converters;

	if(codeConverters_.outputCharConverter)
	{
	    charConversionRequired_ = true;
	    charWriterRequired_ =
		codeConverters_.outputCharConverter -> writerRequired();
	    charWriterOrConversionRequired_ = true;
	}

	if(codeConverters_.outputWcharConverter)
	{
	    wCharConversionRequired_ =
		codeConverters_.outputWcharConverter -> conversionRequired();
	}
    }
    else
    {
	codeConverters_.inputCharConverter = 0;
	codeConverters_.outputCharConverter = 0;
	codeConverters_.inputWcharConverter = 0;
	codeConverters_.outputWcharConverter = 0;
    }
}

OutputStreamImpl::~OutputStreamImpl()
{
}

// ----------------------------------------------------------------------
// OutputStreamImpl public member implementations
// ----------------------------------------------------------------------

void
OutputStreamImpl::_OB_beginValue(Long tag, const OB::StrSeq< int >& ids,
                                 bool chunked)
{
    ValueWriter_var w = valueWriter();
    w -> beginValue(tag, ids, chunked);
}

void
OutputStreamImpl::_OB_endValue()
{
    ValueWriter_var w = valueWriter();
    w -> endValue();
}

DataOutputStream*
OutputStreamImpl::_OB_createDataOutputStream()
{
    return new DataOutputStream_impl(this);
}

InputStream_ptr
OutputStreamImpl::create_input_stream()
{
    OCI::Buffer_var buf = new OCI::Buffer(buf_ -> len_);
    memcpy(buf -> data_, buf_ -> data_, buf_ -> len_);
    return new InputStream(buf, 0, false, &codeConverters_);
}

void
OutputStreamImpl::write_char(Char value)
{
    addCapacity(1);

    if(charWriterOrConversionRequired_)
    {
        CodeConverterBase* converter = codeConverters_.outputCharConverter;

	if(charWriterRequired_)
	    converter -> write_char(buf_, converter -> convert(value));
	else
	    *(buf_ -> cur_++) = converter -> convert(value);
    }
    else
        *(buf_ -> cur_++) = (Octet)value;
}

void
OutputStreamImpl::write_char_array(const Char* value, ULong length)
{
    if(length)
    {
	assert_nca(value != 0, NCANullValue);
	addCapacity(length);

        if(!charConversionRequired_)
        {
            memcpy(buf_ -> cur_, value, length);
            buf_ -> cur_ += length;
        }
        else
	{
	    CodeConverterBase* converter =
		codeConverters_.outputCharConverter;

	    for(ULong i = 0 ; i < length ; i++)
	    {
		*(buf_ -> cur_) = (Octet)converter -> convert(value[i]);
            }
        }
    }
}

void
OutputStreamImpl::write_wchar(WChar value)
{
    if(wCharConversionRequired_)
    {
	CodeConverterBase* converter = codeConverters_.outputWcharConverter;

	WChar v = converter -> convert(value);

	//
	// For GIOP 1.1 non byte-oriented wide characters are written
	// as ushort or ulong, depending on their maximum length
	// listed in the code set registry.
	//
	switch(GIOPVersion_)
	{

	case 0x0101:
	{
	    if(converter -> getTo() -> max_bytes <= 2)
		write_ushort(v);
	    else
		write_ulong(v);

	    break;
	}

	default:
	{
	    ULong length = converter -> count_wchar(v);
	    write_octet((Octet)length);
	    addCapacity(length);
	    converter -> write_wchar(buf_, v);

	    break;
	}

	}
    }
    //
    // UTF-16
    //
    else
    {
	switch(GIOPVersion_)
	{

	//
        // Orbix2000/ORBacus/E compatible GIOP 1.0 marshaling
        //
	case 0x0100:
	{
	    addCapacity(2, 2);

	    //
	    // Write as big endian
	    //
	    *(buf_ -> cur_++) = (Octet)(value >> 8);
	    *(buf_ -> cur_++) = (Octet)value;

	    break;
	}

	case 0x0101:
	{
	    write_ushort(value);

	    break;
	}

	default:
	{
	    addCapacity(3);
	    *(buf_ -> cur_++) = (Octet)2;
	    *(buf_ -> cur_++) = (Octet)(value >> 8);
	    *(buf_ -> cur_++) = (Octet)value;

	    break;
	}

	}
    }
}

#if SIZEOF_LONG_DOUBLE < 12

void
OutputStreamImpl::write_longdouble(LongDouble value)
{
    throw DATA_CONVERSION(MinorLongDoubleNotSupported, COMPLETED_NO);
}

#else

void
OutputStreamImpl::write_longdouble(LongDouble value)
{
    addCapacity(16, 8);

#if SIZEOF_LONG_DOUBLE == 16

    *(LongDouble*)(buf_ -> cur_) = value;

#endif

#if SIZEOF_LONG_DOUBLE == 12

#   ifdef WORDS_BIGENDIAN

#       error 12-bytes long doubles only supported for little endian!

#   else

    register Octet* cur = buf_ -> cur_;
    *(ULongLong*)cur = 0;
    *(LongDouble*)(cur + 6) = value;
    *(cur + 13) &= 0x7f; // Clear leading significant bit
    *(ULongLong*)(cur + 6) <<= 1; // Adjust mantissa

#   endif

#endif

    buf_ -> cur_ += 16;
}

#endif

void
OutputStreamImpl::write_string(const char* value)
{
    assert_nca(value != 0, NCANullString);

    ULong len = strlen(value);

    ULong capacity = len + 1;
    write_ulong(capacity);
    addCapacity(capacity);

    if(!charWriterOrConversionRequired_)
    {
        memcpy(buf_ -> cur_, value, len);
        buf_ -> cur_ += len;
    }
    else
    {
        CodeConverterBase* converter = codeConverters_.outputCharConverter;

	for(ULong i = 0 ; i < len ; i++)
	{
	    if(!value[i])
	    {
		String_var err = string_dup("illegal char value for string: ");
		err += value[i];
		throw DATA_CONVERSION(err);
	    }

	    if(charWriterRequired_)
		converter -> write_char(buf_, converter -> convert(value[i]));
	    else
		*(buf_ -> cur_++) = (Octet)converter -> convert(value[i]);
	}
    }

    *(buf_ -> cur_++) = '\0';
}

void
OutputStreamImpl::write_wstring(const wchar_t* value)
{
    assert_nca(value != 0, NCANullString);
    ULong len = OB::wcslen(value);

    //
    // 15.3.2.7: For GIOP version 1.1, a wide string is encoded as an
    // unsigned long indicating the length of the string in octets or
    // unsigned integers (determined by the transfer syntax for wchar)
    // followed by the individual wide characters. Both the string length
    // and contents include a terminating null. The terminating null
    // character for a wstring is also a wide character.
    //
    switch(GIOPVersion_)
    {

    case 0x0100:
    case 0x0101:
    {
	write_ulong(len + 1);

	write_wchar_array(value, len);

	write_wchar(0);

	break;
    }

    default:
    {
	//
	// For octet count
	//
	ULong start = writeGap();

	if(wCharConversionRequired_)
	{
	    CodeConverterBase* converter =
		codeConverters_.outputWcharConverter;

	    for(ULong i = 0 ; i < len ; i++)
	    {
		WChar v = converter -> convert(value[i]);

		if(!v)
		{
		    String_var err =
			string_dup("illegal wchar value for wstring: ");
		    //
		    // Cast to CORBA::Long in order to display decimal value.
		    // We cannot add a wchar to a narrow string.
		    //
		    err += (CORBA::Long)value[i];
		    throw DATA_CONVERSION(err);
		}

		addCapacity(converter -> count_wchar(v));
		converter -> write_wchar(buf_, v);
	    }

	}
	//
	// UTF-16
	//
	else
	{
	    addCapacity(2 * len);

	    for(ULong i = 0 ; i < len ; i++)
	    {
		WChar v = value[i];

		*(buf_ -> cur_++) = (Octet)(v >> 8);
		*(buf_ -> cur_++) = (Octet)v;
	    }
	}

	//
	// Write octet count
	//
	writeLength(start);

	break;
    }

    }
}

void
OutputStreamImpl::write_Object(Object_ptr value)
{
    if(is_nil(value))
    {
        IOP::IOR ior;
        ior.type_id = string_dup("");
        ior._OB_marshal(this);
    }
    else
    {
	value -> _OB_marshalOrigIOR(this);
    }
}

void
OutputStreamImpl::write_TypeCode(TypeCode_ptr tc)
{
    //
    // NOTE:
    //
    // No data with natural alignment of greater than four octets is
    // needed for TypeCode. Therefore it is not necessary to do
    // encapsulation in a seperate buffer.
    //

    if(is_nil(tc))
        throw BAD_TYPECODE();

    WriteTCHistory history;
    writeTypeCodeImpl(tc, history);
}

void
OutputStreamImpl::write_Principal(Principal_ptr)
{
    //
    // Just marshal an empty sequence of octets
    //
    write_ulong(0);
}

void
OutputStreamImpl::write_any(const Any& value)
{
    write_TypeCode(value.type_);
    write_any_NoTypeCode(value);
}

void
OutputStreamImpl::write_fixed(const Fixed& value)
{
    String_var v = (value.abs()).to_string();

    ULong fixLen = (value.fixed_digits() + 2) / 2;
    addCapacity(fixLen);

    //
    // Append coded sign to value string
    //
    if(value.signum() == -1)
        v += (char)('0' + 0x0d);
    else
        v += (char)('0' + 0x0c);

    String_var s;
    if(strlen(v) % 2 != 0)
        s = string_dup("0");

    s += v;
    size_t len = strlen(s);

    register Octet* newCur = buf_ -> cur_;
    ULong i;
    for(i = 0 ; i < len - 1 ; i += 2)
    {
        *newCur = ((s[i] - '0') << 4) | (s[i + 1] - '0');
        newCur++;
    }

    buf_ -> cur_ = newCur;
}

void
OutputStreamImpl::write_value(ValueBase* value, const char* id)
{
    ValueWriter_var w = valueWriter();
    w -> writeValue(value, id);
}

void
OutputStreamImpl::write_abstract_interface(AbstractBase_ptr value)
{
    ValueWriter_var w = valueWriter();
    w -> writeAbstractInterface(value);
}

void 
OutputStreamImpl::write_boolean_array(const CORBA::Boolean* value, 
				      CORBA::ULong length)
{
    if(length)
    {
	assert_nca(value != 0, NCANullValue);
	addCapacity(length);
#if SIZEOF_BOOL == 1
	memcpy(buf_ -> cur_, value, length);
	buf_ -> cur_ += length;
#else 
	for(CORBA::ULong i = 0; i < length; i++)
	{
	    *(buf_ -> cur_) = value[i] ? 1 : 0;
	    buf_ -> cur_++;
	}
#endif
    }
}

void 
OutputStreamImpl::write_octet_array(const CORBA::Octet* value, 
				    CORBA::ULong length)
{
    if(length)
    {
	assert_nca(value != 0, NCANullValue);
	addCapacity(length);
	memcpy(buf_ -> cur_, value, length);
	buf_ -> cur_ += length;
    }
}

void 
OutputStreamImpl::write_short_array(const CORBA::Short* value, 
				    CORBA::ULong length)
{
    if(length)
    {
	assert_nca(value != 0, NCANullValue);
	CORBA::ULong arrsize = length * 2;
	addCapacity(arrsize, 2);
	memcpy(buf_ -> cur_, value, arrsize);
	buf_ -> cur_ += arrsize;
    }
}

void 
OutputStreamImpl::write_ushort_array(const CORBA::UShort* value, 
				     CORBA::ULong length)
{
    if(length)
    {
	assert_nca(value != 0, NCANullValue);
	CORBA::ULong arrsize = length * 2;
	addCapacity(arrsize, 2);
	memcpy(buf_ -> cur_, value, arrsize);
	buf_ -> cur_ += arrsize;
    }
}

void 
OutputStreamImpl::write_long_array(const CORBA::Long* value, 
				   CORBA::ULong length)
{
    if(length)
    {
	assert_nca(value != 0, NCANullValue);
	CORBA::ULong arrsize = length * 4;
	addCapacity(arrsize, 4);
	memcpy(buf_ -> cur_, value, arrsize);
	buf_ -> cur_ += arrsize;
    }
}

void 
OutputStreamImpl::write_ulong_array(const CORBA::ULong* value, 
				    CORBA::ULong length)
{
    if(length)
    {
	assert_nca(value != 0, NCANullValue);
	CORBA::ULong arrsize = length * 4;
	addCapacity(arrsize, 4);
	memcpy(buf_ -> cur_, value, arrsize);
	buf_ -> cur_ += arrsize;
    }
}

void 
OutputStreamImpl::write_longlong_array(const CORBA::LongLong* value,
				       CORBA::ULong length)
{
    if(length)
    {
	assert_nca(value != 0, NCANullValue);
	CORBA::ULong arrsize = length * 8;
	addCapacity(arrsize, 8);
	memcpy(buf_ -> cur_, value, arrsize);
	buf_ -> cur_ += arrsize;
    }
}

void 
OutputStreamImpl::write_ulonglong_array(const CORBA::ULongLong* value,
					CORBA::ULong length)
{
    if(length)
    {
	assert_nca(value != 0, NCANullValue);
	CORBA::ULong arrsize = length * 8;
	addCapacity(arrsize, 8);
	memcpy(buf_ -> cur_, value, arrsize);
	buf_ -> cur_ += arrsize;
    }
}

void 
OutputStreamImpl::write_float_array(const CORBA::Float* value, 
				    CORBA::ULong length)
{
    if(length)
    {
	assert_nca(value != 0, NCANullValue);
	CORBA::ULong arrsize = length * 4;
	addCapacity(arrsize, 4);
	memcpy(buf_ -> cur_, value, arrsize);
	buf_ -> cur_ += arrsize;
    }
}

void 
OutputStreamImpl::write_double_array(const CORBA::Double* value,
				     CORBA::ULong length)
{
    if(length)
    {
	assert_nca(value != 0, NCANullValue);
	CORBA::ULong arrsize = length * 8;
	addCapacity(arrsize, 8);
	memcpy(buf_ -> cur_, value, arrsize);
	buf_ -> cur_ += arrsize;
    }
}


void
OutputStreamImpl::write_wchar_array(const WChar* value, ULong length)
{
    if(length)
    {
        assert_nca(value != 0, NCANullValue);
        ULong i;
        for(i = 0 ; i < length ; i++)
            write_wchar(value[i]);
    }
}

#if SIZEOF_LONG_DOUBLE < 12

void
OutputStreamImpl::write_longdouble_array(const LongDouble* value, ULong length)
{
    throw DATA_CONVERSION(MinorLongDoubleNotSupported, COMPLETED_NO);
}

#else

void
OutputStreamImpl::write_longdouble_array(const LongDouble* value, ULong length)
{
    if(length)
    {
#if SIZEOF_LONG_DOUBLE == 16

        ULong arrsize = length * 16;
        addCapacity(arrsize, 8);
        memcpy(buf_ -> cur_, value, arrsize);
        buf_ -> cur_ += arrsize;

#endif

#if SIZEOF_LONG_DOUBLE == 12

        ULong i;
        for(i = 0 ; i < length ; i++)
            write_longdouble(value[i]);

#endif
    }
}

#endif

void
OutputStreamImpl::write_any_NoTypeCode(const Any& any)
{
    TypeCode_ptr p = GetOrigType_nodup(any.type_);
    
    switch(p -> kind())
    {
    case tk_null:
    case tk_void:
        break;
	    
    case tk_short:
	write_short(*(const Short*)any.value_);
	break;
	
    case tk_long:
	write_long(*(const Long*)any.value_);
	break;

    case tk_longlong:
	write_longlong(*(const LongLong*)any.value_);
	break;
	
    case tk_ushort:
	write_ushort(*(const UShort*)any.value_);
	break;
	
    case tk_ulong:
    case tk_enum:
	write_ulong(*(const ULong*)any.value_);
	break;

    case tk_ulonglong:
	write_ulonglong(*(const ULongLong*)any.value_);
	break;

    case tk_float:
	write_float(*(const Float*)any.value_);
	break;
	
    case tk_double:
	write_double(*(const Double*)any.value_);
	break;
	
    case tk_longdouble:
	write_longdouble(*(const LongDouble*)any.value_);
	break;
	
    case tk_fixed:
    {
	Fixed f(*(const Fixed*)any.value_, 0);
	write_fixed(f);
	break;
    }
	
    case tk_boolean:
	write_boolean(*(const Boolean*)any.value_);
	break;
	
    case tk_char:
	write_char(*(const Char*)any.value_);
	break;

    case tk_wchar:
	write_wchar(*(const WChar*)any.value_);
	break;

    case tk_octet:
	write_octet(*(const Octet*)any.value_);
	break;
	
    case tk_any:
	write_any(*(const Any*)any.value_);
	break;
	
    case tk_TypeCode:
	write_TypeCode((TypeCode_ptr)any.value_);
	break;
	
    case tk_Principal:
	write_Principal((Principal_ptr)any.value_);
	break;
	
    case tk_objref:
	write_Object((Object_ptr)any.value_);
	break;
	
    case tk_struct:
    case tk_sequence:
    case tk_union:
    case tk_array:
    case tk_except:

	if(p -> _OB_isSystemException())
	{
	    ((SystemException*)any.value_) -> _OB_marshal(this);
	}
	else if(any.info_)
	{
	    any.info_ -> marshal(any.value_, this);
	}
	else
	{
            InputStream_ptr in = (InputStream_ptr)any.value_;
            in -> _OB_reset();
	    write_InputStreamImpl(in, p);
	}
	break;
	
    case tk_string:
	write_string((const char*)any.value_);
	break;
	
    case tk_wstring:
	write_wstring((const wchar_t*)any.value_);
	break;

    case tk_value:
    case tk_value_box:
    case tk_abstract_interface:

	if(any.info_)
	{
	    any.info_ -> marshal(any.value_, this);
	}
	else
	{
            InputStream_ptr in = (InputStream_ptr)any.value_;
            in -> _OB_reset();
	    write_InputStreamImpl(in, p);
	}
        break;

    case tk_local_interface:
    case tk_native: // Native not allowed here
	throw MARSHAL(MinorLocalObject, COMPLETED_NO);
	break;

    case tk_alias: // Alias not possible here
        assert(false);
        break;     // this is to keep the compiler happy
    }
}

void
OutputStreamImpl::write_InputStreamImpl(InputStreamImpl* in, TypeCode_ptr tc)
{
    switch(tc -> kind())
    {
    case tk_null:
    case tk_void:
	break;
	
    case tk_boolean:
	write_boolean(in -> read_boolean());
	break;

    case tk_char:
	write_char(in -> read_char());
	break;

    case tk_octet:
	write_octet(in -> read_octet());
	break;

    case tk_wchar:
	write_wchar(in -> read_wchar());
	break;

    case tk_short:
	write_short(in -> read_short());
	break;

    case tk_ushort:
	write_ushort(in -> read_ushort());
	break;
    
    case tk_long:
	write_long(in -> read_long());
	break;
    
    case tk_ulong:
    case tk_enum:
	write_ulong(in -> read_ulong());
	break;
    
    case tk_float:
	write_float(in -> read_float());
	break;
    
    case tk_longlong:
	write_longlong(in -> read_longlong());
	break;
    
    case tk_ulonglong:
	write_ulonglong(in -> read_ulonglong());
	break;
    
    case tk_double:
	write_double(in -> read_double());
	break;

    case tk_longdouble:
	write_longdouble(in -> read_longdouble());
	break;

    case tk_fixed:
	write_fixed(in -> read_fixed());
	break;

    case tk_any:
    {
	// Don't do this:
	//
	// Any_var any = in -> read_any();
	// write_any(any);
	//
	// This is faster:

	TypeCode_var p = in -> read_TypeCode();
	write_TypeCode(p);
	write_InputStreamImpl(in, p);
	break;
    }
    
    case tk_TypeCode:
    {
        // Don't do this:
        //
        // TypeCode_var p = in -> read_TypeCode();
        // write_TypeCode(p);
        //
        // This is faster:

        TCKind kind = (TCKind)in -> read_ulong();

        //
        // An indirection is not permitted at this level
        //
        assert(kind != (TCKind)0xffffffff);

        write_ulong((ULong)kind);

        switch(kind)
        {
        case tk_null:
        case tk_void:
        case tk_short:
        case tk_long:
        case tk_ushort:
        case tk_ulong:
        case tk_float:
        case tk_double:
        case tk_boolean:
        case tk_char:
        case tk_octet:
        case tk_any:
        case tk_TypeCode:
        case tk_Principal:
        case tk_longlong:
        case tk_ulonglong:
        case tk_longdouble:
        case tk_wchar:
            break;

        case tk_fixed:
            write_ushort(in -> read_ushort());
            write_short(in -> read_short());
            break;

        case tk_objref:
        case tk_struct:
        case tk_except:
        case tk_union:
        case tk_enum:
        case tk_sequence:
        case tk_array:
        case tk_alias:
        case tk_value:
        case tk_value_box:
        case tk_abstract_interface:
        case tk_native:
	case tk_local_interface:
        {
            ULong len = in -> read_ulong();
            write_ulong(len);
            addCapacity(len);
            in -> read_octet_array(buf_ -> cur_, len);
            buf_ -> cur_ += len;
            break;
        }

        case tk_string:
        case tk_wstring:
        {
            ULong bound = in -> read_ulong();
            write_ulong(bound);
            break;
        }
        }

        break;
    }
    
    case tk_Principal:
    {
	Principal_var p = in -> read_Principal();
	write_Principal(p);
	break;
    }

    case tk_objref:
    {
	// Don't do this:
	//
	// Object_var p = in -> read_Object();
	// write_Object(p);
	//
	// This is faster:

	IOP::IOR ior;
	IOP::IOR::_OB_unmarshal(ior, in);
	ior._OB_marshal(this);
	break;
    }

    case tk_struct:
    {
	for(ULong i = 0 ; i < tc -> memberTypes_.length() ; i++)
	    write_InputStreamImpl(in, tc -> memberTypes_[i]);
	
	break;
    }
	
    case tk_except:
    {
	String_var id = in -> read_string();
	write_string(id);

	for(ULong i = 0 ; i < tc -> memberTypes_.length() ; i++)
	    write_InputStreamImpl(in, tc -> memberTypes_[i]);
	
	break;
    }
	
    case tk_union:
    {
	Long defaultIndex = tc -> default_index();
	Long memberIndex = -1;

	TypeCode_ptr discType = GetOrigType_nodup(tc -> discriminatorType_);

	switch(discType -> kind())
	{
	case tk_short:
	{
	    Short val = in -> read_short();
	    write_short(val);
	    
	    for(ULong i = 0 ; i < tc -> labels_.length() ; i++)
		if((Long)i != defaultIndex)
		    if(val == *(Short*)tc -> labels_[i].value())
		    {
			memberIndex = i;
			break;
		    }
	    
	    break;
	}

	case tk_ushort:
	{
	    UShort val = in -> read_ushort();
	    write_ushort(val);

	    for(ULong i = 0 ; i < tc -> labels_.length() ; i++)
		if((Long)i != defaultIndex)
		    if(val == *(UShort*)tc -> labels_[i].value())
		    {
			memberIndex = i;
			break;
		    }

	    break;
	}

	case tk_long:
	{
	    Long val = in -> read_long();
	    write_long(val);

	    for(ULong i = 0 ; i < tc -> labels_.length() ; i++)
		if((Long)i != defaultIndex)
		    if(val == *(Long*)tc -> labels_[i].value())
		    {
			memberIndex = i;
			break;
		    }

	    break;
	}

	case tk_ulong:
	case tk_enum:
	{
	    ULong val = in -> read_ulong();
	    write_ulong(val);

	    for(ULong i = 0 ; i < tc -> labels_.length() ; i++)
		if((Long)i != defaultIndex)
		    if(val == *(ULong*)tc -> labels_[i].value())
		    {
			memberIndex = i;
			break;
		    }

	    break;
	}

	case tk_longlong:
	{
	    LongLong val = in -> read_longlong();
	    write_longlong(val);

	    for(ULong i = 0 ; i < tc -> labels_.length() ; i++)
		if((Long)i != defaultIndex)
		    if(val == *(LongLong*)tc -> labels_[i].value())
		    {
			memberIndex = i;
			break;
		    }

	    break;
	}

	case tk_ulonglong:
	{
	    ULongLong val = in -> read_ulonglong();
	    write_ulonglong(val);

	    for(ULong i = 0 ; i < tc -> labels_.length() ; i++)
		if((ULong)i != (ULong)defaultIndex)
		    if(val == *(ULongLong*)tc -> labels_[i].value())
		    {
			memberIndex = i;
			break;
		    }

	    break;
	}

	case tk_char:
	{
	    Char val = in -> read_char();
	    write_char(val);

	    for(ULong i = 0 ; i < tc -> labels_.length() ; i++)
		if((Long)i != defaultIndex)
		    if(val == *(Char*)tc -> labels_[i].value())
		    {
			memberIndex = i;
			break;
		    }

	    break;
	}

	case tk_boolean:
	{
	    Boolean val = in -> read_boolean();
	    write_boolean(val);

	    for(ULong i = 0 ; i < tc -> labels_.length() ; i++)
		if((Long)i != defaultIndex)
		    if(val == *(Boolean*)tc -> labels_[i].value())
		    {
			memberIndex = i;
			break;
		    }

	    break;
	}

	default:
	    assert(false);
	    break;
	}

	if(memberIndex >= 0)
	    write_InputStreamImpl(in, tc -> memberTypes_[memberIndex]);
	else if(defaultIndex >= 0)
	    write_InputStreamImpl(in, tc -> memberTypes_[defaultIndex]);

	break;
    }
	
    case tk_string:
    {
	String_var s = in -> read_string();
	write_string(s);
	break;
    }

    case tk_wstring:
    {
	WString_var s = in -> read_wstring();
	write_wstring(s);
	break;
    }

    case tk_sequence:
    case tk_array:
    {
	ULong len;

	if(tc -> kind() == tk_sequence)
	{
	    len = in -> read_ulong();
	    write_ulong(len);
	}
	else
	    len = tc -> length_;
	
	if(len > 0)
	{
	    assert(!is_nil(tc -> contentType_));

	    TypeCode_ptr origContentType = 
		GetOrigType_nodup(tc -> contentType_);
	    
	    switch(origContentType -> kind())
	    {
	    case tk_null:
	    case tk_void:
		break;
		
	    case tk_short:
	    {
                ULong arrsize = len * 2;
                addCapacity(arrsize, 2);
		in -> read_short_array((Short*)(buf_ -> cur_), len);
		buf_ -> cur_ += arrsize;
		break;
	    }
	    
	    case tk_ushort:
	    {
                ULong arrsize = len * 2;
                addCapacity(arrsize, 2);
		in -> read_ushort_array((UShort*)(buf_ -> cur_), len);
		buf_ -> cur_ += arrsize;
		break;
	    }
	    
	    case tk_long:
	    {
                ULong arrsize = len * 4;
                addCapacity(arrsize, 4);
		in -> read_long_array((Long*)(buf_ -> cur_), len);
		buf_ -> cur_ += arrsize;
		break;
	    }

	    case tk_ulong:
	    {
                ULong arrsize = len * 4;
                addCapacity(arrsize, 4);
		in -> read_ulong_array((ULong*)(buf_ -> cur_), len);
		buf_ -> cur_ += arrsize;
		break;
	    }

	    case tk_float:
	    {
                ULong arrsize = len * 4;
                addCapacity(arrsize, 4);
		in -> read_float_array((Float*)(buf_ -> cur_), len);
		buf_ -> cur_ += arrsize;
		break;
	    }

	    case tk_longlong:
	    {
                ULong arrsize = len * 8;
                addCapacity(arrsize, 8);
		in -> read_longlong_array((LongLong*)(buf_ -> cur_), len);
		buf_ -> cur_ += arrsize;
		break;
	    }

	    case tk_ulonglong:
	    {
                ULong arrsize = len * 8;
                addCapacity(arrsize, 8);
		in -> read_ulonglong_array((ULongLong*)(buf_ -> cur_), len);
		buf_ -> cur_ += arrsize;
		break;
	    }
	    
	    case tk_double:
	    {
                ULong arrsize = len * 8;
                addCapacity(arrsize, 8);
		in -> read_double_array((Double*)(buf_ -> cur_), len);
		buf_ -> cur_ += arrsize;
		break;
	    }

	    case tk_longdouble:
	    {
                ULong arrsize = len * 16;
                addCapacity(arrsize, 8);
		in -> read_longdouble_array((LongDouble*)(buf_ -> cur_), len);
		buf_ -> cur_ += arrsize;
		break;
	    }

	    case tk_boolean:
	    {
                addCapacity(len);
		in -> read_boolean_array((Boolean*)(buf_ -> cur_), len);
		buf_ -> cur_ += len;
		break;
	    }

	    case tk_char:
	    {
                addCapacity(len);
		in -> read_char_array((Char*)(buf_ -> cur_), len);
		buf_ -> cur_ += len;
		break;
	    }

	    case tk_octet:
	    {
                addCapacity(len);
		in -> read_octet_array(buf_ -> cur_, len);
		buf_ -> cur_ += len;
		break;
	    }

	    case tk_wchar:
	    {
                ULong sz = sizeof(wchar_t);
                ULong offs = (sz - 1) - ((ULong((long)(buf_ -> cur_)) +
                                   (sz - 1)) % sz);
                addCapacity(offs + (len * sz));
                buf_ -> cur_ += offs;
		in -> read_wchar_array((WChar*)(buf_ -> cur_), len);
		buf_ -> cur_ += len * sz;
		break;
	    }
    
	    case tk_alias:
		assert(false);
		break;
		
	    default:
		for(ULong i = 0 ; i < len ; i++)
		    write_InputStreamImpl(in, tc -> contentType_);
		break;
	    }
	}

	break;
    }

    case tk_alias:
	write_InputStreamImpl(in, tc -> contentType_);
	break;

    case tk_value:
    case tk_value_box:
    {
        in -> _OB_remarshalValue(tc, this);
        break;
    }

    case tk_abstract_interface:
    {
        Boolean b = in -> read_boolean();
        write_boolean(b);
        if(b)
        {
            Object_var obj = in -> read_Object();
            write_Object(obj);
        }
        else
        {
            //
            // We have no TypeCode information about the valuetype,
            // so we must use _tc_ValueBase and rely on the type
            // information sent on the wire
            //
            in -> _OB_remarshalValue(_tc_ValueBase, this);
        }
        break;
    }

    case tk_local_interface:
    case tk_native:
        assert(false);
        break;
    }
}

// ----------------------------------------------------------------------
// InputStream constructors and destructor
// ----------------------------------------------------------------------

InputStream::InputStream(OCI::Buffer_ptr buf, ULong offs, bool swap,
                         const CodeConverters* codeConverters,
			 UShort GIOPVersion)
    : InputStreamImpl(buf, offs, swap, codeConverters, GIOPVersion),
      bufVar_(OCI::Buffer::_duplicate(buf))
{
}

InputStream::~InputStream()
{
}

// ----------------------------------------------------------------------
// InputStream public member implementations
// ----------------------------------------------------------------------

InputStream_ptr
InputStream::_OB_clone()
{
    InputStream_ptr result;

    OCI::Buffer_var buf = new OCI::Buffer(buf_ -> len_);
    memcpy(buf -> data_, buf_ -> data_, buf_ -> len_);
    result = new InputStream(buf, origCur_ - buf_ -> data_, origSwap_,
			     &codeConverters_);
    result -> _OB_ORBInstance(orbInstance_);
    
    return result;
}

// ----------------------------------------------------------------------
// OutputStream constructors and destructor
// ----------------------------------------------------------------------

OutputStream::OutputStream(OCI::Buffer_ptr buf,
			   const CodeConverters* codeConverters,
			   UShort GIOPVersion)
    : OutputStreamImpl(buf, codeConverters, GIOPVersion),
      bufVar_(OCI::Buffer::_duplicate(buf))
{
}

OutputStream::~OutputStream()
{
}
