// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>

#include <OB/TypeCodeCache.h>

//#define OB_TYPECODECACHE_DEBUG

using namespace CORBA;
using namespace OB;

JTCMutex TypeCodeCache::instanceMutex_;
TypeCodeCache* TypeCodeCache::instance_ = 0;

namespace OB
{

class TypeCodeCacheDestroyer
{
public:

    ~TypeCodeCacheDestroyer()
    {
	delete TypeCodeCache::instance_;
	TypeCodeCache::instance_ = 0;
    }
};

} // End of namespace OB

static TypeCodeCacheDestroyer destroyer;

// ----------------------------------------------------------------------
// TypeCodeCache constructor and destructor
// ----------------------------------------------------------------------

TypeCodeCache::TypeCodeCache()
    : cache_(63)
{
}

TypeCodeCache::~TypeCodeCache()
{
    TCCache::Enumerator e = cache_.keys();
    for(ULong i = 0 ; i < e.length() ; i++)
    {
	const char *key = e[i];

#ifdef OB_TYPECODECACHE_DEBUG
	OB_COUT << "Cache remove: " << key << OB_ENDL;
#endif

	//
	// Get the TypeCode
	//
	TypeCode_var tc;
	bool rs = cache_.get(key, tc);
	assert(rs);

	//
	// Re-enable the garbage collection of this TypeCode
	//
	tc -> _OB_enableGC();

	//
	// Remove the TypeCode from the Hashtable (we need to remove
	// each TypeCode since we need to delete the string used for
	// the key). The key could have been a String_var but this is
	// not very efficient since for each get() a
	// string_dup/string_free would have implicitly been done.
	//
	cache_.remove(key);

	//
	// Deallocates the memory used for the key
	//
	CORBA::string_free((char*)key);
    }
}

// ----------------------------------------------------------------------
// TypeCodeCache public member implementation
// ----------------------------------------------------------------------

TypeCodeCache*
TypeCodeCache::instance()
{
    //
    // Double-checked locking
    //
    if(instance_ == 0)
    {
	JTCSynchronized sync(instanceMutex_);
	if(instance_ == 0)
	{
	    instance_ = new TypeCodeCache();
	}
    }

    return instance_;
}

bool 
TypeCodeCache::get(const char* id, TypeCode_var& tc)
{
    JTCSynchronized sync(*this);

    return cache_.get(id, tc);
}

void
TypeCodeCache::put(const char* id, TypeCode_ptr tc)
{
    JTCSynchronized sync(*this);
    if(cache_.containsKey(id))
	return;

    cache_.put(string_dup(id), TypeCode::_duplicate(tc));

#ifdef OB_TYPECODECACHE_DEBUG
	OB_COUT << "Cache add: " << id << OB_ENDL;
#endif
    
    //
    // Disable the garbage collection of this TypeCode. We'll
    // re-enable the garbage collection when the TypeCode is
    // removed from the cache.
    //
    tc -> _OB_disableGC();
}
