// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/Basic.h>
#include <OB/Except.h>
#include <OB/Any.h>
#include <OB/Object.h>
#include <OB/MarshalStubImpl.h>
#include <OB/NativeTypes.h>
#include <OB/Logger.h>
#include <OB/OCI.h>
#include <OB/OBORB.h>

#include <UnknownException_impl.h>

using namespace CORBA;
using namespace OB;

// ----------------------------------------------------------------------
// UnknownExceptionInfo_impl constructor/destructor
// ----------------------------------------------------------------------

UnknownExceptionInfo_impl::UnknownExceptionInfo_impl(
    const char* operation,
    bool responseExpected,
    OCI::TransportInfo_ptr transportInfo)
    : operation_(operation), responseExpected_(responseExpected),
      transportInfo_(OCI::TransportInfo::_duplicate(transportInfo))
{
}

UnknownExceptionInfo_impl::~UnknownExceptionInfo_impl()
{
}

// ----------------------------------------------------------------------
// UnknownExceptionInfo_impl public member implementation
// ----------------------------------------------------------------------

char*
UnknownExceptionInfo_impl::operation()
{
    return string_dup(operation_);
}

CORBA::Boolean
UnknownExceptionInfo_impl::response_expected()
{
    return responseExpected_;
}

OCI::TransportInfo_ptr
UnknownExceptionInfo_impl::transport_info()
{
    return OCI::TransportInfo::_duplicate(transportInfo_);
}

char*
UnknownExceptionInfo_impl::describe_exception()
{
    throw NO_IMPLEMENT();
    return 0; // some compilers need this
}

void
UnknownExceptionInfo_impl::raise_exception()
{
    //
    // The unknown exception strategy is always called from within
    // a catch block, so it is possible to re-throw the exception
    //
    throw;
}

// ----------------------------------------------------------------------
// UnknownExceptionStrategy_impl constructor/destructor
// ----------------------------------------------------------------------

UnknownExceptionStrategy_impl::UnknownExceptionStrategy_impl(
    OBCORBA::ORB_ptr orb)
    : orb_(OBCORBA::ORB::_duplicate(orb))
{
}

UnknownExceptionStrategy_impl::~UnknownExceptionStrategy_impl()
{
}

// ----------------------------------------------------------------------
// UnknownExceptionStrategy_impl public member implementation
// ----------------------------------------------------------------------

void
UnknownExceptionStrategy_impl::unknown_exception(UnknownExceptionInfo_ptr info)
{
    String_var msg = string_dup("Servant method raised a non-CORBA exception");
            
    if(info -> response_expected())
        msg += "\nClient receives this exception as CORBA::UNKNOWN";

    String_var op = info -> operation();
    msg += "\noperation name: \"";
    msg += op;
    msg += '"';

    OCI::TransportInfo_var transportInfo = info -> transport_info();
    if(!is_nil(transportInfo))
    {
        String_var desc = transportInfo -> describe();
        msg += '\n';
        msg += desc;
    }
    else
    {
        msg += "\nCollocated method call";
    }

    Logger_var logger = orb_ -> logger();
    logger -> warning(msg);
}

void
UnknownExceptionStrategy_impl::destroy()
{
    orb_ = OBCORBA::ORB::_nil();
}
