// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#ifndef ANY_VALUE_H
#define ANY_VALUE_H

class IdlAnyValue;

//
// Forward declaration required for friend function
//
OB_STD(ostream)& operator<<(OB_STD(ostream)&, const IdlAnyValue&);

class IdlAnyValue
{
    CORBA::TCKind kind_;
    CORBA::Any value_;

    //
    // Determine the kind used for arithmetic operations
    //
    static CORBA::TCKind computeKind(const IdlAnyValue&, const IdlAnyValue&);

    //
    // Is the given kind one of the integer types?
    //
    static bool isIntegerKind(CORBA::TCKind);

    //
    // Is the given kind one of the floating point types?
    //
    static bool isFloatKind(CORBA::TCKind);

    //
    // Extract a fixed-point value
    //
    void extractFixed(CORBA::Fixed&) const;

public:
    IdlAnyValue();
    IdlAnyValue(const CORBA::Any&);
    IdlAnyValue(const IdlAnyValue&);
    ~IdlAnyValue();

    IdlAnyValue& operator=(const IdlAnyValue&);

    friend OB_STD(ostream)& operator<<(OB_STD(ostream)&, const IdlAnyValue&);

    CORBA::TCKind kind() const;

    //
    // Get a copy of the Any value
    //
    CORBA::Any* getValue() const;

    //
    // Is this value one of the integer kinds?
    //
    bool isInteger() const;

    //
    // Is this value one of the floating point kinds?
    //
    bool isFloat() const;

    //
    // Is this value a numeric kind?
    //
    bool isNumeric() const;

    //
    // Does this value equal zero?
    //
    bool isZero() const;

    //
    // Does this value hold a negative number?
    //
    bool isNegative() const;

    //
    // Can arithmetic operations be performed on these two kinds?
    //
    static bool compatible(CORBA::TCKind, CORBA::TCKind);

    //
    // Obtain the number of bits used by an integer value (ignoring sign)
    //
    CORBA::UShort numBits() const;

    //
    // DATA_CONVERSION is raised if range is exceeded
    //

    //
    // Compare for equality
    //
    bool equal(const IdlAnyValue&) const;

    //
    // Arithmetic ops
    //

    IdlAnyValue add(const IdlAnyValue&) const;
    IdlAnyValue sub(const IdlAnyValue&) const;
    IdlAnyValue mult(const IdlAnyValue&) const;
    IdlAnyValue div(const IdlAnyValue&) const;
    IdlAnyValue mod(const IdlAnyValue&) const;
    IdlAnyValue shiftLeft(const IdlAnyValue&) const;
    IdlAnyValue shiftRight(const IdlAnyValue&) const;
    IdlAnyValue bitAnd(const IdlAnyValue&) const;
    IdlAnyValue bitOr(const IdlAnyValue&) const;
    IdlAnyValue bitXor(const IdlAnyValue&) const;

    IdlAnyValue neg() const;
    IdlAnyValue twiddle() const;

    //
    // Convert the value to the given kind
    //
    IdlAnyValue convert(CORBA::TCKind) const;

    //
    // Extract a numeric value (with range checking),
    // converting if necessary
    //
    CORBA::Octet to_octet() const;
    CORBA::Short to_short() const;
    CORBA::UShort to_ushort() const;
    CORBA::Long to_long() const;
    CORBA::ULong to_ulong() const;
    CORBA::LongLong to_longlong() const;
    CORBA::ULongLong to_ulonglong() const;
    CORBA::Float to_float() const;
    CORBA::Double to_double() const;
    CORBA::LongDouble to_longdouble() const;
};

#endif
