// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#ifndef GEN_HTML_H
#define GEN_HTML_H

//
// The HTML code generator class
//
class IdlHTMLGenerator : public GenTextBase
{
    //
    // Special HTML "Pretty-Print" class
    //
    class PrettyPrint : public IdlPrettyPrint
    {
        CORBA::String_var path_;
        OB_OFSTREAM out_;

    public:

        PrettyPrint(const char*, const char*, const char*, unsigned int = 4);
        virtual ~PrettyPrint();

        const char* path() const { return path_; }
        int good() { return out_.good(); }

        virtual void printChar(char);
    };

    //
    // The top-level file name (without ".html")
    //
    static const char* top_;

    //
    // The program name
    //
    const char* prog_;

    //
    // If set, write .html files to this path
    //
    const char* outDir_;

    //
    // Cache for links
    //
    CORBA::StringSeq linkSeq_;

    //
    // Output the start or end of a code block
    //
    virtual void startBlock(IdlPrettyPrint&);
    virtual void endBlock(IdlPrettyPrint&);

    //
    // Output the start or end of extra clauses
    //
    virtual void startExtra(IdlPrettyPrint&);
    virtual void endExtra(IdlPrettyPrint&);

    //
    // Output the start or end of an index
    //
    virtual void startIndex(IdlPrettyPrint&);
    virtual void endIndex(IdlPrettyPrint&);

    //
    // Output the start or end of an object header
    //
    virtual void startObject(const char*, IdlPrettyPrint&, bool);
    virtual void endObject(const char*, IdlPrettyPrint&,
                           CORBA::Container_ptr = 0, const char* = 0);

    //
    // Output various text formatting properties
    //
    virtual void textBold(bool, IdlPrettyPrint&);
    virtual void textItalic(bool, IdlPrettyPrint&);
    virtual void textLiteral(bool, IdlPrettyPrint&);
    virtual void textUnderline(bool, IdlPrettyPrint&);
    virtual void textLarge(bool, IdlPrettyPrint&);
    virtual void textSmall(bool, IdlPrettyPrint&);

    //
    // Generate a hyperlink reference string for an object
    //
    virtual char* getLinkRef(const char*);

    //
    // Output a hyperlink reference string
    //
    virtual void putLinkRef(const char*, const char*, IdlPrettyPrint&);

    //
    // Output a hyperlink reference string
    //
    virtual void putLinkTag(const char*, const char*, IdlPrettyPrint&);

    //
    // Output a horizontal rule (line)
    //
    virtual void putRule(IdlPrettyPrint&);

    //
    // Output the title of the file
    //
    virtual void putTitle(const char*, const char*, IdlPrettyPrint&);

    //
    // Output the master definition (level 1 heading)
    //
    virtual void putLevel1(const char*, const char*, IdlPrettyPrint&);

    //
    // Output a group association (level 2 heading)
    //
    virtual void putLevel2(const char*, const char*, IdlPrettyPrint&);

    //
    // Output an index entry
    //
    virtual void putEntry(const char*, const char*, IdlPrettyPrint&, bool,
                          const char*);

    //
    // Output a single @ clause from a comment
    //
    virtual void putClause(const char*, const char*, const char*, const char*,
                           IdlPrettyPrint&, bool, bool);

    //
    // Output the comment for an object. Do this as an arbitrary
    // paragraph, the introductory sentence, or the whole thing.
    //
    virtual void putParagraph(const char*, IdlPrettyPrint&);
    virtual void putIntro(const char*, const char*, IdlPrettyPrint&, bool,
                          bool);

    //
    // Generate body for a particular type
    //
    virtual int formatModule(CORBA::Container_ptr, const char*,
                             const CORBA::Container::DescriptionSeq&,
                             IdlPrettyPrint&);
    virtual int formatInterface(CORBA::Container_ptr, const char*,
                                const CORBA::Container::DescriptionSeq&,
                                IdlPrettyPrint&);
    virtual int formatValuetype(CORBA::Container_ptr, const char*,
                                const CORBA::Container::DescriptionSeq&,
                                IdlPrettyPrint&);

public:

    IdlHTMLGenerator(const char*, CORBA::Repository_ptr, bool,
                     const IdlStringHash&, const char*, bool, bool, bool,
		     bool);

    //
    // Generate HTML code
    //
    int gen();
};

#endif
