%{

// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>
#include <OB/IFR.h>
#include <IFR1_impl.h>
#include <OB/Hashtable.h>
#include <OB/list.h>
#include <OB/seq_converter.h>

#include <Util.h>
#include <GramUtil.h>
#include <AnyValue.h>

#include <stdlib.h>
#include <ctype.h>

#ifdef WIN32
#   include <malloc.h>
#ifndef BORLAND
#   define alloca _alloca
#endif
#endif

#ifdef __DECCXX
#   include <alloca.h>
#endif

#ifdef HAVE_SSTREAM
#   include <sstream>
#else
#   if defined(HAVE_STRSTREA_H)
#       include <strstrea.h>
#   else
#       include <strstream.h>
#   endif
#endif

#define HANDLE_ERROR	if (IdlIgnoreError) YYERROR; else YYABORT

#ifdef __QNXNTO__
    using namespace std;
#endif
    
using namespace CORBA;
using namespace OB;

void yyerror(const char* s);
int yylex();

static bool CreateValueFwd(const char*, Boolean);
static bool CreateValue(const char*, const IdlValueInfo&, Boolean, Boolean);

typedef OB::VarVar<OB_STL::list<CORBA::String_var> >
    StringList_var;

typedef OB::VarVar<OB_STL::list<CORBA::Any_var> >
    AnyList_var;

typedef OB::VarVar<OB_STL::list<CORBA::StructMember_var> >
    StructMemberList_var;

typedef OB::VarVar<OB_STL::list<CORBA::UnionMember> >
    UnionMemberList_var;

typedef OB::VarVar<OB_STL::list<CORBA::InterfaceDef_var> >
    InterfaceDefList_var;

typedef OB::VarVar<OB_STL::list<CORBA::ExceptionDef_var> >
    ExceptionDefList_var;

typedef OB::VarVar<OB_STL::list<IdlDeclarator_var> >
    IdlDeclaratorList_var;

typedef OB::VarVar<OB_STL::list<CORBA::ContextIdentifier_var> >
    ContextIdentifierList_var;

typedef OB::VarVar<OB_STL::list<CORBA::ParameterDescription_var> >
    ParameterDescriptionList_var;

%}

//
// Expect shift/reduce conflicts
//
%expect 4

/*** Return type ***/

%union
{
    //
    // General types
    //
    char* string;
    wchar_t* wstring;
    CORBA::StringSeq* stringSeq;
    OB_STL::list<CORBA::String_var>* stringList;

    CORBA::Long longVal;
    CORBA::ULong uLongVal;
    CORBA::ULongLong uLongLongVal;
    CORBA::Char charVal;
    CORBA::WChar wcharVal;
    CORBA::Double doubleVal;
    CORBA::Boolean booleanVal;
    CORBA::Fixed* fixedVal;

    CORBA::Any* any;
    CORBA::AnySeq* anySeq;
    OB_STL::list<CORBA::Any_var>* anyList;

    //
    // Special types
    //
    IdlDeclarator* declarator;
    OB_STL::list<IdlDeclarator_var>* declaratorList;
    IdlValueInfo* valueInfo;

    //
    // Types from the IR
    //
    CORBA::IDLType_ptr idlType;
    CORBA::ExceptionDef_ptr exceptionDef;
    CORBA::AttributeMode attributeMode;
    CORBA::DefinitionKind definitionKind;
    CORBA::StructDef_ptr structDef;
    CORBA::StructMember* structMember;
    OB_STL::list<CORBA::StructMember_var>* structMemberList;
    CORBA::UnionDef_ptr unionDef;
    OB_STL::list<CORBA::UnionMember>* unionMemberList;
    CORBA::UnionMember* unionMember;
    CORBA::EnumDef_ptr enumDef;
    OB_STL::list<CORBA::String_var>* enumMemberList;
    CORBA::InterfaceDef_ptr interfaceDef;
    OB_STL::list<CORBA::InterfaceDef_var>* interfaceDefList;
    CORBA::OperationMode operationMode;
    CORBA::ParameterMode parameterMode;
    CORBA::ParameterDescription* parameterDescription;
    OB_STL::list<CORBA::ParameterDescription_var>* parDescriptionList;
    OB_STL::list<CORBA::ExceptionDef_var>* exceptionDefList;
    OB_STL::list<CORBA::ContextIdentifier_var>* contextIdList;
    CORBA::Visibility memberVisibility;
    CORBA::Initializer* initializer;
};

/*** Token definitions ***/

%token				IDL_ABSTRACT
%token				IDL_ANY
%token				IDL_ATTRIBUTE
%token				IDL_BOOLEAN
%token				IDL_CASE
%token				IDL_CHAR
%token				IDL_CONST
%token				IDL_CONTEXT
%token				IDL_CUSTOM
%token				IDL_DEFAULT
%token				IDL_DOUBLE
%token				IDL_ENUM
%token				IDL_EXCEPTION
%token				IDL_FACTORY
%token				IDL_FIXED
%token				IDL_FLOAT
%token				IDL_IN
%token				IDL_INOUT
%token				IDL_INTERFACE
%token                          IDL_LOCAL
%token				IDL_LONG
%token				IDL_MODULE
%token				IDL_NATIVE
%token				IDL_OBJECT
%token				IDL_OCTET
%token				IDL_ONEWAY
%token				IDL_OUT
%token				IDL_PRIVATE
%token				IDL_PUBLIC
%token				IDL_RAISES
%token				IDL_READONLY
%token				IDL_SEQUENCE
%token				IDL_SHORT
%token				IDL_STRING
%token				IDL_STRUCT
%token				IDL_SUPPORTS
%token				IDL_SWITCH
%token				IDL_TRUNCATABLE
%token				IDL_TYPEDEF
%token				IDL_UNSIGNED
%token				IDL_UNION
%token				IDL_VALUEBASE
%token				IDL_VALUETYPE
%token				IDL_VOID
%token				IDL_WCHAR
%token				IDL_WSTRING
%token				IDL_LEFT_SHIFT
%token				IDL_RIGHT_SHIFT
%token				IDL_SCOPE_DELIMITOR
%token <string>			IDL_IDENTIFIER
%token <uLongLongVal>		IDL_INTEGER_LITERAL
%token <string>			IDL_STRING_LITERAL
%token <string>		        IDL_WIDE_STRING_LITERAL
%token <charVal>		IDL_CHARACTER_LITERAL
%token <wcharVal>		IDL_WIDE_CHARACTER_LITERAL
%token <doubleVal>		IDL_FLOATING_PT_LITERAL
%token <fixedVal>               IDL_FIXED_PT_LITERAL
%token <booleanVal>		IDL_FALSE
%token <booleanVal>		IDL_TRUE

/*** Types definitions ***/

%type <any>			literal
%type <booleanVal>		boolean_literal
%type <string>		        string_literal
%type <stringList>		COMMA_string_literal_STAR
%type <wstring>		        wide_string_literal
%type <string>			interface_name
%type <stringList>              COMMA_interface_name_STAR
%type <string>			scoped_name
%type <stringList>              COMMA_scoped_name_STAR
%type <uLongVal>		positive_int_const
%type <uLongVal>		fixed_array_size
%type <interfaceDefList>	interface_inheritance_spec
%type <interfaceDefList>	interface_inheritance_spec_OPT

%type <string>			value_name
%type <stringList>		COMMA_value_name_STAR
%type <valueInfo>		value_inheritance_spec_OPT
%type <valueInfo>		value_inheritance_spec
%type <valueInfo>		value_inheritance_spec_1
%type <interfaceDefList>	value_inheritance_spec_2
%type <structMemberList>	init_param_decls_OPT
%type <structMemberList>	init_param_decls
%type <structMemberList>	COMMA_init_param_decl_STAR
%type <structMember>		init_param_decl

%type <idlType>			type_spec
%type <idlType>			const_type
%type <idlType>			simple_type_spec
%type <idlType>			param_type_spec
%type <idlType>			base_type_spec
%type <idlType>			template_type_spec
%type <idlType>			constr_type_spec
%type <idlType>			floating_pt_type
%type <idlType>			integer_type
%type <idlType>			signed_int
%type <idlType>			signed_long_int
%type <idlType>			signed_longlong_int
%type <idlType>			signed_short_int
%type <idlType>			unsigned_int
%type <idlType>			unsigned_long_int
%type <idlType>			unsigned_longlong_int
%type <idlType>			unsigned_short_int
%type <idlType>			char_type
%type <idlType>			wide_char_type
%type <idlType>			boolean_type
%type <idlType>			octet_type
%type <idlType>			any_type
%type <idlType>			object_type
%type <idlType>			sequence_type
%type <idlType>			string_type
%type <idlType>			wide_string_type
%type <idlType>			fixed_pt_type
%type <idlType>			fixed_pt_const_type
%type <idlType>			value_base_type
%type <idlType>			switch_type_spec
%type <idlType>			op_type_spec

%type <declaratorList>		declarators
%type <declaratorList>		COMMA_declarator_STAR
%type <declarator>		declarator
%type <declarator>		simple_declarator
%type <declaratorList>		COMMA_simple_declarator_STAR
%type <declarator>		complex_declarator
%type <declarator>		array_declarator
%type <declarator>		fixed_array_size_PLUS

%type <structDef>		struct_type
%type <structMemberList>	member_list
%type <structMemberList>	member_PLUS
%type <structMemberList>	member_STAR
%type <structMemberList>	member
%type <structMemberList>	member_NO_SEM

%type <unionDef>		union_type
%type <unionMemberList>		switch_body
%type <unionMemberList>		case_PLUS
%type <unionMemberList>		case
%type <unionMemberList>		case_NO_SEM
%type <anyList>			case_label_PLUS
%type <any>			case_label
%type <unionMember>		element_spec

%type <enumDef>			enum_type
%type <string>			enumerator
%type <enumMemberList>		enumerator_COMMA_enumerator_STAR
%type <enumMemberList>		COMMA_enumerator_STAR

%type <operationMode>		op_attribute
%type <operationMode>		op_attribute_OPT
%type <parDescriptionList>	COMMA_param_dcl_STAR
%type <parDescriptionList>	parameter_dcls
%type <parameterDescription>	param_dcl
%type <parameterMode>		param_attribute

%type <exceptionDefList>	raises_expr
%type <exceptionDefList>	raises_expr_OPT

%type <contextIdList>		context_expr
%type <contextIdList>		context_expr_OPT

%type <any>			const_exp
%type <any>			or_expr
%type <any>			xor_expr
%type <any>			and_expr
%type <any>			shift_expr
%type <any>			add_expr
%type <any>			mult_expr
%type <any>			unary_expr
%type <charVal>			unary_operator
%type <any>			primary_expr

%type <attributeMode>		readonly_OPT
%type <definitionKind>          interfaceType_OPT
%type <booleanVal>		truncatable_OPT
%type <memberVisibility>	public_OR_private

%%

start		: specification
		{
		}
		| /* empty */
		{
		}
		;

/* (1) */
specification	:
		{
		}
		definition_PLUS
		{
		}
		;

/* (2) */
definition_PLUS	: definition
		{
		}
		| definition definition_PLUS
		{
		}
		;

/* (2) */
definition	: definition_NO_SEM ';'
		{
		}
		| definition_NO_SEM
		{
		    IdlError("`;' missing after definition");
		}
		| error ';'
		{
		    yyerrok;
		}
		;

/* (2) */
definition_NO_SEM: type_dcl
		{
		}
		| const_dcl
		{
		}
		| except_dcl
		{
		}
		| interface
		{
		}
		| module
		{
		}
		| value
		{
		}
		;

/* (3) */
module		: IDL_MODULE IDL_IDENTIFIER
		{
		    //
		    // Get container from container stack
		    //
		    Container_var container = IdlNameMgr -> getContainer();

		    //
		    // Get name, id and version
		    //
		    Identifier_var name = $2;
		    RepositoryId_var id = IdlGetRepositoryId(name);
		    VersionSpec_var version = string_dup("1.0");

		    //
		    // Check if name already exists
		    //
		    Contained_var c;
		    try
		    {
			c = container -> lookup(name);
		    }
		    catch(const SystemException& ex)
		    {
			IdlPrintException(ex, id, name);
			// Can't continue, therefore jump to the next
			// yyerrok
			HANDLE_ERROR;
		    }

		    ModuleDef_var module;
		    if(!is_nil(c))
		    {
			//
			// Name exists, check if it is an module declaration
			//
			module = ModuleDef::_narrow(c);

			if(is_nil(module))
			{
                            if(IdlAllowModuleReopening)
                            {
                                String_var err = string_dup("`");
                                err += name;
                                err += "' is not a module";
                                IdlError(err);
                                // Can't continue, therefore jump to the next
                                // yyerrok
                                HANDLE_ERROR;
                            }
                            else
                            {
                                String_var err = string_dup("`");
                                err += name;
                                err += "' is already defined";
                                IdlError(err);
                                // Can't continue, therefore jump to the next
                                // yyerrok
                           }
			}
                        else
                        {
                            if(!IdlAllowModuleReopening)
                            {
                                String_var err = string_dup("module `");
                                err += name;
                                err += "' cannot be reopened";
                                IdlError(err);
                                // Can't continue, therefore jump to the next
                                // yyerrok
                            }                               
                        }

			//
			// The ID and version might have changed
			//
			id = module -> id();
			version = module -> version();

			//
			// Update the ID. Note: This must be done
			// after the id has been re-fetched from the
			// ModuleDef.
			//
			IdlNameMgr -> updateID(id);
		    }
		    else
		    {
			//
			// Add the name to this scope. Only allow prior
                        // use of the name when CORBA::TypeCode or
                        // CORBA::Principal has been enabled.
			//
                        bool allowPriorUse = false;
                        if(strcmp(name, "CORBA") == 0 &&
                           strcmp(id, "IDL:omg.org/CORBA:1.0") == 0 &&
                           (IdlTypeCodeEnabled || IdlPrincipalEnabled))
                        {
                            allowPriorUse = true;
                        }
                        if(!IdlNameMgr -> defineName(name, id, true, 0,
                                                     false, allowPriorUse))
			{
			    // Can't continue, therefore jump to the next
			    // yyerrok
			    HANDLE_ERROR;
			}

			//
			// Create new module
			//
			try
			{
			    module =
				container -> create_module(id, name, version);
			}
			catch(const SystemException& ex)
			{
			    IdlPrintException(ex, id, name);
			    // Can't continue, therefore jump to the next
			    // yyerrok
			    HANDLE_ERROR;
			}

			//
			// Set the container
			//
			IdlNameMgr -> setContainer(id, module);
		    }

		    //
		    // Push the container on the stack
		    //
		    IdlNameMgr -> pushContainer(id);
		}
		'{' definition_PLUS '}'
		{
		    //
		    // Check for incomplete declarations.
		    // TODO: This is probably wrong because it does not consider
		    // that the declaration may be completed in a subsequently
		    // re-opened module.
		    // On the other hand, doing so is not good style, and
		    // better a too strict check like that than no check at all.
		    // Also note that it is illegal to complete a declaration in
		    // a separate source file, which is prevented by this check.
		    //
		    IdlNameMgr -> checkIncompleteDecl();

		    //
		    // Pop container on the stack
		    //
		    IdlNameMgr -> popContainer();
		}
		;

/* (4) */
interface	: interface_dcl
		{
		}
		| forward_dcl
		{
		}
		;

/* (5) */
interface_dcl	: interface_header '{' interface_body '}'
		{
		    //
		    // Check for incomplete declarations.
		    //
		    IdlNameMgr -> checkIncompleteDecl();

		    //
		    // Pop container on the stack
		    //
		    IdlNameMgr -> popContainer();
		}
		;

/* (6) */
forward_dcl	: interfaceType_OPT IDL_INTERFACE IDL_IDENTIFIER
		{
		    DefinitionKind kind = $1;

		    //
		    // Get container from container stack
		    //
		    Container_var container = IdlNameMgr -> getContainer();

		    //
		    // Get name, id and version
		    //
		    Identifier_var name = $3;
		    RepositoryId_var id = IdlGetRepositoryId(name);
		    VersionSpec_var version = string_dup("1.0");

		    //
		    // Add the forward declaration to this scope
		    //
		    if(IdlNameMgr -> addForwardDecl(name, id))
		    {
			//
			// Check if name already exists
			//
			try
			{
			    Contained_var c = container -> lookup(name);
			    if(!is_nil(c))
			    {
				//
				// Name exists, check if it is an interface
				//
				InterfaceDef_var interf =
				    InterfaceDef::_narrow(c);
				if(is_nil(interf))
				{
				    String_var err = 
					string_dup("conflicting types "
						   "for `interface ");
				    err += name;
				    err += "'";
				    IdlError(err);
				}
				else
				{
				    //
				    // Make sure forward decl matches
				    //
				    if(kind != interf -> def_kind())
				    {
					String_var err;
					err = string_dup("interface `");
					err += name;
					err += "' does not match forward "
					    "declaration";
					IdlError(err);
				    }
				}
			    }
			    else
			    {
				//
				// Create "empty" interface as forward
				// declaration, do not put it on the container
				// stack, but put it in the list of forward
				// declared interfaces
				//
                                if(kind == dk_AbstractInterface)
                                {
                                    AbstractInterfaceDefSeq seq;
                                    AbstractInterfaceDef_var interf =
                                        container ->
                                        create_abstract_interface(id, name,
                                                                  version,
                                                                  seq);
                                }
				else if(kind == dk_LocalInterface)
				{
                                    InterfaceDefSeq seq;
                                    LocalInterfaceDef_var interf =
					container ->
					create_local_interface(id, name,
							       version,
							       seq);
				}
                                else // kind == dk_Interface
                                {
                                    InterfaceDefSeq seq;
                                    InterfaceDef_var interf = container ->
                                        create_interface(id, name,
                                                         version, seq);
                                }
			    }
			}
			catch(const SystemException& ex)
			{
			    IdlPrintException(ex, id, name);
			}
		    }
		}
		;

/* (7) */
interface_header: interfaceType_OPT IDL_INTERFACE IDL_IDENTIFIER
		  interface_inheritance_spec_OPT
		{
		    DefinitionKind kind = $1;

		    //
		    // Get container from container stack
		    //
		    Container_var container = IdlNameMgr -> getContainer();

		    //
		    // Get name, id and version
		    //
		    Identifier_var name = $3;
		    RepositoryId_var id = IdlGetRepositoryId(name);
		    VersionSpec_var version = string_dup("1.0");

		    //
		    // Get interface inheritance specification sequence
		    //
		    InterfaceDefList_var seq = $4;

		    //
		    // Get the inheritance spec as a sequence of IDs
		    //
		    StringSeq idSeq(seq -> size());
                    idSeq.length(seq -> size());
                    OB_STL::list<InterfaceDef_var>::iterator interfaceIter;
                    ULong i;
                    for(interfaceIter = seq -> begin(), i = 0;
			interfaceIter != seq -> end();
			interfaceIter++, i++)
                    {
			idSeq[i] = (*interfaceIter) -> id();
                    }

		    //
		    // Add the name to this scope
		    //
		    if(!IdlNameMgr -> defineName(name, id, true, &idSeq, true))
		    {
			// Can't continue, therefore jump to the next
			// yyerrok
			HANDLE_ERROR;
		    }

		    //
		    // Check if name already exists
		    //
		    Contained_var c;
		    try
		    {
			c = container -> lookup(name);
		    }
		    catch(const SystemException& ex)
		    {
			IdlPrintException(ex, id, name);
			// Can't continue, therefore jump to the next
			// yyerrok
			HANDLE_ERROR;
		    }

		    InterfaceDef_var interf;
		    if(!is_nil(c))
		    {
			//
			// Name exists, check if it is an interface
			// forward declaration
			//
			interf = InterfaceDef::_narrow(c);
			if(is_nil(interf))
			{	
			    String_var err = string_dup("conflicting types "
							"for `interface ");
			    err += name;
			    err += "'";
			    IdlError(err);
			    // Can't continue, therefore jump to the next
			    // yyerrok
			    HANDLE_ERROR;
			}

			//
			// Make sure forward decl matches
			//
			if(kind != interf -> def_kind())
			{
			    String_var err;
			    err = string_dup("interface `");
			    err += name;
			    err += "' does not match forward declaration";
			    IdlError(err);
			    // Can't continue, therefore jump to the next
			    // yyerrok
			    HANDLE_ERROR;
			}

			try
			{
			    //
			    // Fill interface
			    //
                            InterfaceDefSeq iDefSeq(seq -> size());
                            iDefSeq.length(seq -> size());
                            copy_to_seq(seq -> begin(), seq -> end(), iDefSeq);
			    interf -> base_interfaces(iDefSeq);
			    
			    //
			    // Move interface (this will make this interface
			    // the last entry in the container)
			    //
			    interf -> move(container, name, version);
			}
			catch(const SystemException& ex)
			{
			    IdlPrintException(ex, id, name);
			    // Can't continue, therefore jump to the next
			    // yyerrok
			    HANDLE_ERROR;
			}
		    }
		    else
		    {
			//
			// Create new interface
			//
			try
			{
                            if(kind == dk_AbstractInterface)
                            {
                                AbstractInterfaceDefSeq iDefSeq(seq -> size());
                                iDefSeq.length(seq -> size());
                                for(interfaceIter = seq -> begin(), i = 0 ;
                                    interfaceIter != seq -> end() ;
                                    interfaceIter++, i++)
                                {
                                    iDefSeq[i] = AbstractInterfaceDef::_narrow(
                                        *interfaceIter);
                                    if(is_nil(iDefSeq[i]))
                                        throw BAD_PARAM(
                                            MinorInvalidAbstractInterfaceInheritance,
                                            COMPLETED_NO);
                                }

                                interf = container ->
                                    create_abstract_interface(id, name,
                                                              version,
                                                              iDefSeq);
                            }
			    else if(kind == dk_LocalInterface)
			    {
                                InterfaceDefSeq iDefSeq(seq -> size());
                                iDefSeq.length(seq -> size());
                                copy_to_seq(seq -> begin(), seq -> end(),
                                            iDefSeq);
                                interf = container ->
                                    create_local_interface(id, name,
							   version,
							   iDefSeq);
			    }
                            else // kind == dk_Interface
                            {
                                InterfaceDefSeq iDefSeq(seq -> size());
                                iDefSeq.length(seq -> size());
                                copy_to_seq(seq -> begin(), seq -> end(),
                                            iDefSeq);

                                interf = container ->
                                    create_interface(id, name, version,
                                                     iDefSeq);
                            }
			}
			catch(const SystemException& ex)
			{
			    IdlPrintException(ex, id, name);
			    // Can't continue, therefore jump to the next
			    // yyerrok
			    HANDLE_ERROR;
			}
		    }
		    
		    //
		    // Set and push new container on the stack
		    //
		    IdlNameMgr -> setContainer(id, interf);
		    IdlNameMgr -> pushContainer(id);
		}
		;
/* (8) */
interface_body	: export_STAR
		{
		}
		;

/* (9) */
export_STAR	: export export_STAR
		{
		}
		| /* empty */
		{
		}
		;

/* (9) */
export		: export_NO_SEM ';'
		{
		}
		| export_NO_SEM
		{
		    IdlError("`;' missing after definition");
		}
		| error ';'
		{
		    yyerrok;
		}
		;

/* (9) */
export_NO_SEM	: type_dcl
		{
		}
		| const_dcl
		{
		}
		| except_dcl
		{
		}
		| attr_dcl
		{
		}
		| op_dcl
		{
		}
		;

/* (10) */
interface_inheritance_spec_OPT: interface_inheritance_spec
		{
		    $$ = $1;
		}
		| /* empty */
		{
		    $$ = new OB_STL::list<InterfaceDef_var>;
		}
		;

/* (10) */
interface_inheritance_spec: ':' interface_name COMMA_interface_name_STAR
		{
		    StringList_var scopedNameSeq = $3;
		    scopedNameSeq -> push_front($2);

		    //
		    // Get container from container stack
		    //
		    Container_var container = IdlNameMgr -> getContainer();

		    //
		    // Create interface inheritance specification sequence
		    //
		    InterfaceDefList_var iDefSeq =
		        new OB_STL::list<InterfaceDef_var>;

                    OB_STL::list<String_var>::iterator nameIter;
                    for(nameIter = scopedNameSeq -> begin();
			nameIter != scopedNameSeq -> end();
			nameIter++)
		    {
			//
			// Find scoped name
			//
			const char* scoped = *nameIter;

			IRObject_var irObj =
			    IdlFindScopedName(container, scoped);

			if(is_nil(irObj))
			    continue;

			while(true)
			{
			    AliasDef_var aliasDef = AliasDef::_narrow(irObj);
			    
			    if(is_nil(aliasDef))
				break;
			    
			    irObj = aliasDef -> original_type_def();
			}

			InterfaceDef_var interfaceDef =
			    InterfaceDef::_narrow(irObj);

			if(is_nil(interfaceDef))
			{	
			    String_var err = string_dup("`");
			    err += scoped;
			    err += "' is not an interface type";
			    IdlError(err);
			    continue;
			}

			iDefSeq -> push_back(interfaceDef);
		    }

		    $$ = iDefSeq._retn();
		}
		;

/* (11) */
COMMA_interface_name_STAR: ',' interface_name COMMA_interface_name_STAR
		{
		    $3 -> push_front($2);
		    $$ = $3;
		}
		| /* empty */
		{
		    $$ = new OB_STL::list<String_var>;
		}
		;

/* (11) */
interface_name	: scoped_name
		{
		    $$ = $1;
		}
		;

/* (12) */
COMMA_scoped_name_STAR: ',' scoped_name COMMA_scoped_name_STAR
		{
		    $3 -> push_front($2);
		    $$ = $3;
		}
		| /* empty */
		{
		    $$ = new OB_STL::list<String_var>;
		}
		;

/* (12) */
scoped_name	: IDL_IDENTIFIER
		{
		    $$ = $1;
		}
		| IDL_SCOPE_DELIMITOR IDL_IDENTIFIER
		{
		    Identifier_var ident = $2;

		    $$ = string_alloc(2 + strlen(ident));
		    strcpy($$, "::");
		    strcat($$, ident);
		}
		| scoped_name IDL_SCOPE_DELIMITOR IDL_IDENTIFIER
		{
		    Identifier_var scoped = $1;
		    Identifier_var ident = $3;

		    $$ = string_alloc(strlen(scoped) + 2 + strlen(ident));
		    strcpy($$, scoped);
		    strcat($$, "::");
		    strcat($$, ident);
		}
		;

/* (13) */
/*
value		: value_dcl
		{
		}
		| value_abs_dcl
		{
		}
		| value_box_dcl
		{
		}
		| value_forward_dcl
		{
		}
		;
*/
value		: CUSTOM_value_dcl
		{
		}
		| IDL_VALUETYPE value_dcl
		{
		}
		| value_abs_dcl
		{
		}
		| IDL_VALUETYPE value_box_dcl
		{
		}
		| ABSTRACT_value_forward_dcl
		{
		}
		| IDL_VALUETYPE value_forward_dcl
		{
		}
		;

/* (14) */
/*
value_forward_dcl: abstract_OPT IDL_VALUETYPE IDL_IDENTIFIER
		{
		}
		;
*/

/* (14) */
ABSTRACT_value_forward_dcl: IDL_ABSTRACT IDL_VALUETYPE IDL_IDENTIFIER
		{
		    Identifier_var name = $3;
		    if(!CreateValueFwd(name, true))
			HANDLE_ERROR;
		}
		;

/* (14) */
value_forward_dcl: IDL_IDENTIFIER
		{
		    Identifier_var name = $1;
		    if(!CreateValueFwd(name, false))
			HANDLE_ERROR;
		}
		;

/* (15) */
value_box_dcl	: IDL_IDENTIFIER type_spec
		{
		    //
		    // Get container from container stack
		    //
		    Container_var container = IdlNameMgr -> getContainer();

		    //
		    // Get name, id and version
		    //
		    Identifier_var name = $1;
		    RepositoryId_var id = IdlGetRepositoryId(name);
		    VersionSpec_var version = string_dup("1.0");

		    IDLType_var idlType = $2;

		    //
		    // Add the name to this scope
		    //
		    if(IdlNameMgr -> defineName(name, id))
		    {
			//
			// Create new valuebox
			//
			try
			{
			    ValueBoxDef_var box =
				container -> create_value_box(id, name,
							      version,
							      idlType);
			}
			catch(const SystemException& ex)
			{
			    IdlPrintException(ex, id, name);
			}
		    }
		}
		;

/* (16) */
value_abs_dcl	: IDL_ABSTRACT IDL_VALUETYPE IDL_IDENTIFIER
		  value_inheritance_spec_OPT
		{
		    Identifier_var name = $3;
		    IdlValueInfo_var info = $4;
		    if(!CreateValue(name, info, false, true))
			HANDLE_ERROR;
		}
		'{' export_STAR '}'
		{
		    //
		    // Pop container on the stack
		    //
		    IdlNameMgr -> popContainer();
		}
		;

/* (17) */
/*
value_dcl	: value_header '{' value_element_STAR '}'
		{
		}
		;
*/

/* (17) */
CUSTOM_value_dcl: CUSTOM_value_header '{' value_element_STAR '}'
		{
		    //
		    // Pop container on the stack
		    //
		    IdlNameMgr -> popContainer();
		}
		;

/* (17) */
value_dcl	: value_header '{' value_element_STAR '}'
		{
		    //
		    // Pop container on the stack
		    //
		    IdlNameMgr -> popContainer();
		}
		;

/* (18) */
/*
value_header	: custom_OPT IDL_VALUETYPE IDL_IDENTIFIER
		  value_inheritance_spec_OPT
		{
		}
		;
*/

/* (18) */
CUSTOM_value_header: IDL_CUSTOM IDL_VALUETYPE IDL_IDENTIFIER
		  value_inheritance_spec_OPT
		{
		    Identifier_var name = $3;
		    IdlValueInfo_var info = $4;
		    if(!CreateValue(name, info, true, false))
			HANDLE_ERROR;
		}
		;

/* (18) */
value_header	: IDL_IDENTIFIER value_inheritance_spec_OPT
		{
		    Identifier_var name = $1;
		    IdlValueInfo_var info = $2;
		    if(!CreateValue(name, info, false, false))
			HANDLE_ERROR;
		}
		;

/* (19) */
value_inheritance_spec_OPT: value_inheritance_spec
		{
		    $$ = $1;
		}
		| /* empty */
		{
		    $$ = new IdlValueInfo;
		    $$ -> is_truncatable = false;
		}
		;

/* (19) */
/*
This causes a reduce/reduce conflict:

value_inheritance_spec: value_inheritance_spec_1_OPT
			value_inheritance_spec_2_OPT
*/
value_inheritance_spec: value_inheritance_spec_1
		{
		    $$ = $1;
		}
		| value_inheritance_spec_2
		{
		    InterfaceDefList_var interfaces = $1;
		    IdlValueInfo_var info = new IdlValueInfo;
		    info -> supported_interfaces = interfaces.in();
		    info -> is_truncatable = false;
		    $$ = info._retn();
		}
		| value_inheritance_spec_1 value_inheritance_spec_2
		{
		    InterfaceDefList_var interfaces = $2;
		    $$ = $1;
		    $$ -> supported_interfaces = interfaces.in();
		}
		;

/* (19) */
value_inheritance_spec_1: ':' truncatable_OPT value_name COMMA_value_name_STAR
		{
		    IdlValueInfo_var info = new IdlValueInfo;
		    info -> is_truncatable = $2;

		    String_var baseName = $3;
		    StringList_var names = $4;

		    names -> push_front(baseName._retn());

		    //
		    // Get container from container stack
		    //
		    Container_var container = IdlNameMgr -> getContainer();

                    OB_STL::list<String_var>::iterator nameIter;
                    for(nameIter = names -> begin();
			nameIter != names -> end();
			nameIter++)
		    {
			//
			// Find scoped name
			//
			const char* scoped = *nameIter;
			IRObject_var irObj =
			    IdlFindScopedName(container, scoped);

			if(is_nil(irObj))
			    continue;

			while(true)
			{
			    AliasDef_var aliasDef = AliasDef::_narrow(irObj);

			    if(is_nil(aliasDef))
				break;

			    irObj = aliasDef -> original_type_def();
			}

			ValueDef_var valueDef = ValueDef::_narrow(irObj);

			if(is_nil(valueDef))
			{
			    String_var err = string_dup("`");
			    err += scoped;
			    err += "' is not a valuetype";
			    IdlError(err);
			    continue;
			}

			info -> base_values.push_back(valueDef);
		    }

		    $$ = info._retn();
		}
		;

/* (19) */
value_inheritance_spec_2: IDL_SUPPORTS interface_name COMMA_interface_name_STAR
		{
		    String_var name = $2;
		    StringList_var names = $3;

		    names -> push_front(name._retn());

		    //
		    // Get container from container stack
		    //
		    Container_var container = IdlNameMgr -> getContainer();

		    InterfaceDefList_var seq =
		        new OB_STL::list<InterfaceDef_var>;

		    //
		    // Validate that all names are interfaces. A valuetype
		    // can only support one concrete interface, but can
		    // support multiple abstract interfaces.
		    //
		    ULong concreteCount = 0;

                    OB_STL::list<String_var>::iterator nameIter;
		    for(nameIter = names -> begin();
                        nameIter != names -> end();
                        nameIter++)
		    {
			//
			// Find scoped name
			//
			const char* scoped = *nameIter;
			IRObject_var irObj =
			    IdlFindScopedName(container, scoped);

			if(is_nil(irObj))
			    continue;

			while(true)
			{
			    AliasDef_var aliasDef = AliasDef::_narrow(irObj);

			    if(is_nil(aliasDef))
				break;

			    irObj = aliasDef -> original_type_def();
			}

			InterfaceDef_var interfaceDef =
			    InterfaceDef::_narrow(irObj);

			if(is_nil(interfaceDef))
			{
			    String_var err = string_dup("`");
			    err += scoped;
			    err += "' is not an interface type";
			    IdlError(err);
			    continue;
			}

			if(interfaceDef -> def_kind() == dk_Interface)
			{
			    concreteCount++;
			    if(concreteCount > 1)
			    {
				String_var err =
				    string_dup("value can't support multiple "
					       "concrete interfaces `");
				err += scoped;
				err += "'";
				IdlError(err);
				continue;
			    }
			}

			seq -> push_back(interfaceDef);
		    }

		    $$ = seq._retn();
		}
		;

/* (19) */
COMMA_value_name_STAR: ',' value_name COMMA_value_name_STAR
		{
		    $3 -> push_front($2);
		    $$ = $3;
		}
		| /* empty */
		{
		    $$ = new OB_STL::list<String_var>;
		}
		;

/* (20) */
value_name	: scoped_name
		{
		    $$ = $1;
		}
		;

/* (21) */
value_element_STAR: value_element value_element_STAR
		{
		}
		| /* empty */
		{
		}
		;
/* (21) */
/*
value_element	: export
		{
		}
		| state_member
		{
		}
		| init_dcl
		{
		}
		;
*/
value_element	: export_NO_SEM ';'
		{
		}
		| export_NO_SEM
		{
		    IdlError("`;' missing after definition");
		}
		| state_member_NO_SEM ';'
		{
		}
		| state_member_NO_SEM
		{
		    IdlError("`;' missing after state member");
		}
                | init_dcl_NO_SEM ';'
		{
		}
		| init_dcl_NO_SEM
		{
		    IdlError("`;' missing after factory declaration");
		}
		| error ';'
		{
		    yyerrok;
		}
		;

/* (22) */
/*
state_member	: state_member_NO_SEM ';'
		{
		}
		| state_member_NO_SEM
		{
		    IdlError("`;' missing after state member");
		}
		| error ';'
		{
		    yyerrok;
		}
		;
*/

/* (22) */
state_member_NO_SEM: public_OR_private type_spec declarators
		{
		    //
		    // Get container from container stack
		    //
		    Container_var container = IdlNameMgr -> getContainer();

		    ValueDef_var valueDef = ValueDef::_narrow(container);
		    assert(!is_nil(valueDef));

		    Visibility access = $1;
		    IDLType_var simpleIDLType = $2;
		    IdlDeclaratorList_var declarators = $3;

                    //
                    // A valuetype state member cannot be a local type
                    //
                    TypeCode_var tc = simpleIDLType -> type();
                    if(!IdlIsMarshallable(tc))
                    {
                        IdlError("valuetype state member is not "
                                 "marshallable");
                        // Can't continue, therefore jump to the next
                        // yyerrok
                        HANDLE_ERROR;
                    }

		    Repository_var rep = IdlNameMgr -> getRepository();

                    OB_STL::list<IdlDeclarator_var>::iterator declIter;
                    for(declIter = declarators -> begin();
			declIter != declarators -> end();
			declIter++)
		    {
			const IdlDeclarator& declarator = *declIter;

			//
			// Get name, id and version
			//
			Identifier_var name = declarator.identifier;
			RepositoryId_var id = IdlGetRepositoryId(name);
			VersionSpec_var version = string_dup("1.0");

			//
			// Handle arrays
			//
			IDLType_var idlType = simpleIDLType;

                        OB_STL::list<ULong>::const_reverse_iterator sizesIter;
                        for(sizesIter = declarator.sizes.rbegin();
			    sizesIter != declarator.sizes.rend();
			    sizesIter++)
			{
			    ArrayDef_var arrayDef = rep ->
				create_array(*sizesIter, idlType);

			    idlType = IDLType::_duplicate(arrayDef);
			}

			//
			// Add the name to this scope
			//
			if(IdlNameMgr -> defineName(name, id))
			{
			    //
			    // Create member
			    //
			    try
			    {
				release(valueDef ->
					create_value_member(id, name, version,
							    idlType, access));
			    }
			    catch(const SystemException& ex)
			    {
				IdlPrintException(ex, id, name);
			    }
			}
		    }
		}
		;

/* (23) */
/*
init_dcl	: init_dcl_NO_SEM ';'
		{
		}
		| init_dcl_NO_SEM
		{
		    IdlError("`;' missing after factory declaration");
		}
		| error ';'
		{
		    yyerrok;
		}
		;
*/

/* (23) */
init_dcl_NO_SEM	: IDL_FACTORY IDL_IDENTIFIER '(' init_param_decls_OPT ')'
		{
		    if(!IdlAllowValueTypes)
		    {
			IdlError("IDL value types are not supported");
			// Can't continue, therefore jump to the next
			// yyerrok
			HANDLE_ERROR;
		    }

                    String_var name = $2;
		    StructMemberList_var members = $4;

                    if(IdlNameMgr -> defineDecl(name))
                    {
                        Initializer_var init = new Initializer;
                        init -> name = name;
                        init -> members.length(members -> size());
                        copy_to_seq(members -> begin(),
                                    members -> end(),
                                    init -> members);

                        //
                        // Get container from container stack
                        //
                        Container_var container = IdlNameMgr -> getContainer();

                        ValueDef_var valueDef = ValueDef::_narrow(container);
                        assert(!is_nil(valueDef));

                        //
                        // Add the new initializer
                        //
                        InitializerSeq_var seq = valueDef -> initializers();
                        ULong initCount = seq -> length();
                        seq -> length(initCount + 1);
                        seq[initCount] = init;
                        try
                        {
                            valueDef -> initializers(seq);
                        }
                        catch(const SystemException& ex)
                        {
                            RepositoryId_var id = valueDef -> id();
                            IdlPrintException(ex, id, init -> name);
                        }
                    }
		}
		;

/* (24) */
init_param_decls_OPT: init_param_decls
		{
		    $$ = $1;
		}
		| /* empty */
		{
		    $$ = new OB_STL::list<StructMember_var>;
		}
		;
/* (24) */
init_param_decls: init_param_decl COMMA_init_param_decl_STAR
		{
		    StructMember_var member = $1;
		    $2 -> push_front(member);
		    $$ = $2;
		}
		;

/* (25) */
COMMA_init_param_decl_STAR: ',' init_param_decl COMMA_init_param_decl_STAR
		{
		    StructMember_var member = $2;
		    $3 -> push_front(member);
		    $$ = $3;
		}
		| /* empty */
		{
		    $$ = new OB_STL::list<StructMember_var>;
		}
		;
/* (25) */
init_param_decl: init_param_attribute param_type_spec simple_declarator
		{
		    StructMember_var member = new StructMember;
		    IdlDeclarator_var declarator = $3;
		    assert(declarator -> sizes.size() == 0);
		    member -> name = declarator -> identifier;
		    member -> type_def = $2;
		    member -> type = $2 -> type();
		    $$ = member._retn();
		}
		;

/* (26) */
init_param_attribute: IDL_IN
		{
		}
		;

/* (27) */
const_dcl	: IDL_CONST const_type IDL_IDENTIFIER '=' const_exp
		{
		    //
		    // Get type and value
		    //
		    IDLType_var idlType = $2;
		    Any_var any = $5;

		    //
		    // Get name, id and version
		    //
		    Identifier_var name = $3;
		    RepositoryId_var id = IdlGetRepositoryId(name);
		    VersionSpec_var version = string_dup("1.0");

		    //
		    // Check if conversion is necessary
		    //
		    TypeCode_var tc1 = idlType -> type();
		    TypeCode_var origTC1 = GetOrigType(tc1);
		    TypeCode_var tc2 = any -> type();
		    TypeCode_var origTC2 = GetOrigType(tc2);
		    bool ok = true;

		    if(!origTC1 -> equivalent(origTC2))
		    {
			const char* err =
			    "wrong type for constant initialization";
		    
                        try
                        {
                            //
                            // Convert value type
                            //
                            switch(origTC1 -> kind())
                            {
                            case tk_octet:
                            case tk_short:
                            case tk_ushort:
                            case tk_long:
                            case tk_ulong:
                            case tk_longlong:
                            case tk_ulonglong:
                            {
                                IdlAnyValue val(any);

                                if(!val.isInteger())
                                {
                                    IdlError(err);
				    ok = false;
                                }
				else
				{
				    IdlAnyValue av =
					val.convert(origTC1 -> kind());
				    any = av.getValue();
				}
				break;
                            }

                            case tk_float:
                            case tk_double:
                            case tk_longdouble:
                            {
                                IdlAnyValue val(any);

                                if(!val.isFloat())
                                {
                                    IdlError(err);
				    ok = false;
                                }
				else
				{
				    IdlAnyValue av =
					val.convert(origTC1 -> kind());
				    any = av.getValue();
				}
				break;
                            }

                            case tk_fixed:
                            {
                                if(origTC2 -> kind() != tk_fixed)
                                {
                                    IdlError(err);
				    ok = false;
                                }
				else
				{
                                    //
                                    // The fixed_pt_const_type rule creates
                                    // a temporary object that we replace
                                    // with one that reflects the digits and
                                    // scale of the actual value.
                                    //
                                    // Note: Don't destroy the IDLType if
                                    // the kind is tk_alias.
                                    //
                                    if(tc1 -> kind() != tk_alias)
                                        idlType -> destroy();
				    Repository_var rep =
					IdlNameMgr -> getRepository();
				    idlType = rep -> create_fixed(
					origTC2 -> fixed_digits(),
					origTC2 -> fixed_scale());
				}
				break;
                            }

                            case tk_string:
                            {
                                const char* s;
                                if(any >>= s)
                                {
                                    ULong len = origTC1 -> length();
				    if(strlen(s) > len)
				    {
					IdlError("string is too large for "
						 "bounded string constant");
					ok = false;
				    }
				    else
				    {
					Any::from_string from(s, len);
					any <<= from;
				    }
				}
                                else
                                {
                                    IdlError(err);
				    ok = false;
                                }
                                break;
                            }

                            case tk_wstring:
                            {
                                const wchar_t* s;
                                if(any >>= s)
                                {
                                    ULong len = origTC1 -> length();
				    if(OB::wcslen(s) > len)
				    {
					IdlError("wide string is too large "
						 "for bounded wide string "
						 "constant");
					ok = false;
				    }
				    else
				    {
					ULong len = origTC1 -> length();
					Any::from_wstring from(s, len);
					any <<= from;
				    }
                                }
                                else
                                {
                                    IdlError(err);
				    ok = false;
                                }
                                break;
                            }

                            default:
                                IdlError(err);
				ok = false;
                                break;
                            }
                        }
                        catch(const DATA_CONVERSION&)
                        {
                            IdlError("constant value exceeds allowable "
                                     "range");
			    ok = false;
                        }
		    }

		    if(ok)
		    {
			//
			// Get container from container stack
			//
			Container_var container = IdlNameMgr -> getContainer();
			
			//
			// Add the name to this scope
			//
			if(IdlNameMgr -> defineName(name, id))
			{
			    //
			    // Create constant definition
			    //
			    try
			    {
                                TypeCode_var p = idlType -> type();
                                any -> type(p);
				release(container ->
					create_constant(id, name, version,
							idlType, any));
			    }
			    catch(const SystemException& ex)
			    {
				IdlPrintException(ex, id, name);
			    }
			}
		    }
		}
		;

/* (28) */
const_type	: integer_type
		{
		    $$ = $1;
		}
		| char_type
		{
		    $$ = $1;
		}
		| wide_char_type
		{
		    $$ = $1;
		}
		| boolean_type
		{
		    $$ = $1;
		}
		| floating_pt_type
		{
		    $$ = $1;
		}
		| string_type
		{
		    $$ = $1;
		}
		| wide_string_type
		{
		    $$ = $1;
		}
		| fixed_pt_const_type
		{
		    $$ = $1;
		}
		| scoped_name
		{
		    ScopedName_var scoped = $1;

		    //
		    // Get container from container stack
		    //
		    Container_var container = IdlNameMgr -> getContainer();

		    //
		    // Find scoped name
		    //
		    IRObject_var irObj = IdlFindScopedName(container, scoped);

		    if(is_nil(irObj))
		    {
			// Can't continue, therefore jump to the next
			// yyerrok
			HANDLE_ERROR;
		    }

		    bool valid = false;

		    IDLType_var idlType = IDLType::_narrow(irObj);
		    if(!is_nil(idlType))
		    {
			TypeCode_var tc = idlType -> type();
			tc = GetOrigType(tc);

			TCKind k = tc -> kind();

			if(k == tk_short || k == tk_ushort ||
			   k == tk_long || k == tk_ulong ||
			   k == tk_longlong || k == tk_ulonglong ||
			   k == tk_float || k == tk_double ||
			   k == tk_longdouble || k == tk_char ||
                           k == tk_boolean || k == tk_octet ||
                           k == tk_wchar || k == tk_fixed || k == tk_enum ||
			   k == tk_string || k == tk_wstring)
			    valid = true;
		    }

		    if(!valid)
		    {
			String_var err = string_dup("`");
			err += scoped;
			err += "' is not a valid constant type";
			IdlError(err);
			// Can't continue, therefore jump to the next
			// yyerrok
			HANDLE_ERROR;
		    }

		    $$ = idlType._retn();
		}
		| octet_type
		{
		    $$ = $1;
		}
		;

/* (29) */
const_exp	: or_expr
		{
		    $$ = $1;
		}
		;

/* (30) */
or_expr		: xor_expr
		{
		    $$ = $1;
		}
		| or_expr '|' xor_expr
		{
		    Any_var left = $1;
		    Any_var right = $3;
                    IdlAnyValue lval(left), rval(right);

                    if(!lval.isInteger() || !rval.isInteger())
                    {
                        IdlError("invalid operand for binary operator |");
			// Can't continue, therefore jump to the next
			// yyerrok
                        HANDLE_ERROR;
                    }

                    IdlAnyValue result = lval.bitOr(rval);
                    $$ = result.getValue();
		}
		;

/* (31) */
xor_expr	: and_expr
		{
		    $$ = $1;
		}
		| xor_expr '^' and_expr
		{
		    Any_var left = $1;
		    Any_var right = $3;
                    IdlAnyValue lval(left), rval(right);

                    if(!lval.isInteger() || !rval.isInteger())
                    {
                        IdlError("invalid operand for binary operator ^");
			// Can't continue, therefore jump to the next
			// yyerrok
                        HANDLE_ERROR;
                    }

                    IdlAnyValue result = lval.bitXor(rval);
                    $$ = result.getValue();
		}
		;

/* (32) */
and_expr	: shift_expr
		{
		    $$ = $1;
		}
		| and_expr '&' shift_expr
		{
		    Any_var left = $1;
		    Any_var right = $3;
                    IdlAnyValue lval(left), rval(right);

                    if(!lval.isInteger() || !rval.isInteger())
                    {
                        IdlError("invalid operand for binary operator &");
			// Can't continue, therefore jump to the next
			// yyerrok
                        HANDLE_ERROR;
                    }

                    IdlAnyValue result = lval.bitAnd(rval);
                    $$ = result.getValue();
		}
		;

/* (33) */
shift_expr	: add_expr
		{
		    $$ = $1;
		}
		| shift_expr IDL_RIGHT_SHIFT add_expr
		{
		    Any_var left = $1;
		    Any_var right = $3;
                    IdlAnyValue lval(left), rval(right);

                    if(!lval.isInteger() || !rval.isInteger())
                    {
			IdlError("invalid operand for binary operator >>");
			// Can't continue, therefore jump to the next
			// yyerrok
			HANDLE_ERROR;
                    }

                    try
                    {
                        if(rval.isNegative() || rval.to_ushort() >= 64)
                        {
			    IdlError("right operand of binary operator >> "
				     "must be in the range 0 <= right "
				     "operand < 64");
			    // Can't continue, therefore jump to the next
			    // yyerrok
			    HANDLE_ERROR;
			}

                        IdlAnyValue result = lval.shiftRight(rval);
                        $$ = result.getValue();
                    }
                    catch(const DATA_CONVERSION&)
                    {
			IdlError("precision exceeded for binary operator >>");
			// Can't continue, therefore jump to the next
			// yyerrok
			HANDLE_ERROR;
                    }
		}
		| shift_expr IDL_LEFT_SHIFT add_expr
		{
		    Any_var left = $1;
		    Any_var right = $3;
                    IdlAnyValue lval(left), rval(right);

                    if(!lval.isInteger() || !rval.isInteger())
                    {
			IdlError("invalid operand for binary operator <<");
			// Can't continue, therefore jump to the next
			// yyerrok
			HANDLE_ERROR;
                    }

                    try
                    {
                        if(rval.isNegative() || rval.to_ushort() >= 64)
                        {
			    IdlError("right operand of binary operator << "
				     "must be in the range 0 <= right "
				     "operand < 64");
			    // Can't continue, therefore jump to the
			    // next yyerrok
			    HANDLE_ERROR;
			}

                        IdlAnyValue result = lval.shiftLeft(rval);
                        $$ = result.getValue();
                    }
                    catch(const DATA_CONVERSION&)
                    {
			IdlError("precision exceeded for binary operator <<");
			// Can't continue, therefore jump to the next
			// yyerrok
			HANDLE_ERROR;
                    }
		}
		;

/* (34) */
add_expr	: mult_expr
		{
		    $$ = $1;
		}
		| add_expr '+' mult_expr
		{
		    Any_var left = $1;
		    Any_var right = $3;
                    IdlAnyValue lval(left), rval(right);

                    if(!IdlAnyValue::compatible(lval.kind(), rval.kind()))
                    {
			IdlError("invalid operand for binary operator +");
			// Can't continue, therefore jump to the next
			// yyerrok
			HANDLE_ERROR;
                    }

                    try
                    {
                        IdlAnyValue result = lval.add(rval);
                        $$ = result.getValue();
                    }
                    catch(const DATA_CONVERSION&)
                    {
			IdlError("precision exceeded for binary operator +");
			// Can't continue, therefore jump to the next
			// yyerrok
			HANDLE_ERROR;
                    }
		}
		| add_expr '-' mult_expr
		{
		    Any_var left = $1;
		    Any_var right = $3;
                    IdlAnyValue lval(left), rval(right);

                    if(!IdlAnyValue::compatible(lval.kind(), rval.kind()))
                    {
			IdlError("invalid operand for binary operator -");
			// Can't continue, therefore jump to the next
			// yyerrok
			HANDLE_ERROR;
                    }

                    try
                    {
                        IdlAnyValue result = lval.sub(rval);
                        $$ = result.getValue();
                    }
                    catch(const DATA_CONVERSION&)
                    {
			IdlError("precision exceeded for binary operator -");
			// Can't continue, therefore jump to the next
			// yyerrok
			HANDLE_ERROR;
                    }
		}
		;

/* (35) */
mult_expr	: unary_expr
		{
		    $$ = $1;
		}
		| mult_expr '*' unary_expr
		{
		    Any_var left = $1;
		    Any_var right = $3;
                    IdlAnyValue lval(left), rval(right);

                    if(!IdlAnyValue::compatible(lval.kind(), rval.kind()))
                    {
			IdlError("invalid operand for binary operator *");
			// Can't continue, therefore jump to the next
			// yyerrok
			HANDLE_ERROR;
                    }

                    try
                    {
                        IdlAnyValue result = lval.mult(rval);
                        $$ = result.getValue();
                    }
                    catch(const DATA_CONVERSION&)
                    {
			IdlError("precision exceeded for binary operator *");
			// Can't continue, therefore jump to the next
			// yyerrok
			HANDLE_ERROR;
                    }
		}
		| mult_expr '/' unary_expr
		{
		    Any_var left = $1;
		    Any_var right = $3;
                    IdlAnyValue lval(left), rval(right);

                    if(!IdlAnyValue::compatible(lval.kind(), rval.kind()))
                    {
			IdlError("invalid operand for binary operator /");
			// Can't continue, therefore jump to the next
			// yyerrok
			HANDLE_ERROR;
                    }

                    if(rval.isZero())
                    {
			IdlError("divide-by-zero error in constant");
			// Can't continue, therefore jump to the next
			// yyerrok
			HANDLE_ERROR;
                    }

                    try
                    {
                        IdlAnyValue result = lval.div(rval);
                        $$ = result.getValue();
                    }
                    catch(const DATA_CONVERSION&)
                    {
			IdlError("precision exceeded for binary operator /");
			// Can't continue, therefore jump to the next
			// yyerrok
			HANDLE_ERROR;
                    }
		}
		| mult_expr '%' unary_expr
		{
		    Any_var left = $1;
		    Any_var right = $3;
                    IdlAnyValue lval(left), rval(right);

                    if(!IdlAnyValue::compatible(lval.kind(), rval.kind()) ||
                       !lval.isInteger() || !rval.isInteger())
                    {
			IdlError("invalid operand for binary operator %");
			// Can't continue, therefore jump to the next
			// yyerrok
			HANDLE_ERROR;
                    }

                    if(rval.isZero())
                    {
			IdlError("divide-by-zero error in constant");
			// Can't continue, therefore jump to the next
			// yyerrok
			HANDLE_ERROR;
                    }

                    try
                    {
                        IdlAnyValue result = lval.mod(rval);
                        $$ = result.getValue();
                    }
                    catch(const DATA_CONVERSION&)
                    {
			IdlError("precision exceeded for binary operator %");
			// Can't continue, therefore jump to the next
			// yyerrok
			HANDLE_ERROR;
                    }
		}
		;

/* (36) */
unary_expr	: unary_operator primary_expr
		{
		    Any_var any = $2;
		    
		    const char* err = 0;
		    
		    if($1 == '~')
			err = "wrong argument type for bit-complement";
		    else if($1 == '+')
			err = "wrong argument type for unary plus";
		    else if($1 == '-')
			err = "wrong argument type for unary minus";
		    else
			assert(false);

                    if($1 == '~')
                    {
                        IdlAnyValue val(any);
                        if(!val.isInteger())
                        {
                            IdlError(err);
			    // Can't continue, therefore jump to the next
			    // yyerrok
                            HANDLE_ERROR;
                        }

                        IdlAnyValue result = val.twiddle();
                        $$ = result.getValue();
                    }
                    else if($1 == '+')
                    {
                        IdlAnyValue val(any);
                        if(!val.isNumeric())
                        {
                            IdlError(err);
			    // Can't continue, therefore jump to the next
			    // yyerrok
                            HANDLE_ERROR;
                        }

                        $$ = val.getValue();
                    }
                    else if($1 == '-')
                    {
                        IdlAnyValue val(any);
                        if(!val.isNumeric())
                        {
                            IdlError(err);
			    // Can't continue, therefore jump to the next
			    // yyerrok
                            HANDLE_ERROR;
                        }

                        try
                        {
                            IdlAnyValue result = val.neg();
                            $$ = result.getValue();
                        }
                        catch(const DATA_CONVERSION&)
                        {
                            IdlError("constant value exceeds allowable "
                                     "range");
			    // Can't continue, therefore jump to the next
			    // yyerrok
                            HANDLE_ERROR;
                        }
                    }
		}
		| primary_expr
		{
		    $$ = $1;
		}
		;

/* (37) */
unary_operator	: '-'
		{
		    $$ = '-';
		}
		| '+'
		{
		    $$ = '+';
		}
		| '~'
		{
		    $$ = '~';
		}
		;

/* (38) */
primary_expr	: scoped_name
		{
		    ScopedName_var scoped = $1;
		    Container_var container = IdlNameMgr -> getContainer();
		    $$ = IdlFindPrimaryExpr(container, scoped);
		    if($$ == 0)
		    {
			// Can't continue, therefore jump to the next
			// yyerrok
			HANDLE_ERROR;
		    }
		}
		| literal
		{
		    $$ = $1;
		}
		| '(' const_exp ')'
		{
		    $$ = $2;
		}
		;

/* (39) */
literal		: IDL_INTEGER_LITERAL
		{
		    Any* any = new Any;
                    if($1 < ULong(~0))
                        *any <<= ULong($1);
                    else
                        *any <<= $1;
		    $$ = any;
		}
		| string_literal
		{
		    Any* any = new Any;
		    *any <<= $1;
		    $$ = any;
		}
		| wide_string_literal
		{
		    Any* any = new Any;
		    *any <<= $1;
		    $$ = any;
		}
		| IDL_CHARACTER_LITERAL
		{
		    Any* any = new Any;
		    *any <<= Any::from_char($1);
		    $$ = any;
		}
		| IDL_WIDE_CHARACTER_LITERAL
		{
		    Any* any = new Any;
		    *any <<= Any::from_wchar($1);
		    $$ = any;
		}
		| IDL_FIXED_PT_LITERAL
		{
                    Fixed* f = $1;
		    Any* any = new Any;
		    *any <<= Any::from_fixed(*f, f -> fixed_digits(),
                                             f -> fixed_scale());
                    delete f;
		    $$ = any;
		}
		| IDL_FLOATING_PT_LITERAL
		{
		    Any* any = new Any;
		    *any <<= $1;
		    $$ = any;
		}
		| boolean_literal
		{
		    Any* any = new Any;
		    *any <<= Any::from_boolean($1);
		    $$ = any;
		}
		;

/* (40) */
boolean_literal	: IDL_TRUE
		{
		    assert($1 == true);
		    $$ = true;
		}
		| IDL_FALSE
		{
		    assert($1 == false);
		    $$ = false;
		}
		;

/* (41) */
positive_int_const: const_exp
		{
		    Any_var any = $1;
                    IdlAnyValue val(any);

                    if(!val.isInteger())
                    {
                        IdlError("not an integer constant");
			// Can't continue, therefore jump to the next
			// yyerrok
                        HANDLE_ERROR;
                        break;
                    }

                    if(val.isNegative())
                    {
                        IdlError("integer constant must be positive");
			// Can't continue, therefore jump to the next
			// yyerrok
                        HANDLE_ERROR;
                    }

                    try
                    {
                        $$ = val.to_ulong();
                    }
                    catch(const DATA_CONVERSION&)
                    {
                        IdlError("integer constant exceeds allowable range");
			// Can't continue, therefore jump to the next
			// yyerrok
                        HANDLE_ERROR;
                        break;
                    }

                    /*
		    TypeCode_var tc = any -> type();
		    tc = GetOrigType(tc);
		    
		    switch(tc -> kind())
		    {
		    case tk_short:
		    {
			Short v;
			any >>= v;
			if(v <= 0)
			{
			    IdlError("integer constant must be > 0");
			    // Can't continue, therefore jump to the next
			    // yyerrok
			    HANDLE_ERROR;
			}
			$$ = (ULong)v;
			break;
		    }

		    case tk_long:
		    {
			Long v;
			any >>= v;
			if(v <= 0)
			{
			    IdlError("integer constant must be > 0");
			    // Can't continue, therefore jump to the next
			    // yyerrok
			    HANDLE_ERROR;
			}
			$$ = (ULong)v;
			break;
		    }
		    
		    case tk_ushort:
		    {
			UShort v;
			any >>= v;
			$$ = (ULong)v;
			break;
		    }

		    case tk_ulong:
		    {
			ULong v;
			any >>= v;
			$$ = (ULong)v;
			break;
		    }
		    
		    default:
		    {
			IdlError("not an integer constant");
			// Can't continue, therefore jump to the next
			// yyerrok
			HANDLE_ERROR;
			break;
		    }
		    }
                    */
		}
		;

/* (42) */
type_dcl	: IDL_TYPEDEF type_declarator
		{
		}
		| struct_type
		{
		    release($1);
		}
		| union_type
		{
		    release($1);
		}
		| enum_type
		{
		    release($1);
		}
		| IDL_NATIVE simple_declarator
		{
		    //
		    // Get container from container stack
		    //
		    Container_var container = IdlNameMgr -> getContainer();

		    IdlDeclarator_var declarator = $2;
		    assert(declarator -> sizes.size() == 0);

		    //
		    // Get name, id and version
		    //
		    Identifier_var name = declarator -> identifier;
		    RepositoryId_var id = IdlGetRepositoryId(name);
		    VersionSpec_var version = string_dup("1.0");

		    //
		    // Add the name to this scope
		    //
		    if(IdlNameMgr -> defineName(name, id))
		    {
			//
			// Create native type
			//
			try
			{
			    release(container -> create_native(id, name,
							       version));
			}
			catch(const SystemException& ex)
			{
			    IdlPrintException(ex, id, name);
			}
		    }
		}
                | constr_forward_dcl
                {
                }
		;

/* (43) */
type_declarator	: type_spec declarators
		{
		    //
		    // Get container from container stack
		    //
		    Container_var container = IdlNameMgr -> getContainer();

		    IDLType_var simpleIDLType = $1;
		    IdlDeclaratorList_var declarators = $2;

                    OB_STL::list<IdlDeclarator_var>::iterator declIter;
                    for(declIter = declarators -> begin();
			declIter != declarators -> end();
			declIter++)
		    {
			IdlDeclarator declarator = *declIter;

			//
			// Get name, id and version
			//
			Identifier_var name = declarator.identifier;
			RepositoryId_var id = IdlGetRepositoryId(name);
			VersionSpec_var version =
			    string_dup("1.0");

			//
			// If simpleIDLType is an anonymous type, we
			// can only use it once
			//
			IDLType_var idlType = simpleIDLType;
			if(declIter != declarators -> begin())
			{
			    switch(idlType -> def_kind())
			    {
			    case dk_none:
			    case dk_all:
			    case dk_Attribute:
			    case dk_Constant:
			    case dk_Exception:
			    case dk_Interface:
			    case dk_LocalInterface:
			    case dk_AbstractInterface:
			    case dk_Module:
			    case dk_Operation:
			    case dk_Typedef:
			    case dk_Alias:
			    case dk_Struct:
			    case dk_Union:
			    case dk_Enum:
			    case dk_Primitive:
			    case dk_Repository:
			    case dk_Value:
			    case dk_ValueBox:
			    case dk_ValueMember:
			    case dk_Native:
				break;

			    case dk_String:
			    {
				StringDef_var def =
				    StringDef::_narrow(idlType);
				assert(!is_nil(def));
				ULong bound = def -> bound();
				Repository_var rep =
				    IdlNameMgr -> getRepository();
				idlType = rep -> create_string(bound);
				break;
			    }

			    case dk_Wstring:
			    {
				WstringDef_var def =
				    WstringDef::_narrow(idlType);
				assert(!is_nil(def));
				ULong bound = def -> bound();
				Repository_var rep =
				    IdlNameMgr -> getRepository();
				idlType = rep -> create_wstring(bound);
				break;
			    }

			    case dk_Sequence:
			    {
				SequenceDef_var def =
				    SequenceDef::_narrow(idlType);
				assert(!is_nil(def));
				ULong bound = def -> bound();
				IDLType_var elem = def -> element_type_def();
				Repository_var rep =
				    IdlNameMgr -> getRepository();
				idlType = rep -> create_sequence(bound, elem);
				break;
			    }

			    case dk_Array:
			    {
				ArrayDef_var def = ArrayDef::_narrow(idlType);
				assert(!is_nil(def));
				ULong length = def -> length();
				IDLType_var elem = def -> element_type_def();
				Repository_var rep =
				    IdlNameMgr -> getRepository();
				idlType = rep -> create_array(length, elem);
				break;
			    }

			    case dk_Fixed:
			    {
				FixedDef_var def = FixedDef::_narrow(idlType);
				assert(!is_nil(def));
				UShort digits = def -> digits();
				Short scale = def -> scale();
				Repository_var rep =
				    IdlNameMgr -> getRepository();
				idlType = rep -> create_fixed(digits, scale);
				break;
			    }
			    }
			}

			//	
			// Handle arrays
			//
                        OB_STL::list<ULong>::reverse_iterator sizesIter;
                        for(sizesIter = declarator.sizes.rbegin();
			    sizesIter != declarator.sizes.rend();
			    sizesIter++)
			{
			    Repository_var rep =
				IdlNameMgr -> getRepository();
			    ArrayDef_var arrayDef = rep ->
				create_array(*sizesIter, idlType);

			    idlType = IDLType::_duplicate(arrayDef);
			}

			//
			// Add the name to this scope
			//
			if(IdlNameMgr -> defineName(name, id))
			{
			    //
			    // Create alias
			    //
			    try
			    {
				release(container ->
					create_alias(id, name, version,
						     idlType));
			    }
			    catch(const SystemException& ex)
			    {
				IdlPrintException(ex, id, name);
			    }
			}
		    }
		}
		;

/* (44) */
type_spec	: simple_type_spec
		{
		    $$ = $1;
		}
		| constr_type_spec
		{
		    $$ = $1;
		}
		;

/* (45) */
simple_type_spec: base_type_spec
		{
		    $$ = $1;
		}
		| template_type_spec
		{
		    $$ = $1;
		}
		| scoped_name
		{
		    ScopedName_var scoped = $1;

		    //
		    // Get container from container stack
		    //
		    Container_var container = IdlNameMgr -> getContainer();

		    //
		    // Find scoped name
		    //
		    IRObject_var irObj =
			IdlFindScopedName(container, scoped);

		    if(is_nil(irObj))
		    {
			// Can't continue, therefore jump to the next
			// yyerrok
			HANDLE_ERROR;
		    }

		    IDLType_var idlType = IDLType::_narrow(irObj);
		    if(is_nil(idlType))
		    {
			String_var err = string_dup("`");
			err += scoped;
			err += "' is not a type specification";
			IdlError(err);
			// Can't continue, therefore jump to the next
			// yyerrok
			HANDLE_ERROR;
		    }

		    $$ = idlType._retn();
		}
		;

/* (46) */
base_type_spec	: floating_pt_type
		{
		    $$ = $1;
		}
		| integer_type
		{
		    $$ = $1;
		}
		| char_type
		{
		    $$ = $1;
		}
		| wide_char_type
		{
		    $$ = $1;
		}
		| boolean_type
		{
		    $$ = $1;
		}
		| octet_type
		{
		    $$ = $1;
		}
		| any_type
		{
		    $$ = $1;
		}
		| object_type
		{
		    $$ = $1;
		}
		| value_base_type
		{
		    $$ = $1;
		}
		;

/* (47) */
template_type_spec: sequence_type
		{
		    $$ = $1;
		}
		| string_type
		{
		    $$ = $1;
		}
		| wide_string_type
		{
		    $$ = $1;
		}
		| fixed_pt_type
		{
		    $$ = $1;
		}
		;

/* (48) */
constr_type_spec: struct_type
		{
		    $$ = $1;
		}
		| union_type
		{
		    $$ = $1;
		}
		| enum_type
		{
		    $$ = $1;
		}
		;

/* (49) */
declarators	: declarator COMMA_declarator_STAR
		{
		    IdlDeclarator_var declarator = $1;
		    $2 -> push_front(declarator);
		    $$ = $2;
		}
		;

/* (50) */
COMMA_declarator_STAR: ',' declarator COMMA_declarator_STAR
		{
		    IdlDeclarator_var declarator = $2;
		    $3 -> push_front(declarator);
		    $$ = $3;
		}
		| /* empty */
		{
		    $$ = new OB_STL::list<IdlDeclarator_var>;
		}
		;

/* (50) */
declarator	: simple_declarator
		{
		    $$ = $1;
		}
		| complex_declarator
		{
		    $$ = $1;
		}
		;

/* (51) */
simple_declarator: IDL_IDENTIFIER
		{
		    $$ = new IdlDeclarator;
		    $$ -> identifier = $1;
		}
		;

/* (51) */
COMMA_simple_declarator_STAR: ',' simple_declarator
			      COMMA_simple_declarator_STAR
		{
		    IdlDeclarator_var declarator = $2;
		    $3 -> push_front(declarator);
		    $$ = $3;
		}
		| /* empty */
		{
		    $$ = new OB_STL::list<IdlDeclarator_var>;
		}
		;

/* (52) */
complex_declarator: array_declarator
		{
		    $$ = $1;
		}
		;

/* (53) */
floating_pt_type: IDL_FLOAT
		{
		    Repository_var rep = IdlNameMgr -> getRepository();
		    $$ = rep -> get_primitive(pk_float);
		}
		| IDL_DOUBLE
		{
		    Repository_var rep = IdlNameMgr -> getRepository();
		    $$ = rep -> get_primitive(pk_double);
		}
		| IDL_LONG IDL_DOUBLE
		{
		    if(!IdlAllowLongDouble)
		    {
			String_var err = string_dup(
			    "IDL long double type is not supported ");
			err += "by this language mapping";
			IdlError(err);
			// Can't continue, therefore jump to the next
			// yyerrok
			HANDLE_ERROR;
		    }

		    Repository_var rep = IdlNameMgr -> getRepository();
		    $$ = rep -> get_primitive(pk_longdouble);
		}
		;

/* (54) */
integer_type	: signed_int
		{
		    $$ = $1;
		}
		| unsigned_int
		{
		    $$ = $1;
		}
		;

/* (55) */
signed_int	: signed_short_int
		{
		    $$ = $1;
		}
		| signed_long_int
		{
		    $$ = $1;
		}
		| signed_longlong_int
		{
		    $$ = $1;
		}
		;

/* (56) */
signed_short_int: IDL_SHORT
		{
		    Repository_var rep = IdlNameMgr -> getRepository();
		    $$ = rep -> get_primitive(pk_short);
		}
		;

/* (57) */
signed_long_int	: IDL_LONG
		{
		    Repository_var rep = IdlNameMgr -> getRepository();
		    $$ = rep -> get_primitive(pk_long);
		}
		;
/* (58) */
signed_longlong_int: IDL_LONG IDL_LONG
		{
		    Repository_var rep = IdlNameMgr -> getRepository();
		    $$ = rep -> get_primitive(pk_longlong);
		}
		;

/* (59) */
unsigned_int	: unsigned_short_int
		{
		    $$ = $1;
		}
		| unsigned_long_int
		{
		    $$ = $1;
		}
		| unsigned_longlong_int
		{
		    $$ = $1;
		}
		;

/* (60) */
unsigned_short_int: IDL_UNSIGNED IDL_SHORT
		{
		    Repository_var rep = IdlNameMgr -> getRepository();
		    $$ = rep -> get_primitive(pk_ushort);
		}
		;

/* (61) */
unsigned_long_int: IDL_UNSIGNED IDL_LONG
		{
		    Repository_var rep = IdlNameMgr -> getRepository();
		    $$ = rep -> get_primitive(pk_ulong);
		}
		;

/* (62) */
unsigned_longlong_int: IDL_UNSIGNED IDL_LONG IDL_LONG
		{
		    Repository_var rep = IdlNameMgr -> getRepository();
		    $$ = rep -> get_primitive(pk_ulonglong);
		}
		;

/* (63) */
char_type	: IDL_CHAR
		{
		    Repository_var rep = IdlNameMgr -> getRepository();
		    $$ = rep -> get_primitive(pk_char);
		}
		;

/* (64) */
wide_char_type	: IDL_WCHAR
		{
		    Repository_var rep = IdlNameMgr -> getRepository();
		    $$ = rep -> get_primitive(pk_wchar);
		}
		;

/* (65) */
boolean_type	: IDL_BOOLEAN
		{
		    Repository_var rep = IdlNameMgr -> getRepository();
		    $$ = rep -> get_primitive(pk_boolean);
		}
		;

/* (66) */
octet_type	: IDL_OCTET
		{
		    Repository_var rep = IdlNameMgr -> getRepository();
		    $$ = rep -> get_primitive(pk_octet);
		}
		;

/* (67) */
any_type	: IDL_ANY
		{
		    Repository_var rep = IdlNameMgr -> getRepository();
		    $$ = rep -> get_primitive(pk_any);
		}
		;

/* (68) */
object_type	: IDL_OBJECT
		{
		    Repository_var rep = IdlNameMgr -> getRepository();
		    $$ = rep -> get_primitive(pk_objref);
		}
		;

/* (69) */
struct_type	: IDL_STRUCT IDL_IDENTIFIER
		{
		    //
		    // Get container from container stack
		    //
		    Container_var container = IdlNameMgr -> getContainer();

		    //
		    // Get name, id and version
		    //
		    const char* name = $2; // No _var here!
		    RepositoryId_var id = IdlGetRepositoryId(name);
		    VersionSpec_var version = string_dup("1.0");

		    //
		    // Add the name to this scope
		    //
		    if(!IdlNameMgr -> defineName(name, id, true, 0, true))
		    {
			// Can't continue, therefore jump to the next
			// yyerrok
			HANDLE_ERROR;
		    }

                    //
                    // Check if name already exists
                    //
                    Contained_var c;
                    try
                    {
                        c = container -> lookup(name);
                    }
                    catch(const SystemException& ex)
                    {
                        IdlPrintException(ex, id, name);
                        // Can't continue, therefore jump to the next
                        // yyerrok
                        HANDLE_ERROR;
                    }

		    StructDef_var st;
                    if(!is_nil(c))
                    {
			st = StructDef::_narrow(c);

			//
			// Redefinition is legal in inherited
			// interface, so check the repository IDs
			//
			RepositoryId_var containedId = c -> id();
			if(strcmp(id, containedId) == 0)
			{
			    //
			    // Name exists, check if it is a struct
			    // forward declaration
			    //
			    if(is_nil(st))
			    {
				String_var err = string_dup(
				    "conflicting types for `struct ");
				err += name;
				err += "'";
				IdlError(err);
				// Can't continue, therefore jump to the next
				// yyerrok
				HANDLE_ERROR;
			    }
			}
		    }

		    if(!is_nil(st))
		    {
			//
			// Struct has been forward declared
			//
			
			$<structDef>$ = st._retn();
		    }
		    //
		    // Create new struct if it has not been forward declared
		    //
		    else
		    {
			//
			// Create empty member sequence
			//
			StructMemberSeq_var members = new StructMemberSeq;

			try
			{
			    $<structDef>$ = container ->
				create_struct(id, name, version, members);
			}
			catch(const SystemException& ex)
			{
			    IdlPrintException(ex, id, name);
			    // Can't continue, therefore jump to the next
			    // yyerrok
			    HANDLE_ERROR;
			}
		    }
		    
		    //
		    // Set and push new container on the stack
		    //
		    IdlNameMgr -> setContainer(id, $<structDef>$);
		    IdlNameMgr -> pushContainer(id);
		}
		'{' member_list '}'
		{
		    CORBA::StructDef_var structDef = $<structDef>3;

		    //
		    // Get name and id
		    //
		    Identifier_var name = $2;
		    RepositoryId_var id = IdlGetRepositoryId(name);

		    //
		    // Get member sequence
		    //
                    StructMemberList_var members = $5;

		    //
		    // Set the member sequence
		    //
		    try
		    {
                        StructMemberSeq memberSeq(members -> size());
                        memberSeq.length(members -> size());
                        copy_to_seq(members -> begin(), members -> end(),
                                    memberSeq);
			structDef -> members(memberSeq);
		    }
		    catch(const SystemException& ex)
		    {
			IdlPrintException(ex, id, name);
			// No HANDLE_ERROR here, otherwise this will jump
			// to the error branch in the member rule
		    }

		    //
		    // Pop container on the stack
		    //
		    IdlNameMgr -> popContainer();

                    $$ = structDef._retn();
		}
		;

/* (70) */
member_list	: member_PLUS
		{
		    $$ = $1;
		}
		;

/* (71) */
member_PLUS	: member
		{
		    $$ = $1;
		}
		| member member_PLUS
		{
		    StructMemberList_var members1 = $1;
		    StructMemberList_var members2 = $2;

                    OB_STL::list<StructMember_var>::iterator members2Iter;
                    for(members2Iter = members2 -> begin();
			members2Iter != members2 -> end();
			members2Iter++)
		    {
			//
			// Check for duplicate struct members
			//
                        OB_STL::list<StructMember_var>::iterator members1Iter;
                        for(members1Iter = members1 -> begin();
			    members1Iter != members1 -> end();
			    members1Iter++)
			{
                            StructMember_var member2 = *members2Iter;
                            StructMember_var member1 = *members1Iter;
                            
			    if(IdlStringCompare(member2 -> name,
                                                member1 -> name) == 0)
			    {
				//
                                // Shouldn't happen anymore, we check
                                // now directly in the member rules
				//
				assert(false);
				String_var err =
				    string_dup("duplicate struct member `");
				err += member2 -> name;
				err += "'";
				IdlError(err);
				// No HANDLE_ERROR here, otherwise this
				// will jump to the error branch in
				// the member rule
				break;
			    }
			}

                        if (members1Iter == members1 -> end())
			    members1 -> push_back(*members2Iter);
		    }

		    $$ = members1._retn();
		}
		;

/* (71) */
member_STAR	: member member_STAR
		{
		    StructMemberList_var members1 = $1;
		    StructMemberList_var members2 = $2;

                    OB_STL::list<StructMember_var>::iterator members2Iter;
                    for(members2Iter = members2 -> begin();
			members2Iter != members2 -> end();
			members2Iter++)
		    {
			//
			// Check for duplicate exception members
			//
                        OB_STL::list<StructMember_var>::iterator members1Iter;
                        for(members1Iter = members1 -> begin();
                            members1Iter != members1 -> end();
                            members1Iter++)
			{
                            StructMember_var member2 = *members2Iter;
                            StructMember_var member1 = *members1Iter;
                            
			    if(IdlStringCompare(member2 -> name,
                                                member1 -> name) == 0)
			    {
				//
                                // Shouldn't happen anymore, we check
                                // now directly in the member rules
				//
				assert(false);
				String_var err =
				    string_dup("duplicate exception member `");
				err += member2 -> name;
				err += "'";
				IdlError(err);
				// No HANDLE_ERROR here, otherwise this
				// will jump to the error branch in
				// the member rule
				break;
			    }
			}

                        if (members1Iter == members1 -> end())
			    members1 -> push_back(*members2Iter);
		    }

		    $$ = members1._retn();
		}
		| /* empty */
		{
		    $$ = new OB_STL::list<StructMember_var>;
		}
		;

/* (71) */
member		: member_NO_SEM ';'
		{
		    $$ = $1;
		}
		| member_NO_SEM
		{
		    $$ = $1;
		    IdlError("`;' missing after struct member");
		}
		| error ';'
		{
		    yyerrok;
		    $$ = new OB_STL::list<StructMember_var>;
		}
		;

/* (71) */
member_NO_SEM	: type_spec declarators
		{
		    IDLType_var typeSpec = $1;
		    IdlDeclaratorList_var declarators = $2;

		    StructMemberList_var memberSeq =
                        new OB_STL::list<StructMember_var>;

		    Repository_var rep = IdlNameMgr -> getRepository();

                    OB_STL::list<IdlDeclarator_var>::iterator declIter;
                    for(declIter = declarators -> begin();
			declIter != declarators -> end();
			declIter++)
		    {
			IdlDeclarator declarator = *declIter;

			//
			// Add the name to this scope
			//
			if(!IdlNameMgr -> defineDecl(declarator.identifier))
			{
			    // Can't continue, therefore jump to the next
			    // yyerrok
			    HANDLE_ERROR;
			}

			//
			// Handle arrays
			//
			IDLType_var idlType = typeSpec;

                        OB_STL::list<ULong>::reverse_iterator sizesIter;
                        for(sizesIter = declarator.sizes.rbegin();
			    sizesIter != declarator.sizes.rend();
			    sizesIter++)
			{
			    ArrayDef_var arrayDef = rep ->
				create_array(*sizesIter, idlType);

			    idlType = IDLType::_duplicate(arrayDef);
			}

			//
			// Create member
			//
			StructMember_var member = new StructMember;
			member -> name = declarator.identifier;
			member -> type = idlType -> type();
			member -> type_def = idlType;

			memberSeq -> push_back(member);
		    }

		    $$ = memberSeq._retn();
		}
		;

/* (72) */
union_type	: IDL_UNION IDL_IDENTIFIER IDL_SWITCH
                {
		    //
		    // Get container from container stack
		    //
		    Container_var container = IdlNameMgr -> getContainer();

		    //
		    // Get name, id and version
		    //
		    const char* name = $2; // No _var here!
		    RepositoryId_var id = IdlGetRepositoryId(name);
		    VersionSpec_var version = string_dup("1.0");

		    //
		    // We don't have the switch type yet, so we use a
		    // dummy type here.
		    //
		    Repository_var rep = IdlNameMgr -> getRepository();
		    IDLType_var disc = rep -> get_primitive(pk_short);

		    //
		    // Add the name to this scope
		    //
		    if(!IdlNameMgr -> defineName(name, id, true, 0, true))
		    {
			// Can't continue, therefore jump to the next
			// yyerrok
			HANDLE_ERROR;
		    }

                    //
                    // Check if name already exists
                    //
                    Contained_var c;
                    try
                    {
                        c = container -> lookup(name);
                    }
                    catch(const SystemException& ex)
                    {
                        IdlPrintException(ex, id, name);
                        // Can't continue, therefore jump to the next
                        // yyerrok
                        HANDLE_ERROR;
                    }

		    UnionDef_var un;
                    if(!is_nil(c))
                    {
			un = UnionDef::_narrow(c);

                        //
                        // Redefinition is legal in inherited
                        // interface, so check the repository IDs
                        //
                        RepositoryId_var containedId = c -> id();
                        if(strcmp(id, containedId) == 0)
			{
			    //
			    // Name exists, check if it is a union
			    // forward declaration
			    //
			    if(is_nil(un))
			    {
				String_var err = string_dup(
				    "conflicting types for `union ");
				err += name;
				err += "'";
				IdlError(err);
				// Can't continue, therefore jump to the next
				// yyerrok
				HANDLE_ERROR;
			    }
			}
		    }

		    if(!is_nil(un))
		    {
			//
			// Union has been forward declared
			//
			$<unionDef>$ = un._retn();
		    }
		    //
		    // Create new union if it has not been forward declared
		    //
		    else
		    {
			//
			// Create empty member sequence
			//
			UnionMemberSeq_var members = new UnionMemberSeq;

			try
			{
			    $<unionDef>$ = container ->
				create_union(id, name, version, disc, members);
			}
			catch(const SystemException& ex)
			{
			    IdlPrintException(ex, id, name);
			    // Can't continue, therefore jump to the next
			    // yyerrok
			    HANDLE_ERROR;
			}
		    }
	    
		    //
		    // Set and push new container on the stack. Note:
		    // We must push the union to the container stack
		    // *before* the discriminator type is
		    // defined. That's because the union's scope
		    // starts with the "(".
		    //
		    IdlNameMgr -> setContainer(id, $<unionDef>$);
		    IdlNameMgr -> pushContainer(id);
                }
                '(' switch_type_spec ')' '{' switch_body '}'
		{
		    $$ = $<unionDef>4;

		    //
		    // Get name and id
		    //
		    Identifier_var name = $2;
		    RepositoryId_var id = IdlGetRepositoryId(name);

		    //
		    // Get union switch type
		    //
		    IDLType_var disc = $6;

		    //
		    // Get member sequence
		    //
		    UnionMemberList_var members = $9;

		    bool ok = true;
		    bool haveDefault = false;
		    TypeCode_var discType = disc -> type();
		    discType = GetOrigType(discType);
                    OB_STL::list<UnionMember>::iterator membersIter;
                    for(membersIter = members -> begin();
			membersIter != members -> end();
			membersIter++)
		    {
                        UnionMember& member = *membersIter;
                        
			//
			// Check for multiple default labels
			//
			Octet oct;
			Boolean b = member.label >>= Any::to_octet(oct);

			if(b)
			{
			    assert(oct == 0);

			    if(haveDefault)
			    {
				IdlError("multiple default labels");
				// No HANDLE_ERROR here, otherwise this
				// will jump to the error branch in
				// the case rule
				ok = false;
			    }

			    haveDefault = true;
			}
			else
			{
			    TypeCode_var tc = member.label.type();
			    tc = GetOrigType(tc);
			    IdlAnyValue val(member.label);

			    //
			    // Check if discriminator type is matched
			    //
			    if(!tc -> equivalent(discType))
			    {
				TCKind k = discType -> kind();
				if((k == tk_short || k == tk_ushort ||
				    k == tk_long || k == tk_ulong ||
				    k == tk_longlong || k == tk_ulonglong ||
				    k == tk_octet) && val.isInteger())
				{
				    try
				    {
					val = val.convert(k);
					Any_var any = val.getValue();
					member.label = any;
				    }
				    catch(const DATA_CONVERSION&)
				    {
#ifdef HAVE_SSTREAM
					OB_STD(ostringstream) ostr;
#else
					char buf[1024];
					OB_STD(ostrstream) ostr(buf, 1024);
#endif
					ostr << val << '\0';
					String_var err =
					    string_dup("case label `");
#ifdef HAVE_SSTREAM
					err += ostr.str().c_str();
#else
					err += buf;
#endif
					err += "' value exceeds allowable"
					    " range for discriminator";
					IdlError(err);
					// No HANDLE_ERROR here, otherwise this
					// will jump to the error branch
					// in the case rule
					ok = false;
				    }
				}
				else
				{
#ifdef HAVE_SSTREAM
				    OB_STD(ostringstream) ostr;
#else
				    char buf[1024];
				    OB_STD(ostrstream) ostr(buf, 1024);
#endif
				    ostr << val << '\0';
				    String_var err =
					string_dup("type of case label `");
#ifdef HAVE_SSTREAM
				    err += ostr.str().c_str();
#else
				    err += buf;
#endif
				    err += "' doesn't match the discriminator"
					" type";
				    IdlError(err);
				    // No HANDLE_ERROR here, otherwise this
				    // will jump to the error branch
				    // in the case rule
				    ok = false;
				}
			    }

			    //
			    // Check for duplicate labels
			    //
                            OB_STL::list<UnionMember>::iterator dupCheckIter;
                            for(dupCheckIter = members -> begin();
				dupCheckIter != membersIter;
				dupCheckIter++)
			    {
				TypeCode_var tc2 =
                                    (*dupCheckIter).label.type();
				tc2 = GetOrigType(tc2);

				if(tc2 -> kind() == tk_octet)
				    continue;

                                IdlAnyValue val2((*dupCheckIter).label);
                                if(val.equal(val2))
                                {
#ifdef HAVE_SSTREAM
				    OB_STD(ostringstream) ostr;
#else
                                    char buf[1024];
                                    OB_STD(ostrstream) ostr(buf, 1024);
#endif
                                    ostr << val << '\0';
                                    String_var err =
                                        string_dup("duplicate case label `");
#ifdef HAVE_SSTREAM
				    err += ostr.str().c_str();
#else
				    err += buf;
#endif
                                    err += "'";
                                    IdlError(err);
				    // No HANDLE_ERROR here, otherwise this
				    // will jump to the error branch
				    // in the case rule
				    ok = false;
                                }
			    }
			}
		    }

		    //
		    // Check for illegal default label
		    //
		    if(haveDefault)
		    {
			if((discType -> kind() == tk_char &&
			    members -> size() > 256) ||
			   (discType -> kind() == tk_boolean &&
			    members -> size() > 2) ||
			   (discType -> kind() == tk_enum &&
			    members -> size() > discType -> member_count()))
			{
			    IdlError("No value available for an explicit "
				     "default label");
			    // No HANDLE_ERROR here, otherwise this will
			    // jump to the error branch in the case
			    // rule
			    ok = false;
			}
		    }
		    
		    //
		    // Set the discriminator type and the member
		    // sequence
		    //
		    if(ok)
		    {
			try
			{
			    $$ -> discriminator_type_def(disc);

                            UnionMemberSeq memberSeq(members -> size());
                            memberSeq.length(members -> size());
                            copy_to_seq(members -> begin(), members -> end(),
                                        memberSeq);
			    $$ -> members(memberSeq);
			}
			catch(const SystemException& ex)
			{
			    IdlPrintException(ex, id, name);
			    // No HANDLE_ERROR here, otherwise this will jump
			    // to the error branch in the case rule
			}
		    }
			
		    //
		    // Pop container on the stack
		    //
		    IdlNameMgr -> popContainer();
		}
		;

/* (73) */
switch_type_spec: integer_type
		{
		    $$ = $1;
		}
		| char_type
		{
		    $$ = $1;
		}
		| boolean_type
		{
		    $$ = $1;
		}
		| enum_type
		{
		    $$ = $1;
		}
		| scoped_name
		{
		    ScopedName_var scoped = $1;

		    //
		    // Get container from container stack
		    //
		    Container_var container = IdlNameMgr -> getContainer();

		    //
		    // Find scoped name
		    //
		    IRObject_var irObj =
			IdlFindScopedName(container, scoped);

		    if(is_nil(irObj))
		    {
			// Can't continue, therefore jump to the next
			// yyerrok
			HANDLE_ERROR;
		    }

		    bool valid = false;

		    IDLType_var idlType = IDLType::_narrow(irObj);
		    if(!is_nil(idlType))
		    {
			TypeCode_var tc = idlType -> type();
			tc = GetOrigType(tc);

			TCKind k = tc -> kind();

			if(k == tk_short || k == tk_ushort ||
			   k == tk_long || k == tk_ulong ||
			   k == tk_char || k == tk_boolean ||
			   k == tk_enum)
			    valid = true;
		    }

		    if(!valid)
		    {
			String_var err = string_dup("`");
			err += scoped;
			err += "' is not a valid union discriminator type";
			IdlError(err);
			// Can't continue, therefore jump to the next
			// yyerrok
			HANDLE_ERROR;
		    }

		    $$ = idlType._retn();
		}
		;

/* (74) */
switch_body	: case_PLUS
		{
		    $$ = $1;
		}
		;

/* (75) */
case_PLUS	: case
		{
		    $$ = $1;
		}
		| case case_PLUS
		{
		    UnionMemberList_var members1 = $1;
		    UnionMemberList_var members2 = $2;

		    if(members1 -> size() == 0)
		    {
			$$ = members2._retn();
		    }
		    else
		    {
                        //
			// Check for duplicate union members
			//
                        OB_STL::list<UnionMember>::iterator members2Iter;
                        for(members2Iter = members2 -> begin();
			    members2Iter != members2 -> end();
			    members2Iter++)
			{
                            UnionMember& member2 = *members2Iter;
                            
			    //
			    // Only compare against the first element of
			    // members1, since all elements in members1
			    // are equal.
			    //
			    if(IdlStringCompare(member2.name,
                                                members1 -> front().name) == 0)
			    {
				//
                                // Shouldn't happen anymore, we check
                                // now directly in the member rules
				//
				assert(false);
				String_var err =
				    string_dup("duplicate union member `");
				err += member2.name;
				err += "'";
				IdlError(err);
				// No HANDLE_ERROR here, otherwise this will
				// jump to the error branch in the case
				// rule
			    }
			    else
				members1 -> push_back(member2);
			}
			
			$$ = members1._retn();
		    }
		}
		;

/* (75) */
case		: case_NO_SEM ';'
		{
		    $$ = $1;
		}
		| case_NO_SEM
		{
		    $$ = $1;
		    IdlError("`;' missing after union member");
		}
		| error ';'
		{
		    yyerrok;
		    $$ = new OB_STL::list<UnionMember>;
		}
		;

/* (75) */
case_NO_SEM	: case_label_PLUS element_spec
		{
		    AnyList_var anySeq = $1;
		    UnionMember_var member = $2;

		    UnionMemberList_var memberSeq =
		        new OB_STL::list<UnionMember>;

                    OB_STL::list<Any_var>::iterator anyIter;
		    for(anyIter = anySeq -> begin();
                        anyIter != anySeq -> end();
                        anyIter++)
		    {
                        memberSeq -> push_back(member);
                        memberSeq -> back().label = *anyIter;
		    }

		    $$ = memberSeq._retn();
		}
		;

/* (76) */
case_label_PLUS	: case_label
		{
		    Any_var any = $1;
		    $$ = new OB_STL::list<Any_var>;
		    $$ -> push_front(any);
		}
		| case_label case_label_PLUS
		{
		    Any_var any = $1;
		    $2 -> push_front(any);
		    $$ = $2;
		}
		;

/* (76) */
case_label	: IDL_CASE const_exp ':'
		{
		    $$ = $2;
		}
		| IDL_DEFAULT ':'
		{
		    $$ = new Any;
		    *$$ <<= Any::from_octet(0);
		}
		;

/* (77) */
element_spec	: type_spec declarator
		{
		    IDLType_var typeSpec = $1;
		    IdlDeclarator_var declarator = $2;

		    //
		    // Add the name to this scope
		    //
		    if(!IdlNameMgr -> defineDecl(declarator -> identifier))
		    {
			// Can't continue, therefore jump to the next
			// yyerrok
			HANDLE_ERROR;
		    }
		    
		    //	
		    // Handle arrays
		    //
		    IDLType_var idlType = typeSpec;
		    
		    Repository_var rep = IdlNameMgr -> getRepository();
                    
                    OB_STL::list<ULong>::reverse_iterator sizesIter;
                    for(sizesIter = declarator -> sizes.rbegin();
			sizesIter != declarator -> sizes.rend();
			sizesIter++)
		    {
			ArrayDef_var arrayDef = rep ->
			    create_array(*sizesIter, idlType);

			idlType = IDLType::_duplicate(arrayDef);
		    }
		    
		    //
		    // Create member
		    //
		    $$ = new UnionMember;
		    $$ -> name = declarator -> identifier;
		    $$ -> type = idlType -> type();
		    $$ -> type_def = idlType;
		}
		;

/* (78) */
enum_type	: IDL_ENUM IDL_IDENTIFIER
		{
		    //
		    // Get container from container stack
		    //
		    Container_var container = IdlNameMgr -> getContainer();

		    //
		    // Get name, id and version
		    //
		    const char* name = $2; // No _var here!
		    RepositoryId_var id = IdlGetRepositoryId(name);
		    VersionSpec_var version = string_dup("1.0");

		    //
		    // Add the name to this scope
		    //
		    if(!IdlNameMgr -> defineName(name, id))
		    {
			// Can't continue, therefore jump to the next
			// yyerrok
			HANDLE_ERROR;
		    }

		    //
		    // Create empty member sequence
		    //
		    EnumMemberSeq_var members = new EnumMemberSeq;

		    //
		    // Create enumerator
		    //
		    try
		    {
			$<enumDef>$ =
			    container -> create_enum(id, name, version,
						     members);
		    }
		    catch(const SystemException& ex)
		    {
			IdlPrintException(ex, id, name);
			// Can't continue, therefore jump to the next
			// yyerrok
			HANDLE_ERROR;
		    }
		}
                enumerator_COMMA_enumerator_STAR
		{
		    $$ = $<enumDef>3;

		    //
		    // Get name and id
		    //
		    Identifier_var name = $2;
		    RepositoryId_var id = IdlGetRepositoryId(name);

		    //
		    // Get member sequence
		    //
		    StringList_var members = $4;

		    //
		    // Set the member sequence
		    //
		    try
		    {
                        EnumMemberSeq memberSeq;
                        memberSeq.length(members -> size());
                        copy_to_seq(members -> begin(),
                                    members -> end(),
                                    memberSeq);
			$$ -> members(memberSeq);
		    }
		    catch(const SystemException& ex)
		    {
			IdlPrintException(ex, id, name);
		    }
 		}
		;

/* (78) */
enumerator_COMMA_enumerator_STAR: '{' enumerator COMMA_enumerator_STAR '}'
		{
		    $$ = $3;
		    $$ -> push_front($2);
		}
		| '{' error '}'
		{
		    yyerrok;
		    $$ = new OB_STL::list<String_var>;
		}
		;

/* (79) */
COMMA_enumerator_STAR: ',' enumerator COMMA_enumerator_STAR
		{
		    Identifier_var member = $2;
		    StringList_var members = $3;
		    
		    //
		    // Check for duplicate enum members
		    //
                    OB_STL::list<String_var>::iterator enumMemberIter;
                    for(enumMemberIter = members -> begin();
			enumMemberIter != members -> end();
			enumMemberIter++)
		    {
			if(IdlStringCompare(*enumMemberIter, member) == 0)
			{
			    //
                            // Shouldn't happen anymore, we check now
                            // directly in the member rules
			    //
			    assert(false);
			    String_var err =
				string_dup("duplicate enum member `");
			    err += member;
			    err += "'";
			    IdlError(err);
			    break;
			}
		    }

                    if(enumMemberIter == members -> end())
			members -> push_front(member);

		    $$ = members._retn();
		}
		| /* empty */
		{
		    $$ = new OB_STL::list<String_var>;
		}
		| ','
		{
		    $$ = new OB_STL::list<String_var>;
		    IdlError("unexpected `,'");
		}
		;

/* (79) */
enumerator	: IDL_IDENTIFIER
		{
		    //
		    // Add the name to this scope
		    //
		    if(!IdlNameMgr -> defineDecl($1))
		    {
			// Can't continue, therefore jump to the next
			// yyerrok
			HANDLE_ERROR;
		    }

		    $$ = $1;
		}
		;

/* (80) */
sequence_type	: IDL_SEQUENCE '<' simple_type_spec ',' positive_int_const '>'
		{
		    IDLType_var idlType = $3;
		    ULong bound = $5;

                    if(bound == 0)
                    {
                        IdlError("sequence bound must be > 0");
			// Can't continue, therefore jump to the next
			// yyerrok
                        HANDLE_ERROR;
                    }

		    //
		    // Create sequence
		    //
		    Repository_var rep = IdlNameMgr -> getRepository();
		    $$ = rep -> create_sequence(bound, idlType);
		}
		| IDL_SEQUENCE '<' simple_type_spec '>'
		{
		    IDLType_var idlType = $3;

		    //
		    // Create sequence
		    //
		    Repository_var rep = IdlNameMgr -> getRepository();
		    $$ = rep -> create_sequence(0, idlType);
		}
		;

/* (81) */
string_type	: IDL_STRING '<' positive_int_const '>'
		{
		    ULong bound = $3;

                    if(bound == 0)
                    {
                        IdlError("string bound must be > 0");
			// Can't continue, therefore jump to the next
			// yyerrok
                        HANDLE_ERROR;
                    }

		    //
		    // Create string
		    //
		    Repository_var rep = IdlNameMgr -> getRepository();
		    $$ = rep -> create_string(bound);
		}
		| IDL_STRING
		{
		    //
		    // Get string primitive
		    //
		    Repository_var rep = IdlNameMgr -> getRepository();
		    $$ = rep -> get_primitive(pk_string);
		}
		;

/* (82) */
wide_string_type: IDL_WSTRING '<' positive_int_const '>'
		{
		    ULong bound = $3;

                    if(bound == 0)
                    {
                        IdlError("wstring bound must be > 0");
			// Can't continue, therefore jump to the next
			// yyerrok
                        HANDLE_ERROR;
                    }

		    //
		    // Create string
		    //
		    Repository_var rep = IdlNameMgr -> getRepository();
		    $$ = rep -> create_wstring(bound);
		}
		| IDL_WSTRING
		{
		    //
		    // Get string primitive
		    //
		    Repository_var rep = IdlNameMgr -> getRepository();
		    $$ = rep -> get_primitive(pk_wstring);
		}
		;

/* (83) */
array_declarator: IDL_IDENTIFIER fixed_array_size_PLUS
		{
		    $$ = $2;
		    $$ -> identifier = $1;
		}
		;

/* (84) */
fixed_array_size_PLUS: fixed_array_size
		{
		    $$ = new IdlDeclarator;
		    $$ -> sizes.push_front($1);
		}
		| fixed_array_size fixed_array_size_PLUS
		{
		    $$ = $2;
		    $$ -> sizes.push_front($1);
		}
		;

/* (84) */
fixed_array_size: '[' positive_int_const ']'
		{
                    ULong size = $2;

                    if(size == 0)
                    {
                        IdlError("array size must be > 0");
			// Can't continue, therefore jump to the next
			// yyerrok
                        HANDLE_ERROR;
                    }

		    $$ = size;
		}
		;

/* (85) */
attr_dcl	: readonly_OPT IDL_ATTRIBUTE param_type_spec
		  simple_declarator COMMA_simple_declarator_STAR
		{
		    //
		    // Get container from container stack
		    //
		    Container_var container = IdlNameMgr -> getContainer();

		    InterfaceDef_var interf =
			InterfaceDef::_narrow(container);
		    ValueDef_var value =
			ValueDef::_narrow(container);
		    assert(!is_nil(interf) || !is_nil(value));

		    //
		    // Get attribute mode
		    //
		    AttributeMode mode = $1;

		    //
		    // Get parameter/return type
		    //
		    IDLType_var idlType = $3;

		    //
		    // Get declarators
		    //
		    IdlDeclarator_var declarator = $4;
		    IdlDeclaratorList_var declarators = $5;
		    declarators -> push_front(declarator);

                    //
                    // Check for an unmarshallable result type
                    //
                    TypeCode_var tc = idlType -> type();
                    if(!IdlIsMarshallable(tc))
                    {
                        DefinitionKind kind = container -> def_kind();
                        if(kind == dk_Interface ||
                           kind == dk_AbstractInterface)
                        {
			    IdlError("attribute type is not marshallable");
			    // Can't continue, therefore jump to the next
			    // yyerrok
			    HANDLE_ERROR;
                        }
                    }

                    OB_STL::list<IdlDeclarator_var>::iterator declIter;
                    for(declIter = declarators -> begin();
			declIter != declarators -> end();
			declIter++)
		    {
			assert((*declIter)->sizes.size() == 0);

			//
			// Get name, id and version
			//
			Identifier_var name = (*declIter) -> identifier;
			RepositoryId_var id = IdlGetRepositoryId(name);
			VersionSpec_var version =
			    string_dup("1.0");

			//
			// Add the name to this scope
			//
			if(IdlNameMgr -> defineName(name, id))
			{
			    //
			    // Create attribute
			    //
			    try
			    {
				if(!is_nil(interf))
				    release(interf ->
					    create_attribute(id, name, version,
							     idlType, mode));
				else
				    release(value ->
					    create_attribute(id, name, version,
							     idlType, mode));
			    }
			    catch(const SystemException& ex)
			    {
				IdlPrintException(ex, id, name);
			    }
			}
		    }
		}
		;

/* (86) */
except_dcl	: IDL_EXCEPTION IDL_IDENTIFIER
		{
		    //
		    // Get container from container stack
		    //
		    Container_var container = IdlNameMgr -> getContainer();

		    //
		    // Get name, id and version
		    //
		    const char* name = $2; // No _var here!
		    RepositoryId_var id = IdlGetRepositoryId(name);
		    VersionSpec_var version = string_dup("1.0");

		    //
		    // Add the name to this scope
		    //
		    if(!IdlNameMgr -> defineName(name, id, true))
		    {
			// Can't continue, therefore jump to the next
			// yyerrok
			HANDLE_ERROR;
		    }

		    //
		    // Create empty member sequence
		    //
		    StructMemberSeq_var members = new StructMemberSeq;

		    //
		    // Create new exception
		    //
		    try
		    {
			$<exceptionDef>$ = container ->
			    create_exception(id, name, version, members);
		    }
		    catch(const SystemException& ex)
		    {
			IdlPrintException(ex, id, name);
			// Can't continue, therefore jump to the next
			// yyerrok
			HANDLE_ERROR;
		    }
		    
		    //
		    // Set and push new container on the stack
		    //
		    IdlNameMgr -> setContainer(id, $<exceptionDef>$);
		    IdlNameMgr -> pushContainer(id);
		}
		'{' member_STAR '}'
		{
		    ExceptionDef_var ex = $<exceptionDef>3;

		    //
		    // Get name and id
		    //
		    Identifier_var name = $2;
		    RepositoryId_var id = IdlGetRepositoryId(name);

		    //
		    // Get member sequence
		    //
                    StructMemberList_var memberList = $5;
		    StructMemberSeq memberSeq(memberList -> size());
                    memberSeq.length(memberList -> size());
                    copy_to_seq(memberList -> begin(), memberList -> end(),
                                memberSeq);

		    //
		    // Set the member sequence
		    //
		    try
		    {
			ex -> members(memberSeq);
		    }
		    catch(const SystemException& ex)
		    {
			IdlPrintException(ex, id, name);
			// No HANDLE_ERROR here, otherwise this will jump
			// to the error branch in the member rule
		    }
			
		    //
		    // Pop container on the stack
		    //
		    IdlNameMgr -> popContainer();
		}
		;

/* (87) */
op_dcl		: op_attribute_OPT op_type_spec IDL_IDENTIFIER
                {
		    //
		    // Get the repository ID
		    //
		    RepositoryId_var id = IdlGetRepositoryId($3);

		    //
		    // Add the name to this scope. This must be done
		    // before the current container is pushed again
		    // below.
		    //
		    if(!IdlNameMgr -> defineName($3, id, true))
		    {
			// Can't continue, therefore jump to the next
			// yyerrok
			HANDLE_ERROR;
		    }

		    //
		    // Get container from container stack
		    //
		    Container_var container = IdlNameMgr -> getContainer();

                    //
                    // Check for an unmarshallable result type
                    //
                    IDLType_ptr type = $2; // No _var here!
                    TypeCode_var tc = type -> type();
                    if(!IdlIsMarshallable(tc))
                    {
                        DefinitionKind kind = container -> def_kind();
                        if(kind == dk_Interface ||
                           kind == dk_AbstractInterface)
                        {
			    IdlError("operation result type is not "
                                     "marshallable");
			    // Can't continue, therefore jump to the next
			    // yyerrok
			    HANDLE_ERROR;
                        }
                    }

		    //
		    // Set and push the same container again, but with the id
		    // of the operation, in order to introduce a new
		    // scope.
		    //
		    IdlNameMgr -> setContainer(id, container);
		    IdlNameMgr -> pushContainer(id);
                }
                parameter_dcls raises_expr_OPT context_expr_OPT
		{
		    //
		    // Pop container on the stack. This must be done
		    // before the IdlNameMgr -> defineName() below,
		    // so that the correct naming scope is used.
		    //
		    IdlNameMgr -> popContainer();

		    //
		    // Get container from container stack
		    //
		    Container_var container = IdlNameMgr -> getContainer();

		    //
		    // The container must be either an interface or a
		    // valuetype
		    //
		    InterfaceDef_var interf = InterfaceDef::_narrow(container);
		    ValueDef_var value = ValueDef::_narrow(container);
		    assert(!is_nil(interf) || !is_nil(value));

		    //
		    // Get operation mode
		    //
		    OperationMode mode = $1;

		    //
		    // Get return type
		    //
		    IDLType_var result = $2;

		    //
		    // Get member sequence
		    //
		    ParameterDescriptionList_var paramList = $5;
		    ParDescriptionSeq_var paramSeq = new ParDescriptionSeq();
                    paramSeq -> length(paramList -> size());
                    copy_to_seq(paramList -> begin(), paramList -> end(),
				paramSeq);

		    //
		    // Get exceptions definition sequence
		    //
		    ExceptionDefList_var exceptions = $6;

		    //
		    // Get context ID sequence
		    //
		    ContextIdentifierList_var contextList = $7;
		    ContextIdSeq_var contextSeq = new ContextIdSeq;
                    contextSeq -> length(contextList -> size());
                    copy_to_seq(contextList -> begin(),
                                contextList -> end(),
                                contextSeq);

		    //
		    // Get name, id and version
		    //
		    Identifier_var name = $3;
		    RepositoryId_var id = IdlGetRepositoryId(name);
		    VersionSpec_var version = string_dup("1.0");

		    //
		    // Check for oneway constraints
		    //
		    bool ok = true;
		    if(mode == OP_ONEWAY)
		    {
			TypeCode_var resultTC = result -> type();
			if(resultTC -> kind() != tk_void)
			{
			    IdlError("oneway operations cannot have "
				     "return value");
			    ok = false;
			}

			for(ULong i = 0 ; i < paramSeq -> length() ; i++)
			    if(paramSeq[i].mode != PARAM_IN)
			    {
				IdlError("oneway operations can only have "
					 "`in' parameters");
				ok = false;
			    }

			if(exceptions -> size() > 0)
			{
			    IdlError("oneway operations cannot have "
				     "user defined exceptions");
			    ok = false;
			}
		    }

                    //
                    // Check for unmarshallable parameters
                    //
                    DefinitionKind containerKind = container -> def_kind();
                    ULong i;
                    for(i = 0 ; i < paramSeq -> length() ; i++)
                    {
                        if(!IdlIsMarshallable(paramSeq[i].type))
                        {
                            if(containerKind == dk_Interface ||
                               containerKind == dk_AbstractInterface)
                            {
                                String_var err = string_dup("parameter `");
                                err += paramSeq[i].name;
                                err += "' is not marshallable";
                                IdlError(err);
                                ok = false;
                            }
                        }
                    }

                    //
                    // Check for unmarshallable exceptions
                    //
                    ExceptionDefSeq exceptionSeq(exceptions -> size());
                    exceptionSeq.length(exceptions -> size());
                    copy_to_seq(exceptions -> begin(),
                                exceptions -> end(),
                                exceptionSeq);
                    for(i = 0 ; i < exceptionSeq.length() ; i++)
                    {
                        TypeCode_var tc = exceptionSeq[i] -> type();
                        if(!IdlIsMarshallable(tc))
                        {
                            if(containerKind == dk_Interface ||
                               containerKind == dk_AbstractInterface)
                            {
                                String_var err = string_dup("exception `");
                                Identifier_var exName =
                                    exceptionSeq[i] -> name();
                                err += exName;
                                err += "' is not marshallable";
                                IdlError(err);
                                ok = false;
                            }
                        }
                    }

		    //
		    // Create operation
		    //
		    if(ok)
		    {
			try
			{
			    if(!is_nil(interf))
				release(interf ->
					create_operation(id, name, version,
							 result, mode,
							 paramSeq,
							 exceptionSeq,
							 contextSeq));
			    else
				release(value ->
					create_operation(id, name, version,
							 result, mode,
							 paramSeq,
							 exceptionSeq,
							 contextSeq));
			}
			catch(const SystemException& ex)
			{
			    IdlPrintException(ex, id, name);
			}
		    }
		}
		;

/* (88) */
op_attribute_OPT: op_attribute
		{
		     $$ = $1;
		}
		| /* empty */
		{
		     $$ = OP_NORMAL;
		}
		;

/* (88) */
op_attribute	: IDL_ONEWAY
		{
		    $$ = OP_ONEWAY;
		}
		;

/* (89) */
op_type_spec	: param_type_spec
		{
		    $$ = $1;
		}
		| IDL_VOID
		{
		    Repository_var rep = IdlNameMgr -> getRepository();
		    $$ = rep -> get_primitive(pk_void);
		}
		;

/* (90) */
parameter_dcls	: '(' param_dcl COMMA_param_dcl_STAR ')'
                {
		    ParameterDescription_var param = $2;
		    $3 -> push_front(param);
		    $$ = $3;
                }
		| '(' error ')'
		{
		    yyerrok;
		    $$ = new OB_STL::list<ParameterDescription_var>;
		}
		| '(' ')'
		{
		    $$ = new OB_STL::list<ParameterDescription_var>;
		}
		;

/* (91) */
COMMA_param_dcl_STAR: ',' param_dcl COMMA_param_dcl_STAR
		{
		    ParameterDescription_var param = $2;
		    $3 -> push_front(param);
		    $$ = $3;
		}
		| /* empty */
		{
		    $$ = new OB_STL::list<ParameterDescription_var>;
		}
		;

/* (91) */
param_dcl	: param_attribute param_type_spec simple_declarator
		{
		    ParameterMode mode = $1;
		    IDLType_var typeSpec = $2;
		    IdlDeclarator_var declarator = $3;
		    assert(declarator -> sizes.size() == 0);
		    
		    //
		    // Add the name to this scope
		    //
		    if(!IdlNameMgr -> defineDecl(declarator -> identifier))
		    {
			// Can't continue, therefore jump to the next
			// yyerrok
			HANDLE_ERROR;
		    }

		    ParameterDescription_var param = new ParameterDescription;

		    param -> name = declarator -> identifier;
		    param -> type = typeSpec -> type();
		    param -> type_def = typeSpec;
		    param -> mode = mode;

		    $$ = param._retn();
		}
		| param_type_spec
		{
		    IDLType_var typeSpec = $1;

		    IdlError("`in', `inout' or `out' missing");
		    IdlError("parameter name missing");
		    // Can't continue, therefore jump to the next
		    // yyerrok
		    HANDLE_ERROR;
		}
		| param_type_spec simple_declarator
		{
		    IDLType_var typeSpec = $1;
		    IdlDeclarator_var declarator = $2;

		    IdlError("`in', `inout' or `out' missing");
		    // Can't continue, therefore jump to the next
		    // yyerrok
		    HANDLE_ERROR;
		}
		| param_attribute param_type_spec
		{
		    IDLType_var typeSpec = $2;

		    IdlError("parameter name missing");
		    // Can't continue, therefore jump to the next
		    // yyerrok
		    HANDLE_ERROR;
		}
		;

/* (92) */
param_attribute	: IDL_IN
		{
		    $$ = PARAM_IN;
		}
		| IDL_OUT
		{
		    $$ = PARAM_OUT;
		}
		| IDL_INOUT
		{
		    $$ = PARAM_INOUT;
		}
		;

/* (93) */
raises_expr_OPT	: raises_expr
		{
		    $$ = $1;
		}
		| /* empty */
		{
		    $$ = new OB_STL::list<ExceptionDef_var>;
		}
		;

/* (93) */
raises_expr	: IDL_RAISES '(' scoped_name COMMA_scoped_name_STAR ')'
		{
		    StringList_var scopedNameSeq = $4;
		    scopedNameSeq -> push_front($3);

		    //
		    // Get container from container stack
		    //
		    Container_var container = IdlNameMgr -> getContainer();

		    //
		    // Create exception definition sequence
		    //
		    ExceptionDefList_var exDefSeq =
		        new OB_STL::list<ExceptionDef_var>;

                    StringSeq idSeq;
                    OB_STL::list<String_var>::iterator name_iter;
                    for(name_iter = scopedNameSeq -> begin();
			name_iter != scopedNameSeq -> end();
			name_iter++)
		    {
			//
			// Find scoped name
			//
			const char* scoped = *name_iter;
			IRObject_var irObj =
			    IdlFindScopedName(container, scoped);

			if(is_nil(irObj))
			    continue;
			
			ExceptionDef_var exceptionDef =
			    ExceptionDef::_narrow(irObj);

			if(is_nil(exceptionDef))
			{	
			    String_var err = string_dup("`");
			    err += scoped;
			    err += "' is not an exception type";
			    IdlError(err);
			    continue;
			}

                        String_var id = exceptionDef -> id();
                        for(ULong i = 0 ; i < idSeq.length() ; i++)
                            if(strcmp(id, idSeq[i]) == 0)
                            {
                                String_var err = string_dup("exception `");
                                err += scoped;
                                err += "' occurs more than once";
                                IdlError(err);
                                continue;
                            }
                        idSeq.length(idSeq.length() + 1);
                        idSeq[idSeq.length() - 1] = id;

			exDefSeq -> push_back(exceptionDef);
		    }

		    $$ = exDefSeq._retn();
		}
		;

/* (94) */
context_expr_OPT: context_expr
		{
		    $$ = $1;
		}
		| /* empty */
		{
		    $$ = new OB_STL::list<ContextIdentifier_var>;
		}
		;

/* (94) */
context_expr	: IDL_CONTEXT '(' string_literal COMMA_string_literal_STAR ')'
		{
		    String_var string = $3;
		    String_var origString = string_dup(string);
                    ContextIdentifierList_var result = $4;

		    if(strlen(string) == 0)
		    {
			IdlError("empty string cannot be used as context");
			// Can't continue, therefore jump to the next
			// yyerrok
			HANDLE_ERROR;
		    }

                    ULong i;
		    for(i = 0 ; i < strlen(string) ; i++)
		    {
			char c = string[i];

			if(isalpha(c))
			    continue;

			if(i != 0 && (isdigit(c) || c == '_' || c == '.'))
			    continue;

			if(i == strlen(string) - 1 && c == '*')
			    continue;

			IdlError("illegal characters in context");
			// Can't continue, therefore jump to the next
			// yyerrok
			HANDLE_ERROR;
		    }

		    result -> push_front(string);
		    $$ = result._retn();
		}
		;

/* (95) */
param_type_spec	: base_type_spec
		{
		    $$ = $1;
		}
		| sequence_type
		{
		    IDLType_var typeSpec = $1;

		    IdlError("typedefs must be used for sequence type "
			     "parameters");
		    // Can't continue, therefore jump to the next
		    // yyerrok
		    HANDLE_ERROR;
		}
		| string_type
		{
		    $$ = $1;
		}
		| wide_string_type
		{
		    $$ = $1;
		}
		| scoped_name
		{
		    ScopedName_var scoped = $1;

		    //
		    // Get container from container stack
		    //
		    Container_var container = IdlNameMgr -> getContainer();

		    //
		    // Find scoped name
		    //
		    IRObject_var irObj = IdlFindScopedName(container, scoped);

		    if(is_nil(irObj))
		    {
			// Can't continue, therefore jump to the next
			// yyerrok
			HANDLE_ERROR;
		    }

		    IDLType_var idlType = IDLType::_narrow(irObj);
		    if(is_nil(idlType))
		    {
			String_var err = string_dup("`");
			err += scoped;
			err += "' is not a parameter type specification";
			IdlError(err);
			// Can't continue, therefore jump to the next
			// yyerrok
			HANDLE_ERROR;
		    }

		    $$ = idlType._retn();
		}
		;

/* (96) */
fixed_pt_type	: IDL_FIXED '<' positive_int_const ',' positive_int_const '>'
		{
		    if(!IdlAllowFixed)
		    {
			IdlError("IDL fixed type is not supported");
			// Can't continue, therefore jump to the next
			// yyerrok
			HANDLE_ERROR;
		    }

		    ULong digits = $3;
		    ULong scale = $5;

                    if(digits < 1 || digits > 31)
                    {
			IdlError("fixed digits must be in the range 1-31");
			// Can't continue, therefore jump to the next
			// yyerrok
			HANDLE_ERROR;
                    }

                    if(scale > digits)
                    {
			IdlError("fixed scale must be less than or "
                                 "equal to digits");
			// Can't continue, therefore jump to the next
			// yyerrok
			HANDLE_ERROR;
                    }

		    //
		    // Create fixed type
		    //
		    Repository_var rep = IdlNameMgr -> getRepository();
		    $$ = rep -> create_fixed((UShort)digits, (Short)scale);
		}
		;

/* (97) */
fixed_pt_const_type: IDL_FIXED
		{
		    if(!IdlAllowFixed)
		    {
			IdlError("IDL fixed type is not supported");
			// Can't continue, therefore jump to the next
			// yyerrok
			HANDLE_ERROR;
		    }

		    Repository_var rep = IdlNameMgr -> getRepository();
		    $$ = rep -> create_fixed(1, 0);
		}
		;

/* (98) */
value_base_type	: IDL_VALUEBASE
		{
		    if(!IdlAllowValueTypes)
		    {
			IdlError("IDL value types are not supported");
			// Can't continue, therefore jump to the next
			// yyerrok
			HANDLE_ERROR;
		    }

		    Repository_var rep = IdlNameMgr -> getRepository();
		    $$ = rep -> get_primitive(pk_value_base);
		}
		;

/* (99) */
constr_forward_dcl : struct_forward_dcl
		{
		}
                | union_forward_dcl
		{
		}
		;

/* (99) */
struct_forward_dcl : IDL_STRUCT IDL_IDENTIFIER
		{
		    //
		    // Get container from container stack
		    //
		    Container_var container = IdlNameMgr -> getContainer();

		    //
		    // Get name, id and version
		    //
		    Identifier_var name = $2;
		    RepositoryId_var id = IdlGetRepositoryId(name);
		    VersionSpec_var version = string_dup("1.0");

		    //
		    // Add the forward declaration to this scope
		    //
		    if(IdlNameMgr -> addForwardDecl(name, id))
		    {
			//
			// Check if name already exists
			//
			try
			{
			    Contained_var c = container -> lookup(name);

			    StructDef_var st;
			    if(!is_nil(c))
			    {
				st = StructDef::_narrow(c);

				//
				// Redefinition is legal in inherited
				// interface, so check the repository IDs
				//
				RepositoryId_var containedId = c -> id();
				if(strcmp(id, containedId) == 0)
				{
				    //
				    // Name exists, check if it is a struct
				    //
				    if(is_nil(st))
				    {
					String_var err = string_dup(
					    "conflicting types for `struct ");
					err += name;
					err += "'";
					IdlError(err);
					// Can't continue, therefore jump to the next
					// yyerrok
					HANDLE_ERROR;
				    }
				}
			    }

			    if(is_nil(st))
			    {
                                //
                                // Create "empty" struct as forward declaration
                                //
				StructMemberSeq seq;
				StructDef_var st = container ->
				    create_struct(id, name, version, seq);
			    }
			}
			catch(const SystemException& ex)
			{
			    IdlPrintException(ex, id, name);
			}
		    }
		}
		;

/* (99) */
union_forward_dcl : IDL_UNION IDL_IDENTIFIER
		{
		    //
		    // Get container from container stack
		    //
		    Container_var container = IdlNameMgr -> getContainer();

		    //
		    // Get name, id and version
		    //
		    Identifier_var name = $2;
		    RepositoryId_var id = IdlGetRepositoryId(name);
		    VersionSpec_var version = string_dup("1.0");

		    //
		    // Add the forward declaration to this scope
		    //
		    if(IdlNameMgr -> addForwardDecl(name, id))
		    {
			//
			// Check if name already exists
			//
			try
			{
			    Contained_var c = container -> lookup(name);

			    UnionDef_var un;
			    if(!is_nil(c))
			    {
				un = UnionDef::_narrow(c);

				//
				// Redefinition is legal in inherited
				// interface, so check the repository IDs
				//
				RepositoryId_var containedId = c -> id();
				if(strcmp(id, containedId) == 0)
				{
				    //
				    // Name exists, check if it is a union
				    //
				    if(is_nil(un))
				    {
					String_var err = string_dup(
					    "conflicting types for `union ");
					err += name;
					err += "'";
					IdlError(err);
					// Can't continue, therefore jump to the next
					// yyerrok
					HANDLE_ERROR;
				    }
				}
			    }

			    if(is_nil(un))
			    {
                                //
                                // Create "empty" union as forward declaration
                                //
				UnionMemberSeq seq;
				Repository_var rep =
				    IdlNameMgr -> getRepository();
				IDLType_var disc =
				    rep -> get_primitive(pk_short);
				UnionDef_var un = container ->
				    create_union(id, name, version, disc, seq);
			    }
			}
			catch(const SystemException& ex)
			{
			    IdlPrintException(ex, id, name);
			}
		    }
		}
		;

/* other */
readonly_OPT	: IDL_READONLY
		{
		    $$ = ATTR_READONLY;
		}
		| /* empty */
		{
		    $$ = ATTR_NORMAL;
		}
		;

/* other */
string_literal	: IDL_STRING_LITERAL
                {
		    String_var string = $1;

		    String_var origString = string;
		    if(!IdlRemoveEscapes(string.inout(), true))
		    {
			String_var err = string_dup("`");
			err += origString;
			err += "' contains \\0 character";
			IdlError(err);
			// Can't continue, therefore jump to the next
			// yyerrok
			HANDLE_ERROR;
		    }

		    $$ = string._retn();
		}
		| IDL_STRING_LITERAL string_literal
                {
		    String_var string1 = $1;
		    String_var string2 = $2;

		    String_var origString = string1;
		    if(!IdlRemoveEscapes(string1.inout(), true))
		    {
			String_var err = string_dup("`");
			err += origString;
			err += "' contains \\0 character";
			IdlError(err);
			// Can't continue, therefore jump to the next
			// yyerrok
			HANDLE_ERROR;
		    }

		    char* result = string_alloc(strlen(string1) +
						strlen(string2));
		    strcpy(result, string1.in());
		    strcat(result, string2.in());
		    $$ = result;
                }
		;

/* other */
COMMA_string_literal_STAR: ',' string_literal COMMA_string_literal_STAR
		{
		    String_var string = $2;
		    StringList_var result = $3;
		    result -> push_front(string);
		    $$ = result._retn();
		}
		| /* empty */
		{
		    $$ = new OB_STL::list<String_var>;
		}
		;

/* other */
wide_string_literal: IDL_WIDE_STRING_LITERAL
                {
		    String_var string = $1;
                    assert(string[0] == 'L');

		    String_var origString = string;
                    WString_var wstring;
		    if(!IdlRemoveEscapes(string.in() + 1, wstring.out(), true))
		    {
			String_var err = string_dup("`");
			err += origString;
			err += "' contains \\0 character";
			IdlError(err);
			// Can't continue, therefore jump to the next
			// yyerrok
			HANDLE_ERROR;
		    }
		    
		    $$ = wstring._retn();
		}
		| IDL_WIDE_STRING_LITERAL wide_string_literal
                {
		    String_var string1 = $1;
                    assert(string1[0] == 'L');
		    WString_var wstring2 = $2;

		    String_var origString = string1;
                    WString_var wstring1;
		    if(!IdlRemoveEscapes(string1.in() + 1, wstring1.out(),
					 true))
		    {
			String_var err = string_dup("`");
			err += origString;
			err += "' contains \\0 character";
			IdlError(err);
			// Can't continue, therefore jump to the next
			// yyerrok
			HANDLE_ERROR;
		    }

		    wchar_t* result = wstring_alloc(OB::wcslen(wstring1) +
						    OB::wcslen(wstring2));
		    OB::wcscpy(result, wstring1.in());
		    OB::wcscat(result, wstring2.in());
		    $$ = result;
                }
		;

/* other */
interfaceType_OPT   : IDL_ABSTRACT
                {
		    $$ = dk_AbstractInterface;
		}
                | IDL_LOCAL
                {
		    $$ = dk_LocalInterface;
		}
                | /* empty */
                {
		    $$ = dk_Interface;
		}
                ;
 
/* other */
truncatable_OPT	: IDL_TRUNCATABLE
		{
		    $$ = true;
		}
		| /* empty */
		{
		    $$ = false;
		}
		;

/* other */
public_OR_private: IDL_PUBLIC
		{
		    $$ = PUBLIC_MEMBER;
		}
		| IDL_PRIVATE
		{
		    $$ = PRIVATE_MEMBER;
		}
		;

%%

// ----------------------------------------------------------------------
// Do parsing
// ----------------------------------------------------------------------

int
IdlParse(ORB_ptr orb,
	 Repository_ptr repository,
	 StringSeq& includes,
	 bool all,
	 bool ignoreError,
	 bool caseSensitive,
	 bool allowLongDouble,
	 bool allowFixed,
	 bool allowValueTypes,
	 bool allowModuleReopening,
	 IdlStringHash& content,
	 IdlStringHash& contentFwd)
{
    //
    // Initialize global variables
    //	 
    IdlLineno = 1;
    IdlDocComment = string_dup("");
    IdlORB = orb;
    IdlNameMgr =
	new IdlNameManager(repository, includes, all, content, contentFwd);
    IdlTypeCodeEnabled = false;
    IdlPrincipalEnabled = false;
    IdlIgnoreError = ignoreError;
    IdlCaseSensitive = caseSensitive;
    IdlAllowLongDouble = allowLongDouble;
    IdlAllowFixed = allowFixed;
    IdlAllowValueTypes = allowValueTypes;
    IdlAllowModuleReopening = allowModuleReopening;

    //
    // Parse
    //
    yynerrs = 0;
    int yyparse();
    int status = yyparse();
    assert(yynerrs > 0 || status == 0);

    //
    // Check for incomplete forward declarations in global namespace
    //
    if(!yynerrs)
	IdlNameMgr -> checkIncompleteDecl();

    //
    // Do some cleanup
    //
    delete IdlNameMgr;

    //
    // Important note:
    //
    // If IdlParse returns with a status != 0, it is most certain that
    // there are memory leaks. The reason for that is that in case of
    // an error, it is not possible to free any values of the yyval
    // union that have been created with "new".
    //

    //
    // Return the status
    //
    return yynerrs;
}

// ----------------------------------------------------------------------
// Create valuetype forward declaration
// ----------------------------------------------------------------------

bool
CreateValueFwd(const char* name, Boolean isAbstract)
{
    if(!IdlAllowValueTypes)
    {
	IdlError("IDL value types are not supported");
	// Can't continue, therefore jump to the next
	// yyerrok
	return false;
    }

    //
    // Get container from container stack
    //
    Container_var container = IdlNameMgr -> getContainer();

    //
    // Get id and version
    //
    RepositoryId_var id = IdlGetRepositoryId(name);
    VersionSpec_var version = string_dup("1.0");

    //
    // Add the forward declaration to this scope
    //
    if(IdlNameMgr -> addForwardDecl(name, id))
    {
	//
	// Check if name already exists
	//
	try
	{
	    Contained_var c = container -> lookup(name);
	    if(!is_nil(c))
	    {
		//
		// Name exists, check if it is a valuetype
		//
		ValueDef_var value = ValueDef::_narrow(c);
		if(is_nil(value))
		{
		    String_var err =
			string_dup("conflicting types for `valuetype ");
		    err += name;
		    err += "'";
		    IdlError(err);
		    return true;
		}
		else
		{
		    //
		    // Make sure forward decl matches
		    //
		    if(isAbstract != value -> is_abstract())
		    {
			String_var err = string_dup("valuetype `");
			err += name;
			err += "' does not match forward declaration";
			IdlError(err);
			return true;
		    }
		}
	    }
	    else
	    {
		//
		// Create "empty" valuetype as forward declaration, do
		// not put it on the container stack, but put it in
		// the list of forward declared valuetypes
		//
		ValueDefSeq baseValues;
		InitializerSeq initializers;
		InterfaceDefSeq supportedInterfaces;

		ValueDef_var value = container ->
		    create_value(id, name, version, false, isAbstract,
				 ValueDef::_nil(), false, baseValues,
				 supportedInterfaces, initializers);
	    }
	}
	catch(const SystemException& ex)
	{
	    IdlPrintException(ex, id, name);
	    // Can't continue, therefore jump to the next
	    // yyerrok
	    return false;
	}
    }

    return true;
}

// ----------------------------------------------------------------------
// Create valuetype
// ----------------------------------------------------------------------

bool
CreateValue(const char* name, const IdlValueInfo& info, Boolean isCustom,
	    Boolean isAbstract)
{
    if(!IdlAllowValueTypes)
    {
	IdlError("IDL value types are not supported");
	// Can't continue, therefore jump to the next
	// yyerrok
	return false;
    }

    //
    // Get container from container stack
    //
    Container_var container = IdlNameMgr -> getContainer();

    //
    // Get id and version
    //
    RepositoryId_var id = IdlGetRepositoryId(name);
    VersionSpec_var version = string_dup("1.0");

    //
    // Get the inheritance spec as a sequence of IDs
    //
    ULong nameCount =
	info.supported_interfaces.size() + info.base_values.size();
    
    StringSeq idSeq(nameCount);
    idSeq.length(nameCount);

    ULong i = 0;

    OB_STL::list<InterfaceDef_var>::const_iterator iDefIter;
    for(iDefIter = info.supported_interfaces.begin();
	iDefIter != info.supported_interfaces.end();
	iDefIter++)
    {
	idSeq[i] = (*iDefIter) -> id();
        i++;
    }

    OB_STL::list<ValueDef_var>::const_iterator baseValuesIter;
    for(baseValuesIter = info.base_values.begin();
	baseValuesIter != info.base_values.end();
	baseValuesIter++)
    {
	idSeq[i] = (*baseValuesIter) -> id();
        i++;
    }

    //
    // Add the name to this scope
    //
    if(!IdlNameMgr -> defineName(name, id, true, &idSeq, true))
    {
	// Can't continue, therefore jump to the next
	// yyerrok
	return false;
    }

    //
    // Check if name already exists
    //
    Contained_var c;
    try
    {
	c = container -> lookup(name);
    }
    catch(const SystemException& ex)
    {
	IdlPrintException(ex, id, name);
	// Can't continue, therefore jump to the next yyerrok
	return false;
    }

    ValueDef_var value;
    if(!is_nil(c))
    {
	//
	// Name exists, check if it is a valuetype forward declaration
	//
	value = ValueDef::_narrow(c);
	if(is_nil(value))
	{	 
	    String_var err = string_dup("conflicting types for `valuetype ");
	    err += name;
	    err += "'";
	    IdlError(err);
	    // Can't continue, therefore jump to the next yyerrok
	    return false;
	}
	
	//
	// Make sure forward decl matches
	//
	if(isAbstract != value -> is_abstract())
	{
	    String_var err = string_dup("valuetype `");
	    err += name;
	    err += "' does not match forward declaration";
	    IdlError(err);
	    // Can't continue, therefore jump to the next yyerrok
	    return false;
	}

	try
	{
            ValueDef_var baseValue;
            ValueDefSeq abstractBaseValuesSeq;

            //
            // Determine base value and abstract base values. If this
            // value is abstract, then all base values must be abstract.
            // Otherwise, if the first base value is abstract, then
            // all base values must be abstract. (If there is a concrete
            // base value, it must be listed first)
            //
            if(info.base_values.size() > 0)
            {
                OB_STL::list<ValueDef_var> abstractBaseValues;

                if(isAbstract)
                {
                    abstractBaseValues = info.base_values;
                }
                else if(info.base_values.front() -> is_abstract())
                {
                    abstractBaseValues = info.base_values;
                }
                else
                {
                    baseValue = ValueDef::_duplicate(info.base_values.front());
                    abstractBaseValues = info.base_values;
                    abstractBaseValues.pop_front();
                }

                abstractBaseValuesSeq.length(abstractBaseValues.size());
                copy_to_seq(abstractBaseValues.begin(),
                            abstractBaseValues.end(),
                            abstractBaseValuesSeq);
            }

            //
            // A truncatable valuetype must have a concrete base valuetype
            //
            if(info.is_truncatable && is_nil(baseValue))
            {
                String_var err = string_dup("truncatable valuetype `");
                err += name;
                err += "' does not have a concrete base valuetype";
                IdlError(err);
                // Can't continue, therefore jump to the next yyerrok
                return false;
            }

	    //
	    // Fill valuetype
	    //
            InterfaceDefSeq iDefSeq(info.supported_interfaces.size());
            iDefSeq.length(info.supported_interfaces.size());
            copy_to_seq(info.supported_interfaces.begin(),
                        info.supported_interfaces.end(),
                        iDefSeq);
	    value -> is_custom(isCustom);
            value -> base_value(baseValue);
            value -> abstract_base_values(abstractBaseValuesSeq);
	    value -> supported_interfaces(iDefSeq);
	    value -> is_truncatable(info.is_truncatable);

	    //
	    // Move value (this will make this valuetype
	    // the last entry in the container)
	    //
	    value -> move(container, name, version);
	}
	catch(const SystemException& ex)
	{
	    IdlPrintException(ex, id, name);
	    // Can't continue, therefore jump to the next yyerrok
	    return false;
	}
    }
    else
    {
	//
	// Create new value
	//
	try
	{
	    InitializerSeq initializers;
	    ValueDef_var baseValue;
	    OB_STL::list<ValueDef_var> abstractBaseValues;

            //
            // Determine base value and abstract base values. If this
            // value is abstract, then all base values must be abstract.
            // Otherwise, if the first base value is abstract, then
            // all base values must be abstract. (If there is a concrete
            // base value, it must be listed first)
            //
	    if(isAbstract) 
            {
		abstractBaseValues = info.base_values;
            }
	    else if(info.base_values.size() > 0)
	    {
		if(info.base_values.front() -> is_abstract())
		    abstractBaseValues = info.base_values;
		else
		{
		    baseValue = ValueDef::_duplicate(info.base_values.front());
		    abstractBaseValues = info.base_values;
                    abstractBaseValues.pop_front();
		}
	    }

            //
            // A truncatable valuetype must have a concrete base valuetype
            //
            if(info.is_truncatable && is_nil(baseValue))
            {
                String_var err = string_dup("truncatable valuetype `");
                err += name;
                err += "' does not have a concrete base valuetype";
                IdlError(err);
                // Can't continue, therefore jump to the next yyerrok
                return false;
            }

            ValueDefSeq abstractBaseValuesSeq(abstractBaseValues.size());
            abstractBaseValuesSeq.length(abstractBaseValues.size());
            copy_to_seq(abstractBaseValues.begin(),
                        abstractBaseValues.end(),
                        abstractBaseValuesSeq);

            InterfaceDefSeq iDefSeq(info.supported_interfaces.size());
            iDefSeq.length(info.supported_interfaces.size());
            copy_to_seq(info.supported_interfaces.begin(),
                        info.supported_interfaces.end(),
                        iDefSeq);

	    value = container ->  create_value(id, name, version,
					       isCustom, isAbstract,
					       baseValue,
					       info.is_truncatable,
					       abstractBaseValuesSeq,
					       iDefSeq,
					       initializers);
	}
	catch(const SystemException& ex)
	{
	    IdlPrintException(ex, id, name);
	    // Can't continue, therefore jump to the next yyerrok
	    return false;
	}
    }

    //
    // Set and push new container on the stack
    //
    IdlNameMgr -> setContainer(id, value);
    IdlNameMgr -> pushContainer(id);

    return true;
}

// ----------------------------------------------------------------------
// Error output
// ----------------------------------------------------------------------

void
yyerror(const char* s)
{
    IdlError(s);
}
