// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>
#include <OB/Hashtable.h>

#include <Util.h>
#include <IFR3_impl.h>
#ifdef HAVE_STRINGS_H
#    include <strings.h>
#endif

using namespace CORBA;
using namespace OB;
using namespace PortableServer;

// ----------------------------------------------------------------------
// Repository_impl member implementations
// ----------------------------------------------------------------------

void
Repository_impl::add(Contained_impl* cont)
{
    RepositoryId_var id = cont -> id();
    assert(!repIdMap_ -> containsKey(id));
    repIdMap_ -> put(id, cont);
}

void
Repository_impl::remove(Contained_impl* cont)
{
    RepositoryId_var id = cont -> id();
    assert(repIdMap_ -> containsKey(id));
    repIdMap_ -> remove(id);
}

void
Repository_impl::removeAnon(IDLType_impl* anon)
{
    OB_STL::list<IDLType_impl*>::iterator iter;
    for(iter = anonImplList_.begin();
	iter != anonImplList_.end();
	iter++)
    {
	if(*iter == anon)
	{
	    anonImplList_.erase(iter);
	    anon -> _remove_ref();
	    return;
	} 
    }

    assert(false);
}

Repository_impl::Repository_impl(ORB_ptr orb, POA_ptr poa)
    : IRObject_impl(orb, poa, 0),
      Container_impl(orb, poa, 0),
      count_(0)
{
    repository_ = this;

    primDefNullI_ = 0;
    primDefVoidI_ = 0;
    primDefShortI_ = 0;
    primDefLongI_ = 0;
    primDefUShortI_ = 0;
    primDefULongI_ = 0;
    primDefFloatI_ = 0;
    primDefDoubleI_ = 0;
    primDefBooleanI_ = 0;
    primDefCharI_ = 0;
    primDefOctetI_ = 0;
    primDefAnyI_ = 0;
    primDefTypeCodeI_ = 0;
    primDefPrincipalI_ = 0;
    primDefStringI_ = 0;
    primDefObjrefI_ = 0;
    primDefLongLongI_ = 0;
    primDefULongLongI_ = 0;
    primDefLongDoubleI_ = 0;
    primDefWCharI_ = 0;
    primDefWStringI_ = 0;
    primDefValueBaseI_ = 0;

    repIdMap_ = new RepIdMap(1023);

#ifndef NDEBUG
    objectCount_ = 0;
#endif
}    
    
Repository_impl::~Repository_impl()
{
    delete repIdMap_;
    repository_ = 0;
}

void
Repository_impl::destroy()
    throw(SystemException)
{
    //
    // Invoking destroy on a Repository is an error
    //
    throw BAD_INV_ORDER(MinorIndestructibleObject, COMPLETED_NO);
}

void
Repository_impl::destroyRepository()
    throw(SystemException)
{
    if(primDefNullI_)
    {
	primDefNullI_ -> destroyPrimitive();
	primDefNullI_ -> _remove_ref();
	primDefNullI_ = 0;
    }

    if(primDefVoidI_)
    {
	primDefVoidI_ -> destroyPrimitive();
	primDefVoidI_ -> _remove_ref();
	primDefVoidI_ = 0;
    }

    if(primDefShortI_)
    {
	primDefShortI_ -> destroyPrimitive();
	primDefShortI_ -> _remove_ref();
	primDefShortI_ = 0;
    }

    if(primDefLongI_)
    {
	primDefLongI_ -> destroyPrimitive();
	primDefLongI_ -> _remove_ref();
	primDefLongI_ = 0;
    }

    if(primDefUShortI_)
    {
	primDefUShortI_ -> destroyPrimitive();
	primDefUShortI_ -> _remove_ref();
	primDefUShortI_ = 0;
    }

    if(primDefULongI_)
    {
	primDefULongI_ -> destroyPrimitive();
	primDefULongI_ -> _remove_ref();
	primDefULongI_ = 0;
    }

    if(primDefFloatI_)
    {
	primDefFloatI_ -> destroyPrimitive();
	primDefFloatI_ -> _remove_ref();
	primDefFloatI_ = 0;
    }

    if(primDefDoubleI_)
    {
	primDefDoubleI_ -> destroyPrimitive();
	primDefDoubleI_ -> _remove_ref();
	primDefDoubleI_ = 0;
    }

    if(primDefBooleanI_)
    {
	primDefBooleanI_ -> destroyPrimitive();
	primDefBooleanI_ -> _remove_ref();
	primDefBooleanI_ = 0;
    }

    if(primDefCharI_)
    {
	primDefCharI_ -> destroyPrimitive();
	primDefCharI_ -> _remove_ref();
	primDefCharI_ = 0;
    }

    if(primDefOctetI_)
    {
	primDefOctetI_ -> destroyPrimitive();
	primDefOctetI_ -> _remove_ref();
	primDefOctetI_ = 0;
    }

    if(primDefAnyI_)
    {
	primDefAnyI_ -> destroyPrimitive();
	primDefAnyI_ -> _remove_ref();
	primDefAnyI_ = 0;
    }

    if(primDefTypeCodeI_)
    {
	primDefTypeCodeI_ -> destroyPrimitive();
	primDefTypeCodeI_ -> _remove_ref();
	primDefTypeCodeI_ = 0;
    }

    if(primDefPrincipalI_)
    {
	primDefPrincipalI_ -> destroyPrimitive();
	primDefPrincipalI_ -> _remove_ref();
	primDefPrincipalI_ = 0;
    }

    if(primDefStringI_)
    {
	primDefStringI_ -> destroyPrimitive();
	primDefStringI_ -> _remove_ref();
	primDefStringI_ = 0;
    }

    if(primDefObjrefI_)
    {
	primDefObjrefI_ -> destroyPrimitive();
	primDefObjrefI_ -> _remove_ref();
	primDefObjrefI_ = 0;
    }

    if(primDefLongLongI_)
    {
	primDefLongLongI_ -> destroyPrimitive();
	primDefLongLongI_ -> _remove_ref();
	primDefLongLongI_ = 0;
    }

    if(primDefULongLongI_)
    {
	primDefULongLongI_ -> destroyPrimitive();
	primDefULongLongI_ -> _remove_ref();
	primDefULongLongI_ = 0;
    }

    if(primDefLongDoubleI_)
    {
	primDefLongDoubleI_ -> destroyPrimitive();
	primDefLongDoubleI_ -> _remove_ref();
	primDefLongDoubleI_ = 0;
    }

    if(primDefWCharI_)
    {
	primDefWCharI_ -> destroyPrimitive();
	primDefWCharI_ -> _remove_ref();
	primDefWCharI_ = 0;
    }

    if(primDefWStringI_)
    {
	primDefWStringI_ -> destroyPrimitive();
	primDefWStringI_ -> _remove_ref();
	primDefWStringI_ = 0;
    }

    if(primDefValueBaseI_)
    {
	primDefValueBaseI_ -> destroyPrimitive();
	primDefValueBaseI_ -> _remove_ref();
	primDefValueBaseI_ = 0;
    }

    OB_STL::list<IDLType_impl*> anonImplListCopy = anonImplList_;
    OB_STL::list<IDLType_impl*>::iterator iter;

    for(iter = anonImplListCopy.begin();
	iter != anonImplListCopy.end();
	iter++)
    {
	(*iter) -> destroy();
    }

    assert(anonImplList_.empty());

    Container_impl::destroy();

    assert(objectCount_ == 0);
}

Contained_ptr
Repository_impl::lookup(const char* search_name)
    throw(SystemException)
{
    //
    // Make name relative (everything is relative to the repository)
    //
    while(strncmp(search_name, "::", 2) == 0)
	search_name += 2;

    return Container_impl::lookup(search_name);
}

Contained_ptr
Repository_impl::lookup_id(const char* search_id)
    throw(SystemException)
{
    Contained_impl* cont;
    if(repIdMap_ -> get(search_id, cont))
	return cont -> _this();
    else
	return Contained::_nil();
}

TypeCode_ptr
Repository_impl::get_canonical_typecode(TypeCode_ptr tc)
    throw(SystemException)
{
    // ORBacus TypeCodes are always canonical
    return TypeCode::_duplicate(tc);
}

PrimitiveDef_ptr
Repository_impl::get_primitive(PrimitiveKind kind)
    throw(SystemException)
{
    CORBA::String_var name = CORBA::string_dup("PrimitiveDef_");
    switch(kind)
    {
    case pk_null:
	if(!primDefNullI_)
	{
	    primDefNullI_ = new PrimitiveDef_impl(orb_, poa_, this, pk_null);
	    name += count_++;
	    ObjectId_var oid = string_to_ObjectId(name);
	    poa_ -> activate_object_with_id(oid, primDefNullI_);
	}
	return primDefNullI_ -> _this();
	
    case pk_void:
	if(!primDefVoidI_)
	{
	    primDefVoidI_ = new PrimitiveDef_impl(orb_, poa_, this, pk_void);
	    name += count_++;
	    ObjectId_var oid = string_to_ObjectId(name);
	    poa_ -> activate_object_with_id(oid, primDefVoidI_);
	}
	return primDefVoidI_ -> _this();
	
    case pk_short:
	if(!primDefShortI_)
	{
	    primDefShortI_ = new PrimitiveDef_impl(orb_, poa_, this, pk_short);
	    name += count_++;
	    ObjectId_var oid = string_to_ObjectId(name);
	    poa_ -> activate_object_with_id(oid, primDefShortI_);
	}
	return primDefShortI_ -> _this();
	
    case pk_long:
	if(!primDefLongI_)
	{
	    primDefLongI_ = new PrimitiveDef_impl(orb_, poa_, this, pk_long);
	    name += count_++;
	    ObjectId_var oid = string_to_ObjectId(name);
	    poa_ -> activate_object_with_id(oid, primDefLongI_);
	}
	return primDefLongI_ -> _this();
	
    case pk_ushort:
	if(!primDefUShortI_)
	{
	    primDefUShortI_ = new PrimitiveDef_impl(orb_, poa_, this,
                                                    pk_ushort);
	    name += count_++;
	    ObjectId_var oid = string_to_ObjectId(name);
	    poa_ -> activate_object_with_id(oid, primDefUShortI_);
	}
	return primDefUShortI_ -> _this();
	
    case pk_ulong:
	if(!primDefULongI_)
	{
	    primDefULongI_ = new PrimitiveDef_impl(orb_, poa_, this, pk_ulong);
	    name += count_++;
	    ObjectId_var oid = string_to_ObjectId(name);
	    poa_ -> activate_object_with_id(oid, primDefULongI_);
	}
	return primDefULongI_ -> _this();
	
    case pk_float:
	if(!primDefFloatI_)
	{
	    primDefFloatI_ = new PrimitiveDef_impl(orb_, poa_, this, pk_float);
	    name += count_++;
	    ObjectId_var oid = string_to_ObjectId(name);
	    poa_ -> activate_object_with_id(oid, primDefFloatI_);
	}
	return primDefFloatI_ -> _this();
	
    case pk_double:
	if(!primDefDoubleI_)
	{
	    primDefDoubleI_ = new PrimitiveDef_impl(orb_, poa_, this,
                                                    pk_double);
	    name += count_++;
	    ObjectId_var oid = string_to_ObjectId(name);
	    poa_ -> activate_object_with_id(oid, primDefDoubleI_);
	}
	return primDefDoubleI_ -> _this();
	
    case pk_boolean:
	if(!primDefBooleanI_)
	{
	    primDefBooleanI_ = new PrimitiveDef_impl(orb_, poa_, this,
                                                     pk_boolean);
	    name += count_++;
	    ObjectId_var oid = string_to_ObjectId(name);
	    poa_ -> activate_object_with_id(oid, primDefBooleanI_);
	}
	return primDefBooleanI_ -> _this();
	
    case pk_char:
	if(!primDefCharI_)
	{
	    primDefCharI_ = new PrimitiveDef_impl(orb_, poa_, this, pk_char);
	    name += count_++;
	    ObjectId_var oid = string_to_ObjectId(name);
	    poa_ -> activate_object_with_id(oid, primDefCharI_);
	}
	return primDefCharI_ -> _this();
	
    case pk_octet:
	if(!primDefOctetI_)
	{
	    primDefOctetI_ = new PrimitiveDef_impl(orb_, poa_, this, pk_octet);
	    name += count_++;
	    ObjectId_var oid = string_to_ObjectId(name);
	    poa_ -> activate_object_with_id(oid, primDefOctetI_);
	}
	return primDefOctetI_ -> _this();
	
    case pk_any:
	if(!primDefAnyI_)
	{
	    primDefAnyI_ = new PrimitiveDef_impl(orb_, poa_, this, pk_any);
	    name += count_++;
	    ObjectId_var oid = string_to_ObjectId(name);
	    poa_ -> activate_object_with_id(oid, primDefAnyI_);
	}
	return primDefAnyI_ -> _this();
	
    case pk_TypeCode:
	if(!primDefTypeCodeI_)
	{
	    primDefTypeCodeI_ = new PrimitiveDef_impl(orb_, poa_, this,
                                                      pk_TypeCode);
	    name += count_++;
	    ObjectId_var oid = string_to_ObjectId(name);
	    poa_ -> activate_object_with_id(oid, primDefTypeCodeI_);
	}
	return primDefTypeCodeI_ -> _this();
	
    case pk_Principal:
	if(!primDefPrincipalI_)
	{
	    primDefPrincipalI_ = new PrimitiveDef_impl(orb_, poa_, this,
						       pk_Principal);
	    name += count_++;
	    ObjectId_var oid = string_to_ObjectId(name);
	    poa_ -> activate_object_with_id(oid, primDefPrincipalI_);
	}
	return primDefPrincipalI_ -> _this();
	
    case pk_string:
	if(!primDefStringI_)
	{
	    primDefStringI_ = new PrimitiveDef_impl(orb_, poa_, this,
                                                    pk_string);
	    name += count_++;
	    ObjectId_var oid = string_to_ObjectId(name);
	    poa_ -> activate_object_with_id(oid, primDefStringI_);
	}
	return primDefStringI_ -> _this();
	
    case pk_objref:
	if(!primDefObjrefI_)
	{
	    primDefObjrefI_ = new PrimitiveDef_impl(orb_, poa_, this,
                                                    pk_objref);
	    name += count_++;
	    ObjectId_var oid = string_to_ObjectId(name);
	    poa_ -> activate_object_with_id(oid, primDefObjrefI_);
	}
	return primDefObjrefI_ -> _this();

    case pk_longlong:
	if(!primDefLongLongI_)
	{
	    primDefLongLongI_ = new PrimitiveDef_impl(orb_, poa_, this,
                                                      pk_longlong);
	    name += count_++;
	    ObjectId_var oid = string_to_ObjectId(name);
	    poa_ -> activate_object_with_id(oid, primDefLongLongI_);
	}
	return primDefLongLongI_ -> _this();
	
    case pk_ulonglong:
	if(!primDefULongLongI_)
	{
	    primDefULongLongI_ = new PrimitiveDef_impl(orb_, poa_, this,
						       pk_ulonglong);
	    name += count_++;
	    ObjectId_var oid = string_to_ObjectId(name);
	    poa_ -> activate_object_with_id(oid, primDefULongLongI_);
	}
	return primDefULongLongI_ -> _this();
	
    case pk_longdouble:
	if(!primDefLongDoubleI_)
	{
	    primDefLongDoubleI_ = new PrimitiveDef_impl(orb_, poa_, this,
							pk_longdouble);
	    name += count_++;
	    ObjectId_var oid = string_to_ObjectId(name);
	    poa_ -> activate_object_with_id(oid, primDefLongDoubleI_);
	}
	return primDefLongDoubleI_ -> _this();

    case pk_wchar:
	if(!primDefWCharI_)
	{
	    primDefWCharI_ = new PrimitiveDef_impl(orb_, poa_, this, pk_wchar);
	    name += count_++;
	    ObjectId_var oid = string_to_ObjectId(name);
	    poa_ -> activate_object_with_id(oid, primDefWCharI_);
	}
	return primDefWCharI_ -> _this();
	
    case pk_wstring:
	if(!primDefWStringI_)
	{
	    primDefWStringI_ = new PrimitiveDef_impl(orb_, poa_, this,
                                                     pk_wstring);
	    name += count_++;
	    ObjectId_var oid = string_to_ObjectId(name);
	    poa_ -> activate_object_with_id(oid, primDefWStringI_);
	}
	return primDefWStringI_ -> _this();

    case pk_value_base:
	if(!primDefValueBaseI_)
	{
	    primDefValueBaseI_ = new PrimitiveDef_impl(orb_, poa_, this,
						       pk_value_base);
	    name += count_++;
	    ObjectId_var oid = string_to_ObjectId(name);
	    poa_ -> activate_object_with_id(oid, primDefValueBaseI_);
	}
	return primDefValueBaseI_ -> _this();
    }

    assert(false);
    return CORBA::PrimitiveDef::_nil(); // Some compilers need this
}

StringDef_ptr
Repository_impl::create_string(ULong bound)
    throw(SystemException)
{
    StringDef_impl* p = new StringDef_impl(orb_, poa_, this);
    anonImplList_.push_back(p);
    CORBA::String_var name = CORBA::string_dup("StringDef_");
    name += count_++;
    ObjectId_var oid = string_to_ObjectId(name);
    poa_ -> activate_object_with_id(oid, p);
    StringDef_var ref = p -> _this();

    try
    {
        p -> bound(bound);
    }
    catch(...)
    {
        p -> destroy();
        throw;
    }

    return ref._retn();
}

WstringDef_ptr
Repository_impl::create_wstring(ULong bound)
    throw(SystemException)
{
    WstringDef_impl* p = new WstringDef_impl(orb_, poa_, this);
    anonImplList_.push_back(p);
    CORBA::String_var name = CORBA::string_dup("WstringDef_");
    name += count_++;
    ObjectId_var oid = string_to_ObjectId(name);
    poa_ -> activate_object_with_id(oid, p);
    WstringDef_var ref = p -> _this();

    try
    {
        p -> bound(bound);
    }
    catch(...)
    {
        p -> destroy();
        throw;
    }

    return ref._retn();
}

SequenceDef_ptr
Repository_impl::create_sequence(ULong bound, IDLType_ptr element_type)
    throw(SystemException)
{
    SequenceDef_impl* p = new SequenceDef_impl(orb_, poa_, this);
    anonImplList_.push_back(p);
    CORBA::String_var name = CORBA::string_dup("SequenceDef_");
    name += count_++;
    ObjectId_var oid = string_to_ObjectId(name);
    poa_ -> activate_object_with_id(oid, p);
    SequenceDef_var ref = p -> _this();

    try
    {
        p -> bound(bound);
        p -> element_type_def(element_type);
    }
    catch(...)
    {
        p -> destroy();
        throw;
    }

    return ref._retn();
}

ArrayDef_ptr
Repository_impl::create_array(ULong length, IDLType_ptr element_type)
    throw(SystemException)
{
    ArrayDef_impl* p = new ArrayDef_impl(orb_, poa_, this);
    anonImplList_.push_back(p);
    CORBA::String_var name = CORBA::string_dup("ArrayDef_");
    name += count_++;
    ObjectId_var oid = string_to_ObjectId(name);
    poa_ -> activate_object_with_id(oid, p);
    ArrayDef_var ref = p -> _this();

    try
    {
        p -> length(length);
        p -> element_type_def(element_type);
    }
    catch(...)
    {
        p -> destroy();
        throw;
    }

    return ref._retn();
}

FixedDef_ptr
Repository_impl::create_fixed(UShort digits, Short scale)
    throw(SystemException)
{
    FixedDef_impl* p = new FixedDef_impl(orb_, poa_, this);
    anonImplList_.push_back(p);
    CORBA::String_var name = CORBA::string_dup("FixedDef_");
    name += count_++;
    ObjectId_var oid = string_to_ObjectId(name);
    poa_ -> activate_object_with_id(oid, p);
    FixedDef_var ref = p -> _this();

    try
    {
        p -> digits(digits);
        p -> scale(scale);
    }
    catch(...)
    {
        p -> destroy();
        throw;
    }

    return ref._retn();
}

// ----------------------------------------------------------------------
// ModuleDef_impl member implementations
// ----------------------------------------------------------------------

ModuleDef_impl::ModuleDef_impl(ORB_ptr orb, POA_ptr poa, Container_impl* c,
			       Repository_impl* r)
    : IRObject_impl(orb, poa, r),
      Container_impl(orb, poa, r),
      Contained_impl(orb, poa, c, r)
{
}

ModuleDef_impl::~ModuleDef_impl()
{
}

void
ModuleDef_impl::destroy()
    throw(SystemException)
{
    destroyContents();
    Contained_impl::destroy();
}

Contained::Description*
ModuleDef_impl::describe()
    throw(SystemException)
{
    ModuleDescription_var desc = new ModuleDescription;

    desc -> name = name();
    desc -> id = id();

    Contained_impl* contained = dynamic_cast<Contained_impl*>(definedIn_);
    if(contained)
	desc -> defined_in = contained -> id();
    else
	desc -> defined_in = string_dup("");

    desc -> version = version();

    CORBA::Contained::Description_var description =
	new CORBA::Contained::Description;
    description -> kind = def_kind();
    description -> value <<= desc._retn();

    return description._retn();
}

// ----------------------------------------------------------------------
// ConstantDef_impl member implementations
// ----------------------------------------------------------------------

ConstantDef_impl::ConstantDef_impl(ORB_ptr orb, POA_ptr poa, Container_impl* c,
				   Repository_impl* r)
    : IRObject_impl(orb, poa, r),
      Contained_impl(orb, poa, c, r),
      typeI_(0)
{
}

ConstantDef_impl::~ConstantDef_impl()
{
}

void
ConstantDef_impl::destroy()
    throw(SystemException)
{
    if(typeI_)
    {
	typeI_ -> _remove_ref();
	typeI_ = 0;
    }
    
    Contained_impl::destroy();
}

Contained::Description*
ConstantDef_impl::describe()
    throw(SystemException)
{
    ConstantDescription_var desc = new ConstantDescription;

    desc -> name = name();
    desc -> id = id();

    Contained_impl* contained = dynamic_cast<Contained_impl*>(definedIn_);
    if(contained)
	desc -> defined_in = contained -> id();
    else
	desc -> defined_in = string_dup("");

    desc -> version = version();
    desc -> type = type();
    desc -> value = value_;

    CORBA::Contained::Description_var description =
	new CORBA::Contained::Description;
    description -> kind = def_kind();
    description -> value <<= desc._retn();

    return description._retn();
}

TypeCode_ptr
ConstantDef_impl::type()
    throw(SystemException)
{
    if(!typeI_)
        throw BAD_INV_ORDER();
    
    return typeI_ -> type();
}

IDLType_ptr
ConstantDef_impl::type_def()
    throw(SystemException)
{
    if(!typeI_)
        throw BAD_INV_ORDER();

    return typeI_ -> _this();
}

void
ConstantDef_impl::type_def(IDLType_ptr type)
    throw(SystemException)
{
    if(is_nil(type))
	throw BAD_PARAM(); // TODO: Is this exception correct?

    POA_var poa = _default_POA();
    IDLType_impl* impl = 0;
    try
    {
	ServantBase_var servant = poa -> reference_to_servant(type);
	impl = dynamic_cast<IDLType_impl*>(servant.in());
    }
    catch(const POA::WrongAdapter&)
    {
    }
    catch(const POA::WrongPolicy&)
    {
    }
    catch(const POA::ObjectNotActive&)
    {
    }
    if(!impl)
    {
        // Must be same repository
	throw BAD_PARAM(MinorInvalidContainer, COMPLETED_NO);
    }
    
    impl -> _add_ref();
    if(typeI_)
	typeI_ -> _remove_ref();
    typeI_ = impl;
}

Any*
ConstantDef_impl::value()
    throw(SystemException)
{
    return new Any(value_);
}

void
ConstantDef_impl::value(const Any& v)
    throw(SystemException)
{
    //
    // From 10.5.8.2 (99-10-07):
    //
    // "When setting the value attribute, the TypeCode of the supplied
    // any must be equal to the type attribute of the ConstantDef."
    //
    TypeCode_var tc1 = type();
    TypeCode_var tc2 = v.type();
    if(!tc1 -> equal(tc2))
        throw BAD_PARAM(MinorConstantTypeMismatch, COMPLETED_NO);

    value_ = v;
}

// ----------------------------------------------------------------------
// TypedefDef_impl member implementations
// ----------------------------------------------------------------------

TypedefDef_impl::TypedefDef_impl(ORB_ptr orb, POA_ptr poa, Container_impl* c,
				 Repository_impl* r)
    : IRObject_impl(orb, poa, r),
      Contained_impl(orb, poa, c, r),
      IDLType_impl(orb, poa, r)
{
}

TypedefDef_impl::~TypedefDef_impl()
{
}

Contained::Description*
TypedefDef_impl::describe()
    throw(SystemException)
{
    TypeDescription_var desc = new TypeDescription;

    desc -> name = name();
    desc -> id = id();

    Contained_impl* contained = dynamic_cast<Contained_impl*>(definedIn_);
    if(contained)
	desc -> defined_in = contained -> id();
    else
	desc -> defined_in = string_dup("");

    desc -> version = version();
    desc -> type = type();

    CORBA::Contained::Description_var description =
	new CORBA::Contained::Description;
    description -> kind = def_kind();
    description -> value <<= desc._retn();

    return description._retn();
}

// ----------------------------------------------------------------------
// StructDef_impl member implementations
// ----------------------------------------------------------------------

StructDef_impl::StructDef_impl(ORB_ptr orb, POA_ptr poa, Container_impl* c,
			       Repository_impl* r)
    : IRObject_impl(orb, poa, r),
      Container_impl(orb, poa, r),
      Contained_impl(orb, poa, c, r),
      IDLType_impl(orb, poa, r),
      TypedefDef_impl(orb, poa, c, r)
{
}

StructDef_impl::~StructDef_impl()
{
}

void
StructDef_impl::validateName(const char* name,
			     bool exclude_inherited,
			     bool in_inherited)
{
    Container_impl::validateName(name, exclude_inherited, in_inherited);
    
    for(ULong i = 0 ; i < membersIS_.length() ; i++)
	if(IdlStringCompare(members_[i].name, name) == 0)
	    throw BAD_PARAM(MinorNameExists, COMPLETED_NO);
}

void
StructDef_impl::destroy()
    throw(SystemException)
{
    for(ULong i = 0 ; i < membersIS_.length() ; i++)
	membersIS_[i] -> _remove_ref();

    membersIS_.length(0);
    members_.length(0);
    
    destroyContents();
    Contained_impl::destroy();
}

TypeCode_ptr
StructDef_impl::type()
    throw(SystemException)
{
    ContainedIS dummy;
    TypeCode_var tc = typeRecursive(dummy);
    assert(dummy.length() == 0);
    return tc._retn();
}

TypeCode_ptr
StructDef_impl::typeRecursive(ContainedIS& seq)
    throw(SystemException)
{
    //
    // A struct can be recursive even without a sequence, so check
    // for recursion first. For example:
    //
    // union U switch(boolean)
    // {
    // case TRUE: struct S
    //            {
    //                sequence<S> seq1;
    //                sequence<U> seq2;
    //            } the_S;
    // };
    //
    ULong i;
    for(i = 0 ; i < seq.length() ; i++)
    {
        RepositoryId_var id2 = seq[i] -> id();
        if(strcmp(id_, id2) == 0)
            return orb_ -> create_recursive_tc(id_);
    }

    seq.length(seq.length() + 1);
    seq[seq.length() - 1] = this;
    
    //
    // Recalculate TypeCodes every time, as ids may have changed, and
    // also in order to calculate recursive TypeCodes correctly.
    //
    for(i = 0 ; i < membersIS_.length() ; i++)
	members_[i].type = membersIS_[i] -> typeRecursive(seq);

    seq.length(seq.length() - 1);
    return orb_ -> create_struct_tc(id_, name_, members_);
}

StructMemberSeq*
StructDef_impl::members()
    throw(SystemException)
{
    //
    // Recalculate TypeCodes every time, as ids may have changed, and
    // also in order to calculate recursive TypeCodes correctly.
    //
    for(ULong i = 0 ; i < membersIS_.length() ; i++)
    {
	ContainedIS dummy;
	members_[i].type = membersIS_[i] -> typeRecursive(dummy);
	assert(dummy.length() == 0);
    }

    return new StructMemberSeq(members_);
}

void
StructDef_impl::members(const StructMemberSeq& seq)
    throw(SystemException)
{
    ULong i;

    //
    // Ensure we don't have duplicate declarator names
    //
    for(i = 0 ; i < seq.length() ; i++)
	for(ULong j = i + 1 ; j < seq.length() ; j++)
	    if(IdlStringCompare(seq[i].name, seq[j].name) == 0)
		throw BAD_PARAM(MinorDuplicateDeclarator, COMPLETED_NO);

    IDLTypeIS implSeq;
    implSeq.length(seq.length());
    POA_var poa = _default_POA();
    for(i = 0 ; i < seq.length() ; i++)
    {
        if(strlen(seq[i].name) == 0)
	    throw BAD_PARAM(); // TODO: Is this exception correct?

	//
	// Find out whether name already exists. Note:
	// Container_impl::validateName() must be used, so that the
	// current member names are not take into account.
	//
	Container_impl::validateName(seq[i].name, false, false);

	//
	// Check for name redefinition in an immediate scope
	//
	if(IdlStringCompare(seq[i].name, name_) == 0)
	    throw BAD_PARAM(MinorNameRedefinitionInImmediateScope,
			    COMPLETED_NO);
	
	if(is_nil(seq[i].type_def))
	    throw BAD_PARAM(); // TODO: Is this exception correct?

	implSeq[i] = 0;
	try
	{
	    ServantBase_var servant =
		poa -> reference_to_servant(seq[i].type_def);
	    implSeq[i] = dynamic_cast<IDLType_impl*>(servant.in());
	}
	catch(const POA::WrongAdapter&)
	{
	}
	catch(const POA::WrongPolicy&)
	{
	}
	catch(const POA::ObjectNotActive&)
	{
	}
	if(!implSeq[i])
	{
            // Must be same repository
	    throw BAD_PARAM(MinorInvalidContainer, COMPLETED_NO);
	}
    }

    //
    // Check for illegal recursion
    //
    for(i = 0 ; i < seq.length() ; i++)
        checkIllegalRecursion(id_, seq[i].type);

    for(i = 0 ; i < implSeq.length() ; i++)
	implSeq[i] -> _add_ref();
    
    for(i = 0 ; i < membersIS_.length() ; i++)
	membersIS_[i] -> _remove_ref();

    membersIS_ = implSeq;
    members_ = seq;
}

// ----------------------------------------------------------------------
// UnionDef_impl member implementations
// ----------------------------------------------------------------------

UnionDef_impl::UnionDef_impl(ORB_ptr orb, POA_ptr poa, Container_impl* c,
			     Repository_impl* r)
    : IRObject_impl(orb, poa, r),
      Container_impl(orb, poa, r),
      Contained_impl(orb, poa, c, r),
      IDLType_impl(orb, poa, r),
      TypedefDef_impl(orb, poa, c, r),
      discTypeI_(0)
{
}

UnionDef_impl::~UnionDef_impl()
{
}

void
UnionDef_impl::validateName(const char* name,
			    bool exclude_inherited,
			    bool in_inherited)
{
    Container_impl::validateName(name, exclude_inherited, in_inherited);
    
    for(ULong i = 0 ; i < membersIS_.length() ; i++)
	if(IdlStringCompare(members_[i].name, name) == 0)
	    throw BAD_PARAM(MinorNameExists, COMPLETED_NO);
}

void
UnionDef_impl::destroy()
    throw(SystemException)
{
    if(discTypeI_)
    {
	discTypeI_ -> _remove_ref();
	discTypeI_ = 0;
    }
    
    for(ULong i = 0 ; i < membersIS_.length() ; i++)
	membersIS_[i] -> _remove_ref();

    membersIS_.length(0);
    members_.length(0);
    
    destroyContents();
    Contained_impl::destroy();
}

TypeCode_ptr
UnionDef_impl::type()
    throw(SystemException)
{
    ContainedIS dummy;
    TypeCode_var tc = typeRecursive(dummy);
    assert(dummy.length() == 0);
    return tc._retn();
}

TypeCode_ptr
UnionDef_impl::typeRecursive(ContainedIS& seq)
    throw(SystemException)
{
    //
    // A union can be recursive even without a sequence, so check
    // for recursion first. For example:
    //
    // struct S
    // {
    //     union U switch(boolean)
    //     {
    //         case TRUE: sequence<S> seq;
    //     };
    // };
    //
    ULong i;
    for(i = 0 ; i < seq.length() ; i++)
    {
        RepositoryId_var id2 = seq[i] -> id();
        if(strcmp(id_, id2) == 0)
            return orb_ -> create_recursive_tc(id_);
    }

    seq.length(seq.length() + 1);
    seq[seq.length() - 1] = this;
    
    //
    // Recalculate TypeCodes every time, as ids may have changed, and
    // also in order to calculate recursive TypeCodes correctly.
    //
    for(i = 0 ; i < membersIS_.length() ; i++)
	members_[i].type = membersIS_[i] -> typeRecursive(seq);

    seq.length(seq.length() - 1);
    TypeCode_var discTC = discriminator_type();
    return orb_ -> create_union_tc(id_, name_, discTC, members_);
}

TypeCode_ptr
UnionDef_impl::discriminator_type()
    throw(SystemException)
{
    if(!discTypeI_)
        throw BAD_INV_ORDER();

    return discTypeI_ -> type();
}

IDLType_ptr
UnionDef_impl::discriminator_type_def()
    throw(SystemException)
{
    if(!discTypeI_)
        throw BAD_INV_ORDER();

    return discTypeI_ -> _this();
}

void
UnionDef_impl::discriminator_type_def(IDLType_ptr type)
    throw(SystemException)
{
    if(is_nil(type))
	throw BAD_PARAM(); // TODO: Is this exception correct?

    POA_var poa = _default_POA();
    IDLType_impl* impl = 0;
    try
    {
	ServantBase_var servant = poa -> reference_to_servant(type);
	impl = dynamic_cast<IDLType_impl*>(servant.in());
    }
    catch(const POA::WrongAdapter&)
    {
    }
    catch(const POA::WrongPolicy&)
    {
    }
    catch(const POA::ObjectNotActive&)
    {
    }
    if(!impl)
    {
	// Must be same repository
	throw BAD_PARAM(MinorInvalidContainer, COMPLETED_NO);
    }
    
    impl -> _add_ref();
    if(discTypeI_)
	discTypeI_ -> _remove_ref();
    discTypeI_ = impl;
}

UnionMemberSeq*
UnionDef_impl::members()
    throw(SystemException)
{
    //
    // Recalculate TypeCodes every time, as ids may have changed, and
    // also in order to calculate recursive TypeCodes correctly.
    //
    for(ULong i = 0 ; i < membersIS_.length() ; i++)
    {
	ContainedIS dummy;
	members_[i].type = membersIS_[i] -> typeRecursive(dummy);
	assert(dummy.length() == 0);
    }

    return new UnionMemberSeq(members_);
}

void
UnionDef_impl::members(const UnionMemberSeq& seq)
    throw(SystemException)
{
    ULong i;

    //
    // No test for duplicate declarator names, because there may be
    // several members which have the same declarator but different
    // labels. For example:
    //
    // union U switch(long)
    // {
    // case 0:
    // case 1: char c;
    // };
    //

    IDLTypeIS implSeq;
    implSeq.length(seq.length());
    POA_var poa = _default_POA();
    for(i = 0 ; i < seq.length() ; i++)
    {
        if(strlen(seq[i].name) == 0)
	    throw BAD_PARAM(); // TODO: Is this exception correct?

	//
	// Find out whether name already exists. Note:
	// Container_impl::validateName() must be used, so that the
	// current member names are not take into account.
	//
	Container_impl::validateName(seq[i].name, false, false);

	//
	// Check for name redefinition in an immediate scope
	//
	if(IdlStringCompare(seq[i].name, name_) == 0)
	    throw BAD_PARAM(MinorNameRedefinitionInImmediateScope,
			    COMPLETED_NO);

	if(is_nil(seq[i].type_def))
	    throw BAD_PARAM(); // TODO: Is this exception correct?

	implSeq[i] = 0;
	try
	{
	    ServantBase_var servant =
		poa -> reference_to_servant(seq[i].type_def);
	    implSeq[i] = dynamic_cast<IDLType_impl*>(servant.in());
	}
	catch(const POA::WrongAdapter&)
	{
	}
	catch(const POA::WrongPolicy&)
	{
	}
	catch(const POA::ObjectNotActive&)
	{
	}
	if(!implSeq[i])
	{
	    // Must be same repository
	    throw BAD_PARAM(MinorInvalidContainer, COMPLETED_NO);
	}
    }

    //
    // Check for illegal recursion
    //
    for(i = 0 ; i < seq.length() ; i++)
	checkIllegalRecursion(id_, seq[i].type);

    for(i = 0 ; i < implSeq.length() ; i++)
	implSeq[i] -> _add_ref();
    
    for(i = 0 ; i < membersIS_.length() ; i++)
	membersIS_[i] -> _remove_ref();

    membersIS_ = implSeq;
    members_ = seq;
}

// ----------------------------------------------------------------------
// EnumDef_impl member implementations
// ----------------------------------------------------------------------

EnumDef_impl::EnumDef_impl(ORB_ptr orb, POA_ptr poa, Container_impl* c,
			   Repository_impl* r)
    : IRObject_impl(orb, poa, r),
      Contained_impl(orb, poa, c, r),
      IDLType_impl(orb, poa, r),
      TypedefDef_impl(orb, poa, c, r)
{
}

EnumDef_impl::~EnumDef_impl()
{
}

TypeCode_ptr
EnumDef_impl::type()
    throw(SystemException)
{
    return orb_ -> create_enum_tc(id_, name_, members_);
}

EnumMemberSeq*
EnumDef_impl::members()
    throw(SystemException)
{
    return new EnumMemberSeq(members_);
}

void
EnumDef_impl::members(const EnumMemberSeq& seq)
    throw(SystemException)
{
    ULong i;

    //
    // Ensure we don't have duplicate declarator names
    //
    for(i = 0 ; i < seq.length() ; i++)
	for(ULong j = i + 1 ; j < seq.length() ; j++)
	    if(IdlStringCompare(seq[i], seq[j]) == 0)
		throw BAD_PARAM(MinorDuplicateDeclarator, COMPLETED_NO);

    for(i = 0 ; i < seq.length() ; i++)
    {
        if(strlen(seq[i]) == 0)
	    throw BAD_PARAM(); // TODO: Is this exception correct?

	//
	// Find out whether name already exists
	//
	definedIn_ -> Container_impl::validateName(seq[i], false, false);
    }

    members_ = seq;
}

// ----------------------------------------------------------------------
// AliasDef_impl member implementations
// ----------------------------------------------------------------------

AliasDef_impl::AliasDef_impl(ORB_ptr orb, POA_ptr poa, Container_impl* c,
			     Repository_impl* r)
    : IRObject_impl(orb, poa, r),
      Contained_impl(orb, poa, c, r),
      IDLType_impl(orb, poa, r),
      TypedefDef_impl(orb, poa, c, r),
      typeI_(0)
{
}

AliasDef_impl::~AliasDef_impl()
{
}

void
AliasDef_impl::destroy()
    throw(SystemException)
{
    if(typeI_)
    {
	typeI_ -> _remove_ref();
	typeI_ = 0;
    }
    
    Contained_impl::destroy();
}

TypeCode_ptr
AliasDef_impl::type()
    throw(SystemException)
{
    ContainedIS dummy;
    TypeCode_var tc = typeRecursive(dummy);
    assert(dummy.length() == 0);
    return tc._retn();
}

TypeCode_ptr
AliasDef_impl::typeRecursive(ContainedIS& seq)
    throw(SystemException)
{
    if(!typeI_)
        throw BAD_INV_ORDER();

    Contained_impl* contained = dynamic_cast<Contained_impl*>(typeI_);
    if(contained)
    {
	RepositoryId_var id = contained -> id();
	for(ULong i = 0 ; i < seq.length() ; i++)
	{
	    RepositoryId_var id2 = seq[i] -> id();
	    if(strcmp(id, id2) == 0)
	    {
		DefinitionKind elemDefKind = contained -> def_kind();
		if(elemDefKind != dk_Struct &&
		   elemDefKind != dk_Exception &&
		   elemDefKind != dk_Union &&
		   elemDefKind != dk_Value &&
                   elemDefKind != dk_Alias)
		{
		    throw INTF_REPOS(MinorIllegalRecursion, COMPLETED_NO);
		}
	    }
	}
    }

    TypeCode_var tc = typeI_ -> typeRecursive(seq);

    return orb_ -> create_alias_tc(id_, name_, tc);
}


IDLType_ptr
AliasDef_impl::original_type_def()
    throw(SystemException)
{
    if(!typeI_)
        throw BAD_INV_ORDER();

    return typeI_ -> _this();
}

void
AliasDef_impl::original_type_def(IDLType_ptr type)
    throw(SystemException)
{
    if(is_nil(type))
	throw BAD_PARAM(); // TODO: Is this exception correct?

    POA_var poa = _default_POA();
    IDLType_impl* impl = 0;
    try
    {
	ServantBase_var servant = poa -> reference_to_servant(type);
	impl = dynamic_cast<IDLType_impl*>(servant.in());
    }
    catch(const POA::WrongAdapter&)
    {
    }
    catch(const POA::WrongPolicy&)
    {
    }
    catch(const POA::ObjectNotActive&)
    {
    }
    if(!impl)
    {
	// Must be same repository
	throw BAD_PARAM(MinorInvalidContainer, COMPLETED_NO);
    }
    
    impl -> _add_ref();
    if(typeI_)
	typeI_ -> _remove_ref();
    typeI_ = impl;
}

// ----------------------------------------------------------------------
// NativeDef_impl member implementations
// ----------------------------------------------------------------------

NativeDef_impl::NativeDef_impl(ORB_ptr orb, POA_ptr poa, Container_impl* c,
			       Repository_impl* r)
    : IRObject_impl(orb, poa, r),
      Contained_impl(orb, poa, c, r),
      IDLType_impl(orb, poa, r),
      TypedefDef_impl(orb, poa, c, r)
{
}

NativeDef_impl::~NativeDef_impl()
{
}

TypeCode_ptr
NativeDef_impl::type()
    throw(SystemException)
{
    return orb_ -> create_native_tc(id_, name_);
}

// ----------------------------------------------------------------------
// PrimitiveDef_impl member implementations
// ----------------------------------------------------------------------

PrimitiveDef_impl::PrimitiveDef_impl(ORB_ptr orb, POA_ptr poa, 
                                     Repository_impl* r,
				     PrimitiveKind pk)
    : IRObject_impl(orb, poa, r),
      IDLType_impl(orb, poa, r),
      primitiveKind(pk)
{
}

PrimitiveDef_impl::~PrimitiveDef_impl()
{
}

void
PrimitiveDef_impl::destroy()
    throw(SystemException)
{
    //
    // Invoking destroy on a PrimitiveDef is an error
    //
    throw BAD_INV_ORDER(MinorIndestructibleObject, COMPLETED_NO);
}

void
PrimitiveDef_impl::destroyPrimitive()
    throw(SystemException)
{
    IDLType_impl::destroy();
}

TypeCode_ptr
PrimitiveDef_impl::type()
    throw(SystemException)
{
    switch(primitiveKind)
    {
    case pk_null:
	return TypeCode::_duplicate(_tc_null);

    case pk_void:
	return TypeCode::_duplicate(_tc_void);

    case pk_short:
	return TypeCode::_duplicate(_tc_short);

    case pk_long:
	return TypeCode::_duplicate(_tc_long);

    case pk_ushort:
	return TypeCode::_duplicate(_tc_ushort);

    case pk_ulong:
	return TypeCode::_duplicate(_tc_ulong);

    case pk_float:
	return TypeCode::_duplicate(_tc_float);

    case pk_double:
	return TypeCode::_duplicate(_tc_double);

    case pk_boolean:
	return TypeCode::_duplicate(_tc_boolean);

    case pk_char:
	return TypeCode::_duplicate(_tc_char);

    case pk_octet:
	return TypeCode::_duplicate(_tc_octet);

    case pk_any:
	return TypeCode::_duplicate(_tc_any);

    case pk_TypeCode:
	return TypeCode::_duplicate(_tc_TypeCode);

    case pk_Principal:
	return TypeCode::_duplicate(_tc_Principal);

    case pk_string:
	return TypeCode::_duplicate(_tc_string);

    case pk_objref:
	return TypeCode::_duplicate(_tc_Object);

    case pk_longlong:
	return TypeCode::_duplicate(_tc_longlong);

    case pk_ulonglong:
	return TypeCode::_duplicate(_tc_ulonglong);

    case pk_longdouble:
	return TypeCode::_duplicate(_tc_longdouble);

    case pk_wchar:
	return TypeCode::_duplicate(_tc_wchar);

    case pk_wstring:
	return TypeCode::_duplicate(_tc_wstring);

    case pk_value_base:
    {
        ValueMemberSeq members;
        return orb_ -> create_value_tc("IDL:omg.org/CORBA/ValueBase:1.0",
				       "ValueBase", VM_ABSTRACT,
				       TypeCode::_nil(), members);
    }
    }

    assert(false);
    return TypeCode::_nil(); // Some compilers need this
}

PrimitiveKind
PrimitiveDef_impl::kind()
    throw(SystemException)
{
    return primitiveKind;
}

// ----------------------------------------------------------------------
// StringDef_impl member implementations
// ----------------------------------------------------------------------

StringDef_impl::StringDef_impl(ORB_ptr orb, POA_ptr poa, Repository_impl* r)
    : IRObject_impl(orb, poa, r),
      IDLType_impl(orb, poa, r)
{
}

StringDef_impl::~StringDef_impl()
{
}

void
StringDef_impl::destroy()
    throw(SystemException)
{
    repository_ -> removeAnon(this);

    IDLType_impl::destroy();
}

TypeCode_ptr
StringDef_impl::type()
    throw(SystemException)
{
    return orb_ -> create_string_tc(bound_);
}

ULong
StringDef_impl::bound()
    throw(SystemException)
{
    return bound_;
}

void
StringDef_impl::bound(ULong b)
    throw(SystemException)
{
    bound_ = b;
}

// ----------------------------------------------------------------------
// WstringDef_impl member implementations
// ----------------------------------------------------------------------

WstringDef_impl::WstringDef_impl(ORB_ptr orb, POA_ptr poa, Repository_impl* r)
    : IRObject_impl(orb, poa, r),
      IDLType_impl(orb, poa, r)
{
}

WstringDef_impl::~WstringDef_impl()
{
}

void
WstringDef_impl::destroy()
    throw(SystemException)
{
    repository_ -> removeAnon(this);

    IDLType_impl::destroy();
}

TypeCode_ptr
WstringDef_impl::type()
    throw(SystemException)
{
    return orb_ -> create_wstring_tc(bound_);
}

ULong
WstringDef_impl::bound()
    throw(SystemException)
{
    return bound_;
}

void
WstringDef_impl::bound(ULong b)
    throw(SystemException)
{
    bound_ = b;
}

// ----------------------------------------------------------------------
// FixedDef_impl member implementations
// ----------------------------------------------------------------------

FixedDef_impl::FixedDef_impl(ORB_ptr orb, POA_ptr poa, Repository_impl* r)
    : IRObject_impl(orb, poa, r),
      IDLType_impl(orb, poa, r),
      digits_(0),
      scale_(0)
{
}

FixedDef_impl::~FixedDef_impl()
{
}

void
FixedDef_impl::destroy()
    throw(SystemException)
{
    repository_ -> removeAnon(this);

    IDLType_impl::destroy();
}

TypeCode_ptr
FixedDef_impl::type()
    throw(SystemException)
{
    return orb_ -> create_fixed_tc(digits_, scale_);
}

UShort
FixedDef_impl::digits()
    throw(SystemException)
{
    return digits_;
}

void
FixedDef_impl::digits(UShort digits)
    throw(SystemException)
{
    digits_ = digits;
}

Short
FixedDef_impl::scale()
    throw(SystemException)
{
    return scale_;
}

void
FixedDef_impl::scale(Short scale)
    throw(SystemException)
{
    scale_ = scale;
}

// ----------------------------------------------------------------------
// SequenceDef_impl member implementations
// ----------------------------------------------------------------------

SequenceDef_impl::SequenceDef_impl(ORB_ptr orb, POA_ptr poa,
                                   Repository_impl* r)
    : IRObject_impl(orb, poa, r),
      IDLType_impl(orb, poa, r),
      bound_(0),
      elementTypeI_(0)
{
}

SequenceDef_impl::~SequenceDef_impl()
{
}

void
SequenceDef_impl::destroy()
    throw(SystemException)
{
    if(elementTypeI_)
    {
	elementTypeI_ -> _remove_ref();
	elementTypeI_ = 0;
    }

    repository_ -> removeAnon(this);

    IDLType_impl::destroy();
}

TypeCode_ptr
SequenceDef_impl::type()
    throw(SystemException)
{
    ContainedIS dummy;
    TypeCode_var tc = typeRecursive(dummy);
    assert(dummy.length() == 0);
    return tc._retn();
}

TypeCode_ptr
SequenceDef_impl::typeRecursive(ContainedIS& seq)
    throw(SystemException)
{
    if(!elementTypeI_)
        throw BAD_INV_ORDER();
    
    TypeCode_var tc;

    Contained_impl* contained =	dynamic_cast<Contained_impl*>(elementTypeI_);
    if(contained)
    {
	RepositoryId_var id = contained -> id();
	for(ULong i = 0 ; i < seq.length() ; i++)
	{
	    RepositoryId_var id2 = seq[i] -> id();
	    if(strcmp(id, id2) == 0)
	    {
                //
                // An alias can be recursive if it's an alias for a
                // valuetype
                //
		DefinitionKind elemDefKind = contained -> def_kind();
		if(elemDefKind != dk_Struct &&
		   elemDefKind != dk_Exception &&
		   elemDefKind != dk_Union &&
		   elemDefKind != dk_Value &&
                   elemDefKind != dk_Alias)
		{
		    throw INTF_REPOS(MinorIllegalRecursion, COMPLETED_NO);
		}
		
		tc = orb_ -> create_recursive_tc(id);
		break;
	    }
	}
    }
    
    if(is_nil(tc))
	tc = elementTypeI_ -> typeRecursive(seq);
    
    return orb_ -> create_sequence_tc(bound_, tc);
}

ULong
SequenceDef_impl::bound()
    throw(SystemException)
{
    return bound_;
}

void
SequenceDef_impl::bound(ULong b)
    throw(SystemException)
{
    bound_ = b;
}

TypeCode_ptr
SequenceDef_impl::element_type()
    throw(SystemException)
{
    if(!elementTypeI_)
        throw BAD_INV_ORDER();

    return elementTypeI_ -> type();
}

IDLType_ptr
SequenceDef_impl::element_type_def()
    throw(SystemException)
{
    if(!elementTypeI_)
        throw BAD_INV_ORDER();

    return elementTypeI_ -> _this();
}

void
SequenceDef_impl::element_type_def(IDLType_ptr type)
    throw(SystemException)
{
    if(is_nil(type))
	throw BAD_PARAM(); // TODO: Is this exception correct?

    POA_var poa = _default_POA();
    IDLType_impl* impl = 0;
    try
    {
	ServantBase_var servant = poa -> reference_to_servant(type);
	impl = dynamic_cast<IDLType_impl*>(servant.in());
    }
    catch(const POA::WrongAdapter&)
    {
    }
    catch(const POA::WrongPolicy&)
    {
    }
    catch(const POA::ObjectNotActive&)
    {
    }
    if(!impl)
    {
	// Must be same repository
	throw BAD_PARAM(MinorInvalidContainer, COMPLETED_NO);
    }
    
    impl -> _add_ref();
    if(elementTypeI_)
	elementTypeI_ -> _remove_ref();
    elementTypeI_ = impl;
}

// ----------------------------------------------------------------------
// ArrayDef_impl member implementations
// ----------------------------------------------------------------------

ArrayDef_impl::ArrayDef_impl(ORB_ptr orb, POA_ptr poa, Repository_impl* r)
    : IRObject_impl(orb, poa, r),
      IDLType_impl(orb, poa, r),
      length_(0),
      elementTypeI_(0)
{
}

ArrayDef_impl::~ArrayDef_impl()
{
}

void
ArrayDef_impl::destroy()
    throw(SystemException)
{
    if(elementTypeI_)
    {
	elementTypeI_ -> _remove_ref();
	elementTypeI_ = 0;
    }

    repository_ -> removeAnon(this);

    IDLType_impl::destroy();
}

TypeCode_ptr
ArrayDef_impl::type()
    throw(SystemException)
{
    ContainedIS dummy;
    TypeCode_var tc = typeRecursive(dummy);
    assert(dummy.length() == 0);
    return tc._retn();
}

TypeCode_ptr
ArrayDef_impl::typeRecursive(ContainedIS& seq)
    throw(SystemException)
{
    if(!elementTypeI_)
        throw BAD_INV_ORDER();

    TypeCode_var tc;

    Contained_impl* contained =
	dynamic_cast<Contained_impl*>(elementTypeI_);
    if(contained)
    {
	RepositoryId_var id = contained -> id();
	for(ULong i = 0 ; i < seq.length() ; i++)
	{
	    RepositoryId_var id2 = seq[i] -> id();
            if(strcmp(id, id2) == 0)
            {
                //
                // An array can only be recursive if its element type is
                // a valuetype. Skip any aliases to determine the original
                // element type.
                //
                DefinitionKind elemDefKind = contained -> def_kind();
                if(elemDefKind == dk_Alias)
                {
                    AliasDef_impl* aliasImpl =
                        dynamic_cast<AliasDef_impl*>(contained);
                    IDLType_var type = aliasImpl -> original_type_def();
                    elemDefKind = type -> def_kind();
                    while(elemDefKind == dk_Alias)
                    {
                        AliasDef_var alias = AliasDef::_narrow(type);
                        type = alias -> original_type_def();
                        elemDefKind = type -> def_kind();
                    }
                }
		if(elemDefKind != dk_Value)
		{
		    throw INTF_REPOS(MinorIllegalRecursion, COMPLETED_NO);
		}
		
		tc = orb_ -> create_recursive_tc(id);
		break;
	    }
	}
    }
    
    if(is_nil(tc))
	tc = elementTypeI_ -> typeRecursive(seq);
    
    return orb_ -> create_array_tc(length_, tc);
}

ULong
ArrayDef_impl::length()
    throw(SystemException)
{
    return length_;
}

void
ArrayDef_impl::length(ULong b)
    throw(SystemException)
{
    length_ = b;
}

TypeCode_ptr
ArrayDef_impl::element_type()
    throw(SystemException)
{
    if(!elementTypeI_)
        throw BAD_INV_ORDER();

    return elementTypeI_ -> type();
}

IDLType_ptr
ArrayDef_impl::element_type_def()
    throw(SystemException)
{
    if(!elementTypeI_)
        throw BAD_INV_ORDER();

    return elementTypeI_ -> _this();
}

void
ArrayDef_impl::element_type_def(IDLType_ptr type)
    throw(SystemException)
{
    if(is_nil(type))
	throw BAD_PARAM(); // TODO: Is this exception correct?

    POA_var poa = _default_POA();
    IDLType_impl* impl = 0;
    try
    {
	ServantBase_var servant = poa -> reference_to_servant(type);
	impl = dynamic_cast<IDLType_impl*>(servant.in());
    }
    catch(const POA::WrongAdapter&)
    {
    }
    catch(const POA::WrongPolicy&)
    {
    }
    catch(const POA::ObjectNotActive&)
    {
    }
    if(!impl)
    {
	// Must be same repository
	throw BAD_PARAM(MinorInvalidContainer, COMPLETED_NO);
    }
    
    impl -> _add_ref();
    if(elementTypeI_)
	elementTypeI_ -> _remove_ref();
    elementTypeI_ = impl;
}
