// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#ifndef IFR3_IMPL_H
#define IFR3_IMPL_H

#include <IFR2_impl.h>
#include <OB/IFR3_skel.h>

namespace CORBA
{

//
// Forward declaration
//
class PrimitiveDef_impl;

//
// The Repository_impl class
//
class Repository_impl : public virtual POA_CORBA::Repository,
			public virtual Container_impl
{
    PrimitiveDef_impl* primDefNullI_;
    PrimitiveDef_impl* primDefVoidI_;
    PrimitiveDef_impl* primDefShortI_;
    PrimitiveDef_impl* primDefLongI_;
    PrimitiveDef_impl* primDefUShortI_;
    PrimitiveDef_impl* primDefULongI_;
    PrimitiveDef_impl* primDefFloatI_;
    PrimitiveDef_impl* primDefDoubleI_;
    PrimitiveDef_impl* primDefBooleanI_;
    PrimitiveDef_impl* primDefCharI_;
    PrimitiveDef_impl* primDefOctetI_;
    PrimitiveDef_impl* primDefAnyI_;
    PrimitiveDef_impl* primDefTypeCodeI_;
    PrimitiveDef_impl* primDefPrincipalI_;
    PrimitiveDef_impl* primDefStringI_;
    PrimitiveDef_impl* primDefObjrefI_;
    PrimitiveDef_impl* primDefLongLongI_;
    PrimitiveDef_impl* primDefULongLongI_;
    PrimitiveDef_impl* primDefLongDoubleI_;
    PrimitiveDef_impl* primDefWCharI_;
    PrimitiveDef_impl* primDefWStringI_;
    PrimitiveDef_impl* primDefValueBaseI_;

    CORBA::ULong count_;

    OB_STL::list<IDLType_impl*> anonImplList_;

    typedef OB::Hashtable< CORBA::RepositoryId_var, CORBA::Contained_impl*,
	OB::StringHasher> RepIdMap;
    RepIdMap* repIdMap_;

    void add(Contained_impl*);
    void remove(Contained_impl*);
    void removeAnon(IDLType_impl*);
    friend class Contained_impl; // Needs add()/remove()
    friend class StringDef_impl; // Needs removeAnon()
    friend class WstringDef_impl; // Needs removeAnon()
    friend class FixedDef_impl; // Needs removeAnon()
    friend class SequenceDef_impl; // Needs removeAnon()
    friend class ArrayDef_impl; // Needs removeAnon()

#ifndef NDEBUG
    long objectCount_;
    void incIRObjectCount() { objectCount_++; }
    void decIRObjectCount() { objectCount_--; }
    friend class IRObject_impl;
#endif

    Contained_ptr lookupIdInSeq(const ContainedSeq&, const char*);

public:

    Repository_impl(ORB_ptr, PortableServer::POA_ptr);
    virtual ~Repository_impl();

    virtual DefinitionKind def_kind()
        throw(SystemException)
    {
        return dk_Repository;
    }

    virtual void destroy()
        throw(SystemException);
    void destroyRepository()
        throw(SystemException);

    virtual Contained_ptr lookup(const char* search_name)
        throw(SystemException);

    virtual Contained_ptr lookup_id(const char* search_id)
        throw(SystemException);

    TypeCode_ptr get_canonical_typecode(TypeCode_ptr tc)
        throw(SystemException);

    PrimitiveDef_ptr get_primitive(PrimitiveKind kind)
        throw(SystemException);

    StringDef_ptr
    create_string(ULong bound)
        throw(SystemException);
    
    WstringDef_ptr
    create_wstring(ULong bound)
        throw(SystemException);

    SequenceDef_ptr
    create_sequence(ULong bound,
		    IDLType_ptr element_type)
        throw(SystemException);
    
    ArrayDef_ptr
    create_array(ULong length,
		 IDLType_ptr element_type)
        throw(SystemException);
    
    FixedDef_ptr
    create_fixed(UShort digits,
		 Short scale)
        throw(SystemException);
};

//
// The ModuleDef_impl class
//
class ModuleDef_impl : public virtual POA_CORBA::ModuleDef,
		       public virtual Container_impl,
		       public virtual Contained_impl
{
public:
    
    ModuleDef_impl(ORB_ptr, PortableServer::POA_ptr,
                   Container_impl*, Repository_impl*);
    virtual ~ModuleDef_impl();
    
    virtual DefinitionKind def_kind()
        throw(SystemException)
    {
        return dk_Module;
    }

    virtual void destroy()
        throw(SystemException);

    virtual CORBA::Contained::Description* describe()
        throw(SystemException);
};

//
// The ConstantDef_impl class
//
class ConstantDef_impl : public virtual POA_CORBA::ConstantDef,
			 public virtual Contained_impl
{
    IDLType_impl* typeI_;
    Any value_;

public:

    ConstantDef_impl(ORB_ptr, PortableServer::POA_ptr,
                     Container_impl*, Repository_impl*);
    virtual ~ConstantDef_impl();
    
    virtual DefinitionKind def_kind()
        throw(SystemException)
    {
        return dk_Constant;
    }
    
    virtual void destroy()
        throw(SystemException);

    virtual CORBA::Contained::Description* describe()
        throw(SystemException);
    
    virtual TypeCode_ptr type()
        throw(SystemException);
    
    virtual IDLType_ptr type_def()
        throw(SystemException);
    virtual void type_def(IDLType_ptr)
        throw(SystemException);

    virtual Any* value()
        throw(SystemException);
    virtual void value(const Any&)
        throw(SystemException);
};

//
// The TypedefDef_impl class
//
class TypedefDef_impl : public virtual POA_CORBA::TypedefDef,
			public virtual Contained_impl,
			public virtual IDLType_impl
{
public:

    TypedefDef_impl(ORB_ptr, PortableServer::POA_ptr,
                    Container_impl*, Repository_impl*);
    virtual ~TypedefDef_impl();

    virtual bool isAnonymous() { return false; }

    virtual DefinitionKind def_kind()
        throw(SystemException)
    {
        return dk_Typedef;
    }

    virtual CORBA::Contained::Description* describe()
        throw(SystemException);
};

//
// The StructDef_impl class
//
class StructDef_impl : public virtual POA_CORBA::StructDef,
		       public virtual TypedefDef_impl,
		       public virtual Container_impl
{
    StructMemberSeq members_;
    IDLTypeIS membersIS_;
    
public:
    
    StructDef_impl(ORB_ptr, PortableServer::POA_ptr,
                   Container_impl*, Repository_impl*);
    virtual ~StructDef_impl();
    
    virtual void validateName(const char*, bool, bool);
	
    virtual DefinitionKind def_kind()
        throw(SystemException)
    {
        return dk_Struct;
    }
    
    virtual void destroy()
        throw(SystemException);

    virtual TypeCode_ptr type()
        throw(SystemException);
    virtual TypeCode_ptr typeRecursive(ContainedIS&)
        throw(SystemException);

    virtual StructMemberSeq* members()
        throw(SystemException);
    virtual void members(const StructMemberSeq&)
        throw(SystemException);
};

//
// The UnionDef_impl class
//
class UnionDef_impl : public virtual POA_CORBA::UnionDef,
		      public virtual TypedefDef_impl,
		      public virtual Container_impl
{
    IDLType_impl* discTypeI_;
    UnionMemberSeq members_;
    IDLTypeIS membersIS_;
    
public:
    
    UnionDef_impl(ORB_ptr, PortableServer::POA_ptr,
                  Container_impl*, Repository_impl*);
    virtual ~UnionDef_impl();
    
    virtual void validateName(const char*, bool, bool);
	
    virtual DefinitionKind def_kind()
        throw(SystemException)
    {
        return dk_Union;
    }

    virtual void destroy()
        throw(SystemException);

    virtual TypeCode_ptr type()
        throw(SystemException);
    virtual TypeCode_ptr typeRecursive(ContainedIS&)
        throw(SystemException);

    virtual TypeCode_ptr discriminator_type()
        throw(SystemException);

    virtual IDLType_ptr discriminator_type_def()
        throw(SystemException);
    virtual void discriminator_type_def(IDLType_ptr)
        throw(SystemException);

    virtual UnionMemberSeq* members()
        throw(SystemException);
    virtual void members(const UnionMemberSeq&)
        throw(SystemException);
};

//
// The EnumDef_impl class
//
class EnumDef_impl : public virtual POA_CORBA::EnumDef,
		     public virtual TypedefDef_impl
{
    EnumMemberSeq members_;

public:

    EnumDef_impl(ORB_ptr, PortableServer::POA_ptr,
                 Container_impl*, Repository_impl*);
    virtual ~EnumDef_impl();

    virtual DefinitionKind def_kind()
        throw(SystemException)
    {
        return dk_Enum;
    }

    virtual TypeCode_ptr type()
        throw(SystemException);
    virtual TypeCode_ptr typeRecursive(ContainedIS&)
        throw(SystemException)
    {
	return type();
    }

    virtual EnumMemberSeq* members()
        throw(SystemException);
    virtual void members(const EnumMemberSeq&)
        throw(SystemException);
};

//
// The AliasDef_impl class
//
class AliasDef_impl : public virtual POA_CORBA::AliasDef,
		      public virtual TypedefDef_impl
{
    IDLType_impl* typeI_;

public:

    AliasDef_impl(ORB_ptr, PortableServer::POA_ptr,
                  Container_impl*, Repository_impl*);
    virtual ~AliasDef_impl();

    virtual DefinitionKind def_kind()
        throw(SystemException)
    {
        return dk_Alias;
    }

    virtual void destroy()
        throw(SystemException);

    virtual TypeCode_ptr type()
        throw(SystemException);
    virtual TypeCode_ptr typeRecursive(ContainedIS&)
        throw(SystemException);

    virtual IDLType_ptr original_type_def()
        throw(SystemException);
    virtual void original_type_def(IDLType_ptr)
        throw(SystemException);
};

//
// The NativeDef_impl class
//
class NativeDef_impl : public virtual POA_CORBA::NativeDef,
		       public virtual TypedefDef_impl
{
public:

    NativeDef_impl(ORB_ptr, PortableServer::POA_ptr,
                   Container_impl*, Repository_impl*);
    virtual ~NativeDef_impl();

    virtual DefinitionKind def_kind()
        throw(SystemException)
    {
        return dk_Native;
    }

    virtual TypeCode_ptr type()
        throw(SystemException);
    virtual TypeCode_ptr typeRecursive(ContainedIS&)
        throw(SystemException)
    {
	return type();
    }
};

//
// The PrimitiveDef_impl class
//
class PrimitiveDef_impl : public virtual POA_CORBA::PrimitiveDef,
			  public virtual IDLType_impl
{
    PrimitiveKind primitiveKind;

public:

    PrimitiveDef_impl(ORB_ptr, PortableServer::POA_ptr,
                      Repository_impl*, PrimitiveKind);
    virtual ~PrimitiveDef_impl();

    virtual bool isAnonymous() { return false; }

    virtual DefinitionKind def_kind()
	throw(SystemException)
    {
	return dk_Primitive;
    }

    virtual void destroy()
        throw(SystemException);
    void destroyPrimitive()
        throw(SystemException);

    virtual TypeCode_ptr type()
        throw(SystemException);
    virtual TypeCode_ptr typeRecursive(ContainedIS&)
        throw(SystemException)
    {
	return type();
    }

    virtual PrimitiveKind kind()
        throw(SystemException);
};

//
// The StringDef_impl class
//
class StringDef_impl : public virtual POA_CORBA::StringDef,
		       public virtual IDLType_impl
{
    ULong bound_;

public:

    StringDef_impl(ORB_ptr, PortableServer::POA_ptr,
                   Repository_impl*);
    virtual ~StringDef_impl();

    virtual bool isAnonymous() { return true; }

    virtual DefinitionKind def_kind()
        throw(SystemException)
    {
        return dk_String;
    }

    virtual void destroy()
        throw(SystemException);

    virtual TypeCode_ptr type()
        throw(SystemException);
    virtual TypeCode_ptr typeRecursive(ContainedIS&)
        throw(SystemException)
    {
	return type();
    }

    virtual ULong bound()
        throw(SystemException);
    virtual void bound(ULong)
        throw(SystemException);
};

//
// The WstringDef_impl class
//
class WstringDef_impl : public virtual POA_CORBA::WstringDef,
			public virtual IDLType_impl
{
    ULong bound_;

public:

    WstringDef_impl(ORB_ptr, PortableServer::POA_ptr,
                    Repository_impl*);
    virtual ~WstringDef_impl();

    virtual bool isAnonymous() { return true; }

    virtual DefinitionKind def_kind()
        throw(SystemException)
    {
        return dk_Wstring;
    }

    virtual void destroy()
        throw(SystemException);

    virtual TypeCode_ptr type()
        throw(SystemException);
    virtual TypeCode_ptr typeRecursive(ContainedIS&)
        throw(SystemException)
    {
	return type();
    }

    virtual ULong bound()
        throw(SystemException);
    virtual void bound(ULong)
        throw(SystemException);
};

//
// The FixedDef_impl class
//
class FixedDef_impl : public virtual POA_CORBA::FixedDef,
		      public virtual IDLType_impl
{
    UShort digits_;
    Short scale_;
    
public:

    FixedDef_impl(ORB_ptr, PortableServer::POA_ptr,
                  Repository_impl*);
    virtual ~FixedDef_impl();

    virtual bool isAnonymous() { return true; }

    virtual DefinitionKind def_kind()
	throw(SystemException)
    {
	return dk_Fixed;
    }

    virtual void destroy()
        throw(SystemException);

    virtual TypeCode_ptr type()
	throw(SystemException);
    virtual TypeCode_ptr typeRecursive(ContainedIS&)
        throw(SystemException)
    {
	return type();
    }

    virtual UShort digits()
	throw(SystemException);
    virtual void digits(UShort)
	throw(SystemException);

    virtual Short scale()
	throw(SystemException);
    virtual void scale(Short)
	throw(SystemException);
};

//
// The SequenceDef_impl class
//
class SequenceDef_impl : public virtual POA_CORBA::SequenceDef,
			 public virtual IDLType_impl
{
    ULong bound_;
    IDLType_impl* elementTypeI_;

public:

    SequenceDef_impl(ORB_ptr, PortableServer::POA_ptr,
                     Repository_impl*);
    virtual ~SequenceDef_impl();

    virtual bool isAnonymous() { return true; }

    virtual DefinitionKind def_kind()
	throw(SystemException)
    {
	return dk_Sequence;
    }

    virtual void destroy()
        throw(SystemException);

    virtual TypeCode_ptr type()
	throw(SystemException);
    virtual TypeCode_ptr typeRecursive(ContainedIS&)
        throw(SystemException);

    virtual ULong bound()
	throw(SystemException);
    virtual void bound(ULong)
	throw(SystemException);

    virtual TypeCode_ptr element_type()
	throw(SystemException);

    virtual IDLType_ptr element_type_def()
	throw(SystemException);
    virtual void element_type_def(IDLType_ptr)
	throw(SystemException);
};

//
// The ArrayDef_impl class
//
class ArrayDef_impl : public virtual POA_CORBA::ArrayDef,
		      public virtual IDLType_impl
{
    ULong length_;
    IDLType_impl* elementTypeI_;

public:

    ArrayDef_impl(ORB_ptr, PortableServer::POA_ptr,
                  Repository_impl*);
    virtual ~ArrayDef_impl();

    virtual bool isAnonymous() { return true; }

    virtual DefinitionKind def_kind()
	throw(SystemException)
    {
    	return dk_Array;
    }

    virtual void destroy()
        throw(SystemException);

    virtual TypeCode_ptr type()
	throw(SystemException);
    virtual TypeCode_ptr typeRecursive(ContainedIS&)
        throw(SystemException);

    virtual ULong length()
	throw(SystemException);
    virtual void length(ULong)
	throw(SystemException);

    virtual TypeCode_ptr element_type()
	throw(SystemException);

    virtual IDLType_ptr element_type_def()
	throw(SystemException);
    virtual void element_type_def(IDLType_ptr)
	throw(SystemException);
};

} // End of namespace CORBA

#endif
