// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#ifndef IFR4_IMPL_H
#define IFR4_IMPL_H

#include <IFR3_impl.h>
#include <OB/IFR4_skel.h>

namespace CORBA
{

//
// The AbstractInterfaceDef_impl class
//
class AbstractInterfaceDef_impl :
    public virtual POA_CORBA::AbstractInterfaceDef,
    public virtual InterfaceDef_impl
{
public:

    AbstractInterfaceDef_impl(ORB_ptr, PortableServer::POA_ptr,
                              Container_impl*, Repository_impl*);
    virtual ~AbstractInterfaceDef_impl();

    virtual DefinitionKind def_kind()
	throw(SystemException)
    {
	return dk_AbstractInterface;
    }

    virtual void destroy()
        throw(SystemException);

    virtual TypeCode_ptr type()
        throw(SystemException);

    virtual InterfaceDefSeq* base_interfaces()
        throw(SystemException);
    virtual void base_interfaces(const InterfaceDefSeq&)
        throw(SystemException);

    virtual Boolean is_a(const char* interface_id)
        throw(SystemException);
};

//
// The LocalInterfaceDef_impl class
//
class LocalInterfaceDef_impl :
    public virtual POA_CORBA::LocalInterfaceDef,
    public virtual InterfaceDef_impl
{
public:

    LocalInterfaceDef_impl(ORB_ptr, PortableServer::POA_ptr,
                              Container_impl*, Repository_impl*);
    virtual ~LocalInterfaceDef_impl();

    virtual DefinitionKind def_kind()
	throw(SystemException)
    {
	return dk_LocalInterface;
    }

    virtual void destroy()
        throw(SystemException);

    virtual TypeCode_ptr type()
        throw(SystemException);

    virtual Boolean is_a(const char* interface_id)
        throw(SystemException);
};

//
// The ValueMemberDef_impl class
//
class ValueMemberDef_impl : public virtual POA_CORBA::ValueMemberDef,
                            public virtual Contained_impl
{
    IDLType_impl* typeI_;
    Visibility access_;

public:

    ValueMemberDef_impl(ORB_ptr, PortableServer::POA_ptr,
                        Container_impl*, Repository_impl*);
    virtual ~ValueMemberDef_impl();

    virtual DefinitionKind def_kind()
	throw(SystemException)
    {
	return dk_ValueMember;
    }

    virtual void destroy()
        throw(SystemException);

    virtual char* name()
        throw(SystemException)
    {
        return Contained_impl::name();
    }

    virtual void name(const char*)
        throw(SystemException);

    virtual CORBA::Contained::Description* describe()
	throw(SystemException);

    virtual TypeCode_ptr type()
	throw(SystemException);
    virtual TypeCode_ptr typeRecursive(ContainedIS&)
        throw(SystemException);

    virtual IDLType_ptr type_def()
	throw(SystemException);
    virtual void type_def(IDLType_ptr)
	throw(SystemException);

    virtual Visibility access()
	throw(SystemException);
    virtual void access(Visibility)
	throw(SystemException);
};

//
// The ValueDef_impl class
//
class ValueDef_impl : public virtual POA_CORBA::ValueDef,
		      public virtual Container_impl,
		      public virtual Contained_impl,
		      public virtual IDLType_impl
{
    InterfaceDefIS supportedInterfacesIS_;
    InitializerSeq initializers_;
    IDLTypeISS initializersMembersISS_;
    ValueDef_impl* baseValueI_;
    ValueDefIS abstractBaseValuesIS_;
    Boolean isAbstract_;
    Boolean isCustom_;
    Boolean isTruncatable_;

    void validateNameImpl(const char*, bool, bool, bool);
    void validateInheritance(Container_impl*, ContainedSeq&);

public:

    ValueDef_impl(ORB_ptr, PortableServer::POA_ptr,
                  Container_impl*, Repository_impl*, Boolean,
		  Boolean, Boolean);
    virtual ~ValueDef_impl();

    virtual bool isAnonymous() { return false; }

    virtual DefinitionKind def_kind()
	throw(SystemException)
    {
	return dk_Value;
    }

    virtual void destroy()
        throw(SystemException);

    virtual CORBA::Contained::Description* describe()
	throw(SystemException);

    virtual TypeCode_ptr type()
	throw(SystemException);
    virtual TypeCode_ptr typeRecursive(ContainedIS&)
        throw(SystemException);

    virtual Contained_ptr lookup(const char* search_name)
	throw(SystemException);

    virtual ContainedSeq* contents(DefinitionKind limit_type,
                                   Boolean exclude_inherited)
	throw(SystemException);

    virtual void validateName(const char*, bool, bool);

    virtual ContainedSeq* lookup_name(const char* search_name,
                                      Long levels_to_search,
                                      DefinitionKind limit_type,
                                      Boolean exclude_inherited)
	throw(SystemException);

    virtual InterfaceDefSeq* supported_interfaces()
	throw(SystemException);
    virtual void supported_interfaces(const InterfaceDefSeq&)
	throw(SystemException);

    virtual InitializerSeq* initializers()
	throw(SystemException);
    virtual void initializers(const InitializerSeq&)
	throw(SystemException);

    virtual ValueDef_ptr base_value()
	throw(SystemException);
    virtual void base_value(ValueDef_ptr)
	throw(SystemException);

    virtual ValueDefSeq* abstract_base_values()
	throw(SystemException);
    virtual void abstract_base_values(const ValueDefSeq&)
	throw(SystemException);

    virtual Boolean is_abstract()
	throw(SystemException);
    virtual void is_abstract(Boolean)
	throw(SystemException);

    virtual Boolean is_custom()
	throw(SystemException);
    virtual void is_custom(Boolean)
	throw(SystemException);

    virtual Boolean is_truncatable()
	throw(SystemException);
    virtual void is_truncatable(Boolean)
	throw(SystemException);

    virtual Boolean is_a(const char* value_id)
	throw(SystemException);

    virtual CORBA::ValueDef::FullValueDescription* describe_value()
	throw(SystemException);

    virtual ValueMemberDef_ptr create_value_member(const char* id,
						   const char* name,
						   const char* version,
						   IDLType_ptr type,
						   Visibility access)
	throw(SystemException);

    virtual AttributeDef_ptr create_attribute(const char* id,
					      const char* name,
					      const char* version,
					      IDLType_ptr type,
					      AttributeMode mode)
	throw(SystemException);

    virtual OperationDef_ptr create_operation(const char* id,
					      const char* name,
					      const char* version,
					      IDLType_ptr result,
					      OperationMode mode,
					      const ParDescriptionSeq& params,
					      const ExceptionDefSeq&
					      exceptions,
					      const ContextIdSeq& contexts)
	throw(SystemException);
};

//
// The ValueBoxDef_impl class
//
class ValueBoxDef_impl : public virtual POA_CORBA::ValueBoxDef,
                         public virtual TypedefDef_impl
{
    IDLType_impl* typeI_;

public:

    ValueBoxDef_impl(ORB_ptr, PortableServer::POA_ptr,
                     Container_impl*, Repository_impl*);
    virtual ~ValueBoxDef_impl();

    virtual DefinitionKind def_kind()
	throw(SystemException)
    {
	return dk_ValueBox;
    }

    virtual void destroy()
        throw(SystemException);

    virtual TypeCode_ptr type()
	throw(SystemException);
    virtual TypeCode_ptr typeRecursive(ContainedIS&)
        throw(SystemException);

    virtual IDLType_ptr original_type_def()
	throw(SystemException);
    virtual void original_type_def(IDLType_ptr)
	throw(SystemException);
};

} // End of namespace CORBA

#endif
