// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>

#include <TestCodeSets_impl.h>

#include <errno.h>

#ifdef HAVE_UNISTD_H
#   include <unistd.h>
#endif

#ifdef HAVE_IO_H
#   include <io.h>
#endif

#ifdef HAVE_STDLIB_H
#   include <stdlib.h>
#endif

#ifdef HAVE_FSTREAM
#   include <fstream>
#else
#   include <fstream.h>
#endif

using namespace CORBA;

int
run(ORB_ptr orb, int, char** argv)
{
    //
    // Resolve Root POA
    //
    CORBA::Object_var poaObj = orb -> resolve_initial_references("RootPOA");
    PortableServer::POA_var poa = PortableServer::POA::_narrow(poaObj);
	    
    //
    // Activate the POA manager
    //
    PortableServer::POAManager_var manager = poa -> the_POAManager();
    manager -> activate();
    
    //
    // Create implementation object
    //
    TestCodeSets_impl i(orb);
    TestCodeSets_var testCodeSets = i._this();

    //
    // Save references. This must be done after POA manager activation,
    // otherwise there is a potential for a race condition between the
    // client sending request and the server not being ready yet.
    //
    String_var s = orb -> object_to_string(testCodeSets);

    const char* refFile = "TestCodeSets.ref";
    OB_OFSTREAM out(refFile);
    if(out.fail())
    {
	OB_ERROR(argv[0] << ": can't open `" << refFile << "': "
		 << strerror(errno));
	return 1;
    }

    out << s << OB_ENDL;
    out.close();

    //
    // Give up control to the ORB
    //
    orb -> run();

#ifdef HAVE__UNLINK
    _unlink(refFile);
#else
    unlink(refFile);
#endif

    return 0;
}

int
main(int argc, char* argv[])
{
    int status = EXIT_SUCCESS;
    ORB_var orb;

    try
    {
        orb = ORB_init(argc, argv);
	status = run(orb, argc, argv);
    }
    catch(const Exception& ex)
    {
	OB_ERROR(ex);
        status = EXIT_SUCCESS;
    }

    if(!is_nil(orb))
    {
	try
	{
	    orb -> destroy();
	}
	catch(const Exception& ex)
	{
	    OB_ERROR(ex);
	    status = EXIT_FAILURE;
	}
    }

    return status;
}
