// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>
#include <OB/OCI_IIOP.h>
#include <OB/BootManager.h>
#include <URLTest_impl.h>

#ifdef HAVE_UNISTD_H
#   include <unistd.h>
#endif

#ifdef HAVE_IO_H
#   include <io.h>
#endif

#ifdef HAVE_STDLIB_H
#   include <stdlib.h>
#endif

#ifdef HAVE_FSTREAM
#   include <fstream>
#else
#   include <fstream.h>
#endif

using namespace CORBA;

// 
// Simple server providing objects for corba URL tests
// 
int
run(ORB_ptr orb, int argc, char* argv[])
{
    if(argc != 3)
    {
	OB_COUT << "usage: " << argv[0] << " key_string ior_file" << OB_ENDL;
	return 1;
    }
    
    // 
    // corbaloc key (ORBacus iiop URL compatible object will use
    // this key with a null byte appended)
    // 
    const char* keyStr = argv[1];
    const char* iorFile = argv[2];
    
    //
    // Resolve Root POA
    //
    CORBA::Object_var poaObj = orb -> resolve_initial_references("RootPOA");
    PortableServer::POA_var poa = PortableServer::POA::_narrow(poaObj);
	    
    //
    // Activate the POA manager
    //
    PortableServer::POAManager_var manager = poa -> the_POAManager();
    manager -> activate();

    //
    // Create POA
    //
    CORBA::PolicyList policies;
    policies.length(2);
    policies[0] = poa -> create_id_assignment_policy(PortableServer::USER_ID);
    policies[1] = poa -> create_lifespan_policy(PortableServer::PERSISTENT);
    PortableServer::POA_var testPOA =
	poa -> create_POA("testPOA", manager, policies);
    
    //
    // Resolve the Boot Manager
    //
    CORBA::Object_var bmgrObj =
	orb -> resolve_initial_references("BootManager");
    OB::BootManager_var bootManager = OB::BootManager::_narrow(bmgrObj);
    
    //
    // Find the POA Manager's Acceptor Port
    //
    OBPortableServer::POAManager_var obManager =
	OBPortableServer::POAManager::_narrow(manager);
    OCI::AcceptorSeq_var acceptors = obManager -> get_acceptors();
    
    CORBA::StringSeq_var hosts;
    CORBA::UShort port = 0;
    
    for(CORBA::ULong i = 0 ; i < acceptors -> length() ; i++)
    {
	OCI::AcceptorInfo_var info = acceptors[i] -> get_info();
	OCI::IIOP::AcceptorInfo_var iiopInfo =
	    OCI::IIOP::AcceptorInfo::_narrow(info);
	
	if(!CORBA::is_nil(iiopInfo))
	{
	    hosts = iiopInfo -> hosts();
	    port = iiopInfo -> port();
	}
    }
    
    if(port == 0)
    {
	OB_ERROR("Error: cannot determine port objects will listen on");
	return 1;
    }
    
    // 
    // corbaloc test object
    // 
    URLTest::IIOPAddress_var corbaURLObj;
    
    URLTest::IIOPAddress_impl *urlServant = 
	new URLTest::IIOPAddress_impl(orb, hosts[0], port, keyStr, "corbaloc");
    PortableServer::ObjectId_var oid =
	PortableServer::string_to_ObjectId(CORBA::String_var(
	    urlServant -> getKey()));
    
    testPOA -> activate_object_with_id(oid, urlServant);
    corbaURLObj = urlServant -> _this();
    bootManager -> add_binding(oid, corbaURLObj);
    
    //
    // Save references. This must be done after POA manager
    // activation, otherwise there is a potential for a race condition
    // between the client sending request and the server not being
    // ready yet.
    //
    URLTest::TempIORFile iorRef(orb, corbaURLObj, iorFile, URLTest::UnixEOL);
    
    //
    // Give up control to the ORB
    //
    orb -> run();
    
    return 0;
}

int
main(int argc, char* argv[])
{
    int status = EXIT_SUCCESS;
    ORB_var orb;

    try
    {
        orb = ORB_init(argc, argv);
	status = run(orb, argc, argv);
    }
    catch(const Exception& ex)
    {
	OB_ERROR(ex);
        status = EXIT_SUCCESS;
    }

    if(!is_nil(orb))
    {
	try
	{
	    orb -> destroy();
	}
	catch(const Exception& ex)
	{
	    OB_ERROR(ex);
	    status = EXIT_FAILURE;
	}
    }

    return status;
}
