// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>

#include <Test_impl.h>

#include <stdlib.h>
#include <errno.h>

#ifdef HAVE_FSTREAM
#   include <fstream>
#else
#   include <fstream.h>
#endif

#ifdef HAVE_UNISTD_H
#   include <unistd.h>
#endif

#ifdef HAVE_IO_H
#   include <io.h>
#endif

int
run(CORBA::ORB_ptr orb, int /* argc */, char* argv[])
{
    //
    // Resolve Root POA
    //
    CORBA::Object_var poaObj = orb -> resolve_initial_references("RootPOA");
    PortableServer::POA_var rootPOA = PortableServer::POA::_narrow(poaObj);
    
    //
    // Get a reference to the POA manager and activate it
    //
    PortableServer::POAManager_var manager = rootPOA -> the_POAManager();
    manager -> activate();
    
    //
    // Create implementation object
    //
    Test_impl* testImpl = new Test_impl(orb, rootPOA);
    PortableServer::ServantBase_var servant = testImpl;
    Test_var test = testImpl -> _this();

    //
    // Save reference. This must be done after POA manager
    // activation, otherwise there is a potential for a race
    // condition between the client sending a request and the
    // server not being ready yet.
    //
    CORBA::String_var s = orb -> object_to_string(test);
    
    const char* refFile = "Test.ref";
    OB_OFSTREAM out(refFile);
    if(out.fail())
    {
	OB_ERROR(argv[0] << ": can't open `" << refFile << "': "
		 << strerror(errno));
	return EXIT_FAILURE;
    }

    out << s << OB_ENDL;
    out.close();
    
    //
    // Run implementation
    //
    orb -> run();

#ifdef HAVE__UNLINK
    _unlink(refFile);
#else
    unlink(refFile);
#endif

    return EXIT_SUCCESS;
}

int
main(int argc, char* argv[], char*[])
{
    int status = EXIT_SUCCESS;
    CORBA::ORB_var orb;

    try
    {
	orb = CORBA::ORB_init(argc, argv);
	status = run(orb, argc, argv);
    }
    catch(const CORBA::Exception& ex)
    {
	OB_ERROR(ex);
	status = EXIT_FAILURE;
    }

    if(!CORBA::is_nil(orb))
    {
	try
	{
	    orb -> destroy();
	}
	catch(const CORBA::Exception& ex)
	{
	    OB_ERROR(ex);
	    status = EXIT_FAILURE;
	}
    }

    return status;
}
