#!/bin/sh
# **********************************************************************
#
# Copyright (c) 2002
# IONA Technologies, Inc.
# Waltham, MA, USA
#
# All Rights Reserved
#
# **********************************************************************

#
# Function to check for Java distribution and set the Java CLASSPATH.
#
check_for_JAVA()
{
    have_java=
    if test -n "$OB_JAVA_DIST" -a \
            -r "$OB_JAVA_DIST/property/lib/com/ooc/CosPropertyService/Server.class"
    then
	have_java="yes"
    fi

    #
    # Set Java CLASSPATH
    #
    if test -n "$have_java"
    then
        CLASSPATH="$OB_JAVA_DIST/property/lib"
        CLASSPATH="$OB_JAVA_DIST/ob/lib${SEPARATOR}$CLASSPATH"
        export CLASSPATH

        #
        # Set Java BOOTCLASSPATH
        #
        java_version=`$JAVA -version 2>&1`
        if `$echo $java_version |  grep '1\.[234]' > /dev/null`
        then
            BOOTCLASSPATH="-Xbootclasspath/p:$CLASSPATH"
	    export BOOTCLASSPATH
        fi
    fi
}

#
# Messages for the non-availability of Java and JThreads/C++,
# and set the clients and servers for auto testing.
#
set_auto_values_and_msgs()
{
    auto_servers="1"

    java_not_available=
    if test -z "$have_java"
    then
	java_not_available=" (not available)"
    else
	auto_servers="$auto_servers 2"
    fi
}

#
# Function to set the server executable command
#
set_server()
{
    server_num=$1

    if test "$server_num" = "1"
    then
        server="$top_srcdir/bin/propserv${exe}"
    fi

    if test "$server_num" = "2"  -a -n "$have_java"
    then
        server="$JAVA $BOOTCLASSPATH com.ooc.CosPropertyService.Server"
    fi
}

#
# Ask for server
#
prompt_for_server()
{
    while test -z "$server_num"
    do
	$echo
	$echo "Select test server to use:"
	$echo
	$echo "1: C++ server"
	$echo "2: Java server $java_not_available"
	$echo
	$echo "---> \c"

	read server_num
    done
}

#
# Execute the property service test executables.
#
execute_test()
{
    #
    # Display info
    #
    $echo
    if test -n "$have_java"
    then
	$echo Java CLASSPATH is \"CLASSPATH=$CLASSPATH\"
    fi

    #
    # Set name of reference file
    #
    ref="$top_srcdir/test/Property.ref"

    client="$top_srcdir/test/client${exe}"
    $echo Client command is \"$client\"
    $echo Server command is \"$server\"

    #
    # Function to deactivate the server
    #
    srvid=0
    deactivate()
    {
	if test $srvid -ne 0
	then
	    kill $srvid
	fi
    }
    trap deactivate 1 2 3 4 5 6 7 8 10 12 13 14 15

    #
    # Start property server
    #
    $echo
    $echo "Waiting for property server to start up... \c"
    rm -f $ref $db
    if test "$auto" = "true"
    then
	sleep 1
    fi
    $server --ior > $ref 2> server.log &
    srvid=$!
    count=0
    while test ! -s $ref -a $count -lt 8
    do
	sleep 1
	count=`expr $count + 1`
    done
    if test ! -s $ref
    then
	$echo "Failed!"
	$echo "(Server was not started)"
	exit
    else
	$echo "Done!"
	$echo
    fi

    #
    # Start client
    #
    $client -ORBInitRef PropertyService=`cat $ref` 2> client.log

    #
    # Deactivate server
    #
    deactivate
}
