// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>
#include <OB/CosProperty.h>

#include <assert.h>
#include <stdlib.h>
#include <string.h>

void TestUnconstrained(CosPropertyService::PropertySetDefFactory_ptr);
void TestConstrained(CosPropertyService::PropertySetDefFactory_ptr);
void TestInitial(CosPropertyService::PropertySetDefFactory_ptr);

int
run(CORBA::ORB_ptr orb, int /* argc */, char* argv[])
{
    //
    // Get property service
    //
    CosPropertyService::PropertySetDefFactory_var factory;
    try
    {
	factory = CosPropertyService::PropertySetDefFactory::_narrow(
	    CORBA::Object_var(
		orb -> resolve_initial_references("PropertyService")));
    }
    catch(const CORBA::ORB::InvalidName&)
    {
	OB_ERROR(argv[0] << ": can't resolve `PropertyService'");
	return EXIT_FAILURE;
    }
    
    if(CORBA::is_nil(factory.in()))
    {
	OB_ERROR(argv[0]
		 << ": `PropertyService' is not a PropertySetDefFactory "
		 << "object reference");
	return EXIT_FAILURE;
    }

    OB_INFO_LN("Testing unconstrained PropertySet");
    TestUnconstrained(factory);

    OB_INFO_LN("Testing constrained PropertySet");
    TestConstrained(factory);

    OB_INFO_LN("Testing initial PropertySet");
    TestInitial(factory);

    return EXIT_SUCCESS;
}

int
main(int argc, char* argv[], char*[])
{
    int status = EXIT_SUCCESS;
    CORBA::ORB_var orb;

    try
    {
	orb = CORBA::ORB_init(argc, argv);
	status = run(orb, argc, argv);
    }
    catch(const CORBA::Exception& ex)
    {
	OB_ERROR(ex);
	status = EXIT_FAILURE;
    }

    if(!CORBA::is_nil(orb))
    {
	try
	{
	    orb -> destroy();
	}
	catch(const CORBA::Exception& ex)
	{
	    OB_ERROR(ex);
	    status = EXIT_FAILURE;
	}
    }
    
    return status;
}
