// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>
#include <OB/TimeService.h>
#include <OB/TimeHelper.h>

#include <stdlib.h>

#ifdef HAVE_IOSTREAM
#   include <iostream>
#else
#   include <iostream.h>
#endif

#ifdef WIN32
#   include <sys/timeb.h>
#endif

#include <time.h>

#ifdef HAVE_STD_IOSTREAM
using namespace std;
#endif

static int run(CORBA::ORB_ptr);
static int stopwatch(CosTime::TimeService_ptr);

int
main(int argc, char* argv[], char*[])
{
    int status = EXIT_SUCCESS;
    CORBA::ORB_var orb;

    try
    {
        orb = CORBA::ORB_init(argc, argv);
	OB::TimeServiceInit(orb, argc, argv);
        status = run(orb);
    }
    catch(const CORBA::Exception& ex)
    {
        cerr << ex << endl;
        status = EXIT_FAILURE;
    }

    if(!CORBA::is_nil(orb))
    {
        try
        {
            orb -> destroy();
        }
        catch(const CORBA::Exception& ex)
        {
            cerr << ex << endl;
            status = EXIT_FAILURE;
        }
    }

    return status;
}

int
run(CORBA::ORB_ptr orb)
{
    //
    // Get time service
    //
    CORBA::Object_var obj;

    try
    {
        obj = orb -> resolve_initial_references("TimeService");
    }
    catch(const CORBA::ORB::InvalidName&)
    {
        cerr << "Can't resolve `TimeService'" << endl;
        return EXIT_FAILURE;
    }

    if(CORBA::is_nil(obj))
    {
        cerr << "`TimeService' is a nil object reference" << endl;
        return EXIT_FAILURE;
    }

    CosTime::TimeService_var ts = CosTime::TimeService::_narrow(obj);

    if(CORBA::is_nil(ts))
    {
	cerr << "`TimeService' is not a TimeService object reference" << endl;
	return EXIT_FAILURE;
    }

    return stopwatch(ts);
}

static int
stopwatch(CosTime::TimeService_ptr ts)
{
    CosTime::UTO_var start;
    CosTime::UTO_var stop;

    //
    // Get start and stop times
    //
    try
    {
    	cout << "Press Enter to start " << flush;
	cin.get();
	start = ts -> universal_time();
    	cout << "Press Enter to stop " << flush;
	cin.get();
	stop = ts -> universal_time();
    }
    catch(const CosTime::TimeUnavailable&)
    {
	cerr << "Time not available" << endl;
	return EXIT_FAILURE;
    }

    //
    // Display start and stop times
    //
    CORBA::String_var str = OB::TimeHelper::toTimeString(start -> utc_time());
    cout << "Start time:   " << str << endl;

    str = OB::TimeHelper::toTimeString(stop -> utc_time());
    cout << "Stop time:    " << str << endl;

    //
    // Display elapsed time
    //
    TimeBase::TimeT elapsed = stop -> time() - start -> time();
    cout << "Elapsed time: " << (unsigned long)(elapsed / 10000)
    	 << " ms" << endl;

    return EXIT_SUCCESS;
}

