// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>
#include <OB/Logger.h>
#include <OB/Properties.h>

#include <OB/TimeService.h>
#include <OB/TimeHelper.h>

#include <CosTime_impl.h>

#include <stdlib.h>

using namespace CORBA;

namespace OB
{

void
TimeServiceInit(ORB_ptr orb, int& ac, char** av)
{
    //
    // Get the set of properties
    //
    OB::Properties_var properties = OB::Properties::getDefaultProperties();
    OBCORBA::ORB_var oborb = OBCORBA::ORB::_narrow(orb);
    OB::Logger_var logger = oborb -> logger();

    int i = 0;
    while(i < ac)
    {
	if(strcmp(av[i], "-TIMEinaccuracy") == 0)
	{
            if(i + 1 >= ac)
            {
                logger -> error("OB::TimeServiceInit: argument expected for "
				"-TIMEinaccuracy");
                throw INITIALIZE();
            }

            properties -> setProperty("ooc.time.inaccuracy", av[i + 1]);
            
            for(int j = i ; j + 2 < ac ; j++)
                av[j] = av[j + 2];
            
            ac -= 2;
	}
        else if(strncmp(av[i], "-TIME", 4) == 0)
        {
            String_var err =
            string_dup("OB::TimeServiceInit: unknown option: ");
            err += av[i];
            logger -> warning(err);
	    
            for(int j = i ; j + 1 < ac ; j++)
                av[j] = av[j + 1];
	    
            ac -= 1;
	}
	else
	    ++i;
    }

    //
    // Process each property
    //
    ULong inaccuracy = 0;
    OB::Properties::KeySeq keys = properties -> getKeys("ooc.time.");
    ULong j;
    for(j = 0; j < keys.length(); ++j)
    {
        const char* key = keys[j];
        const char* value = properties -> getProperty(key);
        assert(value != 0);

	if(strcmp(key, "ooc.time.inaccuracy") == 0)
	{
	    inaccuracy = atol(value);

	    if(inaccuracy > OB::TimeHelper::MaxInaccuracyT)
	    {
		String_var err =
		    string_dup("OB::TimeServiceInit: inaccuracy greater "
		    	       "than maximum");
		logger -> warning(err);
	    }
	}
	else
	{
	    String_var err =
	    string_dup("OB::TimeServiceInit: unknown property ");
	    err += key;
	    logger -> warning(err);
	}
    }

    TimeService_impl* service = new TimeService_impl(inaccuracy);
    orb -> register_initial_reference("TimeService", service);
}

};
