// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>

#include <OB/TimeService.h>

#include <TestMacro.h>

using namespace CORBA;
using namespace TimeBase;

extern void TestUTO(CosTime::TimeService_ptr);
extern void TestTIO(CosTime::TimeService_ptr);
extern void TestTimeHelper(CosTime::TimeService_ptr);

void
TestMacro(const char* ex, const char* file, unsigned int line)
{
    OB_COUT << " Failed!" << OB_ENDL;
    OB_COUT << "Reason: ( " << ex << " ) == false" << OB_ENDL;
    OB_COUT << "File: " << file << OB_ENDL;
    OB_COUT << "Line: " << line << OB_ENDL << OB_FLUSH;
    abort();
}

int
main(int argc, char** argv)
{
    int status = 0;

    CORBA::ORB_var orb;
    try
    {
        //
        // Create ORB
        //
        orb = CORBA::ORB_init(argc, argv);

	//
	// Initialize the time service
	//
	OB::TimeServiceInit(orb, argc, argv);

	//
	// Resolve the TimeService
	//
	CosTime::TimeService_var ts = CosTime::TimeService::_narrow(
	    CORBA::Object_var(
		orb -> resolve_initial_references("TimeService")));

	OB_COUT << "Testing UTO... " << OB_FLUSH;
	TestUTO(ts);
	OB_COUT << " Done!" << OB_ENDL;

	OB_COUT << "Testing TIO... " << OB_FLUSH;
	TestTIO(ts);
	OB_COUT << " Done!" << OB_ENDL;

	OB_COUT << "Testing TimeHelper... " << OB_FLUSH;
	TestTimeHelper(ts);
	OB_COUT << " Done!" << OB_ENDL;
    }
    catch(const CORBA::SystemException& ex)
    {
	OB_CERR << ex << OB_ENDL << OB_FLUSH;
        status = EXIT_FAILURE;
    }

    if(!CORBA::is_nil(orb))
    {
        try
        {
            orb -> destroy();
        }
        catch(const CORBA::Exception& ex)
        {
            OB_CERR << ex << OB_ENDL << OB_FLUSH;
            status = EXIT_FAILURE;
        }
    }

    return status;
}
