// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>

#include <OB/CosTime.h>
#include <OB/TimeHelper.h>

#include <TestMacro.h>

#include <time.h>

using namespace CORBA;
using namespace TimeBase;

void
TestTIO(CosTime::TimeService_ptr ts)
{
    CosTime::TIO_var tio;
    CosTime::UTO_var uto;
    TimeBase::IntervalT interval;

    //
    // TEST: CosTime::TIO creation
    //
    try
    {
	tio = ts -> new_interval(200, 100);
	TEST(false);
    }
    catch(const BAD_PARAM&)
    {
    }
    try
    {
	tio = ts -> new_interval(OB_ULONGLONG(0x8000000000000064),
				 OB_ULONGLONG(0x8000000000000000));
	TEST(false);
    }
    catch(const BAD_PARAM&)
    {
    }
    try
    {
	tio = ts -> new_interval(OB_ULONGLONG(0x8000000000000000), 200);
	TEST(false);
    }
    catch(const BAD_PARAM&)
    {
    }
    try
    {
	tio = ts -> new_interval(OB::TimeHelper::MaxTimeT, 0);
	TEST(false);
    }
    catch(const BAD_PARAM&)
    {
    }
    tio = ts -> new_interval(100, 200);
    tio = ts -> new_interval(OB_ULONGLONG(0x8000000000000000),
    			     OB_ULONGLONG(0x8000000000000064));
    tio = ts -> new_interval(200, OB_ULONGLONG(0x8000000000000064));
    tio = ts -> new_interval(0, OB::TimeHelper::MaxTimeT);

    //
    // TEST: attribute TimeBase::IntervalT time_interval()
    //

    tio = ts -> new_interval(100, 200);
    interval = tio -> time_interval();
    TEST(interval.lower_bound == 100 && interval.upper_bound == 200);

    tio = ts -> new_interval(OB_ULONGLONG(0x8000000000000000),
    			     OB_ULONGLONG(0x8000000000000064));
    interval = tio -> time_interval();
    TEST(interval.lower_bound == OB_ULONGLONG(0x8000000000000000) &&
	 interval.upper_bound == OB_ULONGLONG(0x8000000000000064));

    tio = ts -> new_interval(0, OB::TimeHelper::MaxTimeT);
    interval = tio -> time_interval();
    TEST(interval.lower_bound == 0 &&
	 interval.upper_bound == OB::TimeHelper::MaxTimeT);

    //
    // TEST: UTO time()
    //

    tio = ts -> new_interval(100, 200);
    uto = tio -> time();
    TEST(uto -> time() == 150 && uto -> inaccuracy() == 50);

    tio = ts -> new_interval(OB_ULONGLONG(0x8000000000000000),
    			     OB_ULONGLONG(0x8000000000000064));
    uto = tio -> time();
    TEST(uto -> time() == OB_ULONGLONG(0x8000000000000032) &&
	 uto -> inaccuracy() == 50);

    tio = ts -> new_interval(0, OB::TimeHelper::MaxTimeT);
    try
    {
	uto = tio -> time();
	TEST(false);
    }
    catch(const DATA_CONVERSION&)
    {
    }

    //
    // TEST: OverlapType spans(in UTO time, out TIO overlap)
    //

    CosTime::TIO_var overlap;
    uto = ts -> new_universal_time(OB_ULONGLONG(0x8000000000000000), 50,
    				   uto -> tdf());

    //
    // Interval the same as UTO
    //
    tio = ts -> new_interval(OB_ULONGLONG(0x7fffffffffffffce),
    			     OB_ULONGLONG(0x8000000000000032));
    TEST(tio -> spans(uto, overlap.out()) == CosTime::OTContained);
    interval = overlap -> time_interval();
    TEST(interval.lower_bound == OB_ULONGLONG(0x7fffffffffffffce) &&
	 interval.upper_bound == OB_ULONGLONG(0x8000000000000032));

    //
    // Interval greater than UTO with no overlap
    //
    tio = ts -> new_interval(OB_ULONGLONG(0x8000000000000064),
    			     OB_ULONGLONG(0x8000000000000096));
    TEST(tio -> spans(uto, overlap.out()) == CosTime::OTNoOverlap);

    //
    // Interval less than UTO with no overlap
    //
    tio = ts -> new_interval(800, 900);
    TEST(tio -> spans(uto, overlap.out()) == CosTime::OTNoOverlap);

    //
    // Interval overlaps at upper end of UTO
    //
    tio = ts -> new_interval(OB_ULONGLONG(0x8000000000000000),
    			     OB_ULONGLONG(0x8000000000000096));
    TEST(tio -> spans(uto, overlap.out()) == CosTime::OTOverlap);
    interval = overlap -> time_interval();
    TEST(interval.lower_bound == OB_ULONGLONG(0x8000000000000000) &&
	 interval.upper_bound == OB_ULONGLONG(0x8000000000000032));

    //
    // Interval overlaps at lower end of UTO
    //
    tio = ts -> new_interval(800, OB_ULONGLONG(0x8000000000000000));
    TEST(tio -> spans(uto, overlap.out()) == CosTime::OTOverlap);
    interval = overlap -> time_interval();
    TEST(interval.lower_bound == OB_ULONGLONG(0x7fffffffffffffce) &&
	 interval.upper_bound == OB_ULONGLONG(0x8000000000000000));

    //
    // Interval completely contains UTO
    //
    tio = ts -> new_interval(OB_ULONGLONG(0x7fffffffffffffb5),
    			     OB_ULONGLONG(0x800000000000004b));
    TEST(tio -> spans(uto, overlap.out()) == CosTime::OTContainer);
    interval = overlap -> time_interval();
    TEST(interval.lower_bound == OB_ULONGLONG(0x7fffffffffffffce) &&
	 interval.upper_bound == OB_ULONGLONG(0x8000000000000032));

    //
    // Interval completely contained by UTO
    //
    tio = ts -> new_interval(OB_ULONGLONG(0x7fffffffffffffe7),
    			     OB_ULONGLONG(0x8000000000000019));
    TEST(tio -> spans(uto, overlap.out()) == CosTime::OTContained);
    interval = overlap -> time_interval();
    TEST(interval.lower_bound == OB_ULONGLONG(0x7fffffffffffffe7) &&
	 interval.upper_bound == OB_ULONGLONG(0x8000000000000019));

    //
    // TEST: OverlapType overlaps(in TIO interval, out TIO overlap);
    //
    CosTime::TIO_var tio2 = ts -> new_interval(OB_ULONGLONG(0x7fffffffffffffce),
					      OB_ULONGLONG(0x8000000000000032));

    //
    // Intervals the same
    //
    TEST(tio2 -> overlaps(tio2, overlap.out()) == CosTime::OTContained);
    interval = overlap -> time_interval();
    TEST(interval.lower_bound == OB_ULONGLONG(0x7fffffffffffffce) &&
	 interval.upper_bound == OB_ULONGLONG(0x8000000000000032));

    //
    // Interval greater than compared Interval with no overlap
    //
    tio = ts -> new_interval(OB_ULONGLONG(0x8000000000000064),
    			     OB_ULONGLONG(0x8000000000000096));
    TEST(tio -> overlaps(tio2, overlap.out()) == CosTime::OTNoOverlap);

    //
    // Interval less than compared Interval with no overlap
    //
    tio = ts -> new_interval(800, 900);
    TEST(tio -> overlaps(tio2, overlap.out()) == CosTime::OTNoOverlap);

    //
    // Interval overlaps at upper end of compared Interval
    //
    tio = ts -> new_interval(OB_ULONGLONG(0x8000000000000000),
    			     OB_ULONGLONG(0x8000000000000096));
    TEST(tio -> overlaps(tio2, overlap.out()) == CosTime::OTOverlap);
    interval = overlap -> time_interval();
    TEST(interval.lower_bound == OB_ULONGLONG(0x8000000000000000) &&
	 interval.upper_bound == OB_ULONGLONG(0x8000000000000032));

    //
    // Interval overlaps at lower end of compared Interval
    //
    tio = ts -> new_interval(925, OB_ULONGLONG(0x8000000000000000));
    TEST(tio -> overlaps(tio2, overlap.out()) == CosTime::OTOverlap);
    interval = overlap -> time_interval();
    TEST(interval.lower_bound == OB_ULONGLONG(0x7fffffffffffffce) &&
	 interval.upper_bound == OB_ULONGLONG(0x8000000000000000));

    //
    // Interval completely contains compared Interval
    //
    tio = ts -> new_interval(OB_ULONGLONG(0x7fffffffffffffb5),
    			     OB_ULONGLONG(0x800000000000004b));
    TEST(tio -> overlaps(tio2, overlap.out()) == CosTime::OTContainer);
    interval = overlap -> time_interval();
    TEST(interval.lower_bound == OB_ULONGLONG(0x7fffffffffffffce) &&
	 interval.upper_bound == OB_ULONGLONG(0x8000000000000032));

    //
    // Interval completely contained by compared Interval
    //
    tio = ts -> new_interval(OB_ULONGLONG(0x7fffffffffffffe7),
    			     OB_ULONGLONG(0x8000000000000019));
    TEST(tio -> overlaps(tio2, overlap.out()) == CosTime::OTContained);
    interval = overlap -> time_interval();
    TEST(interval.lower_bound == OB_ULONGLONG(0x7fffffffffffffe7) &&
	 interval.upper_bound == OB_ULONGLONG(0x8000000000000019));
}
