// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>

#include <OB/CosTime.h>
#include <OB/TimeHelper.h>

#include <TestMacro.h>

#include <time.h>

using namespace CORBA;
using namespace TimeBase;
using namespace OB;

void
TestTimeHelper(CosTime::TimeService_ptr ts)
{
    CosTime::UTO_var uto = ts -> universal_time();

    //
    // TEST: toString(const TimeBase::UtcT&);
    //       toTimeString(const TimeBase::UtcT&);
    //       toString(TimeBase::TimeT);
    //
 
    //
    // testTime = 2 January 1970 00:00:00 (local time)
    //
    ULongLong testTime = OB_ULONGLONG(141428) * OB_ULONGLONG(86400); 
    testTime -= uto -> tdf() * 60;
    testTime *= OB_ULONGLONG(10000000);

    uto = ts -> new_universal_time(testTime, 0, uto -> tdf());

    String_var str = OB::TimeHelper::toString(uto -> utc_time());
    TEST(strcmp("01/02/70 00:00:00:000", str) == 0);

    str = OB::TimeHelper::toTimeString(uto -> utc_time());
    TEST(strcmp("00:00:00:000", str) == 0);

    testTime = OB_ULONGLONG(12345670000);
    uto = ts -> new_universal_time(testTime, 0, uto -> tdf());
    str = OB::TimeHelper::toString(uto -> time());
    TEST(strcmp("1234:567", str) == 0);

    testTime = OB_ULONGLONG(141427) * OB_ULONGLONG(86400) *
	OB_ULONGLONG(10000000) - OB_ULONGLONG(1);
    uto = ts -> new_universal_time(testTime, 0, uto -> tdf());
    str = OB::TimeHelper::toString(uto -> utc_time());
    TEST(strcmp("Time less than 1 January 1970 00:00:00", str) == 0);

    uto = ts -> universal_time();

    //
    // TEST: TimeBase::UtcT utcNow(TimeBase::InaccuracyT = 0);
    //
    // This is already taken care of by the TestUTO test
    // for TimeBase::TimeT time()
    //

    //
    // Test: TimeBase::UtcT utcMin();
    //       TimeBase::UtcT utcMax();
    //
    // No sensible test for this.
    //

    //
    // TEST: timeval toTimeval(const TimeBase::UtcT&);
    //       timeval toTimeval(TimeBase::TimeT);
    //
    timeval tv1 = OB::TimeHelper::toTimeval(uto -> utc_time());
    timeval tv2 = OB::TimeHelper::toTimeval(uto -> time());
    TEST(tv1.tv_sec == tv2.tv_sec && tv1.tv_usec == tv2.tv_usec);

    //
    // TEST: comparison operators
    //

    //
    // UTOs the same
    //
    TEST(!(uto -> utc_time() < uto -> utc_time()));
    TEST(uto -> utc_time() <= uto -> utc_time());
    TEST(!(uto -> utc_time() > uto -> utc_time()));
    TEST(uto -> utc_time() >= uto -> utc_time());
    TEST(uto -> utc_time() == uto -> utc_time());
    TEST(!(uto -> utc_time() != uto -> utc_time()));

    //
    // UTO 2 greater than UTO 1
    //
    CosTime::UTO_var uto2 =
        ts -> new_universal_time(uto -> time() + 10, 0, uto -> tdf());

    TEST(uto -> utc_time() < uto2 -> utc_time());
    TEST(uto -> utc_time() <= uto2 -> utc_time());
    TEST(!(uto -> utc_time() > uto2 -> utc_time()));
    TEST(!(uto -> utc_time() >= uto2 -> utc_time()));
    TEST(!(uto -> utc_time() == uto2 -> utc_time()));
    TEST(uto -> utc_time() != uto2 -> utc_time());

    //
    // UTO 2 less than UTO 1
    //
    uto2 = ts -> new_universal_time(uto -> time() - 10, 0, uto -> tdf());

    TEST(!(uto -> utc_time() < uto2 -> utc_time()));
    TEST(!(uto -> utc_time() <= uto2 -> utc_time()));
    TEST(uto -> utc_time() > uto2 -> utc_time());
    TEST(uto -> utc_time() >= uto2 -> utc_time());
    TEST(!(uto -> utc_time() == uto2 -> utc_time()));
    TEST(uto -> utc_time() != uto2 -> utc_time());
}
