// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>

#if !defined(HAVE_SHARED) && !defined(OB_DLL)
#include <OB/OCI_init.h>
#include <OB/OCI_UDP_init.h>
#endif

#include <FileTransfer.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <stdlib.h>
#include <errno.h>

#ifdef HAVE_FSTREAM
#   include <fstream>
#else
#   include <fstream.h>
#endif

#ifdef HAVE_STD_IOSTREAM
using namespace std;
#endif

int
run(CORBA::ORB_ptr orb, int argc, char* argv[])
{
    //
    // Read a reference to the FileTransfer object
    //
    CORBA::Object_var obj =
        orb -> string_to_object("relfile:/FileTransfer.ref");
    if(CORBA::is_nil(obj))
    {
        cerr << argv[0] << ": cannot read IOR from FileTransfer.ref" << endl;
        return EXIT_FAILURE;
    }
    FileTransfer_var fileTransfer = FileTransfer::_narrow(obj);
    assert(!CORBA::is_nil(fileTransfer));

    //
    // Get the input and the output file name
    //
    if(argc != 3)
    {
        cerr << "usage: " << argv[0] << " source destination" << endl;
        return EXIT_FAILURE;
    }

    const char* inFileName = argv[1];
    const char* outFileName = argv[2];

    //
    // Check for input file
    //
    struct stat statBuf;
    if(stat(inFileName, &statBuf) == -1)
    {
        cerr << argv[0] << ": can't open `" << inFileName << "': "
             << strerror(errno) << endl;
        return EXIT_FAILURE;
    }

    //
    // Open and read the input file
    //
    FILE* fp = 0;
    if((fp = fopen(inFileName, "rb")) == 0)
    {
        cerr << argv[0] << ": can't open `" << inFileName << "': "
             << strerror(errno) << endl;
        return EXIT_FAILURE;
    }

    File file;
    file.length(statBuf.st_size);

    fread((void*)file.get_buffer(), 1, file.length(), fp);
    fclose(fp);

    //
    // Send the file
    //
    fileTransfer -> send(outFileName, file);

    return EXIT_SUCCESS;
}

int
main(int argc, char* argv[], char*[])
{
    int status = EXIT_SUCCESS;
    CORBA::ORB_var orb;

    try
    {
#if !defined(HAVE_SHARED) && !defined(OB_DLL)
        //
        // When linking statically, we need to explicitly register
        // the plug-in prior to ORB initialization
        //
        OCI::register_plugin("udp", OCI_init_udp);
#endif

        //
        // Create ORB
        //
        orb = CORBA::ORB_init(argc, argv);

        status = run(orb, argc, argv);
    }
    catch(const CORBA::Exception& ex)
    {
        cerr << ex << endl;
        status = EXIT_FAILURE;
    }

    if(!CORBA::is_nil(orb))
    {
        try
        {
            orb -> destroy();
        }
        catch(const CORBA::Exception& ex)
        {
            cerr << ex << endl;
            status = EXIT_FAILURE;
        }
    }

    return status;
}
