// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>
#include <OB/BootManager.h>

#if !defined(HAVE_SHARED) && !defined(OB_DLL)
#include <OB/OCI_init.h>
#include <OB/OCI_UDP_init.h>
#endif

#include <Hello_impl.h>

#include <stdlib.h>
#include <errno.h>

#ifdef HAVE_FSTREAM
#   include <fstream>
#else
#   include <fstream.h>
#endif

#ifdef HAVE_STD_IOSTREAM
using namespace std;
#endif

int
run(CORBA::ORB_ptr orb, int /* argc */, char* argv[])
{
    //
    // Resolve Root POA
    //
    CORBA::Object_var poaObj = orb -> resolve_initial_references("RootPOA");
    PortableServer::POA_var rootPOA = PortableServer::POA::_narrow(poaObj);
    PortableServer::POAManager_var manager = rootPOA -> the_POAManager();

    //
    // Create a child POA with a PERSISTENT and USER_ID policy
    //
    CORBA::PolicyList policies(2);
    policies.length(2);
    policies[0] =
        rootPOA -> create_lifespan_policy(PortableServer::PERSISTENT);
    policies[1] = 
        rootPOA -> create_id_assignment_policy(PortableServer::USER_ID);

    PortableServer::POA_var childPOA = 
        rootPOA -> create_POA("child", manager, policies);

    //
    // Create and activate the servant for the Hello object
    //
    Hello_impl* helloImpl = new Hello_impl(orb);
    PortableServer::ServantBase_var srv = helloImpl;
    PortableServer::ObjectId_var oid = 
        PortableServer::string_to_ObjectId("Hello");
    childPOA -> activate_object_with_id(oid, helloImpl);

    //
    // Save a reference to the Hello object
    //
    CORBA::Object_var refObj = childPOA -> id_to_reference(oid);
    CORBA::String_var s = orb -> object_to_string(refObj);
    const char* refFile = "Hello.ref";
    ofstream out(refFile);
    if(out.fail())
    {
        cerr << argv[0] << ": can't open `" << refFile << "': "
             << strerror(errno) << endl;
        return EXIT_FAILURE;
    }
    out << s << endl;
    out.close();

    //
    // Run the implementation
    //
    manager -> activate();
    orb -> run();

    return EXIT_SUCCESS;
}

int
main(int argc, char* argv[], char*[])
{
    int status = EXIT_SUCCESS;
    CORBA::ORB_var orb;

    try
    {
#if !defined(HAVE_SHARED) && !defined(OB_DLL)
        //
        // When linking statically, we need to explicitly register
        // the plug-in prior to ORB initialization
        //
        OCI::register_plugin("udp", OCI_init_udp);
#endif

        //
        // Create ORB
        //
        orb = CORBA::ORB_init(argc, argv);

        status = run(orb, argc, argv);
    }
    catch(const CORBA::Exception& ex)
    {
        cerr << ex << endl;
        return EXIT_FAILURE;
    }

    if(!CORBA::is_nil(orb))
    {
        try
        {
            orb -> destroy();
        }
        catch(const CORBA::Exception& ex)
        {
            cerr << ex << endl;
            status = EXIT_FAILURE;
        }
    }

    return status;
}
