// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>
#include <OB/Properties.h>
#include <OB/OCI_UDP_init.h>

#include <OCI_UDP_impl.h>

using namespace CORBA;
using namespace OB;

static const char* UDPversion = "1.0";

// ----------------------------------------------------------------------
// Global functions
// ----------------------------------------------------------------------

extern "C"
void
OCI_version_udp(CORBA::ORB_ptr /* orb */, const char* version)
{
    if(strcmp(UDPversion, version) != 0)
        throw INITIALIZE("udp: incompatible OCI version");
}

extern "C"
OCI::Plugin_ptr
OCI_init_udp(CORBA::ORB_ptr orb, int& ac, char** av)
{
    OBCORBA::ORB_var oborb = OBCORBA::ORB::_narrow(orb);
    Properties_var props = oborb -> properties();

    //
    // Parse arguments
    //
    OCI_parse_args_udp(ac, av, props);

    return new OCI::UDP::Plugin_impl(orb);
}

void
OCI_parse_args_udp(int& ac, char** av, Properties_ptr props)
{
    const char* bind = 0;
    const char* host = 0;
    bool multicast = false;
    bool numeric = false;
    const char* port = 0;
    bool haveArgs = false;

    int i = 0;
    while(i < ac)
    {
        if(strcmp(av[i], "-UDPbind") == 0)
        {
            if(i + 1 >= ac)
                throw INITIALIZE("udp: argument expected for -UDPbind");

            bind = av[i + 1];
            haveArgs = true;

            for(int j = i ; j + 2 < ac ; j++)
                av[j] = av[j + 2];

            ac -= 2;
        }
        else if(strcmp(av[i], "-UDPhost") == 0)
        {
            if(i + 1 >= ac)
                throw INITIALIZE("udp: argument expected for -UDPhost");

            host = av[i + 1];
            haveArgs = true;

            for(int j = i ; j + 2 < ac ; j++)
                av[j] = av[j + 2];

            ac -= 2;
        }
        else if(strcmp(av[i], "-UDPmulticast") == 0)
        {
            multicast = true;
            haveArgs = true;

            for(int j = i ; j + 1 < ac ; j++)
                av[j] = av[j + 1];

            ac -= 1;
        }
        else if(strcmp(av[i], "-UDPnumeric") == 0)
        {
            numeric = true;
            haveArgs = true;

            for(int j = i ; j + 1 < ac ; j++)
                av[j] = av[j + 1];

            ac -= 1;
        }
        else if(strcmp(av[i], "-UDPport") == 0)
        {
            if(i + 1 >= ac)
                throw INITIALIZE("udp: argument expected for -UDPport");

            port = av[i + 1];
            haveArgs = true;

            for(int j = i ; j + 2 < ac ; j++)
                av[j] = av[j + 2];

            ac -= 2;
        }
        else if(strncmp(av[i], "-UDP", 4) == 0)
        {
            String_var err = string_dup("udp: unknown option `");
            err += av[i];
            err += "'";
            throw INITIALIZE(err);
        }
        else
            i++;
    }

    if(haveArgs)
    {
        String_var propName =
            string_dup("ooc.orb.poamanager.RootPOAManager.endpoint");
        const char* value = props -> getProperty(propName);
        if(value == 0)
        {
            propName = string_dup("ooc.orb.oa.endpoint");
            value = props -> getProperty(propName);
        }

        String_var str = string_dup("udp");
        if(bind != 0)
        {
            str += " --bind ";
            str += bind;
        }
        if(host != 0)
        {
            str += " --host ";
            str += host;
        }
        if(multicast)
        {
            str += " --multicast";
        }
        if(numeric)
        {
            str += " --numeric";
        }
        if(port != 0)
        {
            str += " --port ";
            str += port;
        }

        if(value == 0)
            props -> setProperty(propName, str);
        else
        {
            //
            // Append to existing property value
            //
            String_var newValue = value;
            newValue += ',';
            newValue += str;
            props -> setProperty(propName, newValue);
        }
    }
}
