// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#ifndef PACKET_COLLECTOR_H
#define PACKET_COLLECTOR_H

#include <OB/Hashtable.h>
#include <OB/UDPIOP.h>
#include <OB/Logger_fwd.h>
#include <OB/Timer.h>

namespace OCI
{
    
namespace UDP
{

//
// The Message class
// 
class Message
{
    OCI::Buffer_var buf_;
    CORBA::ULong payload_;
    CORBA::ULong lastPayload_;
    CORBA::ULong numberOfPackets_;
    bool* packetsReceived_;
    bool completed_;
    timeval lastPacketTime_;

public:
     
    Message(CORBA::ULong, CORBA::ULong);
    ~Message();

    void add(OCI::Buffer_ptr, CORBA::ULong);
    OCI::Buffer_ptr getBuffer();
    bool completed() { return completed_; }
    CORBA::ULong numberOfPackets() { return numberOfPackets_; }
    const timeval& lastPacketTime() { return lastPacketTime_; }
};
 
//
// The PacketCollector class
//
class PacketCollector : public JTCMutex
{
    //
    // The trace level
    //
    int trace_;

    //
    // The message timeout
    //
    int messageTimeout_;

    //
    // The logger
    //
    OB::Logger_var logger_;

    //
    // Incomplete messages
    //
    typedef OB::Hashtable<CORBA::ULong, Message*, 
	OB::ULongHasher> MessageHashTable;
    MessageHashTable messages_;

public:

    PacketCollector(int, int, OB::Logger_ptr);
    ~PacketCollector();

    //
    // Add packet to a message and check if the message is complete
    //
    OCI::Buffer_ptr addPacket(OB::InputStream_ptr, CORBA::ULong,
                              const UDPIOP::FixedPacketHeader&);

    //
    // Collect garbage
    //
    void sweep();
};

} // End of namespace UDP

} // End of namespace OCI

#endif
