// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// *********************************************************************

#ifndef __OB_LoadUpdater_impl_h__
#define __OB_LoadUpdater_impl_h__

#include <OB/Balancer_fwd.h>
#include <OB/LocalObject.h>
#include <OB/BalancerUtil.h>

#include <UtilLogger_fwd.h>

//
// IDL:ooc.com/LoadBalancing:1.0
//
namespace LoadBalancing
{

//
// IDL:ooc.com/LoadBalancing/Util:1.0
//
namespace Util
{

//
// Pushes the load to the StrategyProxy at regular intervals
//  
class LoadUpdaterThread : public JTCThread, public JTCMonitor
{
public: // For some compilers this must be public

    struct MemberElement
    {
	GroupId_var groupId;
	MemberId_var memberId;
	StrategyProxy_var proxy;
    };
    typedef OB::stl::list< MemberElement* > MemberList;

private:

    //
    // The logger
    //
    Logger_var logger_;

    //
    // The GroupFactory
    //
    GroupFactory_var factory_;

    //
    // The LoadCalculator
    //
    LoadCalculator_var calc_;

    //
    // The member data
    //
    MemberList members_;

    //
    // The update frequency
    //
    CORBA::ULong freq_;

    //
    // Was the update frequency changed?
    //
    bool freqChanged_;

    //
    // Are we finished?
    //
    bool done_;

    //
    // Push load to the StrategyProxy.
    //
    void pushLoad();

public:

    LoadUpdaterThread(Logger_ptr, GroupFactory_ptr, CORBA::ULong);
    ~LoadUpdaterThread();

    //
    // Get the update frequency.
    //
    CORBA::ULong getFrequency();

    //
    // Set the update frequency.
    //
    void setFrequency(CORBA::ULong);

    //
    // Set the LoadCalculator.
    //
    void setCalculator(LoadCalculator_ptr);

    //
    // Register a member.
    //
    void registerMember(const char* member_id, const char* group_id)
        throw(LoadBalancing::GroupNotFound, CORBA::SystemException);

    //
    // Unregister a member.
    //
    void unregisterMember(const char* member_id, const char* group_id)
        throw(CORBA::SystemException);

    //
    // Destroy the thread.
    //
    void destroy();

    //
    // Run the thread.
    //
    virtual void run();
};

typedef JTCHandleT< LoadUpdaterThread > UpdaterHandle;

//
// IDL:ooc.com/LoadBalancing/LoadUpdater:1.0
//
class LoadUpdater_impl : public LoadUpdater,
			 public OBCORBA::RefCountLocalObject
{
    LoadUpdater_impl(const LoadUpdater_impl&);
    void operator=(const LoadUpdater_impl&);

    //
    // The LoadUpdater thread
    //
    UpdaterHandle updater_;

public:

    LoadUpdater_impl(Logger_ptr, LoadBalancing::GroupFactory_ptr,
		     CORBA::ULong);
    virtual ~LoadUpdater_impl();

    //
    // IDL:ooc.com/LoadBalancing/LoadUpdater/get_update_frequency:1.0
    //
    virtual CORBA::ULong get_update_frequency()
        throw(CORBA::SystemException);

    //
    // IDL:ooc.com/LoadBalancing/LoadUpdater/set_update_frequency:1.0
    //
    virtual void set_update_frequency(CORBA::ULong millis)
        throw(CORBA::SystemException);

    //
    // IDL:ooc.com/LoadBalancing/LoadUpdater/set_load_calculator:1.0
    //
    virtual void set_load_calculator(LoadCalculator_ptr calc)
        throw(CORBA::SystemException);

    //
    // IDL:ooc.com/LoadBalancing/LoadUpdater/register_member:1.0
    //
    virtual void register_member(const char* member_id,
				 const char* group_id)
        throw(LoadBalancing::GroupNotFound, CORBA::SystemException);

    //
    // IDL:ooc.com/LoadBalancing/LoadUpdater/register_member:1.0
    //
    virtual void unregister_member(const char* member_id,
				   const char* group_id)
        throw(CORBA::SystemException);
};

} // End of namespace Util

} // End of namespace LoadBalancing

#endif
