// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#ifndef OB_CODE_SET_UTIL_H
#define OB_CODE_SET_UTIL_H

#include <OB/Any.h>
#include <OB/TypeCodeConst.h>
#include <OB/OCITypes.h>
#include <OB/LocalObject.h>
#include <OB/ValueBase.h>
#include <OB/PIIOR.h>

#include <CodeConverter.h>

namespace IOP
{

//
// Forward declarations
//
struct IOR;
struct TaggedComponent;
struct ServiceContext;

} // End of namespace IOP

namespace CONV_FRAME
{

//
// Forward declarations
//
struct CodeSetComponentInfo;
struct CodeSetComponent;
struct CodeSetContext;

}; // End of namespace CONV_FRAME

namespace OCI
{

struct ProfileInfo;

}; // End of namespace OCI

namespace OB
{

//
// Forward declarations
//
struct CodeConverters;

class CodeSetUtil
{
    //
    // The supported codesets in the preferred order
    //
    static FixSeq< CORBA::ULong, int > supportedCharCodeSets_;
    static FixSeq< CORBA::ULong, int > supportedWcharCodeSets_;

public:

    //
    // Add codeset
    //
    static void addCharCodeSet(CORBA::ULong);
    static void addWcharCodeSet(CORBA::ULong);

    //
    // Create codeset component
    //
    static CONV_FRAME::CodeSetComponent*
        createCodeSetComponent(CORBA::ULong, bool);

    //
    // Get code converters from ProfileInfo and/or IOR
    //
    static void getCodeConverters(ORBInstance_ptr, const OCI::ProfileInfo&,
				  OB::CodeConverters&);

    //
    // Check for code set information in a tagged component
    //
    static bool checkForCodeSetInfo(const IOP::TaggedComponent&,
                                    CONV_FRAME::CodeSetComponentInfo&);

    //
    // Extract code set context from service context
    //
    static void extractCodeSetContext(const IOP::ServiceContext&,
				      CONV_FRAME::CodeSetContext&);

    //
    // Extract codeset information
    //
    static bool
    getCodeSetInfoFromComponents(ORBInstance_ptr, const OCI::ProfileInfo&,
				 CONV_FRAME::CodeSetComponentInfo&);
};

} // End namespace OB

// ----------------------------------------------------------------------
// Add code set information to IOR. This is done with an
// IORInterceptor. This will be called to establish an IOR template
// for each POA.
// ----------------------------------------------------------------------

namespace PortableInterceptor
{

class CodeSetIORInterceptor_impl :
	public PortableInterceptor::IORInterceptor,
	public OBCORBA::RefCountLocalObject
{
    //
    // The native codesets
    //
    CORBA::ULong nativeCs_;
    CORBA::ULong nativeWcs_;

public:

    CodeSetIORInterceptor_impl(CORBA::ULong, CORBA::ULong);

    //
    // IDL to C++ Mapping
    //
    virtual char*
    name()
	throw(CORBA::SystemException);

    virtual void
    destroy()
        throw(CORBA::SystemException);

    virtual void
    establish_components(PortableInterceptor::IORInfo_ptr)
	throw(CORBA::SystemException);

    virtual void
    components_established(PortableInterceptor::IORInfo_ptr)
	throw(CORBA::SystemException);

    virtual void
    adapter_manager_state_change(AdapterManagerId id,
				 AdapterState state)
	throw(CORBA::SystemException);

    virtual void
    adapter_state_change(const ObjectReferenceTemplateSeq&,
		         AdapterState)
	throw(CORBA::SystemException);

};

} // End namespace PortableInterceptor

#endif
