// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#ifndef OB_DISPATCH_IMPL_H
#define OB_DISPATCH_IMPL_H

#include <OB/LocalObject.h>
#include <OB/Except.h>
#include <OB/Dispatch.h>
#include <OB/list.h>
#include <OB/NativeTypes.h>
#include <OB/PIORBInit.h>

namespace OB
{

class ThreadPool;
typedef ThreadPool* ThreadPool_ptr;

} // End of namespace OB

void OBDuplicate(OB::ThreadPool_ptr);
void OBRelease(OB::ThreadPool_ptr);

namespace OB
{

typedef ObjVar< ThreadPool > ThreadPool_var;

//
// Helper class for dispatching a request
//
class ThreadPool : public JTCMonitor, public RefCount
{
    bool destroy_; // True if destroy was called

    struct RequestRecord
    {
	DispatchRequest_var request;
	RequestRecord* next;
    };

    RequestRecord* head_;
    JTCThreadGroupHandle group_; // Thread group for the threads in the pool

public:

    ThreadPool(CORBA::ULong, CORBA::Long);
    ~ThreadPool();

    static inline ThreadPool_ptr _duplicate(ThreadPool_ptr p)
    { if(p) p -> _OB_incRef(); return p; }
    static inline ThreadPool_ptr _nil()
    { return 0; }
    
    void destroy();
    
    void add(DispatchRequest_ptr);

    DispatchRequest_ptr get();
};

} // End of namespace OB

namespace CORBA
{

inline void
release(OB::ThreadPool_ptr p)
{
    if(p)
	p -> _OB_decRef();
}

inline Boolean
is_nil(OB::ThreadPool_ptr p)
{
    return p == 0;
}

} // End of namespace CORBA

namespace OB
{

class DispatchStrategyFactory_impl :
	public DispatchStrategyFactory,
	public OBCORBA::RefCountLocalObject,
	public JTCRecursiveMutex
{

    typedef ObjSeq< ThreadPool, int > ThreadPoolSeq;

    //
    // A sequence of thread pools. The index in the sequence is the
    // thread pool id.
    //
    ThreadPoolSeq pools_;

    //
    // Has the default thread pool been created yet?
    //
    bool haveDefaultThreadPool_;

    //
    // If so, what is the thread pool id?
    //
    ThreadPoolId defaultThreadPool_;

    //
    // Has the factory been destroyed?
    //
    bool destroy_;

    //
    // The ORB instance
    //
    ORBInstance_var orbInstance_;

public:

    DispatchStrategyFactory_impl();
    ~DispatchStrategyFactory_impl();

    //
    // Standard IDL to C++ Mapping
    //
    virtual ThreadPoolId
    create_thread_pool(CORBA::Long)
	throw(CORBA::SystemException);

    virtual void
    destroy_thread_pool(ThreadPoolId id)
	throw(InvalidThreadPool, CORBA::SystemException);

    virtual DispatchStrategy_ptr
    create_thread_pool_strategy(ThreadPoolId)
	throw(InvalidThreadPool, CORBA::SystemException);

    virtual DispatchStrategy_ptr
    create_same_thread_strategy()
	throw(CORBA::SystemException);

    virtual DispatchStrategy_ptr
    create_thread_per_request_strategy()
	throw(CORBA::SystemException);

    virtual DispatchStrategy_ptr
    create_default_dispatch_strategy()
	throw(CORBA::SystemException);

    //
    // Additional ORBacus specific functions
    //
    void _OB_setORBInstance(ORBInstance_ptr);
    void _OB_destroy();
};

} // End of namespace OB

#endif
