// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/Basic.h>
#include <OB/LocalObject.h>
#include <OB/Any.h>
#include <OB/OCI.h>

using namespace CORBA;
using namespace OB;

// ----------------------------------------------------------------------
// External, non-inline duplicate/release for templates
// ----------------------------------------------------------------------

void
OBRelease(CORBA::LocalObject_ptr o)
{
    if(o)
        o -> _remove_ref();
}

void
OBDuplicate(CORBA::LocalObject_ptr o)
{
    if(o)
        o -> _add_ref();
}

// ----------------------------------------------------------------------
// LocalObjectBase constructor/destructor
// ----------------------------------------------------------------------

OBCORBA::LocalObjectBase::LocalObjectBase()
{
}

OBCORBA::LocalObjectBase::~LocalObjectBase()
{
}

// ----------------------------------------------------------------------
// LocalObjectBase public member implementation
// ----------------------------------------------------------------------

InterfaceDef_ptr
OBCORBA::LocalObjectBase::_get_interface()
{
    throw NO_IMPLEMENT(MinorNotSupportedByLocalObject, COMPLETED_NO);
    return 0; // Some compilers need this
}

Boolean
OBCORBA::LocalObjectBase::_is_a(const char*)
{
    throw NO_IMPLEMENT(MinorNotSupportedByLocalObject, COMPLETED_NO);
    return false; // Some compilers need this
}

Boolean
OBCORBA::LocalObjectBase::_non_existent()
{
    return false;
}

Boolean
OBCORBA::LocalObjectBase::_is_equivalent(Object_ptr obj)
{
    LocalObject_ptr lc = dynamic_cast<LocalObject_ptr>(obj);
    return (!is_nil(lc) && this == lc);
}

ULong
OBCORBA::LocalObjectBase::_hash(ULong maximum)
{
    return ((ULong)(unsigned long)this / 256) % (maximum + 1);
}


void
OBCORBA::LocalObjectBase::_create_request(CORBA::Context_ptr, const char*,
			     CORBA::NVList_ptr,
			     CORBA::NamedValue_ptr,
			     CORBA::Request_out, CORBA::Flags)
{
    throw NO_IMPLEMENT(MinorDIINotSupportedByLocalObject, COMPLETED_NO);
}

void
OBCORBA::LocalObjectBase::_create_request(CORBA::Context_ptr, const char*,
			     CORBA::NVList_ptr,
			     CORBA::NamedValue_ptr,
			     CORBA::ExceptionList_ptr,
			     CORBA::ContextList_ptr,
			     CORBA::Request_out, CORBA::Flags)
{
    throw NO_IMPLEMENT(MinorDIINotSupportedByLocalObject, COMPLETED_NO);
}

CORBA::Request_ptr
OBCORBA::LocalObjectBase::_request(const char*)
{
    throw NO_IMPLEMENT(MinorDIINotSupportedByLocalObject, COMPLETED_NO);
    return 0; // Some compilers need this
}

Policy_ptr
OBCORBA::LocalObjectBase::_get_policy(PolicyType)
{
    throw NO_IMPLEMENT(MinorNotSupportedByLocalObject, COMPLETED_NO);
    return 0; // Some compilers need this
}

Policy_ptr
OBCORBA::LocalObjectBase::_get_client_policy(PolicyType)
{
    throw NO_IMPLEMENT(MinorNotSupportedByLocalObject, COMPLETED_NO);
    return 0; // Some compilers need this
}

Object_ptr
OBCORBA::LocalObjectBase::_set_policy_overrides(const PolicyList&,
				   SetOverrideType)
{
    throw NO_IMPLEMENT(MinorNotSupportedByLocalObject, COMPLETED_NO);
    return 0; // Some compilers need this
}

PolicyList*
OBCORBA::LocalObjectBase::_get_policy_overrides(const PolicyTypeSeq&)
{
    throw NO_IMPLEMENT(MinorNotSupportedByLocalObject, COMPLETED_NO);
    return 0; // Some compilers need this
}

Boolean
OBCORBA::LocalObjectBase::_validate_connection(PolicyList_out)
{
    throw NO_IMPLEMENT(MinorNotSupportedByLocalObject, COMPLETED_NO);
    return false; // Some compiler need this
}

OCI::ConnectorInfo_ptr
OBCORBA::LocalObjectBase::_get_oci_connector_info()
{
    throw NO_IMPLEMENT(MinorNotSupportedByLocalObject, COMPLETED_NO);
    return OCI::ConnectorInfo::_nil(); // Some compilers need this
}

OCI::TransportInfo_ptr
OBCORBA::LocalObjectBase::_get_oci_transport_info()
{
    throw NO_IMPLEMENT(MinorNotSupportedByLocalObject, COMPLETED_NO);
    return OCI::TransportInfo::_nil(); // Some compilers need this
}

CORBA::AbstractBase_ptr
OBCORBA::LocalObjectBase::_OB_toAbstractBase()
{
    return 0;
}

OB::RefCountIOR_ptr
OBCORBA::LocalObjectBase::_OB_origIOR() const
{
    throw MARSHAL(MinorLocalObject, COMPLETED_NO);
    return 0; // Some compilers need this
}

OB::RefCountIOR_ptr
OBCORBA::LocalObjectBase::_OB_IOR() const
{
    throw MARSHAL(MinorLocalObject, COMPLETED_NO);
    return 0; // Some compilers need this
}

void
OBCORBA::LocalObjectBase::_OB_marshalOrigIOR(OutputStreamImpl*) const
{
    throw MARSHAL(MinorLocalObject, COMPLETED_NO);
}

const char**
OBCORBA::LocalObjectBase::_OB_ids() const
{
    throw NO_IMPLEMENT(MinorNotSupportedByLocalObject, COMPLETED_NO);
    return 0; // Some compilers need this
}

// ----------------------------------------------------------------------
// LocalObject constructor/destructor
// ----------------------------------------------------------------------

LocalObject::LocalObject()
{
}

LocalObject::~LocalObject()
{
}

// ----------------------------------------------------------------------
// LocalObject public member implementation
// ----------------------------------------------------------------------

void
LocalObject::_add_ref()
{
}

void
LocalObject::_remove_ref()
{
}

// ----------------------------------------------------------------------
// RefCountLocalObject public member implementation
// ----------------------------------------------------------------------

void
OBCORBA::RefCountLocalObject::_add_ref()
{
    _OB_incRef();
}

void
OBCORBA::RefCountLocalObject::_remove_ref()
{
    _OB_decRef();
}
