// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#ifndef OB_OBORB_IMPL_H
#define OB_OBORB_IMPL_H

#include <OB/ValueFactoryBase_fwd.h>
#include <OB/Environment_fwd.h>
#include <OB/ORBInstance_fwd.h>
#include <OB/ORBControl_fwd.h>
#include <OB/PI_fwd.h>
#include <OB/Properties_fwd.h>
#include <OB/Logger_fwd.h>
#include <OB/CoreTraceLevels_fwd.h>
#include <OB/PluginManager_fwd.h>
#include <OB/UnknownException_fwd.h>
#include <OB/Client.h> // For Client::ConcModel

#include <OB/NativeTypes.h>
#include <OB/MarshalStubImpl.h>
#include <OB/OBORB.h>
#include <OB/PI_init.h>
#include <OB/Hashtable.h>
#include <OB/ORB_init.h>
#include <OB/LocalObject.h>

namespace OB
{

ORBInstance_ptr GetORBInstance(CORBA::ORB_ptr); // For the friend declaration

class ModuleManagerDestroyer; // For the friend declaration

} // End of namespace CORBA

namespace OBCORBA
{

//
// The ORB_impl class
//
class ORB_impl : public ORB,
		 public OBCORBA::RefCountLocalObject,
		 public JTCRecursiveMutex
{
    //
    // Hide copy constructor/assignment operator
    //
    ORB_impl(const ORB_impl&);
    void operator=(const ORB_impl&);

    //
    // The ORBInstance object
    //
    OB::ORBInstance_var orbInstance_;

    //
    // GetORBInstance() needs access to orbInstance_
    //
    friend OB::ORBInstance_ptr OB::GetORBInstance(CORBA::ORB_ptr);

    //
    // All ORBs that have been initialized with ORB_init()
    //
    // Note: Do not translate this to Java. For the Java mapping,
    // ORB_init() always returns a new ORB instance.
    //
    typedef OB::SynchronizedHashtable<CORBA::String_var, ORB_var, 
	OB::StringHasher> ORBImplMap;
    static ORBImplMap orbImplMap_;

    //
    // The ORB Control
    //
    OB::ORBControl_var orbControl_;

    //
    // Has the ORB been destroyed?
    //
    bool destroy_;

    //
    // The OCI Plugin Manager
    //
    OB::PluginManager_var pluginManager_;

    //
    // Default set of policies
    //
    CORBA::PolicyList policies_;
    void initializeDefaultPolicies();

    //
    // Private constructor and destructor
    //
    ORB_impl(int&, char**&, const char*, const char*, const char*,
	     OB::Client::ConcModel,
	     OB::CoreTraceLevels_ptr,
	     OB::Properties_ptr,
	     OB::Logger_ptr,
	     OB::Reactor_ptr,
	     CORBA::ULong, CORBA::ULong, CORBA::ULong);
    ~ORB_impl();

    //
    // ORB_init may create ORBs
    //
    friend OBCORBA::ORB_ptr
    OBCORBA::ORB_init(int&, char**, OB::Properties_ptr, OB::Logger_ptr,
		      OB::Reactor_ptr, const char*, const char*);
    typedef char* OAid;

    //
    // ORB_initializer accesses ORB internals
    //
    friend class ::OB::ModuleManagerDestroyer;

public:

    //
    // Standard IDL to C++ Mapping
    //
    virtual char* object_to_string(CORBA::Object_ptr);
    virtual CORBA::Object_ptr string_to_object(const char*);

    virtual void create_list(CORBA::Long, CORBA::NVList_out);
    virtual void create_operation_list(CORBA::OperationDef_ptr,
				       CORBA::NVList_out);
 
    virtual void get_default_context(CORBA::Context_out);

    virtual void create_named_value(CORBA::NamedValue_out);
    virtual void create_exception_list(CORBA::ExceptionList_out);
    virtual void create_context_list(CORBA::ContextList_out);
    virtual void create_environment(CORBA::Environment_out);

    virtual void send_multiple_requests_oneway(const CORBA::ORB::RequestSeq&);
    virtual void send_multiple_requests_deferred(const CORBA::ORB::RequestSeq&);
    virtual CORBA::Boolean poll_next_response();
    virtual void get_next_response(CORBA::Request_out);

    virtual CORBA::Boolean get_service_information(
	CORBA::ServiceType,
	CORBA::ServiceInformation_out);
    virtual CORBA::ORB::ObjectIdList* list_initial_services();
    virtual CORBA::Object_ptr resolve_initial_references(const char*);
    virtual void register_initial_reference(const char*, CORBA::Object_ptr);

    CORBA::TypeCode_ptr create_struct_tc(const char*,
					 const char*,
					 const CORBA::StructMemberSeq&);
    
    CORBA::TypeCode_ptr create_union_tc(const char*,
					const char*,
					CORBA::TypeCode_ptr,
					const CORBA::UnionMemberSeq&);
    
    CORBA::TypeCode_ptr create_enum_tc(const char*,
				       const char*,
				       const CORBA::EnumMemberSeq&);
    
    CORBA::TypeCode_ptr create_alias_tc(const char*,
					const char*,
					CORBA::TypeCode_ptr);
    
    CORBA::TypeCode_ptr create_exception_tc(const char*,
					    const char*,
					    const CORBA::StructMemberSeq&);
    
    CORBA::TypeCode_ptr create_interface_tc(const char*, const char*);
    
    CORBA::TypeCode_ptr create_string_tc(CORBA::ULong);
    
    CORBA::TypeCode_ptr create_wstring_tc(CORBA::ULong);
    
    CORBA::TypeCode_ptr create_fixed_tc(CORBA::UShort, CORBA::Short);

    CORBA::TypeCode_ptr create_sequence_tc(CORBA::ULong, CORBA::TypeCode_ptr);
    
    CORBA::TypeCode_ptr create_recursive_sequence_tc(CORBA::ULong,
						     CORBA::ULong);
    
    CORBA::TypeCode_ptr create_array_tc(CORBA::ULong, CORBA::TypeCode_ptr);
    
    CORBA::TypeCode_ptr create_value_tc(const char*,
					const char*,
					CORBA::ValueModifier,
					CORBA::TypeCode_ptr,
					const CORBA::ValueMemberSeq&);
    
    CORBA::TypeCode_ptr create_value_box_tc(const char*,
					    const char*,
					    CORBA::TypeCode_ptr);
    
    CORBA::TypeCode_ptr create_native_tc(const char*, const char*);
    
    CORBA::TypeCode_ptr create_recursive_tc(const char*);
    
    CORBA::TypeCode_ptr create_abstract_interface_tc(const char*, const char*);
    
    CORBA::TypeCode_ptr create_local_interface_tc(const char*, const char*);

    CORBA::Boolean work_pending();
    void perform_work();
    void run();
    void shutdown(CORBA::Boolean);
    void destroy();

    CORBA::Policy_ptr create_policy(CORBA::PolicyType, const CORBA::Any&);

    CORBA::ValueFactoryBase* register_value_factory(
	const char*, CORBA::ValueFactoryBase*);
    void unregister_value_factory(const char*);
    CORBA::ValueFactoryBase* lookup_value_factory(const char*);
                                                  
    //
    // Additional ORBacus specific functions
    //
    OB::PropertySet properties();

    OB::Logger_ptr logger();

    OB::Reactor_ptr reactor();

    OB::UnknownExceptionStrategy_ptr set_unknown_exception_strategy(
        OB::UnknownExceptionStrategy_ptr);

    OB::ConnectionReusePolicy_ptr
    create_connection_reuse_policy(CORBA::Boolean);

    OB::ProtocolPolicy_ptr
    create_protocol_policy(const OCI::PluginIdSeq&);

    OB::RetryPolicy_ptr
    create_retry_policy(CORBA::Short, CORBA::ULong, CORBA::ULong, 
			CORBA::Boolean);

    OB::TimeoutPolicy_ptr
    create_timeout_policy(CORBA::ULong);

    OB::LocationTransparencyPolicy_ptr
    create_location_transparency_policy(CORBA::Short);

    Messaging::RebindPolicy_ptr
    create_rebind_policy(Messaging::RebindMode);

    Messaging::SyncScopePolicy_ptr
    create_sync_scope_policy(Messaging::SyncScope);

    Messaging::RequestStartTimePolicy_ptr
    create_request_start_time_policy(TimeBase::UtcT);

    Messaging::RequestEndTimePolicy_ptr
    create_request_end_time_policy(TimeBase::UtcT);

    Messaging::ReplyStartTimePolicy_ptr
    create_reply_start_time_policy(TimeBase::UtcT);

    Messaging::ReplyEndTimePolicy_ptr
    create_reply_end_time_policy(TimeBase::UtcT);

    Messaging::RelativeRequestTimeoutPolicy_ptr
    create_relative_request_timeout_policy(TimeBase::TimeT);

    Messaging::RelativeRoundTripTimeoutPolicy_ptr
    create_relative_round_trip_timeout_policy(TimeBase::TimeT);

    OB::InterceptorPolicy_ptr
    create_interceptor_policy(CORBA::Boolean);

    OB::ConnectTimeoutPolicy_ptr
    create_connection_timeout_policy(CORBA::ULong);

    OB::RequestTimeoutPolicy_ptr
    create_request_timeout_policy(CORBA::ULong);
};

} // End of namespace OBCORBA

#endif
