// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>
#include <OB/IFR.h>
#include <OB/Hashtable.h>

#include <Util.h>
#include <GenUtil.h>
#include <GenECPP.h>

using namespace CORBA;
using namespace OB;

// ----------------------------------------------------------------------
// Generate code for the example header file
// ----------------------------------------------------------------------

void
IdlECPPGenerator::genCPlusPlusImplH(Container_ptr container, bool all,
                                   IdlPrettyPrint& out)
{
    //
    // Get container contents descriptions
    //
    ScopedName_var scope;
    // No _var for seq! See comments for the implementation of "describe"
    Container::DescriptionSeq* seq = describe(container, scope.out());

    for(ULong i = 0 ; i < seq -> length() ; i++)
    {
	//
	// Get element description
	//
	Container::Description& seqElem = (*seq)[i];

	//
	// Get container
	//
	Container_var cont = Container::_narrow(seqElem.contained_object);

        switch(seqElem.kind)
        {
        case dk_Module:
        {
            //
            // Module
            //
            const ModuleDescription* desc;
            Boolean b = seqElem.value >>= desc;
            assert(b);

	    if(!check(desc -> id))
		continue;

            Identifier_var name = fixKwd(desc -> name);

            comment(desc -> id, out);
            //out << "\nclass Module_" << name;
            //out << "\n{";

            assert(!is_nil(cont));
            genCPlusPlusImplH(cont, all, out);

            //out.sep();
            //out << "\n}; // End of class Module_" << name;

            break;
        }

        case dk_AbstractInterface:
            //
            // No sample implementation for abstract interface
            //
            break;

	case dk_LocalInterface:
        case dk_Interface:
        {
            //
            // Interface
            //
            const InterfaceDescription* desc;
            Boolean b = seqElem.value >>= desc;
            assert(b);

            if(!check(desc -> id))
                continue;

            ScopedName_var absolute = getAbsolute(desc -> id);
	    String_var flatAbsolute = getFlattenedName(absolute);

            comment(desc -> id, out);

            out.sep();
            out << "\nclass " << flatAbsolute << "_impl : ";

            unsigned int indent = out.getIndent();
            out.setIndent(out.getPos());

	    //
            // TODO: Is virtual inheritance mandatory if there are base
            //       interfaces and all == false?
	    //
            if(!noVirtual_)
                out << "virtual ";
	    if(seqElem.kind == dk_LocalInterface)
                out << "public " << absolute << ", public CORBA::LocalObject";
            else
                out << "public " << "POA_" << absolute;

            InterfaceDef_var intf = InterfaceDef::_narrow(cont);
            assert(!is_nil(intf));
            IdlInterfaceInfoSeq baseInfo;
            IdlGetBaseInterfaces(intf, baseInfo, false);
            ULong numAbstract = 0;

            if(!all)
            {
                for(ULong j = 0 ; j < baseInfo.length() ; j++)
                {
                    //
                    // Impl classes are not generated for abstract interfaces
                    //
                    if(!baseInfo[j].is_abstract)
                    {
                        ScopedName_var baseAbsolute =
                            getAbsolute(baseInfo[j].id, "");
			String_var flatBaseAbsolute =
			    getFlattenedName(baseAbsolute);

                        out << ",\n";

                        if(!noVirtual_)
                            out << "virtual ";

                        out << "public " << flatBaseAbsolute << "_impl";
                    }
                    else
                        numAbstract++;
                }
            }

	    // XXX RefCounted?
#ifdef XXX
            if(all || baseInfo.length() - numAbstract == 0)
            {
                out << ",\n";

                if(!noVirtual_)
                    out << "virtual ";

                out << "public PortableServer::RefCountServantBase";
            }
#endif

            out.setIndent(indent);

            out.start();

            out << '\n' << flatAbsolute << "_impl(const " << flatAbsolute
                << "_impl&);";
            out << "\nvoid operator=(const " << flatAbsolute << "_impl&);";

	    if(seqElem.kind != dk_LocalInterface)
            {
                out.sep();
                out << "\nPortableServer::POA_var poa_;";
            }

            out.sep();
            out.dec();
            out << "\npublic:";
            out.inc();
            out.sep();
	    if(seqElem.kind == dk_LocalInterface)
	    {
                out << '\n' << flatAbsolute << "_impl();";
	    }
            else
	    {
                out << '\n' << flatAbsolute
		    << "_impl(PortableServer::POA_ptr);";
	    }
            out << "\n~" << flatAbsolute << "_impl();";

	    if(seqElem.kind != dk_LocalInterface)
            {
                //
                // _default_POA
                //
                out.sep();
                out << "\nvirtual PortableServer::POA_ptr "
		    << "_default_POA(CORBA::Environment&);";
            }

            //
            // Generate attributes and operations for any base abstract
            // interfaces
            //
            if(!all)
            {
                StringSeq ids;

                for(ULong j = 0 ; j < baseInfo.length() ; j++)
                {
                    if(!baseInfo[j].is_abstract)
                        continue;

		    ScopedName_var dummy;
		    // No _var for seq! See comments for the
		    // implementation of "describe"
		    Container::DescriptionSeq* contents =
			describe(baseInfo[j].intf, dummy.out());

                    for(ULong k = 0 ; k < contents -> length() ; k++)
                    {
                        if((*contents)[k].kind == dk_Operation)
                        {
                            const OperationDescription* opDesc;
                            Boolean b = ((*contents)[k].value) >>= opDesc;
                            assert(b);
			    ULong l;
                            for(l = 0 ; l < ids.length() ; l++)
                                if(strcmp(ids[l], opDesc -> id) == 0)
                                    break;
                            if(l >= ids.length())
                            {
                                genCPlusPlusImplHOperation(absolute, opDesc,
                                                           true, out);
                                ids.length(ids.length()+1);
                                ids[ids.length()-1] = opDesc -> id;
                            }
                        }
                        else if((*contents)[k].kind == dk_Attribute)
                        {
                            const AttributeDescription* atDesc;
                            Boolean b = ((*contents)[k].value) >>= atDesc;
                            assert(b);
			    ULong l;
                            for(l = 0 ; l < ids.length() ; l++)
                                if(strcmp(ids[l], atDesc -> id) == 0)
                                    break;
                            if(l >= ids.length())
                            {
                                genCPlusPlusImplHAttribute(absolute, atDesc,
                                                           true, out);
                                ids.length(ids.length()+1);
                                ids[ids.length()-1] = atDesc -> id;
                            }
                        }
                    }
                }
            }

            //
            // Attributes and operations
            //

            Container::DescriptionSeq_var contents =
                cont -> describe_contents(dk_all, !all, -1);

            for(ULong j = 0 ; j < contents -> length() ; j++)
            {
                if(contents[j].kind == dk_Attribute)
                {
                    const AttributeDescription* atDesc;
                    Boolean b = (contents[j].value >>= atDesc);
                    assert(b);
                    genCPlusPlusImplHAttribute(absolute, atDesc, true, out);
                }
                else if(contents[j].kind == dk_Operation)
                {
                    const OperationDescription* opDesc;
                    Boolean b = (contents[j].value >>= opDesc);
                    assert(b);
                    genCPlusPlusImplHOperation(absolute, opDesc, true, out);
                }
            }

            out.end();
            out << ';';

            break;
        }

        default:
            break;
        }
    }
}

// ----------------------------------------------------------------------
// Generate header code for an example attribute
// ----------------------------------------------------------------------

void
IdlECPPGenerator::genCPlusPlusImplHAttribute(const char* /* absolute */,
                                            const AttributeDescription* desc,
                                            bool /* ex */,
                                            IdlPrettyPrint& out)
{
    Identifier_var name = fixKwd(desc -> name);

    comment(desc -> id, out);

    String_var ret = getTypeString("", desc -> type, GetTypeReturn);

    out.sep();
    out << "\nvirtual " << ret << ' ' << name << "(CORBA::Environment&);";

    if(desc -> mode == ATTR_NORMAL)
    {
        String_var in = getTypeString("", desc -> type, GetTypeIn);
        out << "\nvirtual void " << name << '(' << in
	    << ", CORBA::Environment&);";
    }
}

// ----------------------------------------------------------------------
// Generate header code for an example operation
// ----------------------------------------------------------------------

void
IdlECPPGenerator::genCPlusPlusImplHOperation(const char* /* absolute */,
                                            const OperationDescription* desc,
                                            bool /* ex */,
                                            IdlPrettyPrint& out)
{
    Identifier_var name = fixKwd(desc -> name);

    comment(desc -> id, out);

    String_var ret = getTypeString("", desc -> result, GetTypeReturn);

    out.sep();
    out << "\nvirtual " << ret << ' ' << name << "(";

    unsigned int indent = out.getIndent();
    out.setIndent(out.getPos());

    for(ULong i = 0 ; i < desc -> parameters.length() ; i++)
    {
        GetType tt;
        switch(desc -> parameters[i].mode)
        {
        case PARAM_IN:
            tt = GetTypeIn;
            break;

        case PARAM_INOUT:
            tt = GetTypeInOut;
            break;

        case PARAM_OUT:
            tt = GetTypeOut;
            break;
        }

        String_var argName = fixKwd(desc -> parameters[i].name);

        String_var s =
            getTypeString("", desc -> parameters[i].type, tt, false, argName);

        if(i > 0)
            out << ",\n";

        out << s;
    }

    if(desc -> contexts.length() > 0)
    {
        if(desc -> parameters.length() > 0)
            out << ",\n";

        out << "CORBA::Context_ptr";
    }

    if(desc -> parameters.length() > 0)
	out << ",\n";
    out << "CORBA::Environment&);";

    out.setIndent(indent);
}
